globals
[
  clock        ; keeps the number of times through the go procedure
  avg-lifespan ; the mean lifespan for birds
  available-females ;keeps track of how many females are availble for mating.
]

patches-own
[
  nest            ; initializes to false--true if the patch is a nest
]


;;the male birds require a lot more data than female birds...we separated them into different breeds
breeds [ females males]

females-own
[
  ;;;personal characteristics
  age              ; how old a turtle is
  life-expectancy  ; maximum age that a turtle can reach
  sex              ; the sex of the bird (and often gender)
  search-counter   ; used to keep track of how far she has gone in search of a nest

    ;;;info about children
  child-1-dev       ;the development (0=unconceived, 1-30=incubation 31-90=baby 91+=adult) of child one

  ;;;bird relationship states
  mating            ;true if the bird is currently mating
  available         ;true if the bird is available--single

  ;;;;coordinates of nest
  nest-1-X
  nest-1-Y

]

males-own
[
  ;;;personal characteristics
  age              ; how old a turtle is
  life-expectancy  ; maximum age that a turtle can reach
  sex              ; the sex of the bird (and often gender)
  current-nest     ; the current nest that the male is calling "home"
  search-counter   ; how many spaces you've moved while searching
  child-counter    ; how many children is he currently feeding
  polygamous       ; virtual gene for polygamous behavior - true if polygomous

   ;;;info about children
  child-1-dev       ;the development (0=unconceived, 1-30=incubation 31-90=baby 91+=adult) of child one
  child-2-dev       ;the development (0=unconceived, 1-30=incubation 31-90=baby 91+=adult) of child two

  ;;;bird relationship states
  mating            ;true if the bird is currently mating
  available         ;true if the bird is available--single

  ;;nest coordinates
  nest-1-X
  nest-1-Y
  nest-2-X
  nest-2-Y
]

;;;
;;; SETUP AND HELPERS
;;;

to setup
  ca
  ;; set global variables to appropriate values
  set clock 0
  set avg-lifespan 365
  set available-females 0
  ;; call other procedures to set up various parts of the world
  setup-patches
  setup-turtles
  setup-plots
  ;; plot the initial state of the world
  update-plots
end

to setup-patches
  ask patches
    [set nest false]
end

to recolor-patch  ;; patch procedure -- use color to indicate grain level
  ifelse nest     ;; patch variable--true if patch is a nest
     [set pcolor green]
     [set pcolor black]
end

;; set up the initial values for the turtle variables
to setup-turtles
  locals [found-place]
  set found-place false
  no-display  ;; so we don't see the turtles until they're recolored

  ;;half the population will be female, half male
  create-females floor (num-birds / 2)
  create-males ceiling (num-birds / 2)

  ask turtles
  [
      ;randomly place birds on an un-occupied patch
      while  [not found-place]
      [
         setxy (random-float screen-size-x)
               (random-float screen-size-y)
         if (count turtles-here = 1)
           [set found-place true]
      ]
      set found-place false
      set-initial-turtle-vars
   ]
   ;;have to initialize the male specific values
   ask males
   [
     set-initial-male-vars
     recolor-turtle
   ]
   ask females
   [
      make-nest
      set available-females available-females + 1
      recolor-turtle
   ]
  display
end

;;is called by ask turtles -- in setup-turtles
to set-initial-turtle-vars
 ;Personal characteristics
  set age 91
  rt random-float 360.0
  set life-expectancy random-normal avg-lifespan 50   ;randomly assigns lifespan of birds according to normal distribution, SD = 50

 ;info about children
  set child-1-dev -999        ;;default value for having no children

 ;bird relationship states
  set mating false
  set available true

  set search-counter 0

  set sex 1
end

;;initializes unique male variables
;;called by "ask males" -- in setup-turtles
to set-initial-male-vars
    locals [ index ]
    set index random 100

    set sex 0
    set child-counter 0     ; no kids conceived yet
    set child-2-dev -999    ; 2nd child is not developing--is non-existant
    set current-nest 0      ; male currently has no nest
    set nest-1-X -999
    set nest-2-X -999
    set nest-1-Y -999
    set nest-2-Y -999

    ifelse index < percent-polygamous
    [
        set polygamous true
    ]
    [
        set polygamous false
    ]
end

;; Set colors the turtles by sex
;; blue = male
;; red = female
;;called by ask turtles --whenever an individual turtles status is changed (mating/available or not)
to recolor-turtle
   ifelse (sex = 0)
   [
     ifelse (polygamous)        ;polygamous birds are blue, nonogamous are cyan
       [set color blue]
       [set color cyan]
   ]
   ;female birds are just red--independent of size
    [ set color red ]
end

;;;
;;; GO AND HELPERS
;;;

to go
  set clock clock + 1
  ask turtles
   [if (color = white) and not (pcolor-of patch-here = green)
      [show "whoa"
       wait 2000]]

  ask turtles
  [
    set age age + 1  ;make turtles get one year older
  ]

  ;; birds do their thing
   ask males
   [
      live-male-life
   ]
   ask females
   [
     live-female-life
   ]
   update-plots
end

;;;
;;;Male-life
;;;is called by "ask males" -- from go
to live-male-life
   if age > life-expectancy
     [die]

   ;once males are of age, become eligible for mating
   if age = 91
     [become-adult-male]

   if age > 90        ;if adult
   [
     ;;if are single perform the bachelor behavior, otherwise, deal with mating needs
     ifelse available
        [bachelor-behavior]
        [
           if mating
           [
             if child-1-dev > 0
                [stop-mating]
             if child-2-dev > 0
                [stop-mating]
           ]
        ]


     if child-counter > 0        ;;if you have kids, deal with those variables
       [male-raises-kids]
   ]
end

;;
;;Female Life
;;is called by "ask females" -- from go
to live-female-life
    if age > life-expectancy
     [
        destroy-nest
        die
     ]

   ;;if are a 91, welcome to adulthood, set variables so you can start functioning in the world
   if age = 91
     [become-adult-female]

    if age > 90        ;;if are an adult
    [
     if nest-1-X = -999            ;;if you don't have a nest
       [ find-nesting-ground ]

     if child-1-dev > -999    ;;;if have a child
       [female-raises-kid]

     if mating
     [
         if child-1-dev = 0  ;if your child has been conceived, stop mating
            [set mating false]
     ]
   ]
end


;;;
;;;BIRD BEHAVIOR FUNCTIONS
;;;

;;called by "ask females"  -- from find-nesting-ground and setup-turtles
to make-nest
   ask patch-here
   [
     set nest true
     recolor-patch
   ]
   set nest-1-X xcor
   set nest-1-Y ycor
end

;;called by "ask females" -- in live-female-life (when she dies)
;;used when female dies, to clean up her area
to destroy-nest
   ask patch-here
   [
     set nest false
     recolor-patch
   ]
   ;;kill all turtles that were living w/ mommy (not the most elegant solution--may kill fly-over birds)
   ;;in this instance, the father bird that is off looking for another child will not know of this child's death
   ;;the father will be held back for 30 clock-ticks.  We decided that that was an acceptable imperfection.
   ask other-turtles-here
   [
      die
   ]
end

;;called by ask females - in live-female-life
;;used by new adults that need a nest
to find-nesting-ground
   ;;moves bird far enough away from original nest and then starts looking for a new nest, square by square
   ifelse search-counter < min-nest-distance
   [
      fd 1
      set search-counter search-counter + 1
   ]
   [
      ;;when she finds a nest, she sets the nest-1-x and nest -1-y variables so she knows where home is and so
      ;;the rest of the program knows she has a home
      ifelse nest  ;(nest-of patch-here)
      [        ;;if there is a nest here, keep looking
         fd 1
         set search-counter search-counter + 1
      ]
      [
         set search-counter 0        ;;reset the search counter so it is ready nest time
         make-nest
      ]
   ]
end

;;called by ask-males -- from bachelor-behavior
;;determine which mate to look for (first or second) and call that function
to find-mate
  ;if you haven't mated at all, find first mate, otherwise, look for second
  ifelse child-counter = 0 and current-nest = 0
     [find-first-mate]
     [
       if child-counter = 1
          [find-second-mate]
     ]
end

;;called by ask-males -- from find-mate
to find-first-mate
   locals [single-female]
   rt random-float 360   ;;chose a direction to search in
   fd 1
   ;;if male landed on a nest and if there exactly 2 birds on it (himself and the potential mate)
   ;;and if the other turtle is a woman and available (these should be redundant, unecessary checks)

   set single-female random-one-of other-females-here
   ;;if you found a single-female and if she has her own nest
   if not (single-female = nobody) and not (nest-1-X-of single-female = -999)
   [
     if (nest-of patch-here) and (count turtles-here = 2)
     [
       if (available-of single-female = true)
       [
         ifelse nest-1-X = -999            ;;set the coordinates for this nest
         [
           set nest-1-X floor (xcor)
           set nest-1-Y floor (ycor)
           set current-nest 1 ;make the first nest your current nest
           if (nest-of patch-here = false)
             [show "damn it" wait 2000]
         ]
         [
           set nest-2-X floor (xcor)
           set nest-2-Y floor (ycor)
           set current-nest 2 ;make the second nest your current nest
         ]
         set search-counter 0
         start-mating single-female
       ]
     ]
   ]
end

;;called by ask-males -- from find-mate
to find-second-mate
  locals [single-female]
  ifelse search-counter < min-mate-vision
  [
    fd 1
    set search-counter search-counter + 1
  ]
  [
    ifelse search-counter < max-mate-vision        ;;can't stray too far from first whife
    [
      ;if there is a female here
      set single-female random-one-of other-females-here
      ifelse (nest-of patch-here) and not (single-female = nobody) and  not (nest-1-X-of single-female = -999) and (available-of single-female)
      [
        ;;if there is a female, who is available and owns her own home, start mating
        ifelse nest-1-X = -999            ;;set the coordinates for this nest
        [
          set nest-1-X floor (xcor)
          set nest-1-Y floor (ycor)
          set current-nest 1 ;make the first nest your current nest
        ]
        [
          set nest-2-X xcor
          set nest-2-Y ycor
          set current-nest 2 ;make the second nest your current nest
        ]
        set search-counter 0        ;;reset the search counter so it is ready next time
        start-mating single-female
      ]  ;;end if is available female
      [
        ;if the female here is not available, then keep on trucking my man.  keep on trucking.
        fd 1
        set search-counter search-counter + 1
      ]  ;;end else no available female
    ]    ;;end if you are close enough to home
    [
     ;if you are too far from the base nest, return home (immediately)
     set search-counter 0                      ; prepare to try again
     ifelse current-nest = 1                    ;;determine which nest to return too
     [
       set xcor nest-1-X
       set ycor nest-1-Y
       rt 90                                ;;prepare to try again, in a new direction
     ]
     [
       if current-nest = 2
       [
         set xcor nest-2-X                       ; if you came from the second nest looking for a new first, go back and try again
         set ycor nest-2-Y                       ; in a new direction
         rt 90
       ]
     ]
   ] ;;end else too far from home
 ]   ;;end else far enough from home to look for a nest
end

;;called by ask males - in live-male-life
;;baby birds become adults by re-setting variables so they will be able to function as adults
to become-adult-male
  recolor-turtle
  st        ;;babies were hidden, show them now--they are adults
  set available true            ;;can now go looking for a mate
  set nest-1-X -999             ;;set nest variables so you don't know where home is...have to find a new one
  set nest-1-Y -999
  set nest-2-X -999
   set nest-2-Y -999
   recolor-turtle
end

;;called by ask males - in live-male-life
;;if you are single and there are still single-females
;;find a mate, as necessary
;;only polygamous birds can have multiple children
to bachelor-behavior
  if available-females > 0
  [
     ifelse polygamous
     [
       if child-counter < 2
          [find-mate]
     ]
     [
       if child-counter < 1
          [find-mate]
     ]
   ]
end

;;called by ask males -- in live-male-life
;;father bird must increment his children's ages and possibly let them go (if they are all grown up)
to male-raises-kids
  if (child-1-dev + 1) = -4
  [
   ;  show "hellllllloooooo"
   ;  wait 20000
  ]
  if child-counter > 0 and child-1-dev > -999    ;;;if have a child-1
  [
     set child-1-dev child-1-dev + 1  ;;increment age counter
     if (child-1-dev > 90)              ;;if you're baby is all grown up
     [
       set child-1-dev -999           ;;you are no longer caring for this baby
       set child-counter child-counter - 1          ;;you are responsible for one less baby
       set nest-1-X -999              ;;no longer have to keep track of this nest
       set nest-1-Y -999
       ifelse not (nest-2-X = -999)        ;;if you have a second nest, make it the primary nest
         [set current-nest 2]
         [set current-nest 0]                ;;otherwise, you have no current nest
     ]
  ]

  if child-counter > 0 and child-2-dev > -999    ;;;if have a child-2
  [
     set child-2-dev child-2-dev + 1  ;;increment age counter
     if (child-2-dev > 90)            ;;if you're baby is all grown up
     [
        set child-2-dev -999        ;;you are no longer caring for this baby
        set child-counter child-counter - 1          ;;you are responsible for one less baby
        set nest-2-X -999                    ;;no longer have to keep track of this nest
        set nest-2-Y -999
        ifelse not (nest-1-X = -999)        ;;if you have a first nest, make it the primary nest
          [set current-nest 1]
          [set current-nest 0]                ;;otherwise, you have no current nest
     ]
  ]
end

;;called by ask females -- in live-female-life
;;reset baby's variables so she can function as a grown female bird
to become-adult-female
  st            ;;babies start hidden, now you are visible
  recolor-turtle
  set available true
  set available-females available-females + 1
  ;;set nest variables so you don't know where home is
  set nest-1-X -999
  set nest-1-Y -999
  recolor-turtle            ;;so baby is not white (the inherited color)
end

;;called by ask females -- in live-female-life
;;mama bird has to let child age, and possibly have child leave the nest
to female-raises-kid
  set child-1-dev child-1-dev + 1
  if child-1-dev > 90            ;;if you're baby is all grown up
  [
    set child-1-dev -999        ;;you are no longer caring for this baby
    set available true          ;;you can now mate again
    set available-females available-females + 1
    recolor-turtle            ;;set mama back to red
   ]
end

;;called by ask males -- in find-first-mate and find-second-mate
;;used to set variables to let the world know the couple is mating
;;conceive the baby
to start-mating [single-female]
   locals [ gender only-child ]
   set gender random 2        ;;find gender of new baby
   set only-child not (polygamous)

   ;If female, then set variables for baby--if the baby is female, clone yourself
    ask single-female
    [
      set child-1-dev -5
      if gender = 1
        [conceive-baby 1 only-child]
      ;;female is not longer available
      set available false
      set available-females available-females - 1

      set color white
      set mating true
    ]

    ;Males also set the variables for the baby--if newborn is male, the father clones himself
   ;;determine if this is the first or second baby
    ifelse child-1-dev = -999
     [set child-1-dev -5]
     [
       if child-2-dev = -999
         [set child-2-dev -5]
     ]
     if gender = 0   ;;if new baby is a male, father clones himself
         [conceive-baby 0 only-child]
     set child-counter child-counter + 1        ; are now responsible for one more child
     set available false                        ; male is no longer available
     set mating true
     set color white
end

;;called from start-mating
;;clones the appropriate parent
;;gender of the parent is passed in so baby knows its sex
;;whether or not the father is polygamous is passed in as it affects baby's life-expectancy
to conceive-baby [gender only-child]
   locals [baby-count]

   ;;if parents can have multiple babies, determine how many they should have (1 or 2)
   ;;otherwise they have one
   ifelse twins
   [
      set baby-count 1 + random 2
   ]
   [
      set baby-count 1
   ]
   hatch baby-count
   [
      rt random-float 360            ;;choose a direction to look in
      set age -5                    ;-age because is not born, yet
      set available false        ;baby is not available for mating
      set mating false            ;;baby is currently not mating
      ;;only children have different life-expectancies than children w/ polygamous fathers
      ifelse only-child
         [set life-expectancy random-normal only-child-life-expectancy 50]
         [set life-expectancy random-normal multiple-child-life-expectancy 50]
      set sex gender
      set child-1-dev 0
      set search-counter 0
      if gender = 0            ;;if a boy baby, reset those variables
      [
         set child-counter 0
         set child-2-dev 0
         set nest-2-X  -999
         set nest-2-Y  -999
         set current-nest 0        ;;are not responsible for any nests
      ]
      ht                            ;don't want to see turtle, until grown up
   ]
end

;;called by ask males -- in live-male-life
to stop-mating
   set available true
   set mating false
   recolor-turtle
end


;;;
;;; PLOTTING
;;;

to setup-plots
  set-current-plot "Population by Sex"
  set-plot-y-range 0 num-birds
  set-current-plot "Total Population"
  set-plot-y-range 0 num-birds
  set-current-plot "Reproduction Strategy"
  set-plot-y-range 0 num-birds
end

to update-plots
  update-gender-population-plot
  update-population-plot
  update-strategy-plot
  ;update-lorenz-and-gini-plots
end

;; this does a line plot of the number of people of each class
to update-gender-population-plot
  set-current-plot "Population by Sex"
  set-current-plot-pen "females"
  plot count females
  set-current-plot-pen "males"
  plot count males
end

to update-population-plot
  set-current-plot "Total Population"
  set-current-plot-pen "default"
  plot count turtles
end

to update-strategy-plot
  locals [ counter ]
  set counter 0
  ask males
  [
      if polygamous
      [ set counter counter + 1 ]
  ]
  set-current-plot "Reproduction Strategy"
  set-current-plot-pen "Monogamous"
  plot count males - counter
  set-current-plot-pen "Polygamous"
  plot counter
end

to-report count-nests
  locals [ counter ]
  set counter 0
  ask patches
  [
     if nest
       [set counter counter + 1]
   ]
   report counter
end


@#$#@#$#@
GRAPHICS-WINDOW
237
10
657
451
20
20
10.0
1
10
1
1
1

CC-WINDOW
656
44
939
450
Command Center

BUTTON
20
57
109
90
setup
setup
NIL
1
T
OBSERVER

BUTTON
143
55
227
89
go
go
T
1
T
OBSERVER

SLIDER
18
103
186
136
num-birds
num-birds
2
1000
150
1
1
NIL

MONITOR
42
481
129
530
time elapsed
clock
3
1

SLIDER
18
183
186
216
min-mate-vision
min-mate-vision
0
10
1
1
1
NIL

SLIDER
18
216
187
249
max-mate-vision
max-mate-vision
0
40
7
1
1
NIL

PLOT
225
459
441
609
Population by Sex
Time
Number of Birds
0.0
100.0
0.0
100.0
true
false
PENS
"Females" 1.0 0 -65536 true
"Males" 1.0 0 -16776961 true

MONITOR
145
481
202
530
Nests
count-nests
3
1

PLOT
448
459
648
609
Total Population
Time
Num Birds
0.0
100.0
0.0
100.0
true
false

SLIDER
18
260
186
293
min-nest-distance
min-nest-distance
1
30
10
1
1
NIL

SLIDER
18
150
187
183
percent-polygamous
percent-polygamous
0
100
10
5
1
NIL

PLOT
655
458
883
609
Reproduction Strategy
Time
Number
0.0
100.0
0.0
100.0
true
false
PENS
"Polygamous" 1.0 0 -16776961 true
"Monogamous" 1.0 0 -16711681 true

SLIDER
18
311
229
344
only-child-life-expectancy
only-child-life-expectancy
100
730
480
20
1
NIL

SLIDER
18
343
229
376
multiple-child-life-expectancy
multiple-child-life-expectancy
100
720
360
20
1
NIL

SWITCH
18
393
121
426
twins
twins
0
1
-1000

@#$#@#$#@
Notes from Victor and Leema
---------------------------

The model which we have build is based on information we had recently discovered in Diamond's Why is Sex Fun? The evolution of human sexuality 1997).  In Diamond's book, he describes the reproductive strategies of the Pied Flycatcher, a small bird that has been found in Europe.  The Pied Flycatcher is a bird which engages in polygamy as part of its reproductive strategy.  In the flycatcher's system of reproductive behavior, males can be either monogamous or polygamous, and females will choose to mate with a male regardless of his orientation.  As part of the strategy, once the polygamous male has fertilized an egg of his first mate, he will seek a secondary mate that is a minimum distance from their first mate and mate with her.  Once this is completed, he will return to the first mate and aid in child rearing by bringing food.  As a secondary priority, the male bird will aid in the child rearing of the second nest, which consequently yields smaller and weaker birds that have a reduced chance of survival.  It appears that through evolution, this polygamous behavior has shown to be effective and perpetuated itself through time, though it does not totally eliminate monogamous behavior.



Overview of a bird's life
-------------------------
Our program is run on clock-ticks--on each clock-tick the birds age and, if necessary, move the distance of one patch.  While programming this, we found it easiest to think about clock-ticks as days.

Regardless of gender, these first 90 days are the same for all birds:  sit with your mother and do nothing while you mature.  Once they reach 91 days old everything changes:
- Female birds go look for an available patch in which they will spend the rest of their natural lives, waiting for a mate or raising their young (we too abhor the sexist nature of this, but it works in the model) .  Note that female birds are only available to mate when they have no young in their nest.  So, once a male has mated with the female, she will be unavailable for 95 days (5 days for incubation and 90 days for the baby's maturation).
- Male monogamous birds begin their lives by looking for a mate (a single available female sitting in an empty nest).  Once they find one, they too sit with the baby ("caring" for it) for 95 days.  When the baby is all grown up the male bird will fly off, looking for a new mate.  Every male child of a monogamous bird inherits the monogamy gene.
- Male polygamous birds also begin by looking for a mate.  Once they have found one, they sit with her for 5 days--for incubation.  When that is complete, they go wandering looking for a second wife.  If they find a 2nd mate, they mate with her and stay there until their first child is 90 days old (they can only "care for" 2 nests at a time).  When the first child is grown up this cheating male can go off and find a 3rd mate where he will remain until the 2nd child reaches adulthood.  This cycle continues.  Every male child of a polygamous child inherits the polygamy gene.
Note that as all searches take multiple clock-ticks (a bird can only move one patch per tick), a male bird will never find a new mate every 90 (or 95) ticks.  Instead it is more like one mating session every 100-150 clock-ticks.

Also, note that in the physical bird communities, children of polygamous fathers may not survive babyhood because the father is unable to care for all his young.  Given our time constraints, we were unable to model the complexity of finding food, feeding all the young and mating.  We used the life-expectancy variables to attempt to mimic this feature of bird communities.


A key:
------
- Black patches are available for nesting
- Green patches are nests
- Red turtles are available females
- White turtles are unavailable (they are either mating males or females that are taking care of their young)
- Blue turtles are polygamous males
- Cyan (light blue) turtles are monogamous males

In order to allow users to investigate the various aspects that affect our population, we created the following controls:
- num-birds -- controls the size of the starting population.
- percent-polygamous -- percent of male birds that can have 2 nests (for a total of up to 4 children), in the starting population.  In subsequent generations all male children of polygamous fathers will also be polygamous.
- only-child-life-expectancy -- the average life-expectancy for children of monogamous fathers.  In the bird communities we referenced for this model, children of monogamous fathers have a greater life expectancy because they are taken better care of than children of polygamous fathers.  The life-expectancy is assigned on a normal-curve with this value being as the mean.  Note that first-generation birds have the same (general, it is also on a normal-curve) life-expectancy.
- multiple-child-life-expectancy -- the average life-expectancy for children of polygamous fathers.
- min-mate-vision -- this is the minimum distance that polygamous males must move before they can begin searching for a second mate (in the physical bird communities he can not be too close to the first mate or else the females will fight over him).
- max-mate-vision -- this is the maximum distance that  a polygamous male's second nest can be from his first nest (in the physical bird communities have nests too far apart means that the children of one die immediately).
- min-nest-distance -- this is the distance that young-females must travel before they can set up a nest.  In general the young-female will fly in a straight line until she finds an open patch that is at least this far from their mother's nest.
- twins (on/off) -- if this switch is on each mating will produce either 1 or 2 children.  If this switch is off each mating will produce exactly 1 child.

Some interesting phenomenon we found while playing with the controls:
- In a strictly monogamous population (with twins on), it is interesting to see how the only-child-life-expectancy affects the population growth (if twins are off it simply dies off because parents are not replacing themselves).  We found that a only-child-life-expectancy of 420 would result in a stable (albeit high) population -- this age allows gives each couple the potential of having between 3 and 6 children.  Although given the search times it is probably closer to 2 (hence the stability).
- If the community has any percentage of polygamous birds they will take over, regardless of the difference in live-expectancies.
- If you set the min-nest-distance very high it slows down the population growth because the female birds are spending so much time looking for nests instead of being available to mate.
- Setting the min-nest-distance to a low value creates come interesting nest clumps.

Just a note
-----------
Shortly after we completed this model we saw Mirah and Tiffany's code.  They have an excellent solution wherein individual turtles maintain lists of the turtles they know.  Using these lists their turtles can essentially "talk to each other."  After seeing this code, we realized that it would've been an excellent approach for approaching our model.  The male turtles could have maintained 2 lists (one for mates and one for children) and eliminated the child-1-dev, child-2-dev and child-counter variables (if not more).  This change would not have necessarily changed the model functionality but it would have made the code more readable.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.0alpha1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
