breeds [balls paddles prizes]

globals [
  score
  cur-level ;; current level
  lives
  initial-paddle-size ;;length of paddle
  x-diff ;horizontal dstance between the paddle center and the mouse
  y-diff ;vertical distance between the paddle center and the mouse
  old-score ;compare scores etween rounds to see if extra life has been earned
  size-increase-for-power-ball ;;value to add to size variable when ball is powered up
]

patches-own [
  level ;;how many hits from a normal ball before destoryed
  point-value ;;points earned for destroying the brick
  partner ;; used to make bricks two patches wide
]

balls-own [
  ;;if the ball makes a horizontal or vertical collision, set
  ;;flip-x or flipy, respectively, to -1 to signify
  ;;an orientation change should occur when new heading is set
  flip-x
  flip-y
]

;;;;;;;;;;;;;;;;;;;;;;;;
;;;                  ;;;
;;; SETUP PROCEDURES ;;;
;;;                  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

to new-game
  ca
  set lives 5
  ;;prepare to move up to the intial level
  set cur-level initial-level - 1
  ;;set how much bigger ball gets with each strength power up
  set size-increase-for-power-ball 0.2
  ;; build the paddle
  setup-paddle
  ;;increment cur-level the next level and build the brick wall
  setup-next-level
  ;;create pit to catch missed balls
  ask patches [
    if (pycor = (- screen-edge-y)) [
      set pcolor gray
    ]
  ]
end

to setup-paddle
  set initial-paddle-size paddle-size
  ;;create turtles for paddle
  ;;then set breeds and position paddle
  ask patches [
    if (pycor = (- screen-edge-y)) and (abs pxcor) <= initial-paddle-size [
      sprout 1 [
        set breed paddles
        set shape "paddle"
        set color blue
        set heading 0
        fd 2
      ]
    ]
  ]
end

to setup-next-level
  ;;remove all balls from active play
  ;;increment lives by one for each ball that was in play
  ask balls [
    set lives lives + 1
    die
  ]
  ;;increment level
  set cur-level cur-level + 1
  ;;create bricks in the new level's wall and assign them partners
  ;; to make the bricks two patches wide
  ask patches [
    if random-float 1.0 < density [
      if(pycor > 0) and (pycor < screen-edge-y - 2) and (abs pxcor < screen-edge-x) [
        ;;bricks on even patches partner to the right, those on odd patches to the left
        ifelse pxcor mod 2 = 0 [
          set partner pxcor + 1
        ][
          set partner pxcor - 1
        ]
        set level cur-level ;set the number of hits to destroy the brick
        set point-value 100 * level ;;set points recieved for destroying the brick
        set pcolor hsb (level / cur-level) 1 1 ;;color the bricks by percent of max health
      ]
    ]
  ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                     ;;;
;;; RUN TIME PROCEDURES ;;;
;;;                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

to play
  ;;check if a ball should be released
  if (mouse-down? and (count balls = 0)) [release-ball]

  move-paddle
  ask balls [move-balls]
  move-prizes

  ;;extra life every 40000 points
  if score mod 40000 < old-score mod 40000 [
    set lives lives + 1
  ]
  set old-score score

  ;;end game when no balls are left
  if count balls + lives = 0 [ stop ]

  ;;increment level when board is clear
  if (count patches with [level > 0]) = 0 [ setup-next-level ]
end

to release-ball
  ;;release ball near the middle of the screen
  ;;with an rientation between -60 and 60 or,
  ;; in other words, heading towards the brick wall
  ifelse lives > 0 [
    ask patch 0 ((- screen-edge-y) / 2 )[
      sprout 1 [
        set breed balls
        set shape "super ball"
        set color white
        set heading (random-float 120) - 60
      ]
    ]
    set lives lives - 1 ;;decerement lives for released ball
  ] [
    stop
  ]
end

to move-paddle
  ;;record the difference in position between mouse and paddle
  set x-diff round mouse-xcor - (xcor-of turtle initial-paddle-size)
  set y-diff round mouse-ycor - (ycor-of turtle initial-paddle-size)

  ask paddles [
    ;;move horizontally
    ifelse( x-diff < 0 ) [set heading 270] [set heading 90]

    ;;prevent left/right oscillation
    ;;by requiring difference between paddle and mouse of at least one paddle step
    ;;before movemnt can occur
    ifelse abs x-diff < paddle-speed [set x-diff 0]  [ fd paddle-speed]

    ;;move vertically in a set range and prevent up/down oscillation
    ;;by requiring difference between paddle and mouse of at least one half paddle step
    ;;before movemnt can occur (the paddle is only one half patch tall
    ifelse( 2 * y-diff < (- paddle-speed) ) [
      set heading 180
      ifelse ycor > (1 - screen-edge-y) [ fd paddle-speed ] [set y-diff 0 ]
    ] [fd 0]
    ifelse (2 * y-diff > paddle-speed )[
      set heading 0
      ifelse ycor < 4 - screen-edge-y [ fd paddle-speed ] [set y-diff 0 ]
    ] [fd 0]

    ;;hide wrap around
    ifelse (floor abs (xcor - xcor-of turtle initial-paddle-size)
           > initial-paddle-size) [ht] [st]

    ;; hit balls and catch prizes
    if not (hidden?) [
      ask balls-here [
        set heading atan (dx + x-diff) ( abs dy + y-diff)
        fd 1
      ]
      ;;let prizes that have landed on the paddle have their effect
      ask prizes-here [
        if shape = "dollar sign" [set score score + 5000]
        if shape = "multi ball" [set lives lives + 1 release-ball]
        if shape = "power up ball" [ask balls [set size size + size-increase-for-power-ball] ]
        die
      ]
    ]
  ]
end

to move-prizes
  ;;drop down the prizes
  ask prizes [
    fd ball-speed / 2
    if pcolor = gray [die]
  ]
end

to move-balls ;;Turtle Procedure
  ;;use local variable so patch procedure can access power easily
  let power round (1 + (size - 1) / size-increase-for-power-ball)
  ;;check if landed in ball pit
  ;;if, so remove ball from play
  if pcolor = gray [die]

  ;;check if we hit a wall
  wall-check
  ;;collide east and west using DX and DY to estimate future position
  ;;record orientation change in flip-x
  if (abs (round xcor + dx)) <= screen-edge-x [
    if value-from (patch-at dx 0) [level] > 0 [
      ask patch-at dx 0 [ take-wide-hit power]
      set flip-x -1
    ]
  ]
  ;;use atan to incorporate orientation change
  ;;then move forward one ball step
  set heading atan (flip-x * dx) (flip-y * dy)
  fd ball-speed

  ;;check if we hit a wall
  wall-check
  ;;collide north and south using DX and DY to estimate future position
  if value-from (patch-at 0 dy) [level ] > 0 [
    ask patch-at 0 dy [ take-wide-hit power]
    set flip-y -1
  ]
  ;;use atan to incorporate orientation change
  ;;then move forward one ball step
  set heading atan (flip-x * dx) (flip-y * dy)
  fd ball-speed

  ;;check if we hit a wall - sets flip-x and flip-y if so to -1, 1 if not
  wall-check
  ;;collide with the paddle
  if(count paddles-here > 0) [
    ;;use atan to incorporate orientation change
    ;;then move forward one ball step
    set heading atan (flip-x * dx) (abs dy)
    fd ball-speed * 5
  ]
  ;;add noise to the ball's motion
  rt (random-float 0.25) - 0.125
end

;;;wall and ball collision
to wall-check ;;Turtle Procedure
  ;;allow balls to collide with walls and ceiling
  set flip-x 1
  set flip-y 1
  ;;set flip values to change orientation
  ;;if wall will be hit in next ball step
  if ycor + dy >= screen-edge-y [set flip-y -1 ]
  if abs (xcor + dx) >= screen-edge-x [set flip-x -1 ]
end

;;;brick and ball collision
to take-wide-hit [ball-power] ;;Patch Procedure
  ;;called by the first brick patch hit by a ball
  ;;manages entire brick hit
  set level level - ball-power ;;more powerful balls cause extra damage
  take-hit ;;change color, possibly being destroyed and award points
  ;;if partner is still present, have them get hit
  ask patch partner pycor [
    if level > 0 [
      set level level - ball-power
      take-hit
    ]
  ]
  ;;release a prize with probability PRIZE-CHANCE
  if (level <= 0) and (random-float 1.0 < prize-chance) [release-prize]
end

to take-hit  ;;Patch Procedure
  ;;called with ball and brick collision
  ;;change block colorand award points as needed
  ifelse level <= 0 [
    set pcolor black
    set score score + point-value / (1 + initial-paddle-size)
  ] [
    set pcolor hsb (level / cur-level) 1 1
    set score score + 50
  ]
end

to release-prize ;;Patch Procedure
  ;;create a prize with a randomly choosen effect to be "released"
  let effect random-float 1.0
  sprout 1 [
    set breed prizes
    if(effect < 0.33) [set shape "dollar sign"]
    if(effect > 0.33 and effect < 0.66) [set shape "multi ball"]
    if(effect > 0.66) [set shape "power up ball"]
    set heading 180
  ]
end


; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.

; Permission to use, copy, or modify this software and its documentation for
; educational and research purposes only and without fee is hereby granted,
; provided that this copyright notice and the original authors' names appear
; on all copies and supporting documentation. For any other uses of this
; software, in original or modified form, including but not limited to
; distribution in whole or in part, specific prior permission must be
; obtained from Uri Wilensky. These programs shall not be used, rewritten,
; or adapted as the basis of a commercial software or hardware product
; without first obtaining appropriate licenses from Uri Wilensky. We make no
; representations about the suitability of this software for any purpose. It
; is provided "as is" without express or implied warranty.
@#$#@#$#@
GRAPHICS-WINDOW
121
10
689
419
15
10
18.0
1
5
1
1
1
0

CC-WINDOW
5
539
698
634
Command Center

BUTTON
9
42
115
75
NIL
new-game
NIL
1
T
OBSERVER
T
NIL

SLIDER
177
459
349
492
paddle-size
paddle-size
0
3
1
1
1
NIL

BUTTON
9
75
115
108
NIL
play
T
1
T
OBSERVER
T
NIL

SLIDER
177
492
349
525
ball-speed
ball-speed
0
0.3
0.08
0.01
1
NIL

MONITOR
9
157
98
206
NIL
score
0
1

MONITOR
9
206
98
255
NIL
lives
0
1

SLIDER
177
426
349
459
paddle-speed
paddle-speed
0.01
1.0
1.0
0.01
1
NIL

SLIDER
14
459
177
492
density
density
0
1
1.0
0.01
1
NIL

MONITOR
9
108
98
157
Level
cur-level
0
1

SLIDER
14
492
177
525
prize-chance
prize-chance
0.0
1.0
0.25
0.01
1
NIL

TEXTBOX
11
257
101
287
Extra Ball every 40,000 points

SLIDER
14
426
177
459
initial-level
initial-level
1
10
1
1
1
NIL

TEXTBOX
16
394
106
412
Game Settings

@#$#@#$#@
WHAT IS IT?
-----------
This is a version of the age old computer game called breakout.  The goal of the game is to break as many red bricks as possible in the wall.  You are given a movable paddle and a ball.  When the ball touches a brick it breaks the brick.  You move the paddle to catch the ball on its bounce and send it back towards the wall.

This model is created to serve as a demonstration of game programming in NetLogo.


HOW IT WORKS
-----------------------
Objective:
To destroy the brick wall.  Once the wall is destroyed, you will advance to the next level.  The level of difficulty increases by one for each round.  For example, at level one you only need one hit to destroy a brick, whereas at level two you need two hits and at level three you need three hits.

Controls:
Click in the graphics window to release your ball.

Move the mouse inside the graphics window to move the paddle. The paddle has the ability to move right, left, up and down.

The direction the paddle is moving when the ball hits will affect the path and speed of the ball.

Scoring:
50 points for every time a brick is hit
100 * level / (1 + paddle-size) points for each brick destroyed
Extra ball every 40000 points

Prizes:  Some bricks contain a prize.  Destroying a brick releases its prize.  Catching a prize with your paddle gives the following rewards:

Dollar Sign - Adds 5000 points to your score
Large Ball - Makes your balls larger and more powerful.
Multi-ball - Releases an additional ball


HOW TO USE IT
-------------
Buttons:
  NEW-GAME: Setup a new game
  PLAY: Run the game.  May be used to pause and resume play.

Monitors:
  LEVEL: Your current level
  SCORE: Your current score
  LIVES: The number of lives you have remaining

Sliders: (use to customize the game)
  INITIAL-LEVEL:  The level you start at.
  DENSITY:  The density of the brick wall.
  PRIZE-CHANCE:  The chance a brick will contain a prize.
  PADDLE-SPEED:  How fast the paddle moves
  PADDLE-LENGTH: The length of the paddle
  BALL-SPEED:  How fast balls and prizes move


THINGS TO NOTICE
----------------
Any prizes that are falling when you destroy the last brick in a level will keep falling in the next level.  Try to release your ball as soon as possible in the new level to take advantage of the power up abilities of the falling prize.

Every ball you are playing with when a level is completed disappears and increases your number of lives by one.  To earn extra lives, beat a level with multiple balls in play.


THINGS TO TRY
-------------
1. Attempt to get the highest score you can.  What strategies are most successful?  How does your strategy change as the levels increase?

2. Use the command center to get extra lives.

3. Examine what happens to the game speed when you change the paddle size.  What causes this?

4. Turn off EXACT TURTLE POSITIONS in the graphics window.  How does this affect the game?  Why?


EXTENDING THE MODEL
-------------------
1.  Modify the SETUP-NEXT-LEVEL procedure to cause the blocks to start with a random strength level less than or equal to that of the current level.

2.  The balls currently move at a fixed distance with each turn, independent of their heading.  Modify the model to obey the laws of physics.


NETLOGO FEATURES
----------------
In order to prevent the paddle from rapidly oscillating around the mouse, MOVE-PADDLE places a threshold on how large X-DIFF and Y-DIFF must be before the paddle will more horizontally or vertically.

When setting the POWER in MOVE-BALLS variable we must use the ROUND primitive to prevent error from floating point arithmetic does not cause collisions in TAKE-WIDE-HIT to give bricks non-integer levels.

Collision detection is performed using the DX and DY primitives in the MOVE-BALLS procedure to anticipate where the ball will go.  This helps to avoid situations where we have a ball on top of other objects.

Turtle sizes are used to allow the balls to grow larger as they become more powerful.

Exact turtle positions are used to allow smooth movement of the balls.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002). NetLogo Breakout model. http://ccl.northwestern.edu/netlogo/models/breakout. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston,IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

dollar sign
true
0
Rectangle -7566196 true true 136 0 164 297
Rectangle -7566196 true true 75 239 225 268
Rectangle -7566196 true true 74 135 225 165
Rectangle -7566196 true true 75 31 225 60
Rectangle -7566196 true true 105 58 104 58
Rectangle -7566196 true true 74 59 104 135
Rectangle -7566196 true true 194 165 225 238

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

multi ball
true
0
Circle -7566196 true true 29 29 120
Circle -7566196 true true 152 31 119
Circle -7566196 true true 89 134 121

paddle
false
0
Rectangle -7566196 true true 4 4 319 74

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

power up ball
false
0
Circle -7566196 true true 54 54 191

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

super ball
true
0
Circle -7566196 true true 104 -1 92

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
new-game
@#$#@#$#@
@#$#@#$#@
