globals [ ticks ]

breeds [commuters]

commuters-own [ speed speed-max speed-min phase travel-offset leave arrive travel-time moving? subway? my-avg-travel-time]
patches-own [ road? home? work? ]

to setup-road
  ask patches [
    ifelse ((pycor = -1) or (pycor = 1)) [
      set pcolor black
      set road? true
      set home? false
      set work? false
    ]
    [
      set pcolor brown
      set road? false
      set home? false
      set work? false
    ]
  ]
end

to setup-home
  ask patches [if (((pycor <= 1) and (pycor >= -1)) and (pxcor = ( -1 * screen-edge-x))) [
    set pcolor green
    set road? false
    set home? true
  ]]
end

to setup-work
  ask patches [if (((pycor <= 1) and (pycor >= -1)) and ( pxcor = screen-edge-x ) ) [
    set pcolor red
    set road? false
    set work? true
  ]]
end

to setup-commuters
  set-default-shape commuters "car"
  cct-commuters number-of-commuters [
     set color blue
     set xcor -1 *  screen-size-x / 2 + 0.5
     set ycor -1  
     set heading  90
     ;;; set initial speed to be in range 1.0 ;;;; to 3.0  
     set speed 0.0 ;;;  + random 2.0
     
     ;; all commuters originally try and drive
     set subway? false
     
     ;; collect information about avg-travel-time
     set my-avg-travel-time -1
     
     ;; determine max speed of this commuter when driving
     ifelse (same-max-speed?) [
       set speed-max 0.75
     ]
     [
       set speed-max ( 0.5 + random-float 0.5 )
     ]
     set speed-min 0.1   ;; the minimum speed of the commuter
     set phase 1  ;; commuter starts out at home
     
     ;; setup an offest from the normal travel time
     ifelse (same-travel-time-of-day?) [
       set travel-offset 0
     ]
     [
       set travel-offset ( random 240 ) - 120
     ]
   ]
end

to setup
  ca
  set ticks 300

  setup-road
  setup-home
  setup-work
  setup-commuters

end

; this function is needed so when we click "Setup" we
; don't end up with any two commuters on the same patch
to separate-commuters  ;; turtle procedure
  if any? other-turtles-here
    [ fd 1
      separate-commuters ]
end

to go
  set ticks ticks + 1

  ask commuters [
    
    ;; determine if we should be moving to or from work
    determine-phase
    
    ifelse (phase = 1) [
      ;; move toward work
      ifelse (not (work? = true)) [
        commute
      ]
      [
        if (not (speed = 0)) [
          arrive-at-work
        ]
      ]
    ]
    [
      ;; move toward home
      ifelse (not (home? = true) ) [
        commute
      ]
      [
        if (not (speed = 0)) [
          arrive-at-home
        ]
      ]
    ]
  ]
  plot-commuters
end

;; This procedure resets the turtle when they arrive home
to arrive-at-home
  set speed 0
  set arrive ticks
  set travel-time arrive - leave
  set ycor ycor - 2
  set xcor (-1 * (screen-size-x / 2)) + 0.5
  set heading 90
  
  ;; update my-avg-travel-time
  ;; if its -1 this is the first time we've set it
  ifelse (my-avg-travel-time = -1) [
    set my-avg-travel-time travel-time
  ]
  [
    set my-avg-travel-time (past-discount-value * my-avg-travel-time) + ( ( 1 - past-discount-value) * travel-time )
   ]
  
  decide-on-subway
end

;; This procedure resets the commuter when they arrive at work
to arrive-at-work
  set speed 0
  set arrive ticks
  set travel-time arrive - leave
  set ycor ycor + 2
  set xcor (screen-size-x / 2) - 0.5
  set heading -90
  
  ;; update my-avg-travel-time
  ;; if its -1 this is the first time we've set it
  ifelse (my-avg-travel-time = -1) [
    set my-avg-travel-time travel-time
  ]
  [
    set my-avg-travel-time (past-discount-value * my-avg-travel-time) + ( ( 1 - past-discount-value) * travel-time )
   ]
end

;; report on the number of people taking the subway
to-report count-subway-riders
  report count commuters with [subway? = true]
end

;; this procedure has the commuter decide if they should use the subway next time
to decide-on-subway
  
  ;; if we are already taking the subway decide if we should drive  
  let rv random-float 1.0
  ifelse ((subway? = true) and (rv < probability-exploring)) [
        set subway? false
  ]
  [
    ;; if the last travel-time of this commuter was greater than the subway time
    ;;   then switch to using the subway
    if (my-avg-travel-time > subway-time) [
      set subway? true
    ]
  ]
end

;; this causes the commuter to take the subway
to take-subway

  ;; move the commuter
  set xcor ((phase * (screen-size-x / 2)) + (-1 * phase * 0.5))
  set ycor ycor + (phase * 2)
  set heading -1 * phase * 90

  ;; update the stats
  set arrive leave + subway-time
  set travel-time arrive - leave
  
  ;; also check to see if we should take subway next time
  if (phase = -1) [
    decide-on-subway
  ]
  
  ;; update my-avg-travel-time
  ;; if its -1 this is the first time we've set it
  ifelse (my-avg-travel-time = -1) [
    set my-avg-travel-time travel-time
  ]
  [
    set my-avg-travel-time (past-discount-value * my-avg-travel-time) + ( ( 1 - past-discount-value) * travel-time )
   ]
end

;; move either on the subway or drive the commuter
to commute
  ifelse (subway? = true) [
    take-subway
  ]
  [
    drive-car
  ]
end

to drive-car
  let can-move? false

  ;; see if there are any other commuters right here
  ifelse (any? (turtles-at 0 0) with [speed > 0]) [

    ;; check to see if any of the other commuters are moving if not we can
    if (not any? (turtles-at 0 0) with [moving? = true]) [
      set can-move? true
    ]
  ]
  [
    set can-move? true
  ]
  
  if can-move? [
    set moving? true
    ifelse any? turtles-at 1 0
      [ set speed (speed-of one-of turtles-at 1 0)
        slow-down-commuter ]
      ;; otherwise, speed up
      [ speed-up-commuter ]
    ;;; don't slow down below speed minimum or speed up beyond speed limit
    if speed < speed-min  [ set speed speed-min ]
    if speed > speed-max   [ set speed speed-max ]
    fd speed
    set moving? false
  ]
end

to determine-phase

  ;; going to work
  if ( ( ticks - ( 450 + travel-offset ) ) mod 1440 = 0) [
     set phase 1
     set leave ticks
  ]
  
  ;; going home
  if ( ( ticks - ( 1020 + travel-offset ) ) mod 1440 = 0 ) [
     set phase -1
     set leave ticks
  ]  
end

to slow-down-commuter  ;; turtle procedure
  set speed speed - ( slow-down / 1000 )
end

to speed-up-commuter  ;; turtle procedure
  set speed ( speed + ( speed-up / 10000 ) )
end

to-report count-commuters-on-road
  locals [total road-patches]
  set total 0
  set road-patches patches with [ road? = true ]
  ask road-patches [
    set total total + count turtles-here
  ]
  report total
end

to plot-commuters
  set-current-plot "Cars On Road"
  set-current-plot-pen "Number of Cars"
  plot count-commuters-on-road
end

to-report time
  locals [days hours mins temp place1 place2]
  
  set place1 ""
  set place2 ""
  
  set days int (ticks / 1440)
  set temp ticks - ( days * 1440 )
  set hours int ( temp / 60 )
  set mins int ( temp - (hours * 60) )
  
  if (hours < 10) [
    set place1 "0"
  ]
  
  if (mins < 10) [
    set place2 "0"
  ]
  
  report "Day " + days + ", " + place1 + hours + ":" + place2 + mins
end

to-report average-travel-time 
  locals [avg-travel-time]
  
  ask commuters [
    set avg-travel-time avg-travel-time + travel-time
  ]
  set avg-travel-time avg-travel-time / count commuters
  
  report avg-travel-time
end

; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1997 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1997).  NetLogo Traffic Basic model.
; http://ccl.northwestern.edu/netlogo/models/TrafficBasic.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/TrafficBasic
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
283
315
696
411
15
2
13.0
1
10
1
1
1
0
1
0
1

CC-WINDOW
5
434
705
529
Command Center
0

BUTTON
175
142
247
183
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
176
193
247
233
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
5
15
269
48
number-of-commuters
number-of-commuters
1
41
20
1
1
commuters

SLIDER
15
193
142
226
slow-down
slow-down
0.0
99.0
50.0
1.0
1
NIL

SLIDER
15
146
142
179
speed-up
speed-up
0.0
99.0
50.0
1.0
1
NIL

SWITCH
51
57
223
90
same-max-speed?
same-max-speed?
1
1
-1000

SWITCH
28
95
246
128
same-travel-time-of-day?
same-travel-time-of-day?
1
1
-1000

PLOT
283
10
676
233
Cars On Road
Ticks
Number of Cars
0.0
10.0
0.0
10.0
true
false
PENS
"Number Of Cars" 1.0 0 -16777216 true

MONITOR
340
250
436
299
Day and Time
time
3
1

MONITOR
498
250
637
299
Average Travel Time
average-travel-time
3
1

SLIDER
14
247
200
280
subway-time
subway-time
0
400
290
1
1
minutes

MONITOR
20
371
191
420
Number of Subway Riders
count-subway-riders
3
1

SLIDER
12
288
202
321
probability-exploring
probability-exploring
0
1
0.5
0.01
1
NIL

SLIDER
16
328
200
361
past-discount-value
past-discount-value
0
1
1.0
0.01
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This is an extension of the original Traffic Basic Model which had a simple set of rules: cars slow down if they see a car close ahead, and speed up if they do not see a car ahead.  The original model was meant to demonstrate how traffic jams arise without a centralized cause.

This model extends the Traffic model to modeling Commuters going to and from work.  Besides driving, commuters now have a choice and can use a subway instead of driving which takes a fixed amount of time.

HOW TO USE IT
-------------
Click on the SETUP button to set up the commuters. Set the NUMBER slider to change the number of commuters.  SAME-MAX-SPEED? determines whether all commuters drive the same maximum speed or if there speed varies.  SAME-TRAVEL-TIME-OF-DAY? determines whether the commuters all go to work and return from work at the same time.  

Click on GO to start the commuters moving.  When they get to work they stop, until it is time to go home at which point they return home.

The SPEED-UP slider controls the rate at which cars accelerate when there are no cars ahead.

When a car sees another car right in front, it matches that car's speed and then slows down a bit more.  How much slower it goes than the car in front of it is controlled by the SLOW-DOWN slider.

The other sliders control how the commuters decided to take the subway.  SUBWAY-TIME determines how long the subway takes to get from work to home and vice versa.  PROBABILITY-OF-EXPLORING determines how frequently a commuter will attempt to drive if they are currently taking the subway.  PAST-DISCOUNT-VALUE determines how much to weigh the past versus the last trip when deciding between driving and taking the subway.

THINGS TO NOTICE
----------------
Of course much of the behavior of the Traffic Basic model is also present here, and you can watch traffic jams forming with no specific cause.

The plot shows the number of cars on the road at any one time.  Average Travel Time indicates the average of all the commuters last trip.  Time Of Day maps the ticks used internally in the model to a day and time.  Subway Riders indicates the number of commuters currently using the subway.

THINGS TO TRY
--------------
See how various setup options affect the model performance:  SAME-MAX-SPEED?, SAME-TRAVEL-TIME-OF-DAY?, SPEED-UP, and SLOW-DOWN.

Try changing the SUBWAY-TIME as the model runs and see what effect it has on the number of Subway Riders.

Under what conditions can you get the fastest average travel time, if you set the SUBWAY-TIME to a constant like 50?

EXTENDING THE MODEL
------------
Make a model for two-lane traffic.

Create multiple homes and multiple work locations with an intersecting grid.

Add variability to the subway travel time.

Currently the agents have a simple learning mechanism where they see if their average-travel-time is greater than the subway time.  Implement a more advanced learning mechanism, or take away agent's ability to know the subway time how would they determine which method of transportation to utilize then?

NETLOGO FEATURES
-----------------
Since multiple turtles start on the same patch, there has to be a mechanism to determine who gets to move first, but since in NetLogo all turtles act in parallel this can be difficult.  Thus a token is created MOVING? which a turtle picks up when it starts to move and puts down when its done.  This prevents any other turtle in the same square from moving at the same time.

RELATED MODELS
---------------
"Traffic" (in StarLogoT) adds graphics, trucks, and a radar trap.

"Basic Traffic" is the original model this is based on.

"Gridlock" (a HubNet model which can be run as a participatory simulation) looks at traffic in a grid with many intersections.


CREDITS AND REFERENCES
-----------------------
This model was developed at the MIT Media Lab. See Resnick, M. (1994) "Turtles, Termites and Traffic Jams: Explorations in Massively Parallel Microworlds." Cambridge, Ma: MIT Press. Adapted to StarLogoT, 1997, as part of the Connected Mathematics Project. Adapted to NetLogo, 2000, as part of the Participatory Simulations Project.

To refer to this model in academic publications, please use: Wilensky, U. (1997).  NetLogo Traffic Basic model. http://ccl.northwestern.edu/netlogo/models/TrafficBasic. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1997 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/TrafficBasic for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
setup
repeat 180 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
