globals [running?]
breeds [scholars]
turtles-own [my-clique huddle friends-list popularity busy-shmoozing? my-shmoozee]

to startup
    ask patches with [(abs pxcor) = screen-edge-x or (abs pycor) = screen-edge-y] [set pcolor red]
    ask patches with [(pxcor = screen-edge-x) and (abs pycor <= 1)] [set pcolor green]
end

to setup
    if clique >= number-scholars [set clique number-scholars - 1] ;; otherwise, it wouldn't make sense
    set running? true
    setup-scholars
    setup-plots
end

to setup-scholars
    ask scholars [die]
    cct number-scholars
    [
      set breed scholars
      set friends-list []
      set huddle []
      set busy-shmoozing? false
      ifelse normal?
      [set my-clique random-poisson clique]
      [set my-clique clique]
      setxy screen-edge-x 0
      set heading 270
    ]
    wait .5
    ask scholars
    [
      repeat (random screen-size-x) [walk]
      rt random-float 360
    ]
end

to setup-plots
    clear-all-plots
    set-current-plot "Popularity Distribution"
    set-plot-y-range 0 number-scholars / 4
    set-current-plot "Huddle-Size Distribution"
    set-plot-y-range 0 number-scholars
    set-plot-x-range 0 round (number-scholars / 2)

end

;; RUNTIME PROCEDURES
;;--------------------
;;---------------------

to go
   if not running? [stop]
    ask scholars
    [
      ifelse mean values-from scholars [popularity] <= saturation * clique
      [mingle]
      [set running? false]
    ]
end

to mingle ;; turtle procedure
    if busy-shmoozing?
    [
      wait shmooze
      set busy-shmoozing? false
    ]
    who-is-here?
      ;; 'group' is a local variable that will allow us to select scholars from the huddle
    let group huddle
    have-we-met? group
    update-stuff
    ifelse (not busy-shmoozing?) and (popularity < ( saturation * my-clique ) )
    [orient  advance]
      ;; scholars who are "stuck" in a huddle still have a chance to excuse themselves
    [if random 11 > patience [escape]]
end

to who-is-here?  ;; turtle procedure
    set huddle values-from other-scholars-here [who] ;; a huddle is the group of scholars standing together
end

to have-we-met? [group]  ;; turtle procedure   ;; scholars look for new guy in the huddle
    if length group < 1 [stop]

        ;; scholars wish to be more popular, but reluctantly spend a moment with their new friend before going on
    if (length group = 1) and (member? (first group) friends-list) and not (my-shmoozee = first group)
    [
      set my-shmoozee first group
      set busy-shmoozing? true
      stop
    ]
    ifelse member? (first group) friends-list
                ;; nevertheless, I've given them a chance of meeting someone else in the huddle
                ;; before shmoozing with someone they have met before
    [have-we-met? butfirst group]
    [
      set friends-list fput (first group) friends-list
      stop
    ]
end

to orient ;; turtle procedure
       ;; scholars turn roughly to where the action seems to be --
       ;; scholars' presence is weighted by their popularity
    let x-center mean (values-from scholars in-radius radius [xcor * (popularity + 1) / my-clique]) /
      mean (values-from scholars in-radius radius [(popularity + 1) / my-clique])
    let y-center mean (values-from scholars in-radius radius [ycor * (popularity + 1) / my-clique]) /
      mean (values-from scholars in-radius radius [(popularity + 1) / my-clique])
    if to-center? and distancexy-nowrap x-center y-center > 0
      [set heading towardsxy-nowrap x-center y-center]
end

to advance ;; turtle procedure
       ;; the more popular you are, the more reluctant you are to seek new guys
       ;; but if you're stuck in a group, the larger it is the further away you'll go
    repeat ( 1 - (popularity / my-clique ) ) * (radius)
    [
      walk
      if count other-scholars-here > 0 [stop]
    ]
end


to update-stuff
    update-popularity
    plot-pupularity
    plot-huddles
end

to escape ;; turtle procedure
    rt random-float 360
    repeat random screen-size-x [walk]
end

to walk
   rt ((random-float (2 * wiggle)) - wiggle) ;; somewhere within your visual field
   fd 1
   if pcolor != black [bk 1 rt 180]
end

to update-popularity ;; turtle procedure
    set popularity length friends-list
    ifelse label? [set label popularity] [set label no-label]
end

to plot-pupularity
    if remove 0 values-from scholars [popularity] = [] [stop]
    set-current-plot "Popularity Distribution"
    set-plot-x-range 0 max values-from scholars [popularity]
    histogram-from scholars [popularity]
end

to plot-huddles
    let helper max values-from scholars [length huddle]
    if helper = 0 [stop]
    set-current-plot "Huddle-Size Distribution"
    set-plot-x-range 0 max list plot-x-max helper
    histogram-list values-from patches with [count turtles-here > 0] [count turtles-here]
end


@#$#@#$#@
GRAPHICS-WINDOW
302
10
754
483
8
8
26.0
1
10
1
1
1
0

CC-WINDOW
5
644
763
739
Command Center

BUTTON
12
167
75
200
NIL
go
T
1
T
OBSERVER
T
NIL

BUTTON
11
26
74
59
NIL
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
83
25
282
58
number-scholars
number-scholars
0
100
50
1
1
NIL

SLIDER
191
133
283
166
shmooze
shmooze
0
1
1.0
0.1
1
NIL

PLOT
15
313
284
463
Popularity Distribution
popularity
scholars
0.0
2.0
0.0
50.0
true
false
PENS
"default" 1.0 1 -16777216 true

SWITCH
167
275
284
308
label?
label?
1
1
-1000

SLIDER
83
62
282
95
clique
clique
0
100
10
1
1
NIL

PLOT
15
480
283
630
Huddle-Size Distribution
NIL
NIL
0.0
5.0
0.0
100.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
85
202
283
235
wiggle
wiggle
0
40
20
5
1
NIL

SWITCH
50
275
166
308
to-center?
to-center?
0
1
-1000

SWITCH
84
133
187
166
normal?
normal?
1
1
-1000

SLIDER
85
238
284
271
radius
radius
0
10
3
1
1
NIL

SLIDER
83
98
282
131
saturation
saturation
0
1
1.0
0.1
1
NIL

SLIDER
84
168
282
201
patience
patience
0
10
5
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
'Conference' simulates how scholars interact socially when they arrive at an international conference. For this model, I have assumed they do not know each other before. The model shows how social groups form dynamically, contingent on the scholars' sense of their own  popularity and that of others.

HOW IT WORKS
------------
Each scholar wishes to meet other scholars. They mingle and introduce themselves, and the more scholars they meet the less enthusiastic they are to mingle. Once they meet enough scholars so as to form a clique, they feel adequately popular, and stay put, though others may still approach them. So popularity is (in this version) how many people you have met. Once you're in a group, you get to meet everyone and have to repeatedly shmooze with them. However, the lower your patience is (that's a slider) the higher your chances to escape. The simulation will run until all the scholars have reached the level of popularity (the "saturation") -- from 0.0 thru 1.0 -- that you have set on the slider.


HOW TO USE IT
-------------
Try the current settings - just press go.

While it runs, you can reset the radius in which scholars search for others.
















THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
