globals
[
  clock tick-length          ;; clock variables
  box-edge                   ;; distance of box edge from axes
  last-display-update
  dpx dpy dvx dvy pp pv vv sum-r D1 D2 tmin t1 t2 Det
  colliding-particles
  colliding-pair sorted-colliding-particles
  dt winners winner firstrow
  bad-collisions
  colliding-1
  colliding-2
  original-tick-length
  IframeA
  IframeB
  sub
  min-distance
  length-sorted-colliding-particles
  result
  pressure
  total-particle-number
]

breeds [particles]

particles-own
[
  speed
  mass                 ;; particle info
  x-speed
  y-speed
  particle-size
]

to setup-plotting
 ;; set-current-plot "Distance"
  ;set-plot-y-range  (- screen-edge-y) screen-edge-y          ; set y range to graphics window size
end

to do-plot
  ;set-current-plot-pen "default"
  ;plot pp           ; plot xcor -- there's only one turtle, so its id must be 0
  ;;set-current-plot-pen "badcollisions"
;;  plot bad-collisions           ; plot xcor -- there's only one turtle, so its id must be 0
;;  set-current-plot-pen "length-colliding-particles"
;;  plot length-sorted-colliding-particles

    set-current-plot "Particle 0 Heading vs. Time"
  plotxy clock (heading-of particle 0)

  set-current-plot "Particle 0 Speed vs. Time"
  plotxy clock (speed-of particle 0)
end

to setup
  ;random-seed 17
  ca
    set  total-particle-number initial-number
  set colliding-particles []
  set-default-shape particles "circle"
  set clock 0
  set box-edge (screen-edge-x - 1)
  make-box
  make-particles
  ask turtles [ set label who set label-color orange + 3 ]
  set tick-length (1 / (ceiling max values-from particles [speed]))
  set original-tick-length tick-length
  setup-plotting
      ifelse labels?
    [ask turtles [ set label who set label-color orange + 3 ]]
    [ask turtles [ set label ""]]
end

to go

locals [speeds]

set colliding-particles []
ask particles [without-interruption [check-for-collision-plane]]
ask particles [without-interruption [check-for-collision ]]
sort-collisions
ask particles [without-interruption [ move ]]
if colliding-particles != []
    [collide-winners]
;ask particles [check-distance]
set clock clock + tick-length
do-plot
if change-display? [
;;if clock - last-display-update > display-update-rate
 ;; [ display
;;    set last-display-update clock ]]
  ask particles [
    if show-speed-as-color? = "red-green-blue" [ recolor ]
    if show-speed-as-color? = "purple shades" [ recolorshade ]
    if show-speed-as-color?  = "one color" [ recolornone ]
    ]
     ifelse labels?
    [ask turtles [ set label who set label-color orange + 3 ]]
    [ask turtles [ set label ""]]
]
end

to check-distance

ask particles [if distance myself < 3.9 and distance myself != 0
[ set bad-collisions bad-collisions + 1
if print-values?
 [print distance myself]
 ]]

end

to benchmark
  set initial-number 10
  random-seed 17
  reset-timer
  setup
  repeat 500 [ go ]
  set result timer
end

to move  ;; particle procedure

  jump (speed * tick-length)

end

to-report convert-heading-xy [heading-angle direction]

  if direction = "x" [report (sin (heading-angle))]
  if direction = "y" [report (cos (heading-angle))]

end

to check-for-collision  ;; particle procedure
  locals [candidate my-x my-y my-particle-size]
  set my-x xcor
  set my-y ycor
  set my-particle-size particle-size
  set x-speed (speed * convert-heading-xy heading "x")
  set y-speed (speed * convert-heading-xy heading "y")

  ask particles with [self != myself]
  [
      without-interruption [
         set dpx (xcor - my-x)
         set dpy (ycor - my-y)
         set x-speed (speed * convert-heading-xy heading "x")
         set y-speed (speed * convert-heading-xy heading "y")
         set dvx (x-speed - x-speed-of myself)
         set dvy (y-speed - y-speed-of myself)
         set sum-r my-particle-size
         ;set sum-r (((my-particle-size) / 2) + ((particle-size-of myself) / 2))
         set pp ((dpx * dpx) + (dpy * dpy)) - (sum-r ^ 2)
         set pv ((dpx * dvx) + (dpy * dvy))
         set vv ((dvx * dvx) + (dvy * dvy))
         ;set IframeA (pv + vv)
         ;set IframeB (vv + 2 * pv + pp)
         ;set Det (pv ^ 2) - ((pp - sum-r ^ 2) * vv)
         set D1 pv ^ 2 - pp * vv
         ;set tmin ((- pv ) / vv);
         ;set D2 pp + pv * tmin
         ;set sub ((vv * (tmin ^ 2)) + (2 * pv * tmin) + (pp - (sum-r ^ 2)))
         ifelse D1 >= 0
            [set t1 (- pv - sqrt D1) / vv]
             ;set t2 (- pv + sqrt D1) / vv]
            [set t1 -1
             ];set t2 -1]

         ;set min-distance (pp - ((pv ^ 2) / vv))

         if who < who-of myself
           [
           if t1 > 0 and t1 < 1
             [
              set colliding-pair (list t1 self myself)
              set colliding-particles lput colliding-pair colliding-particles
             ]
          ]

  ]
  ]
end

to check-for-collision-plane ;; particle procedure
  locals [xpos-plane xneg-plane ypos-plane yneg-plane
          contact-point-xpos contact-point-xneg contact-point-ypos contact-point-yneg
           dpxpos dpxneg dpypos dpyneg t-plane-xpos t-plane-xneg t-plane-ypos t-plane-yneg]
  set x-speed (speed * convert-heading-xy heading "x")
  set y-speed (speed * convert-heading-xy heading "x")
  set xpos-plane (box-edge - 0.5)
  set xneg-plane (- box-edge + 0.5)
  set ypos-plane (box-edge - 0.5)
  set yneg-plane (- box-edge + 0.5)

         set contact-point-xpos (xcor + (particle-size / 2))
         set contact-point-xneg (xcor - (particle-size / 2))
         set contact-point-ypos (ycor + (particle-size / 2))
         set contact-point-yneg (ycor - (particle-size / 2))

         set dpxpos (xpos-plane - contact-point-xpos)
         set dpxneg (xneg-plane - contact-point-xneg)
         set dpypos (ypos-plane - contact-point-ypos)
         set dpyneg (yneg-plane - contact-point-yneg)

         set x-speed (speed * convert-heading-xy heading "x")
         set y-speed (speed * convert-heading-xy heading "y")

         set t-plane-xpos 0
         ifelse  x-speed != 0 [set t-plane-xpos (dpxpos / x-speed)] [set t-plane-xpos 0]
          if t-plane-xpos > 0 and t-plane-xpos  < 1
             [
              set colliding-pair (list t-plane-xpos self "plane-xpos" )
              set colliding-particles lput colliding-pair colliding-particles
             ]

         set t-plane-xneg 0
         ifelse  x-speed != 0 [set t-plane-xneg (dpxneg / x-speed)] [set t-plane-xneg 0]
          if t-plane-xneg > 0 and t-plane-xneg  < 1
             [
              set colliding-pair (list t-plane-xneg self "plane-xneg" )
              set colliding-particles lput colliding-pair colliding-particles
             ]
         set t-plane-ypos 0
         ifelse  y-speed != 0 [set t-plane-ypos (dpypos / y-speed)] [set t-plane-ypos 0]
          if t-plane-ypos > 0 and t-plane-ypos  < 1
             [
              set colliding-pair (list t-plane-ypos self "plane-ypos" )
              set colliding-particles lput colliding-pair colliding-particles
             ]

         set t-plane-yneg 0
         ifelse  y-speed != 0 [set t-plane-yneg (dpyneg / y-speed)] [set t-plane-yneg 0]
         if t-plane-yneg > 0 and t-plane-yneg  < 1
             [
              set colliding-pair (list t-plane-yneg self "plane-yneg" )
              set colliding-particles lput colliding-pair colliding-particles
             ]

;show who + " " + t-plane-xpos + ", " + t-plane-xneg + ", " + t-plane-ypos + ", " + t-plane-yneg


end

to print-values

print who-of myself + "-" + who + "-pp:" + precision pp 2 + " pv:"
         + precision pv 2 + " vv:" + precision vv 2 + " frA:"
         + precision IframeA 2 + " frB:" + precision IframeB 2 + " tmin:"
         + precision tmin  2 + " D1:" + precision D1 2 + " D2:"
         + precision D2 2 + " t1:" + precision t1 2 + " t2:"
         + precision t2 2 + " sub:" + precision sub 2

end

to sort-collisions

ifelse colliding-particles != []
   [
    set sorted-colliding-particles sort-by [first ?1 < first ?2] colliding-particles
    if print-values2? [print "candidates: " + sorted-colliding-particles]
    set length-sorted-colliding-particles (length sorted-colliding-particles) * 100
    set firstrow first sorted-colliding-particles
    set dt min map [first ?] sorted-colliding-particles
    set winners filter [first ? = dt] sorted-colliding-particles
    set colliding-1 item 1 first winners
    set colliding-2 item 2 first winners

      if dt > 0
         [set tick-length dt]

   ]
   [set tick-length original-tick-length]
end

to collide-winners
    locals [current_heading]
    if print-values2? [print "just collided: " + who-of colliding-1 + " - " + who-of colliding-2]
    set current_heading heading-of colliding-1
    if ((is-string? colliding-2) and (current_heading = 45 or current_heading = 315 or current_heading = 135 or current_heading = 225))

    [ask colliding-1
        [set heading (heading + 180)]
     stop
    ]

    if colliding-2 = "plane-xpos" or colliding-2 = "plane-xneg"
         [ask colliding-1 [set heading (- heading)]
          stop]
    if colliding-2 = "plane-ypos" or colliding-2 = "plane-yneg"
         [ask colliding-1 [set heading (180 - heading)]
          stop]

    ask colliding-1 [collide-with colliding-2]

end

to collide-with [ other-particle ] ;; particle procedure
  locals
  [
    ;; local copies of other-particle's relevant quantities
    mass2 speed2 heading2

    ;; quantities used in the collision itself
    theta   ;; heading of vector from my center to the center of other-particle.
    v1t     ;; velocity of self along direction theta
    v1l     ;; velocity of self perpendicular to theta
    v2t v2l ;; velocity of other-particle, represented in the same way
    vcm     ;; velocity of the center of mass of the colliding particles,
            ;;   along direction theta
  ]

  ;;; PHASE 1: initial setup

  ;; for convenience, grab some quantities from other-particle
  set mass2 mass-of other-particle
  set speed2 speed-of other-particle
  set heading2 heading-of other-particle

  ;;MODIFIED SO THAT THETA IS HEADING TOWARD OTHER PARTICLE
  set theta towards-nowrap other-particle


  ;;; PHASE 2: convert velocities to theta-based vector representation

  ;; now convert my velocity from speed/heading representation to components
  ;; along theta and perpendicular to theta
  set v1t (speed * cos (theta - heading))
  set v1l (speed * sin (theta - heading))

  ;; do the same for other-particle
  set v2t (speed2 * cos (theta - heading2))
  set v2l (speed2 * sin (theta - heading2))

  ;;; PHASE 3: manipulate vectors to implement collision

  ;; compute the velocity of the system's center of mass along theta
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2) )

  ;; now compute the new velocity for each particle along direction theta.
  ;; velocity perpendicular to theta is unaffected by a collision along theta,
  ;; so the next two lines actually implement the collision itself, in the
  ;; sense that the effects of the collision are exactly the following changes
  ;; in particle velocity.
  set v1t (2 * vcm - v1t)
  set v2t (2 * vcm - v2t)


  ;;; PHASE 4: convert back to normal speed/heading

  ;; now convert my velocity vector into my new speed and heading
  set speed sqrt ((v1t * v1t) + (v1l * v1l))
  ;; if the magnitude of the velocity vector is 0, atan is undefined. but
  ;; speed will be 0, so heading is irrelevant anyway. therefore, in that
  ;; case we'll just leave it unmodified.
  if v1l != 0 or v1t != 0
    [ set heading (theta - (atan v1l v1t)) ]

  ;; and do the same for other-particle
  set speed-of other-particle sqrt (((v2t * v2t) + (v2l * v2l)))
  if v2l != 0 or v2t != 0
    [ set heading-of other-particle (theta - (atan v2l v2t)) ]

  ;; PHASE 5: final updates

  ;; now recolor, since color is based on quantities that may have changed

end

to recolor  ;; particle procedure
  ifelse speed < (0.3 * 1)
  [
    set color blue
  ]
  [
    ifelse speed > (0.80 * 1)
      [ set color red ]
      [ set color green ]
  ]
end

to recolorshade
  ifelse speed < 1.4
  [ set color 111 + (10 * speed / 1.5) ]
  [ set color 119.999 ]
end

to recolornone
  set color green - 1
end

;;;
;;; drawing procedures
;;;


to make-box
  ask patches with [ ((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                     ((abs pycor = box-edge) and (abs pxcor <= box-edge)) ]
    [ set pcolor yellow ]
end

;; creates some particles
to make-particles
  create-custom-particles initial-number
  [
    set speed random-float 1
    set particle-size 1
    set mass 1.0
    set size particle-size
    random-position
    if show-speed-as-color? = "red-green-blue" [ recolor ]
    if show-speed-as-color? = "purple shades" [ recolorshade ]
    if show-speed-as-color?  = "one color" [ recolornone ]
  ]

end


;; place particle at random location inside the box.
to random-position ;; particle procedure
locals [count-loops]
set count-loops 0
  while [(count particles in-radius particle-size) > 1 and count-loops < 5000]  ;;this loop only works with < 25 particles
  [
  setxy ((1 - (box-edge - particle-size )) + random-float ((2 * (box-edge - particle-size )) - 2))
        ((1 - (box-edge - particle-size )) + random-float ((2 * (box-edge - particle-size )) - 2))
        set count-loops count-loops + 1
  ]
  set heading random-float 360
end

;; make turtles appear more like particles
to-report particle [n]
  report turtle n
end

to fade-patches
  locals [ trace-patches ]
  set trace-patches patches with
    [ not shade-of? pcolor black and not shade-of? pcolor yellow ]
  if any? trace-patches
    [ ask trace-patches
      [ set pcolor ( pcolor - 0.05 )
        if (pcolor mod 10 < 1)
          [ set pcolor black ] ] ]
end
@#$#@#$#@
GRAPHICS-WINDOW
224
10
465
272
16
16
7.0
1
20
1
1
1
0
1
1
1

CC-WINDOW
12
175
224
272
Command Center
1

BUTTON
147
10
224
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

MONITOR
148
75
224
124
clock
floor clock
0
1

SLIDER
13
10
146
43
initial-number
initial-number
2
20
20
1
1
NIL

MONITOR
12
125
120
174
particle 0 speed
speed-of particle 0
1
1

MONITOR
119
125
224
174
particle 0 heading
heading-of particle 0
0
1

SWITCH
247
349
344
382
trace-0?
trace-0?
1
1
-1000

MONITOR
247
386
311
435
NIL
tmin
2
1

SWITCH
418
350
536
383
print-values?
print-values?
1
1
-1000

BUTTON
147
42
224
75
go/stop
go
T
1
T
OBSERVER
T
NIL

SWITCH
418
388
536
421
print-values2?
print-values2?
1
1
-1000

SLIDER
310
429
512
462
display-update-rate
display-update-rate
0
2
1.5
0.1
1
NIL

CHOOSER
12
43
147
88
show-speed-as-color?
show-speed-as-color?
"red-green-blue" "purple shades" "one color" "custom color"
2

PLOT
465
10
681
136
Particle 0 Heading vs. Time
NIL
NIL
0.0
100.0
0.0
360.0
true
false

PLOT
465
136
681
272
Particle 0 Speed vs. Time
NIL
NIL
0.0
100.0
0.0
2.0
true
false

SWITCH
44
88
147
121
labels?
labels?
1
1
-1000

SWITCH
311
530
459
563
change-display?
change-display?
0
1
-1000

@#$#@#$#@
(in this section of the interface, information about the model is usually provided)

You may freely download NetLogo at your home!
Go to our site: http://ccl.northwestern.edu/netlogo/

You can view models through a web browser as well:
http://ccl.northwestern.edu/netlogo/models/

This model is part of the Connected Chemistry high school curriculum.  Connected Chemistry is currently under development; for more information please refer to:
http://ccl.northwestern.edu/curriculum/chemistry/


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo P1_Particle_Behavior_5 model.  http://ccl.northwestern.edu/netlogo/models/P1_Particle_Behavior_5.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/P1_Particle_Behavior_5 for terms of use.




benchmarks 16 feb 2005
0.99 - 4.703
sem bounce 4.5
check-collision s das mveis 2.812
sem check-distance 2.25
sem plot 2.11
sem operaes inteis em check-collision 1.735
mais operaes inteis em check-collision 1.454


Com 2 sqrt 3.922
3.812 - no locals, only 1 sq. root;

@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7500403 true true 35 35 230
Circle -7500403 true true 2 2 295

vector
true
0
Line -7500403 true 150 15 150 150
Polygon -7500403 true true 120 30 150 0 180 30 120 30

@#$#@#$#@
NetLogo 3.0beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
