globals
[
  clock tick-length          ;; clock variables
  box-edge                   ;; distance of box edge from axes
  pressure
  pressure-history
  wall-hits-per-particle     ;; average number of wall hits per particle
  particles-to-add
  new-particles              ;; agentset of particles added via add-particles-middle
  message-shown?             ;; whether we've shown the warning message yet
  length-horizontal-surface  ;; the size of the wall surfaces that run horizontally - the top and bottom of the box
  length-vertical-surface    ;; the size of the wall surfaces that run vertically - the left and right of the box
  total-particle-number
                      ;; show ID of particles
  collisions?
  instant-pressure
    maxparticles
;; show-speed-as-color?
]

breeds [ particles flashes clocker ]

flashes-own [birthday]

particles-own
[
  speed mass                 ;; particle info
  wall-hits                  ;; # of wall hits during this clock cycle ("big tick")
  momentum-difference        ;; used to calculate pressure from wall hits
  momentum-instant
  last-collision
  new?                       ;; used to build the new-particles agentset; this is
                             ;; only ever set to true by add-particles-middle
 darkparticle?
]

to startup
  set message-shown? false
end

to setup
  locals [tmp]
  set tmp message-shown?
  ca
    set maxparticles 400
;;  set show-speed-as-color? "one color"
  set message-shown? tmp
  set-default-shape particles "circle"
  set clock 0
  set pressure 0
  set particles-to-add 0
  set labels? false
  ;; box has constant size...
  set box-edge (screen-edge-x - 1)
  ;;; the length of the horizontal or vertical surface of
  ;;; the inside of the box must exclude the two patches
  ;; that are the where the perpendicular walls join it,
  ;;; but must also add in the axes as an additional patch
  ;;; example:  a box with an box-edge of 10, is drawn with
  ;;; 19 patches of wall space on the inside of the box
  set length-horizontal-surface  ( 2 * (box-edge - 1) + 1)
  set length-vertical-surface  ( 2 * (box-edge - 1) + 1)
  make-box
;;  make-particles initial-number
 make-particles maxparticles

  set pressure-history [0 0 0]  ;; plotted pressure will be averaged over the past 3 entries
  setup-plots
  do-plotting

  if labels?
  [ask turtles [ set label who set label-color orange + 3 ]]
    set collisions? true
end

to go

  locals [old-clock]
    ask particles
      [ set new? false ]
  ;;  add-particles-side
    ask particles [ bounce ]
    ask particles [ move ]
    if collisions? [
    ask particles with [darkparticle? = false]
      [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision-regular ] ]

    ask particles with [darkparticle? = true]
      [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision-dark ] ]
    ]
    set old-clock clock
    set clock clock + tick-length

        calculate-instant-pressure


       if floor clock > floor (clock - tick-length)
      [ ifelse any? particles with [darkparticle? = false]
          [ set wall-hits-per-particle mean values-from particles with [darkparticle? = false] [wall-hits] ]
          [ set wall-hits-per-particle 0 ]
        ask particles
          [ set wall-hits 0 ]
         calculate-pressure

        do-plotting ]
    calculate-tick-length
  ;;  ask clocker [ set heading clock * 360 ]
    ask flashes with [clock - birthday > 0.4]
      [ set pcolor yellow
        die ]

    ifelse labels?
      [ask turtles [ set label who set label-color orange + 3 ]]
      [ask turtles [ set label ""]]
   ;; now recolor, since color is based on quantities that may have changed


    if show-speed-as-color? = "red-green-blue" [ask particles  [recolor]]
    if show-speed-as-color? = "purple shades" [ask particles  [recolorshade]]
    if show-speed-as-color? = "one color" [ask particles  [recolornone]]

      if (show-wall-hits? = false) [
      ask flashes
      [ set pcolor yellow ]
      ]

end

to calculate-tick-length
  ifelse any? particles with [speed > 0]
    [ set tick-length 1 / (ceiling max values-from particles [speed]) ]
    [ set tick-length 1 ]
end

;;; Pressure is defined as the force per unit area.  In this context,
;;; that means the total momentum per unit time transferred to the walls
;;; by particle hits, divided by the surface area of the walls.  (Here
;;; we're in a two dimensional world, so the "surface area" of the walls
;;; is just their length.)  Each wall contributes a different amount
;;; to the total pressure in the box, based on the number of collisions, the
;;; direction of each collision, and the length of the wall.  Conservation of momentum
;;; in hits ensures that the difference in momentum for the particles is equal to and
;;; opposite to that for the wall.  The force on each wall is the rate of change in
;;; momentum imparted to the wall, or the sum of change in momentum for each particle:
;;; F = SUM  [d(mv)/dt] = SUM [m(dv/dt)] = SUM [ ma ], in a direction perpendicular to
;;; the wall surface.  The pressure (P) on a given wall is the force (F) applied to that
;;; wall over its surface area.  The total pressure in the box is sum of each wall's
;;; pressure contribution.



to calculate-instant-pressure
  ;; by summing the momentum change for each particle,
  ;; the wall's total momentum change is calculated
  set instant-pressure 15 * sum values-from particles with [darkparticle? = false] [momentum-instant]
  output-print precision instant-pressure 1
  ask particles
    [ set momentum-instant 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit

end


to calculate-pressure
  ;; by summing the momentum change for each particle,
  ;; the wall's total momentum change is calculated

  set pressure 15 * sum values-from particles with [darkparticle? = false] [momentum-difference]
  set pressure-history lput pressure but-first pressure-history

  ask particles
    [ set momentum-difference 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit
end

to bounce  ;; particle procedure
  locals [new-px new-py]
  ;; if we're not about to hit a wall (yellow patch), or if we're already on a
  ;; wall, we don't need to do any further checks
  if shade-of? yellow pcolor or not shade-of? yellow pcolor-of patch-at dx dy
    [ stop ]
  ;; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  ;; if hitting left or right wall, reflect heading around x axis
  if (abs new-px = box-edge)
    [ set heading (- heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a vertical wall, only the horizontal component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall

      set momentum-instant  (abs (sin heading * 2 * mass * speed) / length-vertical-surface)
      set momentum-difference momentum-difference + momentum-instant
 ]

  ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-edge)
    [ set heading (180 - heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a horizontal wall, only the vertical component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall

    set momentum-instant  (abs (cos heading * 2 * mass * speed) / length-horizontal-surface)
    set momentum-difference momentum-difference + momentum-instant  ]


  if (darkparticle? = false) [
  ask patch new-px new-py
    [ sprout 1 [ ht
                 set breed flashes
                 set birthday clock
                 set pcolor yellow - 3 ] ]
      ]
end

to move  ;; particle procedure
  if patch-ahead (speed * tick-length) != patch-here
    [ set last-collision nobody ]
  jump (speed * tick-length)
end

to check-for-collision-regular  ;; particle procedure
  locals [ candidate ]

  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)

  if count other-particles-here with [darkparticle? = false] = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    set candidate random-one-of other-particles-here with
      [who < who-of myself and myself != last-collision and darkparticle? = false]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
    ]
  ]
end




to check-for-collision-dark  ;; particle procedure
  locals [ candidate ]

  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)

  if count other-particles-here with [darkparticle? = true] = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    set candidate random-one-of other-particles-here with
      [who < who-of myself and myself != last-collision and darkparticle? = true]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
    ]
  ]
end


;; implements a collision with another particle.
;;
;; THIS IS THE HEART OF THE PARTICLE SIMULATION, AND YOU ARE STRONGLY ADVISED
;; NOT TO CHANGE IT UNLESS YOU REALLY UNDERSTAND WHAT YOU'RE DOING!
;;
;; The two particles colliding are self and other-particle, and while the
;; collision is performed from the point of view of self, both particles are
;; modified to reflect its effects. This is somewhat complicated, so I'll
;; give a general outline here:
;;   1. Do initial setup, and determine the heading between particle centers
;;      (call it theta).
;;   2. Convert the representation of the velocity of each particle from
;;      speed/heading to a theta-based vector whose first component is the
;;      particle's speed along theta, and whose second compenent is the speed
;;      perpendicular to theta.
;;   3. Modify the velocity vectors to reflect the effects of the collision.
;;      This involves:
;;        a. computing the velocity of the center of mass of the whole system
;;           along direction theta
;;        b. updating the along-theta components of the two velocity vectors.
;;   4. Convert from the theta-based vector representation of velocity back to
;;      the usual speed/heading representation for each particle.
;;   5. Perform final cleanup and update derived quantities.
to collide-with [ other-particle ] ;; particle procedure
  locals
  [
    ;; local copies of other-particle's relevant quantities
    mass2 speed2 heading2

    ;; quantities used in the collision itself
    theta   ;; heading of vector from my center to the center of other-particle.
    v1t     ;; velocity of self along direction theta
    v1l     ;; velocity of self perpendicular to theta
    v2t v2l ;; velocity of other-particle, represented in the same way
    vcm     ;; velocity of the center of mass of the colliding particles,
            ;;   along direction theta
  ]



  ;;; PHASE 1: initial setup

  ;; for convenience, grab some quantities from other-particle
  set mass2 mass-of other-particle
  set speed2 speed-of other-particle
  set heading2 heading-of other-particle

  ;; since particles are modeled as zero-size points, theta isn't meaningfully
  ;; defined. we can assign it randomly without affecting the model's outcome.
  set theta (random-float 360)



  ;;; PHASE 2: convert velocities to theta-based vector representation

  ;; now convert my velocity from speed/heading representation to components
  ;; along theta and perpendicular to theta
  set v1t (speed * cos (theta - heading))
  set v1l (speed * sin (theta - heading))

  ;; do the same for other-particle
  set v2t (speed2 * cos (theta - heading2))
  set v2l (speed2 * sin (theta - heading2))



  ;;; PHASE 3: manipulate vectors to implement collision

  ;; compute the velocity of the system's center of mass along theta
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2) )

  ;; now compute the new velocity for each particle along direction theta.
  ;; velocity perpendicular to theta is unaffected by a collision along theta,
  ;; so the next two lines actually implement the collision itself, in the
  ;; sense that the effects of the collision are exactly the following changes
  ;; in particle velocity.
  set v1t (2 * vcm - v1t)
  set v2t (2 * vcm - v2t)



  ;;; PHASE 4: convert back to normal speed/heading

  ;; now convert my velocity vector into my new speed and heading
  set speed sqrt ((v1t * v1t) + (v1l * v1l))
  ;; if the magnitude of the velocity vector is 0, atan is undefined. but
  ;; speed will be 0, so heading is irrelevant anyway. therefore, in that
  ;; case we'll just leave it unmodified.
  if v1l != 0 or v1t != 0
    [ set heading (theta - (atan v1l v1t)) ]

  ;; and do the same for other-particle
  set speed-of other-particle sqrt ((v2t * v2t) + (v2l * v2l))
  if v2l != 0 or v2t != 0
    [ set heading-of other-particle (theta - (atan v2l v2t)) ]



  ;; PHASE 5: final updates

  ;; now recolor, since color is based on quantities that may have changed
end

to recolor  ;; particle procedure
  ifelse speed < (0.5 * 10)
  [
    set color blue
  ]
  [
    ifelse speed > (1.5 * 10)
      [ set color red ]
      [ set color green ]
  ]
end

to recolorshade
  ifelse speed < 27
  [ set color 111 + speed / 3 ]
  [ set color 119.999 ]
end

to recolornone
  set color green - 1
end


;;;
;;; drawing procedures
;;;

;; draws the box
to make-box
  ask patches with [ ((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                     ((abs pycor = box-edge) and (abs pxcor <= box-edge)) ]
    [ set pcolor yellow ]
  ask patches with [pycor = 0 and pxcor < (1 - box-edge)]
  [
    set pcolor yellow - 5  ;; trick the bounce code so particles don't go into the inlet
    set pcolor-of patch-at 0  1 yellow
    set pcolor-of patch-at 0 -1 yellow
  ]
end

;; creates initial particles
to make-particles [number]
  create-custom-particles number
  [
    setup-particle
    set speed random-float 20
    random-position
    set color red
  ]

  set total-particle-number initial-number

  ask particles with [who < initial-number]
    [
      set shape "circle"
      set darkparticle? false
       if show-speed-as-color? = "red-green-blue" [ recolor ]
      if show-speed-as-color? = "purple shades" [ recolorshade ]
      if show-speed-as-color? = "one color" [ recolornone ]
    ]


  calculate-tick-length
end

;; adds particles from the left
to add-particles-side

       set particles-to-add number-to-add
  ifelse ((particles-to-add + total-particle-number ) > maxparticles)
    [user-message "The maximum number of particles allowed in this model is " + maxparticles + ".  You can not add " + number-to-add +
     " more particles to the " + (count particles with [darkparticle? = false]) + " you already have in the model"]
    [


      if particles-to-add > 0 [

       ask particles with [who < (total-particle-number + particles-to-add) and who >= total-particle-number]
        [ set darkparticle? false
          set shape "circle"
          setxy (- box-edge) 0
          set heading 90 ;; east
          rt 45 - random-float 90
          set speed 10
          if show-speed-as-color? = "red-green-blue" [ recolor ]
          if show-speed-as-color? = "purple shades" [ recolorshade ]
          if show-speed-as-color? = "one color" [ recolornone ]
        ]
      set total-particle-number (total-particle-number + particles-to-add)
      set particles-to-add 0

      calculate-tick-length
      ]
     ]

end



to setup-particle  ;; particle procedure
  set new? false
  set speed 10
  set mass 1.0
  set last-collision nobody
  set wall-hits 0
  set momentum-difference 0
  set darkparticle? true
      ifelse showdark? [set shape "default"][set shape "nothing" set color green]

end

;; place particle at random location inside the box.
to random-position ;; particle procedure
  setxy ((1 - box-edge) + random-float ((2 * box-edge) - 2))
        ((1 - box-edge) + random-float ((2 * box-edge) - 2))
  set heading random-float 360
end

;; make turtles appear more like particles
to-report particle [n]
  report turtle n
end

;;; plotting procedures

to setup-plots

end


to do-plotting



  set-current-plot "Pressure vs. Time"
   set-current-plot-pen "default"
  if length pressure-history > 0
    [ plotxy clock (mean pressure-history) ]

 set-current-plot "Particle 0 Speed vs. Time"
 set-current-plot-pen "particle-0"
 plotxy clock (speed-of particle 0)
end



; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials were created as part of the project:
; MODELING ACROSS THE CURRICULUM.  The project gratefully acknowledges the support of the
; National Science Foundation, National Institute of Health and the Department of Education
; (IERI program) -- grant number REC # 0115699. Additional support was provided through the
; project: PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS
; -- NSF (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Chem Pressure 2 model.
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
212
10
487
306
26
26
5.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
3
223
213
306
Command Center
1

MONITOR
140
75
212
124
clock
floor clock
0
1

BUTTON
140
42
212
75
go/stop
go
T
1
T
OBSERVER
T
NIL

BUTTON
140
10
212
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
382
335
505
368
number-to-add
number-to-add
0
100
50
1
1
NIL

BUTTON
583
343
669
376
add particles
add-particles-side
NIL
1
T
OBSERVER
NIL
NIL

SLIDER
2
10
139
43
initial-number
initial-number
1
400
100
1
1
NIL

MONITOR
4
122
70
171
number
count particles with [darkparticle? = false]
0
1

PLOT
492
10
695
155
Pressure vs. Time
time
pressure
0.0
20.0
0.0
100.0
true
false

MONITOR
287
395
465
444
average wall hits per particle
wall-hits-per-particle
2
1

CHOOSER
3
43
139
88
show-speed-as-color?
show-speed-as-color?
"red-green-blue" "purple shades" "one color" "custom color"
2

SWITCH
161
391
282
424
showdark?
showdark?
1
1
-1000

OUTPUT
72
141
149
171

TEXTBOX
83
122
136
140
Pressure

SWITCH
474
384
611
417
show-wall-hits?
show-wall-hits?
0
1
-1000

SWITCH
3
88
122
121
labels?
labels?
1
1
-1000

PLOT
493
155
695
305
Particle 0 Speed vs. Time
time
speed
0.0
10.0
0.0
10.0
true
false
PENS
"particle-0" 1.0 0 -16777216 true
"average" 1.0 0 -10899396 true

MONITOR
3
174
104
223
particle 0 speed
speed-of particle 0
1
1

MONITOR
104
174
213
223
particle 0 heading
heading-of particle 0
0
1

@#$#@#$#@
You may freely download NetLogo at your home!
Go to our site: http://ccl.northwestern.edu/netlogo/

Connected Chemistry Curriculum is currently under development, for more information please refer to http://ccl.northwestern.edu/curriculum/chemistry/

WHAT IS IT?
-----------
This model simulates pumping up a bicycle tire.  It consists of gas particles that are trapped in a fixed and rigid container.  The model specifically looks at how the number of particles affects the pressure of system.  Air can be added into the container through a valve.  Adding a group of particles into the container increases the number of particles that are moving and colliding with each other and hitting the walls.

This model is part of the Connected Chemistry High School Curriculum and is part of the second and third activities, in which the concept of pressure is explored.  In this model, students look at the relationship between number of particles and pressure. In addition, one can follow the average number of wall hits in one model clock tick.  A clock hand is used to mark the passage of time in the model.  When the particles hit the walls they change their color temporarily.  These models have been adapted from the model GasLab Pressure Box, which can be found in the Unverified section of the GasLab folder found under Chemistry and Physics.

These pressure models are part of a suite of models that students use to gain deeper insight on Gas Laws and particle behavior. In all of the Connected Chemistry Curriculum models, the same basic rules are used for expressing what happens when gas particles collide.  Each model has different features in order to show different aspects of the behavior of gases.


HOW IT WORKS
------------
Particles are modeled as perfectly elastic particles with no energy except their kinetic energy -- which is due to their motion.  Collisions between particles are elastic.  Particles are colored according to speed -- blue for slow, green for medium, and red for high speeds.

The exact way two particles collide is as follows:
1. Two turtles "collide" when they find themselves on the same patch.
2. A random axis is chosen, as if they are two balls that hit each other and this axis is the line connecting their centers.
3. They exchange momentum and energy along that axis, according to the conservation of momentum and energy.  This calculation is done in the center of mass system.
4. Each turtle is assigned its new velocity, energy, and heading.
5. If a turtle finds itself on or very close to a wall of the container, it "bounces" -- that is, reflects its direction and keeps its same speed.


HOW TO USE IT
-------------
Buttons:
  SETUP - sets up the initial conditions set on the sliders.  Be sure to wait till the Setup button stops before pushing GO/STOP.
  GO/STOP - runs and stops the model.
  ADD-PARTICLES - when pressed releases particles into the box while the simulation is running.

Sliders:
  INITIAL-NUMBER - sets the number of gas particles in the box when the simulation starts.
  NUMBER-TO-ADD - the number of gas particles released into the box when the ADD-PARTICLES button is pressed.

Monitors:
  CLOCK - number of clock cycles that GO has run.
  NUMBER - the number of particles in the box.
  PRESSURE - the total pressure in the box.
  WALL HITS PER PARTICLE - the average number of wall hits in one clock tick

Plots:
  PRESSURE VS TIME - plots the pressure in the box over time.
  NUMBER VS TIME - plots the number of particles in the box over time.  This plot is laid  under the WALL HITS PER PARTICLE plot, as in scripted activities the two plots can be exposed in different screens.
  WALL HITS PER PARTICLE - the average number of wall hits in one clock tick

Initially, the particles are not moving. Therefore the initial pressure is zero.  When the particles start moving, they repeatedly collide, exchange energy and head off in new directions, and the speeds are dispersed -- some particles get faster, some get slower.  When they hit the wall they change their heading, but not their speed.


THINGS TO NOTICE
----------------
Can you relate what you can see happening to the particles in the box with changes in pressure?

Why does the pressure change over time, even when the number of particles is the same?  How long does it take for the pressure to stabilize?

What happens to the wall hits per particle when particles are added to the box?

In what ways is this model an incorrect idealization of the real world?


THINGS TO TRY
-------------
Try different settings, especially the extremes.   Are the particles behaving in a similar way?  How does this affect the pressure?

You can pen-down a particle through the command center or by using the turtle menus.  What do you notice about a particle's path when there more and less particles in the box?

Can you make the pressure graph smooth?  Can you do it in more than one way?

Sometimes, when going up in an elevator, airplane or up a mountain we feel a 'popping' sensation in our ears.  This is associated with changes in pressure.  Can you relate between this model and these changes in pressure?  Are the temperature and volume constant in this situation?


EXTENDING THE MODEL
-------------------
Add a histogram showing all the particles speeds throughout the run of the model.

Add a switch that would continuously add particles to the box?

Make a hole in the box and observe what happens to the pressure.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo P2_pressure_1 model.  http://ccl.northwestern.edu/netlogo/models/P2Pressure1. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/P2Pressure1 for terms of use.


You can view models through a web browser as well:
http://ccl.northwestern.edu/netlogo/models/

This model is part of the Connected Chemistry high school curriculum.  Connected Chemistry is currently under development; for more information please refer to:
http://ccl.northwestern.edu/curriculum/chemistry/
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7500403 true true 30 30 240

clocker
true
0
Polygon -5825686 true false 150 30 105 195 135 180 135 270 165 270 165 180 195 195

nothing
true
0

@#$#@#$#@
NetLogo 3.0beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
