globals
[
  clock tick-length          ;; clock variables
  max-tick-length            ;; the largest a tick length is allowed to be
  box-edge                   ;; distance of box edge from axes
  pressure
  pressure-history
  zero-pressure-count        ;; how many zero entries are in pressure-history
  wall-hits-per-particle     ;; average number of wall hits per particle
  length-horizontal-surface  ;; the size of the wall surfaces that run horizontally - the top and bottom of the box
  length-vertical-surface    ;; the size of the wall surfaces that run vertically - the left and right of the box
  walls                      ;; agentset containing patches that are the walls of the box
  init-avg-speed init-avg-energy  ;; initial averages
  avg-speed avg-energy particle0-speed           ;; current averages
  fast medium slow                ;; current counts
  outside-energy
  min-outside-energy
  max-outside-energy
  temp-increment
  instant-pressure
  maxparticles
  total-particle-number
  volume
  temperature
]

breeds [ particles dark-particles flashes spinner ]

flashes-own [birthday]

particles-own
[
  speed mass energy          ;; particle info
  wall-hits                  ;; # of wall hits during this clock cycle ("big tick")
  momentum-difference        ;; used to calculate pressure from wall hits
  last-collision
  momentum-instant
]

dark-particles-own
[
  speed mass energy          ;; particle info
  wall-hits                  ;; # of wall hits during this clock cycle ("big tick")
  momentum-difference        ;; used to calculate pressure from wall hits
  last-collision
  momentum-instant
]

to setup
  ca

   set maxparticles 400
  set temp-increment 7.5
  set min-outside-energy 0
  set max-outside-energy 300
  set-default-shape particles "circle"
  set-default-shape dark-particles "nothing"
  set-default-shape flashes "square"
  set-default-shape spinner "clock"
  set clock 0
  set max-tick-length 0.1073
  ;; box has constant size...
  set box-edge (screen-edge-x - 1)
  ;;; the length of the horizontal or vertical surface of
  ;;; the inside of the box must exclude the two patches
  ;; that are the where the perpendicular walls join it,
  ;;; but must also add in the axes as an additional patch
  ;;; example:  a box with an box-edge of 10, is drawn with
  ;;; 19 patches of wall space on the inside of the box
  set length-horizontal-surface  ( 2 * (box-edge - 1) + 1)
  set length-vertical-surface  ( 2 * (box-edge - 1) + 1)
  set outside-energy 100
  make-box
  make-particles maxparticles
 ;; make-spinner
  set pressure-history [0 0 0]  ;; plotted pressure will be averaged over the past 3 entries
  set zero-pressure-count 0
  update-variables

      set temperature (avg-energy * 6)
  set init-avg-speed avg-speed

  ask particles with [ who = 0 ] [set particle0-speed speed]
  set init-avg-energy avg-energy
  setup-plots
  setup-histograms
  do-plotting
    if speed-as-color? = "red-green-blue" [ ask particles [recolor ] ]
    if speed-as-color? = "purple shades" [ ask particles [recolorshade ]]
    if speed-as-color? = "one color" [ ask particles [recolornone ]]
end

to update-variables
  set medium count particles with [color = green]
  set slow   count particles with [color = blue]
  set fast   count particles with [color = red]
  set avg-speed  mean values-from particles [speed]
  set avg-energy mean values-from particles [energy]
end

to showlabel
 ifelse labels?
    [
      set label who
      set label-color white
      if (who = 0) [set label-color (orange + 1)]
    ]
    [set label ""]
end

to go
  ask walls
    [set pcolor box-color]
  ask particles [ bounce ]
  ask particles [ move ]
  ask dark-particles [ bounce-dark ]
  ask dark-particles [ move ]
  ask particles [ showlabel ]
  if collide? [
    ask particles
      [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision ] ]

    ask dark-particles
      [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision ] ]

  ]
  set clock clock + tick-length

  calculate-instant-pressure

  if floor clock > floor (clock - tick-length)
  [
    ifelse any? particles
      [ set wall-hits-per-particle mean values-from particles [wall-hits] ]
      [ set wall-hits-per-particle 0 ]
    ask particles
      [ set wall-hits 0 ]
    calculate-pressure
    update-variables
    do-plotting
  ]
  calculate-tick-length
  ask particles with [ who = 0 ] [set particle0-speed speed]

  ask flashes with [clock - birthday > 0.4]
    [ die ]
;  set outside-energy wall-temp

  ;;  show outside-energy

    if speed-as-color? = "red-green-blue" [ ask particles [recolor ] ]
    if speed-as-color? = "purple shades" [ ask particles [recolorshade ]]
    if speed-as-color? = "one color" [ ask particles [recolornone ]]
    set temperature (avg-energy * 6)
end

to cool
  ifelse ( outside-energy > 20 ) [ set outside-energy outside-energy - temp-increment ] [ set outside-energy 0 ]
  if (outside-energy = 0)
   [ user-message "You are currently trying to cool the walls of the container below absolute zero (OK or -273C).  Absolute zero is the lowest theoretical temperature for all matter in the universe and has never been achived in a real-world laboratory"]
end

to heat
  set outside-energy outside-energy + temp-increment
    if (outside-energy > 300)
    [set outside-energy 300
    user-message "You have reached the maximum allowable temperature for the walls of the container in this model."]

end


to calculate-tick-length
  ifelse any? particles with [speed > 0]
    [ set tick-length min list (1 / (ceiling max values-from particles [speed])) max-tick-length ]
    [ set tick-length max-tick-length ]
end

;;; Pressure is defined as the force per unit area.  In this context,
;;; that means the total momentum per unit time transferred to the walls
;;; by particle hits, divided by the surface area of the walls.  (Here
;;; we're in a two dimensional world, so the "surface area" of the walls
;;; is just their length.)  Each wall contributes a different amount
;;; to the total pressure in the box, based on the number of collisions, the
;;; direction of each collision, and the length of the wall.  Conservation of momentum
;;; in hits ensures that the difference in momentum for the particles is equal to and
;;; opposite to that for the wall.  The force on each wall is the rate of change in
;;; momentum imparted to the wall, or the sum of change in momentum for each particle:
;;; F = SUM  [d(mv)/dt] = SUM [m(dv/dt)] = SUM [ ma ], in a direction perpendicular to
;;; the wall surface.  The pressure (P) on a given wall is the force (F) applied to that
;;; wall over its surface area.  The total pressure in the box is sum of each wall's
;;; pressure contribution.

to calculate-pressure
  ;; by summing the momentum change for each particle,
  ;; the wall's total momentum change is calculated
  set pressure 15 * sum values-from particles [momentum-difference]
  set pressure-history lput pressure but-first pressure-history
  ask particles
    [ set momentum-difference 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit
end

to bounce  ;; particle procedure
  ;; get the coordinates of the patch we'll be on if we go forward 1
  let new-patch patch-ahead 1
  let new-px pxcor-of new-patch
  let new-py pycor-of new-patch
  ;; if we're not about to hit a wall, we don't need to do any further checks
  if (abs new-px != box-edge and abs new-py != box-edge)
    [stop]
  ;; if hitting left or right wall, reflect heading around x axis
  if (abs new-px = box-edge)
    [ set heading (- heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a vertical wall, only the horizontal component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-instant  (abs (sin heading * 2 * mass * speed) / length-vertical-surface)
      set momentum-difference momentum-difference + momentum-instant
      ]
   ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-edge)
    [ set heading (180 - heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a horizontal wall, only the vertical component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
       set momentum-instant  (abs (cos heading * 2 * mass * speed) / length-horizontal-surface)
    set momentum-difference momentum-difference + momentum-instant
    ]
  if value-from patch new-px new-py [heated-wall?]  ;; check if the patch ahead of us is heated
  [
    set energy ((energy +  outside-energy ) / 2)
    set speed sqrt (2 * energy / mass )
    recolor
  ]


    ask patch new-px new-py
    [ sprout 1 [
      set breed flashes
      set color 11 ;;pcolor-of 11
      if (color < 10)
        [set color 10]
      set birthday clock
      ]
    ]

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to bounce-dark  ;; particle procedure
  ;; get the coordinates of the patch we'll be on if we go forward 1
  let new-patch patch-ahead 1
  let new-px pxcor-of new-patch
  let new-py pycor-of new-patch
  ;; if we're not about to hit a wall, we don't need to do any further checks
  if (abs new-px != box-edge and abs new-py != box-edge)
    [stop]
  ;; if hitting left or right wall, reflect heading around x axis
  if (abs new-px = box-edge)
    [ set heading (- heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a vertical wall, only the horizontal component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-instant  (abs (sin heading * 2 * mass * speed) / length-vertical-surface)
      set momentum-difference momentum-difference + momentum-instant
      ]
   ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-edge)
    [ set heading (180 - heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a horizontal wall, only the vertical component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
       set momentum-instant  (abs (cos heading * 2 * mass * speed) / length-horizontal-surface)
    set momentum-difference momentum-difference + momentum-instant
    ]
  if value-from patch new-px new-py [heated-wall?]  ;; check if the patch ahead of us is heated
  [
    set energy ((energy +  outside-energy ) / 2)
    set speed sqrt (2 * energy / mass )
    recolor
  ]


end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move  ;; particle procedure
  if patch-ahead (speed * tick-length) != patch-here
    [ set last-collision nobody ]
  jump (speed * tick-length)
end


to check-for-collision  ;; particle procedure
  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)
  let others-here nobody

  ifelse breed = particles
  [ set others-here other-particles-here ]
  [ set others-here other-dark-particles-here ]

  if count others-here = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    let candidate random-one-of others-here with
      [who < who-of myself and myself != last-collision ]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
    ]
  ]
end

;; implements a collision with another particle.
;;
;; THIS IS THE HEART OF THE PARTICLE SIMULATION, AND YOU ARE STRONGLY ADVISED
;; NOT TO CHANGE IT UNLESS YOU REALLY UNDERSTAND WHAT YOU'RE DOING!
;;
;; The two particles colliding are self and other-particle, and while the
;; collision is performed from the point of view of self, both particles are
;; modified to reflect its effects. This is somewhat complicated, so I'll
;; give a general outline here:
;;   1. Do initial setup, and determine the heading between particle centers
;;      (call it theta).
;;   2. Convert the representation of the velocity of each particle from
;;      speed/heading to a theta-based vector whose first component is the
;;      particle's speed along theta, and whose second compenent is the speed
;;      perpendicular to theta.
;;   3. Modify the velocity vectors to reflect the effects of the collision.
;;      This involves:
;;        a. computing the velocity of the center of mass of the whole system
;;           along direction theta
;;        b. updating the along-theta components of the two velocity vectors.
;;   4. Convert from the theta-based vector representation of velocity back to
;;      the usual speed/heading representation for each particle.
;;   5. Perform final cleanup and update derived quantities.
to collide-with [ other-particle ] ;; particle procedure
  locals [mass2 speed2 heading2 theta v1t v1l v2t v2l vcm]

  ;;; PHASE 1: initial setup

  ;; for convenience, grab some quantities from other-particle
  set mass2 mass-of other-particle
  set speed2 speed-of other-particle
  set heading2 heading-of other-particle

  ;; since particles are modeled as zero-size points, theta isn't meaningfully
  ;; defined. we can assign it randomly without affecting the model's outcome.
  set theta (random-float 360)



  ;;; PHASE 2: convert velocities to theta-based vector representation

  ;; now convert my velocity from speed/heading representation to components
  ;; along theta and perpendicular to theta
  set v1t (speed * cos (theta - heading))
  set v1l (speed * sin (theta - heading))

  ;; do the same for other-particle
  set v2t (speed2 * cos (theta - heading2))
  set v2l (speed2 * sin (theta - heading2))



  ;;; PHASE 3: manipulate vectors to implement collision

  ;; compute the velocity of the system's center of mass along theta
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2) )

  ;; now compute the new velocity for each particle along direction theta.
  ;; velocity perpendicular to theta is unaffected by a collision along theta,
  ;; so the next two lines actually implement the collision itself, in the
  ;; sense that the effects of the collision are exactly the following changes
  ;; in particle velocity.
  set v1t (2 * vcm - v1t)
  set v2t (2 * vcm - v2t)



  ;;; PHASE 4: convert back to normal speed/heading

  ;; now convert my velocity vector into my new speed and heading
  set speed sqrt ((v1t ^ 2) + (v1l ^ 2))
  set energy (0.5 * mass * (speed ^ 2))
  ;; if the magnitude of the velocity vector is 0, atan is undefined. but
  ;; speed will be 0, so heading is irrelevant anyway. therefore, in that
  ;; case we'll just leave it unmodified.
  if v1l != 0 or v1t != 0
    [ set heading (theta - (atan v1l v1t)) ]

  ;; and do the same for other-particle
  set speed-of other-particle sqrt ((v2t ^ 2) + (v2l ^ 2))
  set energy-of other-particle (0.5 * (mass-of other-particle) * ((speed-of other-particle) ^ 2))
  if v2l != 0 or v2t != 0
    [ set heading-of other-particle (theta - (atan v2l v2t)) ]



  ;; PHASE 5: final updates

  ;; now recolor, since color is based on quantities that may have changed
  ;;recolor
  ;;ask other-particle
  ;;  [ recolor ]
end

to recolor  ;; particle procedure
   ifelse speed < (0.5 * 10)
  [
    set color blue
  ]
  [
    ifelse speed > (1.5 * 10)
      [ set color red ]
      [ set color green ]
  ]
end


to recolorshade
  ifelse speed < 27
  [ set color 111 + speed / 3 ]
  [ set color 119.999 ]
end

to recolornone
  set color green - 1
end


;; reports color of box according to temperature and position
;; if only one side is heated, the other walls will be yellow
to-report box-color
     ifelse heated-wall?
       [report scale-color red outside-energy -60 340]
       [report yellow]
end

;; reports true if there is a heated wall at the given location
to-report heated-wall?

      if (( abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                     ((abs pycor = box-edge) and (abs pxcor <= box-edge))
        [report true]
     ;]
     report false
end

;;;
;;; drawing procedures
;;;

;; draws the box
to make-box
  set walls patches with [ ((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                     ((abs pycor = box-edge) and (abs pxcor <= box-edge)) ]
  ask walls
    [ set pcolor box-color ]
end

;; creates initial particles
to make-particles  [n]
  create-custom-particles number
  [
    setup-particle
    if speed-as-color? = "red-green-blue" [ recolor ]
    if speed-as-color? = "purple shades" [ recolorshade ]
    if speed-as-color? = "one color" [ recolornone ]
  ]

  create-custom-dark-particles ( n - number )
  [
    setup-particle
    set color red
    if showdark? [set shape "default"]
  ]
  set total-particle-number number

  calculate-tick-length
end


to setup-particle  ;; particle procedure
  set speed random-float 20
  set mass 1.0
  set energy (0.5 * mass * speed * speed)
  set last-collision nobody
  set wall-hits 0
  set momentum-difference 0
  random-position
end

;; place particle at random location inside the box.
to random-position ;; particle procedure
  setxy ((1 - box-edge) + random-float ((2 * box-edge) - 2))
        ((1 - box-edge) + random-float ((2 * box-edge) - 2))
  set heading random-float 360
end

;; adds particles from the left
to add-particles
  if particles-to-add > 0
    [ cct-particles particles-to-add
        [ setup-particle
          setxy (- box-edge) 0
          set heading 90 ;; east
          rt 45 - random-float 90
          recolor
        ]
      set particles-to-add 0
      calculate-tick-length
    ]
end
;;; plotting procedures

to setup-plots
end

to setup-histograms
 locals [init-particle-speed]
  set init-particle-speed 20  ;;the initial speeds in this model cannot be faster than 20 at setup
 ;; set particle-mass 1    ;; each particle has a mass of 1

  set-current-plot "Speed Histogram"
  set-plot-x-range 0 (init-particle-speed * 2)
  set-plot-y-range 0 ceiling (number / 6)
  set-current-plot-pen "medium"
  set-histogram-num-bars 40
  set-current-plot-pen "slow"
  set-histogram-num-bars 40
  set-current-plot-pen "fast"
  set-histogram-num-bars 40
  if other-speeds?
  [
    set-current-plot-pen "init-avg-speed"
    draw-vert-line init-avg-speed

    set-current-plot-pen "particle0-speed"
    draw-vert-line particle0-speed
    ]

end


to do-plotting
  set-current-plot "Pressure vs. Time"
  plotxy clock (mean pressure-history)


  set-current-plot "Gas Temp. vs. Time"
;  set-current-plot-pen "outside"
 ; plotxy clock wall-temp
  ;set-current-plot-pen "inside"
  plotxy clock avg-energy * 6

  set-current-plot "Average Speed vs. Time"
  plotxy clock avg-speed

;  if clock > 1
 ; [
;     set-current-plot "Wall Hits per Particle"
 ;    plotxy clock wall-hits-per-particle
;  ]

  plot-histograms
end


to plot-histograms
;  set-current-plot "Energy histogram"
 ; set-current-plot-pen "fast"
  ;histogram-from particles with [color = red] [ energy ]
;  set-current-plot-pen "medium"
 ; histogram-from particles with [color = green] [ energy ]
  ;set-current-plot-pen "slow"
;  histogram-from particles with [color = blue] [ energy ]
 ; set-current-plot-pen "avg-energy"
  ;plot-pen-reset
;  draw-vert-line avg-energy

  set-current-plot "Speed histogram"
  set-current-plot-pen "fast"
  histogram-from particles with [color = red] [ speed ]
  set-current-plot-pen "medium"
  histogram-from particles with [color = green] [ speed ]
  set-current-plot-pen "slow"
  histogram-from particles with [color = blue] [ speed ]
  set-current-plot-pen "init-avg-speed"
  plot-pen-reset
      draw-vert-line init-avg-speed
  if other-speeds?
  [
      set-current-plot-pen "avg-speed"
     plot-pen-reset
      draw-vert-line avg-speed
          set-current-plot-pen "particle0-speed"
      plot-pen-reset
      draw-vert-line particle0-speed
      ]
end

;; histogram procedure
to draw-vert-line [ xval ]
  plotxy xval plot-y-min
  plot-pen-down
  plotxy xval plot-y-max
  plot-pen-up
end

to calculate-instant-pressure
  ;; by summing the momentum change for each particle,
  ;; the wall's total momentum change is calculated
  set instant-pressure 15 * 30 * sum values-from particles [momentum-instant]
  output-print precision instant-pressure 1
  ask particles
    [ set momentum-instant 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit
end

to make-spinner
  create-custom-spinner 1
  [
    setxy (screen-edge-x - 6) (screen-edge-y - 6)
    set color grey - 1.5
    set size 10
    set heading 0
    set label 0
  ]
end

to-report particle [n]
  report turtle n
end

; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials were created as part of the project:
; MODELING ACROSS THE CURRICULUM.  The project gratefully acknowledges the support of the
; National Science Foundation, National Institute of Health and the Department of Education
; (IERI program) -- grant number REC # 0115699. Additional support was provided through the
; project: PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS
; -- NSF (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Chem Pressure 2 model.
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
222
10
500
309
33
33
4.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
1
217
221
309
Command Center
1

BUTTON
55
43
122
76
go/stop
go
T
1
T
OBSERVER
T
NIL

BUTTON
-1
43
55
76
NIL
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
0
10
122
43
number
number
1
400
100
1
1
NIL

PLOT
500
159
714
307
Pressure vs. Time
time
pressure
0.0
20.0
0.0
100.0
true
false
PENS
"default" 1.0 0 -955883 true

MONITOR
110
168
221
217
average speed
avg-speed
2
1

PLOT
500
10
718
159
Gas Temp. vs. Time
time
temp.
0.0
100.0
0.0
100.0
true
false
PENS
"temperature" 1.0 0 -6459832 true

SWITCH
439
383
529
416
collide?
collide?
0
1
-1000

PLOT
20
342
236
492
Speed Histogram
speed
count
0.0
50.0
0.0
100.0
true
false
PENS
"fast" 5.0 1 -2674135 true
"medium" 5.0 1 -10899396 true
"slow" 5.0 1 -13345367 true
"avg-speed" 1.0 0 -7500403 true
"init-avg-speed" 1.0 0 -16777216 true
"particle0-speed" 1.0 0 -39373 true

SWITCH
132
76
222
109
labels?
labels?
1
1
-1000

MONITOR
259
440
362
489
average energy
avg-energy
2
1

MONITOR
1
168
110
217
total energy
count particles * avg-energy
0
1

SLIDER
384
342
500
375
particles-to-add
particles-to-add
0
100
0
1
1
NIL

BUTTON
312
387
423
420
add particles
add-particles
NIL
1
T
OBSERVER
T
NIL

BUTTON
123
10
222
43
warm up walls
heat
NIL
1
T
OBSERVER
T
NIL

BUTTON
123
43
222
76
cool down walls
cool
NIL
1
T
OBSERVER
T
NIL

PLOT
500
159
717
311
Average Speed vs. Time
time
avg. speed
0.0
100.0
0.0
12.0
true
false

SWITCH
553
377
680
410
other-speeds?
other-speeds?
1
1
-1000

CHOOSER
0
75
127
120
speed-as-color?
speed-as-color?
"red-green-blue" "purple shades" "one color" "custom color"
0

SWITCH
248
345
367
378
showdark?
showdark?
1
1
-1000

OUTPUT
129
128
221
158

TEXTBOX
150
110
215
128
pressure

MONITOR
1
119
126
168
gas temperature
temperature
1
1

@#$#@#$#@
WHAT IS IT?
-----------

This model is one in a series of GasLab models. They use the same basic rules for simulating the behavior of gases.  Each model integrates different features in order to highlight different aspects of gas behavior.

The basic principle of the models is that gas particles are assumed to have two elementary actions: they move and they collide - either with other particles or with any other objects such as walls.

This model is illustrates the relationship between temperature and pressure in
a fixed volume gas container.

This model is part of the Connected Mathematics "Making Sense of Complex Phenomena" Modeling Project.




HOW IT WORKS
------------
The particles are modeled as hard balls with no internal energy except that which is due to their motion.  Collisions between particles are elastic.

1. A particle moves in a straight line without changing its speed, unless it collides with another particle or bounces off the wall.
2. Two particles "collide" if they find themselves on the same patch.In this model, two turtles are aimed so that they will collide at the origin.
3. An angle of collision for the particles is chosen, as if they were two solid balls that hit, and this angle describes the direction of the line connecting their centers.
4. The particles exchange momentum and energy only along this line, conforming to the conservation of momentum and energy for elastic collisions.
5. Each particle is assigned its new speed, heading and energy.

As the walls of the box are heated, the sides of the walls will change color from a deep red (cool) to a bright red, to pink to a pale pink white (hot).  The walls contain a constant heat value throughout the simulation.  If ONE-SIDE? is set to ON, only the left wall will be heated, while the other three walls remain yellow.


The exact way particles gain energy from the walls of the box is as follows:
1. Particles check their state of energy.
2. They hit or bounce off the wall.
3. They find wall energy and recalculate their new energy.
4. They change their speed and direction after the wall hit.


HOW TO USE IT
-------------
Initial settings:
- NUMBER-OF-PARTICLES: number of particles within in the box

Buttons:
The SETUP button will set these initial conditions.
The GO button will begin the simulation.

Other Settings:
  OUTSIDE-TEMPERATURE: temperature of the outside of the box and the wall of the box.
  ONE-SIDE?: heats only the left wall if enabled.  the other walls are colored yellow, and do not affect the energy of the particles that bounce into it.
- COLLIDE?: Turns collisions between particles on and off.


Plots:
- SPEED COUNTS: plots the number of particles in each range of speed.
- SPEED HISTOGRAM: speed distribution of all the particles.  The gray line is the average value, and the black line is the initial average.
- ENERGY HISTOGRAM: distribution of energies of all the particles, calculated as m*(v^2)/2.
- PRESSURE VS. TIME: plots average pressure of the inside of the box over time.
- TEMPERATURE VS. TIME: plots particle temperature inside the box over time and wall temperature over time.
- WALL HITS PER PARTICLE: plots average wall hits per particle over time.

THINGS TO NOTICE
----------------

How does adding heat to the box walls affect the pressure?

How does adding heat to the wall affect the particle behavior?

How does the particle behavior or system response change with only one wall heated instead of all walls heated?

Does the system reach an equilibrium temperature faster when the wall is heated or cooled the same amount in comparison to the temperature of the particles?


THINGS TO TRY
-------------
Try to get the inside temperature to reach the outside temperature.  Is this possible?

Try to increase the wall hits per particle.


EXTENDING THE MODEL
-------------------
Give the wall a mass and see how that affects the behavior of the model.

Close off the right side of the box.  Create two valves on either side to the wall that allow the user to "spurt" particles into the chambers to see how number of particles affects pressure.

Vary the width and length of the box, does this effect how fast the particle temperature changes?


NETLOGO FEATURES
----------------
Notice how the collisions are detected by the turtles and how the code guarantees the same two particles do not collide twice.  What happens if we let the patches detect them?


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2003).  NetLogo GasLab Heat Box model. http://ccl.northwestern.edu/netlogo/models/GasLabHeatBox. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2003 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/GasLabHeatBox for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7500403 true true 35 35 230

clock
true
0
Circle -7500403 true true 30 30 240
Polygon -16777216 true false 150 31 128 75 143 75 143 150 158 150 158 75 173 75
Circle -16777216 true false 135 135 30

nothing
true
0

square
false
0
Rectangle -7500403 true true 0 0 297 299

@#$#@#$#@
NetLogo 3.0beta3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@

