globals
[
  clock tick-length          ;; clock variables
  max-tick-length            ;; the largest a tick length is allowed to be
  box-edge-y                  ;; distance of box edge from axes
  box-edge-x
  pressure
  pressure-history
particles-to-add
  zero-pressure-count        ;; how many zero entries are in pressure-history
  wall-hits-per-particle     ;; average number of wall hits per particle
  length-horizontal-surface  ;; the size of the wall surfaces that run horizontally - the top and bottom of the box
  length-vertical-surface    ;; the size of the wall surfaces that run vertically - the left and right of the box
  walls                      ;; agentset containing patches that are the walls of the box
  heated-walls
  piston-wall
  init-avg-speed init-avg-energy  ;; initial averages
  avg-speed avg-energy particle0-speed           ;; current averages
  fast medium slow                ;; current counts
  outside-energy
  min-outside-energy
  max-outside-energy
  temp-increment
  piston-position            ;; xcor of piston wall
  piston-color               ;; color of piston
  wall-color                 ;; color of wall
  run-go?                    ;; flag of whether or not its safe for go to run
  volume
  total-particle-number
  temperature
  temp-volume
  temp-target-wall
  maxparticles
]

breeds [ particles flashes volume-target]

flashes-own [birthday]

particles-own
[
  speed mass energy          ;; particle info
  wall-hits                  ;; # of wall hits during this clock cycle ("big tick")
  momentum-difference        ;; used to calculate pressure from wall hits
  last-collision
  darkparticle?
]

to setup
  ca
  set maxparticles 400
    set run-go? true

  set temp-increment 7.5
  set min-outside-energy 0
  set max-outside-energy 300
  set-default-shape particles "circle"
  set-default-shape flashes "square"
   set particles-to-add 0
  set clock 0
  set max-tick-length 0.1073
  ;; box has constant size...
  set box-edge-y (screen-edge-y - 1)
  set box-edge-x (screen-edge-x - 1)
  ;;; the length of the horizontal or vertical surface of
  ;;; the inside of the box must exclude the two patches
  ;; that are the where the perpendicular walls join it,
  ;;; but must also add in the axes as an additional patch
  ;;; example:  a box with an box-edge of 10, is drawn with
  ;;; 19 patches of wall space on the inside of the box
  set piston-position init-wall-position - box-edge-x

  set length-horizontal-surface  ( 2 * (box-edge-x - 1) + 1) - (abs (piston-position - box-edge-x))
  set length-vertical-surface  ( 2 * (box-edge-y - 1) + 1)
  set volume (length-horizontal-surface * length-vertical-surface * 1)  ;;depth of 1
  set outside-energy 100
  set piston-color orange
  set wall-color (scale-color red outside-energy -60 340)

 ;; make-box

  draw-box-piston
  make-particles maxparticles
 ;; make-spinner
  set pressure-history [0 0 0]  ;; plotted pressure will be averaged over the past 3 entries
  set zero-pressure-count 0
  update-variables
  set init-avg-speed avg-speed

  ask particles with [who = 0] [set particle0-speed speed]
  set init-avg-energy avg-energy
  set temperature avg-energy * 6
 create-custom-volume-target 1 [set color white ht]
  setup-plots
  set total-particle-number initial-number
  do-plotting
    if speed-as-color? = "red-green-blue" [ ask particles [recolor ] ]
    if speed-as-color? = "purple shades" [ ask particles [recolorshade ]]
    if speed-as-color? = "one color" [ ask particles [recolornone ]]
end

to update-variables
  set medium count particles with [color = green]
  set slow   count particles with [color = blue]
  set fast   count particles with [color = red]
  set avg-speed  mean values-from particles [speed]
  set avg-energy mean values-from particles [energy]
end

to showlabel
 ifelse labels?
    [
      set label who
      set label-color white
      if (who = 0) [set label-color (orange + 1)]
    ]
    [set label ""]
end

to go

  if not run-go? [stop]

  set temperature avg-energy * 6
  ask particles [ bounce ]
  ask particles [ move ]
   ask particles [showlabel]
  if collide? [
    ask particles with [darkparticle? = false]
    [ ;; without-interruption is needed here so one collision is
      ;; happening at a time
      without-interruption
        [check-for-collision]
    ]
    ask particles with [darkparticle? = true]
    [ ;; without-interruption is needed here so one collision is
        ;; happening at a time
        without-interruption
          [ check-for-collision-dark ]
    ]
  ]
  set clock clock + tick-length
  if floor clock > floor (clock - tick-length)
  [
    ifelse any? particles
          [ set wall-hits-per-particle mean values-from particles with [darkparticle? = false] [wall-hits] ]
          [ set wall-hits-per-particle 0 ]
    ask particles
      [ set wall-hits 0 ]
    calculate-pressure
    update-variables
    do-plotting
  ]
  calculate-tick-length
  calculate-wall-color
  ask particles with [who = 0] [set particle0-speed speed]

    ask flashes with [clock - birthday > 0.4]

      [ if ( shade-of? pcolor piston-color)
          [set pcolor piston-color]
        if ( shade-of? pcolor wall-color)
          [set pcolor wall-color]
         die
         ]
  ask patches with [pycor = 0 and pxcor < (1 - box-edge-x)]
  [
    set pcolor 10 ;; trick the bounce code so particles don't go into the inlet
  ]

    if speed-as-color? = "red-green-blue" [ ask particles [recolor ] ]
    if speed-as-color? = "purple shades" [ ask particles [recolorshade ]]
    if speed-as-color? = "one color" [ ask particles [recolornone ]]
end


to calculate-wall-color
  set wall-color (scale-color red outside-energy -60 340)
end


to cool
  ifelse ( outside-energy > 20 ) [ set outside-energy outside-energy - temp-increment ] [ set outside-energy 0 ]
  if (outside-energy = 0)
   [ user-message "You are currently trying to cool the walls of the container below absolute zero (OK or -273C).  Absolute zero is the lowest theoretical temperature for all matter in the universe and has never been achived in a real-world laboratory"]
  calculate-wall-color
  ask heated-walls
      [set pcolor wall-color]
end

to heat
  set outside-energy outside-energy + temp-increment
    if (outside-energy > 300)
    [set outside-energy 300
    user-message "You have reached the maximum allowable temperature for the walls of the container in this model."]
    calculate-wall-color
    ask heated-walls
      [set pcolor wall-color]
end


to calculate-tick-length
  ifelse any? particles with [speed > 0]
    [ set tick-length min list (1 / (ceiling max values-from particles [speed])) max-tick-length ]
    [ set tick-length max-tick-length ]
end

;;; Pressure is defined as the force per unit area.  In this context,
;;; that means the total momentum per unit time transferred to the walls
;;; by particle hits, divided by the surface area of the walls.  (Here
;;; we're in a two dimensional world, so the "surface area" of the walls
;;; is just their length.)  Each wall contributes a different amount
;;; to the total pressure in the box, based on the number of collisions, the
;;; direction of each collision, and the length of the wall.  Conservation of momentum
;;; in hits ensures that the difference in momentum for the particles is equal to and
;;; opposite to that for the wall.  The force on each wall is the rate of change in
;;; momentum imparted to the wall, or the sum of change in momentum for each particle:
;;; F = SUM  [d(mv)/dt] = SUM [m(dv/dt)] = SUM [ ma ], in a direction perpendicular to
;;; the wall surface.  The pressure (P) on a given wall is the force (F) applied to that
;;; wall over its surface area.  The total pressure in the box is sum of each wall's
;;; pressure contribution.

to calculate-pressure
  ;; by summing the momentum change for each particle,
  ;; the wall's total momentum change is calculated

  set pressure 15 * sum values-from particles with [darkparticle? = false] [momentum-difference]
  set pressure-history lput pressure but-first pressure-history

  ask particles
    [ set momentum-difference 0 ]  ;; once the contribution to momentum has been calculated
                                   ;; this value is reset to zero till the next wall hit
end

to bounce  ;; particle procedure
  locals [new-patch new-px new-py]
  ;; get the coordinates of the patch we'll be on if we go forward 1
  if (shade-of? black pcolor and shade-of? black pcolor-of patch-at dx dy)
    [stop]
  ;; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  ;; if hitting left wall or piston (on right), reflect heading around x axis
  if ((abs new-px = box-edge-x or new-px = piston-position))
    [
      set heading (- heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a vertical wall, only the horizontal component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-difference momentum-difference + (abs (sin heading * 2 * mass * speed) / length-vertical-surface) ]
  ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-edge-y)
    [ set heading (180 - heading)
      set wall-hits wall-hits + 1
  ;;  if the particle is hitting a horizontal wall, only the vertical component of the speed
  ;;  vector can change.  The change in velocity for this component is 2 * the speed of the particle,
  ;; due to the reversing of direction of travel from the collision with the wall
      set momentum-difference momentum-difference + (abs (cos heading * 2 * mass * speed) / length-horizontal-surface)  ]

  if value-from patch new-px new-py [heated-wall?]  ;; check if the patch ahead of us is heated
  [
    set energy ((energy +  outside-energy ) / 2)
    set speed sqrt (2 * energy / mass )
    recolor
  ]
   if (darkparticle? = false) [
  ask patch new-px new-py
  [ sprout 1 [
                ht
                 set breed flashes
                 set birthday clock
                 ifelse shade-of? (pcolor-of patch-here) piston-color

                   [set pcolor piston-color - 3]
                   [set pcolor 11]
    ]   ]
  ]
end

to move  ;; particle procedure
  if patch-ahead (speed * tick-length) != patch-here
    [ set last-collision nobody ]
  jump (speed * tick-length)
end

to check-for-collision  ;; particle procedure
  locals [candidate]
  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)

  if count other-particles-here with [darkparticle? = false] = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    set candidate random-one-of other-particles-here with
      [who < who-of myself and myself != last-collision]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
    ]
  ]
end



to check-for-collision-dark  ;; particle procedure
  locals [ candidate ]

  ;; Here we impose a rule that collisions only take place when there
  ;; are exactly two particles per patch.  We do this because when the
  ;; student introduces new particles from the side, we want them to
  ;; form a uniform wavefront.
  ;;
  ;; Why do we want a uniform wavefront?  Because it is actually more
  ;; realistic.  (And also because the curriculum uses the uniform
  ;; wavefront to help teach the relationship between particle collisions,
  ;; wall hits, and pressure.)
  ;;
  ;; Why is it realistic to assume a uniform wavefront?  Because in reality,
  ;; whether a collision takes place would depend on the actual headings
  ;; of the particles, not merely on their proximity.  Since the particles
  ;; in the wavefront have identical speeds and near-identical headings,
  ;; in reality they would not collide.  So even though the two-particles
  ;; rule is not itself realistic, it produces a realistic result.  Also,
  ;; unless the number of particles is extremely large, it is very rare
  ;; for three or more particles to land on the same patch (for example,
  ;; with 400 particles it happens less than 1% of the time).  So imposing
  ;; this additional rule should have only a negligible effect on the
  ;; aggregate behavior of the system.
  ;;
  ;; Why does this rule produce a uniform wavefront?  The particles all
  ;; start out on the same patch, which means that without the only-two
  ;; rule, they would all start colliding with each other immediately,
  ;; resulting in much random variation of speeds and headings.  With
  ;; the only-two rule, they are prevented from colliding with each other
  ;; until they have spread out a lot.  (And in fact, if you observe
  ;; the wavefront closely, you will see that it is not completely smooth,
  ;; because some collisions eventually do start occurring when it thins out while fanning.)

  if count other-particles-here with [darkparticle? = true] = 1
  [
    ;; the following conditions are imposed on collision candidates:
    ;;   1. they must have a lower who number than my own, because collision
    ;;      code is asymmetrical: it must always happen from the point of view
    ;;      of just one particle.
    ;;   2. they must not be the same particle that we last collided with on
    ;;      this patch, so that we have a chance to leave the patch after we've
    ;;      collided with someone.
    set candidate random-one-of other-particles-here with
      [who < who-of myself and myself != last-collision and darkparticle? = true]
    ;; we also only collide if one of us has non-zero speed. It's useless
    ;; (and incorrect, actually) for two particles with zero speed to collide.
    if (candidate != nobody) and (speed > 0 or speed-of candidate > 0)
    [
      collide-with candidate
      set last-collision candidate
      set last-collision-of candidate self
    ]
  ]
end

;; implements a collision with another particle.
;;
;; THIS IS THE HEART OF THE PARTICLE SIMULATION, AND YOU ARE STRONGLY ADVISED
;; NOT TO CHANGE IT UNLESS YOU REALLY UNDERSTAND WHAT YOU'RE DOING!
;;
;; The two particles colliding are self and other-particle, and while the
;; collision is performed from the point of view of self, both particles are
;; modified to reflect its effects. This is somewhat complicated, so I'll
;; give a general outline here:
;;   1. Do initial setup, and determine the heading between particle centers
;;      (call it theta).
;;   2. Convert the representation of the velocity of each particle from
;;      speed/heading to a theta-based vector whose first component is the
;;      particle's speed along theta, and whose second compenent is the speed
;;      perpendicular to theta.
;;   3. Modify the velocity vectors to reflect the effects of the collision.
;;      This involves:
;;        a. computing the velocity of the center of mass of the whole system
;;           along direction theta
;;        b. updating the along-theta components of the two velocity vectors.
;;   4. Convert from the theta-based vector representation of velocity back to
;;      the usual speed/heading representation for each particle.
;;   5. Perform final cleanup and update derived quantities.
to collide-with [ other-particle ] ;; particle procedure
  locals [mass2 speed2 heading2 theta v1t v1l v2t v2l vcm]

  ;;; PHASE 1: initial setup

  ;; for convenience, grab some quantities from other-particle
  set mass2 mass-of other-particle
  set speed2 speed-of other-particle
  set heading2 heading-of other-particle

  ;; since particles are modeled as zero-size points, theta isn't meaningfully
  ;; defined. we can assign it randomly without affecting the model's outcome.
  set theta (random-float 360)



  ;;; PHASE 2: convert velocities to theta-based vector representation

  ;; now convert my velocity from speed/heading representation to components
  ;; along theta and perpendicular to theta
  set v1t (speed * cos (theta - heading))
  set v1l (speed * sin (theta - heading))

  ;; do the same for other-particle
  set v2t (speed2 * cos (theta - heading2))
  set v2l (speed2 * sin (theta - heading2))



  ;;; PHASE 3: manipulate vectors to implement collision

  ;; compute the velocity of the system's center of mass along theta
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2) )

  ;; now compute the new velocity for each particle along direction theta.
  ;; velocity perpendicular to theta is unaffected by a collision along theta,
  ;; so the next two lines actually implement the collision itself, in the
  ;; sense that the effects of the collision are exactly the following changes
  ;; in particle velocity.
  set v1t (2 * vcm - v1t)
  set v2t (2 * vcm - v2t)



  ;;; PHASE 4: convert back to normal speed/heading

  ;; now convert my velocity vector into my new speed and heading
  set speed sqrt ((v1t ^ 2) + (v1l ^ 2))
  set energy (0.5 * mass * (speed ^ 2))
  ;; if the magnitude of the velocity vector is 0, atan is undefined. but
  ;; speed will be 0, so heading is irrelevant anyway. therefore, in that
  ;; case we'll just leave it unmodified.
  if v1l != 0 or v1t != 0
    [ set heading (theta - (atan v1l v1t)) ]

  ;; and do the same for other-particle
  set speed-of other-particle sqrt ((v2t ^ 2) + (v2l ^ 2))
  set energy-of other-particle (0.5 * (mass-of other-particle) * ((speed-of other-particle) ^ 2))
  if v2l != 0 or v2t != 0
    [ set heading-of other-particle (theta - (atan v2l v2t)) ]



  ;; PHASE 5: final updates

  ;; now recolor, since color is based on quantities that may have changed
  ;;recolor
  ;;ask other-particle
  ;;  [ recolor ]
end

to recolor  ;; particle procedure
   ifelse speed < (0.5 * 10)
  [
    set color blue
  ]
  [
    ifelse speed > (1.5 * 10)
      [ set color red ]
      [ set color green ]
  ]
end


to recolorshade
  ifelse speed < 27
  [ set color 111 + speed / 3 ]
  [ set color 119.999 ]
end

to recolornone
  set color green - 1
end


;; reports color of box according to temperature and position
;; if only one side is heated, the other walls will be yellow
;;to-report box-color
;;     ifelse heated-wall?
;;       [report wall-color]
;;       [report yellow]
;;end

;; reports true if there is a heated wall at the given location
to-report heated-wall?

      if (( abs pxcor = -1 * box-edge-x) and (abs pycor <= box-edge-y)) or
                     ((abs pycor = box-edge-y) and (abs pxcor <= box-edge-x))
        [report true]
     ;]
     report false
end

;;;
;;; drawing procedures
;;;

;; draws the box
to make-box
  set heated-walls patches with [ ((pxcor = -1 * box-edge-x) and (abs pycor <= box-edge-y)) or
                     ((abs pycor = box-edge-y) and (pxcor <= piston-position) and (abs pxcor <= box-edge-x)) ]


  ask heated-walls  [ set pcolor white ]
  ;;color the left side of the wall grey:
  ask patches with [ ((pxcor = ( box-edge-x)) and (abs pycor <= box-edge-y))]
    [set pcolor grey]
end

;; creates initial particles
to make-particles [number]
  create-custom-particles number
  [
    setup-particle
    set speed random-float 20
    set energy (0.5 * mass * speed * speed)
    random-position
     set darkparticle? true
  ]

  set total-particle-number initial-number

  ask particles with [who <= initial-number]
    [
      set darkparticle? false
      set shape "circle"
       if speed-as-color? = "red-green-blue" [ recolor  ]
       if speed-as-color? = "purple shades" [recolorshade ]
       if speed-as-color? = "one color" [recolornone ]
  ]
  calculate-tick-length
end




to setup-particle  ;; particle procedure

  set speed 10
  set mass 1.0
  set energy (0.5 * mass * speed * speed)
  set last-collision nobody
  set wall-hits 0
  set momentum-difference 0
  set darkparticle? true
  ifelse showdark? [set shape "default"][set shape "nothing" set color green]

end

;; place particle at random location inside the box.
to random-position ;; particle procedure
  setxy ((1 - box-edge-x)  + random-float (box-edge-x + piston-position - 3))
        ((1 - box-edge-y) + random-float (2 * box-edge-y - 2))
  set heading random-float 360
end



to add-particles
    set particles-to-add number-to-add

 ;;   show particles-to-add
 ;;   show total-particle-number
    ifelse ((particles-to-add + total-particle-number ) > maxparticles)
    [user-message "The maximum number of particles allowed in this model is " + maxparticles + ".  You can not add " + number-to-add +
     " more particles to the " + (count particles with [darkparticle? = false]) + " you already have in the model"]
    [
    if number-to-add > 0 [



       ask particles with [who <= (total-particle-number + particles-to-add) and who >= total-particle-number and who <= maxparticles]
        [ set darkparticle? false
          set shape "circle"
          setxy (- box-edge-x + 1) 0
          set heading 90 ;; east
          rt 45 - random-float 90
          set speed 10
          if speed-as-color? = "red-green-blue" [ recolor ]
          if speed-as-color? = "purple shades" [ recolorshade ]
          if speed-as-color? = "one color" [ recolornone ]
        ]
      set total-particle-number (count particles with [darkparticle? = false])
      set particles-to-add 0

      calculate-tick-length
      ]
    ]



end
;;; plotting procedures

to setup-plots
  set-current-plot "1: Volume vs. Time"
  set-plot-y-range (0.5 * volume) (1.5 * volume)
    set-current-plot "2: Volume vs. Time"
  set-plot-y-range (0.5 * volume) (1.5 * volume)
end



to do-plotting
  set-current-plot "1: Pressure vs. Time"
  if length pressure-history > 0
    [ plotxy clock (mean pressure-history) ]

   set-current-plot "2: Pressure vs. Time"
  if length pressure-history > 0
    [ plotxy clock (mean pressure-history) ]

  set-current-plot "1: Volume vs. Time"
  if clock > 1
    [ plotxy clock volume ]

  set-current-plot "2: Volume vs. Time"
  if clock > 1
    [ plotxy clock volume ]

  set-current-plot "1: Temperature vs. Time"
  plotxy clock temperature

  set-current-plot "2: Temperature vs. Time"
  plotxy clock temperature

     set-current-plot "1: Number vs. Time"
  plotxy clock total-particle-number

   set-current-plot "2: Number vs. Time"
  plotxy clock total-particle-number


 ;; set-current-plot "Wall Hits per Particle vs. Time"
;;  if clock > 1
   ;; [ plotxy clock wall-hits-per-particle ]

end




;; ------ Piston ----------
to move-piston

 set run-go? false
  if ((mouse-down?) and (mouse-ycor < (screen-edge-y - 1)))
   ;;note: if user clicks too far to the right, nothing will happen
   [ if (mouse-xcor >= piston-position and mouse-xcor < box-edge-x - 2)
      [ piston-out ceiling (mouse-xcor - piston-position) ]
     set run-go? true
     ask volume-target [ht]
     stop
   ]

   set temp-target-wall 0
   set temp-volume 0

  ifelse (mouse-xcor >= piston-position and mouse-xcor < box-edge-x - 2)
      [set temp-target-wall (( ( 2 * (box-edge-x - 1) + 1) - (abs (mouse-xcor - box-edge-x) - 1) )) ]  ;;depth of 1
      [ set temp-target-wall (2 * box-edge-x - 1) ]

    ifelse (mouse-xcor <= piston-position)
      [ set temp-volume volume ]
      [ set temp-volume (temp-target-wall * length-vertical-surface * 1)]

   ask volume-target [st setxy mouse-xcor mouse-ycor set label ("volume: " + floor temp-volume)]

   if (abs mouse-ycor > box-edge-y) [ask volume-target [ht set label ""]]
end

to piston-out [dist]
  if (dist > 0)
  [ ifelse ((piston-position + dist) < box-edge-x - 1)
    [ undraw-piston
      set piston-position (piston-position + dist)
      draw-box-piston ]
    [ undraw-piston
      set piston-position (box-edge-x - 1)
      draw-box-piston ]
   set length-horizontal-surface  ( 2 * (box-edge-x - 1) + 1) - (abs (piston-position - box-edge-x) - 1)
   set volume (length-horizontal-surface * length-vertical-surface * 1)  ;;depth of 1
  ]
end



to draw-box-piston

  set heated-walls patches with [ ((pxcor = -1 * box-edge-x) and (abs pycor <= box-edge-y)) or
                     ((abs pycor = box-edge-y) and (pxcor <= piston-position) and (abs pxcor <= box-edge-x)) ]
  ask heated-walls  [ set pcolor wall-color ]

  set piston-wall patches with [ ((pxcor = (round piston-position)) and ((abs pycor) < box-edge-y)) ]
  ask piston-wall  [ set pcolor piston-color ]
  ;; make sides of box that are to right right of the piston grey

  ask patches with [pycor = 0 and pxcor < (1 - box-edge-x)]
  [
    set pcolor 10 ;; trick the bounce code so particles don't go into the inlet
    set pcolor-of patch-at 0  1 wall-color
    set pcolor-of patch-at 0 -1 wall-color
  ]

  ask patches with [(pxcor > (round piston-position)) and (abs (pxcor) < box-edge-x)
                    and ((abs pycor) = box-edge-y)]
    [set pcolor grey]
  ask patches with [ ((pxcor = ( box-edge-x)) and (abs pycor <= box-edge-y))]
    [set pcolor grey]
end

to undraw-piston
  ask patches with [ (pxcor = round piston-position) and ((abs pycor) < box-edge-y) ]
    [ set pcolor black ]

 ;; ask flashes with [ (xcor = round piston-position) and ((abs ycor) < box-edge) ]
 ;;   [ set color white ]

end

to-report particle [n]
  report turtle n
end

; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials were created as part of the project:
; MODELING ACROSS THE CURRICULUM.  The project gratefully acknowledges the support of the
; National Science Foundation, National Institute of Health and the Department of Education
; (IERI program) -- grant number REC # 0115699. Additional support was provided through the
; project: PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS
; -- NSF (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Chem Pressure 2 model.
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/ChemPressure2
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
1
122
415
277
50
15
4.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
217
54
442
116
Command Center
1

BUTTON
56
22
127
55
go/stop
go
T
1
T
OBSERVER
T
NIL

BUTTON
0
23
56
56
NIL
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
0
55
127
88
initial-number
initial-number
1
200
100
1
1
NIL

PLOT
492
161
714
309
2: Pressure vs. Time
time
pressure
0.0
20.0
0.0
100.0
true
false

MONITOR
415
116
492
165
pressure
pressure
0
1

MONITOR
14
367
116
416
average speed
avg-speed
2
1

PLOT
492
10
714
159
1: Temperature vs. Time
time
temp.
0.0
100.0
0.0
100.0
true
false
PENS
"temperature" 1.0 0 -2674135 true

SWITCH
412
367
514
400
collide?
collide?
0
1
-1000

SWITCH
321
367
411
400
labels?
labels?
1
1
-1000

MONITOR
115
367
218
416
average energy
avg-energy
2
1

MONITOR
218
367
320
416
total energy
count particles * avg-energy
0
1

SLIDER
0
88
127
121
number-to-add
number-to-add
0
200
50
5
1
NIL

BUTTON
127
88
217
121
add particles
\nadd-particles
NIL
1
T
OBSERVER
T
NIL

BUTTON
127
22
217
55
warm walls
heat
NIL
1
T
OBSERVER
T
NIL

BUTTON
127
55
217
88
cool walls
cool
NIL
1
T
OBSERVER
T
NIL

CHOOSER
515
367
624
412
speed-as-color?
speed-as-color?
"red-green-blue" "purple shades" "one color" "custom color"
0

SLIDER
21
278
405
311
init-wall-position
init-wall-position
4
96
20
1
1
NIL

BUTTON
217
22
299
55
move wall
move-piston
T
1
T
OBSERVER
T
NIL

MONITOR
415
263
492
312
volume
volume
1
1

MONITOR
415
165
492
214
number
total-particle-number
3
1

MONITOR
415
214
492
263
temperature
temperature
0
1

PLOT
492
10
714
160
1: Volume vs. Time
time
volume
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -955883 true

CHOOSER
299
10
395
55
Graph-1
Graph-1
"Number" "Temperature" "Volume" "Pressure"
3

PLOT
492
10
714
160
1: Number vs. Time
time
number
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -10899396 true

CHOOSER
395
10
490
55
Graph-2
Graph-2
"Number" "Temperature" "Volume" "Pressure"
1

PLOT
492
10
714
160
1: Pressure vs. Time
time
pressure
0.0
10.0
0.0
10.0
true
false

PLOT
492
162
714
310
2: Volume vs. Time
time
volume
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -955883 true

PLOT
492
161
714
311
2: Number vs. Time
Time
number
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -10899396 true

PLOT
492
161
714
311
2: Temperature vs. Time
time
temp.
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -2674135 true

MONITOR
442
67
492
116
clock
floor clock
0
1

SWITCH
325
405
444
438
showdark?
showdark?
1
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------

This model is one in a series of GasLab models. They use the same basic rules for simulating the behavior of gases.  Each model integrates different features in order to highlight different aspects of gas behavior.

The basic principle of the models is that gas particles are assumed to have two elementary actions: they move and they collide - either with other particles or with any other objects such as walls.

This model is illustrates the relationship between temperature and pressure in
a fixed volume gas container.

This model is part of the Connected Mathematics "Making Sense of Complex Phenomena" Modeling Project.




HOW IT WORKS
------------
The particles are modeled as hard balls with no internal energy except that which is due to their motion.  Collisions between particles are elastic.

1. A particle moves in a straight line without changing its speed, unless it collides with another particle or bounces off the wall.
2. Two particles "collide" if they find themselves on the same patch.In this model, two turtles are aimed so that they will collide at the origin.
3. An angle of collision for the particles is chosen, as if they were two solid balls that hit, and this angle describes the direction of the line connecting their centers.
4. The particles exchange momentum and energy only along this line, conforming to the conservation of momentum and energy for elastic collisions.
5. Each particle is assigned its new speed, heading and energy.

As the walls of the box are heated, the sides of the walls will change color from a deep red (cool) to a bright red, to pink to a pale pink white (hot).  The walls contain a constant heat value throughout the simulation.  If ONE-SIDE? is set to ON, only the left wall will be heated, while the other three walls remain yellow.


The exact way particles gain energy from the walls of the box is as follows:
1. Particles check their state of energy.
2. They hit or bounce off the wall.
3. They find wall energy and recalculate their new energy.
4. They change their speed and direction after the wall hit.


HOW TO USE IT
-------------
Initial settings:
- NUMBER-OF-PARTICLES: number of particles within in the box

Buttons:
The SETUP button will set these initial conditions.
The GO button will begin the simulation.

Other Settings:
  OUTSIDE-TEMPERATURE: temperature of the outside of the box and the wall of the box.
  ONE-SIDE?: heats only the left wall if enabled.  the other walls are colored yellow, and do not affect the energy of the particles that bounce into it.
- COLLIDE?: Turns collisions between particles on and off.


Plots:
- SPEED COUNTS: plots the number of particles in each range of speed.
- SPEED HISTOGRAM: speed distribution of all the particles.  The gray line is the average value, and the black line is the initial average.
- ENERGY HISTOGRAM: distribution of energies of all the particles, calculated as m*(v^2)/2.
- PRESSURE VS. TIME: plots average pressure of the inside of the box over time.
- TEMPERATURE VS. TIME: plots particle temperature inside the box over time and wall temperature over time.
- WALL HITS PER PARTICLE: plots average wall hits per particle over time.

THINGS TO NOTICE
----------------

How does adding heat to the box walls affect the pressure?

How does adding heat to the wall affect the particle behavior?

How does the particle behavior or system response change with only one wall heated instead of all walls heated?

Does the system reach an equilibrium temperature faster when the wall is heated or cooled the same amount in comparison to the temperature of the particles?


THINGS TO TRY
-------------
Try to get the inside temperature to reach the outside temperature.  Is this possible?

Try to increase the wall hits per particle.


EXTENDING THE MODEL
-------------------
Give the wall a mass and see how that affects the behavior of the model.

Close off the right side of the box.  Create two valves on either side to the wall that allow the user to "spurt" particles into the chambers to see how number of particles affects pressure.

Vary the width and length of the box, does this effect how fast the particle temperature changes?


NETLOGO FEATURES
----------------
Notice how the collisions are detected by the turtles and how the code guarantees the same two particles do not collide twice.  What happens if we let the patches detect them?


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2003).  NetLogo GasLab Heat Box model. http://ccl.northwestern.edu/netlogo/models/GasLabHeatBox. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2003 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/GasLabHeatBox for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7500403 true true 30 30 240

clock
true
0
Circle -7500403 true true 30 30 240
Polygon -16777216 true false 150 31 128 75 143 75 143 150 158 150 158 75 173 75
Circle -16777216 true false 135 135 30

nothing
true
0

square
false
0
Rectangle -7500403 true true 0 0 297 299

@#$#@#$#@
NetLogo 3.0beta3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@

