breeds [ atoms arrow ]

atoms-own
[
  left-neighbors    ;; neighboring 10 turtles to the left
  right-neighbors   ;; neighboring 9 turtles to the right
]

globals
[
  moveable-atoms    ;; agent set of all the atoms that can bend
  min-crystal-width ;; used with force to find moveable-turtles
  old-force         ;; used to check if force changed
  bend-amount       ;; keeps track of how much the specimen has bent
  num-crystals      ;; keeps track of the number of distinct crystals
]

to setup
  ca
  let color-list []
  set color-list n-values initial-crystal-num [random-color]

  ;; create atoms in a row across the entire screen
  ask patches [
    if (pycor = screen-edge-y - 5)
    [
      sprout 1
      [
        set breed atoms
        set size 2
        set heading 200
        ;; set each group of atoms to a different color to represent crystals
        let index floor (initial-crystal-num * (xcor + screen-edge-x) / screen-size-x)
        set color (item index color-list)
        ifelse (index mod 2 = 0)
        [ set shape "column" ]
        [ set shape "column2" ]
      ]
    ]
  ]

  ask atoms
  [
    set  left-neighbors atoms with [(xcor > xcor-of myself - 11) and (xcor < xcor-of myself)]
    set right-neighbors atoms with [(xcor > xcor-of myself) and (xcor < xcor-of myself + 10)]
  ]

  ;; show force size and direction
  create-custom-arrow 1
  [
    set shape "arrow"
    set color red
    set heading 180
    ;; put arrow as close to end of specimen as possible without covering it
    setxy (screen-edge-x + 0.5 - force / 450) (screen-edge-y - 3.5 + force / 450)
    set size (force / 200)
  ]

  ;; initialize global variables
  set bend-amount 0
  set num-crystals initial-crystal-num
  set min-crystal-width abs (force / 100 - 11)
  set old-force force
end


to go
  ; check if force has changed
  if (force != old-force)
  [
    ask arrow
    [
      set size (force / 200)
      setxy (xcor + (old-force - force) / 450) (ycor - (old-force - force) / 450)
    ]
    set min-crystal-width abs (force / 100 - 11)
    set old-force force
  ]

  set moveable-atoms atoms with
    [(count  left-neighbors with [color = color-of myself] >= min-crystal-width) and
     (count right-neighbors with [color = color-of myself] >= min-crystal-width - 1)]

  ;; stop if crystals too small
  ifelse (any? moveable-atoms)
  [ move ]
  [
    ifelse (min-crystal-width = 2)
    [ break ]
    [ output-print (word "The specimen can support " force " units of force") ]
    stop
  ]
end

to move
  let moved-turtle (random-one-of moveable-atoms)
  let move-point xcor-of moved-turtle
  let left-of-point (turtles with [(xcor >= move-point - 1) and (xcor < move-point)])
  ask turtles with [xcor >= move-point] [fd 0.05]
  set bend-amount (bend-amount + 0.5)

  ; check if crystal has bent too much
  if (any? left-of-point)
  [
    if (ycor-of one-of left-of-point >= 0.1 + ycor-of moved-turtle)
    [ split-crystal moved-turtle ]
  ]
end

to split-crystal [moved-turtle]
  let old-color color-of moved-turtle
  let new-color random-color
  ask turtles with [xcor >= xcor-of moved-turtle]
  [
    fd 0.2
    ;; change color of right half of broken crystal
    if (color = old-color)
    [ set color new-color ]
  ]
  set num-crystals 1 + num-crystals
end

to break
  ;; break in random place
  let random-x random-int-or-float screen-size-x - screen-edge-x
  ask turtles
  [ if (xcor >= random-x) [ fd 3 ] ]
end

to-report random-color
  report 5 + 10 * random 14 + 4 - random 6
end


; *** NetLogo Model Copyright Notice ***

; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.

; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.

; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Crystallization Bending model.
; http://ccl.northwestern.edu/netlogo/models/CrystallizationBending.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.

; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/CrystallizationBending
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice***
@#$#@#$#@
GRAPHICS-WINDOW
11
10
731
291
35
12
10.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
443
740
538
Command Center
0

BUTTON
186
309
254
342
go
go
T
1
T
OBSERVER
T
NIL

BUTTON
25
309
91
342
NIL
setup
NIL
1
T
OBSERVER
T
NIL

MONITOR
385
302
487
351
bend-amount
bend-amount
1
1

BUTTON
105
309
172
342
go once
go
NIL
1
T
OBSERVER
T
NIL

MONITOR
283
302
366
351
num-crystals
num-crystals
0
1

SLIDER
62
354
221
387
initial-crystal-num
initial-crystal-num
1
20
1
1
1
NIL

SLIDER
62
396
221
429
force
force
0
900
778
1
1
NIL

OUTPUT
269
359
709
408

@#$#@#$#@
WHAT IS IT?
-----------
This model shows how crystal size affects the strength of a single type of metal.  Each individual crystal is represented by a different color.  The bar of metal is held stationary at the left end, while it is pushed downward from the right end.  Due to this applied force, the specimen begins to bend.

Crystals can only bend so much before the regular atomic pattern in the crystal breaks, causing a deformation to form in the middle of it.  This is called cold working.  When a deformation occurs, it is as if the crystal splits into two smaller crystals.  The smaller a crystal is, the more force is required to break it.  Therefore, small forces can only bend large crystals.  Once there are no more crystals large enough to break, the specimen will stop bending, and thus support the force.  However, if the force is large enough, it will break the specimen.

(Note that the actual number of atoms is small compared to a real metal specimen and the metal is only two-dimensional.)


HOW TO USE IT
-------------
Buttons:
SETUP: Resets the simulation, and sets the metal to the correct size.
GO-ONCE: Runs the simulation for one time step.
GO: Runs the simulation continuously until either the GO button is pressed again, or the specimen breaks.

Sliders:
INITIAL-CRYSTAL-NUM: Varies the number of crystals in the initial specimen.
FORCE: Controls how much force is applied to the end of the specimen.

Monitors:
NUM-CRYSTALS: Monitors the average temperature of all the atoms.
BEND-AMOUNT: Keeps track of the time that has elapsed during each run.


THINGS TO TRY
---------------
Run the model several times with different INITIAL-CRYSTAL-NUM and FORCE settings.  How does the initial number of crystals affect the specimen's ductility?  Remember, the more ductile a material is, the more easily it bends.

Set INITIAL-CRYSTAL-NUM to 1 and FORCE to 800.  Run the model several times, increasing the INITIAL-CRYSTAL-NUM each time.  How does the initial number of crystals affect how the specimen bends?  What are some situations where it would be good to use a metal that bends before breaking?


EXTENDING THE MODEL
-------------------
In the real world, the specimen will always begin to bend where it is most ductile, which is in the middle of the largest crystal.  To add this effect to the model, change how the MOVED-TURTLE is selected from MOVEABLE-ATOMS in the MOVE procedure.

In the real world, increasing the initial crystal number also increases how much force is required to break the specimen.  Change the code so this occurs in the model.


NETLOGO FEATURES
------------------
Because the metal bends in only one direction, it only needs one row of turtles.  However, the crystals need several rows of atoms to show their uniform pattern.  Therefore, I created a shape that looks like a stack of three close-packed atoms. I also increased the shape size so each stack is directly next to its neighbor stack.

When the specimen is first generated, the turtles are all lined up in a row.  However, to prevent the atoms in different crystals from lining up, I used two shapes that are mirror images of each other, column and column2.


RELATED MODELS
--------------
Crystallization Basic
Crystallization Directed
Crystallization Moving


CREDITS AND REFERENCES
----------------------
Original implementation: Carrie Hobbs, for the Center for Connected Learning and Computer-Based Modeling.

To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo Crystallization Bending model. http://ccl.northwestern.edu/netlogo/models/CrystallizationBending. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/CrystallizationBending for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

column
false
0
Circle -7566196 true true 133 194 95
Circle -7566196 true true 135 1 92
Circle -7566196 true true 75 99 92

column2
false
0
Circle -7566196 true true 74 0 93
Circle -7566196 true true 76 197 90
Circle -7566196 true true 134 104 94

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7566196 true true 30 30 240

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11030470 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11030470 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11030470 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6525626 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11030470 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11030470 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11030470 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11030470 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11030470 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0pre1
@#$#@#$#@
need-to-manually-make-preview-for-this-model
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
