breeds [acid                            ;;parent acid molecuels
        hydroxide                       ;;base molecules
        hydronium                       ;;hydronium molecules
        water                           ;;water molecules
        con-base                        ;;conjugate base molecules
        con-base2]                      ;;second conjugate base molecules

globals [pH pOH                                         ;;variables for calculating pH
         mmolH mmolOH mmolA mmolA2 mmolHA               ;;variables for calculating amount of molecules
         Hconc OHconc Aconc HAconc A2conc volume        ;;variables for calculating concentrations
         tick]                                          ;;reaction variables

turtles-own [id]                                        ;;variable to control reactions

;;observer procedure to setup the system

to setup
        ca
        ;; setup-plot                                                      ;;sets plotting up
        create-acid starting-acid                                            ;;creates variable amount of acid

        ;;initializes the amount of dissociated acid
        ask acid [if ((random-int-or-float 100) > (99 - Ka1)) [hatch 1 [set breed hydronium fd 3] set breed con-base]]                                                                ;;initializes amount of dissociated acid

        ;;initializes the amount of dissociated conjugate base
        ask con-base [if ((random-int-or-float 100) > (99 - Ka2)) [hatch 1 [set breed hydronium fd 3] set breed con-base2]]                                                          ;;initializes amount of dissociated con-base
        if see-starting-water? [ create-water (100 - (count hydronium)) ]
        calculate-ions                                                  ;;used to calculate pH
        setshapes                                                       ;;assigns shapes
        ask turtles [ set id -1 ]                                       ;;tells turtles to react at setup
        set mmolOH 0
        set volume count turtles
        set tick 1

        ;;randomizes position and headings
        ask turtles [setxy random-int-or-float screen-size-x random-int-or-float screen-size-y rt random-int-or-float 360]
end

;;turtle procedure to assign graphics

to setshapes
        ask water [set shape "water"]
        ask hydronium [set shape "hydronium"]
        ask acid [set shape "acid"]
        ask con-base [set shape "conbase"]
        ask con-base2 [set shape "conbase2"]
        ask hydroxide [set shape "hydroxide"]
end

;;main turtle procedure

to go
        set tick tick + 1
        collide
        setshapes
        ask turtles [fd 1]
        calculate-ions
        if (tick > 30) [plot-pH plot-molecules]
end

;;turtle procedure that tells turtles how to find other turtles
to collide
        ;;hydroxide molecules will react with all proton donor molecules in the model
        if (count hydroxide > 0)
        [ ask hydroxide
                [without-interruption [
                    if any? hydronium-here [set id random-one-of hydronium-here react-bases]
                    if any? acid-here [set id random-one-of acid-here react-bases ]
                    if any? con-base-here [set id random-one-of con-base-here react-bases]
                    ]]
        ]
        ;;acid molecules always have a chance to dissociate to hydronium depending on their


        if (count acid > 0)
        [ ask acid
                [without-interruption [if ((random-int-or-float 100) > (99 - Ka1)) [
                    if any? water-here [set id random-one-of water-here react-acids]
                    ]]]
        ]
        ;;conjugate base molecules always have a chance to dissociate to hydronium depending on their Ka
        if count con-base > 0
        [ ask con-base
                [without-interruption [if ((random-int-or-float 100) > (99 - Ka2))  [
                    if any? water-here [set id random-one-of water-here react-bases]
                    ]]]
        ]
        ;;hydronium molecules always have a chance to recombine with conjugate base to form acid
        if count hydronium > 0
        [ ask hydronium
                [without-interruption [if ((random-int-or-float 100) > Ka1) [
                    if any? con-base-here [set id random-one-of con-base-here react-acids]
                    ]]]
        ]
        ;;hydronium molecules can also recombine with the second conjugate base to form conjugate base
        if count hydronium > 0
        [ask hydronium
                [without-interruption [if ((random-int-or-float 100) > Ka2)  [
                    if any? con-base2-here [ set id random-one-of con-base2-here react-acids ]
                    ]]]
        ]
end

;;turtle procedure that tells base turtles how to react

to react-bases

;;all hydroxide molecules will instantly react with hydronium molecules
        if ((breed-of id) = hydronium) [set breed-of id water set breed water]

;;hydroxide molecules have a high probability of reacting with acid molecules
        if ((breed-of id) = acid) [if ((random-int-or-float 100) > (99 - Ka1)) [set breed-of id con-base set breed water]]

;;hydroxide molecules have a low probablity of reacting with conjugate base
        if ((breed-of id) = con-base) [if ((random-int-or-float 100) > (99 - Ka2)) [set breed-of id con-base2 set breed water]]

;;conjugate base molecules react with water to form hydronium and con-base2
        if ((breed-of id) = water) [set breed-of id hydronium set breed con-base2]
end

;;turtle procedure that tells acid turtles how to react

to react-acids

;;acid molecules react with water to form hydronium and con-base
        if ((breed-of id) = water) [set breed-of id hydronium set breed con-base]

;;hydronium molecules react with con-base to form a
        if ((breed-of id) = con-base) [set breed-of id acid set breed water rt random-int-or-float 360 fd 2]

;;hydronium molecules react with con-base2 to form con-base
        if ((breed-of id) = con-base2) [set breed-of id con-base set breed water rt random-int-or-float 360 fd 2]
end

;;observer procedure that calculates variables for determining the pH

to calculate-ions
        set mmolH count hydronium
        set mmolOH count hydroxide
        set mmolA count con-base
        set mmolHA count acid
        set mmolA2 count con-base2
        set volume count turtles
        set Hconc (mmolH / volume)
        set OHconc (mmolOH / volume)
        set Aconc (mmolA / volume)
        set A2conc (mmolA2 / volume)
        set HAconc (mmolHA / volume)
        calculate-pH
end

;;observer procedure that calculates the ph; above the endpoint, the pH is calculated as a strong acid

to calculate-pH
  locals [ concH concOH ]
  set concH (count hydronium / volume)
  set concOH (count hydroxide / volume)
  ifelse (concH = concOH)
    [ set pH 7 ]
    [ ifelse (concH > concOH)
      [ set pH (- log (concH / 1000) 10) ]
      [ set pOH (- log (concOH / 1000) 10)
        set pH 14 - pOH ] ]
end

;;observer procedure used to add more base to the system

to add-base
        create-custom-hydroxide base-added
                [fd 1 set id who rt random-int-or-float 360]
        setshapes
end

;;plotting procedures



to record-pH
        set-current-plot "Titration Curve"
                set-plot-pen-interval base-added          ;before next plotting, move along x-axis by magnitude of amount added
                plot pH
end

to plot-pH
        set-current-plot "pH Curve"
                plot pH
end

to plot-molecules
       set-current-plot "molecules"
       set-current-plot-pen "con-base"
       plot count con-base
       set-current-plot-pen "acid"
       plot count acid
       set-current-plot-pen "hydroxide"
       plot count hydroxide
       set-current-plot-pen "hydronium"
       plot count hydronium
       set-current-plot-pen "water"
       plot count water
end



; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Copyright 2001 by Uri Wilensky. All rights reserved.

; Permission to use, copy, or modify this software and its documentation for
; educational and research purposes only and without fee is hereby granted,
; provided that this copyright notice and the original authors' names appear
; on all copies and supporting documentation. For any other uses of this
; software, in original or modified form, including but not limited to
; distribution in whole or in part, specific prior permission must be
; obtained from Uri Wilensky. These programs shall not be used, rewritten,
; or adapted as the basis of a commercial software or hardware product
; without first obtaining appropriate licenses from Uri Wilensky. We make no
; representations about the suitability of this software for any purpose. It
; is provided "as is" without express or implied warranty.
@#$#@#$#@
GRAPHICS-WINDOW
266
10
651
416
12
12
15.0
1
10
1
1
1
0

CC-WINDOW
5
641
938
736
Command Center

BUTTON
9
11
98
44
setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
10
45
258
78
go
go
T
1
T
OBSERVER
T
NIL

BUTTON
100
226
164
259
record pH
record-pH
NIL
1
T
OBSERVER
T
NIL

BUTTON
12
80
98
113
add base
add-base
NIL
1
T
OBSERVER
T
NIL

SLIDER
100
10
260
43
starting-acid
starting-acid
0.0
100.0
50.0
1.0
1
NIL

SLIDER
100
80
259
113
base-added
base-added
0.0
100.0
15.0
1.0
1
NIL

PLOT
14
456
253
616
pH Curve
Time
pH
0.0
100.0
0.0
14.0
true
true
PENS
"ph" 1.0 0 -65536 true

PLOT
8
269
253
438
Titration Curve
Vol base
pH
0.0
100.0
0.0
14.0
true
true
PENS
"ph" 1.0 0 -16776961 true

PLOT
265
422
929
627
molecules
time
#
0.0
100.0
0.0
100.0
true
true
PENS
"acid" 1.0 0 -256 true
"hydronium" 1.0 0 -11352576 true
"con-base" 1.0 0 -65281 true
"hydroxide" 1.0 0 -65536 true
"water" 1.0 0 -16776961 true

SWITCH
48
190
226
223
see-starting-water?
see-starting-water?
0
1
-1000

SLIDER
13
115
185
148
Ka1
Ka1
0
10
9
1
1
NIL

SLIDER
14
152
186
185
Ka2
Ka2
0
10
1
1
1
NIL

MONITOR
687
260
780
309
# hydronium
count hydronium
0
1

MONITOR
736
311
833
360
pH
pH
1
1

MONITOR
782
260
884
309
# hydroxide
count hydroxide
0
1

@#$#@#$#@
WHAT IS IT?
-----------
This model is part of the ACID-BASE package - a sub-section of the Connected Chemistry package. Here we have yet another variant on determining the pH of a solution. It is best viewed as the fourth model in the ACID-BASE package. This model depicts a diprotic acid, or an acid that can donate two molecules of hydrogen to a base. The chemical reaction is shown below.

                      Ka1     +      -   Ka2     +     2-
        H A  +  2H O  -->   H O  + H-A   -->  2H O  + A
         2        2          3                  3

We can see that the first proton is donated to water to make a hydronium molecule. After the initial acid is consumed, the second proton is donated to form a second molecule of hydronium ion. It is important to note that the Ka of the first proton is MUCH GREATER than the second Ka. This is because the loss of the first proton generates a negatively charged anion. It is very difficult for bases which have a negative charge or a high electron density to come in close proximity to this anion and pull off the second proton. Because of this, the first proton is most often consumed before the second proton can be donated. The pH for the reaction is determined using the Henderson-Hasselbach equation in two separate instances. While the original acid (H2A) is present, the pH is determined by:
                                    -
                pH = pK1 + log ([H-A ] / [H A])
                                           2
Once the weak acid is depleted, the pH is then determined by:

                                  2-       -
                pH = pK2 + log ([A ] / [H-A ])

The model uses a short-cut equation to approximate the pH of the solution. The equation, which can be used for each pk value of a polyprotic acid transforms the above two equations into the following, respectively.

                pH = 0.5 * (pK1 + log ([HA])
                                        -
                pH = 0.5 * (pK2 + log ([A])


HOW TO USE IT
--------------
Decide how much acid should be present at the start of the simulation with the VOL-ACID slider and press SETUP. Turtles will distribute randomly across the screen. BLUE turtles represent water molecules, GREEN turtles represent hydronium ions, YELLOW turtles are acid molecules, and finally ORANGE turtles are conjugate base molecules. A set amount of water molecules is added each time to the solution. In this model we are using the Ka of acetic-acid, which means that approximately 1% of the original acid turtles are dissociated into 1 conjugate base molecule and 1 hydronium molecule.

Press GO. The turtles will move randomly across the screen and the pH of the solution will be plotted over time on the pH Curve and displayed in the pH monitor.

Observe the effect of adding base to the solution by setting the volume of base with the VOL-BASE slider and pressing ADD-BASE.

When the pH remains at a steady value, press RECORD-PH, which will plot the pH versus the amount of bases added on the titration curve.


THINGS TO NOTICE
----------------
Run a titration and observe the curve. Is there anything unique about its shape?

Look for light blue anions in the solution. How much base does it take before you start seeing them? Is this surprising?

Pay attention to how the molecules interact. Which molecules react with each other?


THINGS TO TRY
-------------
Compare the titration curve of the diprotic acid with that of a buffer? Do you see any similarities? How can you alter the code to test if this diprotic acid acts as a buffer.

Increase the dissociation percentage so that more hydronium ions are generated at setup. What does this do to the pH? Is the Henderson-Hasselbach equation still valid with large Ka?

Notice that the code requires hydroxide molecules to first react with hydronium molecules on a patch before they react with acid molecules. Can you explain why this is? Reverse the code and observe the effect on the system.


EXTENDING THE MODEL
------------------
Alter the code so that base turtles only react with hydronium molecules. What effect is observed? What additional changes do you need to make so that the pH continues to rise with the addition of base?

Substitute short-cut equation for calculating pH with the full equation. Are the values similar?

Try substituting the various pKa values below into the Henderson-Hasselbach equation and observe their effect on the titration curve. What affect does this have on the pH.

        weak acid       pK1             pK2
        ---------       ----            ----
        carbonic                6.5             10.2
        oxalic          1.27            4.27
        glycine         2.34            9.60
        maleic          2.00            6.20


CREDITS AND REFERENCES
----------------------
Thanks to Mike Stieff for his work on this model.


To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo DiProtic Acid model. http://ccl.northwestern.edu/netlogo/models/DiproticAcid. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

acid
true
0
Circle -256 true false 73 74 156
Circle -1 true false 48 199 55
Circle -1 true false 197 198 58

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

conbase
true
0
Circle -65281 true false 78 79 146
Circle -1 true false 42 194 66

conbase2
true
0
Circle -16711681 true false 93 94 116
Circle -1 true false 62 185 57

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

hydronium
true
0
Circle -11352576 true false 91 92 118
Circle -1 true false 66 187 50
Circle -1 true false 186 186 52
Circle -1 true false 127 43 49

hydroxide
true
0
Circle -65536 true false 80 80 142
Circle -1 true false 63 199 55

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

water
true
0
Circle -16776961 true false 78 79 146
Circle -1 true false 40 41 69
Circle -1 true false 194 42 71

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
