globals [
  empty-patches
  cc-count
  cd-count
  dd-count
  dc-count
  current-view
]

turtles-own [
  cooperate-with-same?
  cooperate-with-different?
  birth-rate
  partner
  previous-partners
]

breeds [
  reds
  blues
  greens
  violets
]

to setup
  ca
  immigrate
end

to update-view
  if ( current-view != user-view )
    [ set current-view user-view
      ask turtles  [ update-shape update-color ]
    ]
end

to go
  update-view
  immigrate

  ;; we only need to update this within a cycle if we are not
  ;; requiring births to be local, since then we want to know which
  ;; patches are empty after immigration
  if ( local-birth? = false)
    [ set empty-patches patches with [ not any? turtles-here ] ]
  ask turtles [ birth ]

  ;; perform Prisoner's Dilemma interactions
  interact

  ask turtles [ death ]

  do-plots
end

;; observer-procedure
to interact
  ;; clear our global counter for this round
  set cc-count 0
  set cd-count 0
  set dd-count 0
  set dc-count 0

  ;; reset our history of interaction partners
  ask turtles [ set previous-partners (list) ]

  ifelse local-interact?
    [ ask turtles [ interact-all ] ]
    [ ask turtles [ interact-once ] ]
end

;; observer procedure
to immigrate
  set empty-patches patches with [not any? turtles-here]
  ;; ask up to immigration-rate random empty patches to sprout a new turtle
  ask random-n-of ( min list (count empty-patches) immigration-rate ) empty-patches
  [ sprout 1
    [
      ifelse random 2 = 0 [ set cooperate-with-same? TRUE ] [ set cooperate-with-same? FALSE ]
      ifelse random 2 = 0 [ set cooperate-with-different? TRUE ] [ set cooperate-with-different? FALSE ]
      change-breed
      set birth-rate initial-birth-rate
      update-color
      update-shape
    ]
  ]
end

;; turtle procedure
to give-birth [ empty-neighbor ]
  if empty-neighbor != nobody
    [ hatch 1
      [ mutate
        setxy pxcor-of empty-neighbor pycor-of empty-neighbor
        update-color
        update-shape
      ]
    ]
end

;; turtle procedure
to birth
  if random-float 100 < birth-rate
    [  ifelse local-birth?
         [ give-birth random-one-of neighbors4 with [ count turtles-here = 0 ] ]
         [ ;; we filter out any patches that have been occupied since we computed empty-patches
           give-birth random-one-of empty-patches with [ count turtles-here = 0 ] ]
    ]
  ;; reset birth rate after they get a chance to reproduce
  set birth-rate initial-birth-rate
end

;; turtle procedure
to mutate
  ;; should mutation occur on each gene?
  if random-float 100 < mutation-rate [ change-breed ]
  if random-float 100 < mutation-rate
    [ ifelse cooperate-with-same? [ set cooperate-with-same?  FALSE ] [ set cooperate-with-same? TRUE ] ]
  if random-float 100 < mutation-rate
    [ ifelse cooperate-with-different? [ set cooperate-with-different? FALSE ] [ set cooperate-with-different? TRUE ] ]
end

;; turtle procedure
to change-breed
  let newbreed random-one-of [ "reds" "blues" "greens" "violets" ]
  set breed runresult newbreed
end

;; turtle procedure
to-report breed-color
  report runresult but-last (word breed )
end

to-report behavior-color
  ifelse ethnocentric?
    [ report orange ]
    [ report gray ]
end

to-report current-view-color
  ifelse current-view = "behavior"
    [ report behavior-color ]
    [ report breed-color ]
end

to-report current-view-shape
  ifelse current-view = "behavior"
    [ report behavior-shape ]
    [ report breed-shape ]
end

to-report behavior-shape
  report "circle"
end

to-report breed-shape
  ifelse ethnocentric?
    [ report "x" ]
    [ report "square" ]
end

;; turtle procedure
to update-shape
  set shape current-view-shape
end

to update-color
  set color current-view-color
end

to hilite-groups
  ask turtles [ set pcolor breed-color - 2 ]
end




;; turtle procedure
to death
  if random-float 100 < death-rate [die]
end

;; turtle procedure
to interact-once
  ifelse local-interact?
     [ set partner random-one-of turtles-on neighbors4 ]
     [ set partner random-one-of turtles ]
  if partner != nobody [ update-fitness ]
end


;; turtle procedure
to interact-all
  let partners turtles-on neighbors4
  ;; only interact with turtles we have not interacted with in this round
  foreach values-from partners with [ not member? myself previous-partners ] [ self ]
    [ (set partner ?)  update-fitness ]
end

;; turtle procedure
to update-fitness
  ;; note that we have interacted with this partner
  set previous-partners fput partner previous-partners
  ifelse breed = breed-of partner or random-float 100 >= distinguishability
     [ifelse cooperate-with-same?
        [ifelse cooperate-with-same?-of partner        [ set cc-count cc-count + 1
                                                         set birth-rate birth-rate + ccscore
                                                         set birth-rate-of partner birth-rate-of partner + ccscore]
                                                       [ set cd-count cd-count + 1
                                                         set birth-rate birth-rate + cdscore
                                                         set birth-rate-of partner birth-rate-of partner + dcscore]]
        [ifelse cooperate-with-same?-of partner        [ set dc-count dc-count + 1
                                                         set birth-rate birth-rate + dcscore
                                                         set birth-rate-of partner birth-rate-of partner + cdscore]
                                                       [ set dd-count dd-count + 1
                                                         set birth-rate birth-rate + ddscore
                                                         set birth-rate-of partner birth-rate-of partner + ddscore]]]

     [ifelse cooperate-with-different?
       [ifelse cooperate-with-different?-of partner    [ set cc-count cc-count + 1
                                                         set birth-rate birth-rate + ccscore
                                                         set birth-rate-of partner birth-rate-of partner + ccscore]
                                                       [ set cd-count cd-count + 1
                                                         set birth-rate birth-rate + cdscore
                                                         set birth-rate-of partner birth-rate-of partner + cdscore]]
       [ifelse cooperate-with-different?-of partner    [ set dc-count dc-count + 1
                                                         set birth-rate birth-rate + dcscore
                                                         set birth-rate-of partner birth-rate-of partner + cdscore]
                                                       [ set dd-count dd-count + 1
                                                         set birth-rate birth-rate + ddscore
                                                         set birth-rate-of partner birth-rate-of partner + ddscore]]]
end

;; turtle procedure
to-report ethnocentric?
  report ( cooperate-with-same? ) and (not cooperate-with-different?)
end

;; turtle procedure
to-report egoist?
  report (not cooperate-with-same? ) and (not cooperate-with-different?)
end

;; turtle procedure
to-report altruist?
  report cooperate-with-same? and cooperate-with-different?
end

;; turtle procedure
to-report total-interactions
  report cc-count + cd-count + dd-count + dc-count
end

;; observer procedure
to remove-ethnocentrics
  ask turtles [ while [ ethnocentric? ] [ mutate ] update-shape ]
end


;; observer procedure
to plot-interactions
  if total-interactions > 0
    [ set-current-plot "interactions"
      set-current-plot-pen "cc"
      plot 100 * ( cc-count / total-interactions )
      set-current-plot-pen "cd"
      plot 100 * ( cd-count / total-interactions )
      set-current-plot-pen "dd"
      plot 100 * ( dd-count / total-interactions )
      set-current-plot-pen "dc"
      plot 100 * ( dc-count / total-interactions )
    ]
end

;; observer procedure
to do-plots
  let total count turtles
  let red-total count reds
  let blue-total count blues
  let green-total count greens
  let yellow-total count violets

  set-current-plot "behavior-counts"
  set-current-plot-pen "ethnocentric"
  plot 100 * ( count turtles with [ ethnocentric? ] ) / total
  set-current-plot-pen "non-ethnocentric"
  plot 100 * ( count turtles with [ not ethnocentric? ] ) / total
  set-current-plot-pen "egoist"
  plot 100 * ( count turtles with [ egoist? ] ) / total
  set-current-plot-pen "altruist"
  plot 100 * ( count turtles with [ altruist? ] ) / total

  set-current-plot "ethnicity-counts"
  set-current-plot-pen "reds"
  plot 100 * red-total / total
  set-current-plot-pen "blues"
  plot 100 * blue-total / total
  set-current-plot-pen "greens"
  plot 100 * green-total / total
  set-current-plot-pen "violets"
  plot 100 * yellow-total / total

  set-current-plot "ethnocentricity"
  set-current-plot-pen "reds"
  carefully [ plot 100 * ( count reds with [ ethnocentric?  ] ) / red-total ] [ plot 0 ]
  set-current-plot-pen "blues"
  carefully [ plot 100 * ( count blues with [ ethnocentric? ]  ) /  blue-total ] [ plot 0 ]
  set-current-plot-pen "greens"
  carefully [ plot 100 * ( count greens with [ ethnocentric? ] ) / green-total ] [ plot 0 ]
  set-current-plot-pen "violets"
  carefully [ plot 100 * ( count violets with [ ethnocentric? ] ) / yellow-total ] [ plot 0 ]

  plot-interactions
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2005 by Uri Wilensky.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo Ethnocentrism model.
; http://ccl.northwestern.edu/netlogo/models/Ethnocentrism.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/Ethnocentrism
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
189
10
658
500
25
25
9.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
681
960
776
Command Center
0

BUTTON
10
16
76
49
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
112
16
175
49
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
10
66
186
99
immigration-rate
immigration-rate
0
100
10
1
1
NIL

SLIDER
9
107
188
140
initial-birth-rate
initial-birth-rate
0
100
12
1
1
%

SLIDER
10
147
182
180
death-rate
death-rate
0
100
10
1
1
%

SLIDER
9
185
186
218
mutation-rate
mutation-rate
0
100.0
0.5
0.5
1
%

SLIDER
11
251
183
284
ccscore
ccscore
-5
5
2
1
1
NIL

SLIDER
11
288
183
321
cdscore
cdscore
-5
5
-1
1
1
NIL

SLIDER
11
328
183
361
dcscore
dcscore
-5
5
3
1
1
NIL

SLIDER
11
370
183
403
ddscore
ddscore
-5
5
0
1
1
NIL

PLOT
4
517
532
667
behavior-counts
time
percent
0.0
10.0
0.0
100.0
true
true
PENS
"ethnocentric" 1.0 0 -33024 true
"non-ethnocentric" 1.0 0 -7500403 true
"egoist" 1.0 0 -16777216 true
"altruist" 1.0 0 -556609 true

MONITOR
13
409
107
458
# ethno
count turtles with [ethnocentric?]
0
1

MONITOR
12
463
108
512
# non-ethno
count turtles with [ethnocentric? = FALSE]
0
1

TEXTBOX
666
10
756
38
Xs are ethnocentric

TEXTBOX
755
10
859
55
Squares are not ethnocentric

PLOT
664
45
951
223
ethnicity-counts
Time
Percent
0.0
100.0
0.0
100.0
true
true
PENS
"reds" 1.0 0 -2674135 true
"blues" 1.0 0 -13345367 true
"greens" 1.0 0 -10899396 true
"violets" 1.0 0 -8630108 true

PLOT
665
230
951
386
ethnocentricity
Time
Percent
0.0
100.0
0.0
100.0
true
true
PENS
"reds" 1.0 0 -2674135 true
"blues" 1.0 0 -13345367 true
"greens" 1.0 0 -10899396 true
"violets" 1.0 0 -8630108 true

BUTTON
722
580
871
613
remove ethno
ask turtles with [ ethnocentric? ] [ die ]
NIL
1
T
OBSERVER
T
NIL

SWITCH
540
623
672
656
local-birth?
local-birth?
0
1
-1000

SWITCH
724
623
874
656
local-interact?
local-interact?
0
1
-1000

TEXTBOX
542
557
692
575
Model variations:

PLOT
666
392
926
552
interactions
Time
Percent
0.0
100.0
0.0
100.0
true
true
PENS
"cc" 1.0 0 -955883 true
"dd" 1.0 0 -16777216 true
"dc" 1.0 0 -556609 true
"cd" 1.0 0 -1184463 true

SLIDER
540
578
714
611
distinguishability
distinguishability
0
100
0
1
1
%

CHOOSER
544
505
656
550
user-view
user-view
"breed" "behavior"
0

@#$#@#$#@
WHAT IS IT?
-----------
This model suggests that pre-programmed "ethnocentric" behavior can proliferate under some conditions.  Agents compete for limited space via Prisoner Dilemma's type interactions.   "Ethnocentric" agents treat agents within their group differently from those outside their group.  The model includes a mechanism for inheritance (genetic or cultural) of strategies.


HOW IT WORKS
------------
Each agent has three traits: a) color, b) whether they cooperate with same colored agents, and c) whether they cooperate with different colored agents.  An "ethnocentric" agent is one which cooperates with same colored agents, but does not cooperate with different colored agents. Ethnocentric agents are represented as triangles. An "altruist" cooperates with all agents, while an "egoist" cooperates with no one.

At each time step, the following events occur:

1. Up to IMMIGRATION-RATE, empty patches sprout turtles with random traits.

2. Agents start with an INITIAL-BIRTH-RATE chance of reproducing.  Each pair of adjacent agents interact in a one-move Prisoner's Dilemma in which each chooses whether or not to help the other.  They either gain, or lose some of their potential to reproduce.  See documentation of CCSCORE and related sliders for details.

3. In random order, each agent is given a chance to reproduce.  Offspring have the same traits as their parents, with a MUTATION_RATE chance of each trait mutating.  Each agent's birth-rate is reset to the INITIAL-BIRTH-RATE.

4. Each agent rolls a hundred sided die, and if the result is below DEATH-RATE, it dies, making room for future offspring and immigrants.


HOW TO USE IT
-------------
To prepare the simulation for a new run, press SETUP.  Press GO to start the simulation running, press GO again to stop it.

The USER-VIEW chooser allows you to watch the world from two different perspectives. In the "breed" view, each agent is colored according to its color attribute.  If it is an ethnocentric agent, then it is drawn as an X, otherwise it is drawn as a circle.  In the "behavior" view, all agents are drawn as circles and they are orange if their behavior is ethnocentric, and gray if not.

CCSCORE, CDSCORE, DCSCORE and DDSCORE determine the benefit or cost interacting agents receive when they cooperate or defect.  If both cooperate, they both get CCSCORE added to their birth-rate.  If one cooperates and the other doesn't, the cooperating turtle loses CDSCORE from their birth-rate and the defecting turtle gets DCSCORE added to their birth-rate.  If both defect, they get DDSCORE added to their birth-rates.

DISTINGUISHABILITY determines how accurately turtles can tell the color of the turtle they are interacting with.  It is the chance they will incorrectly perceive the turtle as being the same color.

LOCAL-BIRTHS? controls whether turtles give birth to their offspring only in adjacent empty patches, or in any empty patch.

LOCAL-INTERACTIONS? controls whether turtles interact only with adjacent turtles, or with any turtle on the screen.

The REMOVE-ETHNO button will mutate all ethnocentric agents in the world until they are no longer ethnocentric.


THINGS TO NOTICE
----------------
Observe the interaction along the edge of a group of ethnocentric agents, and non-ethnocentric agents.  What behaviors do you see?  Is one more stable?  Does one expand into the other group?

Observe the ETHNICITY-COUNTS plot.  Does one color grow faster than others?  If so, does it remain the largest ethnicity over a long run?

Observer the INTERACTIONS plot.  Does one strategy occur more than others?  What happens when we change the model?  Is there a relation between the ethnocentricity of the population, as plotted in the BEHAVIOR-COUNTS plot, and INTERACTIONS plot?

Observer the ETHNOCENTRICITY plot.  Is there a relation between the ethnocentricity of a given color, and it's population in the model?  Are all colors uniformly ethnocentric?


THINGS TO TRY
-------------
Turn off LOCAL-BIRTH? and observe the BEHAVIOR-COUNTS and INTERACTIONS plots.

Turn off LOCAL-INTERACTION? and observe the BEHAVIOR-COUNTS and INTERACTIONS plots.

Press the REMOVE-ETHNO button to remove ethnocentric agents.  Do they re-appear?  Why?

Vary the DISTINGUISHABILITY? slider and observe the BEHAVIOR-COUNTS and INTERACTIONS plots.  How does distinguishability effect them?

Change the values of CCSCORE, DDSCORE, DCSCORE, and CDSCORE and observe the effects on the model and the level of ethnocentricity.


EXTENDING THE MODEL
-------------------
Add more colors to the model.  Will the behavior change?

Make some patches richer than others, so that agents on them have a higher chance of reproducing.  Distribute this advantage across the world in different ways, in blobs, randomly, in discrete quarters.

Tag patches with a color.  distribute the colors across the world in different ways: blobs, randomly, in discrete quarters.  Agents use the patch color under other agents to determine whether to cooperate with them or not.


NETLOGO FEATURES
----------------
This model comes with three BehaviorSpace experiments defined:

The "Distinguishability Dynamics" experiment runs the model while varying the distinguishability variable.

The "Mutation Dynamics" experiment runs the model while varying the mutation rate.

The "Localization Dynamics" runs the model while varying the local-birth? and local-interaction? switches.


RELATED MODELS
--------------
See the Segregation model.


CREDITS AND REFERENCES
----------------------
This model is a NetLogo version of the ethnocentrism model presented by Robert Axelrod at Northwestern University at the NICO (Northwestern Institute on Complex Systems) conference on October 25th, 2003.

See also: Robert Axelrod and Ross A. Hammond, The Evolution of Ethnocentric Behavior, http://www-personal.umich.edu/~axe/research/AxHamm_Ethno.pdf

To refer to this model in academic publications, please use:
Wilensky, U. (2005). NetLogo Ethnocentrism model. http://ccl.northwestern.edu/netlogo/models/Ethnocentrism. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use:
Copyright 2005 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/Ethnocentrism for terms of use.

@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="Distinguishability Dynamics" repetitions="1" runMetricsEveryTick="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit ticks="500"/>
    <metric>count turtles</metric>
    <metric>count turtles with [ ethnocentric? ]</metric>
    <metric>count turtles with [ egoist? ]</metric>
    <metric>count turtles with [ altruist? ]</metric>
    <metric>count turtles with [ not ethnocentric? ]</metric>
    <metric>total-interactions</metric>
    <metric>cc-count</metric>
    <metric>cd-count</metric>
    <metric>dd-count</metric>
    <metric>dc-count</metric>
    <metric>count reds with [ ethnocentric? ]</metric>
    <metric>count reds with [ not ethnocentric? ]</metric>
    <metric>count reds with [ altruist? ]</metric>
    <metric>count reds with [ egoist? ]</metric>
    <metric>count blues with [ ethnocentric? ]</metric>
    <metric>count blues with [ not ethnocentric? ]</metric>
    <metric>count blues with [ altruist? ]</metric>
    <metric>count blues with [ egoist? ]</metric>
    <metric>count greens with [ ethnocentric? ]</metric>
    <metric>count greens with [ not ethnocentric? ]</metric>
    <metric>count greens with [ altruist? ]</metric>
    <metric>count greens with [ egoist? ]</metric>
    <metric>count violets with [ ethnocentric? ]</metric>
    <metric>count violets with [ not ethnocentric? ]</metric>
    <metric>count violets with [ altruist? ]</metric>
    <metric>count violets with [ egoist? ]</metric>
    <steppedValueSet variable="distinguishability" first="0" step="10" last="100"/>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ddscore">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="cdscore">
      <value value="-1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dcscore">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="death-rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-interact?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-birth?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ccscore">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-birth-rate">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immigration-rate">
      <value value="10"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Mutation Dynamics" repetitions="1" runMetricsEveryTick="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit ticks="500"/>
    <metric>count turtles</metric>
    <metric>count turtles with [ ethnocentric? ]</metric>
    <metric>count turtles with [ egoist? ]</metric>
    <metric>count turtles with [ altruist? ]</metric>
    <metric>count turtles with [ not ethnocentric? ]</metric>
    <metric>total-interactions</metric>
    <metric>cc-count</metric>
    <metric>cd-count</metric>
    <metric>dd-count</metric>
    <metric>dc-count</metric>
    <metric>count reds with [ ethnocentric? ]</metric>
    <metric>count reds with [ not ethnocentric? ]</metric>
    <metric>count reds with [ altruist? ]</metric>
    <metric>count reds with [ egoist? ]</metric>
    <metric>count blues with [ ethnocentric? ]</metric>
    <metric>count blues with [ not ethnocentric? ]</metric>
    <metric>count blues with [ altruist? ]</metric>
    <metric>count blues with [ egoist? ]</metric>
    <metric>count greens with [ ethnocentric? ]</metric>
    <metric>count greens with [ not ethnocentric? ]</metric>
    <metric>count greens with [ altruist? ]</metric>
    <metric>count greens with [ egoist? ]</metric>
    <metric>count violets with [ ethnocentric? ]</metric>
    <metric>count violets with [ not ethnocentric? ]</metric>
    <metric>count violets with [ altruist? ]</metric>
    <metric>count violets with [ egoist? ]</metric>
    <enumeratedValueSet variable="distinguishability">
      <value value="100"/>
    </enumeratedValueSet>
    <steppedValueSet variable="mutation-rate" first="0" step="0.5" last="50"/>
    <enumeratedValueSet variable="ddscore">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="cdscore">
      <value value="-1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dcscore">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="death-rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-interact?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-birth?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ccscore">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-birth-rate">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immigration-rate">
      <value value="10"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Localization Dynamics" repetitions="5" runMetricsEveryTick="true">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit ticks="500"/>
    <metric>count turtles</metric>
    <metric>count turtles with [ ethnocentric? ]</metric>
    <metric>count turtles with [ egoist? ]</metric>
    <metric>count turtles with [ altruist? ]</metric>
    <metric>count turtles with [ not ethnocentric? ]</metric>
    <metric>total-interactions</metric>
    <metric>cc-count</metric>
    <metric>cd-count</metric>
    <metric>dd-count</metric>
    <metric>dc-count</metric>
    <metric>count reds with [ ethnocentric? ]</metric>
    <metric>count reds with [ not ethnocentric? ]</metric>
    <metric>count reds with [ altruist? ]</metric>
    <metric>count reds with [ egoist? ]</metric>
    <metric>count blues with [ ethnocentric? ]</metric>
    <metric>count blues with [ not ethnocentric? ]</metric>
    <metric>count blues with [ altruist? ]</metric>
    <metric>count blues with [ egoist? ]</metric>
    <metric>count greens with [ ethnocentric? ]</metric>
    <metric>count greens with [ not ethnocentric? ]</metric>
    <metric>count greens with [ altruist? ]</metric>
    <metric>count greens with [ egoist? ]</metric>
    <metric>count violets with [ ethnocentric? ]</metric>
    <metric>count violets with [ not ethnocentric? ]</metric>
    <metric>count violets with [ altruist? ]</metric>
    <metric>count violets with [ egoist? ]</metric>
    <enumeratedValueSet variable="distinguishability">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutation-rate">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ddscore">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="cdscore">
      <value value="-1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dcscore">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="death-rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-interact?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="local-birth?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ccscore">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-birth-rate">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immigration-rate">
      <value value="10"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
