turtles-own
[
  behavior  ;; Holds the turtle's behavior procedure
  dna       ;; Holds the turtle's genetic code
  memory    ;; Holds the turtle's memory, behavior is responsible for use of this variable
  ;attribute; ;value;
   max-food    food
   max-water   water
   max-age     age
   vision      sight
]

patches-own
[
  soil     ;; quality of the soil
  aquafer  ;; ammount of water in soil
  humidity ;; percent humidity
  growth   ;; ammount of vegitation
]

;;;;;;;;;;;;;;;;;;;;;;;;
;;; Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;
to setup
  ca
  create-terrain
end

to create-terrain
  ask patches
  [
    alt-soil (mean-soil-quality + random-int-or-float 50 - random-int-or-float 50)
    aquafy (mean-aquafer + random-int-or-float 50 - random-int-or-float 50)
    humidify (mean-humidity + random-int-or-float 50 - random-int-or-float 50)
  ]
  diffuse soil 1.0
  diffuse aquafer 1.0
  diffuse humidity 1.0
  ask patches with [pcolor != blue]
  [
    ifelse aquafer * soil > random-int-or-float 1000.0
    [ grow (aquafer * soil / 125 + random-int-or-float 50 - random-int-or-float 50) ]
    [ color-soil ]
  ]
end

to create-animals
  cct number
  [ create-animal ]
end

to create-animal
  set dna "f:"  + (10 + random-int-or-float 100)
        + ",w:" + (10 + random-int-or-float 100)
        + ",v:" + (1 + random-int-or-float 5)
        + ",a:" + (100 + random-int-or-float 50)
        + ",c:" + white
        + ",s:" + "\"sheep\""
  create-from-dna dna
  set food max-food
  set water max-water
  let loc random-one-of patches with [pcolor != blue]
  setxy pxcor-of loc pycor-of loc
  set behavior
  " if (water / max-water) < 0.2 or water < 10\n" +
  " [ find-water ]\n" +
  " if (food / max-food) < 0.2 or food < 10\n" +
  " [ find-food ]\n" +
  " move\n eat\n drink\n bread\n"
end

to breed-animal [parent1 parent2]
  let genome1 get-genome dna-of parent1
  let genome2 get-genome dna-of parent2
  set dna get-dna splice-genomes genome1 genome2
  create-from-dna dna
end

to create-from-dna [dna-in]
  let genome get-genome dna-in
  let n 0
  repeat length genome
  [
    let gene-n (gene n genome)
    let gene-name name gene-n
    if gene-name = "f" [ set max-food  value gene-n ]
    if gene-name = "w" [ set max-water value gene-n ]
    if gene-name = "v" [ set vision    value gene-n ]
    if gene-name = "a" [ set max-age   value gene-n ]
    if gene-name = "c" [ set color     value gene-n ]
    if gene-name = "s" [ set shape read-from-string value gene-n ]
    set n n + 1
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Runtime Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;
to go-environ  ;; Patch Procedure
  ask patches
  [
    ifelse growth <= 0
    [ alt-soil -1 ] ;; Erosion
    [
      ifelse aquafer < 20
      [ grow -0.2 ] ;; Wither w/o enough water
      [
        grow 1
        ask random-one-of neighbors
        [ if pcolor != blue [ grow 0.1 ] ]
      ]
      alt-soil 0.1
    ]
    if pcolor = blue
    [
      alt-soil 1 ;; Sedementation
      aquafy -2 ;; Evaporation
    ]
  ]
  diffuse soil 0.1
  ;diffuse aquafer 0.1
  diffuse humidity 1.0
  ask patches
  [
    if humidity > 70
    [ aquafy 0.1 * humidity ]
    if shade-of? pcolor brown [ color-soil ]
    if shade-of? pcolor green [ color-growth ]
  ]
end

to go-animals ;; Turtle Procedure
  run behavior
  if food <= 0 or water <= 0 or age > max-age or pcolor = blue
  [
    let died-from ""
    if food <= 0 [ set died-from died-from + " Starvation" ]
    if water <= 0 [ set died-from died-from + " Thirst" ]
    if age > max-age [ set died-from died-from + " Old-Age" ]
    if pcolor = blue [ set died-from died-from + " Drowning" ]
    if obituaries?
    [ show-report died-from ]
    die
  ]
  set food food - 1
  set water water - 1
  set age age + 1
end

to show-report [died-from]
  print "Turtle#" + who + ":" + died-from
  print "  Age: " + precision age 3 + "/" + precision max-age 3
  print "  Food: " + precision food 3 + "/" + precision max-food 3
  print "  Water: " + precision water 3 + "/" + precision max-water 3
  print "  Vision: " + precision vision 3
  ;print "  Color: " + color
  ;print "  DNA: " + dna
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Behavior Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;
to look
  set sight patches in-radius vision
end

to find-water
  look
  ;set color red show vision
  ifelse any? sight with [pcolor = blue]
  [ set heading towards min-one-of sight with [pcolor = blue] [distance myself] ]
  [ rt random-int-or-float 360.0 ]
end

to find-food
  look
  ifelse any? sight with [shade-of? pcolor green]
  [ set heading towards min-one-of sight with [shade-of? pcolor green] [distance myself] ]
  [ rt random-int-or-float 360.0 ]
end

to move
  while [pcolor-of (patch-ahead 1) = blue and any? neighbors with [pcolor != blue] ]
  [ set heading towards random-one-of neighbors with [pcolor != blue] ]
  if pcolor-of (patch-ahead 1) != blue
  [ fd 1 ]
end

to eat
  if (growth > 0)
  [
    if food < max-food
    [
      let n min list growth 10
      ifelse food + n < max-food
      [
        grow (- n)
        set food food + n
      ]
      [
        grow (food - max-food)
        set food max-food
      ]
    ]
  ]
end

to drink
  if any? neighbors with [pcolor = blue]
  [
    if water < max-water
    [
      ifelse water + 10 < max-water
      [ set water water + 10 ]
      [ set water max-water ]
    ]
  ]
  ;set color scale-color blue water 0 max-water
end

to bread
  if age > 20 and any? other-turtles-here with [age > 20] and (food / max-food > 0.4) and (water / max-water > 0.4)
  [
    let partner random-one-of other-turtles-here with [age > 20]
    hatch 1
    [
      breed-animal myself partner
      set age 0
      set food 0.2 * max-food-of myself
      set water 0.2 * max-water-of myself
    ]
    set food food - 0.2 * max-food
    set water water - 0.2 * max-water
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Utility Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;
to grow [ammount]  ;; Patch Procedure (Turtle Usable)
  set growth growth + ammount
  ;; Evaporate Water to Grow
  if ammount > 0
  [
    aquafy (ammount / -2)
  ]
  if growth > soil [ set growth soil ]
  ifelse growth > 0
  [ color-growth ]
  [
    set growth 0
    color-soil
  ]
end

to alt-soil [ammount]  ;; Patch Procedure
  set soil soil + ammount
  if soil > 100 [ set soil 100 ]
  if soil < 0 [ set soil 0 ]
  if shade-of? pcolor brown [ color-soil ]
end

to aquafy [ammount]  ;; Patch Procedure
  set aquafer aquafer + ammount
  humidify -2 * ammount
  if aquafer > 100 [ set aquafer 100 ]
  if aquafer < 0 [ set aquafer 0 ]
  ;; Surface Water if Aquafer full enough
  if aquafer > 90
  [
    set pcolor blue
    set growth 0
    alt-soil 10
  ]
  if not shade-of? pcolor green and aquafer < 75
  [ color-soil ]
end

to humidify [ammount]  ;; Patch Procedure
  set humidity humidity + ammount
  if humidity > 100 [ set humidity 100 ]
  if humidity < 0 [ set humidity 0 ]
end

to color-growth
  set pcolor scale-color green growth 110 -30
end

to color-soil
  set pcolor scale-color brown soil 120 -20
end

to color-aquafer
  set pcolor scale-color blue aquafer 120 -20
end

to color-humidity
  set pcolor scale-color sky humidity 120 -20
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Genetics Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;
to-report string-to-list [string delimiter]
  let lst []
  while [not empty? string]
  [
    let sub ""
    while [not empty? string and first string != delimiter]
    [
      set sub sub + first string
      set string but-first string
    ]
    set string but-first string
    set lst lput sub lst
  ]
  report lst
end

to-report get-genome [dna-in]
  let genome string-to-list dna-in ","
  let n 0
  repeat length genome
  [
    let gene-n (string-to-list (item n genome) ":")
    if (item 0 (value gene-n)) != "\""
    [ set gene-n (replace-item 1 gene-n (read-from-string (value gene-n))) ]
    set genome (replace-item n genome gene-n)
    set n n + 1
  ]
  report genome
end

to-report get-dna [genome]
  let dna-out ""
  let n 0
  repeat length genome
  [
    set dna-out dna-out + name (gene n genome) + ":" + (value (gene n genome)) + ","
    set n n + 1
  ]
  report but-last dna-out
end

to-report gene-by-name [name1 genome]
  let n 0
  repeat length genome
  [
    if (item 0 (item n genome)) = name1
    [ report (item n genome) ]
    set n n + 1
  ]
  report list "-" "0.0"
end

to-report gene [n genome]
  report (item n genome)
end

to-report name [gene1]
  report (item 0 gene1)
end

to-report value [gene1]
  report (item 1 gene1)
end

to-report splice-genomes [genome1 genome2]
  let genome []
  let n 0
  repeat length genome1
  [
    let gene1 (gene n genome1)
    let gene2 (gene-by-name (name gene1) genome2)
    set genome lput (splice gene1 gene2) genome
    set n n + 1
    set genome2 remove gene2 genome2
  ]
  set n 0
  repeat length genome2
  [
    let gene2 (gene n genome2)
    let gene1 (gene-by-name (name gene2) genome1)
    set genome lput (splice gene2 gene1) genome
    set n n + 1
  ]
  report genome
end

to-report splice [gene1 gene2]
  if is-number? value gene1 and is-number? value gene2
  [ report list (name gene1) (((value gene1) + (value gene2)) / 2) ]
  if is-string? value gene1 and is-string? value gene2
  [ report list (name gene1) (value gene1) ]
  if is-string? value gene1
  [ report list (name gene1) (value gene1) ]
  if is-string? value gene2
  [ report list (name gene1) (value gene2) ]
end

@#$#@#$#@
GRAPHICS-WINDOW
321
10
646
356
17
17
9.0
1
10
1
1
1
0

CC-WINDOW
5
495
801
590
Command Center

BUTTON
9
42
119
75
NIL
Setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
9
141
119
174
NIL
Go-Animals
T
1
T
TURTLE
T
NIL

BUTTON
9
75
119
108
NIL
Go-Environ
T
1
T
OBSERVER
T
NIL

SLIDER
128
108
302
141
mean-aquafer
mean-aquafer
0
100.0
85.0
1
1
%

SLIDER
128
141
302
174
number
number
0
100
20
1
1
animals

MONITOR
9
174
119
223
NIL
count turtles
0
1

SWITCH
128
174
302
207
obituaries?
obituaries?
1
1
-1000

MONITOR
321
370
414
419
avg max-food
mean values-from turtles [max-food]
3
1

MONITOR
433
371
533
420
avg max-water
mean values-from turtles [max-water]
3
1

MONITOR
319
431
407
480
avg max-age
mean values-from turtles [max-age]
3
1

MONITOR
430
432
499
481
avg vision
mean values-from turtles [vision]
3
1

SLIDER
128
42
302
75
mean-soil-quality
mean-soil-quality
0
100.0
85.0
1
1
%

SLIDER
128
75
302
108
mean-humidity
mean-humidity
0
100.0
90.0
1
1
%

TEXTBOX
653
55
792
183
Legend:\nBlue = Water\nBrown = Soil\nLight <----> Dark\nPoor  <----> Rich\nGreen = Vegetation\nLight <----> Dark\nThin  <----> Heavy

MONITOR
652
189
746
238
soil-here
soil-of patch-at mouse-xcor mouse-ycor
3
1

MONITOR
652
287
746
336
humidity-here
humidity-of patch-at mouse-xcor mouse-ycor
3
1

MONITOR
652
336
746
385
growth-here
growth-of patch-at mouse-xcor mouse-ycor
3
1

BUTTON
9
108
119
141
NIL
Create-Animals
NIL
1
T
OBSERVER
T
NIL

MONITOR
652
238
746
287
aquafer-here
aquafer-of patch-at mouse-xcor mouse-ycor
3
1

@#$#@#$#@
add model documentation here
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7566196 true true 30 75 210 225
Circle -7566196 true true 135 75 150
Rectangle -16777216 true false 166 225 219 286
Rectangle -16777216 true false 62 225 120 285
Circle -16777216 true false 207 88 92

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
