globals [
  id-last-selected
  critter-xcor critter-ycor
  selection-state mouse-button-state
  min-segment-size max-segment-size
  min-init-body-segments
  number-generations
  total-critters-created
  segment-shape
  ]

breeds [critter-segments selectors critters edges]
selectors-own [parent-id selected?]
critters-own [selected? drag? new? DNA tRNA relative-heading relative-r]
critter-segments-own [parent-id relative-heading relative-r]



to setup
  ca
  set total-critters-created 0
  set number-generations 0
  set-default-shape edges "line"
  set-default-shape critter-segments "circle"
  set-default-shape selectors "selected"
  set mouse-button-state ""
  set id-last-selected nobody
  set  selection-state "deselected"
  set critter-xcor 0
  set critter-ycor 0
  set min-segment-size 1
  set max-segment-size 3
  set min-init-body-segments 2
  make-initial-critters
  ask patches [set pcolor background-color]
end


to setup-selectors
  set breed selectors
  set hidden? true
  set size 3.5
end


to go
  ;; visualization changes
 ask patches [set pcolor background-color]
 ifelse (family-tree?) [ask edges [set hidden? false]] [ask edges [set hidden? true]]
 ask selectors [set heading (heading + 1)]

 ;; mouse events
 listen-select-critters
 without-interruption [listen-move-critters]

 if move-critters? [without-interruption  [move-critters]]
 without-interruption [ grow-all-segments]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


to make-initial-critters
  let id-for-parts nobody
  repeat init-number-critters [          ;;; make a variable set of critters
    without-interruption [create-custom-critters 1 [
    set new? true
    set color 15
    set id-for-parts who
    set selected? true
    set drag? false
    set DNA []
    set tRNA []

    ;;; for each critter make its body segments
      without-interruption [repeat (min-init-body-segments + random (max-init-body-segments - min-init-body-segments)) [
          if (random 100 < initial-body-variability) [change-next-piece]  ;;; setup random build instructions
          set-DNA                                                         ;;; add build instructions to DNA string
        ]
        set tRNA DNA
       ]]
     ]
  ]

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


to make-all-offspring
  let selector-parent-id -1
  set number-generations ( number-generations + 1)

  without-interruption [ ask critters with [selected?] [
    set total-critters-created (total-critters-created + 1)
    set selector-parent-id self

    without-interruption [hatch 1  [  ;; create a copy of the DNA from the parent and the edge between them
      apply-mutations
      set tRNA DNA
      set new? true
      __create-edges-from selector-parent-id [set color grey  __set-line-thickness .1 set hidden? false]
      ]
    ]
    ifelse (keep-parents-selected?) [select-this-critter]
       [ask critters with [not new?] [without-interruption [deselect-this-critter]] ] ;;  deselect all parents
  ]]
end


to apply-mutations
    if (random-float 100 < chance-substitution-mutation) [substitute-gene-segment]
    if (random-float 100 < chance-deletion-mutation) [delete-gene-segment]
    if (random-float 100 < chance-duplication-mutation) [duplicate-gene-segment]
end



to grow-all-segments            ;; this procedure builds the body segments on newly hatched offspring
  ask critters with [new?]  [
    let id-for-parts who        ;; identify the parent-id that the segments will belowng to
    let critter-origin self
    repeat length DNA [add-one-segment set tRNA remove-item 0 tRNA]  ;; for each segment add, remove one triplet from the tRNA copy of the DNA
      set new? false hatch 1 [setup-selectors set parent-id id-for-parts]   ;; make one selector turtle for this critter
          ifelse (add-offspring-to-selected?)
             [select-this-critter]
             [deselect-this-critter]


         without-interruption [ask critter-segments with [id-for-parts = parent-id] [
            set relative-r distance critter-origin          ;; these are relative coordinates in (r, theta) of where
            set relative-heading towards critter-origin    ;; the segements are located relative to the critter's head
            set relative-heading (relative-heading + 180)
       ]  ]

  ]

end


to add-one-segment
 let critter-origin self

 let id-for-parts who
 let mRNA item 0 tRNA  ;;mRNA is the next triplet from the tRNA


 run item 0 mRNA   ;;the third NetLogo instruction for the segment color

 run item 1 mRNA   ;;the first NetLogo instruction for the segment size
 let this-segment-radius (size / 2)
 fd this-segment-radius * (segment-separation / 100)

 hatch 1 [
    set breed critter-segments
    set parent-id id-for-parts
    set hidden? false

  ]

 run item 2 mRNA   ;;the second NetLogo instruction for the segment heading
 fd this-segment-radius * (segment-separation / 100)

end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move-critters
   without-interruption [ask critters with [not drag? and not new?] [
     set heading (heading + random-float wander-angle - random-float  wander-angle)
     forward wander-speed
     let empty-reporter (move-critter-to xcor ycor)
   ]]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



to change-next-piece
        set heading (heading + random-float max-angle-drift - random-float max-angle-drift)
        let new-size (size + ((random-float 2) -  (random-float 2)) / 2 )
        if new-size < 1 [set new-size 1]
        if  new-size  > 3 [set  new-size  3]
        set size  new-size
        set color (color + (random-float max-color-drift) - (random-float max-color-drift))
end


to set-DNA
  let triplet []
  set triplet lput ("set color " + color) triplet
  set triplet lput ("set size " + size) triplet
  set triplet lput ("set heading " + heading) triplet
  set DNA lput triplet DNA
end


to drift-triplet-from-last
  let triplet []
  set triplet lput ("set color " + color) triplet
  set triplet lput ("set size " + size) triplet
  set triplet lput ("set heading " + heading) triplet
  set DNA lput triplet DNA
end


to substitute-gene-segment

  let number-original-items (length DNA)
  let target-item (random number-original-items)
  let DNA-first-set sublist DNA 0 (target-item)
  let DNA-last-set sublist DNA (target-item) (number-original-items)

  change-next-piece

  let triplet []
  set triplet lput ("set color " + color) triplet
  set triplet lput ("set size " + size) triplet
  set triplet lput ("set heading " + heading) triplet

  set DNA-first-set lput triplet DNA-first-set
  set DNA sentence DNA-first-set DNA-last-set

end


to duplicate-gene-segment
  let number-original-items (length DNA)
  let target-item (random number-original-items)
  let DNA-first-set sublist DNA 0 (target-item + 1)
  let DNA-target-set item (target-item) DNA
  let DNA-last-set sublist DNA (target-item + 1) (number-original-items)
  set DNA-first-set lput DNA-target-set DNA-first-set
  set DNA sentence DNA-first-set DNA-last-set
end


to delete-gene-segment
  let number-original-items (length DNA)
  let target-item (random number-original-items)
  if (length DNA > 0) [set DNA remove-item (target-item) DNA  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


to listen-select-critters
  let snap-shot-mouse-xcor mouse-xcor
  let snap-shot-mouse-ycor mouse-ycor

 if interact-mode = "select" [

   if not mouse-down? [set mouse-button-state "up"]

   if (mouse-down? and mouse-button-state = "up") [
      if  ( mouse-button-state != "down deselect" ) [

         if (any? critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [
           ask random-one-of critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected parent-id]
           ask critters with [who = id-last-selected and not selected?] [select-this-critter set mouse-button-state "down select"]
         ]

         if (any? critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [                                 ;; if no critters currently being dragged
           ask random-one-of critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected who]
           ask critters with [who = id-last-selected and not selected?] [select-this-critter set mouse-button-state "down select"]
         ]
      ]

      if  ( mouse-button-state != "down select") [
         if (any? critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [
           ask random-one-of critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected parent-id]
           ask critters with [who = id-last-selected and selected?] [deselect-this-critter set mouse-button-state "down deselect"]
         ]

         if (any? critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [                                 ;; if no critters currently being dragged
           ask random-one-of critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected who]
           ask critters with [who = id-last-selected and selected?] [deselect-this-critter set mouse-button-state "down deselect"]
         ]
      ]

    ]

  ]
end



to listen-move-critters
  let snap-shot-mouse-xcor (mouse-xcor)
  let snap-shot-mouse-ycor ( mouse-ycor)

  if interact-mode = "move" [
    ;; for any critters in drag state
    if (any? critters with [drag?]) [
      ask critters with [drag?] [let empty-reporter (move-critter-to snap-shot-mouse-xcor snap-shot-mouse-ycor)]
    ]
   ;; if there are critters here
   if (any? critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [
     if (mouse-down? and not any? critters with [drag?]) [                                 ;; if no critters currently being dragged
       ask random-one-of critter-segments with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected parent-id]
       ask critters with [who = id-last-selected] [set drag? true]
     ]
   ]
   ;; if there are critters here
     if (any? critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor]) [
        if (mouse-down? and not any? critters with [drag?]) [                                 ;; if no critters currently being dragged
           ask random-one-of critters with [pxcor = round snap-shot-mouse-xcor and pycor = round snap-shot-mouse-ycor] [set id-last-selected who]
           ask critters with [who = id-last-selected] [set drag? true]
     ]
   ]



  if (not mouse-down?) [set id-last-selected nobody ask critters [set drag? false] ]
]
end


to remove-selected
 without-interruption [
   ask critters with [selected?] [
     set id-last-selected who
     ask selectors with [parent-id = id-last-selected] [die]
     ask critter-segments with [parent-id = id-last-selected] [die]
     die
   ]
 ]
end


to remove-unselected
 without-interruption [
   ask critters with [not selected?] [
     set id-last-selected who
     ask selectors with [parent-id = id-last-selected] [die]
     ask critter-segments with [parent-id = id-last-selected] [die]
     die
   ]
 ]
end


to deselect-all
 without-interruption [
   ask critters [
     set id-last-selected who
     set selected? false
     ask selectors with [parent-id = id-last-selected] [set hidden? true]
   ]
 ]
end


to select-all
 without-interruption [
   ask critters [
     set id-last-selected who
     set selected? true
     ask selectors with [parent-id = id-last-selected] [set hidden? false]
   ]
 ]
end


to select-this-critter
  set selected? true set id-last-selected who
  ask selectors with [parent-id = id-last-selected] [set hidden? false]
end


to deselect-this-critter
  set selected? false set id-last-selected who
  ask selectors with [parent-id = id-last-selected] [set hidden? true]
end



to-report  move-critter-to [x y ]
     set id-last-selected who
     setxy x y
     let theta heading
     ask selectors with [parent-id = id-last-selected] [setxy x y]  ;; rebuild the selector around the new location
     ask critter-segments with [parent-id = id-last-selected] [                          ;; rebuild the critter parts around the new location
     setxy x y                                 ;; set the parts at the location of the head, then..
     set heading (relative-heading)
  ;;   if relative-heading > 270 [set heading (relative-heading + theta + 180)]
  ;; if who = 25 or who = 22 [set heading (relative-heading + theta + 180)]
     jump relative-r
   ]
     report 0
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


@#$#@#$#@
GRAPHICS-WINDOW
303
10
805
533
20
20
12.0
1
16
1
1
1
0
1
1
1

CC-WINDOW
5
744
814
839
Command Center
0

BUTTON
206
10
296
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
204
46
298
79
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
0
112
190
145
background-color
background-color
100
110
103
1
1
NIL

SLIDER
1
44
198
77
max-init-body-segments
max-init-body-segments
2
20
6
1
1
NIL

SLIDER
1
10
192
43
init-number-critters
init-number-critters
0
50
1
1
1
NIL

BUTTON
200
82
297
115
make offspring
make-all-offspring
NIL
1
T
OBSERVER
T
NIL

CHOOSER
199
121
297
166
interact-mode
interact-mode
"move" "select"
0

SWITCH
45
407
166
440
family-tree?
family-tree?
1
1
-1000

SLIDER
3
511
207
544
chance-substitution-mutation
chance-substitution-mutation
0
100
20
1
1
NIL

SLIDER
3
447
207
480
chance-deletion-mutation
chance-deletion-mutation
0
100
20
1
1
NIL

SLIDER
2
480
207
513
chance-duplication-mutation
chance-duplication-mutation
0
100
95
1
1
NIL

SLIDER
0
78
192
111
initial-body-variability
initial-body-variability
0
100
50
1
1
NIL

SWITCH
3
279
202
312
keep-parents-selected?
keep-parents-selected?
1
1
-1000

SWITCH
5
244
201
277
add-offspring-to-selected?
add-offspring-to-selected?
0
1
-1000

BUTTON
146
358
288
391
remove unselected
remove-unselected
NIL
1
T
OBSERVER
T
NIL

BUTTON
147
322
287
355
remove selected
remove-selected
NIL
1
T
OBSERVER
T
NIL

SLIDER
66
546
204
579
max-angle-drift
max-angle-drift
0
180
90.7
0.1
1
NIL

SLIDER
65
612
206
645
max-size-drift
max-size-drift
0
2
1.9
0.1
1
NIL

SLIDER
65
581
206
614
max-color-drift
max-color-drift
0
10
5.0
0.1
1
NIL

BUTTON
8
359
116
392
deselect all
deselect-all
NIL
1
T
OBSERVER
T
NIL

BUTTON
10
321
117
354
select all
select-all
NIL
1
T
OBSERVER
T
NIL

MONITOR
5
189
101
238
critters created
total-critters-created
3
1

MONITOR
105
189
193
238
generations
number-generations
3
1

SLIDER
134
660
283
693
wander-angle
wander-angle
0
20
2.4
0.1
1
NIL

SWITCH
64
697
213
730
move-critters?
move-critters?
0
1
-1000

SLIDER
3
660
131
693
wander-speed
wander-speed
0
1
0.18
0.01
1
NIL

SLIDER
1
144
189
177
segment-separation
segment-separation
0
100
90
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
A model showing the evolution of complex body structures through artificial selection and mutation in genetic code across multiple generations.

The model captures a very simple mechanism for emergent body structures from the repetitive use of three NetLogo commands.  It also shows divergence of DNA in a population under selective forces.  It also shows convergent evolution in body structures, as the user may select an infinite number of breeding pathways to similar body structures.  The model shows the family tree for the parent / offspring relationships.


HOW IT WORKS
------------
Critters are creatures in the model, created using a series of body segments or cells.  A creature can be built of 1 to an infinite number of segments.  Each segment/cell is created using 3 NetLogo commands.  These commands are referred to as a triplet.

These are the commands in a triplet:
  set size a
  set heading b
  set color c

The values for a, b, c are randomly set in the first batch of creatures created.  The number of segments is randomly assigned to the first batch of creatures using a number less than the setting of MAX-INIT-BODY-SEGMENTS.

Creatures may be selected for produce offspring.  When offspring are produced a duplicate copy of the instructions to create the parent are passed on to the offspring. These "genetically inherited instructions" are referred to as the critter's DNA.  If no mutations occur in this process, the offspring will have the exact same body structure.

Genetic variation can be introduced with three types of random mutations:  deletion, insertion, and substitution.  The deletion mutation removes one of the triplets from the DNA of the offspring.  The insertion mutation duplicates one of the triplets in the DNA of the offspring.  The substitution mutation takes one of the triplets and adds a random amount of drift to the three values of a, b, and c, in the triplet.  The drift can be adjusted using the sliders in the model (see below).

As new offspring are produced, the user may select which parents to use and not use for continued generation of new offspring.  As each generation of artificial selections made, the gradual changes seen in the first few generations accumulate, leading to rapid divergence of body structures in the family tree of critters.




HOW TO USE IT
-------------

INIT-NUMBER-CRITTER and MAX-INIT-BODY-SEGMENTS respectively set how many creatures you start with on SETUP and what the largest number of segments these creatures can have.

INITIAL-BODY-VARIABILITY is the chance that each body segment in the starting creatures will be different in some way then the one next to is.

SEGMENT-SEPARATION is the the percentage that the segments are separated compared to the sum of their two radii.

MAKE OFFSPRING is used while the model is running (with GO).  When this button is depressed any critters that are selected will make one offspring.  Selected critters have a rotating target over their heads.  Each time this button is pressed, the GENERATIONS monitor value is incremented by one.  Every critter created when this happens increments the CRITTER CREATED monitor value by one

ADD-OFFSPRING-TO-SELECTED will auto-select all offspring.

KEEP-PARENTS-SELECTED? will keep the parents selected after a MAKE OFFSPRING event.  If turned off, then all the parents will be deselected after a MAKE OFFSPRING event.

If INTERACT-MODE is set to select, the user may toggle the selection of any critter using the mouse in the World & View and clicking on a critter.  If the INTERACT-MODE is set to move, the user may drag and drop any critter in the World & View, by pressing and holding the mouse button while the cursor is over a critter and dragging the location of the cursor to a new spot in the World & View and releasing the mouse button.

FAMILY-TREE? will show the edges between parent and offspring.

CHANCE-DELETION-MUTATION is the % chance that any triplet in the DNA of the offspring is removed.

CHANCE-DUPLICATION-MUTATION is the % chance that any triplet in the DNA of the offspring is duplicated at its current location.

CHANCE-SUBSTITUTION-MUTATION is the % chance that any triplet in the DNA of the offspring is altered.  MAX-ANGLE-DRIFT is the largest amount of possible angle change that can occur in the triplet during a substitution mutation.  MAX-COLOR-DRIFT is the largest amount of possible color change that can occur in the triplet during a substitution mutation.  MAX-SIZE-DRIFT is the largest amount of possible color change that can occur in the triplet during a substitution mutation (drift size is also bounded by threshold values set during SETUP).

MOVE-CRITTERS? Allows the critters to move apart from each other, which is a useful way to have the offspring separate from the parents.  WANDER-SPEED sets how far forward each critter moves with each simulation step and WANDER-ANGLE sets how much random change in heading the critter might have with each simulation step.

SEGEMENT-SEPARATION determines how far apart the segments are from 100% (just touching) to 0% (completely overlapping).


THINGS TO NOTICE
----------------
Artificial selection leads to divergent forms that converge on the more user desired emergent structures.


THINGS TO TRY
-------------
Run the model over 20 or more generations and try to raise unique color or body patterns in different lines of critters.  Try to get the critters to look like different breeds of a species or even different species:  snakes, birds, different mammals, etc


EXTENDING THE MODEL
-------------------
Adding sexual reproduction, treating each triplet as a chromosome would be easy.  Speciation could also be added by treating unequal number of chromosomes at different species.  Since DNA insertion and deletion still might cause nearby species to become recompatible (albeit rarely), this would model the gray boundary between species that remains for a while as species diverge.

Onco-genes could be added that hatching a new sequence of existing triplets.  This might lead to branching structures (e.g. limbs) on the critters.

Kinematics between segments could be added (using code to model linkages from the Infernal Machines model).  With this added, motor functions could be coded to some of the segments.  Different movement behaviors could then be selected for.

GasLab Circular Particles code could be added to allow segments of different critters to collide with other objects (critters or food) in the environment.  In such a way, different survival goals (such as knocking away other critter or gathering food) could be added and emergent survival behavior could modeled.


NETLOGO FEATURES
----------------
This model contains code that shows how to build parent / child relationships between turtles.  This linkage allows the set of turtles that make up the critter to be treated as one unit: both for motion and selection.


RELATED MODELS
--------------
Other EvoLab models


CREDITS AND REFERENCES
----------------------

@#$#@#$#@
default
true
0
Circle -7500403 true true 2 2 295
Circle -1 true false 23 36 108
Circle -1 true false 169 36 108
Circle -16777216 true false 207 41 48
Circle -16777216 true false 45 41 48
Polygon -7500403 true true 150 300 300 165 165 15 135 15 0 165

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

default-selected
true
0
Polygon -7500403 true true 150 0 30 240 150 300 270 240
Polygon -16777216 false false 150 0 30 240 150 300 270 240
Circle -1 true false 40 145 68
Circle -1 true false 190 145 68
Circle -16777216 true false 60 165 30
Circle -16777216 true false 210 165 30
Circle -1 false false 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line-half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

selected
true
0
Circle -1 false false 12 12 277
Line -1 false 150 0 150 30
Line -1 false 0 150 30 150
Line -1 false 150 270 150 300
Line -1 false 270 150 300 150

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@

