patches-own [ disturbance ]
turtles-own [ random-movement? ] ;Whether the fish moves randomly or uses the disturbances

breeds [ prey eaters ]
eaters-own
[ food    ;Total number of prey eaten
  eaten ] ;Number of prey eaten in one turn cycle

globals
[ follow-color  ;Color of the 'following' eater fish
  random-color  ;Color of the randomly moving eater fish
  fish-color ]  ;Color of the prey fish

;;;;;;;;;;;;;;;;;;;;;;;
;;;Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;
to setup
  ca
  set follow-color red - 1
  set random-color orange
  set fish-color   yellow
  setup-water
  setup-fish
  setup-plots
end

to setup-water
  ask patches
  [ set disturbance 0
    set pcolor scale-color blue disturbance -2 2
  ]
end

to setup-fish
  create-custom-eaters 2
  [ set shape  "fish"
    set color  follow-color
    set food   0
    set random-movement? false
    setup-headings
    setxy random-float screen-size-x
          random-float screen-size-y
    set ycor ((random-float (screen-size-y - 2)) - screen-edge-y)
  ]
  ;; Swtich one of the eaters to be the randomly moving one
  set random-movement?-of turtle 0 true
  set color-of turtle 0 random-color

  create-custom-prey fish
  [  set shape "fish"
    set color fish-color
    set random-movement? prey-random?
    setup-headings
    setxy random-float screen-size-x
          random-float screen-size-y
  ]
end

;; Sets the random heading of the fish to one of the 8 cardinal directions
to setup-headings  ;; Turtle Procedure
  if (heading > 337.5 or heading <= 22.5) ;North
  [ set heading 0 ]
  if (heading > 22.5 and heading <= 67.5) ;North-East
  [ set heading 45 ]
  if (heading > 67.5 and heading <= 112.5) ;East
  [ set heading 90 ]
  if (heading > 112.5 and heading <= 157.5) ;South-East
  [ set heading 135 ]
  if (heading > 157.5 and heading <= 202.5) ;South
  [ set heading 180 ]
  if (heading > 202.5 and heading <= 247.5) ;South-West
  [ set heading 225 ]
  if (heading > 247.5 and heading <= 292.5) ;West
  [ set heading 270 ]
  if (heading > 292.5 and heading <= 337.5) ;North-West
  [ set heading 315 ]
end

to setup-plots
  ;; Setup for Total Fish Eaten Plot
  set-current-plot "Total Fish Eaten"
  clear-plot
  auto-plot-on
  set-plot-x-range 0 200
  set-plot-y-range 0 fish

  create-temporary-plot-pen "Follower"
  set-plot-pen-color follow-color

  create-temporary-plot-pen "Random"
  set-plot-pen-color random-color

  ;; Setup for the Follower's Rate of Consumption Plot
  set-current-plot "Rate of Consumption - Follower"
  clear-plot
  auto-plot-on
  set-plot-x-range 0 200
  set-plot-y-range 0 5

  create-temporary-plot-pen "Follower"
  set-plot-pen-color follow-color

  ;; Setup for the Randomly Moving Eater's Rate of Consumption Plot
  set-current-plot "Rate of Consumption - Random"
  clear-plot
  auto-plot-on
  set-plot-x-range 0 200
  set-plot-y-range 0 5

  create-temporary-plot-pen "Random"
  set-plot-pen-color random-color
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Runtime-Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;
to go
  move-fish
  propagate-disturbance
  do-plot
end

to move-fish
  ask eaters
  [ move-eater ]
  ask prey
  [ ifelse (random-movement?)
    [ random-heading ]
    [ find-heading ]
    fd 1
    set disturbance (disturbance - 2)
  ]
end

to move-eater ;;Eater Procedure
  let eating 0
  set eaten 0
  ;; Eater-speed is used to tell the eaters how many times they can move in a cycle
  repeat Eater-speed
  [ ifelse (random-movement?)
    [ random-heading ]
    [ find-heading ]
    fd 1
    set disturbance (disturbance - 2)
    set eating count prey-here
    set eaten (eaten + eating)
    ask prey-here
    [ die ]
  ]
  set food (food + eaten)
end

to random-heading ;; Turtle Procedure
  set heading (heading + (45 * ((random 3) - 1)))
end

to find-heading ;; Turtle Procedure
  let disturb-list []
  ;; Builds a list of the disturbances in the 3 patches in front of the turtle
  if (heading = 0) ;North
  [ set disturb-list list (abs (disturbance-of patch-at -1 1)) (abs (disturbance-of patch-at 0 1))
    set disturb-list lput (abs (disturbance-of patch-at 1 1)) disturb-list
  ]
  if (heading = 45) ;North-East
  [ set disturb-list list (abs (disturbance-of patch-at 0 1)) (abs (disturbance-of patch-at 1 1))
    set disturb-list lput (abs (disturbance-of patch-at 1 0)) disturb-list
  ]
  if (heading = 90) ;East
  [ set disturb-list list (abs (disturbance-of patch-at 1 1)) (abs (disturbance-of patch-at 1 0))
    set disturb-list lput (abs (disturbance-of patch-at 1 -1)) disturb-list
  ]
  if (heading = 135) ;South-East
  [ set disturb-list list (abs (disturbance-of patch-at 1 0)) (abs (disturbance-of patch-at 1 -1))
    set disturb-list lput (abs (disturbance-of patch-at 0 -1)) disturb-list
  ]
  if (heading = 180) ;South
  [ set disturb-list list (abs (disturbance-of patch-at 1 -1)) (abs (disturbance-of patch-at 0 -1))
    set disturb-list lput (abs (disturbance-of patch-at -1 -1)) disturb-list
  ]
  if (heading = 225) ;South-West
  [ set disturb-list list (abs (disturbance-of patch-at 1 0)) (abs (disturbance-of patch-at 1 -1))
    set disturb-list lput (abs (disturbance-of patch-at 0 -1)) disturb-list
  ]
  if (heading = 270) ;West
  [ set disturb-list list (abs (disturbance-of patch-at 0 -1)) (abs (disturbance-of patch-at -1 -1))
    set disturb-list lput (abs (disturbance-of patch-at -1 0)) disturb-list
  ]
  if (heading = 315) ;North-West
  [ set disturb-list list (abs (disturbance-of patch-at -1 0)) (abs (disturbance-of patch-at -1 1))
    set disturb-list lput (abs (disturbance-of patch-at 0 1)) disturb-list
  ]
  ;; Chooses the direction to turn towards based on the largest disturbance in the list
  if ((item 1 disturb-list) != (max disturb-list))
  [ ifelse ((item 0 disturb-list) > (item 2 disturb-list))
    [ lt 45 ]
    [ rt 45 ]
  ]
end

;; Causes the wave motion seen on the patches, alternation between light and dark is for effect only
to propagate-disturbance
  ask patches
  [ set pcolor scale-color blue disturbance -2 2
    set disturbance (-0.99 * disturbance)
  ]
  diffuse disturbance .2
end

to do-plot
  ; Plot Total Fish Eaten
  set-current-plot "Total Fish Eaten"
  create-temporary-plot-pen "Random"
  plot-pen-down
  plot (food-of turtle 0)

  create-temporary-plot-pen "Follower"
  plot-pen-down
  plot (food-of turtle 1)

  ;Plot Follower's Rate of Consumption
  set-current-plot "Rate of Consumption - Follower"
  create-temporary-plot-pen "Follower"
  plot-pen-down
  plot (eaten-of turtle 1)

  ;Plot Randomly Moving Eater's Rate of Consumption
  set-current-plot "Rate of Consumption - Random"
  create-temporary-plot-pen "Random"
  plot-pen-down
  plot (eaten-of turtle 0)
end

; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Copyright 2001 by Uri Wilensky. All rights reserved.
; Permission to use, copy, or modify this software and its documentation for
; educational and research purposes only and without fee is hereby granted,
; provided that this copyright notice and the original authors' names appear
; on all copies and supporting documentation. For any other uses of this
; software, in original or modified form, including but not limited to
; distribution in whole or in part, specific prior permission must be
; obtained from Uri Wilensky. These programs shall not be used, rewritten,
; or adapted as the basis of a commercial software or hardware product
; without first obtaining appropriate licenses from Uri Wilensky. We make no
; representations about the suitability of this software for any purpose. It
; is provided "as is" without express or implied warranty.
@#$#@#$#@
GRAPHICS-WINDOW
251
10
705
485
18
18
12.0
1
10
1
1
1
0

CC-WINDOW
5
577
714
672
Command Center

BUTTON
9
10
64
43
Setup
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
64
10
243
43
fish
fish
1
100
100
1
1
NIL

BUTTON
9
43
64
109
Go
go
T
1
T
OBSERVER
T
NIL

PLOT
9
153
243
304
Total Fish Eaten
Time
Eaten
0.0
200.0
0.0
100.0
true
false

PLOT
9
304
243
433
Rate of Consumption - Follower
Time
Eaten
0.0
200.0
0.0
5.0
true
false

PLOT
9
433
243
563
Rate of Consumption - Random
Time
Eaten
0.0
200.0
0.0
5.0
true
false

SLIDER
64
43
243
76
Eater-speed
Eater-speed
1
5
3
1
1
NIL

MONITOR
126
109
243
158
Random's Food
food-of turtle 0
3
1

MONITOR
9
109
126
158
Follower's Food
food-of turtle 1
3
1

SWITCH
64
76
243
109
prey-random?
prey-random?
1
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
    This model explores the benefit of the following algorithm from the 'fish' model for predatory fish in water.  As fish move around in the environment they create a disturbance in the water that propagates behind them.  The predatory fish then notice these disturbances and turn towards them in order to seek their prey.  This system results in predatory fish that can seek out their prey by following the disturbance waves caused by the movement of their prey.


HOW TO USE IT
-------------
    The FISH slider determines the number of fish that are created during setup.
    The PREY-RANDOM? switch can be changed at anytime, and changes whether the fish use the 'following' movement algorithm where they turn to follow disturbances or instead turn randomly as they move, ignoring the disturbances in the water entirely.
    EATER-SPEED sets the speed of the pretaory fish as they move through the water. (1.0 is the speed that the prey fish move)
    SETUP builds the initial world with the chosen number of fish, as well as two additional predatory fish, one red one that uses the 'following' algorithm and one orange one that uses the random movement algorithm.
    GO starts the simulation having the fish begin to move about the environment.

    The background of the simulation window shows the disturbances in the water caused by the movement of the fish.  Blue is still, relatively undisturbed water and the darker and lighter patches are areas of high disturbance, towards which the fish will tend to turn.  The alternation between light and dark is for effect only and does not affect the fishes' movement.


THINGS TO NOTICE
----------------
    As the fish move, you'll notice that though they will often begin following one another, frequently, they won't continue following forever and thus they won't school up and begin moving as one large group.  The reason for this is that they simply turn towards the area of greatest disturbance ahead of them.  And although, much of the time this will be the wake of another fish, sometimes they will encounter their own wake or be passed by another fish moving in another direction and lose the path they were following.  The fish have no direct knowledge of the existence of the other fish, only the disturbances they caused.  All of this applies for the eater fish as well.
    The plots and the monitors show the success of the two types of preatory fish as the simulation runs.
    With the eater-speed set to 1.0 the follower fish performs about the same as the randomly moving fish.  The higher the eater-speed the better the follower fish performs relative to the randomly moving fish.  The randomly moving fish tends to perform better with more prey fish, while the follower's performance is not nearly as dependant on the concentration of fish.


THINGS TO TRY
-------------
    See how the following affect the sucess of the eaters:
    -- Run the model with both few prey and many prey.
    -- Run the model with the prey-random? switch on and off.
    -- Change the Eater-speed

    This model is well suited to use the BehaviorSpace tool.  Use it to run many trials of the model with different settings and compare the success of the different types of eaters in each run.


EXTENDING THE MODEL
-------------------
    Often the motivation for the movement of fish towards disturbances is to try and catch things to eat, such as bugs that land on the surface of the water or smaller fish swimming about, instead of simply to follow fellow fish.  This motivation is what fishermen exploit in order to lure fish to their hook.

 -- Add food that causes disturbances that the prey can go towards, setting up a food-chain of sorts.

 -- Add an evoluionary aspect, allowing the fish (both prey adn eaters) to reproduce and seeing if the 'following' fish population overtakes the randomly moving fish population.

 -- Add a genetic algorithm for the likelyhood of using the 'following' algorithm versus the randomly moving algorithm.  Starting with primarliy randomly moving fish the population might begin to shift towards follower fish as they tend to be more sucessful.

NETLOGO FEATURES
----------------
This model makes use of the breeds feature of NetLogo.  The prey and the eaters are two different breeds of turtles in this model, and can thus be given commands seperate from one another.

The following algorithm in the 'find-heading' procedure makes use of a list of patch values from the three patches directly in front of it to decide the direction in which to turn.  This is a different way of modeling environmentally based navigation than many other models use, and may be more or less appropriate for other models that involve turtle navigation.


RELATED MODELS
--------------
    The original 'Fish' model explores a more simple space in which you can watch the following algorithm for a pool of fish, without predators.


CREDITS AND REFERENCES
----------------------
(model begun by Brent Collins)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
true
0
Polygon -7566196 true true 150 31 120 60 104 133 133 224 92 255 211 255 166 224 195 132 180 59

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
