globals [ curr-color-sep ]   ;; spread of the colors in the kaleidoscope

turtles-own [new?]

;;SET-UP THE GRAPHICS WINDOW FOR CREATING DESIGNS
to setup
  ca
  cct nturtles
    [pen-down
     set new? false
    ]
  set curr-color-sep color-sep
end

;;GO FUNCTION
;;-----------
;;THIS IS THE MAIN FUNCTION WHICH CREATES THE DESIGN PATTERNS.
;;WHEN THE CLEAR PATCH MODE HAS BEEN INITIATED, YOU CAN INVOKE FOLLOW TURTLE.
to go        ;; turtle procedure
  if (who + 1 > max-turtles)
    [die]
  if (who = 0)
    [if ( color-sep != 0 )
       [set curr-color-sep color-sep]
    ]
  ifelse (new?)
    [set color (who / curr-color-sep)
     if (follow-turtle?)
       [if (who = (nturtles + 1))
         [set color white
          pen-down
         ]
       ]
     if (draw-triangle?)
       [draw-polygon 3 false expander]
     if (draw-left-square?)
       [draw-polygon 4 true expander]
     if (draw-pentagon?)
       [draw-polygon 5 false 35]
     if (draw-hexagon?)
       [draw-polygon 6 false 35]
     if (draw-octagon?)
       [draw-polygon 8 true 30]
     if (draw-nine-gon?)
       [draw-polygon 9 true 35]
     if (draw-left-circle?)
       [draw-polygon 45 true 6]
     if (draw-right-circle?)
       [draw-polygon 45 false 6]
     die
    ]
    [fd step
     rt degrees
     hatch 1
       [set new? true]
    ]
end

;;THIS FUNCTION DOES THE ACTUAL DRAWING OF EACH POLYGON, THE SHAPE BEING SPECIFIED BY ITS INPUTS
to draw-polygon [corners turn-left? step-size]        ;; turtle procedure
  locals [angle]
  set angle (360 / corners)
  repeat corners
    [fd step-size
     ifelse turn-left?
       [lt angle]
       [rt angle]
    ]
end

;;THIS FUNCTION CLEARS THE PATCHES, LIFTS THE PEN AND KILLS ALL TURTLES GREATER THAN
;;INITIAL NUMBER
to clear-patches-lift-pen
  ask turtles
    [pen-up
     if ( who >= nturtles)
       [die]
    ]
  clear-patches
end

;;THIS FUNCTION PUTS THE PEN DOWN AND KILLS TURTLES GREATER THAN INITIAL NUMBER
to restore        ;; turtle procedure
  if (who >= nturtles)
    [die]
  pen-down
end

; This model was originally  created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.

; Copyright 1998 by Uri Wilensky. All rights reserved.

; This model was converted to NetLogo (and was changed in the process to
; be an alternate version of the model) as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.
@#$#@#$#@
GRAPHICS-WINDOW
316
13
728
446
100
100
2.0
0
10
1
1
1
0

CC-WINDOW
5
519
737
614
Command Center

SLIDER
23
104
129
137
nturtles
nturtles
1
40
6
1
1
NIL

SLIDER
23
210
128
243
step
step
0.0
1.0
0.0
0.1
1
NIL

SLIDER
22
252
128
285
degrees
degrees
0.0
1.0
0.0
0.1
1
NIL

SWITCH
16
384
140
417
follow-turtle?
follow-turtle?
1
1
-1000

SWITCH
159
182
302
215
draw-octagon?
draw-octagon?
1
1
-1000

SWITCH
159
96
302
129
draw-pentagon?
draw-pentagon?
1
1
-1000

SWITCH
159
10
302
43
draw-triangle?
draw-triangle?
1
1
-1000

SWITCH
159
268
302
301
draw-left-circle?
draw-left-circle?
1
1
-1000

SWITCH
159
311
302
344
draw-right-circle?
draw-right-circle?
1
1
-1000

SWITCH
159
139
302
172
draw-hexagon?
draw-hexagon?
1
1
-1000

SWITCH
159
225
302
258
draw-nine-gon?
draw-nine-gon?
1
1
-1000

SWITCH
159
53
302
86
draw-left-square?
draw-left-square?
1
1
-1000

SLIDER
16
340
140
373
expander
expander
0.0
90.0
40.0
1.0
1
NIL

BUTTON
23
11
129
44
setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
23
56
129
89
go
go
T
1
T
TURTLE
T
NIL

MONITOR
23
150
129
199
turtles
count turtles
1
1

BUTTON
165
409
295
442
clear-patches-lift-pen
clear-patches-lift-pen
NIL
1
T
OBSERVER
T
NIL

BUTTON
165
364
295
397
restore
restore
NIL
1
T
TURTLE
T
NIL

SLIDER
16
472
140
505
color-sep
color-sep
-10.0
10.0
2.0
1.0
1
NIL

SLIDER
16
428
140
461
max-turtles
max-turtles
0
2048
1024
256
1
NIL

TEXTBOX
8
302
157
331
expander slider changes size of square and triangle

@#$#@#$#@
WHAT IS IT?
-----------
This project is an extension of a previous NetLogo project called "Kaleidoscope".  The current version has added six new shapes to the model.  The name of this model is called Geometron-Bottom-Up because it allows the user the ability to create his/her own pattern using the provided shape-functions.


HOW TO USE IT
-------------
Set NTURTLES to a desired value between 0 and 40 (default value is 6).  NTURTLES determines how many initial turtles you want to start out with. These initial turtles will spawn other turtles.  When you have set NTURTLES, press the SETUP button to set up the model.

Now click on the GO button.  GO is a forever button that operates on NTURTLES in conjunction with the eight switches located to the right of the interface window. Each one of these switches corresponds to a shape-function and is initially set to OFF.  To activate a shape, turn any of the eight switches "on" to 1.  This has the effect of telling each of the NTURTLES to hatch a turtle that will draw the shape designated by the switch.  When switched "on":
DRAW-TRIANGLE? will draw a triangle in the right direction,
DRAW-LEFT-SQUARE? will draw a square in the left direction,
DRAW-PENTAGON? will draw a pentagon in the right direction,
DRAW-HEXAGON? will draw a hexagon in the right direction.
DRAW-OCTAGON? will draw an octagon in the right direction,
DRAW-NINE-GON? will draw a nonagon in the right direction,
DRAW-RIGHT-CIRCLE? will draw a circle in the right direction,
DRAW-LEFT-CIRCLE? will draw a circle in the left direction,
More than one switch can be activated.  In fact, different combinations of activated switches are responsible for the creation of different geometrical patterns and kaleidoscopes.  Whenever you want to stop the entire pattern, click on GO again.  However, before stopping, each turtle will finish making its current shapes.

Two sliders, called STEP and DEGREES, are designed to operate on the original NTURTLES.  STEP is numbered between 0 and 1 (default value is 0) and can be incremented by 0.1 (i.e., one-tenth).  STEP has the effect of moving the original NTURTLES forward by the number indicated on the STEP slider.  DEGREES is also numbered between 0 and 1 (default value is 0) and can be incremented by 0.1 (i.e., one-tenth). DEGREES has the effect of turning the original NTURTLES to the right by the number of degrees indicated on the DEGREES slider.

The slider EXPANDER is designed specifically for the square and triangle shapes.  EXPANDER determines how large each leg of the square and triangle will be (default value is 25).

Other added features of the model are the CLEAR-PATCHES-LIFT-PEN and RESTORE buttons.  CLEAR-PATCHES-LIFT-PEN does exactly what it says: it clears the screen of all patches, lifts the pen on all turtles and kills all turtles greater than NTURTLES.  RESTORE kills all turtles greater than NTURTLES and tells all turtles to put their pen down. Killing all turtles greater than NTURTLES restores the original number of turtles on the screen.

A useful tool that can be used in conjunction with the CLEAR-PATCHES-LIFT-PEN and RESTORE buttons is a switch called FOLLOW-TURTLE (default value is "off" or 0). When you press CLEAR-PATCHES-LIFT-PEN, you see the skeleton of the designed pattern since the turtles are moving without their pens down. But when the switch is turned on, one hatched turtle will put its pen down.  However, since turtles are constantly being born and dying it may take time for this particular turtle to appear.

You also have one monitor called COUNT-TURTLES, in the middle of the Interface Window, which will display the current number of turtles.


THINGS TO NOTICE
----------------
An important thing to notice here is the number given in COUNT-TURTLES. Right away, it becomes much larger than NTURTLES, but it quickly settles on some big number. Take a look at the Procedures Window. Initially, upon setup, there are NTURTLES. Once one of the pattern buttons is pressed, each of these turtles repeatedly hatches a new turtle and turns by a specified degree. The newly hatched turtles begin to draw different patterns, self-destructing upon completion. As the original turtles execute their commands much quicker than the hatched turtles, they produce many turtles during one loop of a pattern; eventually, though, turtles start to die off. At this point, the number of turtles who are born is roughly equal to the number who die at any given step.


THINGS TO TRY
-------------
Try changing the code in the Procedures Window. Increase the size of the shapes drawn by each of the turtles, or try changing the size of the angle each of the turtles turns through in the shape-functions.  What different shapes can you think of for the turtles to draw?  Try to predict what overall shape will emerge.

Try different numbers of turtles to help you understand the geometry of the model.

Try clicking on the CLEAR-PATCHES-LIFT-PEN and RESTORE buttons while GO is running.


EXTENDING THE MODEL
-------------------
Find some way to introduce randomness into the model.  It could apply to the STEP and DEGREES sliders.

Try drawing figures that are not closed polygons.


NETLOGO FEATURES
-----------------
Look at the function DRAW-POLYGON.  It very nicely demonstrates two NetLogo features.  The first is creating functions that take inputs, in this case TURN-LEFT? and STEP-SIZE, this is accomplished by enclosing them in brackets right after the "to PROCEDURE" code.  The other feature to notice is the use of locals, which declares ANGLE and allows it to be used in the function without specifying it as a global variable (actually in this case, specifying it as a global variable would change the behavior of the model.  Can you guess why?  Hint: DRAW-POLYGONS is a turtle procedure).


SLIDER DESCRIPTIONS
-------------------

NTURTLES
--------
Numbered from 1 to 40 and increments by 1.  This slider is used for creating the initial number of turtles.

STEP
----
Numbered from 0 to 1 and increments by .1 (i.e., one-tenth).  After every iteration of GO, the original turtles created by NTURTLES will move
forward by the number indicated by STEP.

DEGREES
-------
Numbered from 0 to 1 and increments by .1 (i.e., one-tenth).  After every iteration of GO, the original turtles created by NTURTLES will turn right by the number of degrees indicated by DEGREES.

EXPANDER
--------
Numbered from 0 to 90 and increments by 1.  Has the effect of increasing the size of the square drawn by DRAW-LEFT-SQUARE and the size of the
triangle drawn by DRAW-TRIANGLE.

MAX-TURTLES
-----------
Numbered from 0 to 2048 and increments by 256.  After every iteration of GO, if there are more turtles than indicated by MAX-TURTLES, the excess turtles will be removed.

COLOR-SEP
---------
Numbered from -10 to 10 and increments by 1.  Has the effect of increasing the distance turtles travel before changing colors.

IMPORTANT NOTES
************************************************************
    Notice that the number of times the draw polygon function repeats multiplied by the number of degrees the turtle turns either to the right or left is equal to 360 degrees.  This means that each of the turtles is creating a polygon.
************************************************************
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
