;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 1. Declaring globals, patches and breeds        ;;
;;    a. globals                                   ;;
;;    b. breeds                                    ;;
;;    c. patches-own                               ;;
;;    d. students-own                              ;;
;; 2. Startup                                      ;;
;; 3. Monitors                                     ;;
;;    a. place                                     ;;
;;    b. right-direction                           ;;
;;    c. left-direction                            ;;
;;    d. mouse-coords                              ;;
;; 4. Buttons                                      ;;
;;    a. setup                                     ;;
;;    b. go                                        ;;
;;    c. enter-equations                           ;;
;;    d. clear-equations                           ;;
;;    e. linear-regression                         ;;
;;    f. clear-last-eq                             ;;
;;    g. enter-points                              ;;
;;    h. clear-points                              ;;
;; 5. Check and manipulate phrase for evaluation   ;;
;;    a. check-phrase-for-correct-decimals         ;;
;;    b. check-phrase-for-correct-use-of-functions ;;
;;    c. check-for-unidentified-letters            ;;
;;       check-phrase-for-correct-point-form
;;    d. check-phrase-for-inconsistencies          ;;
;;    e. compile-list-into-list                    ;;
;;    f. translate-phrase-into-list                ;;
;; 6. Other non-hubnet functions                   ;;
;;    a. evaluate-function                         ;;
;;    b. draw-axes                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 1. Declaring globals, patches and breeds  ;;
;;   a. globals                              ;;
;;   b. breeds                               ;;
;;   c. patches-own                          ;;
;;   d. students-own                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals
[
         init-phrase        ;the text of the equation, exactly the way the user entered it
         phrase-prefix      ;a string of the left side of the equation
         phrase-ok?         ;if the init-phrase is syntacticly correct
         phrase             ;a list with each element of the  right side
                            ;of the equation init-phrase, in the same order it was entered
         answer             ;what the equation equals with each x

         numbers            ;a list of numbers 0 through 9
         vars               ;a list of variables including x, e, and p (for pi)
         functions          ;a list of letters representing functions
                            ;s=sin, c=cos, t=tan, l=ln, q=sqrt, o=log a=abs
         operators          ;a list of operators including + - * / ^
         numbers-and-vars   ;a list including numbers and variables
         parenth            ;a list of ( and )
         x                  ;the variable x in the y=f(x) equation

         op-list            ;a list of the operators, in the order they appear in the equation
         num-list           ;a list of the numbers, in the order they appear in the equation
         ans-list           ;a list to numbers and operators, showing the equation in reverse polish notation
         inequality?        ;whether the equation represents an inequality or not

         x-range-bigger-than-y?  ;if the x range is bigger than y, for scaling the graph when it is 1 to 1
         xfactor            ;if the scale is 1 to 1, this is the number of patches forming the x axis
         yfactor            ;if the scale is 1 to 1, this is the number of patches forming the y axis

         graph-on?          ;if the graph button is currently running

         rule               ;the text of a verbal rule, that the user can enter
         equation-list      ;a list of equations that the students entered to the teacher, from their client
         equation-list-index     ;the index of which item in the equation-list is visible to the teacher
         part-of-equation-list   ;the equation that is visible to the teacher
         show-equation-list?     ;show an item from equation-list if at least one student has entered an equation
         clear-last-eq?      ;whether the function clear-last-eq is running
         clear-last-point?   ;whether the function clear-last-point is running

         shape-list         ;a list of the combinations of turtle shapes and colors
         x-increment        ;the amount x increases, to have a new x in each iteration of plotting the equation

         color-index        ;the index of which color to plot an equation in, so the colors can rotate
         label-place        ;the y value of the patch of where to label the equation

         old-x-max          ;what the previous value of x-maximum slider was
         old-x-min          ;what the previous value of x-minimum slider was
         old-y-max          ;what the previous value of y-maximum slider was
         old-y-min          ;what the previous value of y-minimum slider was
         old-1-to-1         ;what the previous value of one-to-one? switch was
         old-labels         ;what the previous value of labels switch was
         old-grid

         x-axis             ;x value of patches representing x-axis
         y-axis             ;y value of patches representing y-axis

         setup?             ;if you want to resize the screen, and redraw current functions
         temp-equation-list ;a temporary list of equations
         graphed-equation-list      ;list of equations, currently graphed
         graphed-point-string       ;list of points, currently graphed
         lin-reg?           ;if you want to graph a linear regression of the students
         lin-reg-eq         ;a string of the linear regression equation
         r-square           ;r^2, from linear regression
         m                  ;slope of linear regression
         rb                 ;y-intercept of linear regression

         in-sex             ;x length of graph, based on screen-edge-x
         in-sey             ;y length of graph, based on screen-edge-y

         aa                 ;represents the x coordinate when plotting a point
         bb                 ;represents the y coordinate when plotting a point

         location-of-calculator-data
         save-responses-folder
         save-worlds-folder
         time-of-last-save
]

breeds
[
         teacher            ;the turtle that graphs equations on the teacher's computer
         point              ;the turtle that graphs points
]

patches-own
[
         border?            ;whether or not the patch is located in the border of the graph
         temp-x             ;the x value that the patch represents on the graph
         temp-y             ;the y value that the patch represents on the graph
         object             ;is "equation" if their is a label or temp-label with an equation in it
         temp-label         ;holds the description of an equation if it is of object "equation"
]

point-own
[
         my-x               ;the x value of the student's position on the graph
         my-y               ;the y value of the student's position on the graph
]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 2. Startup                            ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to startup

  ;initialize variables
  set numbers ["0" "1" "2" "3" "4" "5" "6" "7" "8" "9"]
  set numbers-and-vars ["0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "x" "e" "p"]
  set vars ["x" "e" "p"] ;p=pi
  set functions [ "s" "c" "t" "l" "q" "o" "a"] ;s=sin, c=cos, t=tan, l=ln, q=sqrt, o=log a=abs
  set operators ["+" "-" "*" "/" "^"]
  set parenth ["(" ")"]
  set init-phrase ""
  set rule ""
  set phrase ""
  set phrase-ok? true
  set answer ""
  set equation-list []
  set graphed-equation-list []
  set setup? false
  set clear-last-eq? false
  set clear-last-point? false
  set equation-list-index -1
  set show-equation-list? false
  set part-of-equation-list ""
  set color-index -1
  set in-sex (screen-edge-x * 0.85)
  set in-sey (screen-edge-y * 0.85)
  set label-place (-1 * (in-sex + 6))
  set graph-on? false
  set old-x-max x-maximum
  set old-x-min x-minimum
  set old-y-max y-maximum
  set old-y-min y-minimum
  set old-grid grid?
  set old-1-to-1 one-to-one?
  set inequality? false
  set old-labels labels?
  set lin-reg? false
  set rb ""
  set m ""
  set r-square ""
  set shape-list [ "hr" "so" "cy" "dn" "ag" "rb" "tu" "hp"
                   "sr" "co" "dy" "an" "rg" "tb" "hu" "sp"
                   "cr" "do" "ay" "rn" "tg" "hb" "su" "cp"
                   "dr" "ao" "ry" "tn" "hg" "sb" "cu" "dp"
                   "ar" "ro" "ty" "hn" "sg" "cb" "du" "ap"
                   "rr" "to" "hy" "sn" "cg" "db" "au" "rp"
                   "tr" "ho" "sy" "cn" "dg" "ab" "ru" "tp" ]
                  ;the first letter represents the shape
                  ;"h" heart, "s" square, "c" circle, "d" diamond, "a" star, "r" rhombus, "t" triangle
                  ;the second letter represents the color
                  ;"p" pink, "o" orange, "g" green, "b" blue, "u" purple, "r" red, "y" yellow, "n" brown



  create-teacher 1
  draw-axes

end

to change-folder
  set save-worlds-folder user-choose-directory
end

to load-file-from
locals [temp]
  set temp user-choose-file
  if is-string? temp
  [ set location-of-calculator-data temp ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 3. Monitors                              ;;
;;    a. place                              ;;
;;    b. right-direction                    ;;
;;    c. left-direction                     ;;
;;    d. mouse-coords                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;the students send the teacher equations from their clients
;this shows the number of the equation, out of the list of the equation that is visible
to-report find-place
  ifelse (equation-list-index = -1)
  [
    report ""
  ]
  [ report ( word  (equation-list-index + 1) "/" length equation-list  ) ]

end

;this makes an equation visible, that is later in the teacher's list of equations
to right-direction

  if (equation-list-index < (length equation-list - 1))
  [
    set equation-list-index equation-list-index + 1
    set part-of-equation-list item equation-list-index equation-list
  ]
end

;this makes an equation visible, that is earlier in the teacher's list of equations
to left-direction

  if (equation-list-index > 0)
  [
    set equation-list-index equation-list-index - 1
    set part-of-equation-list item equation-list-index equation-list
  ]
end

;this shows the coordinates of the mouse, with respect to the range and scale of the graph
to-report mouse-coords
  locals
  [
    the-x   ;the x position on the graph
    the-y   ;the y position on the graph
  ]

  ifelse one-to-one?
  [
    ifelse x-range-bigger-than-y?
    [ set the-x ((mouse-xcor * ((x-maximum - x-minimum) * 0.5 * (1 / in-sex))) + ((x-maximum + x-minimum) / 2))
      set the-y ((mouse-ycor * ((y-maximum - y-minimum) * 0.5 * (1 / yfactor))) + ((y-maximum + y-minimum) / 2)) ]
    [ set the-x ((mouse-xcor * ((x-maximum - x-minimum) * 0.5 * (1 / xfactor))) + ((x-maximum + x-minimum) / 2))
      set the-y ((mouse-ycor * ((y-maximum - y-minimum) * 0.5 * (1 / in-sey))) + ((y-maximum + y-minimum) / 2)) ]
  ]

  [ set the-x ((mouse-xcor * ((x-maximum - x-minimum) * 0.5 * (1 / in-sex))) + ((x-maximum + x-minimum) / 2))
    set the-y ((mouse-ycor * ((y-maximum - y-minimum) * 0.5 * (1 / in-sey))) + ((y-maximum + y-minimum) / 2)) ]

    report "(" + (precision the-x 2) + "," + ( precision the-y 2 ) + ")"
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 4. Buttons                            ;;
;;    a. setup                           ;;
;;    b. go                              ;;
;;    c. enter-equations                 ;;
;;    d. clear-equations                 ;;
;;    e; linear-regression               ;;
;;    f. enter-points                    ;;
;;    g. clear-points                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;this sets up the graph so that it has the specified range, scale, and label visibility
to setup
  locals
  [
    the-x  ;the x position, in the form of patch x coordinate
    the-y  ;the y position, in the form of patch y coordinate
  ]

  set setup? true
  ;if the x or y ranges or scale have changed, redraw the graph
  if ((old-x-max != x-maximum) or (old-x-min != x-minimum) or (old-y-max != y-maximum)
     or (old-y-min != y-minimum) or (old-1-to-1 != one-to-one?) or clear-last-eq? or (old-grid != grid?) or clear-last-point?)
  [

    if (x-minimum < x-maximum) and (y-minimum < y-maximum) [set phrase-ok? true]

    ;draw-axes
    if phrase-ok?
    [
      ;re-draw equations
      clear-equations
      set temp-equation-list graphed-equation-list
      while [length temp-equation-list > 0]
      [
        enter-equation
        set temp-equation-list but-first temp-equation-list
      ]
      ;re-draw points
      clear-points
      enter-points
    ]
  ]

  ;if the labels switch has changed, then do what the change specifies
  if (labels? != old-labels)
  [
    set old-labels labels?
    ifelse labels?
    [
      ask patches with [object = "equation"]
      [
        set plabel temp-label
      ]
    ]
    [
      ask patches with [object = "equation"]
      [
        set plabel no-label
      ]
    ]
  ]

  set setup? false

end

;asks the user to input an equation
;checks to make sure the equation is syntactically correct
;asks turtle to plot the equation, within the bounds of the graph
;if the y value is labeled undefined at a specific x, the turtle doesn't graph it
to enter-equation
locals [index done? next-x next-y prev-x prev-y
  stamp? skip?  prev-in? x-value y-value choice which-color? temp-phrase
  rounded-prev-x rounded-prev-y color-list old-rounded-prev-x]

  set graph-on? true
  set phrase ""
  set answer ""
  set rule ""
  set phrase-ok? true
  set color-list [14 24 44 64 94 114 134]
  set old-rounded-prev-x -100

  ifelse lin-reg?
  [
    set lin-reg? false
    set phrase lin-reg-eq
    set labels? false
    set color-index 6
  ]
  [
    ifelse setup?
    [ set phrase first temp-equation-list ]
    [
      ;asks the user to input an equation
      ifelse (equation-list-index = -1)
      [ set phrase user-input "Enter equation: ( for example: y=2x+3 )" ]
      [ ifelse "New Equation" = (user-choice "Which equation do you want to graph?"
          ["New Equation" "Student's Equation"] )
        [ set phrase user-input "Enter equation: ( for example: y=2x+3 )" ]
        [ set phrase item equation-list-index equation-list ]
      ]
    ]
  ]


 ;checks to make sure the equation is syntactically correct
  set init-phrase phrase
  check-phrase-for-inconsistencies
  if phrase-ok? [ check-for-unidentified-letters]
  if phrase-ok? [ check-phrase-for-correct-decimals ]
  if phrase-ok? [ check-phrase-for-correct-use-of-functions ]
  if phrase-ok?
  [
    no-display

    ;for use when redrawing equation, using "clear-last-eq"
    ;since it doesn't error check the equation, because it has been checked before
    ;it doesn't know if it is an inequality

    ;add this equation to the equation list, if setup? isn't on
    if not setup?
    [
      set temp-phrase word phrase-prefix phrase
    ;  set temp-phrase word "y=" phrase
      set graphed-equation-list lput temp-phrase graphed-equation-list
    ]

    ;decide which color to graph the equation in
    ;decide where to label the equation
    ifelse (color-index < 6)
    [ set color-index color-index + 1  ]
    [ set color-index 0 ]
    set which-color? item color-index color-list
    set label-place label-place + 9

    ask teacher
    [
      ifelse ((inequality?) and (length phrase-prefix = 2))
 ;     [set color black]
      [  pu ]
      [ set color which-color? ]
    ]
    ask patch-at (in-sex - 2) label-place
    [
      set temp-label init-phrase
      set object "equation"
      set plabel-color which-color?
    ]


    ifelse (old-labels)
    [ ask patch-at (in-sex - 2) label-place [set plabel init-phrase set temp-label init-phrase] ]
    [ ask patch-at (in-sex - 2) label-place [set temp-label init-phrase] ]

    ;translate the phrase into a list containing each part of the equation, in the order it was entered
    ;this changes it from "1.0+2*sqrt4" to ["1.0" "+" "2" "*" "sqrt" "4"]
    translate-phrase-into-list
    set x "x"

    ;this puts the list containing the equation, into reverse polish notation
    ;it also lets the strings of numbers, become numbers (not strings)
    ;so  ["1" "+" "2.3"] becomes [1 2.3 "+"]
    ;and ["1" "+" "sqrt" "4"] becomes [1 4 "sqrt" "+"]
    compile-list-into-list

    set index old-x-min
    set next-x -200
    set next-y 0
    set skip? true
    set stamp? false
    set prev-in? false
    ask teacher [pu]

    ;evaluate the equation at every x that is an x-increment between x-minimum and x-maximum
    while [(precision index 2) <= old-x-max ]
    [
      set x precision index 10
      evaluate-function
      ifelse(answer != "undefined")
      [

        ;use the range given from old-values, because you don't want to graph an equation with one range,
        ;change the slider, and graph a new equation in a new range, while it appears on the old range.
        ask teacher
        [
          set prev-x next-x
          set prev-y next-y
          set old-rounded-prev-x rounded-prev-x
          set rounded-prev-x round(prev-x)
          set rounded-prev-y round(prev-y)

          if ((inequality?) and (old-rounded-prev-x != rounded-prev-x))
          [
            ifelse(item 1 phrase-prefix = "<")
            [  ;move from prev-x prev-y down to base of screen
              ask patches with [((pxcor = rounded-prev-x) and (pycor < rounded-prev-y))]
              [
                ifelse (pcolor = black)
                [ set pcolor (which-color? + 2) ]
                [ set pcolor (pcolor + 1) ]
              ]
            ]
            [
              ;move from prev-x prev-y up to base of screen
              ask patches with [((pxcor = rounded-prev-x) and (pycor > rounded-prev-y) )]
              [
                ifelse (pcolor = black)
                [ set pcolor (which-color? + 2) ]
                [ set pcolor (pcolor + 1) ]
              ]
            ]
          ]

          if is-string? answer [set answer runresult answer]

          ifelse one-to-one?
          [
            ifelse x-range-bigger-than-y?
            [ set next-x ((x - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / in-sex)))
              set next-y  ((answer - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / yfactor)))
            ]
            [ set next-x  ((x - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / xfactor)))
              set next-y  ((answer - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / in-sey)))
            ]
          ]
          [
            set next-x ((x - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / in-sex)))
            set next-y ((answer - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / in-sey)))
          ]

          ifelse ((abs(next-x) <= in-sex + 1 ) and (abs(next-y) <= in-sey + 1 ))
          [
            ifelse prev-in?

            ;if prev-in? true, in-range to in-range
            [

              if ((prev-x != next-x) or (prev-y != next-y))
              [
                set heading towardsxy-nowrap next-x next-y
                fd distancexy-nowrap next-x next-y
              ]
            ];end if of ifelse

            ;if prev-in? false, out-of-range to in-range
            [
              if not skip?
              [
                ;setxy at intersection of line prev-xy to next-xy, at border
                ifelse( ((prev-y < 0) and (prev-x < abs(prev-y))) or
                       ((prev-y > 0) and (prev-x < abs(prev-y))) )

                ;use x values
                [
                  set x-value (abs(prev-x - next-x) * abs(abs(next-y) - in-sey + 1) / abs(abs(prev-y) - abs(next-y)))
                  ifelse (prev-x >= next-x)
                  [
                    ifelse (prev-y > 0)
                    [ ask teacher [setxy (next-x + x-value) (in-sey + 1)]]
                    [ ask teacher [setxy (next-x + x-value) (-1 * (in-sey + 1)) ]]
                  ]
                  [
                    ifelse (prev-y > 0)
                    [ ask teacher [setxy (next-x - x-value) (in-sey + 1) ]]
                    [ ask teacher [setxy (next-x - x-value) (-1 * (in-sey + 1)) ]]
                  ]
                ]

                ;use y values
                [
                  set y-value (abs(prev-y - next-y) * abs(abs(next-x) - in-sex + 1) / abs(abs(prev-x) - abs(next-x)))
                  ifelse (prev-y >= next-y)
                  [
                    ifelse (prev-x > 0)
                    [ ask teacher [setxy (in-sex + 1) (next-y + y-value) ]]
                    [ ask teacher [setxy (-1 * (in-sex + 1)) (next-y + y-value) ]]
                  ]
                  [
                    ifelse (prev-x > 0)
                    [ ask teacher [setxy (in-sex + 1) (next-y - y-value) ]]
                    [ ask teacher [setxy (-1 * (in-sex + 1)) (next-y - y-value) ]]
                  ]
                ]

                ;pd, from border to next-xy

                ifelse ((inequality?) and (length phrase-prefix = 2))
                [ ask teacher [pu]]
                [ ask teacher [pd]]

                set heading towardsxy-nowrap next-x next-y
                fd distancexy-nowrap next-x next-y

              ];end else of ifelse
            ] ;end of if
            ifelse ((inequality?) and (length phrase-prefix = 2))
            [ ask teacher [pu]]
            [ ask teacher [pd]
              set pcolor which-color? ]

            set prev-in? true
          ] ;end if of ifelse
          [
            ifelse prev-in?
            ;if prev-in? true, in-range to out-of-range
            [
              ;pd, from prev-xy until border
              set heading towardsxy-nowrap next-x next-y
              ifelse( ((next-y < 0) and (next-x < abs(next-y))) or
                   ((next-y > 0) and (next-x < abs(next-y))) )

              ;use x values
              [ fd ((distancexy-nowrap next-x next-y) * abs((in-sey + 1) - abs(prev-y)) / abs(abs(next-y) - abs(prev-y))) ]

              ;use y values
              [ fd ((distancexy-nowrap next-x next-y) * abs((in-sex + 1) - abs(prev-x)) / abs(abs(next-x) - abs(prev-x))) ]

              ask teacher [pu]
              set skip? true
            ]
            ;if prev-in? false, out-of-range to out-of-range
            [ set skip? true ]

              set prev-in? false
            ];end else of ifelse

            if skip?
            [
              setxy next-x next-y
              set skip? false
            ]

          ]; end ask
        ];end if
        [
          ask teacher [pu]
          set skip? true
        ]
        set index index + x-increment
      ] ;end while

      if inequality?
      [
        set prev-x next-x
        set prev-y next-y
        set old-rounded-prev-x rounded-prev-x
        set rounded-prev-x round(prev-x)
        set rounded-prev-y round(prev-y) ;;

        if ((inequality?) and (old-rounded-prev-x != rounded-prev-x))
        [
          ifelse(item 1 phrase-prefix = "<")
          [  ;move from prev-x prev-y down to base of screen
            ask patches with [((pxcor = rounded-prev-x) and (pycor < rounded-prev-y))]
            [
              ifelse (pcolor = black)
             ;[ set pcolor (which-color? - 2)]
             ; [ set pcolor (round ((which-color? + pcolor) / 20)) * 10 + 6 ]
                [ set pcolor (which-color? + 2) ]
                [ set pcolor (pcolor + 1) ]
            ]
          ]
          [
            ;move from prev-x prev-y up to base of screen
            ask patches with [((pxcor = rounded-prev-x) and (pycor > rounded-prev-y))]
             [
                 ifelse (pcolor = black)
                 ;[ set pcolor (which-color? - 2)]
              ;   [ set pcolor (round ((which-color? + pcolor) / 20)) * 10 + 6 ];
                [ set pcolor (which-color? + 2) ]
                [ set pcolor (pcolor + 1) ]
             ]
          ]
       ]
     ]


    ] ;end if

  ;redraw inside edges of border
  ask patches [if border? [ set pcolor 2]]

  ask teacher [hide-turtle]
  display
  set graph-on? false

end

;this clears the equations, and their labels, from the graph
to clear-equations

  clear-output
  ask patches with [not border?] [set pcolor black]
  set color-index -1
  set label-place (-1 * (in-sex + 6))

  ask patches with [object = "equation"] [set plabel no-label set temp-label no-label]

  if not setup? and not clear-last-eq? and not clear-last-point?
  [ set graphed-equation-list [] ]
  draw-axes
end

to linear-regression
  locals
  [
    ave-x  ;; the average of all the turtles' xcors
    ave-y  ;; the average of all the turtles' ycors
    sum-x  ;; the sum of all the turtles' xcors
    sum-y  ;; the sum of all the turtles' ycors
    sum-xy  ;; the sum of the product of each turtle's xcor and ycor
    sum-x-square  ;; the sum of the product of each turtle's xcor squared
    sum-y-square  ;; the sum of the product of each turtle's ycor squared
    num-turtles  ;; the number of turtles
    denominator  ;; holds the denominator for the calculations of the slope and the y-intersept
    xcors  ;; a list containing the xcors of the turtles
    ycors  ;; a list containing the ycors of the turtles
     reg-point?
  ]

  ;; you need to have at least two points to make a line
  if count patches with [ any? point-here ] > 1
  [

    clear-equations

    ;; initialize the variables
    set num-turtles count point
    set xcors values-from point [ xcor ]
    set ycors values-from point [ ycor ]
    set sum-x sum xcors
    set sum-y sum ycors
    set ave-x sum-x / num-turtles
    set ave-y sum-y / num-turtles
    set sum-xy sum values-from point [ xcor * ycor ]
    set sum-x-square sum values-from point [ xcor * xcor ]
    set sum-y-square sum values-from point [ ycor * ycor ]

    ;; compute and store the demonimator for the slope and y-intercept so we don't have to do it twice
    set denominator ( sum-x-square - num-turtles * ( ave-x ^ 2 ) )

    ;; if the denominator = 0 or the min of xcors = the max of xcors, the turtles are in a vertical line
    ;; thus, the line of regression will be undefined and have a r^2 value of 1.0
    ifelse ( denominator = 0 ) or ( min xcors = max xcors )
    [
      set m 0
      set rb 0
      set lin-reg-eq "Undefined"
      ;; since all the turtles are in a line r^2 must be 1
      set r-square 1
      ;; set all the patches in the column to be on the line
      ask patches with [ pxcor = xcor-of one-of turtles ]
      [ set reg-point? true ]
    ]
    [
      ;; otherwise, we have some other type of line.  so find the y-intersept and slope of the line
      set rb ( ( ave-y * sum-x-square ) - ( ave-x * sum-xy ) ) / denominator
      ifelse one-to-one?
      [
        ifelse x-range-bigger-than-y?
        [ set rb ((rb * ((x-maximum - x-minimum) * 0.5 * (1 / in-sex))) + ((x-maximum + x-minimum) / 2)) ]
        [ set rb ((rb * ((x-maximum - x-minimum) * 0.5 * (1 / xfactor))) + ((x-maximum + x-minimum) / 2))  ]
      ]
      [ set rb ((rb * ((x-maximum - x-minimum) * 0.5 * (1 / in-sex))) + ((x-maximum + x-minimum) / 2)) ]


      set m ( sum-xy - num-turtles * ave-x * ave-y ) / denominator

      ;; compute the value of r^2
      ifelse ( ( sum-y-square - num-turtles * ( ave-y ^ 2 ) ) = 0 ) or ( min ycors = max ycors )
      [ set r-square 1 ]
      [
        set r-square ( ( sum-xy - num-turtles * ave-x * ave-y ) ^ 2 ) /
          ( ( sum-x-square - num-turtles * ( ave-x ^ 2 ) ) * ( sum-y-square - num-turtles * ( ave-y ^ 2 ) ) )
      ]
      set lin-reg? true

      set lin-reg-eq (word "y=" (precision m 3) "x+" (precision rb 3))
      enter-equation

    ]
  ]
end

to clear-last-point
locals [index]
  set clear-last-point? true
  if (length graphed-point-string > 0)
  [
    set index length graphed-point-string - 1
    while [item index graphed-point-string != "("]
    [ set index index - 1 ]
    set graphed-point-string substring graphed-point-string 0 index
show graphed-point-string
    ask point [die]
    ask patches with [ object = "point"] [set plabel no-label set temp-label no-label]
    enter-points
  ]
  set clear-last-point? false

end


to clear-last-eq
  set clear-last-eq? true
  if (length graphed-equation-list > 0)
  [
    set graphed-equation-list butlast graphed-equation-list
    setup
  ]
  set clear-last-eq? false

end

to enter-points
  locals [temp-point point-color point-ok?  next-x next-y index point-list lower-index upper-index phrase-list done?]

  set phrase-ok? true
  set point-ok? true

  set phrase ""
  set answer ""
  set rule ""

  ifelse setup? or clear-last-point?
  [
    set phrase-list graphed-point-string
  ]
  [
    set phrase-list user-input "Enter a point: ( for example: (-0.5,2) )\nor\nEnter a list of points ( for example: (-0.5,2) (0,3) (2,4) )"
  ]
  set index 0
  set done? false
  while [(index < length phrase-list) and not done?]
  [
    set lower-index index
    set phrase-ok? false
    while [(index < length phrase-list) and not phrase-ok?]
    [
      ifelse (item index phrase-list != ")")
      [ set index index + 1]
      [ set phrase-ok? true]
    ]
    set index index + 1
    set upper-index index
    ifelse phrase-ok? [ set phrase substring phrase-list lower-index upper-index ]
    [ user-message "Please use the form (x,y)"]
    check-phrase-for-correct-point-form

    if (phrase-ok? and not setup? and not clear-last-point?)
    [
      set graphed-point-string word graphed-point-string phrase
    ]

    ifelse phrase-ok?
    [
      if ((aa <= old-x-max) and (aa >= old-x-min) and (bb <= old-y-max) and (bb >= old-y-min))
      [
        ifelse one-to-one?
        [
          ifelse x-range-bigger-than-y?
          [ set next-x ((aa - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / in-sex)))
            set next-y  ((bb - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / yfactor)))
          ]
          [ set next-x  ((aa - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / xfactor)))
            set next-y  ((bb - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / in-sey)))
          ]
        ]
        [
          set next-x ((aa - ((old-x-max + old-x-min) / 2)) / ((old-x-max - old-x-min) * 0.5 * (1 / in-sex)))
          set next-y  ((bb - ((old-y-max + old-y-min) / 2)) / ((old-y-max - old-y-min) * 0.5 * (1 / in-sey)))
        ]
        create-custom-point 1 [set color white set shape "circle" set size 6 setxy next-x next-y]
        ifelse (old-labels)
        [ ask patch-at (next-x + 7) (next-y + 5) [set temp-label phrase set plabel phrase] ]
        [ ask patch-at (next-x + 7) (next-y + 5) [set temp-label phrase ] ]
        ask patch-at (next-x + 7) (next-y + 5) [ set object "point" set plabel-color white ]
      ]
    ]
    [ set done? true]
  ]

end

;this clears the points, and their labels, that you entered
to clear-points

  ask point [die]
 ; set point-list ""
  ask patches with [ object = "point"] [set plabel no-label set temp-label no-label]
  if not setup? [set graphed-point-string ""]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 5. Check and manipulate phrase for evaluation   ;;
;;    a. check-phrase-for-correct-decimals         ;;
;;    b. check-phrase-for-correct-use-of-functions ;;
;;    c. check-for-unidentified-letters            ;;
;;    d. check-phrase-for-inconsistencies          ;;
;;    e. compile-list-into-list                    ;;
;;    f. translate-phrase-into-list                ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;this checks that whole and decimal numbers are in the correct form
to check-phrase-for-correct-decimals
locals [index]

  set index 0
  while [index < (length phrase - 1)]
  [
    if (item index phrase = ".")
    [
      if not (member? item (index + 1) phrase numbers)
      [set phrase-ok? false
       user-message "Please have numbers to the right of a decimal point"]
    ]
    set index index + 1
  ]
end

;this checks to make sure functions have arguments, and appear in correct places
to check-phrase-for-correct-use-of-functions
locals [index temp-phrase end-functions]

  set end-functions ["n" "s" "t" "g"]
  set index 0

  ;functions must not precede nothing
  if (member? last phrase end-functions)
  [
    set phrase-ok? false
    user-message "Please follow a function with an argument. "
  ]
  while [(index < length phrase) and phrase-ok?]
  [
    if (member? item index phrase end-functions)
    [
      ;functions must not precede a ) or operator, unless operator is -
      if((index + 1) < length phrase)
      [
        if (item (index + 1) phrase = ")")
        [ set phrase-ok? false
          user-message "Please do not follow a function with a right parentheses." ]
        if ((member? item (index + 1) phrase operators) and
           (item (index + 1) phrase != "-"))
        [ set phrase-ok? false
          user-message "Please do not follow a function with an operation. " ]
        ;function - , number or variable, must follow
        if ((item (index + 1) phrase = "-") and (index + 2 <= length phrase))
        [
          if not(member? item (index + 2) phrase numbers-and-vars)
          [
            set phrase-ok? false
            user-message "If you follow a function with a minus, only follow that with a number or variable. "
          ]
        ]
      ]
    ]
    if (member? item index phrase functions)
    [

      ;functions can follow a ), number or variable
      ; but a * must be inserted between them
      if (index > 0)
      [
        if ((item (index - 1) phrase = ")") or
           (member? item (index - 1) phrase numbers-and-vars))
        [
          set temp-phrase word substring phrase 0 index "*"
          set phrase word temp-phrase substring phrase index length phrase
        ]
      ]


    ]
    set index index + 1
  ]

end

;this checks to make sure the only letters in the phrase are
;describing functions, x, y, pi, and e

to check-phrase-for-correct-point-form
locals [decimal? place index]

  ;;remove double spaces
  set index 1
  while [index < length phrase]
  [
    if (item (index - 1) phrase = " ") and (item index phrase = " ")
    [
      set phrase replace-item index phrase "z"
      set phrase remove "z" phrase
    ]
    set index index + 1
  ]

  ;;make sure there are no numbers space numbers
  set index 2
  while [index < length phrase]
  [

    if ( ((member? (item (index - 2) phrase) numbers) or
           (item (index - 2) phrase = "i") or (item (index - 2) phrase = "e"))
         and
         (item (index - 1)  phrase = " ")
         and
         ((member? (item index phrase) numbers) or
           (item index phrase = "p") or (item index phrase = "e")))
       [set phrase-ok? false
        user-message "Please place a comma between consecutive numbers"]
    set index index + 1
  ]
  set aa ""
  set bb ""

  ;;take out all spaces
  set phrase remove " " phrase

  if phrase-ok? [check-phrase-for-correct-decimals]

  if phrase-ok?
  [ ;make sure it begins with "("
    if (item 0 phrase != "(")
      [set phrase-ok? false
          user-message "Please begin with description of point with a left parentheses"]]

  if phrase-ok?
  [ ;make sure it ends with ")"
    if (item (length phrase - 1) phrase != ")")
      [set phrase-ok? false
        user-message "Please end with description of point with a right parentheses"]]

  if phrase-ok?
  [
    if (length phrase < 5)
      [set phrase-ok? false
        user-message "Please use the form (x,y)"]
  ]

  ;make sure there is a comma in between two items
  if phrase-ok?
  [
    set index 2
    set decimal? false
    while [index < (length phrase - 2)]
    [
      if (item index phrase = ",")
      [
        if (decimal? = true)
        [
          set phrase-ok? false
          user-message "Please do not use more than one comma."
        ]
        set place index
        set decimal? true
      ]
      set index index + 1
    ]
  ]

  if phrase-ok?
  [ ;make sure there the first argument is a number, decimal number, negative number, e or pi; call it
    if (item 1 phrase = "-")
    ;argument 1 must be a negative number
    [
     ;make sure each part from 2 to place - 1 is a number or "."
      set index 2
      while [index < place]
      [
        if not((member? item index phrase numbers) or (item index phrase = "."))
        [ set phrase-ok? false
          user-message "Please let the x value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set aa substring phrase 1 place
        set aa runresult aa
      ]
    ]]

  if phrase-ok?
  [ if (item 1 phrase = "p")
    ;argument 1 must be pi
    [
      if (item 2 phrase != "i")
      [ set phrase-ok? false
          user-message "Please let the x value in (x,y) be a whole or decimal number, positive or negative number, or e or pi."]
      set aa pi ]]
  if phrase-ok?
  [ if (item 1 phrase = "e")
    ;argument 1 must be e
    [  set aa e ]
  ]
  if phrase-ok?
  [ if (member? item 1 phrase numbers)
    ;argument 1 must be a number or decimal number
    [
      ;make sure each part from 1 to place - 1 is a number or "."
      set index 1
      while [index < place]
      [
        if not((member? item index phrase numbers) or (item index phrase = "."))
        [ set phrase-ok? false
          user-message "Please let the x value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set aa substring phrase 1 place
        set aa runresult aa
      ]
    ]
  ]
  if phrase-ok?
  [ if (item 1 phrase = ".")
    ;argument 1 must be a number or decimal number
    [
      ;make sure each part from 1 to place - 1 is a number
      set index 2
      while [index < place]
      [
        if not(member? item index phrase numbers)
        [ set phrase-ok? false
          user-message "Please let the x value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set aa substring phrase 1 place
        set aa runresult aa
      ]
    ]
  ]

  if phrase-ok?
  [
    if (aa = "")
    [set phrase-ok? false
        user-message "Please let the x value in (x,y) be a whole or decimal number, positive or negative number, or e or pi."]
  ]

  ;make sure there the second argument is a number, decimal number, negative number, e or pi; call it b
  if phrase-ok?
  [
    if (item (place + 1) phrase = "-")
    ;argument 2 must be a negative number
    [
     ;make sure each part from (place + 2) to (length phrase - 2) is a number or "."
      set index place + 2
      while [index < (length phrase - 2)]
      [
        if not((member? item index phrase numbers) or (item index phrase = "."))
        [ set phrase-ok? false
          user-message "Please let the y value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set bb substring phrase (place + 1) (length phrase - 1)
        set bb runresult bb
      ]
    ]]

  if phrase-ok?
  [ if (item (place + 1) phrase = "p")
    ;argument 2 must be pi
    [
      if (item (place + 2) phrase != "i")
      [ set phrase-ok? false
          user-message "Please let the y value in (x,y) be a whole or decimal number, positive or negative number, or e or pi."]
      set bb pi ]]
  if phrase-ok?
  [ if (item (place + 1) phrase = "e")
    ;argument 2 must be pi
    [  set bb e ]]
  if phrase-ok?
  [ if (member? item (place + 1) phrase numbers)
    ;argument 2 must be a number or decimal number
    [
      ;make sure each part from place + 1 to length phrase - 2 is a number or "."
      set index place + 1
      while [index < (length phrase - 1)]
      [
        if not((member? item index phrase numbers) or (item index phrase = "."))
        [ set phrase-ok? false
          user-message "Please let the y value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set bb substring phrase (place + 1) (length phrase - 1)
        set bb runresult bb
      ]
    ]]

  if phrase-ok?
  [ if (item (place + 1) phrase = ".")
    ;argument 2 must be a number or decimal number
    [
      ;make sure each part from place + 1 to length phrase - 2 is a number or "."
      set index place + 2
      while [index < (length phrase - 1)]
      [
        if not((member? item index phrase numbers))
        [ set phrase-ok? false
          user-message "Please let the y value in (x,y) be a whole or decimal number."]
        set index index + 1
      ]
      if phrase-ok?
      [
        set bb substring phrase (place + 1) (length phrase - 1)
        set bb runresult bb
      ]
    ]]
  if phrase-ok?
  [
    if (bb = "")
    [ set phrase-ok? false
        user-message "Please let the y value in (x,y) be a whole or decimal number, positive or negative number, or e or pi."]
  ]

end

to check-for-unidentified-letters
locals [index ok-letters done? error?]

      ;[ set phrase-ok? false
      ;  user-message "Since your equation begins with y=, please eliminate "
      ;                +"the y value(s) from the right side of your equation."]
  set ok-letters ["x" "e" "p" "s" "c" "t" "l" "a"]
  set index 0
  set done? true
  while [(index < length phrase) and phrase-ok?]
  [
    set error? false
    ;if there are unacceptable symbols, phrase is not ok
    if not ((member? item index phrase ok-letters) or
            (member? item index phrase numbers) or
            (member? item index phrase operators) or
            (member? item index phrase parenth) or
            (item index phrase = "."))
    [ set error? true]

    ;if there is a t, make sure it is tan
    if (item index phrase = "t")
    [
      ifelse(index + 2 < length phrase)
      [
        if not((item (index + 1) phrase = "a") and (item (index + 2) phrase = "n"))
        [ set error? true ]
        set index index + 3
      ]
      [ set error? true]
    ]

    ;if there is a a, make sure it is abs
    if (index < length phrase)
    [
      if (item index phrase = "a")
      [
        ifelse (index + 2 < length phrase)
        [
          if not((item (index + 1) phrase = "b") and (item (index + 2) phrase = "s"))
          [ set error? true]
          set index index + 3
        ]
        [ set error? true]
      ]
    ]

    ;if there is a c, make sure it is cos
    if (index < length phrase)
    [
      if (item index phrase = "c")
      [
        ifelse (index + 2 < length phrase)
        [
          if not((item (index + 1) phrase = "o") and (item (index + 2) phrase = "s"))
          [ set error? true]
          set index index + 3
        ]
        [ set error? true]
      ]
    ]

    ;if there is a l, make sure it is ln or log
    if (index < length phrase)
    [
      if (item index phrase = "l")
      [
        ifelse(index + 1 < length phrase)
        [
          if (not(item (index + 1) phrase = "n") and
            not(item (index + 1) phrase = "o"))
          [ set error? true ]
          ;ln
          if (item (index + 1) phrase = "n")
          [ set index index + 2 ]
          if (index + 2 < length phrase)
          [
            ;log
            if ((item (index + 1) phrase = "o") and (item (index + 2) phrase = "g"))
            [ set index index + 3 ]
          ]
        ]
        [ set error? true ]
      ]
    ]

    ;if there is an s, make sure it is is sin or sqrt
    if (index < length phrase)
    [
      if (item index phrase = "s")
      [
        ifelse(index + 2 < length phrase)
        [
          if (not((item (index + 1) phrase = "i") and (item (index + 2) phrase = "n")) and
            not((item (index + 1) phrase = "q") and (item (index + 2) phrase = "r")))
          [ set error? true ]
          ;sin
          if (item (index + 1) phrase = "i") and (item (index + 2) phrase = "n")
          [ set index index + 3 ]
          if (index + 3 < length phrase)
          [
            ;sqrt
            if ((item (index + 1) phrase = "q") and (item (index + 2) phrase = "r")
            and (item (index + 3) phrase = "t"))
            [ set index index + 4 ]
          ]
        ]
        [ set error? true ]
      ]
    ]

    ;if there is a p, make sure it is pi
    if (index < length phrase)
    [
      if (item index phrase = "p")
      [
        ifelse (index + 1 < length phrase)
        [
          if not(item (index + 1) phrase = "i")
          [ set error? true]
          set index index + 2
        ]
        [ set error? true]
      ]
    ]

    if error?
    [ set phrase-ok? false
        user-message "Please limit your letters to those which spell the following: "
                      +"x, e, pi, sin, cos, tan, log, ln, sqrt, abs."
    ]

    if (index < length phrase)
    [
      if ((member? item index phrase numbers) or
          (member? item index phrase operators) or
          (member? item index phrase parenth) or
          (item index phrase = "x") or
          (item index phrase = "e") or
          (item index phrase = "."))
      [ set index index + 1]
    ]
  ]
end

;this checks to make sure functions, numbers, variables and parentheses
;exist in a syntacticly correct order
to check-phrase-for-inconsistencies
locals
  [index
   temp-phrase
   parenth-counter
   tem-phrase
  ]


  ;;replace Y with y and X with x
  set index 0
  while [index < length phrase]
  [
    if (item index phrase = "Y") [set phrase replace-item index phrase "y"]
    if (item index phrase = "X") [set phrase replace-item index phrase "x"]
    set index index + 1
  ]

  ;if there is a carriage return represented by /n, remove it from equation
  set phrase remove "\n" phrase

  ;;remove double spaces
  set index 1
  while [index < length phrase]
  [
    if (item (index - 1) phrase = " ") and (item index phrase = " ")
    [
      set phrase replace-item index phrase "z"
      set phrase remove "z" phrase
    ]
    set index index + 1
  ]

  ;;make sure there are no numbers space numbers
  set index 4
  while [index < length phrase]
  [
    if ((member? (item (index - 2) phrase) numbers) and
       (item (index - 1)  phrase = " ") and
       (member? (item index phrase) numbers))
       [set phrase-ok? false
        user-message "Please place an operation between consecutive numbers"]
    set index index + 1
  ]

  ;;take out all spaces
  set phrase remove " " phrase

   ;;must begin with y=
  if (length phrase < 3)
  [ set phrase-ok? false
    user-message "Please begin your equation with y= "
  ]
  if (phrase-ok?)
  [
    ;>=
    ;<=
    ;=
    ;<
    ;>

    if not ((item 0 phrase = "y") and
            ((item 1 phrase = "=") or (item 1 phrase = "<") or (item 1 phrase = ">")))
    [ set phrase-ok? false
        user-message "Please begin your equation with y=, y>=, y<=, y>, or y<"]
    ;]
  ;  [ if not ((item 0 phrase = "y") and
  ;           ((item 1 phrase = "<") or (item 1 phrase = ">") or (item 1 phrase = "=")))
  ;    [ set phrase-ok? false
  ;      user-message "Please begin your equation with y=, y>=, y<=, y> or y<"]
  ;  ]

    ifelse ((item 1 phrase = "<") or (item 1 phrase = ">"))
    [set inequality? true]
    [set inequality? false]

  ]

  if (phrase-ok?)
  [
    ifelse  (inequality? and (item 2 phrase = "="))
    [

      ;>=, <=
      set phrase-prefix substring phrase 0 3
      set phrase substring phrase 3 length phrase
    ]
    [

      ;=, <, >
      set phrase-prefix substring phrase 0 2
      set phrase substring phrase 2 length phrase
    ]

  ]
  if (phrase-ok?)
  [
    ;;if y=, must have no other y's in equation,
    if(item 0 phrase = "y")
    [
      ;;remove y= and make sure no other y's are in equation
   ;   set phrase butfirst phrase
   ;   set phrase butfirst phrase
      if (member? "y" phrase)
      [ set phrase-ok? false
        user-message "Since your equation begins with y=, please eliminate "
                      +"the y value(s) from the right side of your equation."]
    ]
  ]

  ;must have something after y=
  if (phrase-ok?)
  [
    if (empty? phrase)
    [
      set phrase-ok? false
      user-message "Please add a right side of the equation"
    ]
  ]
  ;if there is a number or variable next to a number or variable, add a * in between
  ;remember number next to a number has already been removed
  ;or if there is a number or variable after the letter i, from pi
  ;  if the letter i is not in the "pi",
  ;  then this will be caught in check-phrase-for-unidentified-letters
  ;and not number next to number
  set index 1
  while [index < length phrase]
  [
    if (not ((member? item index phrase numbers) and
           (member? item (index - 1) phrase numbers)))
    [

      if (((member? item index phrase numbers-and-vars) and
           (member? item (index - 1) phrase numbers-and-vars)) or
         ((member? item index phrase numbers-and-vars) and
           (item (index - 1) phrase = "i")))

      [
        set tem-phrase word (substring phrase 0 index) "*"
        set phrase word (tem-phrase) (substring phrase index length phrase)
      ]
    ]
    set index index + 1

  ]

  ;;must have even number of parentheses
  set index 0
  set parenth-counter 0
  while [index < length phrase]
  [
    if (item index phrase = "(") [set parenth-counter parenth-counter + 1]
    if (item index phrase = ")") [set parenth-counter parenth-counter - 1]
    set index index + 1
  ]
  if (parenth-counter != 0)
  [
    set phrase-ok? false
    user-message "Please use an equal number of right and left parentheses."
  ]

  set index 0
  set parenth-counter 0
  while [phrase-ok? and (index < length phrase)]
  [

    if (index > 0)
    [
      ;;must not have rightparenth next to leftparenth
      if ((item (index - 1) phrase = "(") and
         (item index phrase = ")"))
        [
          set phrase-ok? false
          user-message "Please do not place a right parentheses next to a left parentheses."
        ]

      ;;if there is a leftparenth next to rightparenth, add a * in between
      if ((item (index - 1) phrase = ")") and
         (item index phrase = "("))
        [
          set tem-phrase word (substring phrase 0 index) "*"
          set phrase word (tem-phrase) (substring phrase index length phrase)
        ]

      ;;must not have operator rightparenth
      if ((member? (item (index - 1) phrase) operators) and
         (item index phrase = ")"))
        [
          set phrase-ok? false
          user-message "Please do not place an operation next to a right parentheses."
        ]

      ;;must not have leftparenth operator, unless operator is a - and
      ;;what is after is a number or variable
      if (index < (length phrase - 1))
      [
        if ((item (index - 1) phrase = "(") and
           (member? (item index phrase) operators))
          [
            ifelse(item index phrase != "-")
            [
              set phrase-ok? false
              user-message "Please do not place a left parentheses next to an operation."
            ]
            [
              if (not(member? item (index + 1) phrase numbers-and-vars))
              [
                set phrase-ok? false
                user-message "Please do not place a left parentheses next to an operation."
              ]
            ]
          ]
      ]

      ;;must not have operator operator, unless second operator is a - and
      ;;what is after is a number or variable
      if ((index < length phrase - 1) and (index > 0))
      [
        if ((member? (item (index - 1) phrase) operators) and
           (member? (item index phrase) operators))
        [
          if(item index phrase != "-")
          [
            set phrase-ok? false
            user-message "Please do not place an operation next to an operation."
          ]
        ]
      ]
      if ((index < length phrase - 1) and (index > 1))
      [
        if ((member? item (index - 2) phrase operators) and
            (member? (item (index - 1) phrase) operators))
        [
          if (member? (item index phrase) operators)
          [
            set phrase-ok? false
            user-message "Please do not place an operation between operations."
          ]
          if(((item index phrase = "(") or (member? (item index phrase) functions)) and
             (item (index - 1) phrase = "-"))
          [ set temp-phrase word substring phrase 0 index "1*"
            set phrase word temp-phrase substring phrase index length phrase
            set index index + 2
          ];end i
        ]
      ];end if

      ;;if number next to left parentheses, add a times between it
      if ((member? (item (index - 1) phrase) numbers-and-vars) and
         (item index phrase = "("))
        [
          set tem-phrase word (substring phrase 0 index) "*"
          set phrase word tem-phrase (substring phrase index length phrase)
          ;set phrase-ok false
          ;user-message "Please do not place a number next to a left parentheses."
        ]

      ;;must not have right parentheses next to number
      if ((item (index - 1) phrase = ")") and
         (member? (item index phrase) numbers-and-vars))
        [
          set tem-phrase word (substring phrase 0 index) "*"
          set phrase word tem-phrase (substring phrase index length phrase)
        ]
    ];end if
    set index index + 1
  ];end while

  if (phrase-ok?)
  [
    ;;must not begin a string with ")" or an operation, unless operation is -
    ;;and character after that is a number or variable or decimal
    if ((item 0 phrase = ")") or
       (member? item 0 phrase operators))
    [
      ifelse(item 0 phrase != "-")
      [
         set phrase-ok? false
          user-message "Please do not begin the right side of the equation with "
                       + "a right parentheses or an operation."

      ]
      [
        ;don't have phrase be "-"
        ifelse (length phrase = 1)
        [  set phrase-ok? false
           user-message "Please do not begin the right side of the equation with "
                       + "an operation."
        ]
        [
          ;if it is -function, make it -1*function
          ifelse((member? item 1 phrase functions) or
                 (member? item 1 phrase vars) or
                 (item 1 phrase  = "."))
          [ set phrase word "-1*" substring phrase 1 length phrase ]
          [
            if not((member? item 1 phrase numbers-and-vars) or (item 1 phrase = "("))
            [
               set phrase-ok? false
               user-message "Please do not begin the right side of the equation with "
                         + "a right parentheses or an operation."

            ];end if
            if (item 1 phrase = "(")
            [
              set phrase word "-1*" substring phrase 1 length phrase
            ]
          ];end else
        ];end else
      ];end else
    ];end if

    ;;must not end a string with "(" or an operation
    if ((item (length phrase - 1) phrase = "(") or
       (member? item (length phrase - 1) phrase operators))
    [
       set phrase-ok? false
       user-message "Please do not end the right side of the equation with "
                       + "a left parentheses or an operation."
    ] ;end if
  ];end if

end

;translate the phrase into a list containing each part of the equation, in the order it was entered
;this changes it from "1.0+2*sqrt4" to ["1.0" "+" "2" "*" "sqrt" "4"]
to translate-phrase-into-list
  locals[ index temp-phrase add-to-temp-phrase? end-functions temp init-index done?]

  set end-functions ["n" "s" "t" "g"]
  set index 0
  set temp-phrase []

  while [index < length phrase]
  [
    set add-to-temp-phrase? false

    ;if item index phrase is parenth
    if (member? item index phrase parenth)
    [ set temp-phrase lput item index phrase temp-phrase
      set index index + 1]

    if (index < length phrase)
    [
      ;if item is a function, then add to op-list
      if (member? item index phrase functions)
      [
        ;if function is abs
        if (item index phrase = "a")
        [ set temp-phrase lput "abs" temp-phrase           set index index + 3]
        ;if function is cos
        if (item index phrase = "c")
        [ set temp-phrase lput "cos" temp-phrase           set index index + 3]
        ;if function is tan
        if (item index phrase = "t")
        [ set temp-phrase lput "tan" temp-phrase           set index index + 3]

        ;if function is ln or log
        if (item index phrase = "l")
        [ ifelse (item (index + 1) phrase = "n")
          [ set temp-phrase lput "ln" temp-phrase          set index index + 2]
          [ set temp-phrase lput "log" temp-phrase         set index index + 3]
        ]
        ;if function is sin or sqrt
        if (item index phrase = "s")
        [ ifelse (item (index + 1) phrase = "i")
          [ set temp-phrase lput "sin" temp-phrase         set index index + 3]
          [ set temp-phrase lput "sqrt" temp-phrase        set index index + 4]
        ]
      ]
    ]

    if (index < length phrase)
    [
      ;if item is a variable, then add to num-list
      if (member? item index phrase vars)
      [
        ifelse(item index phrase != "p")
        ;if item is not "p"
        [ set temp-phrase lput item index phrase temp-phrase set index index + 1]
        ;if item is a "p"
        [ set temp-phrase lput "pi" temp-phrase          set index index + 2]
      ]
    ]

    if (index < length phrase)
    [
      ;if item is a number, then add to num-list
      if (member? item index phrase numbers or item index phrase = ".")
      [
        set add-to-temp-phrase? true
      ]
    ]

    if (index < length phrase)
    [
      ;if item is an operator and not a "-", then add to op-list
      if ((member? item index phrase operators) and (item index phrase != "-"))
      [
        set temp-phrase lput item index phrase temp-phrase    set index index + 1]
    ]

    if (index < length phrase)
    [
      if (item index phrase = "-")
      [
        ;if "-" is the first part of string, add it to num-list
        ifelse(index = 0)
        [
          if (item 0 phrase = "-")
          ;add to num-list
          [ set add-to-temp-phrase? true ]
        ]
        [
          ;if item is a "-" and prev item is an operator or end-function or "(" then add to num-list
          ;else then add to op-list
          ifelse((member? item (index - 1) phrase operators) or
                (member? item (index - 1) phrase end-functions) or
                (item (index - 1) phrase = "("))
          ;add to num-list
          [ set add-to-temp-phrase? true ]
          ;add to op-list
          [ set temp-phrase lput "-" temp-phrase      set index index + 1]
        ]
      ]
    ]

    if add-to-temp-phrase?
    [
      set temp []
      set init-index index
      set temp item index phrase
      set index index + 1
      set done? false

      while [(index < length phrase) and not done?]
      [
        ;if the next part of the phrase is a continuation of the phrase,
        ;add it onto temp
        ifelse((member? item index phrase numbers) or (item index phrase = "."))
        [ set temp substring phrase init-index (index + 1) set index index + 1]
        [ set done? true ]
      ]

      set temp-phrase lput temp temp-phrase
      ;if its a negative variable
      if (length temp = 1 and temp = "-")
      [
        set temp-phrase butlast temp-phrase
        ifelse(item index phrase = "p")
        [
          set temp "pi"
          set index index + 3
        ]
        [
          set temp item index phrase
          set index index + 2
        ]
        set temp-phrase lput "(" temp-phrase
        set temp-phrase lput "-1" temp-phrase
        set temp-phrase lput "*" temp-phrase
        set temp-phrase lput temp temp-phrase
        set temp-phrase lput ")" temp-phrase
      ]
    ]
  ]

  ;if a number or variable had parenth around itself,
  ;if would appear as "(" ")" in op-list
  ;remove this if it happens
  set index 2
  while [index < length temp-phrase]
  [
    if ((item (index - 2) temp-phrase = "(") and
       (item index temp-phrase = ")"))
       [
         set temp-phrase remove-item (index - 2) temp-phrase
         set temp-phrase remove-item (index - 1) temp-phrase
       ]
       set index index + 1
  ]

  set phrase temp-phrase

end



;this puts the list containing the equation, into reverse polish notation
;it also lets the strings of numbers, become numbers (not strings)
;so  ["1" "+" "2.3"] becomes [1 2.3 "+"]
;and ["1" "+" "sqrt" "4"] becomes [1 4 "sqrt" "+"]
to compile-list-into-list
locals [index t tt stack precedence-first-in-stack precedence-t index2 done?]
;doesnt work for functions yet
  set ans-list []
  set stack []
  set index 0

  while [index < length phrase]
  [

    set t item index phrase
;show t
    ifelse(t = "(") ;push t on the stack ;set stack lput t ans-list
    [ set stack lput t stack ]
    [

      ifelse((member? t operators) or (member? item 0 t functions))
      [
        ;find precedence
        ;call precedence-first-in-stack the leftmost operation after the rightmost "("
        ifelse (not empty? stack)
        [
          set index2 (length stack - 1)
          set done? false
          while [(index2 >= 0) and not done?]
          [
            ifelse (member? item index2 stack parenth)
            [
              if ((index2 + 1) < length stack) [ set index2 index2 + 1]
              set done? true
            ]
            [ set index2 index2 - 1 ]
          ]
          if (index2 = -1) [set index2 0 ]

          ;set default 4= sqrt cos sin tan ln log abs
          set precedence-first-in-stack 4
          ;5= ( )
          if (member? item index2 stack parenth) [set precedence-first-in-stack 0]
          ;3= ^
          if (item index2 stack = "^") [set precedence-first-in-stack 3]
          ;2= / *
          if ((item index2 stack = "/") or
            (item index2 stack = "*")) [set precedence-first-in-stack 2]
          ;1= + -
          if ((item index2 stack = "+") or
            (item index2 stack = "-")) [set precedence-first-in-stack 1]
        ]
        [ set precedence-first-in-stack -1]

        ;set default 4= sqrt cos sin tan ln log abs
        set precedence-t 4
        ;5= ( )
        if (member? t parenth) [set precedence-t 0]
        ;3= ^
        if (t = "^") [set precedence-t 3]
        ;2= / *
        if ((t = "/") or
          (t = "*")) [set precedence-t 2]
        ;1= + -
        if ((t = "+") or
          (t = "-")) [set precedence-t 1]
        if
        ;(precedence-first-in-stack >= precedence-t) and
          not ((precedence-first-in-stack = 4) and
           (precedence-t = 4))
        [
          while [(not empty? stack) and
             (item (length stack - 1) stack != "(")
             and (precedence-first-in-stack >= precedence-t)
              ]
          [
            set tt last stack
            set stack butlast stack
            set ans-list lput tt ans-list
          ]
        ]
         set stack lput t stack
       ]
       [
         ifelse(t = ")")
         [
           while [(length stack != 0)  and
           (last stack != "(")]
           [
             set tt last stack
             set stack butlast stack
             set ans-list lput tt ans-list
           ]
           set stack butlast stack
         ]
         [
           ifelse((member? (item 0 t) numbers-and-vars) or (item 0 t = "."))
           [
            ;entering it as a number, not string
            set ans-list lput (runresult t) ans-list

           ]
          [
            if (length t > 1)
            [
              if (member? (item 1 t) numbers)
              [set ans-list lput t ans-list]
            ]
          ]
        ]
      ]
    ]
    set index index + 1
  ];end while
  while [not empty? stack]
  [
    set tt last stack
    set stack butlast stack
    set ans-list lput tt ans-list

  ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 6. Other non-hubnet functions                   ;;
;;    a. evaluate-function                         ;;
;;    b. draw-axes                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;this evaluates the function by looking at the equation
;in the form of reverse-polish notation, found in ans-list
;and evaluates it at the specific value of x, it returns answer as that number
;if the equation is undefined at that x, it returns answer as "undefined"
to evaluate-function
locals [t answer-list a b w done? temp-ans-list]

  set temp-ans-list ans-list
  set answer-list []
  set answer 0
  set done? not true
  while [not empty? temp-ans-list and not done?]
  [
    set t first temp-ans-list
    set temp-ans-list butfirst temp-ans-list
    ifelse(is-string? t)
    [

      ;if t is a positive variable
      if (member? item 0 t vars)
      [ set answer-list lput t answer-list
        set w t ]
      if (length t > 1)
      [
        ;if t is a function
        if (member? item 0 t functions)
        [
          set a last answer-list
          if (is-string? a) [set a runresult a]
          set answer-list butlast answer-list

          if (t = "abs") [set w abs (a) ]
          if (t = "sin") [set w sin (a * 180 / pi)]
          if (t = "cos") [set w cos (a * 180 / pi)]
          if (t = "tan") [set w tan (a * 180 / pi)]
          if (t = "sqrt")
          [
            ifelse (a < 0)
            [ set answer "undefined" set done? true ]
            [set w sqrt a]
          ]
          if (t = "ln")
          [ ifelse ( a <= 0 )
            [ set answer "undefined" set done? true ]
            [ set w ln a]
          ]
          if (t = "log")
          [
            ifelse ((a <= 0) or (a = 1))
            [set answer "undefined" set done? true ]
            [set w log a 10]
          ]
          set answer-list lput w answer-list
        ]
        ;if t is a negative number or negative variable
        if (item 0 t = "-")
        [ set answer-list lput t answer-list
          set w t ]
      ]
      ;if t is an operation
      if (member? t operators)
      [
        set b last answer-list
        if (is-string? b) [set b runresult b ]
        set answer-list butlast answer-list
        set a last answer-list
        if (is-string? a) [set a runresult a ]
        set answer-list butlast answer-list

        if (t = "+") [set w a + b]
        if (t = "-") [set w a - b]
        if (t = "*") [set w a * b]
        if (t = "/")
        [
          ifelse(b = 0)
          [ set answer "undefined" set done? true ]
          [ set w a / b ]
        ]
        if (t = "^")
        [
          ifelse (((a = 0) and (b < 0)) or
               ((a < 0) and (ceiling b != b)))
          [ set answer "undefined" set done? true ]
          [ set w a ^ b ]
        ]

        set answer-list lput w answer-list
      ]
    ]
    ;if t is a positive number
    [ set answer-list lput t answer-list
      set w t]
  ]
  if (answer != "undefined") [set answer w]
  if (answer = "x") [set answer runresult answer ]
end

;this draws the axes of the graph, depending on the range and scale
to draw-axes
  locals [grid-index grid-index-max grid-increment grid-initial]

  no-display
  set phrase-ok? true

  ;check ranges
  if (x-minimum >= x-maximum)
  [ user-message "Please change the sliders so x-minimum slider is less than x-maximum"
    set phrase-ok? false
    set x-minimum old-x-min
    set x-maximum old-x-max
  ]
  if (y-minimum >= y-maximum)
  [ user-message "Please change the sliders so y-minimum slider is less than y-maximum"
    set phrase-ok? false
    set y-minimum old-y-min
    set y-maximum old-y-max
  ]

  if phrase-ok?
  [
    set old-x-max x-maximum
    set old-x-min x-minimum
    set old-y-max y-maximum
    set old-y-min y-minimum
    set old-1-to-1 one-to-one?
    set old-grid grid?
    set x-increment (x-maximum - x-minimum) / (3 * in-sex)

    ask patches
    [ set pcolor 0
      set border? false
      set temp-x 999
      set temp-y 999
    ]
    ask patches [set plabel no-label set plabel-color white]

    ifelse one-to-one?
    [
      ifelse((x-maximum - x-minimum) >= (y-maximum - y-minimum))
      [
        set x-range-bigger-than-y? true
        set yfactor (in-sey) * (y-maximum - y-minimum) / (x-maximum - x-minimum)

        ;create border
        ask patches with [(abs(pxcor) > in-sey) or (abs(pycor) > yfactor )]
        [ set pcolor 2
          set border? true ]

        ;label x and y , minimums and maximums
        ask patches with [(pxcor = round(-1 * screen-edge-x + 15)) and (pycor = round(-1 * yfactor - 4))]
        [ set plabel x-minimum ]
        ask patches with [(pxcor = round(screen-edge-x - 12)) and (pycor = round(-1 * yfactor - 4))]
        [ set plabel x-maximum ]
        ask patches with [(pxcor = round(-1 * screen-edge-x + 7)) and (pycor = round(-1 * yfactor + 3))]
        [ set plabel y-minimum ]
        ask patches with [(pxcor = round(-1 * screen-edge-x + 7)) and (pycor = round(yfactor - 3))]
        [ set plabel y-maximum ]

        ;show x and y axes
        set x-axis ((0 - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / in-sex)))
        set y-axis ((0 - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / yfactor)))
        ask patches with [not border? and (pxcor = round(x-axis) or pycor = round(y-axis))]
        [set pcolor 3]
      ]
      [
        set x-range-bigger-than-y? false
        set xfactor (in-sey) * (x-maximum - x-minimum) / (y-maximum - y-minimum)

        ;create-border
        ask patches with [(abs(pxcor) > xfactor) or (abs(pycor) > in-sey )]
        [ set pcolor 2
          set border? true ]

        ;label x and y , minimums and maximums
        ask patches with [(pxcor = round(-1 * xfactor + 8)) and (pycor = round(-1 * screen-edge-y + 3))]
        [ set plabel x-minimum ]
        ask patches with [(pxcor = round(xfactor)) and (pycor = round(-1 * screen-edge-y + 3))]
        [ set plabel x-maximum ]
        ask patches with [(pxcor = round(-1 * xfactor - 2)) and (pycor = round(-1 * screen-edge-y + 13))]
        [ set plabel y-minimum ]
        ask patches with [(pxcor = round(-1 * xfactor - 2)) and (pycor = round(screen-edge-y - 13))]
        [ set plabel y-maximum ]

        ;show x and y axes
        set x-axis ((0 - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / xfactor)))
        set y-axis ((0 - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / in-sey)))
        ask patches with [not border? and (pxcor = round(x-axis) or pycor = round(y-axis))]
        [set pcolor 3]
      ]
    ]
    [
      ;create-border
      ask patches with [(abs(pxcor) > in-sex) or (abs(pycor) > in-sey )]
      [ set pcolor 2
        set border? true ]

     ;label x and y , minimums and maximums
      ask patches with [(pxcor = round(-1 * screen-edge-x + 15)) and (pycor = round(-1 * screen-edge-y + 3))]
      [ set plabel x-minimum ]
      ask patches with [(pxcor = round(screen-edge-x - 9)) and (pycor = round(-1 * screen-edge-y + 3))]
      [ set plabel x-maximum ]
      ask patches with [(pxcor = round(-1 * screen-edge-x + 7)) and (pycor = round(-1 * screen-edge-y + 13))]
      [ set plabel y-minimum ]
      ask patches with [(pxcor = round(-1 * screen-edge-x + 7)) and (pycor = round(screen-edge-y - 13))]
      [ set plabel y-maximum ]

      ;show x and y axes
      set x-axis ((0 - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / in-sex)))
      set y-axis ((0 - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / in-sey)))
      ask patches with [not border? and (pxcor = round(x-axis) or pycor = round(y-axis))]
      [set pcolor 3]
    ]

    ;if range is from
    ;if range is from -20 to 20 do every 10
    ; from -10 to 10 do every 1
    ; from -1 to 1 do every .1
    if grid?
    [
      ifelse one-to-one? and ((x-maximum - x-minimum) < (y-maximum - y-minimum))
      [

          ifelse (( y-maximum - y-minimum ) > 20)
          ;if x range is greater than 20, draw axes every 10
          [ set grid-index-max x-maximum
            set grid-increment 10
            set grid-initial x-minimum
            while [grid-initial mod 10 != 0]
            [ set grid-initial grid-initial + 1 ]
          ]
          [ ifelse (( y-maximum - y-minimum ) <= 2)
            ;if x range is less than 2, draw axes every 0.1
            [ set grid-index-max x-maximum - 0.1
              set grid-increment 0.1
              set grid-initial x-minimum + 0.1 ]
            ;if x range is between 2 and 20, draw axes every 1
            [ set grid-index-max x-maximum - 1
              set grid-increment 1
              set grid-initial x-minimum + 1 ]
          ]

      ]
      [

        ; do vertical lines on grid
        ifelse (( x-maximum - x-minimum ) > 20)
        ;if x range is greater than 20, draw axes every 10
        [ set grid-index-max x-maximum
          set grid-increment 10
          set grid-initial x-minimum
          while [grid-initial mod 10 != 0]
          [ set grid-initial grid-initial + 1 ]
        ]
        [ ifelse (( x-maximum - x-minimum ) <= 2)
          ;if x range is less than 2, draw axes every 0.1
          [ set grid-index-max x-maximum - 0.1
            set grid-increment 0.1
            set grid-initial x-minimum + 0.1 ]
          ;if x range is between 2 and 20, draw axes every 1
          [ set grid-index-max x-maximum - 1
            set grid-increment 1
            set grid-initial x-minimum + 1 ]
        ]
      ]
      set grid-index grid-initial
      while [grid-index <= grid-index-max]
      [

        ifelse one-to-one?
        [
          ifelse((x-maximum - x-minimum) >= (y-maximum - y-minimum))
          [ set x-axis ((grid-index - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / in-sex)))  ]
          [ set x-axis ((grid-index - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / xfactor))) ]
        ]
        [ set x-axis ((grid-index - ((x-maximum + x-minimum) / 2)) / ((x-maximum - x-minimum) * 0.5 * (1 / in-sex))) ]
        ask patches with [not border? and (pxcor = round(x-axis)) and pcolor != 3]
        [
          ifelse(precision grid-index 1 = 0)
          [ set pcolor 3 ]
            [ set pcolor 1 ]
        ]
        set grid-index grid-index + grid-increment
      ]

      ifelse one-to-one? and ((x-maximum - x-minimum) > (y-maximum - y-minimum))
      [
        ifelse (( x-maximum - x-minimum ) > 20)
        ;if x range is greater than 20, draw axes every 10
        [ set grid-index-max y-maximum
          set grid-increment 10
          set grid-initial y-minimum
          while [grid-initial mod 10 != 0]
          [ set grid-initial grid-initial + 1 ]
        ]
        [ ifelse (( x-maximum - x-minimum ) <= 2)
          ;if x range is less than 2, draw axes every 0.1
          [ set grid-index-max y-maximum - 0.1
            set grid-increment 0.1
            set grid-initial y-minimum + 0.1 ]
          ;if x range is between 2 and 20, draw axes every 1
          [ set grid-index-max y-maximum - 1
            set grid-increment 1
            set grid-initial y-minimum + 1 ]
        ]

      ]
      [
        ;do horizontal lines on grid
        ifelse (( y-maximum - y-minimum ) > 20)
        ;if x range is greater than 20, draw axes every 10
        [ set grid-index-max y-maximum
          set grid-increment 10
          set grid-initial y-minimum
          while [grid-initial mod 10 != 0]
          [ set grid-initial grid-initial + 1 ]
        ]
        [ ifelse (( y-maximum - y-minimum ) <= 2)
          ;if x range is less than 2, draw axes every 0.1
          [ set grid-index-max y-maximum - 0.1
            set grid-increment 0.1
            set grid-initial y-minimum + 0.1 ]
          ;if x range is between 2 and 20, draw axes every 1
          [ set grid-index-max y-maximum - 1
            set grid-increment 1
            set grid-initial y-minimum + 1 ]
        ]
      ]
        set grid-index grid-initial
        while [grid-index <= grid-index-max]
        [

          ifelse one-to-one?
          [
            ifelse((x-maximum - x-minimum) >= (y-maximum - y-minimum))
            [ set y-axis ((grid-index - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / yfactor)))]
            [ set y-axis ((grid-index - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / in-sey)))]
          ]
          [ set y-axis ((grid-index - ((y-maximum + y-minimum) / 2)) / ((y-maximum - y-minimum) * 0.5 * (1 / in-sey))) ]
          ask patches with [not border? and ( pycor = round(y-axis)) and pcolor != 3]
          [
            ifelse(precision grid-index 1 = 0)
            [ set pcolor 3 ]
            [ set pcolor 1 ]
          ]
          set grid-index grid-index + grid-increment
        ]

      ]
  ]
  display
end



@#$#@#$#@
GRAPHICS-WINDOW
228
10
601
404
70
70
2.5745
1
12
1
1
1
0

CC-WINDOW
5
474
835
569
Command Center

SLIDER
224
420
358
453
x-minimum
x-minimum
-50
50
-10
1
1
NIL

SLIDER
472
420
603
453
x-maximum
x-maximum
-50
50
10
1
1
NIL

SLIDER
8
373
222
406
y-minimum
y-minimum
-50
50
-10
1
1
NIL

SLIDER
10
64
223
97
y-maximum
y-maximum
-50
50
10
1
1
NIL

BUTTON
72
10
159
49
Setup
setup
NIL
1
T
OBSERVER
T
NIL

SWITCH
676
281
805
314
one-to-one?
one-to-one?
1
1
-1000

SWITCH
114
422
204
455
labels?
labels?
1
1
-1000

MONITOR
365
411
467
460
Mouse's Position
mouse-coords
3
1

MONITOR
653
227
825
276
Coefficient of Correlation
r-square
3
1

MONITOR
744
174
826
223
y-intercept
rb
3
1

MONITOR
654
174
740
223
Slope
m
3
1

SWITCH
19
422
109
455
grid?
grid?
0
1
-1000

BUTTON
653
136
825
169
Linear Regression
linear-regression
NIL
1
T
OBSERVER
T
NIL

BUTTON
61
207
174
242
Graph
ifelse (Point-or-Function = "Point")\n[ enter-points ]\n[ enter-equation ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
61
247
174
280
Clear Last
ifelse (Point-or-Function = "Point")\n[ clear-last-point ]\n[ clear-last-eq]
NIL
1
T
OBSERVER
T
NIL

BUTTON
61
284
174
317
Clear All
ifelse (Point-or-Function = "Point")\n[ clear-points ]\n[ clear-equations]
NIL
1
T
OBSERVER
T
NIL

CHOOSER
60
153
174
198
Point-or-Function
Point-or-Function
"Point" "Function"
0

TEXTBOX
653
111
743
129
4. Other buttons

@#$#@#$#@
WHAT IS IT?
-----------
This program works as a graphing calculator.

Graph a point or list of points, in the form of "(x,y)"
Graph functions using the form "y=".
Adjust the range and domain.
Find the linear regression of a set of points.

Use positive and negative numbers, decimal numbers, and the constants pi and e.
Use the following operations: +, -, *, /, and ^.
Use the following functions: sin, cos, tan, sqrt, ln, log, and abs.

HOW IT WORKS
------------
It checks the inputted function or set of points for correct mathematical form, and gives specific errors if necessary. It displays points by placing a turtle at those positions. It displays functions by asking a turtle to trace the functions path, by evaluating the y value at various x values throughout the domain.

HOW TO USE IT
-------------
To Graph a Point or List of Points:
Set the "Point or Function" menu to "Point". Press the Graph button, and enter a point or list of points.

To Graph an Function:
Set the "Point or Function" menu to "Function". Press the Graph button, and enter an equation.

To Clear Points or Functions:
Press the Clear Last button to clear the last item of the type shown in the "Point or Function" menu.
Press the Clear All button to clear all items of the type shown in the "Point or Function" menu.

To View the Linear Regression:
Press the "Linear Regression" button, to see the linear regression of your set of points.

To Adjust the Range or Domain:
Adjust the sliders labeled "x-minimum", "x-maximum", "y-minimum" or "y-maximum". To see the result, press the "Setup" button.

To View or Hide the Grid:
Adjust the "grid?" switch, and press the "Setup" button.

To View or Hide the Labels:
Adjust the "labels?" switch, and press the "Setup" button. The labels identify the position of points, and the name of each function.

To Change the Scale:
Adjust the "one-to-one?" switch, and press the "Setup" button. If "one-to-one?" is on, then the size of the screen will shift so the scale is 1 to 1.

CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
