breeds [
  store
  customer
  house
  arrow
]

globals[
  res-street
  store-street
  customer-pop
  smoothness
  max-ad-reach
  buffer
  time
  wlk-buff
  store-address-list
  temp-list
  tol
  left-edge
  right-edge
  screen-size
  last-store-pos
  n-active-stores
  pareto-share
  pareto-util
  pareto-time
  counter
  arrow-lane
  measure-lane
  sand-color
]

patches-own [
  advertisement
  own-store?
  store-street?
  latitude
  longitude
  num-cust
  old-ad
]

customer-own [
  home-x
  home-y
  finish-shopping?
  finished?
  utility
  end-time
  start-time
  my-time
  my-best-time
  my-store
]

store-own [
  number-customers
  curr-mkt-share
  curr-mkt-length
  avg-cust/address
  address
  at-eq?
  old-address
  set?
  goal
  status
  mkt-share
]

arrow-own [
  address
]

to startup
  ca
end

to go
  locals [avg-util]

  ask store [stamp-label]
  ask customer [
     go-shopping
  ]
  set counter counter + 1
  if all-finished? [
    ask store with [set?] [set at-eq? true]
    if (at-eq?-of (one-of store with [who mod num-Stores = time mod num-Stores]))
      [set (at-eq?-of (one-of store with [who mod num-Stores = time mod num-Stores])) false
       set time time + 1
      ]
    ask store with [address >= left-edge and address <= right-edge] [calculations-special]
    ask store [calculations]
    ask store with [who mod num-Stores = time mod num-Stores and not set?] [
      set old-address address
      move
      if (old-address = address) [set at-eq? true]
    ]
    reset-ad
    if pareto-util = 0
      [set pareto-share (sum values-from store [curr-mkt-share]) / num-Stores
       set pareto-time (sum values-from customer [my-time]) / customer-pop
       set pareto-util (sum values-from customer [utility]) / customer-pop
       ask store [remove-label die]
       setup-patches
       setup-stores
       setup-lst
      ]
    if pareto-util != 0 [
      set avg-util (sum (values-from customer [utility])) / customer-pop
      plot-store-graphs
      if (avg-util !=  1)[plot-cust-graphs]
    ]
    reset
  ]
end


to Pareto-Efficient-xcor
  locals [interval mult stable i add]
  set stable left-edge
  set i 1
  set mult (who mod num-Stores) + 1
  set interval 0.5 * ((screen-size / num-Stores) - 1)
  loop [
    ifelse (i > mult)
      [set address add
       set xcor address
       stop]
      [ifelse (i = 1)
         [set add left-edge + interval
          set stable add + 1
          set i i + 1
         ]
         [set add stable + (2 * interval)
          set stable add + 1
          set i i + 1
         ]
      ]
  ]
end


to-report empty-list
 report (remove 0 (list 0 0))
end

to reset
  ifelse num-Stores mod 2 = 0
   [if (n-active-stores > 4) [converge-store-lst]]
   [if (n-active-stores > 3) [converge-store-lst]]
  set store-address-list (list left-edge right-edge)
  set temp-list (list (- screen-edge-x) screen-edge-x)
  ask patches [set num-cust 0]
  ask store [
    if (xcor >= left-edge and xcor <= right-edge )
     [set store-address-list (fput address store-address-list)]
    set temp-list (fput address temp-list)
  ]
  ask customer [
    set heading 0
    set finish-shopping? false
    set finished? false
    set color green
 ]
 setup-lst
 set last-store-pos (length store-address-list) - 2
 set n-active-stores last-store-pos
end

to reset-ad
  ask patches [
    set advertisement 0
    set own-store? false
  ]
  ask store [
    set advertisement max-ad-reach
    set own-store? true
  ]
  advertise
  ask patches [
    ifelse store-street?
     [if (advertisement = 0) [set pcolor gray]]
     [if (advertisement = 0) [set pcolor sand-color]]
  ]
end


to converge-store-lst
  locals [end-pos half-length]
  set end-pos (length store-address-list) - 2
  set half-length 0.5 * ((screen-size /  n-active-stores) - 1)
  ifelse (num-Stores mod 2 = 0)
    [if ((count (store with [address = (item 1 store-address-list) and set?])) = 2 and
         (count (store with [address = (item end-pos store-address-list) and set?])) = 2)
      [set left-edge odd-even-boundary "left-edge"
       set right-edge odd-even-boundary "right-edge"
      ]

    ]
    [if ((count (store with [address = (item 1 store-address-list) and set?])) = 1  and
         (count (store with [address = (item end-pos store-address-list) and set?])) = 1)
      [set left-edge (odd-even-boundary "left-edge") + 1 + half-length
       set right-edge (odd-even-boundary "right-edge") - 1 - half-length
      ]
    ]
  set screen-size (right-edge - left-edge + 1)
end


to move
  remove-label
  set xcor address
  ifelse (address != goal)
    [go-towards-goal]
    [ifelse (curr-mkt-share < 1 /  n-active-stores)
      [tackle set status "offence"]
      [defend set status "defense"]
    ]
  if ((address = odd-even-boundary "left-edge") or
      (address = odd-even-boundary "right-edge"))
     [ifelse (num-Stores mod 2) = 0
       [if (count store-here <= 2) [set set? true set at-eq? true]]
       [if (count store-here <= 1) [set set? true set at-eq? true]]
     ]
  stamp-label
end

to remove-label
  locals [dist]
  set dist measure-lane - store-street
  ask patch-at 0 dist [set plabel no-label]
end


to go-towards-goal
  ifelse abs (goal - address) <= tol
    [set address goal
     set at-eq? true
    ]
    [if (address < goal) [set address address + tol]
     if (address > goal) [set address address - tol]
    ]
  set xcor address
end

to tackle
  locals [
    list-dist
    lst
    dist-struct
    best-x
  ]

  set lst empty-list
  set list-dist (lst-dist remove-me-list lst)
  set dist-struct best-xcor list-dist

  set best-x (last dist-struct)
  set goal best-x
  ifelse (abs (best-x - address)) <= tol
   [set address best-x
    set xcor address
    set at-eq? true
   ]
   [ifelse best-x > address
     [set address address + tol
      set xcor address
     ]
     [if best-x < address [set address address - tol
                           set xcor address]
     ]
  ]
end


to-report go-toward-edge
  locals [my-pos left-pos right-pos lf-x rt-x r-bound l-bound]

  set my-pos (position address store-address-list)
  set left-pos my-pos - 1
  set right-pos my-pos + 1

  set rt-x (item right-pos store-address-list)

  ifelse (item left-pos store-address-list = "left-edge")
    [set lf-x left-edge]
    [ifelse (item right-pos store-address-list = "right-edge")
      [set rt-x right-edge]
      [set lf-x (item left-pos store-address-list)
       set rt-x (item right-pos store-address-list)]
    ]

  set r-bound (odd-even-boundary "right-edge")
  set l-bound (odd-even-boundary "left-edge")

  ifelse ((position lf-x store-address-list) = 1 and
         (item last-store-pos store-address-list) = (item right-pos store-address-list))
   [ifelse (count store with [address = (item 1 store-address-list)]) <
           (count store with [address = (item last-store-pos store-address-list)])
      [report (check-boundary lf-x "left-edge")]
      [report (check-boundary rt-x "right-edge")]
   ]
   [if (left-pos = 1) [report (check-boundary lf-x "left-boundary")]
    if (item last-store-pos store-address-list) = (item right-pos store-address-list)
       [report (check-boundary rt-x "right-edge")]
    report address
   ]
end

to-report odd-even-boundary [my-pos]
  locals [x-cor other-end]

  ifelse (num-Stores mod 2) = 1
    [set other-end 0.5 * ((screen-size /  n-active-stores) - 1)
    if (my-pos = "left-edge") [set x-cor (- (right-edge - other-end))]
    if (my-pos = "right-edge") [set x-cor (right-edge - other-end)]
    ]
    [set other-end (screen-size / n-active-stores) - 0.5
     if (my-pos = "left-edge") [set x-cor (left-edge + other-end)]
     if (my-pos = "right-edge") [set x-cor (right-edge - other-end)]
    ]

  report x-cor
end


to-report am-edge? [add]
  locals [lt-pos my-pos rt-x]
  set my-pos (position add store-address-list)
  set lt-pos my-pos - 1
  set rt-x (item last-store-pos store-address-list)
  ifelse (lt-pos = 0)
      or (address = rt-x)
    [report true]
    [report false]
end

to-report one-edge?
  locals [my-left my-right my-pos rt-x]
  set my-pos (position address store-address-list)
  set my-left (position address store-address-list) - 1
  set my-right (position address store-address-list) + 1
  set rt-x (item my-right store-address-list)

 ifelse (my-pos = 2) or (item last-store-pos store-address-list = rt-x)
    [report true]
    [report false]
end

to-report zero-edge?
  locals [mypos my-left my-right]
  set mypos (position address store-address-list)
  set my-left mypos - 1
  set my-left mypos + 1

  ifelse (am-edge? (item my-left store-address-list))
      or (am-edge? (item my-right store-address-list))
    [report false]
    [report true]
end

to-report two-edge?
  locals [my-left my-right]
  set my-left (position address store-address-list) - 1
  set my-right (position address store-address-list) + 1
  ifelse (am-edge? (item my-left store-address-list))
         and (am-edge? (item my-right store-address-list))
    [report true]
    [report false]
end


to defend
  locals [
    store-left-add
    store-right-add
    my-pos
    x-cor
    a
  ]

  set my-pos (position address store-address-list)
  if my-pos = 1 [set my-pos "left-edge"]
  if address = (item last-store-pos store-address-list) [set my-pos "right-edge"]

  ifelse my-pos = "left-edge"  or my-pos = "right-edge"
    [if my-pos = "left-edge" [set store-left-add left-edge
                              set store-right-add (item 2 store-address-list)]
     if my-pos = "right-edge" [set store-right-add right-edge
                               set store-left-add (item (last-store-pos - 1) store-address-list)]
    ]
    [set store-left-add  item (my-pos - 1) store-address-list
     set store-right-add item (my-pos + 1) store-address-list
    ]

  ifelse (num-Stores mod 2) = 1
    [set a 0.5 * ((screen-size /  n-active-stores) - 1)
     ifelse (my-pos = "left-edge")
       [set x-cor (- (right-edge - a))]
       [set x-cor (right-edge - a)]
    ]
    [set a (screen-size / n-active-stores) - 0.5
     ifelse (my-pos = "left-edge")
       [set x-cor (left-edge + a)]
       [set x-cor (right-edge - a)]
    ]

  set goal x-cor

  ifelse (am-edge? address)
    [if (curr-mkt-share >= (1 / n-active-stores) or (count other-store-here > 0))
         and (address != x-cor)
      [ifelse (abs (x-cor - address)) <= tol
        [set address x-cor set at-eq? true]
        [if (x-cor < address) [set address address - tol]
         if (x-cor > address) [set address address + tol]
        ]
       set xcor address
      ]
     if address = x-cor [set at-eq? true]
    ]
    [non-edge-defense]
end

to non-edge-defense
  locals [
    store-left-add
    store-right-add
    my-pos
    left-dist
    right-dist
    x-cor
  ]

  set my-pos (position address store-address-list)
  if my-pos = 1 [set my-pos "left-edge"]
  if address = (item last-store-pos store-address-list) [set my-pos "right-edge"]

  ifelse my-pos = "left-edge"  or my-pos = "right-edge"
    [if my-pos = "left-edge" [set store-left-add left-edge
                              set store-right-add (item 2 store-address-list)]
     if my-pos = "right-edge" [set store-right-add right-edge
                               set store-left-add (item (last-store-pos - 1) store-address-list)]
    ]
    [set store-left-add  item (my-pos - 1) store-address-list
     set store-right-add item (my-pos + 1) store-address-list
    ]

  set left-dist abs (store-left-add - address)
  set right-dist abs (store-right-add - address)

  ifelse num-Stores mod 2 = 0
    [ifelse one-edge?
      [set x-cor go-toward-edge
       set goal x-cor
       ifelse (abs (x-cor - address)) <= tol
         [set address x-cor
          set at-eq? true]
         [if (x-cor < address) [set address address - tol]
          if (x-cor > address) [set address address + tol]
         ]
       set xcor address
      ]
      [set goal (store-left-add + store-right-add) / 2
       ifelse (abs (left-dist - right-dist)) <= tol
         [set address goal
          set at-eq? true]
         [ifelse (left-dist > right-dist)
           [set address address - tol
            set xcor address
           ]
           [if (right-dist > left-dist)
              [set address address + tol
               set xcor address
              ]
           ]
         ]
      ]
    ]
    [set goal (store-left-add + store-right-add) / 2
     ifelse (abs (left-dist - right-dist)) <= tol
      [set address goal]
      [ifelse (left-dist > right-dist)
        [set address address - tol
         set xcor address
        ]
        [if (right-dist > left-dist)
          [set address address + tol
           set xcor address
          ]
        ]
      ]
    ]
end

to calculations-special
  locals [x-bf x-af x my-add n]
  set my-add address
  ifelse (count store-here = 1)
    [set x-bf (item ((position address store-address-list) - 1) store-address-list)
     set x-af (item ((position address store-address-list) + 1) store-address-list)
    ]
    [set x count store-here with [address = my-add]
     set x-bf (item ((position address store-address-list) - 1) store-address-list)
     set x-af (item ((position address store-address-list) + x) store-address-list)
    ]
  ifelse (x-bf = "left-edge" and x-af = "right-edge")
    [set curr-mkt-length screen-size]
    [ifelse (x-bf = "left-edge")
      [set curr-mkt-length ((abs (left-edge - address)) + tol) + ((abs (x-af - address)) - tol) / 2]
      [ifelse (x-af = "right-edge")
        [set curr-mkt-length ((abs (right-edge - address)) + tol) + ((abs (x-bf - address)) - tol) / 2]
        [set curr-mkt-length abs (x-bf - x-af) / 2]
      ]
    ]

  if curr-mkt-length = 0 [set curr-mkt-length 1]

  set number-customers num-cust / (count store-here)
  set curr-mkt-share curr-mkt-length / (count store-here * screen-size)
  set avg-cust/address number-customers / curr-mkt-length
end

to calculations
  locals [x-bf x-af x my-add mkt-length n]
  set my-add address
  ifelse (count store-here = 1)
    [set x-bf (item ((position address temp-list) - 1) temp-list)
     set x-af (item ((position address temp-list) + 1) temp-list)
    ]
    [set x count store-here with [address = my-add]
     set x-bf (item ((position address temp-list) - 1) temp-list)
     set x-af (item ((position address temp-list) + x) temp-list)
    ]
  ifelse (x-bf = "left-edge" and x-af = "right-edge")
    [set mkt-length screen-size-x]
    [ifelse (x-bf = "left-edge")
      [set mkt-length ((abs (- (screen-edge-x - address))) + tol) + ((abs (x-af - address)) - tol) / 2]
      [ifelse (x-af = "right-edge")
        [set mkt-length ((abs (screen-edge-x - address)) + tol) + ((abs (x-bf - address)) - tol) / 2]
        [set mkt-length abs (x-bf - x-af) / 2]
      ]
    ]

  if mkt-length = 0 [set curr-mkt-length 1]

  set n num-cust / (count store-here)
  set mkt-share mkt-length / (count store-here * screen-size-x)
end




to-report remove-me-list
  locals [address-lst lst]
  set address-lst store-address-list
  set lst empty-list

  loop [
    ifelse(empty? address-lst)
      [report lst]
      [ifelse (first address-lst) = address
         [set lst (sentence lst (but-first address-lst))
          set address-lst empty-list]
         [set lst (lput (first address-lst) lst)
          set address-lst (but-first address-lst)]
      ]
   ]
end


to-report lst-dist [xcor-lst dist-lst]
  locals [lst st-1 st-2 x1 x2 dist others-1 others-2]
  loop [
   ifelse (length xcor-lst) = 1
     [report dist-lst]
     [set st-1 (item 0 xcor-lst)
      set st-2 (item 1 xcor-lst)
      ifelse st-1 = "left-edge"
        [set x1 left-edge]
        [set x1 st-1]
      ifelse st-2 = "right-edge"
        [set x2 right-edge]
        [set x2 st-2]

      set others-1 (count store with [address = x1])
      set others-2 (count store with [address = x2])
      if others-1 = 0 [set others-1 1]
      if others-2 = 0 [set others-2 1]

      ifelse st-1 = "left-edge"
         [set dist (abs (x2 - left-edge)) / others-2]
         [ifelse st-2 = "right-edge"
           [set dist (abs (right-edge - x1)) / others-1]
           [set dist abs (x2 - x1) / 2]
         ]

      set lst (list st-1 st-2)
      set dist-lst (lput (fput dist lst) dist-lst)
      set xcor-lst (but-first xcor-lst)
     ]
  ]
end

to-report check-boundary [x pos]
  locals [bound]
  set bound odd-even-boundary pos

  ifelse (pos = "right-edge")
    [ifelse x > bound
      [report bound]
      [report x]
    ]
    [ifelse x < bound
      [report bound]
      [report x]
    ]
end

to-report best-xcor [list-dist]
  locals [dist-struct tr-dist-struct x1 x2 n]

  set dist-struct (fput 0 (list 0 0))
  loop [
    ifelse (empty? list-dist)
      [set x1 (first (but-first dist-struct))
       set x2 (last dist-struct)
       ifelse (num-Stores mod 2 = 0)
        [ifelse x1 = "left-edge"
          [report (list (first dist-struct) (check-boundary x2 "left-edge"))]
          [ifelse x2 = "right-edge"
            [report (list (first dist-struct) (check-boundary x1 "right-edge"))]
            [report (list (first dist-struct) (odd-even-analysis x1 x2))]
          ]
        ]
        [ifelse x1 = "left-edge"
          [report (list (first dist-struct) (x2 - tol))]
          [ifelse x2 = "right-edge"
            [report (list (first dist-struct) (x1 + tol))]
            [report (list (first dist-struct) (odd-even-analysis x1 x2))]
          ]
        ]
      ]
      [set tr-dist-struct first list-dist
       set x1 (first (but-first tr-dist-struct))
       set x2 (last tr-dist-struct)
       ifelse num-Stores mod 2 = 1
         [set n (right-edge + 1 - (odd-even-boundary "right-edge"))]
         [set n (right-edge - (odd-even-boundary "right-edge") + 0.5)]

       ifelse ((x1 = "left-edge") or (x2 = "right-edge")) and (first tr-dist-struct > n)
         [set dist-struct tr-dist-struct
          set list-dist empty-list
         ]
         [ifelse (first tr-dist-struct) > (first dist-struct)
           [set dist-struct tr-dist-struct
            set list-dist (but-first list-dist)]
           [set list-dist (but-first list-dist)]
         ]
      ]
  ]
end

to-report odd-even-analysis [x1 x2]
  ifelse (num-Stores mod 2) = 0
    [ifelse one-edge?
      [report go-toward-edge]
      [report (x1 + ((x2 - x1) / 2))]
    ]
    [report (x1 + ((x2 - x1) / 2))]
end


to-report all-finished?
  ifelse (count customer with [finished?]) = customer-pop
    [report true]
    [report false]
end

to plot-store-graphs
  locals [i mkt-share-lst]
  set i 0
  set mkt-share-lst empty-list
  repeat num-Stores [
    ask store [
      if who mod num-Stores = i [
        repeat (mkt-share * 100) [
          set mkt-share-lst (lput (who mod num-Stores)  mkt-share-lst)
        ]
      ]
    ]
    set i i + 1
  ]
  set-current-plot "Mkt-Share"
  set-plot-pen-color green
  histogram-list mkt-share-lst
end

to plot-cust-graphs
  locals [avg-util]
  set avg-util (sum (values-from customer [utility])) / customer-pop
  set-current-plot "Consumer Utility"
  set-current-plot-pen "Utility"
  plot avg-util
end

to setup-plot
  locals [i]
  set i 0
  clear-all-plots
  set-current-plot "Mkt-Share"
  set-histogram-num-bars num-Stores
  set-plot-x-range 0 num-Stores
  set-plot-y-range 0 60
  set-current-plot-pen "Pareto-share"
  repeat num-Stores + 1  [plotxy i  100 / num-Stores
                        set i i + 1]
  set-current-plot-pen "Mkt-share"
  set-plot-pen-color green


  set-current-plot "Consumer Utility"
  set-current-plot-pen "pareto-util"
  repeat plot-x-max  [
    plot 1
  ]
  set-current-plot-pen "utility"
  plot 0.5
end


to setup
  ca
  ; set global variables to appropriate values
  set res-street (- screen-edge-y) + 1
  set store-street screen-edge-y - 4
  set arrow-lane store-street + 3
  set measure-lane store-street + 2
  set customer-pop screen-size-x * num-Customers/House
  set smoothness 10
  set max-ad-reach 10
  set buffer 1
  set time 0
  set wlk-buff .75 * (screen-edge-x)
  set store-address-list (list (- screen-edge-x) (screen-edge-x))
  set tol 1
  set left-edge (- screen-edge-x)
  set right-edge screen-edge-x
  set screen-size (right-edge - left-edge + 1)
  set last-store-pos num-Stores
  set n-active-stores num-Stores
  set counter 0
  set temp-list store-address-list
  set sand-color rgb 0.9 0.9 0.5

  ; call other procedures to set up various parts of the world
  setup-patches
  setup-stores
  ask store [stamp-label]
  setup-houses
  setup-customers
  setup-lst
  setup-store-data
  setup-plot
  if (Show-Pareto-Efficiency?) [setup-arrows]
end

to-report round-num [n]
 report (round (n * 10 ^ 3)) / 10 ^ 3
end

to setup-store-data
  ask store [calculations-special]
end

to setup-lst
  set store-address-list (sort store-address-list)
  set store-address-list (fput "left-edge" (but-first store-address-list))
  set store-address-list (lput "right-edge" (but-last store-address-list))

  set temp-list (sort temp-list)
  set temp-list (fput "left-edge" (but-first temp-list))
  set temp-list (lput "right-edge" (but-last temp-list))
end

;;turtle procedure
to go-shopping
  ifelse finish-shopping?
    [return-home]
    [find-store]
end

to stamp-label
  locals [dist my-add]
  set my-add (round-num address)
  set dist measure-lane - store-street
  ask patch-at 0 dist [set plabel my-add]
end

;;turtle procedure
to find-store
  if (start-time = 0) [set start-time counter]
  if hidden? [st]
  ifelse (not store-found?)
    [walk-randomly]
    [set finish-shopping? true
     set my-store color-of (random-one-of store-here)
     set num-cust num-cust + 1
     ht
     do-calc
    ]
end


to do-calc
  set end-time counter
  set my-time end-time - start-time
  ifelse pareto-util = 0
    [set my-best-time my-time
     set utility 1]
    [set utility ((my-best-time - my-time) / my-best-time) + 1]
  set start-time 0
end

;;turtle procedure
to return-home
  if not finished? [
    ifelse hidden?
      [set color red
       showturtle]
    [
      ifelse ((xcor <= home-x + buffer and xcor >= home-x - buffer) and
               (ycor <= home-y + buffer and ycor >= home-y - buffer))
        [setxy home-x home-y
         ht
         set finished? true
         set color-of (one-of house-here) my-store
        ]
        [set heading towardsxy-nowrap home-x home-y
         fd 1]
     ]
  ]
end

to-report store-found?
  ifelse (own-store?)
    [report true]
    [report false]
end


;; turtle procedure
to walk-randomly
  set heading uphill advertisement
  if (advertisement-of (patch-ahead 1) = 0)[
    ifelse  latitude = "toward-north" or latitude = "south-end"
      [ifelse longitude = "east-end"
        [set heading (random-int-or-float 45) - 67.5]
        [ifelse longitude = "west-end"
          [set heading random-int-or-float 45 + 22.5]
          [ifelse store-street?
            [if (xcor > screen-edge-x - wlk-buff) [set heading 270]
             if (xcor < ((- screen-edge-x) + wlk-buff)) [set heading 90]
            ]
            [set heading (random-int-or-float 90) - 45]
          ]
        ]
      ]
      [if latitude = "toward-south"  or latitude = "north-end"
        [ifelse (longitude-of (patch-ahead 1) = "west-end") or longitude = "east-end"
          [set heading (random-int-or-float 45) - 135]
          [ifelse (longitude-of (patch-ahead 1) = "east-end") or longitude = "west-end"
            [set heading (random-int-or-float 45) + 90]
            [ifelse store-street?
              [if (xcor > screen-edge-x - wlk-buff) [set heading 270]
               if (xcor < ((- screen-edge-x) + wlk-buff)) [set heading 90]
              ]
              [set heading (random-int-or-float 90) + 135]
            ]
          ]
        ]
      ]
  ]
  fd 1
end



to setup-stores
  set-default-shape store "hotdog_stand"
  create-custom-store num-Stores [
    set color 25 + (140 / num-Stores) * (who mod num-Stores)
    set ycor store-street
    ifelse counter = 0
      [Pareto-Efficient-xcor]
      [setxy  ((- screen-edge-x) + 1) + (random-int-or-float (screen-size-x - 2)) store-street]
    set advertisement max-ad-reach
    set own-store? true
    set address xcor
    set store-address-list (fput address store-address-list)
    set temp-list (fput address temp-list)
    set curr-mkt-share 0
    set at-eq? false
    set set? false
    set goal address
  ]
  advertise
end


to advertise
  ask patches with [advertisement > 0] [set old-ad advertisement]
  repeat smoothness [my-diffuse 1]
  if show-advertisement?
   [ask patches with [advertisement != old-ad] [set pcolor scale-color green advertisement -1.5 8]]
end

to my-diffuse [percentage]
  locals [my-ad]
  ask patches with [advertisement > 0][
    set my-ad 1 / 8 * percentage * advertisement
    ask patch-at 1 0 [if (not ad-boundary?) [set advertisement advertisement + my-ad]]
    ask patch-at 1 1 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
    ask patch-at 0 1 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
    ask patch-at -1 1 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
    ask patch-at -1 0 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
    ask patch-at -1 -1 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
    ask patch-at 0 -1 [if (not ad-boundary?) [set advertisement advertisement + my-ad]]
    ask patch-at 1 -1 [if (not ad-boundary?)[set advertisement advertisement + my-ad]]
  ]
end



to-report ad-boundary?
  ifelse (latitude = "north-end" or
          latitude = "south-end" or
          longitude = "east-end" or
          longitude = "west-end" or
          pycor <= res-street + 1 or
          pycor >= screen-edge-y - 1)
  [report true]
  [report false]
end

to setup-patches
  ask patches [
    set advertisement 0
    set own-store? false
    set latitude "steppable"
    set longitude "steppable"
    set store-street? false
    set num-cust 0
    set plabel-color black
    set pcolor sand-color

    if (pycor >= res-street and pycor <= store-street) [set latitude "toward-north"]
    if (pycor > store-street) [set latitude "toward-south"]

    if (pycor = screen-edge-y) [set latitude "north-end"]
    if (pycor = res-street) [set latitude "south-end"]
    if (pxcor = screen-edge-x) [set longitude "east-end"]
    if (pxcor = (- screen-edge-x)) [set longitude "west-end"]

    if (pycor < res-street) [set latitude "unsteppable"
                             set longitude "unsteppable"]
  ;  if (pycor = measure-lane) [set plabel pxcor]
  ]
  ask patches with [(pycor <= store-street + 1) and
                    (pycor >= store-street - 1)]
                   [set pcolor gray
                    set store-street? true]
end

to setup-houses
  set-default-shape house "umbrella"
  create-custom-house screen-size-x [
    setxy who res-street
    set color red
    ifelse xcor > 0
      [set heading 30]
      [ifelse xcor < 0
        [set heading -30]
        [set heading 0]
      ]
  ]
end

to setup-customers
  set-default-shape customer "person"
  create-custom-customer customer-pop [
    set heading 0
    setxy who res-street
    set home-x xcor
    set home-y ycor
    set finish-shopping? false
    set finished? false
    set color green
    ht
    set end-time 0
   set start-time 0
;   if home-x = (- screen-edge-x) [set b who]
  ]
end

to setup-arrows
  create-custom-arrow num-Stores [
    set color red
    set shape "arrow"
    set heading 180
    set ycor arrow-lane
    Pareto-Efficient-xcor
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
12
10
832
239
22
5
18.0
1
10
1
1
1
0

CC-WINDOW
5
414
841
509
Command Center

PLOT
630
244
828
400
Mkt-Share
Store#
Mkt-Share
0.0
5.0
0.0
60.0
true
false
PENS
"Pareto-share" 1.0 0 -65536 true
"Mkt-share" 1.0 1 -11352576 true

PLOT
425
244
623
400
Consumer Utility
Time
Utility
0.0
50.0
-1.0
2.0
true
false
PENS
"Utility" 1.0 0 -16777216 true
"Pareto-Util" 1.0 0 -65536 true

SWITCH
9
334
207
367
Show-Pareto-Efficiency?
Show-Pareto-Efficiency?
0
1
-1000

BUTTON
13
251
94
284
Setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
102
251
183
284
Go
go
T
1
T
OBSERVER
T
NIL

SWITCH
9
296
207
329
Show-Advertisement?
Show-Advertisement?
1
1
-1000

SLIDER
214
296
412
329
num-Customers/House
num-Customers/House
1
10
1
1
1
NIL

SLIDER
214
334
412
367
num-Stores
num-Stores
2
5
5
1
1
NIL

@#$#@#$#@



to-report analyze-x [dist]
  locals [my-pos left-pos right-pos lf-dist rt-dist]
  set my-pos (position address store-address-list)
  set left-pos my-pos - 1
  set right-pos my-pos + 1

  ifelse (curr-mkt-share < 1 / num-Stores)
    [ifelse two-edge? or (zero-edge? and not (am-edge? address) )
      [set lf-dist abs ((- screen-edge-x) - (item left-pos store-address-list))
       set rt-dist abs (screen-edge-x - (item right-pos store-address-list))
       ifelse (lf-dist > 1 / (2 * num-Stores) or rt-dist > 1 / (2 * num-Stores))
         [ifelse lf-dist > rt-dist
           [show (item left-pos store-address-list) - tol set at-eq? true report (item left-pos store-address-list) - tol]
           [show (item right-pos store-address-list) + tol set at-eq? true report (item right-pos store-address-list) + tol]
         ][report dist]
      ]
      [ifelse one-edge?
        [report go-toward-edge]
        [report (last dist)]
      ]
    ] [report (last dist)]
end


to defend-x
  locals [
    store-left-add
    store-right-add
    my-pos
    left-dist
    right-dist
  ]

  set my-pos (position address store-address-list)
  if my-pos = 1 [set my-pos "left-edge"]
  if my-pos = num-Stores [set my-pos "right-edge"]

  ifelse my-pos = "left-edge"  or my-pos = "right-edge"
    [if my-pos = "left-edge" [set store-left-add (- screen-edge-x)
                              set store-right-add (item 2 store-address-list)]
     if my-pos = "right-edge" [set store-right-add screen-edge-x
                               set store-left-add (item (num-Stores - 1) store-address-list)]
    ]
    [set store-left-add  item (my-pos - 1) store-address-list
     set store-right-add item (my-pos + 1) store-address-list
    ]

  set left-dist abs (store-left-add - address)
  set right-dist abs (store-right-add - address)

  ifelse (my-pos = "left-edge")
    [if (curr-mkt-share >= (1 / num-Stores + tol / (2 * screen-size-x)) or (count other-store-here > 0))
        and (address > ((- screen-edge-x) + ((1 / (2 * num-Stores)) * screen-size-x) + tol))
       [set address address - tol
        set xcor address]
    ]
    [ifelse (my-pos = "right-edge")
      [if (curr-mkt-share >= (1 / num-Stores + tol / (2 * screen-size-x))) or (count other-store-here > 0)
        and (address < (screen-edge-x - ((1 / (2 * num-Stores)) * screen-size-x) - tol))
         [set address address + tol
          set xcor address]
      ]
      [ifelse (abs (left-dist - right-dist)) <= tol
         [set address (store-left-add + store-right-add) / 2]
         [ifelse (left-dist > right-dist)
           [set address address - tol
            set xcor address]
           [if (right-dist > left-dist)
              [set address address + tol
               set xcor address
              ]
           ]
        ]
      ]
    ]
end


to move-next
  locals [list-dist lst dist-struct other-store poss-share bad poss]
  set lst empty-list
  set list-dist (lst-dist remove-me-list lst)
  set dist-struct best-xcor list-dist

  set other-store (count store with [xcor = (last dist-struct) and ycor = store-street]) + 1
  set poss-share (((first dist-struct)/ other-store) * avg-cust/address / customer-pop)
  ;set poss ((first dist-struct)/ other-store)

type "poss-share"  show poss-share
type "curr-share"  show curr-mkt-share
  if (poss-share  >= curr-mkt-share)
    [ifelse (position address store-address-list) = 1
         or (position address store-address-list) = num-Stores
       [if (curr-mkt-share < 1 / num-Stores)
         [set address (last dist-struct)
          set xcor address]
       ]
       [ifelse (position address store-address-list = 2) and
               address >= ((- screen-size-x) + (screen-size-x / num-Stores) + tol) and
               address >= (screen-size-x / num-stores)
          [ifelse  abs ((- screen-edge-x) - (last dist-struct)) <= (screen-size-x / num-Stores + tol)
             [set address (last dist-struct)
              set xcor address]
             [set address address - tol
              set xcor address]
          ]
          [ifelse (position address store-address-list = (num-Stores - 1)) and
                  abs (screen-edge-x - address) <= (screen-size-x / num-Stores + tol) and
                  abs (screen-edge-x - address) >= (screen-size-x / num-stores)
            [ifelse  abs (screen-edge-x - (last dist-struct)) <= (screen-size-x / num-Stores + tol)
               [set address (last dist-struct)
                set xcor address]
               [set address address + tol
                set xcor address]
            ]
            [set address (last dist-struct)
             set xcor address]
          ]
       ]
    ]
end

to-report backlash [dist-struct poss-share]
  locals [x]
   set x (last dist-struct)
   type "x" show x
   type "avg/custpop" show avg-cust/address / customer-pop
   if x <= (item 1 store-address-list)
     [report poss-share - (abs ((- screen-edge-x) - (x - tol)))]
   if x >= (item num-Stores store-address-list)
     [report poss-share - (abs (screen-edge-x - (x + tol)))]
   report screen-size-x
end

to move
  locals [rht-mkt-share lft-mkt-share]

  set rht-mkt-share ((calc-mkt-length 1) * num-Customers/House) / customer-pop
  set lft-mkt-share ((calc-mkt-length -1) * num-Customers/House) / customer-pop

  if (curr-mkt-share < rht-mkt-share) or (curr-mkt-share < lft-mkt-share)
    [ifelse (rht-mkt-share > lft-mkt-share)
       [set xcor xcor + 1]
       [ifelse (lft-mkt-share > rht-mkt-share)
          [set xcor xcor - 1]
          [ifelse (random 2) mod 2 = 0 ;;should both shares be equal
             [set xcor xcor + 1]
             [set xcor xcor - 1]
          ]
       ]
    ]
end

to-report calc-mkt-length [dxcor]
  locals [
    new-xcor
    pos-in-lst
    rht-pt
    lft-pt
    right-store-xcor
    left-store-xcor
    num-competitors
    just-side
    x-comp
    num-stores-here
    backlash
    max-mkt-length
  ]

  ;calculates right-store-xcor
  set pos-in-lst (position address store-address-list)

  ifelse pos-in-lst = ((length store-address-list) - 1)
   [set right-store-xcor "right-edge"]
   [set right-store-xcor (item (pos-in-lst + 1) store-address-list)]



  ;calculate left-store-xcor
  ifelse  pos-in-lst = 0
    [set left-store-xcor "left-edge"]
    [set left-store-xcor (item (pos-in-lst - 1) store-address-list)]



  ;calc-rht-mkt-length
  set new-xcor address + dxcor * tol

  ifelse (right-store-xcor = "right-edge") and (left-store-xcor = "left-edge")
    [set rht-pt abs (right-store-xcor - new-xcor)/ 2
     set lft-pt abs (left-store-xcor - new-xcor) / 2]
    [ifelse (right-store-xcor = "right-edge")
      [set rht-pt (abs (screen-edge-x - new-xcor)) + 1]
      [set rht-pt ((abs (right-store-xcor - new-xcor)) - 1)/ 2]

     ifelse (left-store-xcor = "left-edge")
       [set lft-pt (abs ((- screen-edge-x) - new-xcor)) + 1]
       [set lft-pt ((abs (left-store-xcor - new-xcor)) - 1) / 2]
    ]

  ;type "lft-pt"  show lft-pt
  ;type "rht-pt" show rht-pt

  ifelse (dxcor != 0)
    [set num-stores-here (count store-at dxcor 0) + 1]
    [set num-stores-here 1]

  set just-side (lft-pt + rht-pt) / num-stores-here
  set x-comp 0
  ;type "store-at"  show num-stores-here

  set pos-in-lst (position xcor store-address-list)

  ifelse not ((left-store-xcor = "left-edge") and (right-store-xcor = "right-edge"))
    [if dxcor = 1
      [if right-store-xcor != "right-edge"
        [set new-xcor (right-store-xcor + tol)
         set pos-in-lst (pos-in-lst + 1)
         ifelse pos-in-lst = ((length store-address-list) - 1)
           [set x-comp abs (screen-edge-x - new-xcor) + tol]
           [set x-comp (abs ((item (pos-in-lst + 1) store-address-list) - new-xcor) + 1) / 2]
        ]
      ]

     if dxcor = -1
      [if left-store-xcor != "left-edge"
        [set new-xcor (item (pos-in-lst - 1) store-address-list) - 1
         set pos-in-lst (pos-in-lst - 1)
         ifelse pos-in-lst = 0
           [set x-comp abs ((- screen-edge-x) - new-xcor) + 1]
           [set x-comp (abs ((item (pos-in-lst - 1) store-address-list) - new-xcor) + 1) / 2]
        ]
      ]
    ]
    [if (count store-here) > 1 [
       if dxcor = 1 [
         set new-xcor address + tol
         set x-comp abs (screen-edge-x - new-xcor + 1)
       ]
       if dxcor = -1 [
         set new-xcor address - tol
         set x-comp ((abs ((- screen-edge-x) - new-xcor)) + 1)
       ]
     ]
    ]

  set backlash calc-backlash dxcor
  set x-comp (x-comp / ((count store-at (new-xcor - xcor) 0) + 1))
  set max-mkt-length max (list just-side x-comp)
  ifelse backlash > max-mkt-length
    [report max-mkt-length]
    [report 0]
end


to-report calc-backlash [dxcor]
  locals [
    pos
    new-xcor
    right-sd
    left-sd
  ]

  set left-sd screen-size-x
  set right-sd screen-size-x

  set pos (position address store-address-list) + 2 * dxcor
  set new-xcor xcor + dxcor * tol

  ifelse pos = -1
    [set right-sd ((abs ((- screen-edge-x) - new-xcor)) + tol)]
    [ifelse pos = length store-address-list
      [set right-sd ((abs (screen-edge-x - new-xcor)) + tol)]
      [if pos >= 0 and pos < (length store-address-list) [
          set right-sd ((abs (new-xcor - (item pos store-address-list))) + 1) / 2]
      ]
    ]

  set pos (position address store-address-list) - 4 * dxcor
  ifelse pos = -1
    [set left-sd ((abs ((- screen-edge-x) - new-xcor)) + 1)]
    [ifelse pos = length store-address-list
      [set left-sd ((abs (screen-edge-x - new-xcor)) + 1)]
      [if pos >= 0 and pos < (length store-address-list) [
          set left-sd ((abs (new-xcor - (item pos store-address-list))) + 1) / 2]
      ]
    ]

  report min (list left-sd right-sd)
end


to do-calculations
  locals [my-mkt-length]
  set my-mkt-length calc-mkt-length 0
  if my-mkt-length = 0 [set my-mkt-length 1]
  set number-customers num-cust
  set curr-mkt-share (num-cust / (count store-here))/ customer-pop
  set avg-cust/address (num-cust / my-mkt-length)
end
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

hotdog_stand
false
0
Rectangle -7566196 true true 61 105 279 224
Circle -16777216 true false 68 222 55
Circle -16777216 true false 192 220 51
Polygon -16777216 true false 33 78 60 122 76 104 43 47
Rectangle -16777216 true false 79 127 81 162
Rectangle -16777216 true false 81 144 92 147
Rectangle -16777216 true false 92 127 95 161
Rectangle -16777216 true false 104 128 106 164
Rectangle -16777216 true false 105 128 122 131
Rectangle -16777216 true false 118 130 121 162
Rectangle -16777216 true false 106 164 116 164
Rectangle -16777216 true false 105 160 120 163
Rectangle -16777216 true false 125 128 149 131
Rectangle -16777216 true false 134 131 138 162
Rectangle -16777216 true false 156 127 159 161
Rectangle -16777216 true false 159 127 169 129
Line -16777216 false 169 127 180 136
Rectangle -16777216 true false 177 136 180 160
Rectangle -16777216 true false 159 154 179 161
Rectangle -16777216 true false 187 129 190 163
Rectangle -16777216 true false 189 128 206 130
Rectangle -16777216 true false 206 128 209 164
Rectangle -16777216 true false 187 161 208 163
Rectangle -16777216 true false 216 126 242 128
Rectangle -16777216 true false 217 126 220 163
Rectangle -16777216 true false 220 158 241 163
Rectangle -16777216 true false 230 140 250 145
Rectangle -16777216 true false 237 144 240 159
Circle -1 true false 82 237 24
Circle -1 true false 205 234 22

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

umbrella
true
0
Polygon -7566196 true true 0 150 0 135 3 119 8 105 15 85 21 74 29 59 46 44 61 30 85 14 105 7 121 2 134 1 165 1 179 3 214 15 239 30 268 59 284 86 295 114 298 136 298 149 230 119 151 151 80 120
Polygon -1 true false 81 120 149 0 230 119 151 150
Polygon -6524078 true false 143 147 143 286 158 293 159 147

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
