; add model procedures here

globals [
  skills
  nodes-color
  activations-color
  patches-color
  hypercycle-links-color
  cycle-count
  node-count
  cycles
  ticks
  loose-cycles ]


breeds [ nodes activations searchers hypercycle-links]

nodes-own [ skillset ]

searchers-own [ cycle tskill ]



to startup
  set skills (list 1 2 3 4 5)
  set-default-shape nodes "square"
  set nodes-color black
  set-default-shape activations "activation"
  set activations-color red
  set-default-shape hypercycle-links "line"
  set hypercycle-links-color green
  set patches-color orange - 1
  set loose-cycles false
end

to setup
  cp
  ct
  cp
  clear-plot
  set ticks 0
  ask patches [ set pcolor patches-color]
  repeat total-skills / length skills [ foreach skills [ add-skill ? ] ]
  repeat total-skills - ( ( total-skills / length skills) * length skills ) [ add-skill random-one-of skills ]
  set node-count count nodes
end

to go
  if not any? nodes or count nodes = 1 [ stop ]
  clear-dead-nodes
  play
  cycle-search
  do-plot
  set ticks ticks + 1
end

to do-plot
  set-current-plot "HyperCycles"
  set-current-plot-pen "cycles"
  plot cycle-count
  set-current-plot-pen "nodes"
  plot node-count
  set-current-plot-pen "non-cycle-nodes"
  plot count nodes with [ not any? hypercycle-links-here ]
  set-current-plot-pen "cycle-nodes"
  plot count nodes with [ any? hypercycle-links-here ]
end

to initialize-node
  set heading 0
  set color nodes-color
  set skillset (list)
end

to link [ target ]
  if not any? hypercycle-links-here with [ heading = towards target ]
    [ sprout-hypercycle-links 1
      [ set color hypercycle-links-color
        set label no-label
        set size distance target
        set heading towards target fd .5 ]
    ]
end

to activate-link [ target ]
  if not any? activations-here with [ heading = towards target ]
    [ hatch-activations 1
      [ set color activations-color
        set label no-label
        set heading towards target fd .5 ]
    ]
end

to-report activate [ target target-skill ]
  ifelse member? target-skill value-from target [ skillset ]
    [ report target ]
    [ report false ]
end

to-report node-neighborhood
  if ntype = "Moore" [ report neighbors with [ distance myself = distance-nowrap myself ] ]
  if ntype = "Von Neumann" [ report neighbors4 with [ distance myself = distance-nowrap myself ] ]
end

to clear-dead-nodes
  ask nodes with [ length skillset = 0 ]
    [ set node-count node-count - 1
      ask activations-here [ die ]
      die ]
end

to add-skill [ new-skill ]
  let location random-one-of patches
  ifelse ( not any? nodes-on location )
    [ ask location [ sprout-nodes 1 [ initialize-node learn-skill new-skill ] ] ]
    [ ask nodes-on location [ learn-skill new-skill ] ]
end

to update-node-color
  if show-skill-density?
    [ set color scale-color violet length skillset ( total-skills / count nodes ) 0  ]
end

to learn-skill [ skill ]
  set skillset fput skill skillset
  update-node-color
  ifelse node-label?
    [ set label sort remove-duplicates skillset ]
    [ set label no-label ]
end

to play
  ask activations [ die ]
  let orig random-one-of nodes with [ length skillset > 0 ]
  play-node orig random-one-of value-from orig [ skillset ] 0
end

to play-node [ orig orig-skill activation-count ]
  let target-skill next-skill orig-skill
  let target value-from random-one-of neighbor-nodes orig [ one-of nodes-here ]
  if is-turtle? target
  [ let activated activate target target-skill
    if is-turtle? activated
      [ ask orig [ activate-link activated ]
        propogate-skill orig target orig-skill target-skill
        if continue-activations? and activation-count < length skills [ play-node target target-skill activation-count + 1 ] ]
  ]
end

to propogate-skill [ source target source-skill target-skill ]
  if ( mode = "Source" )
    [ ask source [ learn-skill source-skill ]
      kill-skill
     ]
  if ( mode = "Target" )
    [ ask target [ learn-skill target-skill ]
      kill-skill
     ]
  if (mode = "Joint" )
    [ ask target [ learn-skill target-skill ]
      ask source [ learn-skill source-skill ]
      repeat 2 [ kill-skill ]
    ]
  if ( mode = "Exchange" )
    [ ask target [ learn-skill source-skill ]
      ask source [ learn-skill target-skill ]
      repeat 2 [ kill-skill ]
    ]
end

to kill-skill
  let corpse random-one-of nodes with [ length skillset > 0 ]
  ask corpse [ set skillset remove-item ( random (length skillset - 1 ) ) skillset
               update-node-color ]
end

to-report neighbor-nodes [ orig ]
  report value-from orig [ node-neighborhood ]
end

to-report next-skill [ orig-skill]
  let posit position orig-skill skills
  ifelse posit = ( length skills) - 1
    [ report item 0 skills ]
    [ report item ( posit + 1 ) skills ]
end

to cycle-search
  set cycle-count 0
  set cycles (list)
  ask nodes with [ member? first skills skillset ]
    [ ask patch-here
      [ sprout-searchers 1
        [ ht
          set tskill next-skill first skills
          set cycle (list myself )
          search-for-cycles
          die
        ]
      ]
    ]
  no-display
  ask hypercycle-links [ die ]
  if show-cycles? [ foreach cycles [ display-hypercycle ? ] ]
  display
end

to display-hypercycle [ hypercycle ]
  ;; show hypercycle
  if length hypercycle > 1
    [ ask first hypercycle [ link first but-first hypercycle ]
      display-hypercycle but-first hypercycle ]
end

to search-for-cycles
  let current-target-skill tskill
  let current-cycle cycle

  ifelse current-target-skill = first skills
    [
      if ( ( loose-cycles and member? patch-here cycle ) or
           ( last current-cycle = patch-here ) )
      [ set cycle-count cycle-count + 1
        ;set cycle fput patch-here cycle
        if show-cycles? [ set cycles fput cycle cycles ]
        die
      ]
    ]
    [ let next values-from node-neighborhood [ one-of nodes-here with [ member? current-target-skill skillset ] ]
      foreach next
      [
        if is-turtle? ?
          [
            hatch-searchers 1 [ set heading towards ?
                                fd distance ?
                                set tskill next-skill current-target-skill
                                set cycle fput value-from ? [ patch-here ] cycle
                                search-for-cycles
                                die
                              ]
          ]
      ]
    ]
end

@#$#@#$#@
GRAPHICS-WINDOW
483
18
933
489
5
5
40.0
1
10
1
1
1
0

CC-WINDOW
5
531
942
626
Command Center

BUTTON
11
10
80
43
NIL
setup
NIL
1
T
OBSERVER
T
S

BUTTON
87
10
150
43
NIL
go
T
1
T
OBSERVER
T
G

CHOOSER
13
50
155
95
ntype
ntype
"Moore" "Von Neumann"
1

CHOOSER
15
100
153
145
mode
mode
"Source" "Target" "Joint" "Exchange"
3

SWITCH
169
94
312
127
show-cycles?
show-cycles?
0
1
-1000

SLIDER
169
50
350
83
total-skills
total-skills
0
10000
500
100
1
skills

BUTTON
158
11
221
44
Step
go
NIL
1
T
OBSERVER
T
N

MONITOR
26
173
175
222
Current HyperCycles
cycle-count
0
1

PLOT
2
240
440
478
HyperCycles
time
cycles
0.0
1000.0
0.0
10.0
true
true
PENS
"cycles" 1.0 0 -16777216 true
"nodes" 1.0 0 -65536 true
"non-cycle-nodes" 1.0 0 -16776961 true
"cycle-nodes" 1.0 0 -11352576 true

MONITOR
190
173
247
222
NIL
ticks
0
1

SWITCH
168
133
304
166
node-label?
node-label?
1
1
-1000

SWITCH
205
483
400
516
continue-activations?
continue-activations?
1
1
-1000

SWITCH
10
484
195
517
show-skill-density?
show-skill-density?
0
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

activation
true
0
Line -7566196 true 150 0 150 300
Polygon -7566196 true true 120 45 150 0 180 45 120 45

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
