globals
[
  time
  antibody-lifespan  ;; holds how long an antibody lives

  ;; holds the number of times an antigen hits any of the patches with
  ;; lymph-node? true.  the higher the number the faster b-cells are produced
  num-antigen-lymph-node-interactions

  ;; holds the time when an antigen last hit a patch with lymph-node? true
  last-antigen-lymph-node-interaction
]

patches-own
[
  lymph-node?  ;; the area of the patches that produce b-cells
]

breeds
[
  antigen  ;; the infecting cells
  b-cells  ;; cells that create antibodies and other b-cells
  antibodies  ;; proteins that destroy the antigen
]

antibodies-own
[
  time-born  ;; the time it was created
]

;; setup the model.  create some antigen for the infection
;; and some bone marrow to release b-cells
to setup
  ca

  ;; set the shape of the turtles
  set-default-shape antigen "circle"
  set-default-shape b-cells "circle"
  set-default-shape antibodies "circle"

  set time 0
  set num-antigen-lymph-node-interactions 0
  set last-antigen-lymph-node-interaction 0
  set antibody-lifespan 500

  ;; make some bone marrow to produce b-cells
  ask patches
  [
    set lymph-node? ((distancexy 0 0) < 4)
    if lymph-node?
    [ set pcolor violet ]
  ]

  ;; add some antigen cells
  infect
end

;; create some antigen to be floating around
to infect
  create-custom-antigen init-num-antigen
  [
    set color red
    while [ lymph-node? ]
    [ setxy (random-float screen-size-x)
            (random-float screen-size-y) ]
  ]
end

;; run the simulation
to go
  ask turtles
  [
    move
    meet
  ]
  release-b-cells

  ;; remove old antibodies
  ask antibodies with [ antibody-lifespan + time-born < time ]
  [ die ]

  do-plotting  ;; plot the amounts of the cells
  set time time + 1
end

;; have the turtles move around
to move ;; turtle procedure
  wiggle
  fd 1
end

;; turn some
to wiggle  ;; turtle procedure
  rt (random-float 100)
  lt (random-float 100)
end

;; have the cells interact
to meet  ;; turtle procedure
  ifelse (breed = antibodies) and (any? antigen-here)
  [
    ;; if an antibody meets a antigen, they both die
    without-interruption
    [
      ask one-of antigen-here  ;; 1 of the antigen here dies
      [ die ]
      die  ;; the antibody dies as well
    ]
  ]
  [
    if (breed = b-cells) and (any? antigen-here)
    [
      ;; if a b-cells meets a antigen, they both die and the b-cell
      ;; creates some more b-cells and lot of antibodies
      without-interruption
      [
        hatch random 2  ;; create some more b-cells
        []
        hatch random (clone-rate * 10)  ;; create some antibodies
        [
          set breed antibodies
          set color green
          set time-born time
        ]
        ask one-of antigen-here  ;; 1 of the antigen here dies
        [ die ]
        die  ;; the b-cell dies as well
      ]
    ]
  ]
end

;; have the patches with lymph-node? true create and release some mature
;; b-cells, that is b-cells that are attuned to this specific antigen
to release-b-cells
  if any? antigen with [ lymph-node? ]
  [
    set num-antigen-lymph-node-interactions num-antigen-lymph-node-interactions + 1
    set last-antigen-lymph-node-interaction time
  ]

  ;; if there hasn't been antigen-lymph node interaction in a while slow down
  ;; the rate at which the b-cells are created by the lymph node
  if time - last-antigen-lymph-node-interaction > 10 and num-antigen-lymph-node-interactions > 0
  [ set num-antigen-lymph-node-interactions num-antigen-lymph-node-interactions - 1 ]

  ;; if there have been no antigen-lymph-node interactions, then no b-cells
  ;; should be released
  if num-antigen-lymph-node-interactions <= 0
  [ stop ]
  if time mod ceiling (100 / num-antigen-lymph-node-interactions) = 0
  [ create-new-b-cell ]
end

;; have one of the patches with lymph-node? true create a b-cell
to create-new-b-cell
  ask one-of patches with [ lymph-node? ]
  [
    sprout 1
    [
      set color yellow
      setxy 0 0
      set breed b-cells
    ]
  ]
end

;; plot the amounts of the cells
to do-plotting
  set-current-plot-pen "b-cells"
  plot count b-cells
  set-current-plot-pen "antibodies"
  plot count antibodies
  set-current-plot-pen "antigen"
  plot count antigen
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2002.  Updated 2002, 2003.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Immunity model.
; http://ccl.northwestern.edu/netlogo/models/Immunity.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Immunity
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
266
10
560
325
35
35
4.0
1
10
1
1
1
0

CC-WINDOW
5
512
569
607
Command Center

BUTTON
30
29
114
72
Setup
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
28
110
241
143
clone-rate
clone-rate
0
5
2
1
1
NIL

BUTTON
151
29
241
72
Go
go
T
1
T
OBSERVER
T
NIL

MONITOR
77
197
193
246
Num Antibodies
count antibodies
3
1

MONITOR
135
147
241
196
Num B-cells
count b-cells
3
1

MONITOR
28
147
134
196
Num Antigen
count antigen
3
1

BUTTON
68
251
202
284
Introduce Antigen
infect
NIL
1
T
OBSERVER
T
NIL

PLOT
2
290
261
498
Cell Amounts
time
number
0.0
50.0
0.0
200.0
true
true
PENS
"b-cells" 1.0 0 -9472 true
"antibodies" 1.0 0 -11352576 true
"antigen" 1.0 0 -65536 true

SLIDER
28
75
241
108
init-num-antigen
init-num-antigen
0
500
200
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
The human immune system is very complex but yet affective in fighting off foreign cells.  Our immune system is setup to handle all different kinds of attacks to our cells.  This model will be looking at one of the many ways that our immune system handles these attacks.  When a foreign body, also known as an antigen, enters tissue or the blood stream a series of chain reactions start to occur.  Our body trys to fight off the invader by sending a phagocyte (a type of white blood cell) to engulf and digest the foreign cell.  If the foreign cell or antigen resists this general attack response, then our immune system moves into a more direct type of action. This next level of attack is called Antibody-Mediated Immune Response.  In this attack, B-lymphocytes (B-cells) which are found in the lymphatic system and the blood stream connect to the antigen (for example, bacteria or virus). When this connection occurs three reactions result. First the B-cell destroys the antigen.  The connection with the antigen then triggers the B-cell to start cellular division and thus many more B-cells are produced.  During this production, short lived plasma cells will arise which produce 2,000 antibodies per second.  These plasma cells act as factories to produce the specific antibody that fights the antigen.  As the antibodies are released they randomly move around until they bump into an antigen.  The antibody then attaches to the antigen which will ultimately destroy it.

This model demonstrates the role of B-cells (yellow turtles) and antibodies (green turtles) play in fighting off antigens (red turtles) that are circulating in extracellular fluid or are attached to the surface of the host cells.


HOW TO USE IT
-------------
To initialize the model, press the SETUP button.  The SETUP button creates the display of randomly scattered antigens around a fixed purple object which representes the bonemarrow.  B-cells (yellow turtles) which help fight off antigens (red turtles) are manufactured in the bone marrow.  The INIT-NUM-ANTIGEN controls the initial number of antigens that is created when the SETUP button is pressed.  The PERCENTAGE-ANTIBODY-DEATH slider determines the likelyhood that an antibody might die even though it hasn't touched a antigen yet.  The CLONE-RATE slider determines how prolific a B-cell will be when it encounters a antigen cell; higher values will cause B-cells to be more prolific.  Press the GO button to run the model.  The RELEASE-B-CELLS button causes B cells to be released by the bone marrow to attack the antigen.  The CELL AMOUNTS plot shows the amounts of the different types of cells over time.  The three monitors, NUM ANTIGEN, NUM B-CELLS, and NUM ANTIBODIES, show the current amounts of antigen, B-cells, and antibodies.


THINGS TO NOTICE
----------------
Notice that the number of antibodies are extremely small at the beginning but, as the simulation continues, they slowly increase and then, all of a sudden, explode, and seem to be everywhere.  This phenomenon is indicative of the B-cells sensing the antigen and then cloning themselves into self-reproducing phagocytes.


THINGS TO TRY
-------------
What happens when you set the CLONE-RATE to zero?  How does this affect the antigens?

Play around with the sliders.  What happens when you set each slider to its maximum and minimum values?

What settings would be good for a quick immune response to an infection?


EXTENDING THE MODEL
-------------------
This model demonstrates the antibody-mediated immune response, which is a second level response.  Try adding a breed of turtles that act as phagocytes to observe how affective the general attack response is fighting antigens.

Not all antibodies destroy cells, some bind to antigens and flag them for distruction by phagocytes or other types of cells.  Make more then one type of antibody in this model.

Antibodies and mature b-cells are only attuned to a specific type of antigen.  Try adding in more types of antigens and have b-cells and antibodies only respond to one type.

Try adding in more than 1 lymph node.

Change the model to allow the antigens to reproduce.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Immunity model.  http://ccl.northwestern.edu/netlogo/models/Immunity.  Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Immunity for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
