;;;;;;;;;;;;;;;;;;;
;;;  VARIABLES  ;;;
;;;;;;;;;;;;;;;;;;;

globals
[ electric-field
  contact?
  trail-color
  streamer-color
  streamers-out?
  strike-id
  cloud-height
  ground-height
]

breeds
[ vapors         ;; electrically neutral water vapor
  positrons      ;; positively charged particles
  electrons      ;; negatively charged particles
]

turtles-own
[ location ]

patches-own
[ plocation
  plasma-count   ;; used to delay transformation of plasma back into air
  plasma-id      ;; number of electron that 'burned' through here
  resistance     ;; 0 to 1 scale of how difficult it is ionize this air
]

;;;;;;;;;;;;;;
;;;  MAIN  ;;;
;;;;;;;;;;;;;;

to setup
  ca
  set cloud-height screen-edge-y - 8
  set ground-height 8 - screen-edge-y
  set-default-shape turtles "circle"
  set-default-shape electrons "electron"
  set-default-shape positrons "positron"
  set electric-field 0
  set contact? false
  set trail-color 119
  set streamer-color white
  set strike-id -1
  set streamers-out? false
  build-world
  place-ground-charges
end

to go
  if contact?
  [ strike ]
  every 0.2
  [ run-water-cycle
    move-charges
    create-path
  ]
end

;;;;;;;;;;;;;;;
;;;  SETUP  ;;;
;;;;;;;;;;;;;;;

to build-world
  ask patches
  [ set plasma-id -1
    set resistance 1
    set plocation "world"
    set plasma-count 5
  ]
  ask patches with [ pycor < ground-height ]
  [ ifelse ( pycor < 2 - screen-edge-y or
           ( pxcor > 10 - screen-edge-x and
             pxcor < -10 + screen-edge-x ) )
    [ set pcolor brown ]
    [ set pcolor blue
      set plocation "water"
    ]
  ]
  ask patches with [ pycor > cloud-height ]
  [ set pcolor grey ]
  ask patches with [ pycor >= ground-height and pycor <= cloud-height ]
  [ set pcolor black
    set resistance ( random 100 ) / 100
    set plocation "air"
  ]
end

to place-ground-charges
  create-custom-positrons 60
  [ set location "ground"
    set xcor ( screen-edge-x - 11 - random ( 2 * screen-edge-x - 21 ) )
    set ycor ( ground-height - 1 - random 8 )
    set color orange
  ]
  create-custom-electrons 60
  [ set location "ground"
    set xcor ( screen-edge-x - 11 - random ( 2 * screen-edge-x - 21 ) )
    set ycor ( ground-height - 1 - random 8 )
    set color yellow
  ]
end

;;;;;;;;;;;;;;;;;;;;;
;;;  WATER CYCLE  ;;;
;;;;;;;;;;;;;;;;;;;;;

to run-water-cycle
  evaporate
  precipitate
  vapor-advance
end

to evaporate
  if ((( random 10 ) / 10 ) < evaporation-rate )
  [ create-custom-vapors 1
    [ set location "air"
      set xcor (- screen-edge-x) + ( random 11 )  ;; somewhere in the water
      set ycor ground-height - 1                   ;; water's surface
      set heading 0
      set color cyan
    ]
  ]
end

to precipitate
  if (( any? electrons with [ location = "cloud" ] ) and
      (( random 10 ) / 10 ) < precipitation-rate )
  [ ask random-one-of electrons with [ location = "cloud"   ] [ die ]
    ask random-one-of positrons with [ location = "cloud"   ] [ die ]
    set electric-field ( electric-field - 1 )
    create-custom-vapors 1
    [ set location "air"
      set xcor ( screen-edge-x - ( random 11 ))    ;; somewhere over the water
      set ycor cloud-height + 1
      set heading 180
      set color cyan
    ]
  ]
end

to vapor-advance
  ask vapors [ fd 1 ]
  while [ any? vapors with [ ycor > cloud-height ] ]
  [ ask random-one-of vapors with [ ycor > cloud-height ]
      [ die ]
    create-custom-electrons 1
    [ set location "cloud"
      set xcor ( screen-edge-x - random ( 2 * screen-edge-x ))
      set ycor ( cloud-height + 1 + random 3 )
      set color yellow
    ]
    create-custom-positrons 1
    [ set location "cloud"
      set xcor ( screen-edge-x - random ( 2 * screen-edge-x ))
      set ycor ( cloud-height + 6 + random 3 )
      set color orange
    ]
    set electric-field ( electric-field + 1 )
  ]
  while [ any? vapors with [ ycor < ground-height ] ]
  [ ask random-one-of vapors with [ ycor < ground-height ]
    [ die ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  CHARGE MOVEMENT  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

to move-charges
  no-display

  ;; charges in the gorund
  ask positrons with [ location = "ground" ]
  [ if (( random electric-field ) > 40 )
    [ set heading towards-nowrap random-one-of electrons with [ location != "ground" ]
      fd 1
      if ( ycor > ground-height - 1 ) [ set ycor ground-height - 1 - ( random 3 ) ]
      if ( xcor < (- screen-edge-x) + 10 ) [ set xcor (- screen-edge-x) + 11 ]
      if ( xcor > screen-edge-x - 10) [ set xcor screen-edge-x - 11 ]
    ]
  ]
  ask electrons with [ location = "ground" ]
  [ if (( random electric-field ) > 40 )
    [ set heading 180 + towards-nowrap random-one-of electrons with [ location != "ground" ]
      fd 1
      if ( ycor < ground-height - 8 or ycor > 0 ) [set ycor 3 - screen-edge-y - random 3 ]
      if ( xcor < (- screen-edge-x) + 11 ) [ set xcor (- screen-edge-x) + 12 ]
      if ( xcor > screen-edge-x - 11) [ set xcor screen-edge-x - 12 ]
    ]
  ]

  ;; charges in the cloud
  ask positrons with [ location = "cloud" ]
  [ set heading random 360
    fd 1
    if ( ycor < screen-edge-y - 2 ) [ set ycor ( cloud-height + 6 + random 3 ) ]
  ]
  ask electrons with [ location = "cloud" ]
  [ ifelse pcolor-of patch-at 0 -1 != grey and
           pcolor-of patch-at 0 -1 != black
    [ set location "follower"
      set ycor ( ycor - 1 )
      ask patch-here
      [ set pcolor trail-color
        set plasma-count 5
      ]
    ]
    [ set heading random 360
      fd 1
      if ( ycor < cloud-height + 1 ) [ set ycor ( cloud-height + 1 + random 3 ) ]
      if ( ycor > cloud-height + 3 ) [ set ycor ( cloud-height + 1 + random 3 ) ]
    ]
  ]
  display
end

;;;;;;;;;;;;;;;;;;;;;;;
;;;  PATH CREATION  ;;;
;;;;;;;;;;;;;;;;;;;;;;;

to create-path
  if( electric-field > 80 )
  [ go-leaders
    go-streamers
  ]
  dissipate-path
end

to go-leaders
  ;; choose electrons to act as leaders
  if ( random 100 >= 90 and any? electrons with [ ycor = cloud-height + 1 and
                                                 not any? neighbors with [ pcolor = trail-color ] ])
  [ ask random-one-of electrons with [ ycor = cloud-height + 1 and
                                       not any? neighbors with [ pycor < ycor-of myself and pcolor != black ] ]
    [ setxy pxcor-of min-one-of neighbors with [ pycor < pycor-of myself ] [ resistance ]
            pycor-of min-one-of neighbors with [ pycor < pycor-of myself ] [ resistance ]
      set location "leader"
      ask patch-here
      [ set plasma-id who-of myself
        set plasma-count 5
        set pcolor trail-color
      ]
    ]
  ]

  ;; advance leading electrons
  ask electrons with [ location = "leader" ]
  [ ifelse any? neighbors with [ pcolor = streamer-color or pcolor = brown ]
    [ set contact? true
      ifelse any? neighbors with [ pcolor = streamer-color ]
      [ set strike-id plasma-id-of random-one-of neighbors with [ pcolor = streamer-color ]
        ask patches with [ pcolor = streamer-color and pycor > ycor-of myself and plasma-id = strike-id ]
        [ set pcolor black
          set plasma-id -1
          set resistance 1
          if any? turtles-here
          [ ask random-one-of turtles-here [ die ] ]
        ]
      ]
      [ set strike-id who
        ask random-one-of neighbors with [ pcolor = brown ]
        [ set plasma-id strike-id ]
      ]
    ]
    [ ifelse any? neighbors with [ pcolor = trail-color and pycor < ycor-of myself ]
      [ set location "follower" ]
      [ if random 10 < 5
        [ setxy pxcor-of min-one-of neighbors with [ ( pycor < ycor-of myself ) ] [ resistance ]
                pycor-of min-one-of neighbors with [ ( pycor < ycor-of myself ) ] [ resistance ]
          ask patch-here
          [ set plasma-id who-of myself
            set plasma-count 5
            set pcolor trail-color
          ]
        ]
      ]
    ]
  ]

  ;; advance following electrons, some may become leaders
  ask electrons with [ location = "follower" ]
  [ ifelse (( random 100 < 5 ) and
            ( count neighbors with [ pycor < ycor-of myself and pcolor = black ] = 2 ))
    [ set location "leader"
      setxy pxcor-of min-one-of neighbors with [ ( pycor < ycor-of myself ) and ( pcolor = black ) ] [ resistance ]
            pycor-of min-one-of neighbors with [ ( pycor < ycor-of myself ) and ( pcolor = black ) ] [ resistance ]
      ask patch-here [ set plasma-id who-of myself ]
    ]
    [ if random 10 < 5 and any? neighbors with [ pycor < ycor-of myself and pcolor != black ]
      [ setxy pxcor-of random-one-of neighbors with [ pycor < ycor-of myself and pcolor != black ]
              pycor-of random-one-of neighbors with [ pycor < ycor-of myself and pcolor != black ]
      ]
    ]
    ask patch-here
    [ set pcolor trail-color
      set plasma-count 5
    ]
  ]
end

to dissipate-path
  ask patches with [ plocation = "air" and pcolor != black and pcolor != streamer-color ]
  [ set plasma-count ( plasma-count - 1 )
    if (plasma-count < 0 )
    [ set pcolor ( pcolor - 1 )
      if (pcolor mod 10 = 0 ) [ set pcolor black ]
      set plasma-count 5
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;
;;;  STREAMERS  ;;;
;;;;;;;;;;;;;;;;;;;

to go-streamers
  if( electric-field = 120 )
  [ create-streamers ]
  if( streamers-out? = true and electric-field < 120 )
  [ destroy-streamers ]
end

to create-streamers
  repeat random 10
  [ if any? positrons with [ ycor = ground-height - 1 ]
    [ ask random-one-of positrons with [ ycor = ground-height - 1 ]
      [ ask patch-here [ set plasma-id who-of myself ]
        setxy pxcor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
              pycor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
        ask patch-here
        [ set plasma-id who-of myself
          set pcolor streamer-color
        ]
        set location "streamer"
        ;; grow each streamer to a random length
        repeat random 10
        [ setxy pxcor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
                pycor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
          ask patch-here
          [ set plasma-id who-of myself
            set pcolor streamer-color
          ]
        ]
      ]
    ]
  ]
  set streamers-out? true
end

to destroy-streamers
  ask positrons with [ location = "streamer" ] [ die ]
  ask patches with [ pcolor = streamer-color ]
  [ set pcolor black
    set plasma-id -1
  ]
  ask patches with [ plasma-id != -1 and pcolor = brown ]
  [ set plasma-id -1 ]
  set streamers-out? false
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  LIGHTNING STRIKE  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

to strike
  ;; destroy streamers
  ask patches with [ pcolor = streamer-color ]
  [ ifelse( plasma-id = strike-id )
    [ set pcolor trail-color
      set plasma-count 5
    ]
    [ set pcolor black
      set plasma-id -1
      set resistance 1
      while [ any? turtles-here ]
      [ ask random-one-of turtles-here [ die ] ]
    ]
  ]
  ask positrons with [ location = "streamer" ]
  [ set location "strike" ]

  ;; strike
  while [ electric-field > 20 ]
  [ positron-strike
    electron-strike
    dissipate-path
  ]

  ;; cleanup
  ask patches [ set plasma-id -1 ]
  ask patches with [ plocation = "air" ]
  [ set resistance ( random 100 ) / 100 ]
  ;; prevent electrons from moving from cloud into strike before it dissipates
  ask patches with [ pycor = cloud-height ]
  [ set pcolor black ]
  ;; in case strike went through water
  ask patches with [ plocation = "water" ] [ set pcolor blue ]
  ask positrons with [ location = "strike" ] [ die ]
  ask electrons with [ location = "leader" or location = "follower" ]
  [ set location "cloud" ]
  ask turtles with [ location = "ground" ] [ die ]
  place-ground-charges
  set contact? false
  set electric-field count electrons with [ location = "cloud" ]
end

to positron-strike
  ;; ground positrons flow through plasma path towards cloud
  ifelse( any? positrons with [ location = "ground" ] )
  [ ask random-one-of positrons with [location = "ground" ]
    [ set location "strike"
      setxy pxcor-of random-one-of patches with [ pycor = ground-height and plasma-id = strike-id ] ground-height
      ask patch-here [ set pcolor trail-color ]
    ]
    set electric-field ( electric-field - 1 )
    ask random-one-of electrons with [ location = "cloud" ][ die ]
    ask random-one-of positrons with [ location = "cloud" ][ die ]
  ]
  [ create-custom-positrons 5
    [ set location "ground"
    set xcor ( screen-edge-x - 11 - random ( 2 * screen-edge-x - 21 ) )
    set ycor ( ground-height - 1 - random 8 )
    set color orange
    ]
  ]
  ask positrons with [ location = "strike" ]
  [ ifelse not any? neighbors with [ pycor > ycor-of myself and pcolor != black ]
    [ setxy pxcor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
            pycor-of min-one-of neighbors with [ pycor > ycor-of myself ] [ resistance ]
    ]
    [ setxy pxcor-of random-one-of neighbors with [ pycor > ycor-of myself and pcolor != black ]
            pycor-of random-one-of neighbors with [ pycor > ycor-of myself and pcolor != black ]
    ]
    ;; positron's journey ends when it reaches the cloud or encounters an electron
    ifelse pcolor-of patch-here = grey
    [ die ]
    [ ask patch-here [ set pcolor trail-color ]
      if any? electrons-here
      [ ask random-one-of electrons-here [ die ]
        die
      ]
    ]
  ]
end

to electron-strike
  if any? electrons with [ location = "follower" or location = "leader" ]
  [ ask electrons with [ location = "follower" or location = "leader" ]
    [ ifelse not any? neighbors with [ pycor > ycor-of myself and pcolor != black ]
      [ die  ;; stranded in the air
        ask random-one-of positrons with [ location = "cloud" ] [ die ]
        set electric-field ( electric-field - 1 )
      ]
      [ setxy pxcor-of random-one-of neighbors with [ pycor > ycor-of myself and pcolor != black ]
              pycor-of random-one-of neighbors with [ pycor > ycor-of myself and pcolor != black ]
        ifelse pcolor-of patch-here = grey
        [ set location "cloud" ]
        [ ask patch-here [ set pcolor trail-color ] ]
      ]
    ]
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
7
10
724
398
50
25
7.0
1
10
1
1
1
0

CC-WINDOW
5
503
733
598
Command Center

BUTTON
14
405
81
438
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
88
405
151
438
NIL
go
T
1
T
OBSERVER
T
NIL

MONITOR
15
440
151
489
Electric Field Strength
electric-field
1
1

SLIDER
188
412
360
445
evaporation-rate
evaporation-rate
0
1
1.0
0.1
1
NIL

SLIDER
188
449
360
482
precipitation-rate
precipitation-rate
0
1.0
0.0
0.1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
Lightning is one of the most visually impressive and frequently occurring natural phenomena on Earth.  However, very few people actually have a solid understanding of what causes lightning, how it works, and why it occurs.  This model attempts to illustrate lightning strikes from beginning to end at the very low level of individual charges.  The user will observe how the behavior and interaction of extremely small charges can lead to the very powerful and visually impressive action of a lightning strike.

In order to thoroughly understand the phenomenon of lighting one must have a strong understanding of electrical physics and thermodynamics.  In order to be meaningful this model avoids such complicated topics and instead provides a more general basis for charge and particle behavior.  The emphasis in this model is to understand lightning as it relates to individual charges, not the underlying forces behind the charges themselves.


HOW IT WORKS
------------
The initial cause of a lightning strike is a separation of charge within a cloud.  Positively charged particles accumulate at the top of the cloud while negatively charged particles concentrate themselves at the bottom of the cloud.  Unfortunately, it is currently unknown why this charge separation occurs.  This model illustrates a widely held theory in this respect.  The water cycle itself is the primary cause.  As water vapor rises it often acquires a charge as it interacts with dirt, ice, and other materials during its ascent towards the cloud.  Once in the cloud the heavier vapor particles have a negative charge and thus sink to the cloud's bottom.  The actual mechanism behind the creation of charge separation is unimportant in the actual strike and so the water cycle and charge separation are included in this model solely to demonstrate how the charge separation may occur.  Vapors rise off of the water's surface with a frequency specified by the EVAPORATION-RATE slider.  The vapors then rise toward the sky until they reach the cloud, at which time they spawn opposing charges in the upper and lower regions of the cloud.  The ELECTRIC-FIELD is measured by the number of negatively charged particles (electrons) in the cloud.  Since the strength of an electric field is directly proportional to the amount of charge in that field, this is a reasonable measure.

Within the cloud the charges move in a random manner.  However, they are constrained to their charge regions.  The positive charges do not try to reach the negatives nor vice versa.  In this way the charge separation and electric field strength is maintained.  Occasionally charge may dissipate from the cloud.  While the actual mechanism for this dissipation is also unknown, it is illustrated in this model by precipitation in the same manner as the charge buildup.  Charge leaves the cloud at a rate specified by the PRECIPITATION-RATE slider.  When the EVAPORATION-RATE equals the PRECIPITATION-RATE the strength of ELECTRIC-FIELD will remain fairly constant.

When ELECTRIC-FIELD has strength of 40 or greater the charges in the ground begin to be affected.  The negative charges in the bottom of the cloud have such a high concentration that they force the electrons on the Earth's surface deep into the ground.  This also has the affect of pulling the positive charges (positrons) in the Earth's surface to a very shallow depth.  Thus, the ground has a strong positive charge and the cloud has a strong negative charge.  The two surfaces act like two parallel plates (for those who know a bit of electrical physics).  This charge disparity cannot continue forever, and lightning is nature's solution to the problem.

Charge continues to build until the ELECTRIC-FIELD hits the critical value of 80.  With field strength of such a large magnitude the electrons in the cloud begin "breaking down" the air around the cloud and converting it into plasma.  Plasma is the same material that comprises stars.  Without going into too much detail, plasma is matter (in this case air) that has been ionized.  The positive and negative components of the air itself are pulled apart and separated from each other.  This separation allows the electrons to flow through the plasma much more easily than they could through normal air.  The plasma is created in a chain-reaction manner as electrons flowing through the plasma force the air around them to become plasma in turn.  In this way electrons in the cloud "burn out" paths towards the earth.  The paths are known as "step leaders" and grow from cloud to ground in a tentacle-like manner.  The chain reaction that allows the step leaders to grow in size does not have to take place in a straight line.  Air is not homogenous and is littered with impurities that affect the formation of step leaders.  These impurities cause the air to not ionize equally in all directions.  Impurities in the air may cause some patches of air to turn into plasma more easily than others.  Rather than direct lines from cloud to ground, lightning takes the path of least resistance.  This is what gives lightning strikes the forked and jagged patterns that everyone can recognize.  In this model this is illustrated by assigning each patch of air a RESISTANCE value.  Electrons will ionize air and progress towards the ground always trying to follow the path of least resistance.

As the step leaders progress to the ground the ELECTRICAL-FIELD likely will still be increasing in magnitude.  Once it reaches a value of 120 the charges in the ground react by letting off what are known as "streamers".  Streamers are the positive equivalent of the negative step leaders created by clouds.  However, streamers are not self-sufficient and thus do not grow indefinitely towards the cloud.  All objects on the Earth's surface will emit a streamer, though depending on the size and material of the object the streamer's length may vary.  A tall building will release a larger streamer than a person on the sidewalk.  The streamer's release is what causes people to often feel a tingling sensation before a lightning strike.  The positive charges in the ground are reaching out to the negative charges in the sky.  Streamers are released much more quickly than a step leader since they are much smaller and only extend a very limited distance.

Step leaders progress towards the ground until they encounter either the ground or a streamer.  In both cases the "circuit" is completed and charge may flow freely between cloud and ground.  Since step leaders and streamers grow in an irregular way the strike may occur at an otherwise unexpected place.  It is not uncommon for lightning to strike a 6-foot tall person instead of the 40-foot tree nearby.  Once a connection has been made charges begin to move through the path.  Contrary to popular understanding, the actual lightning strike flows from the ground up.  The large concentration of positive charges on the earth's surface flow very quickly through the plasma stream towards the sky and neutralize the electrons in the cloud.  The flash of light that is seen is the rapid movement of charge through the air, exactly the same as the light you see during a spark of static electricity.  This massive movement of charge results in an enormous current that generates an equally enormous amount of heat around the strike.  The current and heat are what cause damage to people and objects during a lightning strike.  When the strike occurs the air surrounding the lightning's path reaches temperatures hotter than the surface of the sun.

When a step leader completes the path between cloud and ground, the charges that were forming other step leaders quickly flow back up towards the cloud.  This rapid movement of charge causes secondary strikes that usually accompany a lightning strike.  While the step leader that actually makes contact is known as the primary strike, several secondary strikes are often visible for short periods of time in the sky surrounding the cloud.  These strikes do not extend all the way to the ground, but can still be impressive to watch.  There may be several primary strikes within a given lightning strike, but only one set of secondary strikes.  In this model you may observe secondary strikes as the electrons in the step leaders move back into the cloud once contact is made.

Once the massive negative charge in the cloud has been neutralized the flow of charge through the air comes to a stop.  The plasma becomes de-ionized and once again becomes air as the step leaders are destroyed.  The environment is now ready to begin the process anew.


HOW TO USE IT
-------------
To run the model the user needs only press SETUP and then GO.

SETUP - this button builds the 'world' and places charges within the ground.

GO - this button runs the model.  The process will run over and over without ever stopping.

EVAPORATION-RATE - this slider specifies the rate at which water evaporates to the cloud.  With a value of 0 no evaporation will take place.  With a value of 1 water will evaporate with every time step.  Evaporation is directly responsible for the buildup of charge in the cloud.

PRECIPIATION-RATE - this slider specifies the rate at which water precipitates from the cloud.  With a value of 0 no precipitation will take place.  With a value of 1 water will precipitate with every time step.  Precipitation is directly responsible for the dissipation of charge in the cloud.

ELECTRIC-FIELD - this monitor displays the current strength of the electric field between the cloud and the ground.  This value is the number of electrons present in the cloud.


THINGS TO NOTICE
----------------
Observe the forked and irregular pattern formed by the step leaders.  This irregularity is dictated by the random resistances of the air patches and is what gives lightning its distinctive visual appearance.

Observe how the lightning strike itself is actually the flow of positive charges from the ground upwards towards the cloud.  While the path the lightning takes is created by the electrons, it is the positrons that actually move during the strike.

Try and observe the secondary strikes at the very beginning of a primary strike.  These strikes are caused by electrons flowing back into the cloud.


THINGS TO TRY
-------------
What happens when precipitation is less than evaporation?  More than?  Equal?  How does this help explain why lightning does not form in every storm and every cloud?


EXTENDING THE MODEL
-------------------
Create two separate smaller clouds instead of one giant one.  Perhaps allow for cloud-to-cloud lightning strikes as well as cloud-to-ground.

Add various sliders to allow the user to control the thresholds at which step-leaders are formed, charges react to each other, streamers are released, etc.

Add objects on the ground and observe how object height impacts where lightning strikes occur.


CREDITS AND REFERENCES
----------------------
Original implementation: Jason Alt, for the Center for Connected Learning and Computer-Based Modeling.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

electron
false
0
Circle -256 true false 35 34 231
Rectangle -16777216 true false 83 135 219 160

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

positron
false
0
Circle -44544 true false 33 31 235
Rectangle -16777216 true false 92 132 217 162
Rectangle -16777216 true false 138 92 173 196

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
