turtles-own [
  temp                 ;; this turtle's temperature
  neighboring-turtles  ;; agentset of surrounding turtles
  sides-exposed        ;; number of sides exposed to cooling walls  (between 0 and 4)
  element              ;; the chemical element
]

globals [
  ave-metal-temp          ;; shows average temperature of all metal
  num-frozen              ;; keeps track of how many atoms are frozen
  time                    ;; keeps track of time passing in system
  logtime                 ;; log of time
  temp-range              ;; for histogram
  colors                  ;; used both to color turtles, and for histogram
  pens                    ;; keeps track of all the histogram's pen names
  xmax                    ;; max x size
  ymax                    ;; max y size
  intercepts              ;; grain size proxy
  average-grain-size      ;; average grain size
  logaverage-grain-size   ;; log of average grain size (for graphing purposes)
  dimensions              ;; image dimensions (image parser)
  dimensionlength         ;; lenght of image dimension string (image parser)
  graylevel               ;; maximum gray level (image parser)
  orientation-for-incercept-count  ;; for grain size calculation (normally 90 degrees)
  mould_heat_range        ;; percent of the mould to which the wall tempetature is applied to

]

breeds [element1 element2] ;; two different chemical elements

to fill-outer-area-with-white

  set ymax (height + 1) / 2
  set xmax (width + 1) / 2

  ask patches with [(abs pycor >= ymax) or
                    (abs pxcor >= xmax)]
                    [set pcolor white ]

end

;;initialize solidification with a simple square mould
to init-box

  setup
  fill-outer-area-with-white

end

to init-file-patches

  setup
  open-file-and-transfer-to-patches

end

to init-file-turtles

  setup
  open-file-and-transfer-to-turtles

end


to setup
;; clears all, resets time and sets main variables

  ca
  set colors sentence (white - 1) [cyan sky blue violet magenta pink red]
  set pens []
  set temp-range (init-metal-temp - melting-temp) / (length colors - 1)
  set ave-metal-temp init-metal-temp
  set time 1

end

;; import image into patches
to open-file-and-transfer-to-patches

  ct
  file-open user-choose-file

  print file-read-line ;; read image header

  set dimensions file-read-line ;; read image dimensions
  set graylevel read-from-string file-read-line ;; read the maximum gray level
  set dimensionlength (length dimensions) ;; length of the image dimension string for parsing purposes
  set width read-from-string substring dimensions 0 ((position " " dimensions) ) ;; parsing of the dimensions string
  set height read-from-string substring dimensions ((position " " dimensions) + 1) dimensionlength ;; parsing of the dimensions string

  ;; read from file until EOF and change patches' colors within the image limits

  fill-outer-area-with-white

  ask (patches) [
                 if ((not file-at-end?) and ((abs pycor < ymax) and (abs pxcor < xmax)))
                   [set pcolor (file-read * (9.9999 / graylevel))]
                ]

  file-close

end

;; import image into turtles
to open-file-and-transfer-to-turtles

  ct
  file-open user-choose-file

  print file-read-line ;; read image header
  ;print file-read-line ;; some pgm files have an
                        ;; additional line for filename

  set dimensions file-read-line ;; read image dimensions
  set graylevel read-from-string file-read-line ;; read the maximum gray level
  set dimensionlength (length dimensions) ;; lenght of the image dimension string for parsing purposes
  set width read-from-string substring dimensions 0 ((position " " dimensions) ) ;; parsing of the dimensions string
  set height read-from-string substring dimensions ((position " " dimensions) + 1) dimensionlength ;; parsing of the dimensions string


  ;; read from file until EOF and change turtles' orientation and color within the image limits

  ask patches
    [
     sprout 1
       [
        ;; creates turtles of the element1
        set heading 0 set color red set breed element1
       ]
    ]

  ask (turtles)
    [
     if (not file-at-end?)
       [set heading (file-read) ]
    ]

  ask turtles
    [
     set color heading
    ]

  file-close

end

to  fill-empty-space-with-turtles
;;fill the remaining space with turtles and sets turtles' variables


  ;; fills first element
  ask patches
  [
    if ((fill-percentage > random-float 100) and (pcolor != white))
    [
      sprout 1
      ;; creates turtles of the element1
      [
          set temp init-metal-temp
          set-color
          ;set color pcolor
          set element 1
          set shape "circ" ;; deactivated for now (19/Apr/2004)
      ]
    ]
  ]

  ;;fills second element

  ask patches
  [
    if ((element2-percent > random-float 100) and (pcolor != white))
      [ask turtles-here
      ;; creates turtles of the element1
        [

         set temp melting-temp + 1
         set color white
         ;set-color
         set element 2
         set shape "circ"
        ]
      ]
  ]

;; defines neighboring turtles and sides exposed for mould contact
ask turtles
    [
     set neighboring-turtles (turtles at-points [[-1  1] [ 0  1] [1  1]
                                                 [-1  0] [ 0  0] [1  0]
                                                 [-1 -1] [ 0 -1] [1 -1]])

     set sides-exposed (count patches at-points [[-1  1] [ 0  1] [1  1]
                                                 [-1  0] [ 0  0] [1  0]
                                                 [-1 -1] [ 0 -1] [1 -1]]
                     with [(not any? turtles-here) and (pcolor = black)])

    if element = 1 [set breed (element1) set shape "rect"]
    if element = 2 [set breed (element2) set shape "circ"]

    ]

  if (count turtles > 0)
  [
    setup-plots
    do-plots
  ]

end

;; turtle procedure -- if metal is liquid, move to a randomly
;; selected neighboring patch that is not already occupied
to move
  locals [move-choices]
  if (temp > melting-temp)
  [
    set move-choices neighbors with [(pcolor != white) and (not any? turtles-here)]
    if (random-one-of move-choices != nobody)
    [
      set move-choices random-one-of move-choices
      setxy pxcor-of move-choices pycor-of move-choices
    ]
  ]
end

;; start solification procedure
to go

  ;; stop if all turtles from element1 are below melting temp
  if (max (values-from element1 [temp]) < melting-temp) [ stop ]
  ;; otherwise...
  set time (time + 1) ;;increase time
  set num-frozen 0

  ask turtles [ move ]
  ask turtles [ rotate ]
  ask element1 [ cool-turtles ] ;;cool only element1
  ask turtles [ set-color ]
  set ave-metal-temp (mean values-from element1 [temp])
  do-plots ;;plots

end

;; turtle procedure -- if metal is liquid and it is next to a solid,
;; change its heading to that of the solid; otherwise, just rotate
;; randomly
to rotate
  locals [frozen-neighbors]
  set neighboring-turtles (turtles at-points [[-1  1] [ 0  1] [1  1]
                                              [-1  0] [1  0]
                                              [-1 -1] [ 0 -1] [1 -1]])
  if (temp > melting-temp)
  [
    set frozen-neighbors (neighboring-turtles with [temp <= melting-temp])
    ifelse (any? frozen-neighbors)
      [ set heading (heading-of (random-one-of frozen-neighbors)) ]
      [ rt random-float 360 ]
  ]
end

;; change turtles' colors to reveal grain structure
to finalcoloring2

  ask turtles [ set color int ((heading / 2)) ]

end

;; turtle procedure -- sets turtle's temp to ave temp of all
;; neighboring turtles and patches added turtle's own temp in twice so
;; it changes more slowly. Heated or cooled walls contribute differently
;; Note that due to the freeform shapes, the mould_heat_range variable
;; defines the amount of mould surface affected by the heat gradient
to cool-turtles
  locals [total-temp total-num walled-sides walled-sides-top walled-sides-bottom walled-sides-left walled-sides-right]
  set walled-sides neighbors with [pcolor = white]
  set walled-sides-top neighbors with [(((pycor <= ymax) and (pycor > (ymax * mould_heat_range))) and pcolor = white)]
  set walled-sides-bottom neighbors with [(((pycor >= (- ymax)) and (pycor < ((- ymax) * mould_heat_range))) and pcolor = white)]
  set walled-sides-left neighbors with [(((pxcor >= (- xmax)) and (pxcor < ((- xmax) * mould_heat_range))) and pcolor = white)]
  set walled-sides-right neighbors with [(((pxcor <= xmax) and (pxcor > (xmax * mould_heat_range))) and pcolor = white)]

  set total-temp ( (sum values-from neighboring-turtles [temp]) +
                   (temp) +
                   (wall-top-temp * (count walled-sides-top)) +
                   (wall-bottom-temp * (count walled-sides-bottom)) +
                   (wall-left-temp * (count walled-sides-left)) +
                   (wall-right-temp * (count walled-sides-right))
                  )

;                 (room-temp * (count walled-sides) + temp)) ;; commented for later use

  set total-num ((count neighboring-turtles) + (count walled-sides) + 1)
  set temp (total-temp / total-num)

;;  if ((temp > 100) and (temp < 110) and (who / 10 = 0)) [die]
  if ((metal-contraction != 0) and (temp > 490) and (temp < 500) and (who mod (100 / metal-contraction) = 0)) [die]

end

;; turtle procedure - colors turtles during solidification
to set-color
  locals [ index ]
  ; create index ranging from 1 to 8 for all melting colors
  set index (floor ((temp - melting-temp) / temp-range)) + 1
  ifelse (index < 1 ) [
    set color white - 1
    set num-frozen (num-frozen + 1)
  ]
  [
    if index >= length colors
      [ set index (length colors) - 1 ]
    set color item index colors
  ]
end

to setup-plots
  set-current-plot "Average Metal Temperature"
  set-plot-y-range room-temp init-metal-temp

  set-current-plot "Number Solidified"
  set-plot-y-range 0 count turtles

end

to do-plots
  set-current-plot "Average Metal Temperature"
  plot ave-metal-temp
  set-current-plot "Number Solidified"
  plot num-frozen ;;commented for increased speed

end
@#$#@#$#@
GRAPHICS-WINDOW
505
24
1068
608
39
39
7.0
1
10
1
1
1
0

CC-WINDOW
5
623
1077
718
Command Center

BUTTON
138
444
193
477
go
go
T
1
T
OBSERVER
NIL
NIL

SLIDER
12
218
146
251
room-temp
room-temp
-20.0
100.0
20.0
10
1
deg C

SLIDER
154
218
296
251
init-metal-temp
init-metal-temp
1550.0
2500.0
1570.0
10.0
1
deg C

SLIDER
306
219
444
252
melting-temp
melting-temp
500.0
1500.0
530.0
10
1
deg C

SLIDER
12
36
121
69
width
width
3
200
41
2
1
atoms

SLIDER
125
36
234
69
height
height
3
200
41
2
1
atoms

PLOT
149
489
309
609
Average Metal Temperature
time
ave-metal-temp
0.0
100.0
20.0
1550.0
true
true
PENS
"ave-metal-temp" 1.0 0 -65536 false

PLOT
311
489
471
609
Number Solidified
time
crystal quant.
0.0
100.0
0.0
625.0
true
false
PENS
"amount" 1.0 0 -16777216 false

BUTTON
73
444
128
477
go once
go
NIL
1
T
OBSERVER
T
NIL

SLIDER
11
107
161
140
wall-top-temp
wall-top-temp
-3000
3000
20
20
1
deg C

SLIDER
168
107
317
140
wall-bottom-temp
wall-bottom-temp
-3000
3000
20
20
1
deg C

SLIDER
167
146
317
179
wall-right-temp
wall-right-temp
-3000
3000
20
20
1
deg C

SLIDER
11
145
161
178
wall-left-temp
wall-left-temp
-3000
3000
20
20
1
deg C

BUTTON
307
363
445
396
see grain structure
finalcoloring2
NIL
1
T
OBSERVER
NIL
NIL

TEXTBOX
13
200
131
218
Room temperature

SLIDER
154
290
295
323
fill-percentage
fill-percentage
0
100
91
1
1
NIL

BUTTON
322
107
425
140
set all to 20C
set wall-top-temp 20\nset wall-bottom-temp 20\nset wall-right-temp 20\nset wall-left-temp 20
NIL
1
T
OBSERVER
NIL
NIL

SLIDER
305
291
443
324
element2-percent
element2-percent
0
10
5
1
1
NIL

BUTTON
307
327
443
360
highlight grain refiner
ask element2 [set color white]
NIL
1
T
OBSERVER
NIL
NIL

BUTTON
241
36
322
69
import image
init-file-patches
NIL
1
T
OBSERVER
T
NIL

BUTTON
341
36
415
69
rectang
init-box
NIL
1
T
OBSERVER
T
NIL

BUTTON
8
444
63
477
setup
fill-empty-space-with-turtles
NIL
1
T
OBSERVER
T
NIL

SLIDER
12
290
145
323
metal-contraction
metal-contraction
0
100
0
1
1
%

TEXTBOX
307
201
455
219
Melting temperature

TEXTBOX
155
200
273
218
Initial metal temperature

TEXTBOX
14
17
104
35
Mould size

TEXTBOX
12
86
168
104
Mould walls temperatures

TEXTBOX
243
16
340
34
Import mould image

TEXTBOX
341
17
431
35
Use rectangular

TEXTBOX
14
270
108
288
Additional features

TEXTBOX
9
423
99
441
Setup & Run

@#$#@#$#@
I am working on the info...
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circ
true
0
Circle -7566196 true true 10 11 278

rect
true
0
Polygon -7566196 true true 67 36 67 262 235 262 235 35

sq
false
0
Rectangle -7566196 true true 15 15 286 285
Line -1 false 6 6 293 6
Line -1 false 293 6 293 293
Line -1 false 293 292 8 292
Line -1 false 8 292 8 6

t
true
0
Rectangle -7566196 true true 46 47 256 75
Rectangle -7566196 true true 135 76 167 297

@#$#@#$#@
NetLogo 2.2pre1
@#$#@#$#@
init-box
fill-empty-space-with-turtles
repeat 37 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
