globals [   clock   vsplit    vclock         ;; Clock Variables
            pos   plotfrq   plot-range
            number-on-left number-on-right  ]

patches-own [box?]

turtles-own [   speed   mass   energy charge  new-speed                      ;; Turtle Info
                v1t   v1l   tmp-turtle                              ;; Turtle 1 -- Collide
                heading2   mass2   speed2   v2t   v2l   idnum2     ;; Turtle 2 -- Collide
                vcm                                                                                 ;; Turtles 1 + 2 -- Collide
                theta                                                                               ;; vcm = CM vel. along dir. theta
                bouncing?   bounces   collisions   bounce-x   bounce-y   tsplit  ]

to setup
  ca
  make-box-and-membrane
  set plotfrq 6
  set clock 0
  ;setup-plots
  crt number-left
  crt number-right
  ask turtles [ set bounces 0
  set collisions 0
  set speed 1
  set new-speed speed
  set bouncing? false
  set mass 1
  set charge 0  ]
  ask turtles [
    set shape "circle"
    if ((who != 0) and (who < number-left))
      [setxy (random-float (2 - screen-edge-x))
             (random-float (screen-size-y - 2) - screen-edge-y)]
    if ((who != 0) and (who > number-left))
      [setxy (random-float (screen-edge-x - 1))
             (random-float (screen-size-y - 2) - screen-edge-y)]
    set heading ( random-float 360 )
    color-turtles
    calculate-energy
  ]
  set vsplit 2 ; was initspeed * 2
  ask turtles [ set tsplit vsplit ]
  set vclock 0
end

to go
  ask turtles [bounce]
  ask turtles [move]
  set number-on-left count turtles with [xcor < 0]
  set number-on-right count turtles with [xcor > 0]
end

to bounce               ;; turtle procedure
  ifelse (pcolor = black)
    [set bouncing? false]
    [ifelse ((pcolor = green or pcolor = yellow) and (not (bouncing? and (bounce-x = (round xcor)))))
      [set heading (heading * -1)
       save-bounce-position]
      [ifelse ((pcolor = green + 1 or pcolor = yellow) and (not (bouncing? and (bounce-y = (round ycor))))) ; what is red + 1?
        [set heading (180 - heading)
         save-bounce-position]
        [ifelse ((pcolor = green + 2) and bouncing? and (bounce-y = (round ycor)) and (not (bounce-x = (round xcor))))
          [set heading (heading * -1)
           save-bounce-position]
          [ifelse ((pcolor = green + 2) and bouncing? and (bounce-x = (round xcor)) and (not (bounce-y = (round ycor))))
            [set heading (180 - heading)
             save-bounce-position]
            [if ((pcolor = (green + 2)) and (not bouncing?))
             [set heading (heading + 180)
               save-bounce-position]]]]]]
if ((round xcor = 0) and (mass = 2) and (not (bouncing? and (bounce-x = (round xcor)))))
  [set heading (180 - heading)
   save-bounce-position]
end

to save-bounce-position
  set bounce-x (round xcor)
  set bounce-y (round ycor)
  set bouncing? true
  set bounces (bounces + 1)
end

to move                    ;;turtle procedure
  jump ( speed / tsplit )
  check-for-collision
end

to check-for-collision ;;turtle procedure
  if ( count turtles-here = 2 ) and (box? = false)
    [set tmp-turtle random-one-of turtles-here
     if ( ( who > who-of tmp-turtle ) and ( idnum2 != who-of tmp-turtle ) )
       [collide]
     color-turtles]
end

to collide            ;;turtle procedure
  get-turtle2-info
  calculate-velocity-components
  set-new-speed-and-headings
end

to get-turtle2-info ;;turtle procedure
  set idnum2 tmp-turtle
  set mass2 mass-of idnum2
  set speed2 new-speed-of idnum2
  set heading2 heading-of idnum2
  set collisions (collisions + 1)
  set collisions-of idnum2 (collisions-of idnum2 + 1)
end

to calculate-velocity-components
  set theta ( random-float 360 )
  set v1l ( new-speed * ( sin ( theta - heading ) ) )
  set v1t ( new-speed * ( cos ( theta - heading ) ) )
  set v2l ( speed2 * ( sin ( theta - heading2 ) ) )
  set v2t ( speed2 * ( cos ( theta - heading2 ) ) )
  set vcm ( ( ( mass * v1t ) + ( mass2 * v2t ) ) / ( mass + mass2 ) )
  set v1t ( vcm + vcm - v1t )
  set v2t ( vcm + vcm - v2t )
end

to set-new-speed-and-headings          ;; turtle procedure
  set new-speed (-0.0005 + (sqrt ( ( v1t * v1t ) + ( v1l * v1l ))))
  ifelse ( ( v1l >= 0 ) and ( v1t >= 0 ) )
    [set heading ( theta - ( a-atan v1l v1t ) )]
    [ifelse ( ( v1l < 0 ) and ( v1t < 0 ) )
      [set heading ( ( theta + 180 ) - ( a-atan v1l v1t ) )]
      [ifelse ( ( v1l >= 0 ) and ( v1t < 0 ) )
        [set heading ( ( theta + 180 ) - ( a-atan v1l v1t ) ) ]
        [if ( ( v1l < 0 ) and ( v1t >= 0 ) )
          [set heading ( theta - ( a-atan v1l v1t ) ) ] ] ] ]
  set new-speed-of idnum2 (-0.0005 + (sqrt (( v2t * v2t) + ( v2l * v2l))))
  ifelse ( ( v2l >= 0 ) and ( v2t >= 0 ) )
    [set heading2 ( theta - ( a-atan v2l v2t ) )]
    [ifelse ( ( v2l < 0 ) and ( v2t < 0 ) )
      [set heading2 ( ( theta + 180 ) - ( a-atan v2l v2t ) )]
      [ifelse ( ( v2l >= 0 ) and ( v2t < 0 ) )
        [set heading2 ( ( theta + 180 ) - ( a-atan v2l v2t ) )]
        [if ( ( v2l < 0 ) and ( v2t >= 0 ) )
          [set heading2 ( theta - ( a-atan v2l v2t ) ) ] ] ] ]
  set heading-of idnum2 heading2
end

to color-turtles ;;turtle procedure
  if (who > number-left and who < (number-left + number-charged))
    [set charge 3]
  if (charge = 0)
    [set color yellow]
 ;; if (mass = 2)
 ;;   [set color green]
  if (charge = 3)
    [set color blue]
end

to calculate-energy ;;turtle procedure
  set energy ( 0.5 * speed * speed * mass )
end

to make-box-and-membrane
  ask patches [
    if ((pxcor = round screen-edge-x) or
       (pxcor = round (- screen-edge-x)) or
       (pycor = round screen-edge-y) or
       (pycor = round (- screen-edge-y)))
      [set pcolor green]

    if ((pxcor = 0) and (pycor mod permeability = 0))
      [set pcolor red + 2]
  ]
  ask patches [
    ifelse (pcolor != black) [set box? true] [set box? false] ;; here "box" is any surface to bounce
                                                        ;; off. not used outside this funct yet
    if (box? and (pxcor > -1)) [ set pcolor red + 2 ]
  ]
end  ;; awkward - could make it look better

to setup-plots
  set-current-plot "Speed Counts"
  clear-plot
  set-plot-x-range 0 20
  set-plot-y-range 0 (number-left + number-right)
  auto-plot-on
  create-temporary-plot-pen "fast"
  set-plot-pen-color red
  create-temporary-plot-pen "average"
  set-plot-pen-color green
  create-temporary-plot-pen "slow"
  set-plot-pen-color blue
end

to-report a-atan [ yy xx ]
  if ( ( yy >= 0 ) and ( xx >= 0 ) )
    [report ( atan yy xx )]
  if ( ( yy >= 0 ) and ( xx < 0 ) )
    [report ( 180 + ( atan yy xx ) )]
  if ( ( yy < 0 ) and ( xx >= 0 ) )
    [report ( atan yy xx )]
  if ( ( yy < 0 ) and ( xx < 0 ) )
    [report ( ( atan yy xx ) - 180 )]
 end





@#$#@#$#@
GRAPHICS-WINDOW
304
10
679
406
36
36
5.0
0
10
1
1
1
0

CC-WINDOW
5
420
688
515
Command Center

SLIDER
155
10
281
43
number-left
number-left
0
250
50
1
1
NIL

SLIDER
147
46
275
79
number-right
number-right
0
250
200
5
1
NIL

BUTTON
23
34
78
67
setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
24
94
79
127
go
go
T
1
T
OBSERVER
T
NIL

MONITOR
11
232
108
281
number on left
number-on-left
0
1

MONITOR
117
234
213
283
number on right
number-on-right
0
1

MONITOR
224
234
274
283
total
count turtles
0
1

SLIDER
128
103
290
136
number-charged
number-charged
0
50
0
1
1
NIL

SLIDER
9
156
115
189
permeability
permeability
1
4
2
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------

This model demonstrates the movement of molecules following a concentration gradient across a cell membrane. The right side represents the inside of the cell.

In simple diffusion, the molecules, atoms, or ions are small and uncharged enough to pass between the molecules of the membrane; they don't require special channels or energy from the cell. They move from an area of higher concentration to an area of lower concentration. This behavior is the result of increased collisions between molecules.

The monitors show the number of molecules on each side and the total number.

-----
TO DO: plot rate at which system reaches equilibrium
       add option for molecules not to bounce off each other
       add ability to move membrane to change pressure on each side*
       add ion channels*
       (add active transport)*
       fix setup so turtles don't start on walls

*these possibly only on an extension of the model


HOW TO USE IT
--------------
SETUP: resets the simulation according to the parameters set by the sliders.

GO: starts and stops the simulation.

NUMBER-LEFT: sets the number of molecules to be created outside the membrane.

NUMBER-RIGHT: sets the number of molecules to be created inside the membrane.

PERMEABILITY: How many gaps there are in the membrane.

NUMBER-CHARGED adds the selected number of charged (blue) molecules to one side. This doesn't do anything significant now, but we could have them require active transport, gated ion channels, etc. in order to pass through the membrane.


THINGS TO NOTICE
----------------

As the simulation progresses, the numbers of ions on each side approach "equilibrium."

THINGS TO TRY
-------------

Change the sliders to see what affects the rate at which equilibrium is reached.

EXTENDING THE MODEL
-------------------

different kinds of particles, specific channels, Na+/K+ pumps... anything involved in membrane transport


CREDITS AND REFERENCES
----------------------
(model begun by Katie Goodrum)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
