;; IMPORTANT: If you don't want to change how the machines work, only how they
;; sound, then go to the bottom of this tab.  The sounds are controlled by a
;; group of procedures that are grouped together there.


__extensions [ "sound.jar" ]  ;; see the "Sound" section of the user manual

globals [
  selected
  dragged
  ticks
  K
  pitches
  drumkit
]

breeds [
  levers
]

levers-own [
  free1?
  instrument
  drum
  free2?
  stretch
  cx cy
  x1 x2 y1 y2
  hook1
  hook2
  xv yv av
  fixed?
  spin?
  armlen
  from-h1 to-h2
  h1x2 h1y2
  h2x1 h2y1
  h1cx h1cy
  h2cx h2cy
  ideal-dist
  ideal-angl
  ratio1
  ratio2
  pitch
]

to setup
    stop-music
    clear-all
    set k 1
    set-default-shape levers "lever"
    set ticks 0
    ; set population screen-edge-x * .5
    create-levers population
    ask levers
    [ set spin? false
      set armlen .5
      set size scale
      set cx screen-size-x - scale * armlen
      set cy ( (who / population) * screen-size-y - screen-edge-y ) * -.4
      set fixed? false
      setxy cx cy
      set cx xcor
      set cy ycor
      ifelse who = 0
      [ set free1? false
        set hook1 self
        set label "1"
        set armlen 1
        set spin? true
        set fixed? true
      ]
      [ set free1? true
        set hook1 turtle ( who - 1 )
      ]
      ifelse who = population - 1
      [ set free2? false
        set hook2 self
        set label "2"
        set fixed? false
      ]
      [ set free2? true
        set hook2 turtle ( who + 1 )
      ]
      set stretch 1.0
      set heading random 360
      set x1 xcor - dx * scale * armlen
      set y1 ycor - dy * scale * armlen
      set x2 xcor + dx * scale * armlen
      set y2 ycor + dy * scale * armlen
    ]
    set selected turtle 0
    setup-music
end

to-report mean-heading [ a b ]
      report atan (( sin a + sin b) * .5)
                  (( cos a + cos b) * .5)
end

to-report mean-heading-w [ a b c w]
      let w/2 (1 - w) * .5
      report atan ( w/2 * sin a + w/2 * sin b + w * sin c )
                  ( w/2 * cos a + w/2 * cos b + w * cos c )
end

to-report xangle-diff-sin [ a b ]
    let diff sin a - cos b
    ifelse diff < -180
    [ set diff diff + 360 ]
    [ if diff > 180 [ set diff diff - 360 ]
    ]

    report sin (diff) - cos (diff)
end

to go
  set ticks ticks + 1
  ask levers
  [ monitor-mouse
    lever-display
  ]
  ask levers
  [ calc-arm-coords
    calc-hook-coords
  ]
  ask levers
  [ lever! ]
  ;ask random-one-of levers [ without-interruption
  ;[ lever!2 ] ]
  chorus
end

to-report sign [ a ]
    ifelse a = 0
    [ report 0 ]
    [  report a / abs a ]
end

to monitor-mouse
    let mouse-here? abs (mouse-xcor - xcor)  < scale / 2 and abs ( mouse-ycor - ycor) < scale / 2
    ifelse mouse-here?
    [ if color != white [ set color white ] ]
    [ if color = white [ set color (5 + (10 * (who mod 14 ))) ] ]
    ifelse mouse-down?
    [ if dragged = nobody and mouse-here?
      [ set dragged self
        set selected self
      ]
    ]
    [ set dragged nobody ]
end

to lever-display
    if size != armlen * 2 * scale
    [ set size scale * 2 * armlen ]
    let shape-name "lever"
    if fixed?
    [ set shape-name word shape-name "-f" ]
    if spin?
    [ set shape-name word shape-name "-r" ]
    ifelse selected = self
    [ set shape-name word shape-name "-s"
      if label != (word who "-" armlen "    " ) [ set label (word who "-" armlen "    ") ]
    ]
    [ if label != no-label
    [ set label no-label ]
    ]
    if shape != shape-name
    [ set shape shape-name ]
end

to calc-arm-coords
    set x1 cx - dx * scale * armlen
    set y1 cy - dy * scale * armlen
    set x2 cx + dx * scale * armlen
    set y2 cy + dy * scale * armlen
end

to calc-hook-coords
    ifelse hook1 = self
    [ set h1x2 x1
      set h1y2 y1
      set h1cx cx - dx * scale * armlen
      set h1cy cy - dy * scale * armlen
    ]
    [ set h1x2 x2-of hook1
      set h1y2 y2-of hook1
      set h1cx cx-of hook1
      set h1cy cy-of hook1
    ]
    ifelse hook2 = self
    [ set h2x1 x2
      set h2y1 y2
      set h2cx cx + scale * dx * armlen
      set h2cy cy + scale * dy * armlen
    ]
    [ set h2x1 x1-of hook2
      set h2y1 y1-of hook2
      set h2cx cx-of hook2
      set h2cy cy-of hook2
    ]
    set ratio1 armlen / armlen-of hook1
    set ratio2 armlen / armlen-of hook2
end

to lever!
    ifelse dragged = self
    [ set cx mouse-xcor
      set cy mouse-ycor
    ]
    [
      if not fixed?
      [ set xv xv + ( ( h1x2 - x1 ) + ( h2x1 - x2 ) ) * .5 * k
        set yv yv + ( ( h1y2 - y1 ) + ( h2y1 - y2 ) ) * .5 * k - gravity * armlen

        set cx cx + xv
        set cy cy + yv

        if abs cx > screen-edge-x [
          hit-wall 1 (1.5 * (ycor + screen-edge-y + 30))
          set xv xv * -.9 set cx screen-edge-x * sign cx + xv
        ]
        if abs cy > screen-edge-y [
          if cy > screen-edge-y
            [ hit-wall 2 (1.5 * (xcor + screen-edge-x + 30)) ]
          set yv yv * -.9 set cy screen-edge-y * sign cy + yv
        ]
      ]
    ]
    ifelse stiff?
    [ new-heading-2 ]
    [ new-heading-1 ]

    if spin? = true [ set heading heading + spin-speed ]
    setxy cx cy
end

to new-heading-1
    ; calculate new heading of lever
    ; if node2 of hook1 is overlapping center, keep same heading
    ; other wise, find heading to node2 of hook1
    ifelse ( distancexy-nowrap h1x2 h1y2 = 0 )
    [ set from-h1 180 + heading ]
    [ set from-h1 180 + towardsxy-nowrap h1x2 h1y2 ]
    ; likewise node1 of hook2
    ifelse ( distancexy-nowrap h2x1 h2y1 = 0 )
    [ set to-h2 heading ]
    [ set to-h2 towardsxy-nowrap h2x1 h2y1 ]
    ; calc heading
    ;;; using "angular velocity"
    ;;set av av * f + k2 * (subtract-headings ( mean-heading from-h1 to-h2 ) heading )
    ;;set heading heading + av + dt
    set heading mean-heading from-h1 to-h2
end

to new-heading-2
    ; calculate new heading of lever
    ; if center of hook1 is overlapping center, keep same heading
    ; other wise, find heading to center of hook1
    ifelse ( distancexy-nowrap h1cx h1cy = 0 )
    [ set from-h1 180 + heading ]
    [ set from-h1 180 + towardsxy-nowrap h1cx h1cy ]
    ; likewise node1 of hook2
    ifelse ( distancexy-nowrap h2cx h2cy = 0 )
    [ set to-h2 heading ]
    [ set to-h2 towardsxy-nowrap h2cx h2cy ]
    ; calc heading
    set heading mean-heading from-h1 to-h2
end

to lever!2
    ; randomly snaps lever to one or the other hook
    ; an attempt to make the hooks NOT stretch.
    calc-hook-coords
    let which-hook? 0
    ifelse hook1 = self
    [ set which-hook? 2 ]
    [ ifelse hook2 = self
      [ set which-hook? 1 ]
      [ set which-hook? 1 + random 2 ]
    ]
    let a 0 let b 0 let c 0
    ifelse which-hook? = 1
    [ set a h1x2
      set b h1y2
      set c 180
    ]
    [ set a h2x1
      set b h2y1
      set c 0

    ]
    let real-angl heading
    let real-dist distancexy a b
    if real-dist > 0
    [ set ideal-angl c + towardsxy a b
      set heading ideal-angl
    ]
    if fixed? = false
    [
    set ideal-dist armlen * scale
    if real-dist != ideal-dist
    [ jump abs ( real-dist - ideal-dist )
    ]
    ; set av av + subtract-headings real-angl ideal-angl
    set cx xcor
    set cy ycor
    ]
    calc-arm-coords


end

to build-crazy-machine
  without-interruption
  [ ask levers
  [ set cx random-float 1 * screen-edge-x * (random 2 * 2 - 1 ) * .8
    set cy random-float 1 * screen-edge-y * .8
    set xv 0
    set yv 0
    setxy cx cy
    set fixed? ( who mod 2 = 0 and random 5 = 0 )
    ifelse fixed? [ set spin? (random 2 = 0) ]
    [ set spin? ( random 4 = 0) ]
    set armlen precision ( .5 + .5 * random-float 2 ) 2
  ]
  ]
end


to reset
  ask turtles
  [ set fixed? false
    set armlen .5
    set spin? false
  ]
  ask turtle 0
  [ set fixed? true
    set cx 0
    set cy (screen-edge-y * .9 )
    setxy cx cy
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;    sound procedures
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-music
  set pitches []
  let offset 0
  ;; C-D-E-G-A (0-2-4-7-9) is an example of a pentatonic scale; the
  ;; pitches are chosen so they that generally all sound good
  ;; together, in any combination
  let five-notes (list pitch1 pitch2 pitch3 pitch4 pitch5)
  ;; now take the pentatonic scale and duplicate it across many octaves,
  ;; to give us a big set of pitches to pick from
  while [length pitches < 50]
    [ set pitches sentence pitches map [? + offset] five-notes
      set offset offset + 12 ]
  ;; don't use the drums that are listed next; this helps keep
  ;; the drums sound somewhat like an actual drumkit
  set drumkit filter [not member? ? ["LOW AGOGO" "HI AGOGO" "LONG WHISTLE"
                                     "MUTE TRIANGLE" "SHORT GUIRO"
                                     "OPEN TRIANGLE" "OPEN CUICA"
                                     "SHORT WHISTLE" "LONG GUIRO"
                                     "CLAVES" "TAMBOURINE" "CABASA"
                                     "MARACAS" "HI WOOD BLOCK" "MUTE CUICA"]]
              drums
  ;; a lever's drum isn't determined until the lever hits the wall.
  ;; but each lever's instrument stays constant.
  ask levers
    [ set drum ""
      set instrument item (who * 3 / count levers) (list instrument1 instrument2 instrument3) ]
end

;; this procedure is used to restrict the pitches we choose
;; to come from the pitch set we made early
to-report nearby [p]
  report first filter [? >= p] pitches
end

to chorus
  ;; every so often, we "modulate" by moving all the pitches up
  ;; a few steps.  this helps keep things interesting.  suggestion:
  ;; instead of happening on a regular schedule, maybe modulation
  ;; could happen in response to something the machine does...?
  if ticks mod 1000 = 500
    [ set pitches map [? + modulation] pitches ]
  ;; then a little later, we "modulate" back down
  if ticks mod 1000 = 900
    [ set pitches map [? - modulation] pitches ]
  ;; we use "every" to synchronize all the sounds to a fixed
  ;; rhythm.  also, we use the who numbers of the levers to make some levers
  ;; change pitch only rarely, others more often.  the goal is to give the
  ;; music both stability and variety.
  every sing-interval
    [ ask levers with [ticks mod (ceiling ((who + 1) / chorusing)) = 0]
        [ sing ] ]
  ;; a lever's drum is set when it hits a wall.  the loudness of each
  ;; drum hit is random.  once the drum is hit, we remove it from the lever.
  every drum-interval
    [ ask levers with [drum != ""]
        [ play-drum drum 100 + random 20
          set drum "" ] ]
end


to sing  ;; lever procedure
  ;; fixed levers don't sing, since they'd always be
  ;; singing the same note.
  ifelse fixed?
    [ stop-note instrument pitch ]
    [ ;; the new pitch is chosen based on the x
      ;; coordinate of the lever, and the volume is based on the lever's
      ;; orientation.
      let new-pitch nearby (xcor + screen-edge-x + 30)
      if pitch != new-pitch
        [ ;; first stop "singing" our old pitch
          stop-note instrument pitch
          set pitch new-pitch
          start-note instrument pitch ((heading + ticks mod pitch-cycle) mod pitch-ratio) ] ]
end

to hit-wall [num p]
  ;; when a lever hits a wall, what drum you hear depends both on the
  ;; position along the wall and the who number of the lever.
  ;; note: num is ignored for now; you could try and see if using it
  ;; leads to interesting results.  num varies according to whether
  ;; the wall is a side wall, or the ceiling.
  set drum item ((p + who) mod length drumkit) drumkit
end


; (C) 2004 Uri Wilensky.  This code may be freely copied, distributed,
; altered, or otherwise used by anyone for any legal purpose.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
; OWNERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
@#$#@#$#@
GRAPHICS-WINDOW
215
10
582
398
25
25
7.0
1
10
1
1
1
0

CC-WINDOW
5
621
826
716
Command Center

BUTTON
13
79
77
112
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
80
79
143
112
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
11
32
211
65
population
population
3
25
7
1
1
NIL

SLIDER
16
238
212
271
scale
scale
0.5
15
10.0
0.5
1
NIL

SLIDER
16
204
212
237
gravity
gravity
0
0.095
0.03
1.0E-4
1
NIL

BUTTON
21
375
84
408
fix/free
ask selected [ set fixed? not fixed? ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
21
411
84
444
bigger
ask selected [ set armlen precision ( armlen + .1 ) 3 ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
88
411
149
444
smaller
ask selected [ if armlen > .1 [ set armlen precision ( armlen - .1 ) 3 ] ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
696
104
784
137
reset-all
reset
NIL
1
T
OBSERVER
T
NIL

BUTTON
153
389
209
430
reset
ask selected [ set armlen .5 ]
NIL
1
T
OBSERVER
T
NIL

SLIDER
16
169
212
202
spin-speed
spin-speed
-100
100
1.0
0.01
1
NIL

BUTTON
88
375
149
408
spin?
ask selected [ set spin? not spin? ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
672
70
811
103
NIL
build-crazy-machine
NIL
1
T
OBSERVER
T
NIL

SWITCH
16
274
106
307
stiff?
stiff?
1
1
-1000

TEXTBOX
13
357
176
375
properties of selected lever:

BUTTON
607
70
670
103
jolt
rt 90
NIL
1
T
TURTLE
NIL
NIL

BUTTON
607
104
695
137
scramble
set cx random-float screen-size-x\nset cy random-float screen-size-y
NIL
1
T
TURTLE
NIL
NIL

TEXTBOX
12
322
179
351
(you can select, move, and edit the "levers" with the mouse)

TEXTBOX
9
132
159
160
These can be changed as the model runs:

MONITOR
150
71
207
120
NIL
ticks
3
1

TEXTBOX
597
22
726
64
suggestion: try the build-crazy-machine button!

BUTTON
610
183
713
216
NIL
stop-music
NIL
1
T
OBSERVER
T
NIL

TEXTBOX
597
150
747
178
only works if you stop the go button, too:

TEXTBOX
596
227
746
255
Music sliders (OK to change as model runs):

SLIDER
602
259
817
292
sing-interval
sing-interval
0.05
1.0
0.2
0.05
1
NIL

SLIDER
602
293
817
326
drum-interval
drum-interval
0.05
1.0
0.2
0.05
1
NIL

SLIDER
602
327
817
360
chorusing
chorusing
1
10
3
1
1
NIL

SLIDER
602
361
817
394
modulation
modulation
0
32
5
1
1
NIL

SLIDER
602
395
817
428
pitch-cycle
pitch-cycle
1
1000
371
1
1
NIL

SLIDER
602
429
817
462
pitch-offset
pitch-offset
0
60
30
1
1
NIL

SLIDER
602
463
817
496
pitch-ratio
pitch-ratio
1
500
120
1
1
NIL

CHOOSER
237
438
377
483
instrument1
instrument1
"ocarina" "drawbar organ" "clavi"
0

CHOOSER
237
484
377
529
instrument2
instrument2
"ocarina" "drawbar organ" "clavi"
1

CHOOSER
237
530
377
575
instrument3
instrument3
"ocarina" "drawbar organ" "clavi"
2

TEXTBOX
231
410
429
438
These take effect only at setup time:

SLIDER
393
438
565
471
pitch1
pitch1
0
11
0
1
1
NIL

SLIDER
393
472
565
505
pitch2
pitch2
0
11
2
1
1
NIL

SLIDER
393
506
565
539
pitch3
pitch3
0
11
4
1
1
NIL

SLIDER
393
540
565
573
pitch4
pitch4
0
11
7
1
1
NIL

SLIDER
393
574
565
607
pitch5
pitch5
0
11
9
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This is a music-making machine.  Actually, it's a music-machine-making-machine!  The sounds the machines make are always different, yet always similar.


HOW IT WORKS
------------
The pieces of the machine are called "levers".  The levers are attached to each other in a chain, so they pull on each other.  They're also subject to gravity.  Some levers spin.

As the levers move around, their positions and angles are used to generate pitches for sounds.


HOW TO USE IT
------------
You can select individual levers with the mouse and drag them around.  Once a lever is selected, you can use the buttons on the lower left to change the characteristics of the lever.

To make a bunch of random levers, press the BUILD-CRAZY-MACHINE button.

You can discover what the rest of the controls do by experimenting with them.


EXTENDING THE MODEL
-------------------
By tinkering with the code, you can drastically change the sound of the music the levers make.  You might focus on making machines that can make interesting music by themselves, without any intervention from you.  Or, you could try to make them especially responsive to your actions witht the mouse, so you can "play" the machine like a musical instrument.


NETLOGO FEATURES
----------------
This uses NetLogo's sound "extension", which adds music and sound capabilities to NetLogo.  See the Sound section of the NetLogo User Manual for more details.


CREDITS AND REFERENCES
----------------------
This is derived from an original model by James Steiner called "Machines".  Thanks to James for submitting it.  We made some changes and added sound.  The original model is available from our user community models site, http://ccl.northwestern.edu/netlogo/models/community/ .
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

ball
true
10
Circle -16776961 true true 15 15 270

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

lever
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105

lever-f
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Rectangle -16776961 true true 135 135 166 166

lever-f-r
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Line -16776961 true 61 30 240 30
Line -16776961 true 240 30 195 44
Line -16776961 true 238 29 193 15
Line -16776961 true 239 269 61 270
Line -16776961 true 61 270 106 255
Line -16776961 true 104 285 61 271
Rectangle -16776961 true true 135 135 165 165

lever-f-r-s
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Line -16776961 true 61 30 240 30
Line -16776961 true 240 30 195 44
Line -16776961 true 238 29 193 15
Line -16776961 true 239 269 61 270
Line -16776961 true 61 270 106 255
Line -16776961 true 104 285 61 271
Rectangle -16776961 true true 135 135 165 165
Circle -16776961 false true 29 30 240

lever-f-s
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Circle -16776961 false true 30 30 240
Rectangle -16776961 true true 135 135 165 165

lever-r
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Line -16776961 true 61 30 240 30
Line -16776961 true 240 30 195 44
Line -16776961 true 238 29 193 15
Line -16776961 true 239 269 61 270
Line -16776961 true 61 270 106 255
Line -16776961 true 104 285 61 271

lever-r-s
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Line -16776961 true 61 30 240 30
Line -16776961 true 240 30 195 44
Line -16776961 true 238 29 193 15
Line -16776961 true 239 269 61 270
Line -16776961 true 61 270 106 255
Line -16776961 true 104 285 61 271
Circle -16776961 false true 29 29 242

lever-s
true
10
Circle -16776961 true true 120 -30 60
Circle -16776961 true true 121 270 58
Line -16776961 true 150 30 150 269
Polygon -16776961 true true 150 90 195 105 149 30 105 105
Circle -16776961 false true 30 30 240

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.2pre2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
