globals [
    times
]

breeds [
    reds
    whites
]

patches-own [
    white-here?
    red-here?
]

to setup
    ca
    set times 0

    ask patches [
        ;;set all patches to "unoccupied"
        set white-here? false
        set red-here? false

        draw-road
        draw-bounds
    ]

    create-white-peds
    create-red-peds
end

to draw-road ;; patch procedure
    if ((pxcor < (screen-edge-x - 20)) and (pxcor > (20 - screen-edge-x))) [
        set pcolor grey
    ]
    if (pxcor = 0) [
        set pcolor yellow
    ]
end

to draw-bounds ;; patch procedure
    if (pycor = (width / 2)) [
        set pcolor white
    ]
    if (pycor = (- (width / 2))) [
        set pcolor white
    ]
end

to create-white-peds                        ;;create pedestrians on left and set to white
    ask patches with [
        ((pxcor = (- screen-edge-x)) AND
        (pycor > (- (width / 2))) AND
        (pycor < (width / 2)) AND
        (red-here? = false) AND
        (white-here? = false))
    ][
        if ((random-float 100) < density) [
             sprout 1 [
                set color white
                set breed whites
                set heading 90                ;;set direction to left
            ]
            set white-here? true
        ]
    ]
end

to create-red-peds                        ;;create pedestrians on right and set to red
    ask patches with [
        ((pxcor = (screen-edge-x)) AND
        (pycor > (- (width / 2))) AND
        (pycor < (width / 2)) AND
        (red-here? = false) AND
        (white-here? = false))
    ][
        if ((random-float 100) < density) [
             sprout 1 [
                set color red
                set breed reds
                set heading 270                ;;set direction to left
            ]
            set red-here? true
        ]
    ]
end

to walk
    ask whites [
        whites-find-heading
        whites-move
        set heading 90 ;; reset heading
    ]

    ask reds [
        reds-find-heading
        reds-move
        set heading 270 ;; reset heading
    ]

    set times (times + 1)
    if (times < columns) [
        create-white-peds
        create-red-peds
    ]
end

to reds-find-heading ;; reds procedure
    ;; heading starts out at 270 (left)

    ;; someone in the way?
    if ((white-here?-of patch-ahead 1) = true) [

        ;; try to follow your way out
        ifelse ((red-here?-of patch-at -1 0) = true) [
            set heading (towards patch-at -1 0)
        ][
            ifelse ((red-here?-of patch-at -1 -1) = true) [
                set heading (towards patch-at -1 -1)
            ][
                ifelse ((red-here?-of patch-at -1 1) = true) [
                    set heading (towards patch-at -1 1)
                ][

                    ;; no one to follow - flip a coin
                    ifelse ((random 2) = 0) [
                        set heading 0
                    ][
                        set heading 180
                    ]
                ]
            ]
        ]
    ]
end

to whites-find-heading ;; whites procedure
    ;; heading starts out at 90 (right)

    ;; someone in the way?
    if ((red-here?-of patch-at 1 0) = true) [

        ;; try to follow your way out
        ifelse ((white-here?-of patch-at 1 0) = true) [
            set heading (towards patch-at 1 0)
        ][
            ifelse ((white-here?-of patch-at 1 -1) = true) [
                set heading (towards patch-at 1 -1)
            ][
                ifelse ((white-here?-of patch-at 1 1) = true) [
                    set heading (towards patch-at 1 1)
                ][

                    ;; no one to follow - flip a coin
                    ifelse ((random 2) = 0) [
                        set heading 0
                    ][
                        set heading 180
                    ]
                ]
            ]
        ]
    ]
end

to reds-move ;; reds procedure
    ifelse ((continue? = false) AND
            ((pxcor - 1) < (- screen-edge-x))) [ ;;use patch coordinates go get integers
        ;; at the edge: don't do aything
    ][
        if ((red-here?-of patch-ahead 1 = false) AND
            (white-here?-of patch-ahead 1 = false) AND
            (pycor-of patch-ahead 1 < (width / 2)) AND
            (pycor-of patch-ahead 1 > (- (width / 2)))) [    ;;if the space ahead is open and in bounds...

            set red-here? false                ;;set old patch to "unoccupied"
            fd 1                                ;;move forward
            set red-here? true                ;;set new patch to "occupied"
        ]
    ]
end

to whites-move ;; whites procedure
    ifelse ((continue? = false) AND
            ((pxcor + 1) > screen-edge-x)) [ ;;use patch coordinates to get integers
        set white-here? false
        set xcor screen-edge-x ;; make sure to get all the way to the edge
        set white-here? true
    ][
        if ((red-here?-of patch-ahead 1 = false) AND
            (white-here?-of patch-ahead 1 = false) AND
            (pycor-of patch-ahead 1 < (width / 2)) AND
            (pycor-of patch-ahead 1 > (- (width / 2)))) [    ;;if the space ahead is open and in bounds...

            set white-here? false                ;;set old patch to "unoccupied"
            fd 1                                ;;move forward
            set white-here? true                ;;set new patch to "occupied"
        ]
    ]
end

to unjam
    ask turtles [
        ifelse ((random 2) = 0) [
            set heading 180                    ;;change up directions to break up jams
        ][
            set heading 0
        ]
    ]
end

;;Copyright 1998 by Uri Wilensky. All rights reserved.
;;Converted from StarLogoT to NetLogo, 8/6/2001.

;;Permission to use, copy, or modify this software and its
;;documentation for educational and research purposes only and without
;;fee is hereby granted, provided that this copyright notice and the
;;original authors' names appear on all copies and supporting
;;documentation. For any other uses of this software, in original or
;;modified form, including but not limited to distribution in whole or
;;in part, specific prior permission must be obtained from Uri
;;Wilensky. These programs shall not be used, rewritten, or adapted as
;;the basis of a commercial software or hardware product without first
;;obtaining appropriate licenses from Uri Wilensky. We make no
;;representations about the suitability of this software for any
;;purpose. It is provided "as is" without express or implied warranty.
@#$#@#$#@
GRAPHICS-WINDOW
321
18
637
355
25
25
6.0
0
10
1
1
1
0

CC-WINDOW
5
369
646
464
Command Center

SLIDER
6
186
151
219
width
width
2
52
40
2
1
NIL

SLIDER
6
121
150
154
density
density
1.0
18.0
18.0
1.0
1
NIL

BUTTON
6
50
102
103
setup
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
156
122
315
155
columns
columns
0
50
50
1
1
NIL

SWITCH
157
185
315
218
continue?
continue?
0
1
-1000

BUTTON
108
50
209
103
walk
walk
T
1
T
OBSERVER
T
NIL

BUTTON
215
50
315
104
unjam
unjam
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
What is it?
-----------

This is a model of pedestrian crosswalk traffic. Each red turtle represents a pedestrian moving from the right to the left, each white moving left to right. Social scientists have observed that human banding is commonly displayed in high density crosswalks. In place of random movements, people move in coordination with others traveling in the same direction in order to avoid oncoming pedestrian traffic.

In order to demonstrate such a phenomenon, this model assumes two general laws of travel followed by humans within the crosswalk: 1) When confronted head-on by a human moving in the opposite direction, each human will step either to the right or left. 2) In order to maximize efficiency, each human will follow the travel of an individual in front of them, whether he/she is directly or diagonally ahead of them.


How to use it?
--------------

This model is meant to model the emergent properties witnessed in a social phenomenon.  More specifically, it attempts to re-create, through simple rules, what researchers have only been able to observe in real life.  It is also meant to be a measure of the different effects of such factors as "density" "width" and "columns".

The DENSITY slider is the per cent probability that a patch at the screen's edge will sprout a pedestrian. The higher the density, the more pedestrians there are.

The COLUMNS slider determines how many waves of pedestrians enter the crosswalk. In other words, the slider determines how many columns of turtles approach the road from each direction.

The WIDTH slider determines the total area of the crosswalk. The white lines illustrate the current width and the borders for where turtles are allowed to travel. To eliminate width as a factor in the simulation, set WIDTH to its maximum (52).

The CONTINUE switch allows the user to choose the turtle's activities once it reaches its destination. Once the turtle makes it across the road, it can either stop at the edge of the screen, therefore causing backup across the screen, or can continue to the other side of the screen along its current pattern. When the continue switch is set to on, the turtles move along in a normal manner. When it is set to off, the turtles stack up at the edge of the screen so as to display the banding lines which they formed within the model.

The UNJAM button redirects every turtle momentarily in order to free up "traffic jams" within the model.

Extending the model.
--------------------

The model could be extended in several ways. As it is, at a small width, the model is subject to frequent backups or "jams", which require the UNJAM button to clear up. While the intent of this is to show the natural patterns of backups, it could also be an area for expansion. How could one program each turtle to get itself out of a "jam"? What is the most efficient way to unclog backup?

The model currently assumes that people only wish to move left and right within the crosswalk. What would happen if turtles were also programmed with the intention of travelling diagonally?

Additionally, we can ask questions about other variables: What happens if we remove the assumption that each turtle moves at the same speed? Can we add a traffic signal?  What if some turtles attempt to break out of the bands and form their own paths?

Can you alter the program so that thicker bands form?

NETLOGO FEATURES
-------------------

This model highlights the use of the SPROUT function. In order to assign only one turtle to a patch at SETUP, the model uses SPROUT instead of CREATE-TURTLES. By doing so, each patch at SCREEN-EDGE-X and -SCREEN-EDGE-X determine whether or not to create a turtle based upon the user's input through the density slider.

CREDITS AND REFERENCES
----------------------
This model derives from a study by Katsuya Yamori at Nara University entitled "Going With the Flow: Micro - Macro Dynamics in the Macrobehavioral Patterns of Pedestrian Crowds. The article can be found in:

  Psychological Review. 1998. Vol. 105, No. 3, 530-557
  Copyright 1998 by the American Psychological Association, Inc.

(worked on by Nathan Jones, Andrew Russell, and Uri Wilensky)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
setup repeat 150 [ walk ]
@#$#@#$#@
@#$#@#$#@
