turtles-own [xpos ypos]

to setup-curve
  ca
  ask patches [if (pxcor = 0) or (pycor = 0) [set pcolor white]]   ;Create the axes
  crt 1
  ask turtles [                 ;Create the initial turtle and have it begin the sequence
    set xpos initial-x          ;of recursive calls to create-next-point
    set ypos initial-y
    set color blue
    setxy xpos ypos
    create-next-point xpos ypos
  ]
end

to create-next-point [prev-xpos prev-ypos]      ;Turtle procedure to create the next data point
  hatch 1 [
    set xpos (prev-xpos * x-scaling-factor)     ;Use xpos and ypos in addition to turtle coords
    set ypos (prev-ypos * y-scaling-factor)     ;because coords greater than the screen edges
    show-or-hide                                ;get wrapped and thus distorted
    setxy xpos ypos
    if (xpos < screen-edge-x) [         ;Keep creating turtles until we hit the right end of the screen
      create-next-point (prev-xpos * x-scaling-factor) (prev-ypos * y-scaling-factor)
    ]
  ]
end

to log-scale    ;Turtle procedure
  setxy (log xpos 10) (log ypos 10)
  ifelse (xcor < screen-edge-x) [st] [ht]
end

to scale [direction]    ;Multiply the data by scale-factor in the given direction
  ask turtles [
    ifelse (direction = "x")
      [set xpos (xpos * scale-factor)]
      [set ypos (ypos * scale-factor)]
    show-or-hide
    setxy xpos ypos
  ]
end

to show-or-hide         ;Turtle procedure to only show turtles located within the screen
  ifelse (xpos > screen-edge-x or ypos > screen-edge-y)
    [ht]
    [st]
end

to show-slope           ;Show the slope of the line in logarithmic scales in the command center
  let xcors (values-from turtles [xcor])
  let ycors (values-from turtles [ycor])
  show ((last ycors) - (first ycors)) / ((last xcors) - (first xcors))
end
@#$#@#$#@
GRAPHICS-WINDOW
297
10
610
344
50
50
3.0
0
10
1
1
1
0

CC-WINDOW
5
358
619
453
Command Center

BUTTON
18
206
90
239
log-scale
log-scale
NIL
1
T
TURTLE
T
NIL

BUTTON
19
27
98
60
setup-curve
setup-curve
NIL
1
T
OBSERVER
T
NIL

SLIDER
18
96
149
129
x-scaling-factor
x-scaling-factor
1.0
5.0
2.0
0.1
1
NIL

SLIDER
18
137
148
170
y-scaling-factor
y-scaling-factor
0.1
4.0
0.5
0.1
1
NIL

SLIDER
162
96
282
129
initial-x
initial-x
1.0E-4
1.0
1.0E-4
0.0010
1
NIL

SLIDER
162
135
282
168
initial-y
initial-y
0.1
10.0
6.1
0.1
1
NIL

BUTTON
16
272
91
305
scale x
scale "x"
NIL
1
T
OBSERVER
T
NIL

BUTTON
16
310
90
344
scale y
scale "y"
NIL
1
T
OBSERVER
T
NIL

TEXTBOX
19
76
140
94
How fast data grows

TEXTBOX
162
76
260
94
Initial data point

TEXTBOX
19
186
148
204
View data on log scale

TEXTBOX
17
251
179
269
scale data in either direction

BUTTON
98
206
178
239
NIL
show-slope
NIL
1
T
OBSERVER
T
NIL

SLIDER
100
288
225
321
scale-factor
scale-factor
0.1
2.0
1.2
0.1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This model produces data according to the so-called power law distribution.  Data shares a power law relation when its x and y coordinates each change by a constant factor, creating exponential growth (or shrinkage) in both directions.  An interesting property of such data is that when plotted on logarithmic scales, a straight line is produced, whose slope can be obtained from the scaling factors in the x and y directions.

A common example of the power law distribution is when the y scale factor is 1/2 and the x scale factor is 2, meaning the x coordinate of each new point is double that of the previous one, while the y coordinate has been halved.  When plotted on logarithmic scales, a straight line of slope -1 emerges.  This example is most often found in situations where the scale (or size) of a phenomenon is plotted along the x axis and the frequency of that phenomena at a given scale is plotted along the y axis.  In these cases, a phenomenon that is twice the size of another one occurs only half as often, meaning that while small events are typical, arbitrarily large events do happen.  This example is an instance of a more general family of distributions called inverse power law distributions, which occur when data fits a function of the form y = k / (x ^ a), where k and a are constants.

Power law distributions are one of the hallmarks of complex systems.  They have been found to hold true for phenomena as diverse as earthquakes, human population density, speciation and extinction of organisms, and technological innovation.  The explanation for their ubiquity in the social and physical world is that systems exhibiting an inverse power law exhibit neither too much order to change and adapt nor too much chaos to behave consistently.  They lie at what complexity scientists call "the edge of chaos".  It is believed that systems tend to evolve towards this region between order and chaos because it is where they are most able to thrive in their environment.

HOW TO USE IT
-------------
Generate data by using X-SCALING-FACTOR and Y-SCALING-FACTOR to determine how the coordinates of each point will grow.  A scale factor of 2 for x means that each new point generated will have twice the x value of the previous one.  Choose the location of the initial data point with  INITIAL-X and INITIAL-Y.  If your scale factor in a particular direction is greater than one, a small initial value is advisable, while if it is less than one, a large one is.

Once the data sliders have been set, press SETUP-CURVE to produce the data.  You can use SCALE X and SCALE Y to scale the points in either direction by multiplying them by SCALE-FACTOR, making the curves easier to see.

To view the data on a log scale, press LOG-SCALE.  To see the slope of the line produced, hit VIEW-SLOPE, and it will appear in the command center.

THINGS TO NOTICE
----------------
When X-SCALING-FACTOR and Y-SCALING-FACTOR are both greater than 1 and are more than 1 apart, the data grows so much faster in the direction of the larger factor that it appears to form a straight line.

No matter what scaling factor or initial point you use to generate data, a straight line always emerges when the data is viewed in logarithmic scales.

Every time you scale, the line seen on logarithmic scales is translated.  Scaling by a factor greater than 1 translates the data in a positive direction, while a factor of less than one translates it in a negative direction.

THINGS TO TRY
-------------


EXTENDING THE MODEL
-------------------


NETLOGO FEATURES
-------------------

CREDITS AND REFERENCES
----------------------
(this model was begun by Ben Shargel)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
