breeds [large-fish
        medium-fish
        small-fish
       ]

turtles-own [curr-best-h         ;; Best heading (towards best patch)
                                 ;;    found so far in search
             curr-best-ele       ;; Best patch elevation found so far in search
             curr-best-sp        ;; Best water speed found so far in search
             size-val
            ]

patches-own [elevation           ;; Deeper water has a smaller elevation value
             water-speed         ;; Shallow water tends to have a higher speed
             speed-set-counter   ;; Index variable for setting up water speed increment
            ]

globals [highest                 ;; Maximum elevation over all patches
         lowest                  ;; Minimum elevation over all patches
         fastest                 ;; Maximum water-speed over all patches
         slowest                 ;; Minimum water-speed over all patches
         head-upstream-chance    ;; Global chance that a fish will decide to head upstream
        ]

to setup
  ca
  set head-upstream-chance 80
  setup-patches
  setup-turtles
end

to setup-patches
  ;; First, determine elevation of each patch.
  ask patches
    [set elevation (random 1000)]
  repeat 2
    [diffuse elevation .8]
  set highest (max values-from patches [elevation] )
  set lowest (min values-from patches [elevation] )
  ask patches
    ;; Patches within 100 units of the highest area are considered 'dry land'
    [if (elevation > (highest - 100) )
       [set pcolor brown]
     if (pxcor = screen-edge-x)
       [set elevation ( (lowest + highest) / 2) ]
     setup-currents
    ]

  repeat 2
    [diffuse water-speed .75]
  ask patches
    [if (pxcor = screen-edge-x) or (pxcor = screen-edge-x - 1)
       [set water-speed ((lowest + highest) / 2)]
    ]
  set fastest (max values-from patches [water-speed])
  set slowest (min values-from patches [water-speed])
  ask patches
  ;; Brown patches are dry land and have bogus values for water-speed and elevation
    [ifelse (pcolor = brown)
       [set water-speed 9999
        set elevation 9999
       ]
     ;; Set a patches' color according to its depth or speed,
     ;; and toggle water view based on switch: either we see the depth or the speed
       [ifelse (water-speed-view)
          [set pcolor scale-color blue elevation lowest (highest * .92)]
          [set pcolor scale-color blue water-speed slowest (fastest + 10) ]
       ]
    ]
end

to setup-turtles
  create-small-fish (round (0.50 * number) )
  ask small-fish
    [set shape "fish small"
     set color green
     set size-val 1
    ]
  create-medium-fish (round (0.35 * number) )
  ask medium-fish
    [set shape "fish medium"
     set color red
     set size-val 2
    ]

  create-large-fish (round (0.15 * number) )
  ask large-fish
    [set shape "fish large"
     set color yellow
     set size-val 3
    ]
  ask turtles
    [setxy ( (random-float (2 * screen-edge-x) ) - screen-edge-x )
           ( (random-float (2 * screen-edge-y) ) - screen-edge-y )
     find-an-empty-patch
     ;; Integer values are easiwer to work with on our patch grid
     set xcor (round xcor)
     set ycor (round ycor)
     set heading ((random 8) * 45)
    ]
end

to find-an-empty-patch
  ifelse (any? other-turtles-here) or (pcolor = brown)
    [rt random-float 360
     fd 1
     find-an-empty-patch
    ]
    [stop]
end

to setup-currents            ;; patch procedure
  if (pxcor = screen-edge-x)
    [set water-speed ((lowest + highest) / 2)]
  set speed-set-counter (screen-edge-x - 1)
  repeat (screen-size-x - 1)
    [if (pxcor = speed-set-counter)
       [set water-speed (water-speed-of patch-at 1 0)
        ifelse ((pcolor-of patch-at 1 0) != brown)
        ;; water
          [ifelse ((elevation-of patch-at 1 0) > elevation)
             ;; patch upstream is shallower
             [set water-speed (water-speed - (0.1 * ((elevation-of patch-at 1 0) - elevation)))]
             ;; patch upstream is deeper
             [set water-speed (water-speed + (0.1 * (elevation - (elevation-of patch-at 1 0))))]
          ]
        ;; land
          [set water-speed (water-speed - 10)
          ]
       ]
     set speed-set-counter (speed-set-counter - 1)
    ]
end

;; RUN-TIME PROCEDURES

to go
  ;; move all the fish
  move-turtles

  ;; Toggle water view based on switch: either we see the depth or the speed
  ask patches
    [if (pcolor != brown)
       [ifelse (water-speed-view)
          [set pcolor scale-color blue elevation lowest (highest * .92)]
          [set pcolor scale-color blue water-speed slowest (fastest + 10)]
       ]
    ]
  if (slowdown?)
    [wait 0.2]
end

to move-turtles
  ask turtles
    [pick-desired-heading]
  move-large-fish
  move-medium-fish
  move-small-fish
end

to move-large-fish
  ask large-fish
    [if (curr-best-h != -1)
       [set heading curr-best-h
        if ((pcolor-of patch-ahead 1) != brown)
          [fd 1]
       ]
     resolve-fish-conflicts
    ]
end

to move-medium-fish
  ask medium-fish
    [if (curr-best-h != -1)
       [set heading curr-best-h
        ifelse any? (turtles-on patch-ahead 1) with [shape = "fish large"]
          ;; there's a bigger fish where I want to be... time to split...
          [get-lost]
          ;; there's NOT a bigger fish- in I go.
          [if ((pcolor-of patch-ahead 1) != brown)
             [fd 1]
          ]
       ]
     resolve-fish-conflicts
    ]
end

to move-small-fish
  ask small-fish
    [if (curr-best-h != -1)
       [set heading curr-best-h
        ifelse any? turtles-on patch-ahead 1
          [get-lost]
          [if ((pcolor-of patch-ahead 1) != brown)
             [fd 1]
          ]
       ]
     resolve-fish-conflicts
    ]

end

to get-lost
  ifelse ((random-float 100) < head-upstream-chance)
    ;; try to go upstream
    [ifelse ((pcolor-of patch-at 1 0) != brown)
       [set heading 90
        fd 1
       ]
       [ifelse ((pcolor-of patch-at 0 -1) != brown)
          [set heading 180
           fd 1
          ]
          [if ((pcolor-of patch-at 0 1) != brown)
             [set heading 0
              fd 1
             ]
          ]
       ]
    ]
    [if ((pcolor-of patch-at -1 0) != brown)
       [set heading 270
        fd 1
       ]
    ]
end

;; RESOLVE-FISH-CONFLICTS tries to minimalize stacking of fish, gives precedence
;;    to fish of greater size when a fish is not the sole occupant of a patch
to resolve-fish-conflicts           ;; turtles procedure
  if (any? other-turtles-here)
    [ifelse (shape = "fish small")
       [get-lost]
       [ifelse (shape = "fish medium")
          [ifelse (any? small-fish-here)
             [ifelse (count small-fish-here + 1 != count turtles-here)
                [ask small-fish-here
                   [get-lost]
                 get-lost
                ]
                [ask small-fish-here
                   [get-lost]
                ]
             ]
             [get-lost]
          ]
          [ifelse (count small-fish-here + count medium-fish-here + 1 != count turtles-here)
             [ask small-fish-here
                [get-lost]
              ask medium-fish-here
                [get-lost]
              get-lost
             ]
             [ask small-fish-here
                [get-lost]
              ask medium-fish-here
                [get-lost]
             ]
          ]
       ]
    ]
end

;; SEARCH ROUTINES
;; also try 'find-deepest' and 'find-slowest'
to pick-desired-heading
  find-deepest-and-slowest
end

to find-deepest
  set curr-best-h -1
  set curr-best-ele elevation
  if ((elevation-of patch-at -1 1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at -1 1)
     set curr-best-h 315
    ]
  if ((elevation-of patch-at 0 1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at 0 1)
     set curr-best-h 0
    ]
  if ((elevation-of patch-at 1 1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at 1 1)
     set curr-best-h 45
    ]
  if ((elevation-of patch-at 1 0) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at 1 0)
     set curr-best-h 90
    ]
  if ((elevation-of patch-at 1 -1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at 1 -1)
     set curr-best-h 135
    ]
  if ((elevation-of patch-at 0 -1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at 0 -1)
     set curr-best-h 180
    ]
  if ((elevation-of patch-at -1 -1) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at -1 -1)
     set curr-best-h 225
    ]
  if ((elevation-of patch-at -1 0) < curr-best-ele)
    [set curr-best-ele (elevation-of patch-at -1 0)
     set curr-best-h 270
    ]
end

to find-slowest
  set curr-best-h -1
  set curr-best-sp water-speed
  if ((water-speed-of patch-at -1 1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at -1 1)
     set curr-best-h 315
    ]
  if ((water-speed-of patch-at 0 1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at 0 1)
     set curr-best-h 0
    ]
  if ((water-speed-of patch-at 1 1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at 1 1)
     set curr-best-h 45
    ]
  if ((water-speed-of patch-at 1 0) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at 1 0)
     set curr-best-h 90
    ]
  if ((water-speed-of patch-at 1 -1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at 1 -1)
     set curr-best-h 135
    ]
  if ((water-speed-of patch-at 0 -1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at 0 -1)
     set curr-best-h 180
    ]
  if ((water-speed-of patch-at -1 -1) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at -1 -1)
     set curr-best-h 225
    ]
  if ((water-speed-of patch-at -1 0) < curr-best-sp)
    [set curr-best-sp (water-speed-of patch-at -1 0)
     set curr-best-h 270
    ]
end

to find-deepest-and-slowest
  set curr-best-h -1
  set curr-best-ele elevation
  set curr-best-sp water-speed
  if (((elevation-of patch-at -1 1) < curr-best-ele) and ((water-speed-of patch-at -1 1) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at -1 1)
     set curr-best-sp (water-speed-of patch-at -1 1)
     set curr-best-h 315
    ]
  if (((elevation-of patch-at 0 1) < curr-best-ele) and ((water-speed-of patch-at 0 1) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at 0 1)
     set curr-best-sp (water-speed-of patch-at 0 1)
     set curr-best-h 0
    ]
  if (((elevation-of patch-at 1 1) < curr-best-ele) and ((water-speed-of patch-at 1 1) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at 1 1)
     set curr-best-sp (water-speed-of patch-at 1 1)
     set curr-best-h 45
    ]
  if (((elevation-of patch-at 1 0) < curr-best-ele) and ((water-speed-of patch-at 1 0) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at 1 0)
     set curr-best-sp (water-speed-of patch-at 1 0)
     set curr-best-h 90
    ]
  if (((elevation-of patch-at 1 -1) < curr-best-ele) and ((water-speed-of patch-at 1 -1) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at 1 -1)
     set curr-best-sp (water-speed-of patch-at 1 -1)
     set curr-best-h 135
    ]
  if (((elevation-of patch-at -1 -1) < curr-best-ele) and ((water-speed-of patch-at -1 -1) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at -1 -1)
     set curr-best-sp (water-speed-of patch-at -1 -1)
     set curr-best-h 225
    ]
  if (((elevation-of patch-at -1 0) < curr-best-ele) and ((water-speed-of patch-at -1 0) < curr-best-sp))
    [set curr-best-ele (elevation-of patch-at -1 0)
     set curr-best-sp (water-speed-of patch-at -1 0)
     set curr-best-h 270
    ]
end


; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
; Copyright 1998 by Uri Wilensky. All rights reserved.

; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Converted from StarLogoT to NetLogo, 2001.
@#$#@#$#@
GRAPHICS-WINDOW
9
10
690
272
30
10
11.0
1
10
1
1
1
0

CC-WINDOW
2
539
727
658
Command Center

BUTTON
33
282
88
315
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
93
282
148
315
go
go
T
1
T
OBSERVER
T

SLIDER
33
325
148
358
number
number
0
500
250
1
1
NIL

MONITOR
365
283
475
332
highest
highest
3
1

MONITOR
365
348
475
397
lowest
lowest
3
1

SWITCH
173
282
319
315
water-speed-view
water-speed-view
0
1
-1000

SWITCH
173
324
319
357
slowdown?
slowdown?
1
1
-1000

MONITOR
505
283
615
332
fastest
fastest
3
1

MONITOR
505
348
615
397
slowest
slowest
3
1

@#$#@#$#@
WHAT IS IT?
-----------
This model illustrates the movement of trout in a river. The river flows from right to left (i.e. 'upstream' is at the right of the screen, and 'downstream' is at the left of the screen). Rocks which jut above the water's surface are colored brown. Deeper water is dark blue, and shallower water is light blue. Water speed can also be observed by turning on the WATER-VIEW
switch in the interface window, which toggles the display between water speed and water depth. When observing the water-speed, faster moving water is a lighter blue, while slower moving water is shaded darker.

Three sizes of trout inhabit our model riverbed. (Yellow fish are the largest, red fish are of moderate size, and green fish are the smallest.) All of these trout will seek out the ideal spots in which to live, using a particular search routine. There are three pre-defined search routines, to give you an idea of how trout will react under a given search strategy. The first is to seek out the deepest pool, the second to find the calmest pool, and the third a combination of the two. In addition, size is a factor- larger fish have a natural advantage because they can scare smaller fish away from choice areas. Fish try to be the sole occupant of their patch.


HOW TO USE IT
-------------
Select the number of fish with the NUMBER slider, and press the SETUP button. Each time SETUP is pressed a random riverbed is created. You can press the SETUP button again and again until you are satisfied with the look of the riverbed.

Press the GO button to begin the model. If you find it is moving too fast for you, set the SLOWDOWN? switch to ON.

To view water-level elevation, set the WATER-SPEED-VIEW switch to OFF. To view the speed of the water, set WATER-SPEED-VIEW to ON.

There are four monitors which report some information about the riverbed. FASTEST and SLOWEST report the fastest and slowest water-speed of all the patches, and HIGHEST and LOWEST report the maximum and minimum values of riverbed elevation.

The first couple of times you run the model, pick out one fish, and try to follow it as it moves around. In a model where the number of fish is high, it can be confusing to try and watch the behavior of the system as a whole. If your fish momentarily disappears, don't worry. Since as fish compete for certain choice spots, sometimes two (or more) fish try to move into the same patch. Another fish is probably just sitting on top of your fish.


THINGS TO NOTICE
----------------
Watch the activity within certain pools (dark blue patches when viewing water-depth- WATER-SPEED-VIEW should be set to OFF). Where do the trout settle? Does there seem to be a general guideline as to where the water is deepest?

Watch the interaction between different sized-trout. See if you can find two large (yellow) fish competing for the same spot. See if you can find a fish who has left a busy pool, and is travelling far to find another good spot.

With WATER-SPEED-VIEW set to ON, watch how the fish settle. Toggle the slider back and forth- which do the fish seem to prefer, deep but fast pools, or shallow but calm pools? What do you notice about the water-speed of the river as a whole? Where is it calmest?

How do you think the trout are searching for deep and calm areas? Try running the model with just one fish, or a very small amount. The details of the search mechanism are explained below under 'Extending the Model', and you should look at the procedures window to best understand the algorithm.


THINGS TO TRY
-------------
Increase the size of the graphics window if you can. (Using the mouse highlight the graphics window, then hit Edit from the toolbor, and modify the values for window size in the window that comes up.) Alternatively, shrink the window size. Given a set number of fish, observe how their behavior changes in a small or a large world.

A related experiment is to press the SETUP button until you get a riverbed with very few brown patches, so that the trout have a lot of room to move around. Watch how they act. Press SETUP again, until you have a lot of brown patches. How will the fish interact now, in a densely-packed river?

Try changing the NUMBER slider- add a lot of fish to the river. How is this different from a riverbed where there are very few fish?


EXTENDING THE MODEL
-------------------
The trout use the 'pick-desired-heading' procedure to pick the direction in which they wish to move. Currently, 'pick-desired-heading' calls another procedure, 'find-deepest-and-slowest', one of the three search procedures included at the bottom of the procedures window. Each trout, before swimming somewhere, looks around and decides (according to the rules of the current search technique) what the best patch next to it is. This model has been written so that writing your own search procedures are easy. Look at the three procedures 'find-deepest', 'find-slowest', and 'find-deepest-and-slowest' and try to figure out how they work. Then see if you can write your own simple rules for each fish. For starters, try giving each size of fish (each breed) its own search algorithm.

This model does not take into account the feeding patterns of trout.  Create a new version which includes insect life and the feeding patterns of the fish.  Create a new search routine which has the trout search for areas which provide the maximum amount of food.

The model doesn't take temperature into account either. Add a patch variable for water temperature, and incorporate it into a search routine. Trout enjoy cool sections of rivers, and
temperatures above 78 degrees Fahrenheit will kill them. (Deeper water is generally cooler... why is this?)

There are many other directions this model can go, especially in regards to the biology behind the riverbed ecosystem. For instance, predation, both from within the system (bigger fish eating smaller fish) and outside the system (a bear fishing for food); spawning (which might be a good thing to include if you add predation); or different species of fish, each perhaps with different kinds of behavior.


NETLOGO FEATURES
----------------
For the 'riverbed' model, we wanted the water-speed variable value to propagate backwards from right to left. Normal NetLogo patch parallelism prevents us from simply saying:
  if (xcor = screen-edge-x) [set water-speed 1000]
  set water-speed (water-speed-of patch-at 1 0 + (random 10))

because each patch, including the patches at the edges, will simultaneously set their values to that of their neighbor. (Hence water-speed will be 0 for each patch.) Thus, there is the special procedure 'setup-currents', which uses the patch variable 'speed-set-counter'. We have a repeat loop that repeats a number of times equal to the screen-size, incrementing the speed-set-counter each time through the loop, and only set water-speed when the patches' xcor is equal to speed-set-counter.


RELATED MODELS AND LINKS
------------------------
At the Computer Museum in Boston, MA, the Virtual Fishtank exhibit opened on June 13, 1998. Designed by Mitch Resnick and the MIT Media Lab, the Virtual Fishtank allows you to design your own fish, write rules for its behavior, and see how it interacts in a virtual environment containing 100 other fish. (Similar, but more in-depth than, this riverbed model.) For more details, please visit the Computer Museum's Virtual Fishtank website at:
  http://www.tcm.org/fishtank.html
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

fish large
false
0
Polygon -7566196 true true 75 30 239 30 285 149 241 269 75 269 44 149
Polygon -7566196 true true 0 30 74 151 0 270 0 30
Polygon -16777216 true false 196 65 223 65 232 81 232 99 224 113 197 113 190 100 190 79 196 65
Polygon -16777216 true false 270 198 184 177 251 248

fish medium
false
0
Polygon -7566196 true true 75 90 226 90 255 150 225 210 75 210 52 150 75 90
Polygon -7566196 true true 14 68 71 151 16 233
Polygon -16777216 true false 190 108 212 108 220 122 220 144 215 155 191 155 184 146 184 120
Polygon -16777216 true false 200 168 244 176 232 198 232 198 200 168

fish small
false
0
Polygon -7566196 true true 120 120 180 120 210 150 204 164 180 180 120 180 90 151 120 120
Polygon -7566196 true true 90 151 73 119 73 180
Rectangle -16777216 true false 174 132 184 141

@#$#@#$#@
NetLogo 2.1pre2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
