;;  Southern Africans and Cows-Polyandry
;;       (1 female can marry many males, but not vice-versa.
;;       Women own cattle, pay dowry, and get pregnant)
;;  Copyright 2001, 2002 by Baba Kofi Weusijana. All rights reserved.
;;  Version 1.0
;;  Based on Wolf-Sheep Predation Model (c) 1997 Uri Wilensky, Center for Connected Learning.

globals[cattle-impregnation-rate human-impregnation-rate
  cattle-gestation-period human-gestation-period
  cattle-count human-count women-count mean-female-wealth
  married-men mean-human-energy steps colors-already-set?
  births-limit
]
patches-own[ptype nutrition moisture cows bulls men women
  bachelors bachelor bachelor-owner-id
  bachelorettes bachelorette bachelorette-owner-id
  owned-cattle num-owned-cattle
  growth-countdown growth-potential]
turtles-own[gender energy age elderly-stage metabolism spouse
  reproduction-rate gestation births owner-id owned-by temp]
breeds[humans cattle]

to setup
  ca
  clear-output
  setup-ground
  set cattle-impregnation-rate 20
  set human-impregnation-rate 20
  set cattle-gestation-period 9
  set human-gestation-period 9
  set births-limit 10
  setup-women
  setup-cattle
  setup-men
  ; put cattle and humans at random screen locations
  ask turtles
  [
    setxy (random-int-or-float screen-size-x) (random-int-or-float screen-size-y)
    set spouse nobody
  ]
  set colors-already-set? false
  set-colors
  set cattle-count (count cattle)
  set human-count (count humans)
  set married-men 0
  calc-mean-female-wealth
  set mean-human-energy (mean values-from humans [energy])
  set steps 0
  setup-plots
  do-plots
end

to go
  ask turtles
  [
    move
    eat
    reproduce
    death
  ]
  ask humans[cultivate]
  ask patches[go-patches]
  ask patches[marry] ;this doesn't happen until all patches have finished with previous procedures
  set-colors
  set cattle-count (count cattle)
  set human-count (count humans)
  set married-men (count (humans with [(spouse != nobody) and (gender = "male")]))
  calc-mean-female-wealth
  ifelse(any? humans)
  [set mean-human-energy (mean values-from humans [energy])]
  [set mean-human-energy 0]
  set steps (steps + 1)
  do-plots
  if( (human-count < 1) or (cattle-count < 1) )[stop]
end

;;;;; setup procedures ;;;;;

to setup-ground
  ask patches
  [
    ifelse(grass-grows?)
    [; grass growth is delayed by grass-delay
      ifelse( (random-int-or-float 3) = 0)
      [
        set pcolor green
        set nutrition max-patch-kcal
      ]
      [
        set pcolor (green - 5) ;brown
        set nutrition 0
      ]
    ]
    [; grass is always grown
      set pcolor green
      set nutrition max-patch-kcal
    ]
  ]
end

to setup-cattle
  ;make cows
  create-cattle ((4 / 5) * init-cattle)
  ask cattle
  [
    ;setup cows and their ownership
    set gender "female"
    set color white
    set shape "cow"
  ]

  ;make bulls
  create-cattle ((1 / 5) * init-cattle)
  ask cattle with[gender = 0]
  [
    set gender "male"
    set color (red - 3)
    set shape "bull"
  ]

  ;finish cattle setup
  ask cattle
  [
    ; set cattle energy to a week's worth
    set energy (7 * steps-per-day * cattle-metabolism)
    set age 0
    set elderly-stage 10 * steps-per-day * 365 ; 10 years
    set gestation 0
    set births 0
    set owned-by round(random-int-or-float (init-humans / 2))
    set temp nobody
    ]
end

to setup-women
  create-humans (init-humans / 2)
    ask humans
    [
    set gender "female"
    set color (brown - 3)
    set shape gender
    set owner-id who
    ; set human energy to a week's worth
    set energy (7 * steps-per-day * human-metabolism)
    set age 0
    set elderly-stage 50 * steps-per-day * 365 ; 50 years
    set gestation 0
    set births 0
    set temp nobody
  ]
end

to setup-men
  create-humans ((init-humans / 2) + 1)
    ask humans with [gender = 0]
    [
    set gender "male"
    set color (brown - 3)
    set shape gender
    set owner-id round(random-int-or-float (init-humans / 2))
    ; set human energy to a week's worth
    set energy (7 * steps-per-day * human-metabolism)
    set age 0
    set elderly-stage 8000
    set gestation 0
    set births 0
    set temp nobody
  ]
end

to calc-mean-female-wealth
  ;Calculate mean-female-wealth and women-count and avoid the possibility of dividing by 0
  set women-count (count humans with[gender = "female"])
  if-else(women-count != 0)
  [set mean-female-wealth (count cattle) / women-count]
  [set mean-female-wealth 0]
end

;;;;; go procedures ;;;;;

to set-colors
  ifelse(watch-ownership?)
  [
    ask humans[set color (owner-id * 10) + 5]
    ask cattle[set color (owned-by * 10) + 5]
    set colors-already-set? false
  ]
  [
    if not colors-already-set? ;so we don't set colors when we dont need to
    [
      ask cattle
      [
        ifelse(gestation > 0)
        [set color cyan] ;pregnant
        [
          ifelse(gender = "female")
          [set color white]
          [set color red - 3] ;bulls
        ]
      ]
      ask humans
      [
        ifelse(gestation > 0)
        [set color cyan] ;pregnant
        [set color brown - 3]
      ]
      set colors-already-set? true
    ]
  ]
end

to go-patches ;; patch procedure
  ifelse(grass-grows?)
  [
    ifelse(growth-countdown = 1)
    [; planted seeds are ready for harvest!
      if(growth-potential < 1)[set growth-potential 1]
      set nutrition max-patch-kcal
      set growth-potential (growth-potential - 1)
      set growth-countdown growth-potential
    ]
    [; continue to grow seeds
      set growth-countdown (growth-countdown - 1)
    ]
    set nutrition (nutrition + (cal-growth-per-step / 1000))
    if(nutrition > max-patch-kcal) [set nutrition max-patch-kcal]
  ]
  [
    set nutrition max-patch-kcal
  ]
  if(nutrition < 0) [set nutrition 0] ;so there is no such thing as negative nutrition
  set pcolor ( green - 5 + (5 * (nutrition / max-patch-kcal) ) )
  set cows (count turtles-here with[shape = "cow"])
  ; no need to set bulls because nobody is following them
  ;set bulls (count turtles-here with[shape = "bull"])
  set men (count turtles-here with[shape = "male"])
  set women (count turtles-here with[shape = "female"])
end

to cultivate  ;; human procedure
  if(growth-countdown < 1)[set growth-countdown (7 * steps-per-day)]
  set growth-potential (growth-potential + 1)
end

to move  ;; turtle procedure
  ifelse (breed = humans)
  [; humans
    ifelse ((random-int-or-float 100) < 20)
    [; look for food
      ifelse ((random-int-or-float 100) < 50)
      [toward-human-food]
      [rt (random-int-or-float 180)]
    ]
    [; go to your mate
      ifelse(spouse != nobody)
      [;married, go to spouse
        setxy (xcor-of spouse) (ycor-of spouse)
      ]
      [;not married, look for opposite gender
        ifelse(gender = "female")
        [set heading uphill men]
        [set heading uphill women]
      ]
    ]
    fd 1
    set energy (energy - (human-metabolism / steps-per-day)) ;deducting walking energy
  ]
  [; cattle
    ifelse ((random-int-or-float 100) < 50)
    [toward-humans] ;herding
    [
      ifelse ((random-int-or-float 100) < 50)
      [; look for food
        set heading uphill4 nutrition ;toward grass
      ]
      [; bulls look for a mate, cows walk around at random so cattle don't bunch up too much
        ifelse(gender = "male")
        [set heading uphill cows]
        [rt (random-int-or-float 180)]
      ]
    ]
    fd 1
    set energy (energy - (cattle-metabolism / steps-per-day)) ;deducting walking energy
  ]
  set age (age + 1) ;incrementing age
end

to toward-human-food  ;; human procedure
  ifelse(cattle-count > 0)
  [set heading uphill cows]
  [set heading uphill nutrition]
end

to toward-humans ;; cattle procedure
  ifelse ((random-int-or-float 100) < 50)
  [set heading uphill women]
  [set heading uphill men]
end

to eat  ;; turtle procedure
  ifelse(breed = cattle)
  [
    ;Eat grass if you have less than 30 days of energy.
    if(energy < (30 * steps-per-day * cattle-metabolism))
    [
      if(nutrition > 0)
      [; eat all the grass available
        eat-all-grass
      ]
    ]
  ]
  [
    ;Eat if you have less than 30 days of energy.
    if(energy < (30 * steps-per-day * human-metabolism))
    [
      ;Check to see that there are at least as many cattle as we started with, and
      ;check if self or community has less than an average 7 days of energy left
      ifelse( (cattle-count > init-cattle) and
       ( (energy < ( 7 * steps-per-day * human-metabolism) )
       or ( mean-human-energy < (7 * steps-per-day * human-metabolism) ) )
      )
      [; catch and eat cattle if possible
        if((count cattle) > 0)
        [
          set temp one-of cattle-here with [owned-by = owner-id-of myself]
          if(temp != nobody)
          [
            print "Human " + who + " eating cattle " + who-of temp + "."
            set energy (energy + (energy-of temp)) ;getting energy from cattle
            ask temp [set energy 0]  ;deducting energy from cattle
          ]
        ]
        if(nutrition > 0)
        [; eat all the grass available
          eat-all-grass
        ]
      ]
      [
        ; just eat a little grass if I can
        ifelse(nutrition >= (human-metabolism / steps-per-day))
        [
          set energy (energy + (human-metabolism / steps-per-day)) ;getting a day's energy from grass
          set nutrition (nutrition - (human-metabolism / steps-per-day)) ;deducting energy from grass
        ]
        [; eat all the grass available
          eat-all-grass
        ]
      ]
    ]
  ]
end

to eat-all-grass ;; turtle procedure
  ; eat all the grass available on the current patch
  if(nutrition > 0)
  [
    set energy (energy + nutrition) ;getting all energy from grass
    set nutrition 0 ;deducting energy from grass
    set pcolor (green - 5) ;black
  ]
end

to marry ;; patch procedure
  set bachelorettes (humans-here with[shape = "female"]) ;It doesn't matter if the women are already married
  set bachelors ( (humans-here with[shape = "male"]) with[spouse = nobody] )
  if (  ( (count bachelors) > 0) and ( (count bachelorettes ) > 0)  )
  [
    set bachelor ( random-one-of (humans-here with[shape = "male"]))
    set bachelorette ( random-one-of bachelorettes)
    set bachelorette-owner-id owner-id-of bachelorette
    set owned-cattle (cattle with[owned-by = bachelorette-owner-id])
    set num-owned-cattle (count owned-cattle)
    if( (bachelor-owner-id != bachelorette-owner-id) and (num-owned-cattle >= dowry-price) )
    [ ;They are not in the same family and the bachelorette can afford the dowry, marry them!

      ;pay dowry by setting the owner-id of about dowry-price number of bachelorette's cattle
      ;to the bachelor's family's owner-id
      ask owned-cattle
      [
        if( (random-int-or-float num-owned-cattle) < dowry-price)[set owned-by bachelor-owner-id]
      ]

      ;perform marriage by setting spouse and owner-id variables
      ask bachelor
      [
        set spouse bachelorette
        set owner-id bachelorette-owner-id ;he is now part of her family
      ]
      ask bachelorette
      [
        set spouse bachelor ;just to denote she is married to someone
      ]
      show "Man " + who-of bachelor + " got married to woman " + who-of bachelorette + "."
    ]
  ]
end

to reproduce  ;; turtle procedure
  if(gender = "female")
  [
    ifelse(breed = cattle)
    [
      ifelse( (gestation < 1) and (births < births-limit) )
      [; can reproduce and not pregnant, if a bull is here try to get pregnant
        without-interruption
        [
          if(any? (turtles-here with [shape = "bull"]) )
          [
            if ((random-int-or-float 101)  < cattle-impregnation-rate) [impregnate-cow]
          ]
        ]
      ]
      [; pregnant, might give birth
        set gestation (gestation + 1)
        if(gestation >= cattle-gestation-period)[birth-calf]
      ]
    ]
    [; human females
      ifelse(gestation < 1)
      [
        without-interruption
        [
          if( (spouse != nobody) and (births < births-limit) and ((age * steps-per-day) < elderly-stage) )
          [; can reproduce, married, not elderly, and not pregnant, if my spouse is here try to get pregnant
            if(any? (humans-here with [myself = spouse]) )
            [
              if ((random-int-or-float 101)  < human-impregnation-rate) [impregnate-woman]
            ]
          ]
        ]
      ]
      [; pregnant, might give birth
        set gestation (gestation + 1)
        if(gestation >= human-gestation-period)[birth-baby]
      ]
    ]
  ]
end

to impregnate-cow ;; cattle procedure
  set color cyan
  set gestation 1
end

to birth-calf ;; cattle procedure
  set energy (energy / 2) ;mother losing energy to child
  set color white
  set gestation 0
  set births (births + 1)
  hatch 1
  [
    ifelse((random-int-or-float 10) = 0) ;more females than males are born
    [
      set gender "male"
      set shape "bull"
      set color (red - 3)
    ]
    [
      set gender "female"
      set shape "cow"
    ]
    set age 0
    set births 0
  ]
end

to impregnate-woman ;; human procedure
  set color cyan
  set gestation 1
end

to birth-baby ;; human procedure
  set energy (energy / 2) ;mother losing energy to child
  set color brown - 3
  set gestation 0
  set births (births + 1)
  hatch 1
  [
    ifelse((random-int-or-float 2) = 0) ;more females than males are born
    [
      set gender "male"
      set shape "male"
      show "The boy " + who + " has just been born to family " + owner-id + "!"
    ]
    [
      set gender "female"
      set shape "female"
      show "The girl " + who + " has just been born to family " + owner-id + "!"
    ]
    set age 0
    set spouse nobody
    set births 0
  ]
end

to death  ;; turtle procedure
  ifelse(energy < 1)
  [ ;death by lack of nutrition
    without-interruption
    [ ;If there is a spouse, let them know you are dead
      if (spouse != nobody) [set spouse-of spouse nobody]

      show "The " + shape + " " + who + " " +
           " died of low nutrition at age " + age + "."
      die
    ]
  ]
  [
    if (random-int-or-float (age * 1.2)) > elderly-stage
    [ ;death by old age or disease
      without-interruption
      [ ;If there is a spouse, let them know you are dead
        if (spouse != nobody) [set spouse-of spouse nobody]

        show "The " + shape + " " + who + " " +
             " died of old age or disease at age " + age + "."
        die
      ]
    ]
  ]
end

;;;;; plot procedures ;;;;;

; initializes all plot windows
to setup-plots
  set-current-plot "Population Sizes"
  auto-plot-on
  set-plot-x-range 0 100
  set-plot-y-range 0 count turtles
  create-temporary-plot-pen "grass"
  set-plot-pen-color green
  create-temporary-plot-pen "cattle"
  set-plot-pen-color blue
  create-temporary-plot-pen "humans"
  set-plot-pen-color red
end

; does the actual plotting
to do-plots
  set-current-plot "Population Sizes"
  if(grass-grows?)
  [
    create-temporary-plot-pen "grass"
    plot count patches with [pcolor = green] ;green = 55
  ]
  create-temporary-plot-pen "cattle"
  plot cattle-count
  create-temporary-plot-pen "humans"
  plot human-count
end


; *** NetLogo Model Copyright Notice ***

; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation ( REPP program ) -- grant number REC #9814682.

; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.

; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.

; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo SAfriAndCowsPolyandry model.
; http://ccl.northwestern.edu/netlogo/models/SAfriAndCowsPolyandry
; Center for Connected Learning and Computer-Based Modeling
; Northwestern University, Evanston, IL.

; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.publications.edu/netlogo/models/SAfriAndCowsPolyandry
; Center for Connected Learning and Computer-Based Modeling
; Northwestern University, Evanston, IL
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice***
@#$#@#$#@
GRAPHICS-WINDOW
431
10
777
377
10
10
16.0
1
10
1
1
1
0

CC-WINDOW
5
554
786
649
Command Center

SLIDER
0
76
185
109
init-cattle
init-cattle
0.0
100.0
25.0
5.0
1
head

SLIDER
1
42
185
75
init-humans
init-humans
0.0
100.0
20.0
2.0
1
people

SWITCH
5
217
135
250
grass-grows?
grass-grows?
0
1
-1000

SLIDER
4
259
235
292
cal-growth-per-step
cal-growth-per-step
0
10000
1000
25
1
cal/step

SLIDER
186
76
414
109
cattle-metabolism
cattle-metabolism
0.0
50.0
25.0
1.0
1
kcal/day

SLIDER
186
42
414
75
human-metabolism
human-metabolism
0.0
5.0
3.0
1.0
1
kcal/day

BUTTON
167
328
222
361
NIL
setup
NIL
1
T
OBSERVER
T
NIL

PLOT
4
362
349
540
Population Sizes
Time
Number
0.0
100.0
0.0
200.0
true
false

BUTTON
224
328
279
361
NIL
go
T
1
T
OBSERVER
T
NIL

MONITOR
4
110
123
159
number of cattle
cattle-count
0
1

MONITOR
124
110
243
159
number of humans
human-count
0
1

BUTTON
281
328
349
361
go 1 step
go
NIL
1
T
OBSERVER
T
NIL

MONITOR
301
110
351
159
men
human-count - women-count
0
1

MONITOR
4
160
123
209
mean cattle energy
mean values-from cattle [energy]
0
1

MONITOR
124
160
254
209
mean human energy
mean-human-energy
0
1

SLIDER
4
293
189
326
max-patch-kcal
max-patch-kcal
1
100
20
1
1
kilocalories

SLIDER
4
328
165
361
steps-per-day
steps-per-day
1
100
3
1
1
steps/day

MONITOR
138
210
235
259
married men
married-men
0
1

MONITOR
244
110
300
159
women
women-count
0
1

SLIDER
236
259
428
292
dowry-price
dowry-price
0
50
2
1
1
heads of cattle

MONITOR
238
210
371
259
mean female wealth
mean-female-wealth
3
1

SWITCH
189
293
350
326
watch-ownership?
watch-ownership?
1
1
-1000

MONITOR
277
160
327
209
steps
steps
0
1

@#$#@#$#@
Southern Africans and Cattle Suite - Polyandry Model
Author: Baba Kofi Weusijana

WHAT IS IT?
-----------
This model, and others in the Southern Africans and Cattle suite, explores the stability of ecosystems that involve agrarian humans and domestic animals.  Each model in the suite is based on a different human family structure.  The Polygamous model is based on the traditional culture or Bantu-Speaking Southern Africans, and is the first model in the suite one should explore.  In all but the No-Marriage model, humans can only marry if a dowry price of a certain number of cows is paid.  In those models, the humans cannot reproduce unless they are married.  Humans also cultivate the land and that increases the food source for themselves and their cattle.  Thus, both species effects the other in ways besides their predator/prey relationship.  All of the models are greatly extended from the Wolf Sheep Predation Model.  They are intended for use in a college anthropology course.  The ecosystems the suite models are called unstable when they tend to result in extinction for one or more of the species involved.  In contrast, systems are stable when they tend to maintain themselves over time, despite fluctuations in population sizes.

These models consist of two types of agents, "patches" and "turtles".  Patches don't move and are the background on with "turtles" move.  Patches are colored various shades of green to represent patches of grass of varying nutritional/energy value.  Normally patches grow with time (simulating energy being absorbed from the sun).  Turtle agents have been divided into two categories that NetLogo calls breeds: Humans and Cattle.  Each breed has male and female genders and shapes.  Humans can eat grass as well as eat and herd cattle.  Each agent has a certain amount of energy that gets used up as they move, grow, reproduce, etc.  Energy can get replenished if a turtle agent eats or a patch of grass grows due to sunlight and/or human cultivation.  If a turtle agent's energy level is zero, or below, it dies.  If a turtle agent gets too old it may die of old age.  Once a patch's grass is eaten it begins to grow again.  To allow the population to continue each female human or cattle has a fixed probability of attempting to be impregnated (pregnant beings are colored cyan).  In all but the No-Marriage model, female humans only attempt to get pregnant with their spouse.  Again, marriage occurs only if a dowry of cattle is paid.  Human ownership of cattle can be tracked.  Humans don't eat cattle when their population size is lower than their initial setting (but cattle can still all die out).


HOW TO USE IT
-------------
1.  Set the "grass-grows?" switch to "On" to include growing grass in the model, or to "Off" to have the grass always be fully-grown.
2.  Adjust the controls (see below), or use the default settings.
3.  Press the "setup" button.
4.  Press the "go" button to begin the simulation.
5.  View the "Population Sizes" plot to watch the populations fluctuate over time
6.  View the "number of cattle" and "number of humans" monitors to view current population sizes
7.  Watch the Command Center for marriage announcements, birth announcements, and obituaries.  Ages are reported in terms of steps and the number of steps per day are controlled with the steps-per-day slider. If you want to speed up the model, stop updating the graphics display by using the "no-display" command in the Command Center.  You can use the "display" command to begin updating the graphics again.
8.  Keep adjusting the controls and running the model until you find a setting that makes the ecosystem stable with the cattle population generally larger than the human population.
- Note: The simulation will stop if all of the humans or cattle die.

Parameters:
init-cattle: The initial size of cattle population
init-humans: The initial size of human population
cattle-metabolism: The amount of energy (in kilocalories) cattle burn in a day's worth of movement.
human-metabolism: The amount of energy (in kilocalories) humans burn in a day's worth of movement.
grass-grows?: If on grass grows, if off grass is always at its maximum nutritional value.
cal-growth-per-step: How many calories of energy a grass patch gains each step
max-patch-kcal: The maximum number of kilocalories of energy each patch can reach
steps-per-day: The number of steps in a day
dowry-price: The average dowry price (in cattle) that must be paid before a marriage can occur
watch-ownership?: If on cattle and humans are color coded so one can tell which humans own what cattle (e.g.  blue humans own the blue cattle).

Monitors:
number of cattle: The current number of cattle
mean cattle energy: The average cattle energy level in calories
number of humans: The current number of humans
mean human energy: The average human energy level in calories
women: The current number of female humans
men: The current number of male humans
married women (except polyandry and no-marriage models): The current number of women who are married
married men (polyandry model only): The current number of men who are married
married humans (no-marriage model only): The current number of humans who are married (always 0)
mean male wealth (except polyandry and no-marriage models): The average number of cattle owned per human male
mean female wealth (polyandry model only): The average number of cattle owned per human female
mean wealth (no-marriage model only): The average number of cattle owned per human


THINGS TO NOTICE
----------------
Which models in the suite are stable under what conditions (for instance after about 3000 steps)?  Why?  Do all the stable conditions have the same quality of life for humans, cattle, and grass?  What about the relative quality of life for each gender?  What are good criteria for determining what is a good quality of life besides ecosystem stability?
When grass is always grown, watch as the cattle and human populations fluctuate.  Notice that increases and decreases in the sizes of each population are related.  In what way are they related?  What eventually happens?

Once grass is allowed to grow normally, notice the green line added to the population plot representing fluctuations in the number of fully-grown grass patches.  How do the sizes of the three populations appear to relate now?  What is the explanation for this?

Why do you suppose that some variations of the model might be stable while others are not?

What are the effects of a very rich family (a family that owns much more cows than other families) on the community?  What are the effects of a poor family on the community?

Do any of these models exhibit genetic drift?  In other words, do competing families of humans, each reproducing with equal likelihood on each turn, ultimately converge to just one family?  Why or why not?  Is a selection pressure forcing any noticeable convergence?  The idea, explained in more detail in Dennett's "Darwin's Dangerous Idea", is that trait drifts can occur without any particular purpose or 'selecting pressure'.


THINGS TO TRY
-------------
Try adjusting the parameters under various settings.  How sensitive is the stability of the model to the particular parameters?  Hint: change steps-per-day only after trying the other controls.

Can you find any parameters that generate a stable ecosystem with the grass always fully grown?

Notice that under stable settings, the populations tend to fluctuate at a predictable pace.  Can you find any parameters that will speed this up or slow it down?


EXTENDING THE MODEL
-------------------
There are a number ways to alter the model so that it will be stable even when the grass is always fully-grown.  Some will require new elements to be coded in or existing behaviors to be changed.  Can you develop such a version?

Try changing the reproduction rules-- for example, what would happen if reproduction depended on energy rather than being determined by a fixed probability?

Try changing the birth rate-- for example, what would happen if gestation is really 9 months instead of 9 steps?  What if the same amount of female humans is born as males?  What if the same amount of cows is born as bulls?

Try changing human eating habits.  What would happen if humans eat cattle only when they are personally within a week of starving?

Try getting humans to share extra energy with each other.  How does this change the answer to the previous question?

Try changing the elderly-stage setting and/or the probability for humans and/or cattle to die as they approach or exceed that age.


LANGUAGE FEATURES
-------------------
Note the use of a BREEDS variable to model humans and cattle.

Note the use of patches to model grass.

Note the use of patches in some models to marry humans (why can't females or males run a marry procedure?).

Note the use of a random number to model reproduction of cattle and humans.

Note use of "one-of" and "random-one-of" agentset commands to select cattle to be eaten by humans and for humans to marry each other.

Note the method of having humans eat cattle.  Since there is no "kill" command, a human gives the cattle it eats an energy value of 0, which causes the cattle to die of low nutrition in the death procedure.

Note how the death procedure does not always kill the elderly.

Note how calculating certain variables only once per step in "go" saves processor time.


RESEARCH & DEVELOPMENT FEEDBACK
-------------------------------
If you have suggestions or questions regarding this model suite, please email kofi@edutek.net and put "models" in the subject line.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

bull
true
0
Circle -7566196 true true 121 31 59
Rectangle -7566196 true true 75 77 224 239
Rectangle -7566196 true true 145 238 153 271
Polygon -256 true false 181 60 196 15 150 30 104 16 120 60 126 44 133 38 151 31 167 36

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
true
0
Circle -7566196 true true 121 31 59
Rectangle -7566196 true true 105 93 195 225
Line -7566196 true 151 224 152 255
Line -7566196 true 152 224 150 256
Rectangle -7566196 true true 127 77 175 95
Circle -16777216 true false 153 99 23
Circle -16777216 true false 117 151 51
Polygon -16777216 true false 106 101 116 109 117 125 112 135 105 138
Polygon -16777216 true false 194 190 184 198 181 208 185 217 195 219

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

female
true
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159
Rectangle -6524078 true false 159 142 217 179
Rectangle -44544 true false 159 80 215 113

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

male
true
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159
Rectangle -6524078 true false 159 142 217 179

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
