globals
[
  game-over?
  flag
  lives
  level

;score tallying variables
  points-for-alien
  points-for-life
  score
  hits
  misses
  accuracy
  level-bonus-percentage
  win?

;speed of units variables
  bullet-speed
  ship-speed
  aliens-speed

;initial numbers of units
  num-init-aliens
  init-lives
]

breeds [ ship bullets aliens alien-bullets ]

to setup
  ca
  set level 1
  set init-lives 3
  create-custom-ship 1 [
    set lives init-lives
    set shape "ship"
    set color green
    setxy 0 (- screen-edge-y)
  ]
  set num-init-aliens 20 * level
  create-custom-aliens num-init-aliens [
    set color red
    setxy random-float screen-size-x 0
  ]
  set points-for-alien 200
  set points-for-life 500
  set ship-speed 0.1
  set aliens-speed 0.3
  set bullet-speed 0.05
  set flag true
  set accuracy 100
  set level-bonus-percentage 1
  set game-over? false
end

to go
  if ( not mouse-down? and (not game-over?) )
    [ set flag true ]
  if ( mouse-down? and flag )
  [ fire-bullet ]
  ask bullets
  [ track-bullets ]
  every bullet-speed
  [
    ask bullets
    [
      fd 1
      if (pycor = screen-edge-y)
      [
        set misses misses + 1
        ht
        set breed turtles
      ]
    ]
  ]
  every (aliens-speed / level)
  [
    ask aliens
    [
      rt random 20 - random 20
      fd 1
      attack
    ]
  ]
  every bullet-speed
  [
    ask alien-bullets
    [ fd 1 ]
  ]
  ask ship
  [
    if (any? alien-bullets-here)
    [
      set lives lives - 1
      if (lives <= 0)
      [ die ]
    ]
  ]
  ask alien-bullets with [ pycor = (- screen-edge-y) ]
  [
    die
  ]
  every ship-speed
  [
    ask ship
    [
      if (round mouse-xcor != pxcor)
      [
        set heading towardsxy-nowrap mouse-xcor (- screen-edge-y)
        fd 1
      ]
    ]
  ]

  if ( (hits > 0) or (misses > 0) and (not game-over?) )
  [ set accuracy (hits / (hits + misses)) * 100 ]
  if ( (not any? ship) and (not game-over?) )
  [
    set win? false
    show-score-info
  ]
  if ( (any? ship) and (count aliens = 0) and (not game-over?) )
  [
    set win? true
    show-score-info
  ]
end

to attack ; alien procedure
  if random 10 = 0
  [
    hatch 1
    [
      set breed alien-bullets
      set shape "square"
      set color orange
      set heading 180
    ]
  ]
end

to fire-bullet
  set flag false
  ask ship
  [
    hatch 1
    [
      set breed bullets
      set color gray
      set shape "circle"
      set heading 0
    ]
  ]
end

to track-bullets  ; bullet-procedure
  let used? false
  if (any? aliens-here)
  [
    ask aliens-here
    [
      set score (score + points-for-alien)
      set hits hits + 1
      die
    ]
    set used? true
  ]
  if (any? alien-bullets-here)
  [
    ask alien-bullets-here
    [
       die
    ]
    set used? true
  ]
  if (used?)
    [ die ]
end

to show-score-info
  let bonus compute-bonus
  ifelse (win?)
  [ output-print "CONGRATULATIONS!!!  You won!" ]
  [ output-print "Game Over.  Sorry.  You lose." ]
  output-print "You get an accuracy bonus of " + bonus + "."
  set score (score + bonus)
  output-print "Your final score was " + score + " out of a possible " + total-possible-score + "."
  print-rank
  set game-over? true
end

to-report compute-bonus
  ifelse (accuracy = 0)
  [ report 0 ]
  [ report (accuracy-bonus accuracy) + (life-bonus lives) ]
end

to-report accuracy-bonus [ my-accuracy ]
  report round (2.5 * (2 ^ ((my-accuracy * level-bonus-percentage / 10) - 1) ))
end

to print-rank
  let rank ""
  let score-percent score-percentage
  ifelse (score-percent = 0)
  [ set rank "YOU SUCK!!!  Your rank is: GARBAGE COLLECTOR" ]
  [ ifelse (score-percent > 0 and score-percent <= 10)
    [ set rank "Very bad! Try harder next time.  Your rank is: RECRUIT DROP-OUT" ]
    [ ifelse (score-percent > 10 and score-percent <= 25)
      [ set rank "Poor. Your rank is: RECRUIT" ]
      [ ifelse (score-percent > 25 and score-percent <= 40)
        [ set rank "Below average!!! Your rank is: PRIVATE" ]
        [ ifelse (score-percent > 40 and score-percent <= 60)
          [ set rank "Average. Your rank is: LIEUTENANT" ]
          [ ifelse (score-percent > 60 and score-percent <= 75)
            [ set rank "Above Average. Your rank is: CAPTAIN" ]
            [ ifelse (score-percent > 75 and score-percent <= 90)
              [ set rank "Excellent. Your rank is: COMMODORE" ]
              [ ifelse (score-percent > 90 and score-percent < 100)
                [ set rank "Superb! Your rank is: ADMIRAL" ]
                [ set rank "PERFECT SCORE!!! Your rank is: FLEET ADMIRAL" ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]
  if ( hits = 0 and misses = 0 )
  [ set rank "You refused or were not able to fight. Your rank is: PACIFIST" ]
  output-print rank
end

to-report total-possible-score
  ifelse (win?)
  [ report ( (num-init-aliens * points-for-alien) + (accuracy-bonus 100) + (life-bonus init-lives) ) ]
  [ report ( (num-init-aliens * points-for-alien) + (accuracy-bonus 100) + (life-bonus init-lives) ) ]
end

to-report life-bonus [ num-lives ]
  report num-lives * points-for-life
end

to-report score-percentage
  report (score / total-possible-score) * 100
end
@#$#@#$#@
GRAPHICS-WINDOW
165
10
544
410
20
20
9.0
1
10
1
1
1
0

CC-WINDOW
5
509
553
604
Command Center

BUTTON
44
78
114
111
NIL
go
T
1
T
OBSERVER
T
NIL

BUTTON
44
37
114
70
NIL
setup
NIL
1
T
OBSERVER
T
NIL

MONITOR
24
142
78
191
Score
score
3
1

MONITOR
25
203
89
252
Accuracy
accuracy
0
1

MONITOR
90
142
140
191
Lives
lives
3
1

OUTPUT
27
414
544
495

@#$#@#$#@
add model documentation here
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

ship
false
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250
Rectangle -7566196 true true 60 210 90 285
Rectangle -7566196 true true 135 195 165 285
Rectangle -7566196 true true 210 210 240 285

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
setup
@#$#@#$#@
@#$#@#$#@
