; add model procedures here
breeds [ target tank aimer shell wall clouds]

globals [ x-vel y-vel velocity height pos]

to setup
  ca
  ask patches [ set pcolor 97 ]
  ask patches with [ pycor < -15 ]
    [ set pcolor black ]
  cct 1
    [ setxy -17 -15
      set color 53
      set shape "tank1"
    ]
  cct 1
    [ setxy -16 -15
      set color 53
      set shape "tank4"
    ]
  cct 1
    [ setxy -15 -15
      set color 53
      set shape "tank2"
    ]
  cct 1
    [ setxy -16 -14
      set color 53
      set shape "tank3"
    ]
  create-custom-target 1
    [ set color black
      set shape "heli"
      setxy 16 (random 32) - 15
      set size 2
    ]
  create-custom-tank 1
    [ setxy -16 -14
      set color 53
      set heading angle
      set shape "gun"
    ]
  create-custom-aimer 1
    [ ;set shape "ball"
      setxy (sin angle) * 2 - 16 (cos angle) * 2 - 14
      set heading towards one-of tank + 180
      set color 17
    ]
  create-custom-clouds 1
    [
      set shape "cloud"
      setxy 0 15
      set color 8
      set size 3.5
      set heading 90
    ]

   ask patches with [ pycor > -16 ]
        [ ifelse (pycor < wall-height - 15 and pxcor = wall-position )
            [ set pcolor gray ]
            [ set pcolor 97 ]
        ]
   set height wall-height
   set pos wall-position
end

to go
  ask tank
    [ if ( angle != heading )
        [ set heading angle
          ask aimer
            [ setxy (sin angle) * 2 - 16 (cos angle) * 2 - 14
              set heading towards one-of tank + 180
            ]
        ]
    ]
  if ( height != wall-height or pos != wall-position )
    [ ask patches with [ pycor > -16 ]
        [ ifelse (pycor < wall-height - 15 and pxcor = wall-position )
            [ set pcolor gray ]
            [ set pcolor 97 ]
        ]
      set height wall-height
      set pos wall-position
    ]
  every .05
    [ ask shell
        [ setxy (xcor + x-vel) (ycor + y-vel)
          set y-vel (y-vel - .01)
          if (wall-block-wind? = false or (wall-block-wind? = true and (ycor > wall-height - 15) or ((xcor - wall-position) * wind < 0)))
              [ set x-vel (x-vel + (wind / 10000)) ]
          set velocity sqrt (( x-vel ^ 2 ) + (y-vel ^ 2))
          check-shell
        ]
      ask clouds
        [ ;fd wind / 100
          setxy (xcor + wind / 100) (ycor) ]
    ]
end

to fire
   if (count shell = 0)
   [ create-custom-shell 1
      [ setxy -16 -14
        set shape "ball"
        set color black
        set x-vel ( sin angle * ( Power / 100 ))
        set y-vel ( cos angle * ( Power / 100 ))
        set velocity Power / 100
      ]
   ]
end

to check-shell
    if ( pycor = -17 and hidden? = false )
        [ hideturtle ]
    if ( pycor = 17 and hidden? = true )
        [ showturtle ]
    if ( hidden? = false )
    [
       if ( xcor > 17 or ycor < -15 or pcolor = gray )
         [ die ]
       if ( any? target-here )
         [ ask patches in-radius 2 [ set pcolor yellow ]
           ask patches in-radius 1 [ set pcolor red ]
           ask target-here [ die ]
           die
         ]
       set pcolor scale-color sky velocity 2 0
    ]
end
@#$#@#$#@
GRAPHICS-WINDOW
225
10
620
426
17
17
11.0
1
10
1
1
1
0

CC-WINDOW
5
497
629
592
Command Center

BUTTON
30
10
111
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
115
10
196
43
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
29
56
201
89
angle
angle
1
90
30
1
1
NIL

SLIDER
29
95
201
128
power
power
1
100
89
1
1
NIL

BUTTON
75
145
156
178
NIL
fire
NIL
1
T
OBSERVER
T
NIL

SLIDER
30
200
202
233
wind
wind
-50
50
32
1
1
NIL

MONITOR
4
305
104
354
NIL
velocity
3
1

MONITOR
104
305
204
354
X Component
x-vel
3
1

MONITOR
4
257
104
306
Y Component
y-vel
3
1

SLIDER
27
369
199
402
wall-height
wall-height
0
30
21
1
1
NIL

SLIDER
27
405
199
438
wall-position
wall-position
-10
10
5
1
1
NIL

SWITCH
32
450
193
483
wall-block-wind?
wall-block-wind?
0
1
-1000

@#$#@#$#@

WHAT IS IT?
-----------
This model is the classic game Tank Wars. Players must adjust angle and power of fire to hit a target with a projectile. Several factors, including gravity and wind must be taken into account.


HOW IT WORKS
------------
The warhead is fired and moves ballistically towards its target.


HOW TO USE IT
-------------
The setup button creates a new randomly-positioned target and obstacle wall.
The go button causes the game logic to be active and must be pressed anytime the game is in play.
The fire button launches the projectile with the currently specified parameters.

The wind slides adjusts the direction and magnitude of the wind.
Properties of the obstacle wall can be configured using sliders and a toggle, including its height, position, and what effect it has on wind.


THINGS TO TRY
-------------
Try to hit the target.


NETLOGO FEATURES
----------------
This model makes use of turtle sizes and shapes.

RELATED MODELS
--------------



CREDITS AND REFERENCES
----------------------
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

ball
true
0
Circle -7566196 true true 75 75 148

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cloud
false
0
Circle -7566196 true true 44 34 85
Circle -7566196 true true 5 93 90
Circle -7566196 true true 72 87 97
Circle -7566196 true true 51 138 105
Circle -7566196 true true 114 118 94
Circle -7566196 true true 111 51 93
Circle -7566196 true true 160 72 105
Circle -7566196 true true 180 155 68
Circle -7566196 true true 161 39 68
Circle -7566196 true true 200 53 79
Circle -7566196 true true 205 125 80
Line -16777216 false 74 117 94 110
Line -16777216 false 95 111 108 120
Line -16777216 false 182 146 196 136
Line -16777216 false 196 136 222 140
Line -16777216 false 97 181 104 178
Line -16777216 false 106 176 133 180
Line -16777216 false 175 85 193 77
Line -16777216 false 193 77 207 79

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

gun
true
0
Rectangle -7566196 true true 108 13 194 293

heli
false
0
Polygon -7566196 true true 85 68 51 94 45 106 33 122 43 174 63 218 99 232 141 236 191 204 237 160 273 134 281 110 285 52 285 22 283 14 259 20 257 36 243 68 227 100 201 116 159 88 117 70
Polygon -16711681 true false 63 118 57 132 73 198 121 218 143 178 125 124 63 118
Polygon -1 true false 85 34 43 54 25 68 7 84 47 106 115 112 167 104 213 70 191 44 151 18 67 24 19 66 25 78
Polygon -1 true false 29 78 75 88
Polygon -1 true false 75 88 139 68
Rectangle -1 true false 21 60 57 70
Rectangle -1 true false 29 46 69 78
Rectangle -1 true false 61 38 83 44
Rectangle -1 true false 53 32 93 52
Rectangle -1 true false 25 68 37 74
Polygon -1 true false 67 24 31 44 11 82 45 82
Line -16777216 false 97 40 59 58
Line -16777216 false 49 70 67 84
Line -16777216 false 117 84 157 68
Line -16777216 false 129 42 147 52
Line -16777216 false 97 62 107 56
Line -16777216 false 105 66 117 60
Polygon -1 true false 271 28 251 44 243 56 247 72 267 90 287 72 291 54 291 36 271 28
Polygon -1 true false 271 82 283 80 291 70 291 56 277 50
Line -16777216 false 255 54 269 42
Line -16777216 false 271 72 281 62
Line -16777216 false 267 56 271 60
Line -16777216 false 259 62 267 66
Line -16777216 false 275 46 281 52

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

tank1
false
0
Polygon -7566196 true true 89 1 31 47 31 179 89 195 295 195 297 1
Polygon -7566196 true true 30 48 30 240 124 298 298 298 294 106 78 52
Rectangle -7566196 true true 260 0 338 326
Circle -16777216 true false 90 192 95
Circle -16777216 true false 193 191 94

tank2
false
0
Polygon -7566196 true true 1 196 209 194 271 182 271 56 211 0 1 0
Rectangle -7566196 true true -43 0 31 194
Rectangle -7566196 true true 3 146 199 196
Polygon -7566196 true true 270 56 270 254 182 298 0 298 2 146 224 132 260 52
Rectangle -7566196 true true -12 162 116 310
Circle -16777216 true false -11 193 90
Circle -16777216 true false 104 190 88

tank3
false
0
Rectangle -7566196 true true 62 121 234 311

tank4
false
0
Rectangle -7566196 true true 0 0 316 306
Circle -16777216 true false 14 186 104
Circle -16777216 true false 163 191 102

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
