breeds [
  farmers
  goats
]

farmers-own [
  curr-total-rev
  revenue-lst
  id
  avg-revenue/goat
  max-profit
]

patches-own [
  grass-amt   ;amount of grass on patch at time = t.
  grass-max   ;max amount of grass that can be grown at any time.
  have-farm   ;the patch is also farm property
]

goats-own [
  food-stored
  storage-max
  bite-size
  owner#
  bought?    ;true: bought by the farmer and grazing
             ;false: not yet bought by the farmer, but next to be bought
]

globals [
  clock

  ;;setting grass amt /patch
  grass-scale
  grass-min

  ;;setting grass growth-rate
  growth-min
  growth-scale

  ;;setting goat parameters
  food-storage-min
  food-storage-scale

  bite-size-min
  bite-size-scale

  milking-time
  total-goats
  total-milk
  cycle

  avg-profit/farmer
]

to go
  incf-clock
;  wait 0.1
  ifelse (clock mod milking-time) != 0
    [ask goats with [bought?][graze]]
    [ask farmers [do-cost-benefit-analysis]
     incf-cycle
     set avg-profit/farmer (mean values-from farmers [curr-total-rev])
     plot-graph
    ]
  reset-patches
end

to reset-patches
  locals [
    new-grass-amt
  ]
  ask patches [
    set new-grass-amt (grass-amt + grass-growth-rate)
    ifelse (new-grass-amt >= grass-max)
      [set grass-amt grass-max]
      [set grass-amt new-grass-amt]
  ]
  color-patches
end

to plot-avg-revenue
  set-current-plot "Average Revenue"
  set-current-plot-pen "revenue"
  plot avg-profit/farmer
end

to plot-milk-supply
  set-current-plot "Milk Supply"
  set-current-plot-pen "milk-amt"
  set total-milk sum values-from farmers [curr-total-rev]
  plot total-milk
end

to plot-goat-pop
  set-current-plot "Goat Population"
  set-current-plot-pen "num-goats"
  plot total-goats
end

to plot-graph
  plot-avg-revenue
  plot-milk-supply
  plot-goat-pop
end


to setup
  ca
  setup-globals
  setup-patches
  setup-farmers
end

;;turtle procedure
to do-cost-benefit-analysis
  locals [
    potential-rev
    available-rev
    present-rev
    marginal-rev
    farmer#
  ]
  milk-goats
  set avg-revenue/goat (curr-total-rev / num-goats-owned)

  set max-profit max revenue-lst

  set available-rev calc-available-rev
  set marginal-rev calc-marginal-rev
  set present-rev calc-present-rev
  set potential-rev marginal-rev + present-rev

  if (potential-rev <= available-rev and
      marginal-rev > (cost/goat + entrance-fee) and
      grass-growth-rate/cycle >= potential-rev)
    [buy-goats 1 id]
end

to-report grass-growth-rate/cycle
   report (count patches) * grass-growth-rate * (milking-time - 1)
end

to-report calc-available-rev
  locals [
    curr-total-grass
    future-grass
  ]
  set curr-total-grass  sum values-from patches [grass-amt]
  set future-grass grass-growth-rate/cycle
  report curr-total-grass + future-grass
end

to-report calc-present-rev
  locals [farmer#]
  set farmer# id
  report sum values-from goats with [owner# = farmer# and bought?]
               [(milking-time - 1) * bite-size]
end

to-report calc-marginal-rev
  locals [farmer#]
  set farmer# id
  report sum values-from goats with [owner# = farmer# and not bought?]
               [(milking-time - 1) * bite-size]
end

to-report num-goats-owned
  locals [farmer#]
  set farmer# id
  report count goats with [owner# = farmer# and bought?]
end

;;turtle procedure
to milk-goats
  locals [farmer#]
  set farmer# id
  set curr-total-rev (round-to-place (sum values-from goats with [owner# = farmer#] [food-stored]) 10)
  ask goats with [owner# = farmer#] [set food-stored 0]
  set revenue-lst (fput curr-total-rev revenue-lst)
end

to-report round-to-place [num multiple]
  report (round (num * multiple)) / multiple
end

to-report nil
  report (remove 0 (list 0 0))
end

;;turtle procedure
to graze
  locals [new-stored-amt]
  set new-stored-amt (food-stored + (get-amt-eaten bite-size))
  ifelse (new-stored-amt < storage-max)
    [set food-stored new-stored-amt]
    [set food-stored storage-max]

  set heading (uphill grass-amt)
  fd 1
end

to-report get-amt-eaten [bite]
  locals [new-amt]
  set new-amt (grass-amt - bite)
  ifelse (new-amt < 0)
    [set grass-amt 0
     report grass-amt]
    [set grass-amt new-amt
     report bite]
end


to incf-clock
  set clock (1 + clock)
end

to incf-cycle
  set cycle (1 + cycle)
end

to setup-globals
  set grass-scale 50
  set grass-min 10

  set growth-min 1
  set growth-scale grass-min

  set food-storage-scale 50
  set food-storage-min 50

  set clock 1
  set milking-time 11

  set bite-size-min  round (food-storage-min / (milking-time - 1))
  set bite-size-scale (round ((food-storage-scale + food-storage-min) / (milking-time - 1)))



  set total-goats 0
  set total-milk 0
  set cycle 0
end


to setup-patches
  ask patches [
    ;;set amt of food at each patch
    set grass-max (randomize grass-min grass-scale)
    set grass-amt grass-max

    set have-farm false
  ]
  color-patches
end

to color-patches
  ask patches [
    set pcolor (scale-color green grass-amt 10 (2 * grass-scale))
  ]
end

to setup-farmers
  create-custom-farmers num-farmers [
    set color brown
    set id (who mod num-farmers) + 1
    set revenue-lst nil
    set avg-revenue/goat 0
    set max-profit 0
    setup-farm
    buy-goats init-num-goats/farmer id
    ht
  ]
end

to setup-farm
  setxy ((random (screen-size-x - 2)) - screen-edge-x + 1)
        ((random (screen-size-y - 2)) - screen-edge-y + 1)
  ;build-farm
  if (show-farm#) [label-farm]
end

to-report adj-patch-have-farm?
  report (member? true (values-from neighbors [have-farm]))
end

;;turtle procedure
to label-farm
  set label id
end

;;turtle procedure
to build-farm
  ifelse (have-farm or adj-patch-have-farm?)
    [setup-farm]
    [set pcolor brown
     set have-farm true
     ask neighbors [build-farm]
    ]
end

;;turtle procedure
to buy-goats [num-goats farmer#]
  ifelse potential-in-stock?
    [buy-potential]
    [hatch num-goats [
       set breed goats
       setup-goats farmer#
       set bought? true
     ]
     setup-next-potential
    ]
  set total-goats (total-goats + num-goats)
end

to buy-potential
  locals [farmer#]
  set farmer# id
  ask goats with [owner# = farmer# and (not bought?)] [
    set bought? true
    st
  ]
  setup-next-potential
end

to setup-next-potential
  locals [farmer#]
  set farmer# id
  hatch 1 [
    set breed goats
    setup-goats farmer#
    set bought? false
    ht
  ]
end

to-report potential-in-stock?
  locals [
    farmer#
    num-stock
  ]
  set farmer# id
  set num-stock (count goats with [owner# = farmer# and (not bought?)])
  ifelse (num-stock > 0)
    [report true]
    [report false]
end

to setup-goats [farmer#]
  if (show-goat#) [
    set label farmer#
    set label-color black
  ]
  setxy (random-float screen-size-x)
        (random-float screen-size-y)
  set color white
  set shape "goat"
  set food-stored 0
  set storage-max (randomize food-storage-min food-storage-scale)
  set owner# farmer#
  set bite-size (randomize bite-size-min bite-size-scale)
end

to-report randomize [minimum scale]
  report (minimum + (random scale))
end

@#$#@#$#@
GRAPHICS-WINDOW
450
13
1006
590
10
10
26.0
1
10
1
1
1
0

CC-WINDOW
5
604
1015
699
Command Center

BUTTON
10
11
103
44
Setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
107
11
200
44
Go
go
T
1
T
OBSERVER
T
NIL

SLIDER
9
47
206
80
Num-Farmers
Num-Farmers
1
10
1
1
1
NIL

SLIDER
9
84
206
117
Init-Num-Goats/Farmer
Init-Num-Goats/Farmer
0
10
2
1
1
goats

SWITCH
209
13
332
46
Show-Goat#
Show-Goat#
0
1
-1000

SWITCH
209
52
332
85
Show-Farm#
Show-Farm#
1
1
-1000

SLIDER
9
120
206
153
Grass-Growth-Rate
Grass-Growth-Rate
0
10
2.0
0.5
1
oz/cycle

SLIDER
9
156
206
189
Cost/Goat
Cost/Goat
0
100
40
1
1
$

MONITOR
340
61
443
110
avg-profit/cycle
avg-profit/farmer
1
1

PLOT
3
326
203
476
Average Revenue
Cycle
Revenue
0.0
100.0
0.0
1000.0
true
false
PENS
"revenue" 1.0 0 -16777216 true

PLOT
207
326
407
476
Milk Supply
Cycle
Milk (gallons)
0.0
100.0
0.0
100.0
true
false
PENS
"milk-amt" 1.0 0 -16777216 true

SLIDER
9
192
206
225
Entrance-Fee
Entrance-Fee
0
25
10.0
0.1
1
 $

PLOT
209
167
409
317
Goat Population
Cycle
Goats
0.0
100.0
0.0
100.0
true
false
PENS
"num-goats" 1.0 0 -16777216 true

MONITOR
340
10
443
59
Goat Population
total-goats
0
1

MONITOR
340
112
443
161
Milk Supply
total-milk
0
1

@#$#@#$#@
add model documentation here
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

goat
false
0
Polygon -7566196 true true 203 116 175 128 134 130 91 119 57 136 56 171 69 188 85 193 85 243 95 244 97 192 131 182 169 185 170 245 184 245 185 184 213 138
Polygon -7566196 true true 57 146 40 152 41 180 47 155
Polygon -7566196 true true 203 113 218 93 239 93 266 134 257 145 241 137 221 133 214 138
Polygon -7566196 true true 242 136 235 150 245 161 242 146
Rectangle -7566196 true true 197 123 212 138
Rectangle -7566196 true true 200 120 212 133
Rectangle -7566196 true true 200 119 212 131
Rectangle -7566196 true true 205 119 214 123
Rectangle -7566196 true true 202 115 212 124
Rectangle -7566196 true true 210 134 213 140
Polygon -6524078 true false 231 91 224 65 186 57 180 68 191 64 217 71 220 93
Polygon -7566196 true true 97 193 99 242 96 243
Polygon -7566196 true true 95 217 99 242 95 242
Polygon -7566196 true true 195 168 219 186 210 223 199 222 206 188 182 183
Polygon -7566196 true true 116 186 127 197 114 223 104 213 117 196
Polygon -6524078 true false 232 92 218 92

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
