globals [ word-list real-word curr-word letter-q total-correct num-tries correct?]
patches-own [ letter? ]

;; the things you need to "start the game"
to setup
  ca
  setup-plots
  set num-tries 1
  reset-gw
  setup-list
  place-word
end

;;sets up the graphs
to setup-plots
  cp
  set-current-plot "frequencies"
  clear-plot
  set-current-plot-pen "ftries"
  set-plot-pen-mode 2
  set-plot-x-range 0 10
  set-plot-y-range 0 10
  auto-plot-on
  set-current-plot "length"
  clear-plot
  set-current-plot-pen "ltries"
  set-plot-pen-mode 2
  set-plot-x-range 0 12
  set-plot-y-range 0 10
  auto-plot-on
end

;; resets the graphics window
to reset-gw
  ask patches [
    set pcolor white
    set plabel-color black
    set plabel ""
    set letter? 0
  ]
end

;; check to see how many valid letters in the word
to-report word-letters [letter-list]
  let iter 0
  let num-letters 0
  repeat length letter-list [
    if (item iter letter-list != "_")
      [ set num-letters num-letters + 1 ]
    set iter iter + 1
  ]
  report num-letters
end

;; to substitute in missing letters
to mess-word
  repeat maximum-number-of-missing-letters [
    if (word-letters (first curr-word) > 1) [
      set curr-word
          list replace-item (random length first curr-word) (first curr-word) "_"
          but-first curr-word
    ]
  ]
end

;;to put the word on the screen
to place-word
  reset-gw
  pick-word
  mess-word
  let iter 0
  repeat length first curr-word [
    cct 1 [
      setxy (x-val iter) 0
      ht
      set plabel item iter first curr-word
      set letter? item iter first curr-word
      if (letter? = "_") [ set letter-q iter ]
    ]
    set iter (iter + 1)
  ]
  set pcolor-of patch-at (x-val letter-q) 0 green
  ask turtles [ die ]
  set num-tries 1
  set correct? 0
end

;;to tell where the letters go on the x-axis... just a convenience
to-report x-val [ iterated-x ]
  report ( 1 + ( iterated-x * 1 ) - screen-edge-x)
end

;; when you click on a letter,
;; it calls this function with that letter as the argument
to pick-letter [ u-let ]
  ifelse item letter-q first real-word = u-let [
    set correct? 1
    set num-tries num-tries + 1
    set total-correct total-correct + 1
  ] [
    set correct? 0
  ]
  ifelse (correct? = 1) [
    let iter 0
    output-type "correct! the word was: "
    repeat length first real-word [ output-type item iter first real-word
                                    set iter iter + 1]
    output-print ""
    set-current-plot "frequencies"
    plot-point (first but-first real-word) num-tries
    set-current-plot "length"
    plot-point length first real-word num-tries
    place-word
  ] [
    output-print "sorry! try again. your letter was: " + u-let + ", number of tries: " + num-tries
    set num-tries num-tries + 1
  ]
end

;; set up the duplicate word (curr-word) so you can match it against the right one (real-word)
to pick-word
 set curr-word random-one-of word-list
 set real-word curr-word
end

;;set up the vocab/word list
to setup-list
  set word-list
[

[ [ "a" "b" "l" "e"  ] 16 ]
[ [ "a" "b" "o" "d" "e"  ] 13 ]
[ [ "a" "b" "o" "u" "t"  ] 278 ]
[ [ "a" "b" "o" "v" "e"  ] 24 ]
[ [ "a" "c" "r" "o" "s" "s"  ] 12 ]
[ [ "a" "c" "t" "u" "a" "l" "l" "y"  ] 12 ]
[ [ "a" "d" "m" "i" "r" "a" "b" "l" "e"  ] 15 ]
[ [ "a" "f" "f" "a" "i" "r"  ] 16 ]
[ [ "a" "f" "f" "a" "i" "r" "s"  ] 20 ]
[ [ "a" "f" "r" "a" "i" "d"  ] 20 ]
[ [ "a" "f" "t" "e" "r"  ] 90 ]
[ [ "a" "f" "t" "e" "r" "w" "a" "r" "d"  ] 16 ]
[ [ "a" "g" "a" "i" "n"  ] 87 ]
[ [ "a" "g" "a" "i" "n" "s" "t"  ] 35 ]
[ [ "a" "g" "e"  ] 13 ]
[ [ "a" "g" "o"  ] 31 ]
[ [ "a" "h"  ] 92 ]
[ [ "a" "i" "d"  ] 13 ]
[ [ "a" "l" "l"  ] 569 ]
[ [ "a" "l" "m" "o" "s" "t"  ] 21 ]
[ [ "a" "l" "o" "n" "e"  ] 37 ]
[ [ "a" "l" "r" "e" "a" "d" "y"  ] 20 ]
[ [ "a" "l" "s" "o"  ] 43 ]
[ [ "a" "l" "w" "a" "y" "s"  ] 104 ]
[ [ "a" "m"  ] 291 ]
[ [ "a" "m" "o" "n" "g"  ] 27 ]
[ [ "a" "n"  ] 247 ]
[ [ "a" "n" "a"  ] 143 ]
[ [ "a" "n" "c" "i" "e" "n" "t"  ] 13 ]
[ [ "a" "n" "d"  ] 3862 ]
[ [ "a" "n" "o" "t" "h" "e" "r"  ] 49 ]
[ [ "a" "n" "s" "w" "e" "r"  ] 14 ]
[ [ "a" "n" "s" "w" "e" "r" "e" "d"  ] 11 ]
[ [ "a" "n" "y"  ] 242 ]
[ [ "a" "n" "y" "b" "o" "d" "y"  ] 33 ]
[ [ "a" "n" "y" "o" "n" "e"  ] 18 ]
[ [ "a" "n" "y" "t" "h" "i" "n" "g"  ] 62 ]
[ [ "a" "n" "y" "w" "h" "e" "r" "e"  ] 32 ]
[ [ "a" "p" "p" "e" "a" "r"  ] 19 ]
[ [ "a" "p" "p" "e" "a" "r" "a" "n" "c" "e"  ] 15 ]
[ [ "a" "p" "p" "e" "a" "r" "e" "d"  ] 16 ]
[ [ "a" "p" "p" "e" "a" "r" "s"  ] 11 ]
[ [ "a" "r" "e"  ] 478 ]
[ [ "a" "r" "m"  ] 11 ]
[ [ "a" "r" "m" "s"  ] 18 ]
[ [ "a" "r" "t"  ] 12 ]
[ [ "a" "s"  ] 827 ]
[ [ "a" "s" "i" "d" "e"  ] 11 ]
[ [ "a" "s" "k"  ] 22 ]
[ [ "a" "s" "k" "e" "d"  ] 19 ]
[ [ "a" "s" "s" "u" "r" "e"  ] 19 ]
[ [ "a" "s" "s" "u" "r" "e" "d" "l" "y"  ] 12 ]
[ [ "a" "t"  ] 429 ]
[ [ "a" "w" "a" "y"  ] 56 ]
[ [ "b" "a" "c" "k"  ] 73 ]
[ [ "b" "e"  ] 671 ]
[ [ "b" "e" "a" "u" "t" "i" "f" "u" "l"  ] 27 ]
[ [ "b" "e" "a" "u" "t" "y"  ] 29 ]
[ [ "b" "e" "c" "a" "m" "e"  ] 15 ]
[ [ "b" "e" "c" "a" "u" "s" "e"  ] 146 ]
[ [ "b" "e" "c" "o" "m" "e"  ] 33 ]
[ [ "b" "e" "d"  ] 21 ]
[ [ "b" "e" "e" "n"  ] 160 ]
[ [ "b" "e" "f" "o" "r" "e"  ] 81 ]
[ [ "b" "e" "g" "a" "n"  ] 32 ]
[ [ "b" "e" "g" "i" "n" "n" "i" "n" "g"  ] 11 ]
[ [ "b" "e" "h" "i" "n" "d"  ] 16 ]
[ [ "b" "e" "h" "o" "l" "d"  ] 11 ]
[ [ "b" "e" "i" "n" "g"  ] 79 ]
[ [ "b" "e" "l" "i" "e" "v" "e"  ] 58 ]
[ [ "b" "e" "s" "i" "d" "e" "s"  ] 32 ]
[ [ "b" "e" "s" "t"  ] 32 ]
[ [ "b" "e" "t" "t" "e" "r"  ] 40 ]
[ [ "b" "e" "t" "w" "e" "e" "n"  ] 43 ]
[ [ "b" "e" "y" "o" "n" "d"  ] 22 ]
[ [ "b" "i" "g"  ] 13 ]
[ [ "b" "i" "s" "h" "o" "p"  ] 11 ]
[ [ "b" "i" "t"  ] 11 ]
[ [ "b" "l" "a" "c" "k"  ] 56 ]
[ [ "b" "l" "o" "o" "d"  ] 17 ]
[ [ "b" "l" "u" "e"  ] 17 ]
[ [ "b" "o" "d" "y"  ] 38 ]
[ [ "b" "o" "o" "k"  ] 17 ]
[ [ "b" "o" "t" "h"  ] 23 ]
[ [ "b" "o" "y"  ] 48 ]
[ [ "b" "r" "i" "g" "h" "t"  ] 19 ]
[ [ "b" "r" "o" "u" "g" "h" "t"  ] 17 ]
[ [ "b" "r" "o" "w" "n"  ] 30 ]
[ [ "b" "u" "s" "i" "n" "e" "s" "s"  ] 30 ]
[ [ "b" "u" "t"  ] 820 ]
[ [ "b" "y"  ] 378 ]
[ [ "c" "a" "l" "i" "b" "u" "r" "n"  ] 14 ]
[ [ "c" "a" "l" "l"  ] 33 ]
[ [ "c" "a" "l" "l" "e" "d"  ] 24 ]
[ [ "c" "a" "m" "e"  ] 101 ]
[ [ "c" "a" "n"  ] 182 ]
[ [ "c" "a" "n" "d" "l" "e" "s"  ] 11 ]
[ [ "c" "a" "n" "n" "o" "t"  ] 91 ]
[ [ "c" "a" "r" "e"  ] 27 ]
[ [ "c" "a" "r" "r" "i" "e" "d"  ] 12 ]
[ [ "c" "a" "s" "e"  ] 11 ]
[ [ "c" "a" "s" "t" "l" "e"  ] 14 ]
[ [ "c" "a" "v" "e"  ] 34 ]
[ [ "c" "e" "n" "t" "a" "u" "r"  ] 41 ]
[ [ "c" "e" "r" "t" "a" "i" "n"  ] 19 ]
[ [ "c" "e" "r" "t" "a" "i" "n" "l" "y"  ] 101 ]
[ [ "c" "h" "i" "l" "d"  ] 13 ]
[ [ "c" "h" "i" "l" "d" "r" "e" "n"  ] 26 ]
[ [ "c" "h" "l" "o" "r" "i" "s"  ] 55 ]
[ [ "c" "h" "o" "i" "c" "e"  ] 11 ]
[ [ "c" "i" "t" "y"  ] 19 ]
[ [ "c" "l" "e" "v" "e" "r"  ] 35 ]
[ [ "c" "o" "c" "a" "i" "g" "n" "e"  ] 42 ]
[ [ "c" "o" "l" "o" "r"  ] 18 ]
[ [ "c" "o" "l" "o" "r" "e" "d"  ] 32 ]
[ [ "c" "o" "l" "o" "r" "s"  ] 11 ]
[ [ "c" "o" "m" "e"  ] 127 ]
[ [ "c" "o" "m" "e" "s"  ] 14 ]
[ [ "c" "o" "m" "f" "o" "r" "t"  ] 14 ]
[ [ "c" "o" "m" "m" "o" "n"  ] 26 ]
[ [ "c" "o" "m" "p" "a" "n" "y"  ] 13 ]
[ [ "c" "o" "m" "p" "r" "o" "m" "i" "s" "e"  ] 11 ]
[ [ "c" "o" "m" "p" "r" "o" "m" "i" "s" "e" "s"  ] 12 ]
[ [ "c" "o" "n" "c" "e" "d" "e"  ] 12 ]
[ [ "c" "o" "n" "d" "u" "c" "t"  ] 11 ]
[ [ "c" "o" "n" "s" "c" "i" "e" "n" "c" "e"  ] 22 ]
[ [ "c" "o" "n" "s" "e" "q" "u" "e" "n" "c" "e"  ] 15 ]
[ [ "c" "o" "n" "s" "i" "d" "e" "r"  ] 32 ]
[ [ "c" "o" "n" "s" "i" "d" "e" "r" "a" "b" "l" "e"  ] 13 ]
[ [ "c" "o" "n" "s" "i" "d" "e" "r" "e" "d"  ] 25 ]
[ [ "c" "o" "n" "s" "o" "r" "t"  ] 11 ]
[ [ "c" "o" "n" "t" "e" "n" "t"  ] 19 ]
[ [ "c" "o" "n" "t" "i" "n" "u" "e" "d"  ] 14 ]
[ [ "c" "o" "t" "h"  ] 36 ]
[ [ "c" "o" "u" "l" "d"  ] 162 ]
[ [ "c" "o" "u" "n" "t"  ] 18 ]
[ [ "c" "o" "u" "n" "t" "e" "s" "s"  ] 15 ]
[ [ "c" "o" "u" "n" "t" "r" "y"  ] 19 ]
[ [ "c" "o" "u" "r" "s" "e"  ] 84 ]
[ [ "c" "r" "e" "a" "t" "e" "d"  ] 12 ]
[ [ "c" "r" "e" "a" "t" "u" "r" "e"  ] 18 ]
[ [ "c" "r" "e" "a" "t" "u" "r" "e" "s"  ] 15 ]
[ [ "c" "r" "i" "e" "d"  ] 25 ]
[ [ "c" "r" "o" "s" "s"  ] 15 ]
[ [ "c" "u" "r" "i" "o" "u" "s"  ] 31 ]
[ [ "c" "u" "s" "t" "o" "m"  ] 18 ]
[ [ "d" "a" "m" "e"  ] 72 ]
[ [ "d" "a" "r" "e"  ] 12 ]
[ [ "d" "a" "r" "k"  ] 38 ]
[ [ "d" "a" "r" "k" "n" "e" "s" "s"  ] 14 ]
[ [ "d" "a" "r" "l" "i" "n" "g"  ] 22 ]
[ [ "d" "a" "u" "g" "h" "t" "e" "r"  ] 41 ]
[ [ "d" "a" "w" "n"  ] 20 ]
[ [ "d" "a" "y"  ] 76 ]
[ [ "d" "a" "y" "s"  ] 25 ]
[ [ "d" "e"  ] 39 ]
[ [ "d" "e" "a" "d"  ] 35 ]
[ [ "d" "e" "a" "l"  ] 41 ]
[ [ "d" "e" "a" "r"  ] 109 ]
[ [ "d" "e" "a" "t" "h"  ] 24 ]
[ [ "d" "e" "l" "i" "g" "h" "t"  ] 15 ]
[ [ "d" "e" "n" "y"  ] 17 ]
[ [ "d" "e" "s" "i" "r" "e"  ] 48 ]
[ [ "d" "e" "v" "i" "l"  ] 14 ]
[ [ "d" "e" "v" "i" "l" "s"  ] 34 ]
[ [ "d" "i" "d"  ] 160 ]
[ [ "d" "i" "e"  ] 13 ]
[ [ "d" "o"  ] 398 ]
[ [ "d" "o" "e" "s"  ] 77 ]
[ [ "d" "o" "i" "n" "g"  ] 24 ]
[ [ "d" "o" "l" "o" "r" "e" "s"  ] 19 ]
[ [ "d" "o" "n" "e"  ] 60 ]
[ [ "d" "o" "o" "r"  ] 28 ]
[ [ "d" "o" "r" "o" "t" "h" "y"  ] 70 ]
[ [ "d" "o" "u" "b" "t"  ] 22 ]
[ [ "d" "o" "w" "n"  ] 33 ]
[ [ "d" "r" "e" "a" "m"  ] 18 ]
[ [ "d" "r" "e" "w"  ] 12 ]
[ [ "d" "r" "i" "n" "k"  ] 13 ]
[ [ "d" "u" "e"  ] 14 ]
[ [ "d" "u" "k" "e"  ] 44 ]
[ [ "d" "u" "t" "y"  ] 19 ]
[ [ "e" "a" "c" "h"  ] 49 ]
[ [ "e" "a" "r" "t" "h"  ] 25 ]
[ [ "e" "i" "t" "h" "e" "r"  ] 29 ]
[ [ "e" "l" "s" "e"  ] 55 ]
[ [ "e" "m" "p" "e" "r" "o" "r"  ] 37 ]
[ [ "e" "n" "c" "o" "u" "n" "t" "e" "r" "e" "d"  ] 11 ]
[ [ "e" "n" "d"  ] 23 ]
[ [ "e" "n" "o" "u" "g" "h"  ] 53 ]
[ [ "e" "n" "t" "e" "r"  ] 13 ]
[ [ "e" "q" "u" "a" "l" "l" "y"  ] 12 ]
[ [ "e" "u" "b" "o" "n" "i" "a"  ] 19 ]
[ [ "e" "v" "e" "n"  ] 144 ]
[ [ "e" "v" "e" "n" "i" "n" "g"  ] 24 ]
[ [ "e" "v" "e" "n" "t"  ] 15 ]
[ [ "e" "v" "e" "n" "t" "s"  ] 17 ]
[ [ "e" "v" "e" "r"  ] 121 ]
[ [ "e" "v" "e" "r" "y"  ] 62 ]
[ [ "e" "v" "e" "r" "y" "b" "o" "d" "y"  ] 25 ]
[ [ "e" "v" "e" "r" "y" "t" "h" "i" "n" "g"  ] 44 ]
[ [ "e" "v" "e" "r" "y" "w" "h" "e" "r" "e"  ] 26 ]
[ [ "e" "v" "i" "l"  ] 15 ]
[ [ "e" "x" "a" "m" "p" "l" "e"  ] 16 ]
[ [ "e" "x" "c" "e" "l" "l" "e" "n" "t"  ] 15 ]
[ [ "e" "x" "c" "e" "p" "t"  ] 13 ]
[ [ "e" "x" "p" "e" "c" "t" "e" "d"  ] 20 ]
[ [ "e" "x" "p" "l" "a" "i" "n"  ] 16 ]
[ [ "e" "x" "p" "l" "a" "i" "n" "e" "d"  ] 13 ]
[ [ "e" "y" "e" "s"  ] 106 ]
[ [ "f" "a" "c" "e"  ] 70 ]
[ [ "f" "a" "c" "t"  ] 30 ]
[ [ "f" "a" "i" "r"  ] 53 ]
[ [ "f" "a" "i" "r" "l" "y"  ] 33 ]
[ [ "f" "a" "i" "t" "h"  ] 33 ]
[ [ "f" "a" "l" "l"  ] 13 ]
[ [ "f" "a" "m" "i" "l" "y"  ] 13 ]
[ [ "f" "a" "r"  ] 47 ]
[ [ "f" "a" "r" "e" "w" "e" "l" "l"  ] 20 ]
[ [ "f" "a" "t" "h" "e" "r"  ] 44 ]
[ [ "f" "e" "a" "r"  ] 16 ]
[ [ "f" "e" "e" "l"  ] 22 ]
[ [ "f" "e" "e" "t"  ] 31 ]
[ [ "f" "e" "l" "l"  ] 14 ]
[ [ "f" "e" "l" "l" "o" "w"  ] 58 ]
[ [ "f" "e" "l" "l" "o" "w" "s"  ] 15 ]
[ [ "f" "e" "l" "t"  ] 16 ]
[ [ "f" "i" "e" "l" "d"  ] 14 ]
[ [ "f" "i" "e" "l" "d" "s"  ] 12 ]
[ [ "f" "i" "g" "u" "r" "e"  ] 15 ]
[ [ "f" "i" "n" "d"  ] 58 ]
[ [ "f" "i" "n" "e"  ] 78 ]
[ [ "f" "i" "n" "g" "e" "r"  ] 13 ]
[ [ "f" "i" "n" "g" "e" "r" "s"  ] 15 ]
[ [ "f" "i" "r" "e"  ] 11 ]
[ [ "f" "i" "r" "s" "t"  ] 84 ]
[ [ "f" "l" "a" "m" "e"  ] 18 ]
[ [ "f" "l" "e" "s" "h"  ] 22 ]
[ [ "f" "o" "l" "l" "o" "w" "e" "d"  ] 25 ]
[ [ "f" "o" "n" "d"  ] 14 ]
[ [ "f" "o" "o" "l"  ] 12 ]
[ [ "f" "o" "o" "l" "i" "s" "h"  ] 17 ]
[ [ "f" "o" "r"  ] 975 ]
[ [ "f" "o" "r" "e" "s" "t"  ] 30 ]
[ [ "f" "o" "r" "e" "v" "e" "r"  ] 19 ]
[ [ "f" "o" "r" "g" "e" "t"  ] 20 ]
[ [ "f" "o" "r" "g" "o" "t" "t" "e" "n"  ] 14 ]
[ [ "f" "o" "r" "m"  ] 15 ]
[ [ "f" "o" "r" "t" "y"  ] 13 ]
[ [ "f" "o" "r" "w" "a" "r" "d"  ] 17 ]
[ [ "f" "o" "u" "n" "d"  ] 52 ]
[ [ "f" "o" "u" "r"  ] 21 ]
[ [ "f" "r" "e" "t" "t" "e" "d"  ] 11 ]
[ [ "f" "r" "i" "e" "n" "d"  ] 22 ]
[ [ "f" "r" "o" "m"  ] 225 ]
[ [ "f" "u" "l" "l"  ] 19 ]
[ [ "f" "u" "r" "t" "h" "e" "r"  ] 12 ]
[ [ "g" "a" "r" "d" "e" "n"  ] 43 ]
[ [ "g" "a" "r" "m" "e" "n" "t"  ] 11 ]
[ [ "g" "a" "v" "e"  ] 25 ]
[ [ "g" "a" "w" "r"  ] 14 ]
[ [ "g" "e" "n" "e" "r" "a" "l" "l" "y"  ] 12 ]
[ [ "g" "e" "n" "t" "l" "e" "m" "a" "n"  ] 27 ]
[ [ "g" "e" "t"  ] 52 ]
[ [ "g" "e" "t" "t" "i" "n" "g"  ] 13 ]
[ [ "g" "h" "o" "s" "t"  ] 14 ]
[ [ "g" "i" "r" "l"  ] 68 ]
[ [ "g" "i" "r" "l" "s"  ] 14 ]
[ [ "g" "i" "v" "e"  ] 32 ]
[ [ "g" "i" "v" "e" "n"  ] 27 ]
[ [ "g" "l" "a" "s" "s"  ] 11 ]
[ [ "g" "l" "i" "t" "t" "e" "r" "i" "n" "g"  ] 14 ]
[ [ "g" "o"  ] 84 ]
[ [ "g" "o" "d"  ] 74 ]
[ [ "g" "o" "d" "m" "o" "t" "h" "e" "r"  ] 15 ]
[ [ "g" "o" "d" "s"  ] 25 ]
[ [ "g" "o" "g" "y" "r" "v" "a" "n"  ] 54 ]
[ [ "g" "o" "i" "n" "g"  ] 39 ]
[ [ "g" "o" "l" "d"  ] 26 ]
[ [ "g" "o" "n" "e"  ] 11 ]
[ [ "g" "o" "o" "d"  ] 63 ]
[ [ "g" "o" "t"  ] 15 ]
[ [ "g" "r" "a" "n" "d" "f" "a" "t" "h" "e" "r"  ] 32 ]
[ [ "g" "r" "a" "n" "d" "m" "o" "t" "h" "e" "r"  ] 24 ]
[ [ "g" "r" "a" "n" "t"  ] 11 ]
[ [ "g" "r" "a" "y"  ] 31 ]
[ [ "g" "r" "e" "a" "t"  ] 76 ]
[ [ "g" "r" "e" "e" "n"  ] 17 ]
[ [ "h" "a" "d"  ] 495 ]
[ [ "h" "a" "i" "r"  ] 32 ]
[ [ "h" "a" "l" "f"  ] 29 ]
[ [ "h" "a" "l" "l"  ] 15 ]
[ [ "h" "a" "n" "d"  ] 58 ]
[ [ "h" "a" "n" "d" "s"  ] 22 ]
[ [ "h" "a" "n" "d" "s" "o" "m" "e"  ] 17 ]
[ [ "h" "a" "p" "p" "e" "n"  ] 26 ]
[ [ "h" "a" "p" "p" "e" "n" "e" "d"  ] 33 ]
[ [ "h" "a" "p" "p" "e" "n" "s"  ] 20 ]
[ [ "h" "a" "p" "p" "y"  ] 23 ]
[ [ "h" "a" "r" "d"  ] 18 ]
[ [ "h" "a" "r" "d" "l" "y"  ] 25 ]
[ [ "h" "a" "r" "m"  ] 12 ]
[ [ "h" "a" "s"  ] 148 ]
[ [ "h" "a" "v" "e"  ] 638 ]
[ [ "h" "a" "v" "i" "n" "g"  ] 27 ]
[ [ "h" "e"  ] 829 ]
[ [ "h" "e" "a" "d"  ] 60 ]
[ [ "h" "e" "a" "r"  ] 14 ]
[ [ "h" "e" "a" "r" "d"  ] 52 ]
[ [ "h" "e" "a" "r" "t"  ] 74 ]
[ [ "h" "e" "a" "t" "h"  ] 13 ]
[ [ "h" "e" "a" "v" "e" "n"  ] 96 ]
[ [ "h" "e" "l" "d"  ] 20 ]
[ [ "h" "e" "l" "e" "n"  ] 64 ]
[ [ "h" "e" "l" "l"  ] 60 ]
[ [ "h" "e" "l" "p"  ] 20 ]
[ [ "h" "e" "r"  ] 430 ]
[ [ "h" "e" "r" "e"  ] 109 ]
[ [ "h" "e" "r" "e" "a" "b" "o" "u" "t" "s"  ] 20 ]
[ [ "h" "i" "g" "h"  ] 48 ]
[ [ "h" "i" "m"  ] 317 ]
[ [ "h" "i" "m" "s" "e" "l" "f"  ] 58 ]
[ [ "h" "i" "s"  ] 638 ]
[ [ "h" "i" "s" "t" "o" "r" "y"  ] 12 ]
[ [ "h" "i" "t" "m" "a" "n"  ] 42 ]
[ [ "h" "o" "m" "e"  ] 27 ]
[ [ "h" "o" "n" "o" "r"  ] 21 ]
[ [ "h" "o" "r" "r" "i" "b" "l" "e"  ] 14 ]
[ [ "h" "o" "r" "s" "e"  ] 14 ]
[ [ "h" "o" "u" "r"  ] 24 ]
[ [ "h" "o" "u" "s" "e"  ] 19 ]
[ [ "h" "o" "w"  ] 156 ]
[ [ "h" "o" "w" "e" "v" "e" "r"  ] 29 ]
[ [ "h" "u" "m" "a" "n"  ] 14 ]
[ [ "h" "u" "m" "o" "r"  ] 18 ]
[ [ "h" "u" "r" "t"  ] 11 ]
[ [ "h" "u" "s" "b" "a" "n" "d"  ] 29 ]
[ [ "i" "f"  ] 165 ]
[ [ "i" "l" "l"  ] 13 ]
[ [ "i" "l" "l" "u" "s" "i" "o" "n" "s"  ] 16 ]
[ [ "i" "m" "a" "g" "i" "n" "e"  ] 14 ]
[ [ "i" "m" "p" "o" "r" "t" "a" "n" "c" "e"  ] 11 ]
[ [ "i" "n"  ] 1605 ]
[ [ "i" "n" "d" "e" "e" "d"  ] 70 ]
[ [ "i" "n" "s" "t" "e" "a" "d"  ] 21 ]
[ [ "i" "n" "t" "e" "l" "l" "i" "g" "e" "n" "t"  ] 11 ]
[ [ "i" "n" "t" "e" "r" "e" "s" "t"  ] 11 ]
[ [ "i" "n" "t" "o"  ] 181 ]
[ [ "i" "s"  ] 1313 ]
[ [ "i" "t"  ] 1146 ]
[ [ "i" "t" "s"  ] 48 ]
[ [ "j" "o" "h" "n"  ] 18 ]
[ [ "j" "o" "y"  ] 14 ]
[ [ "j" "u" "s" "t"  ] 104 ]
[ [ "j" "u" "s" "t" "i" "c" "e"  ] 63 ]
[ [ "k" "e" "e" "p"  ] 27 ]
[ [ "k" "e" "p" "t"  ] 16 ]
[ [ "k" "i" "n" "d"  ] 17 ]
[ [ "k" "i" "n" "g"  ] 132 ]
[ [ "k" "n" "e" "w"  ] 36 ]
[ [ "k" "n" "o" "w"  ] 162 ]
[ [ "k" "n" "o" "w" "l" "e" "d" "g" "e"  ] 11 ]
[ [ "k" "n" "o" "w" "n"  ] 22 ]
[ [ "k" "n" "o" "w" "s"  ] 17 ]
[ [ "l" "a" "d"  ] 11 ]
[ [ "l" "a" "d" "i" "e" "s"  ] 16 ]
[ [ "l" "a" "d" "y"  ] 49 ]
[ [ "l" "a" "k" "e"  ] 13 ]
[ [ "l" "a" "n" "c" "e"  ] 16 ]
[ [ "l" "a" "n" "d"  ] 12 ]
[ [ "l" "a" "r" "g" "e"  ] 29 ]
[ [ "l" "a" "s" "t"  ] 37 ]
[ [ "l" "a" "t" "e" "r"  ] 15 ]
[ [ "l" "a" "u" "g" "h"  ] 13 ]
[ [ "l" "a" "u" "g" "h" "e" "d"  ] 16 ]
[ [ "l" "a" "u" "g" "h" "t" "e" "r"  ] 12 ]
[ [ "l" "a" "w"  ] 26 ]
[ [ "l" "a" "y"  ] 11 ]
[ [ "l" "e" "a" "r" "n" "e" "d"  ] 13 ]
[ [ "l" "e" "a" "s" "t"  ] 25 ]
[ [ "l" "e" "a" "v" "e"  ] 21 ]
[ [ "l" "e" "a" "v" "i" "n" "g"  ] 12 ]
[ [ "l" "e" "f" "t"  ] 54 ]
[ [ "l" "e" "s" "s"  ] 68 ]
[ [ "l" "e" "t"  ] 53 ]
[ [ "l" "e" "u" "k"  ] 35 ]
[ [ "l" "i" "b" "r" "a" "r" "y"  ] 13 ]
[ [ "l" "i" "f" "e"  ] 86 ]
[ [ "l" "i" "g" "h" "t"  ] 26 ]
[ [ "l" "i" "k" "e"  ] 149 ]
[ [ "l" "i" "p" "s"  ] 20 ]
[ [ "l" "i" "s" "a"  ] 78 ]
[ [ "l" "i" "t" "t" "l" "e"  ] 146 ]
[ [ "l" "i" "v" "e"  ] 30 ]
[ [ "l" "i" "v" "e" "d"  ] 21 ]
[ [ "l" "i" "v" "e" "s"  ] 11 ]
[ [ "l" "i" "v" "i" "n" "g"  ] 27 ]
[ [ "l" "o" "g" "r" "e" "u" "s"  ] 42 ]
[ [ "l" "o" "n" "g"  ] 104 ]
[ [ "l" "o" "n" "g" "e" "r"  ] 32 ]
[ [ "l" "o" "o" "k"  ] 49 ]
[ [ "l" "o" "o" "k" "e" "d"  ] 44 ]
[ [ "l" "o" "o" "k" "i" "n" "g"  ] 38 ]
[ [ "l" "o" "r" "d"  ] 11 ]
[ [ "l" "o" "s" "t"  ] 17 ]
[ [ "l" "o" "v" "e"  ] 121 ]
[ [ "l" "o" "v" "e" "d"  ] 81 ]
[ [ "l" "o" "v" "e" "l" "i" "e" "s" "t"  ] 14 ]
[ [ "l" "o" "v" "e" "l" "i" "n" "e" "s" "s"  ] 12 ]
[ [ "l" "o" "v" "e" "l" "y"  ] 35 ]
[ [ "l" "o" "w"  ] 13 ]
[ [ "l" "y" "i" "n" "g"  ] 13 ]
[ [ "m" "a" "d" "a" "m" "e"  ] 51 ]
[ [ "m" "a" "d" "e"  ] 123 ]
[ [ "m" "a" "g" "i" "c"  ] 20 ]
[ [ "m" "a" "j" "e" "s" "t" "y"  ] 24 ]
[ [ "m" "a" "k" "e"  ] 59 ]
[ [ "m" "a" "k" "e" "s"  ] 21 ]
[ [ "m" "a" "k" "i" "n" "g"  ] 24 ]
[ [ "m" "a" "n"  ] 196 ]
[ [ "m" "a" "n" "a" "g" "e"  ] 12 ]
[ [ "m" "a" "n" "l" "y"  ] 12 ]
[ [ "m" "a" "n" "n" "e" "r"  ] 16 ]
[ [ "m" "a" "n" "y"  ] 86 ]
[ [ "m" "a" "r" "r" "i" "e" "d"  ] 46 ]
[ [ "m" "a" "r" "r" "y"  ] 15 ]
[ [ "m" "a" "s" "t" "e" "r"  ] 31 ]
[ [ "m" "a" "t" "h" "e" "m" "a" "t" "i" "c" "s"  ] 15 ]
[ [ "m" "a" "t" "t" "e" "r"  ] 60 ]
[ [ "m" "a" "t" "t" "e" "r" "s"  ] 43 ]
[ [ "m" "a" "y"  ] 150 ]
[ [ "m" "e"  ] 561 ]
[ [ "m" "e" "a" "n"  ] 33 ]
[ [ "m" "e" "a" "n" "i" "n" "g"  ] 15 ]
[ [ "m" "e" "a" "n" "w" "h" "i" "l" "e"  ] 26 ]
[ [ "m" "e" "m" "o" "r" "y"  ] 19 ]
[ [ "m" "e" "n"  ] 95 ]
[ [ "m" "e" "r" "e"  ] 14 ]
[ [ "m" "e" "r" "e" "l" "y"  ] 25 ]
[ [ "m" "e" "t"  ] 13 ]
[ [ "m" "i" "c" "h" "a" "e" "l"  ] 37 ]
[ [ "m" "i" "g" "h" "t"  ] 76 ]
[ [ "m" "i" "n" "d"  ] 25 ]
[ [ "m" "i" "n" "e"  ] 22 ]
[ [ "m" "i" "r" "r" "o" "r"  ] 16 ]
[ [ "m" "o" "m" "e" "n" "t"  ] 55 ]
[ [ "m" "o" "n" "s" "t" "r" "o" "u" "s"  ] 21 ]
[ [ "m" "o" "n" "t" "h" "s"  ] 13 ]
[ [ "m" "o" "o" "n"  ] 33 ]
[ [ "m" "o" "o" "n" "l" "i" "g" "h" "t"  ] 11 ]
[ [ "m" "o" "r" "e"  ] 197 ]
[ [ "m" "o" "r" "e" "o" "v" "e" "r"  ] 11 ]
[ [ "m" "o" "r" "n" "i" "n" "g"  ] 23 ]
[ [ "m" "o" "s" "t"  ] 65 ]
[ [ "m" "o" "t" "h" "e" "r"  ] 67 ]
[ [ "m" "o" "v" "e" "d"  ] 13 ]
[ [ "m" "o" "v" "i" "n" "g"  ] 11 ]
[ [ "m" "u" "c" "h"  ] 119 ]
[ [ "m" "u" "s" "t"  ] 148 ]
[ [ "m" "y"  ] 604 ]
[ [ "m" "y" "s" "e" "l" "f"  ] 31 ]
[ [ "m" "y" "t" "h"  ] 13 ]
[ [ "m" "y" "t" "h" "s"  ] 11 ]
[ [ "n" "a" "m" "e"  ] 35 ]
[ [ "n" "a" "m" "e" "s"  ] 11 ]
[ [ "n" "a" "t" "u" "r" "e"  ] 28 ]
[ [ "n" "e" "a" "r"  ] 14 ]
[ [ "n" "e" "c" "e" "s" "s" "a" "r" "y"  ] 16 ]
[ [ "n" "e" "e" "d"  ] 41 ]
[ [ "n" "e" "i" "t" "h" "e" "r"  ] 13 ]
[ [ "n" "e" "s" "s" "u" "s"  ] 25 ]
[ [ "n" "e" "v" "e" "r"  ] 135 ]
[ [ "n" "e" "w"  ] 25 ]
[ [ "n" "e" "x" "t"  ] 17 ]
[ [ "n" "i" "g" "h" "t"  ] 69 ]
[ [ "n" "o"  ] 425 ]
[ [ "n" "o" "b" "o" "d" "y"  ] 65 ]
[ [ "n" "o" "n" "e"  ] 69 ]
[ [ "n" "o" "n" "s" "e" "n" "s" "e"  ] 34 ]
[ [ "n" "o" "r"  ] 54 ]
[ [ "n" "o" "t"  ] 930 ]
[ [ "n" "o" "t" "h" "i" "n" "g"  ] 128 ]
[ [ "n" "o" "t" "i" "c" "e"  ] 16 ]
[ [ "n" "o" "t" "i" "c" "e" "d"  ] 12 ]
[ [ "n" "o" "t" "i" "o" "n"  ] 21 ]
[ [ "n" "o" "t" "i" "o" "n" "s"  ] 13 ]
[ [ "n" "o" "w"  ] 459 ]
[ [ "n" "o" "w" "h" "e" "r" "e"  ] 11 ]
[ [ "n" "u" "m" "b" "e" "r"  ] 13 ]
[ [ "o" "b" "s" "e" "r" "v" "e"  ] 18 ]
[ [ "o" "b" "s" "e" "r" "v" "e" "d"  ] 26 ]
[ [ "o" "f"  ] 2941 ]
[ [ "o" "f" "f"  ] 39 ]
[ [ "o" "f" "t" "e" "n"  ] 30 ]
[ [ "o" "h"  ] 116 ]
[ [ "o" "l" "d"  ] 104 ]
[ [ "o" "n"  ] 159 ]
[ [ "o" "n" "c" "e"  ] 67 ]
[ [ "o" "n" "e"  ] 256 ]
[ [ "o" "n" "l" "y"  ] 106 ]
[ [ "o" "p" "e" "n"  ] 16 ]
[ [ "o" "p" "e" "n" "e" "d"  ] 13 ]
[ [ "o" "p" "i" "n" "i" "o" "n"  ] 16 ]
[ [ "o" "r"  ] 211 ]
[ [ "o" "r" "d" "e" "r"  ] 30 ]
[ [ "o" "r" "d" "i" "n" "a" "r" "y"  ] 12 ]
[ [ "o" "t" "h" "e" "r"  ] 162 ]
[ [ "o" "t" "h" "e" "r" "s"  ] 18 ]
[ [ "o" "t" "h" "e" "r" "w" "i" "s" "e"  ] 18 ]
[ [ "o" "u" "g" "h" "t"  ] 29 ]
[ [ "o" "u" "r"  ] 49 ]
[ [ "o" "u" "t"  ] 151 ]
[ [ "o" "v" "e" "r"  ] 104 ]
[ [ "o" "w" "n"  ] 75 ]
[ [ "p" "a" "g" "e"  ] 373 ]
[ [ "p" "a" "i" "n" "t" "e" "d"  ] 11 ]
[ [ "p" "a" "l" "a" "c" "e"  ] 15 ]
[ [ "p" "a" "r" "t"  ] 47 ]
[ [ "p" "a" "r" "t" "e" "d"  ] 11 ]
[ [ "p" "a" "r" "t" "i" "c" "u" "l" "a" "r"  ] 12 ]
[ [ "p" "a" "r" "t" "i" "c" "u" "l" "a" "r" "l" "y"  ] 16 ]
[ [ "p" "a" "s" "s"  ] 13 ]
[ [ "p" "a" "s" "s" "e" "d"  ] 42 ]
[ [ "p" "a" "u" "s" "e" "d"  ] 11 ]
[ [ "p" "a" "w" "n" "b" "r" "o" "k" "e" "r"  ] 31 ]
[ [ "p" "e" "o" "p" "l" "e"  ] 100 ]
[ [ "p" "e" "r" "c" "e" "i" "v" "e"  ] 28 ]
[ [ "p" "e" "r" "f" "e" "c" "t"  ] 11 ]
[ [ "p" "e" "r" "f" "e" "c" "t" "l" "y"  ] 19 ]
[ [ "p" "e" "r" "h" "a" "p" "s"  ] 19 ]
[ [ "p" "e" "r" "m" "i" "t" "t" "e" "d"  ] 12 ]
[ [ "p" "e" "r" "s" "o" "n"  ] 56 ]
[ [ "p" "e" "r" "s" "o" "n" "s"  ] 57 ]
[ [ "p" "h" "i" "l" "i" "s" "t" "i" "n" "e" "s"  ] 29 ]
[ [ "p" "h" "i" "l" "o" "l" "o" "g" "i" "s" "t"  ] 19 ]
[ [ "p" "l" "a" "c" "e"  ] 90 ]
[ [ "p" "l" "a" "i" "n"  ] 17 ]
[ [ "p" "l" "a" "i" "n" "l" "y"  ] 12 ]
[ [ "p" "l" "a" "y"  ] 13 ]
[ [ "p" "l" "e" "a" "s" "a" "n" "t"  ] 22 ]
[ [ "p" "l" "e" "a" "s" "a" "n" "t" "l" "y"  ] 11 ]
[ [ "p" "l" "e" "a" "s" "e" "d"  ] 13 ]
[ [ "p" "l" "e" "a" "s" "u" "r" "e"  ] 30 ]
[ [ "p" "l" "e" "a" "s" "u" "r" "e" "s"  ] 13 ]
[ [ "p" "o" "e" "t"  ] 33 ]
[ [ "p" "o" "i" "n" "t"  ] 27 ]
[ [ "p" "o" "o" "r"  ] 36 ]
[ [ "p" "o" "p" "e"  ] 13 ]
[ [ "p" "o" "s" "s" "i" "b" "l" "e"  ] 21 ]
[ [ "p" "o" "s" "s" "i" "b" "l" "y"  ] 19 ]
[ [ "p" "o" "w" "e" "r"  ] 25 ]
[ [ "p" "r" "a" "i" "s" "e"  ] 13 ]
[ [ "p" "r" "a" "y"  ] 13 ]
[ [ "p" "r" "e" "c" "i" "s" "e" "l" "y"  ] 17 ]
[ [ "p" "r" "e" "f" "e" "r"  ] 20 ]
[ [ "p" "r" "e" "s" "e" "n" "t" "l" "y"  ] 45 ]
[ [ "p" "r" "e" "t" "t" "y"  ] 20 ]
[ [ "p" "r" "i" "d" "e"  ] 22 ]
[ [ "p" "r" "i" "e" "s" "t"  ] 20 ]
[ [ "p" "r" "i" "e" "s" "t" "s"  ] 16 ]
[ [ "p" "r" "i" "n" "c" "e"  ] 47 ]
[ [ "p" "r" "i" "n" "c" "e" "s" "s"  ] 48 ]
[ [ "p" "r" "i" "v" "a" "t" "e"  ] 13 ]
[ [ "p" "r" "o" "b" "a" "b" "l" "y"  ] 30 ]
[ [ "p" "r" "o" "p" "e" "r"  ] 13 ]
[ [ "p" "r" "o" "u" "d"  ] 14 ]
[ [ "p" "u" "t"  ] 68 ]
[ [ "p" "u" "z" "z" "l" "e" "d"  ] 11 ]
[ [ "q" "u" "e" "e" "n"  ] 138 ]
[ [ "q" "u" "e" "e" "r"  ] 27 ]
[ [ "q" "u" "e" "s" "t" "i" "o" "n"  ] 19 ]
[ [ "q" "u" "e" "s" "t" "i" "o" "n" "s"  ] 12 ]
[ [ "q" "u" "i" "e" "t"  ] 15 ]
[ [ "q" "u" "i" "t" "e"  ] 117 ]
[ [ "r" "a" "i" "s" "e" "d"  ] 13 ]
[ [ "r" "a" "t" "h" "e" "r"  ] 71 ]
[ [ "r" "e" "a" "d"  ] 17 ]
[ [ "r" "e" "a" "d" "y"  ] 14 ]
[ [ "r" "e" "a" "l"  ] 21 ]
[ [ "r" "e" "a" "l" "l" "y"  ] 65 ]
[ [ "r" "e" "a" "s" "o" "n"  ] 45 ]
[ [ "r" "e" "a" "s" "o" "n" "a" "b" "l" "e"  ] 11 ]
[ [ "r" "e" "a" "s" "o" "n" "a" "b" "l" "y"  ] 13 ]
[ [ "r" "e" "a" "s" "o" "n" "s"  ] 12 ]
[ [ "r" "e" "d"  ] 31 ]
[ [ "r" "e" "f" "l" "e" "c" "t" "e" "d"  ] 23 ]
[ [ "r" "e" "g" "a" "r" "d" "e" "d"  ] 22 ]
[ [ "r" "e" "g" "r" "e" "t"  ] 21 ]
[ [ "r" "e" "m" "a" "r" "k" "a" "b" "l" "e"  ] 12 ]
[ [ "r" "e" "m" "e" "m" "b" "e" "r"  ] 59 ]
[ [ "r" "e" "m" "e" "m" "b" "e" "r" "e" "d"  ] 28 ]
[ [ "r" "e" "p" "e" "a" "t"  ] 13 ]
[ [ "r" "e" "p" "l" "i" "e" "d"  ] 42 ]
[ [ "r" "e" "p" "o" "r" "t"  ] 14 ]
[ [ "r" "e" "s" "p" "e" "c" "t" "a" "b" "l" "e"  ] 12 ]
[ [ "r" "e" "s" "t"  ] 14 ]
[ [ "r" "e" "t" "u" "r" "n"  ] 17 ]
[ [ "r" "i" "g" "h" "t"  ] 46 ]
[ [ "r" "o" "d" "e"  ] 14 ]
[ [ "r" "o" "o" "m"  ] 20 ]
[ [ "r" "o" "s" "e"  ] 13 ]
[ [ "r" "u" "l" "e" "s"  ] 11 ]
[ [ "s" "a" "d"  ] 13 ]
[ [ "s" "a" "i" "d"  ] 379 ]
[ [ "s" "a" "m" "e"  ] 20 ]
[ [ "s" "a" "t"  ] 37 ]
[ [ "s" "a" "t" "a" "n"  ] 45 ]
[ [ "s" "a" "v" "e"  ] 41 ]
[ [ "s" "a" "w"  ] 63 ]
[ [ "s" "a" "y"  ] 97 ]
[ [ "s" "a" "y" "i" "n" "g"  ] 28 ]
[ [ "s" "a" "y" "s"  ] 266 ]
[ [ "s" "e" "a"  ] 21 ]
[ [ "s" "e" "a" "r" "c" "h"  ] 13 ]
[ [ "s" "e" "c" "o" "n" "d"  ] 15 ]
[ [ "s" "e" "e"  ] 130 ]
[ [ "s" "e" "e" "m"  ] 35 ]
[ [ "s" "e" "e" "m" "e" "d"  ] 73 ]
[ [ "s" "e" "e" "m" "s"  ] 40 ]
[ [ "s" "e" "e" "n"  ] 56 ]
[ [ "s" "e" "l" "f"  ] 13 ]
[ [ "s" "e" "n" "s" "e"  ] 27 ]
[ [ "s" "e" "r" "e" "d" "a"  ] 42 ]
[ [ "s" "e" "r" "v" "e"  ] 20 ]
[ [ "s" "e" "t"  ] 16 ]
[ [ "s" "e" "v" "e" "n"  ] 12 ]
[ [ "s" "h" "a" "d" "o" "w"  ] 80 ]
[ [ "s" "h" "a" "l" "l"  ] 87 ]
[ [ "s" "h" "e"  ] 367 ]
[ [ "s" "h" "i" "n" "i" "n" "g"  ] 11 ]
[ [ "s" "h" "i" "r" "t"  ] 49 ]
[ [ "s" "h" "o" "o" "k"  ] 15 ]
[ [ "s" "h" "o" "r" "t"  ] 15 ]
[ [ "s" "h" "o" "u" "l" "d"  ] 85 ]
[ [ "s" "h" "o" "w"  ] 15 ]
[ [ "s" "h" "o" "w" "e" "d"  ] 16 ]
[ [ "s" "h" "r" "u" "g" "g" "e" "d"  ] 21 ]
[ [ "s" "h" "y"  ] 13 ]
[ [ "s" "i" "d" "e"  ] 35 ]
[ [ "s" "i" "g" "h" "e" "d"  ] 16 ]
[ [ "s" "i" "l" "v" "e" "r"  ] 16 ]
[ [ "s" "i" "m" "p" "l" "e"  ] 20 ]
[ [ "s" "i" "m" "p" "l" "y"  ] 15 ]
[ [ "s" "i" "n" "c" "e"  ] 79 ]
[ [ "s" "i" "r"  ] 95 ]
[ [ "s" "i" "s" "t" "e" "r"  ] 17 ]
[ [ "s" "i" "t"  ] 13 ]
[ [ "s" "i" "t" "t" "i" "n" "g"  ] 11 ]
[ [ "s" "l" "e" "e" "p"  ] 13 ]
[ [ "s" "m" "a" "l" "l"  ] 33 ]
[ [ "s" "m" "i" "l" "e" "d"  ] 13 ]
[ [ "s" "m" "i" "l" "i" "n" "g"  ] 17 ]
[ [ "s" "o"  ] 668 ]
[ [ "s" "o" "f" "t"  ] 14 ]
[ [ "s" "o" "m" "e"  ] 94 ]
[ [ "s" "o" "m" "e" "h" "o" "w"  ] 20 ]
[ [ "s" "o" "m" "e" "t" "h" "i" "n" "g"  ] 57 ]
[ [ "s" "o" "m" "e" "t" "i" "m" "e" "s"  ] 25 ]
[ [ "s" "o" "m" "e" "w" "h" "a" "t"  ] 16 ]
[ [ "s" "o" "n"  ] 18 ]
[ [ "s" "o" "n" "g"  ] 13 ]
[ [ "s" "o" "o" "n"  ] 13 ]
[ [ "s" "o" "r" "r" "y"  ] 12 ]
[ [ "s" "o" "r" "t"  ] 55 ]
[ [ "s" "p" "e" "a" "k"  ] 36 ]
[ [ "s" "p" "e" "a" "k" "i" "n" "g"  ] 21 ]
[ [ "s" "p" "o" "k" "e"  ] 33 ]
[ [ "s" "t" "a" "f" "f"  ] 20 ]
[ [ "s" "t" "a" "n" "d"  ] 15 ]
[ [ "s" "t" "a" "n" "d" "i" "n" "g"  ] 13 ]
[ [ "s" "t" "a" "y" "e" "d"  ] 13 ]
[ [ "s" "t" "i" "l" "l"  ] 118 ]
[ [ "s" "t" "o" "n" "e"  ] 15 ]
[ [ "s" "t" "o" "o" "d"  ] 25 ]
[ [ "s" "t" "o" "r" "y"  ] 12 ]
[ [ "s" "t" "r" "a" "n" "g" "e"  ] 23 ]
[ [ "s" "t" "r" "o" "n" "g"  ] 12 ]
[ [ "s" "u" "c" "h"  ] 223 ]
[ [ "s" "u" "n"  ] 21 ]
[ [ "s" "u" "n" "l" "i" "g" "h" "t"  ] 12 ]
[ [ "s" "u" "n" "r" "i" "s" "e"  ] 16 ]
[ [ "s" "u" "p" "p" "e" "r"  ] 12 ]
[ [ "s" "u" "p" "p" "o" "s" "e"  ] 15 ]
[ [ "s" "u" "r" "e"  ] 49 ]
[ [ "s" "u" "r" "e" "l" "y"  ] 11 ]
[ [ "s" "u" "s" "p" "e" "c" "t"  ] 13 ]
[ [ "s" "w" "o" "r" "d"  ] 33 ]
[ [ "t" "a" "k" "e"  ] 65 ]
[ [ "t" "a" "k" "e" "n"  ] 11 ]
[ [ "t" "a" "l" "e"  ] 27 ]
[ [ "t" "a" "l" "k"  ] 57 ]
[ [ "t" "a" "l" "k" "e" "d"  ] 33 ]
[ [ "t" "a" "l" "k" "i" "n" "g"  ] 41 ]
[ [ "t" "a" "l" "l"  ] 22 ]
[ [ "t" "a" "s" "t" "e"  ] 15 ]
[ [ "t" "e" "l" "l"  ] 71 ]
[ [ "t" "e" "l" "l" "i" "n" "g"  ] 21 ]
[ [ "t" "e" "l" "l" "s"  ] 16 ]
[ [ "t" "e" "n" "d" "e" "r"  ] 14 ]
[ [ "t" "e" "r" "e" "u"  ] 11 ]
[ [ "t" "h" "a" "n"  ] 121 ]
[ [ "t" "h" "a" "t"  ] 1686 ]
[ [ "t" "h" "e"  ] 3887 ]
[ [ "t" "h" "e" "i" "r"  ] 139 ]
[ [ "t" "h" "e" "m"  ] 145 ]
[ [ "t" "h" "e" "n"  ] 318 ]
[ [ "t" "h" "e" "r" "e"  ] 359 ]
[ [ "t" "h" "e" "r" "e" "a" "f" "t" "e" "r"  ] 16 ]
[ [ "t" "h" "e" "r" "e" "f" "o" "r" "e"  ] 19 ]
[ [ "t" "h" "e" "s" "e"  ] 195 ]
[ [ "t" "h" "e" "y"  ] 336 ]
[ [ "t" "h" "i" "n"  ] 11 ]
[ [ "t" "h" "i" "n" "g"  ] 113 ]
[ [ "t" "h" "i" "n" "g" "s"  ] 123 ]
[ [ "t" "h" "i" "n" "k"  ] 141 ]
[ [ "t" "h" "i" "n" "k" "i" "n" "g"  ] 34 ]
[ [ "t" "h" "i" "n" "k" "s"  ] 11 ]
[ [ "t" "h" "i" "r" "d"  ] 15 ]
[ [ "t" "h" "i" "r" "t" "y"  ] 15 ]
[ [ "t" "h" "i" "s"  ] 718 ]
[ [ "t" "h" "o" "s" "e"  ] 42 ]
[ [ "t" "h" "o" "u"  ] 15 ]
[ [ "t" "h" "o" "u" "g" "h"  ] 74 ]
[ [ "t" "h" "o" "u" "g" "h" "t"  ] 57 ]
[ [ "t" "h" "o" "u" "g" "h" "t" "s"  ] 12 ]
[ [ "t" "h" "r" "a" "g" "n" "a" "r"  ] 28 ]
[ [ "t" "h" "r" "e" "e"  ] 52 ]
[ [ "t" "h" "r" "o" "a" "t"  ] 13 ]
[ [ "t" "h" "r" "o" "n" "e"  ] 17 ]
[ [ "t" "h" "r" "o" "u" "g" "h"  ] 80 ]
[ [ "t" "h" "u" "s"  ] 112 ]
[ [ "t" "i" "m" "e"  ] 143 ]
[ [ "t" "i" "m" "e" "s"  ] 15 ]
[ [ "t" "i" "s"  ] 139 ]
[ [ "t" "o"  ] 2327 ]
[ [ "t" "o" "g" "e" "t" "h" "e" "r"  ] 31 ]
[ [ "t" "o" "l" "d"  ] 54 ]
[ [ "t" "o" "n" "g" "u" "e"  ] 11 ]
[ [ "t" "o" "o"  ] 121 ]
[ [ "t" "o" "o" "k"  ] 28 ]
[ [ "t" "o" "w" "a" "r" "d"  ] 40 ]
[ [ "t" "r" "e" "e"  ] 25 ]
[ [ "t" "r" "o" "u" "b" "l" "e"  ] 15 ]
[ [ "t" "r" "u" "e"  ] 27 ]
[ [ "t" "r" "u" "t" "h"  ] 30 ]
[ [ "t" "u" "r" "n"  ] 22 ]
[ [ "t" "u" "r" "n" "e" "d"  ] 24 ]
[ [ "t" "w" "e" "l" "v" "e"  ] 14 ]
[ [ "t" "w" "e" "n" "t" "y"  ] 18 ]
[ [ "t" "w" "i" "l" "i" "g" "h" "t"  ] 16 ]
[ [ "t" "w" "o"  ] 104 ]
[ [ "u" "n" "d" "e" "r"  ] 44 ]
[ [ "u" "n" "d" "e" "r" "s" "t" "a" "n" "d"  ] 76 ]
[ [ "u" "n" "f" "a" "i" "r"  ] 14 ]
[ [ "u" "n" "r" "e" "a" "s" "o" "n" "a" "b" "l" "e"  ] 11 ]
[ [ "u" "n" "t" "i" "l"  ] 30 ]
[ [ "u" "n" "u" "s" "u" "a" "l"  ] 11 ]
[ [ "u" "p"  ] 71 ]
[ [ "u" "p" "o" "n"  ] 220 ]
[ [ "u" "s"  ] 107 ]
[ [ "u" "s" "e"  ] 23 ]
[ [ "u" "s" "e" "d"  ] 23 ]
[ [ "v" "a" "i" "n"  ] 12 ]
[ [ "v" "a" "m" "p" "i" "r" "e"  ] 13 ]
[ [ "v" "e" "r" "s" "e" "s"  ] 13 ]
[ [ "v" "e" "r" "y"  ] 352 ]
[ [ "v" "i" "r" "t" "u" "e"  ] 11 ]
[ [ "v" "o" "i" "c" "e"  ] 29 ]
[ [ "w" "a" "i" "t" "e" "d"  ] 12 ]
[ [ "w" "a" "l" "l"  ] 12 ]
[ [ "w" "a" "r"  ] 23 ]
[ [ "w" "a" "s"  ] 1275 ]
[ [ "w" "a" "t" "e" "r"  ] 19 ]
[ [ "w" "a" "y"  ] 120 ]
[ [ "w" "a" "y" "s"  ] 12 ]
[ [ "w" "e"  ] 217 ]
[ [ "w" "e" "a" "r" "i" "n" "g"  ] 12 ]
[ [ "w" "e" "d" "n" "e" "s" "d" "a" "y"  ] 14 ]
[ [ "w" "e" "l" "l"  ] 228 ]
[ [ "w" "e" "n" "t"  ] 98 ]
[ [ "w" "e" "r" "e"  ] 334 ]
[ [ "w" "h" "a" "t"  ] 368 ]
[ [ "w" "h" "a" "t" "e" "v" "e" "r"  ] 60 ]
[ [ "w" "h" "e" "n"  ] 172 ]
[ [ "w" "h" "e" "r" "e"  ] 68 ]
[ [ "w" "h" "e" "r" "e" "a" "s"  ] 12 ]
[ [ "w" "h" "e" "r" "e" "i" "n"  ] 45 ]
[ [ "w" "h" "e" "t" "h" "e" "r"  ] 11 ]
[ [ "w" "h" "i" "c" "h"  ] 403 ]
[ [ "w" "h" "i" "l" "e"  ] 110 ]
[ [ "w" "h" "i" "t" "e"  ] 49 ]
[ [ "w" "h" "o"  ] 346 ]
[ [ "w" "h" "o" "l" "e"  ] 34 ]
[ [ "w" "h" "o" "m"  ] 122 ]
[ [ "w" "h" "o" "s" "e"  ] 30 ]
[ [ "w" "h" "y"  ] 197 ]
[ [ "w" "i" "d" "e"  ] 14 ]
[ [ "w" "i" "f" "e"  ] 94 ]
[ [ "w" "i" "l" "l"  ] 274 ]
[ [ "w" "i" "l" "l" "i" "n" "g"  ] 14 ]
[ [ "w" "i" "n" "d"  ] 11 ]
[ [ "w" "i" "n" "d" "o" "w" "s"  ] 12 ]
[ [ "w" "i" "n" "e"  ] 15 ]
[ [ "w" "i" "s" "d" "o" "m"  ] 11 ]
[ [ "w" "i" "s" "e"  ] 20 ]
[ [ "w" "i" "s" "h"  ] 21 ]
[ [ "w" "i" "t" "h"  ] 859 ]
[ [ "w" "i" "t" "h" "i" "n"  ] 17 ]
[ [ "w" "i" "t" "h" "o" "u" "t"  ] 70 ]
[ [ "w" "o" "m" "a" "n"  ] 145 ]
[ [ "w" "o" "m" "e" "n"  ] 82 ]
[ [ "w" "o" "n" "d" "e" "r"  ] 43 ]
[ [ "w" "o" "n" "d" "e" "r" "e" "d"  ] 18 ]
[ [ "w" "o" "o" "d"  ] 15 ]
[ [ "w" "o" "o" "d" "s"  ] 15 ]
[ [ "w" "o" "r" "d"  ] 42 ]
[ [ "w" "o" "r" "d" "s"  ] 28 ]
[ [ "w" "o" "r" "k"  ] 17 ]
[ [ "w" "o" "r" "l" "d"  ] 59 ]
[ [ "w" "o" "r" "t" "h"  ] 13 ]
[ [ "w" "o" "u" "l" "d"  ] 343 ]
[ [ "w" "r" "o" "n" "g"  ] 25 ]
[ [ "y" "e" "a" "r"  ] 26 ]
[ [ "y" "e" "a" "r" "s"  ] 40 ]
[ [ "y" "e" "l" "l" "o" "w"  ] 14 ]
[ [ "y" "e" "s"  ] 103 ]
[ [ "y" "e" "t"  ] 116 ]
[ [ "y" "o" "l" "a" "n" "d" "e"  ] 18 ]
[ [ "y" "o" "n" "d" "e" "r"  ] 12 ]
[ [ "y" "o" "u"  ] 1647 ]
[ [ "y" "o" "u" "n" "g"  ] 104 ]
[ [ "y" "o" "u" "r"  ] 396 ]
[ [ "y" "o" "u" "r" "s"  ] 16 ]
[ [ "y" "o" "u" "r" "s" "e" "l" "f"  ] 25 ]
[ [ "y" "o" "u" "t" "h"  ] 40 ]

]
end

; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2003 by Uri Wilensky.  Updated 2003.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2003).  NetLogo Word Game model.
; http://ccl.northwestern.edu/netlogo/models/WordGame.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/WordGame
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
161
10
822
134
10
1
31.0
0
24
1
1
1
0

CC-WINDOW
5
506
831
601
Command Center

BUTTON
604
186
814
224
skip this word
place-word
NIL
1
T
OBSERVER
T
NIL

BUTTON
317
186
604
224
start the game
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
316
149
601
182
maximum-number-of-missing-letters
maximum-number-of-missing-letters
1
5
1
1
1
NIL

BUTTON
9
115
64
149
a
pick-letter "a"
NIL
1
T
OBSERVER
T
A

BUTTON
84
115
139
149
b
pick-letter "b"
NIL
1
T
OBSERVER
T
B

BUTTON
9
151
64
186
c
pick-letter "c"
NIL
1
T
OBSERVER
T
C

BUTTON
84
151
139
186
d
pick-letter "d"
NIL
1
T
OBSERVER
T
D

BUTTON
159
151
214
186
e
pick-letter "e"
NIL
1
T
OBSERVER
T
E

BUTTON
234
151
289
186
f
pick-letter "f"
NIL
1
T
OBSERVER
T
F

BUTTON
9
188
64
223
g
pick-letter "g"
NIL
1
T
OBSERVER
T
G

BUTTON
84
188
139
223
h
pick-letter "h"
NIL
1
T
OBSERVER
T
H

BUTTON
159
188
214
223
i
pick-letter "i"
NIL
1
T
OBSERVER
T
I

BUTTON
234
188
289
223
j
pick-letter "j"
NIL
1
T
OBSERVER
T
J

BUTTON
9
225
64
260
k
pick-letter "k"
NIL
1
T
OBSERVER
T
K

BUTTON
84
225
139
260
l
pick-letter "l"
NIL
1
T
OBSERVER
T
L

BUTTON
159
225
214
260
m
pick-letter "m"
NIL
1
T
OBSERVER
T
M

BUTTON
234
225
289
260
n
pick-letter "n"
NIL
1
T
OBSERVER
T
N

BUTTON
9
262
64
297
o
pick-letter "o"
NIL
1
T
OBSERVER
T
O

BUTTON
84
262
139
297
p
pick-letter "p"
NIL
1
T
OBSERVER
T
P

BUTTON
159
262
214
297
q
pick-letter "q"
NIL
1
T
OBSERVER
T
Q

BUTTON
234
262
289
297
r
pick-letter "r"
NIL
1
T
OBSERVER
T
R

BUTTON
9
299
64
334
s
pick-letter "s"
NIL
1
T
OBSERVER
T
S

BUTTON
84
299
139
334
t
pick-letter "t"
NIL
1
T
OBSERVER
T
T

BUTTON
159
299
214
334
u
pick-letter "u"
NIL
1
T
OBSERVER
T
U

BUTTON
234
299
289
334
v
pick-letter "v"
NIL
1
T
OBSERVER
T
V

BUTTON
9
336
64
371
w
pick-letter "w"
NIL
1
T
OBSERVER
T
W

BUTTON
84
336
139
371
x
pick-letter "x"
NIL
1
T
OBSERVER
T
X

BUTTON
159
336
214
371
y
pick-letter "y"
NIL
1
T
OBSERVER
T
Y

BUTTON
234
336
289
371
z
pick-letter "z"
NIL
1
T
OBSERVER
T
Z

PLOT
317
228
574
371
frequencies
frequency of word
tries
0.0
67.0
0.0
10.0
true
false
PENS
"default" 1.0 2 -16777216 true
"ftries" 1.0 0 -16777216 true

PLOT
578
227
815
371
length
length of word
tries
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 2 -16777216 true
"ltries" 1.0 0 -16777216 true

OUTPUT
317
379
677
492

@#$#@#$#@
WHAT IS IT?
-----------
This is a simple word game where you guess the missing letter highlighted in green. As you play the game, you will notice that your actions are being graphed on the bottom right of the screen. The two graphs are "the frequency of word in a book* with comparison to the number of tries it takes you to figure out the word" and "the length of the word with comparison to the number of tries it takes you to figure out the word."

*The words all come from a 1922 public domain book called "Jurgen: A Comedy of Justice" By James Cabell.


HOW IT WORKS
------------
There are two lists for any given word -- one of which is the list with all of the letters, and one of which is the displayed list. When you select a letter, it compares the letter you selected with the letter at the corresponding location in the complete word. It then graphs the number of tries it took you to guess the word against the length of the word and the frquenciy of the word in the corpus.


HOW TO USE IT
-------------
Press the "start the game" button to start.

If you want to guess a letter, select the letter on the left. The "command center" will tell you how many tries you've taken and which letters you've tried. You can try the same letter as many times as you want, though it won't help.  If you are particularly frustrated, you can skip the word using the "skip this word" button; it will be unlogged and ungraphed.


THINGS TO NOTICE
----------------
What's the shape of the graph?
Do you notice any repeated anomalies?


THINGS TO TRY
-------------
Try playing with a different number of missing letters. How does the graph change?
Try playing with a friend. How does the graph change?


EXTENDING THE MODEL
-------------------
 - Easier -
Make it such that another letter other than the last unknown is the highlighted one.
Make the graphs show some other interesting characteristics.
Time the user.
Score the user.
Make the graphs into constantly updating bar graphs. Bar graphs might give some other interesting data here.

 - Harder -
Try using a different text.
Compare texts.


NETLOGO FEATURES
------------------
The words list format is:
set word-list [
[ [ "w" "o" "r" "d" "1" ] <frequency of "word1" in text> ]
[ [ "w" "o" "r" "d" "2" ] <frequency of "word2" in text> ]
[ [ "w" "o" "r" "d" "3" ] <frequency of "word3" in text> ]
...
]


CREDITS AND REFERENCES
-------------------

A thank you to UNC, who put the text on the web from which the words and frequencies were compiled. The text itself from which the words were taken bears this copyright: This work is the property of the University of North Carolina at Chapel Hill. It may be used freely by individuals for research, teaching and personal use as long as this statement of availability is included in the text. http://docsouth.unc.edu/cabell/jurgen.html

To refer to this model in academic publications, please use: Wilensky, U. (2003). NetLogo Word Game model. http://ccl.northwestern.edu/netlogo/models/WordGame. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2003 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/WordGame for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
setup
@#$#@#$#@
@#$#@#$#@
