; add model procedures here


__include "./visualization/convert-to-stl.nls"
__include "./visualization/convert-to-gis.nls"
__include "./visualization/output.nls"
__include "./generic.nls"
__include "./paint-attributes.nls"
__include "./displays.nls"
__include "./roads/roads.nls"
__include "./terrain/terrain.nls"
__include "./developers/developers.nls"
__include "./developers/speculators.nls"
__include "./debugging/viewing-dirty-bits.nls"

__include "./gradients.nls"


globals [
  CITY-POPULATION
  CITY-LAND-COVER
  CENTER-X
  CENTER-Y
  NUMBER-OF-OWNED-PATCHES
  GLOBAL-TICKS

  Number-of-Images
]


to-report find-city-center
  locals [ owned-patches city-pop x y]
  set owned-patches patches with [owner != nobody]
  set city-pop sum values-from buildings [population]
  set x (sum values-from owned-patches [parcel-density * pxcor]) / city-pop
  set y (sum values-from owned-patches [parcel-density * pycor]) / city-pop
  report patch-at x y
end

to add-me-to-city-population
  set CITY-POPULATION (CITY-POPULATION + parcel-density)
  set CENTER-X (CENTER-X + (parcel-density * pxcor))
  set CENTER-Y (CENTER-Y + (parcel-density * pycor))
end

to take-me-from-city-population
  set CENTER-X (CENTER-X - (parcel-density * pxcor))
  set CENTER-Y (CENTER-Y - (parcel-density * pycor))
  set CITY-POPULATION (CITY-POPULATION - parcel-density)
  if (CITY-POPULATION < 0 )
  [ set CENTER-X 0
    set CENTER-Y 0
    set CITY-POPULATION 0
  ]
end

to startup
    SET-TERRAIN-GLOBALS
    level-terrain
    startup-roads
    Reset
end

to setup
  startup
  reset-timer
end

to Reset
    locals [temp]
    ct
    set pins-primaries []
    set pins-tertiaries []
    get-pins-water-boundaries

    set AVG-ELEVATION WATER-LEVEL
    set CENTER-X 0
    set CENTER-Y 0
    set CITY-POPULATION 0
    set CITY-LAND-COVER 0
    set GLOBAL-TICKS 0
    set DEBUG false

    clear-all-plots
    find-dist-to-water
    clear-all-buildings
    clear-roads

    without-interruption [
        ask residential-developers [ die ]
        ask commercial-developers [ die ]
        ask industrial-developers [ die ]
    ]
    create-residential-class
    create-commercial-class
    create-industrial-class

    init-dirty-bits
    init-residential-developers
    init-commercial-developers
    init-industrial-developers

    ask patches
    [ set residential-frequency 0
      set commercial-frequency 0
      set industrial-frequency 0
      set temp site-worth RESIDENTIAL self
      set temp site-worth COMMERCIAL self
      set temp site-worth INDUSTRIAL self
    ]
    Show-Running-Display
    find-dev-from-grid
end

to Go
    locals [ residential-ratio commercial-ratio industrial-ratio ]
    no-display
    DEBUG-PRINT "\nSTEP: " + GLOBAL-TICKS + "\n"
    ;without-interruption [
        generate-tertiary-roads
        create-primary-roads
    ;]
    ;set total-population ( residential-population + commercial-population + industrial-population )
    ; show total-population
    ask residential-developers
    [ without-interruption
      [ developmental-search RESIDENTIAL
      ]
    ]
    ask commercial-developers
    [ without-interruption
      [ developmental-search COMMERCIAL
      ]
    ]

    ask industrial-developers
    [ without-interruption
      [ developmental-search INDUSTRIAL
      ]
    ]


    Show-Running-Display
    ask commercial-developers [debug-print patch-here + " : " + who + " " + ifelse-value(owner-of patch-here = nobody)[patch-seen-before][building-seen-before-of owner-of patch-here]]
    ask patches with [selected] [set selected false]
    set GLOBAL-TICKS (GLOBAL-TICKS + 1)
    display
    update-plots
end

to update-plots
    set-current-plot "Populations"
    set-current-plot-pen "residential"
    plot global-population-of RESIDENTIAL
    set-current-plot-pen "commercial"
    plot global-population-of COMMERCIAL
    set-current-plot-pen "industrial"
    plot global-population-of INDUSTRIAL
end
@#$#@#$#@
GRAPHICS-WINDOW
1
10
614
644
100
100
3.0
1
10
1
1
1
0

CC-WINDOW
5
1284
1146
1379
Command Center

BUTTON
850
626
905
679
NIL
Reset
NIL
1
T
OBSERVER
T
NIL

BUTTON
118
647
207
739
NIL
Go
T
1
T
OBSERVER
NIL
NIL

SLIDER
623
569
827
602
CONNECTION-RADIUS
CONNECTION-RADIUS
1
20
20
1
1
NIL

SLIDER
623
499
827
532
MANHATTAN-RATIO
MANHATTAN-RATIO
0
10
2.0
0.1
1
NIL

SLIDER
622
640
724
673
X-SCALE
X-SCALE
2
10
10
1
1
NIL

SLIDER
726
640
828
673
Y-SCALE
Y-SCALE
2
10
10
1
1
NIL

SLIDER
623
604
724
637
DEV-X
DEV-X
0
20
0
1
1
NIL

SLIDER
727
604
827
637
DEV-Y
DEV-Y
0
20
0
1
1
NIL

SLIDER
623
534
827
567
MAX-ROAD-CONCENTRATION
MAX-ROAD-CONCENTRATION
0
1
0.3
0.05
1
NIL

BUTTON
757
281
884
314
NIL
Inc-DEV-X
T
1
T
OBSERVER
NIL
NIL

BUTTON
759
318
884
351
NIL
Inc-DEV-Y
T
1
T
OBSERVER
NIL
NIL

BUTTON
888
281
1006
314
NIL
Dec-DEV-X
T
1
T
OBSERVER
NIL
NIL

BUTTON
888
318
1006
351
NIL
Dec-DEV-Y
T
1
T
OBSERVER
NIL
NIL

BUTTON
756
354
883
387
NIL
Inc-RATIO
T
1
T
OBSERVER
NIL
NIL

BUTTON
888
355
1006
388
NIL
Dec-RATIO
T
1
T
OBSERVER
NIL
NIL

BUTTON
757
390
883
423
NIL
Inc-CONCENTRATION
T
1
T
OBSERVER
NIL
NIL

BUTTON
887
390
1007
423
NIL
Dec-CONCENTRATION
T
1
T
OBSERVER
NIL
NIL

BUTTON
1010
281
1124
314
NIL
Set-DEV-X
T
1
T
OBSERVER
NIL
NIL

BUTTON
1010
318
1124
351
NIL
Set-DEV-Y
T
1
T
OBSERVER
NIL
NIL

BUTTON
1010
355
1123
388
NIL
Set-RATIO
T
1
T
OBSERVER
NIL
NIL

BUTTON
1011
390
1124
423
NIL
Set-CONCENTRATION
T
1
T
OBSERVER
NIL
NIL

BUTTON
757
426
884
459
NIL
Inc-ELEVATION
T
1
T
OBSERVER
NIL
NIL

BUTTON
889
426
1006
459
NIL
Dec-ELEVATION
T
1
T
OBSERVER
NIL
NIL

BUTTON
1010
425
1125
458
NIL
Set-ELEVATION
T
1
T
OBSERVER
NIL
NIL

SLIDER
800
10
1004
43
GROUND-LEVEL
GROUND-LEVEL
0
255
84
1
1
NIL

SLIDER
800
45
1004
78
WATER-LEVEL
WATER-LEVEL
0
255
80
1
1
NIL

SLIDER
624
10
796
43
BRUSH-RADIUS
BRUSH-RADIUS
0
20
20
1
1
NIL

CHOOSER
624
46
796
91
BRUSH-TYPE
BRUSH-TYPE
"Round" "Square"
0

BUTTON
625
279
753
312
NIL
Show-DEV-X
NIL
1
T
OBSERVER
T
NIL

BUTTON
626
316
752
349
NIL
Show-DEV-Y
NIL
1
T
OBSERVER
T
NIL

BUTTON
626
352
752
385
NIL
Show-RATIO
NIL
1
T
OBSERVER
T
NIL

BUTTON
624
389
753
422
NIL
Show-CONCENTRATION
NIL
1
T
OBSERVER
T
NIL

BUTTON
623
426
751
459
NIL
Show-Terrain
NIL
1
T
OBSERVER
T
NIL

BUTTON
2
741
207
774
NIL
Show-Running-Display
NIL
1
T
OBSERVER
T
NIL

BUTTON
1011
45
1120
91
NIL
Clear-Roads
NIL
1
T
OBSERVER
T
NIL

CHOOSER
1
647
115
692
RUNNING-DISPLAY
RUNNING-DISPLAY
"None" "Terrain" "Roads" "Density" "Age" "Last Updated" "Residential Worth" "Commercial Worth" "Industrial Worth" "Developer Frequencies"
1

SLIDER
623
464
724
497
MIN-DISTANCE
MIN-DISTANCE
1
20
10
1
1
NIL

SLIDER
729
464
827
497
MAX-DISTANCE
MAX-DISTANCE
1
20
20
1
1
NIL

BUTTON
833
462
931
495
NIL
Set-Seed
T
1
T
OBSERVER
NIL
NIL

BUTTON
623
98
751
131
NIL
Show-MIN-DISTANCE
NIL
1
T
OBSERVER
T
NIL

BUTTON
625
134
751
167
NIL
Show-MAX-DISTANCE
NIL
1
T
OBSERVER
T
NIL

BUTTON
754
97
882
130
NIL
Inc-MIN-DISTANCE
T
1
T
OBSERVER
NIL
NIL

BUTTON
754
133
882
166
NIL
Inc-MAX-DISTANCE
T
1
T
OBSERVER
NIL
NIL

BUTTON
885
97
1002
130
NIL
Dec-MIN-DISTANCE
T
1
T
OBSERVER
NIL
NIL

BUTTON
885
134
1003
167
NIL
Dec-MAX-DISTANCE
T
1
T
OBSERVER
NIL
NIL

BUTTON
1006
99
1120
132
NIL
Set-MIN-DISTANCE
T
1
T
OBSERVER
NIL
NIL

BUTTON
1007
134
1122
167
NIL
Set-MAX-DISTANCE
T
1
T
OBSERVER
NIL
NIL

SLIDER
938
499
1127
532
ELEVATION-ROAD-CONST
ELEVATION-ROAD-CONST
0
2
0.4
0.05
1
NIL

BUTTON
725
1091
854
1124
NIL
show-derivative
T
1
T
OBSERVER
T
NIL

SLIDER
936
608
1128
641
MIN-CHANGE-IN-WORTH
MIN-CHANGE-IN-WORTH
0
1
0.0
0.025
1
NIL

BUTTON
1011
10
1120
44
NIL
level-terrain
NIL
1
T
OBSERVER
T
NIL

CHOOSER
1
694
115
739
OVERLAY-DISPLAY
OVERLAY-DISPLAY
"None" "Terrain-Boundaries" "Speculator Boundaries" "Owners" "Seen Before"
0

SLIDER
938
462
1126
495
PRIMARY-ROAD-CONSTRAINT
PRIMARY-ROAD-CONSTRAINT
0
50
24
1
1
NIL

SLIDER
937
537
1127
570
DESIRED-ELEVATION
DESIRED-ELEVATION
0
20
10
1
1
NIL

BUTTON
832
497
931
530
NIL
Draw-Primary
T
1
T
OBSERVER
NIL
NIL

SLIDER
936
573
1127
606
CONSIDER-LOWER-FREQ
CONSIDER-LOWER-FREQ
0
1
0.8
0.01
1
NIL

PLOT
376
647
614
817
Populations
time
population
0.0
10.0
0.0
10.0
true
true
PENS
"residential" 1.0 0 -256 true
"commercial" 1.0 0 -65536 true
"industrial" 1.0 0 -16776961 true

INPUTBOX
4
824
138
928
min-industrial-parcel-size
64
true
false
true

INPUTBOX
138
824
271
928
max-industrial-parcel-size
96
true
false
true

INPUTBOX
4
928
138
1032
min-commercial-parcel-size
5
true
false
true

INPUTBOX
138
928
271
1032
max-commercial-parcel-size
20
true
false
true

INPUTBOX
4
1032
143
1136
min-residential-parcel-size
5
true
false
true

INPUTBOX
138
1032
271
1136
max-residential-parcel-size
10
true
false
true

INPUTBOX
271
824
453
928
initial-industrial-building-density
2
true
false
true

INPUTBOX
271
928
453
1032
initial-commercial-building-density
2
true
false
true

INPUTBOX
271
1032
453
1136
initial-residential-building-density
.5
true
false
true

SWITCH
211
784
372
817
Capture-Movie?
Capture-Movie?
1
1
-1000

SWITCH
211
751
372
784
Capture-Screen-Shots?
Capture-Screen-Shots?
0
1
-1000

INPUTBOX
211
647
372
751
Output-Filename
results
true
false
true

BUTTON
798
1052
873
1085
NIL
test-run
T
1
T
OBSERVER
T
NIL

SLIDER
623
675
827
708
GRID-ANGLE
GRID-ANGLE
-45
45
0
1
1
Degrees

BUTTON
710
1052
795
1085
NIL
test-grids
T
1
T
OBSERVER
T
NIL

SWITCH
4
775
207
808
SimCity?
SimCity?
1
1
-1000

BUTTON
831
533
932
566
NIL
Erase-Buildings
T
1
T
OBSERVER
NIL
NIL

BUTTON
831
569
933
602
NIL
Erase-Roads
T
1
T
OBSERVER
NIL
NIL

BUTTON
921
646
1128
679
link-primary-road-to-world-edge
link-primary-road-to-world-edge patch start-x start-y
NIL
1
T
OBSERVER
T
NIL

SLIDER
918
994
1137
1027
new-grid-x-scale
new-grid-x-scale
2
10
5
1
1
NIL

SLIDER
918
1029
1137
1062
new-grid-y-scale
new-grid-y-scale
2
10
5
1
1
NIL

SLIDER
918
1064
1137
1097
new-grid-angle
new-grid-angle
-45
45
0
1
1
Degrees

BUTTON
918
1099
1026
1132
NIL
display-grid-seeds
NIL
1
T
OBSERVER
T
NIL

BUTTON
918
1202
1026
1235
NIL
move-grid-seed
T
1
T
OBSERVER
T
NIL

BUTTON
918
959
1027
992
NIL
new-grid-seed
NIL
1
T
OBSERVER
T
NIL

BUTTON
1030
959
1137
992
NIL
delete-grid-seed
NIL
1
T
OBSERVER
T
NIL

BUTTON
918
1237
1026
1270
NIL
set-grid-attributes
NIL
1
T
OBSERVER
T
NIL

SLIDER
471
877
669
910
number-of-industrial-developers
number-of-industrial-developers
0
20
5
1
1
NIL

SLIDER
471
910
669
943
number-of-commercial-developers
number-of-commercial-developers
0
20
5
1
1
NIL

SLIDER
471
943
669
976
number-of-residential-developers
number-of-residential-developers
0
20
5
1
1
NIL

SLIDER
471
1036
679
1069
number-of-tertiary-road-extenders
number-of-tertiary-road-extenders
1
20
8
1
1
NIL

SLIDER
471
1069
679
1102
number-of-tertiary-road-connectors
number-of-tertiary-road-connectors
1
20
20
1
1
NIL

SLIDER
471
1102
679
1135
number-of-primary-road-extenders
number-of-primary-road-extenders
0
10
5
1
1
NIL

TEXTBOX
476
844
668
876
These sliders initialize the number of building developers upon a Reset.

TEXTBOX
474
1003
678
1033
These sliders initialize the number of road developers upon a reset.

BUTTON
1029
1237
1137
1270
NIL
add-patches-to-grid
T
1
T
OBSERVER
T
NIL

BUTTON
1029
1202
1137
1235
NIL
select-grid-seed
T
1
T
OBSERVER
T
NIL

BUTTON
918
1133
1137
1166
NIL
show-patches-associated-with-grid
NIL
1
T
OBSERVER
T
NIL

BUTTON
918
1168
1137
1201
NIL
show-grid-alignments
NIL
1
T
OBSERVER
T
NIL

BUTTON
1028
1099
1137
1132
NIL
hide-grid-seeds
NIL
1
T
OBSERVER
T
NIL

BUTTON
625
244
753
277
NIL
Show-GRID-ANGLE
NIL
1
T
OBSERVER
T
NIL

BUTTON
756
244
881
277
NIL
Inc-GRID-ANGLE
T
1
T
OBSERVER
NIL
NIL

BUTTON
887
244
1003
277
NIL
Dec-GRID-ANGLE
T
1
T
OBSERVER
NIL
NIL

BUTTON
1008
244
1123
277
NIL
Set-GRID-ANGLE
T
1
T
OBSERVER
NIL
NIL

TEXTBOX
920
926
1133
956
This will be a better interface for multiple grids in the future ...

BUTTON
625
710
827
743
find-deviations-from-grid
find-dev-from-grid
NIL
1
T
OBSERVER
T
NIL

BUTTON
625
170
751
203
NIL
Show-X-SCALE
NIL
1
T
OBSERVER
T
NIL

BUTTON
754
170
883
203
NIL
Inc-X-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
625
206
751
239
NIL
Show-Y-SCALE
NIL
1
T
OBSERVER
T
NIL

BUTTON
754
206
883
239
NIL
Inc-Y-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
886
170
1004
203
NIL
Dec-X-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
887
206
1004
239
NIL
Dec-Y-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
1008
171
1123
204
NIL
Set-X-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
1009
206
1124
239
NIL
Set-Y-SCALE
T
1
T
OBSERVER
NIL
NIL

BUTTON
724
763
859
796
NIL
Show-Residential-Honey
NIL
1
T
OBSERVER
T
NIL

BUTTON
724
796
859
829
NIL
Show-Commercial-Honey
NIL
1
T
OBSERVER
T
NIL

BUTTON
724
829
859
862
NIL
Show-Industrial-Honey
NIL
1
T
OBSERVER
T
NIL

BUTTON
859
763
993
796
NIL
Inc-Residential-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
859
796
993
829
NIL
Inc-Commercial-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
859
829
993
862
NIL
Inc-Industrial-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
993
763
1133
796
NIL
Dec-Residential-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
993
796
1133
829
NIL
Dec-Commercial-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
993
829
1133
862
NIL
Dec-Industrial-Honey
T
1
T
OBSERVER
NIL
NIL

BUTTON
847
697
997
730
NIL
Erase-Primary-Roads
T
1
T
OBSERVER
T
NIL

SLIDER
689
878
906
911
desired-residential-population
desired-residential-population
0
1
0.54
0.01
1
NIL

SLIDER
689
911
912
944
desired-commercial-population
desired-commercial-population
0
1
0.26
0.01
1
NIL

SLIDER
689
944
899
977
desired-industrial-population
desired-industrial-population
0
1
0.26
0.01
1
NIL

BUTTON
1010
697
1127
730
NIL
fill-parcel-holes
NIL
1
T
OBSERVER
T
NIL

BUTTON
627
763
713
796
show %s
show-land-coverage-percentages
NIL
1
T
OBSERVER
T
NIL

BUTTON
624
803
713
836
NIL
test-roads
T
1
T
OBSERVER
NIL
NIL

SLIDER
473
1160
693
1193
desired-residential-land-cover
desired-residential-land-cover
0
1
0.88
0.01
1
NIL

SLIDER
471
1195
697
1228
desired-commercial-land-cover
desired-commercial-land-cover
0
1
0.07
0.01
1
NIL

SLIDER
472
1231
685
1264
desired-industrial-land-cover
desired-industrial-land-cover
0
1
0.12
0.01
1
NIL

@#$#@#$#@
DESCRIPTION OF CONTROLS
-----------------------

CHANCE-OF-TURNING:  Describes how often a turtle will decide to turn (it may end up going straight, left, or right).  0 means it will never turn, and 100 means that it is constantly trying to turn.

NUM-TURTLES:  Describes the number of turtles to be used in the simulation.  If this number is changed, the user must press Clear-Roads or Reset to effect the change.

RUNNING-DISPLAY:  Determines which view of the simulation ought to be displayed while running.  "Terrain" allows the user to see the relationship between the road network and the terrain, while "Roads" shows the user merely the road network and how much of the surrounding land is being serviced.

REMOVE-STRIPS:  Not really necessary - it removes narrow strips of land that may be formed when the road network is generating loops within itself.  It is not certain whether or not this is a bad thing.

DROP-RANDOM-BUILDER:  Allows the user to pick up a random turtle and drop it at a point in the world while the procedure is running.  Only one turtle is dropped per mouse click.

RESET:  Resets the road network, the terrain, and some global variables.  The default values are set to the global variables listed below.

GO:  Starts/stops the simulation.

SHOW-ROADS:  displays the road network (updates each button press).

CLEAR-ROADS:  Resets the road network only, and initializes the parameters associated with the road network according to the global variables listed below.


GLOBAL VARIABLES ASSOCIATED WITH ROAD PARAMETERIZATION
------------------------------------------------------

X-SCALE:  Determines the unit grid size in the x dimension.

Y-SCALE:  Determines the unit grid size in the y dimension.

DEV-X:  Determines the number of units of deviation in the +/- direction from the unit grid size in the x dimension.

DEV-Y:  Determines the number of units of deviation in the +/- direction from the unit grid size in the y dimention.

MAX-ROAD-CONCENTRATION:  Determines the maximum road concentration around a road segment.  According to the algorithm, when a road segment is considered for building, the ratio of road patches verses total patches surrounding and including the road segment must be less than the maximum road concentration for further consideration.  A max concentration of 0 will prevent the road network from looping, where a value of 100 will cause very dense road networks, that will eventually result in a solid area of pavement.

MANHATTAN-RATIO:  Perhaps a poor name for what it tries to describe.  When a road segment is considered for connecting two portions of the existing road network together, the ratio of the actual distance it takes to travel between the two points along the road network verses the manhattan distance between the two points must be greater than this ratio.  Basically, it is a measure of tolerance.  The turtles will tolerate traveling up to a distance of this ratio times the manhattan distance before it considers building a road.


GLOBAL VARIABLES ASSOCIATED WITH TERRAIN
----------------------------------------

GROUND-LEVEL:  Determines the current elevation of the terrain.

WATER-LEVEL:  Determines which elevation is the water level.


PARAMETER-PAINTING CONTROLS
---------------------------

BRUSH-RADIUS:  Determines the radius of the brush if a circle, and half the width if a square.

BRUSH-TYPE:  Determines if the brush is circular or square.

SHOW-<GLOBAL-VARIABLE>:  Shows in a pictoral manner the distribution of the global parameter.  Lighter areas are higher in value.

INC-<GLOBAL-VARIABLE>:  Increases the value of the global variable according to a feathered filter.

DEC-<GLOBAL-VARIABLE>:  Decreases the value of the global variable according to a feathered filter.

SET-<GLOBAL-VARIABLE>:  Sets the patches within the brush dimensions to the current slider setting of the global variable.


WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.2pre2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
