/******************************
 Created by Pin Ren
 Modified by Craig Brozefsky <craig@ccl.northwestern.edu>
 Modified again by Pin Ren <p-ren@cs.northwestern.edu>
******************************/
#include "cCSVFileReader.h"
#include <iostream>
using namespace std;
int main(int argc, char* argv[])
{
	// First construct a new CSV reader object
	cCSVFileReader * myReader = new cCSVFileReader();
	string Usage("./world-to-svg [-s[b] simplify_threshold] input_file_name output_file_name ");
	string Example1("./world-to-svg /home/pin/data/city/6-9-05.csv /home/pin/working_city/tmp_svg/tom_output6-9-05.svg");
	string Example2("./world-to-svg -s 0.000001 /home/pin/data/city/6-9-05.csv /home/pin/working_city/tmp_svg/tom_output6-9-05.svg");
    string Example3("./world-to-svg -sb 0.01 /home/pin/data/city/6-9-05.csv /home/pin/working_city/tmp_svg/tom_output6-9-05");
	
	if ((argc!=3)&&(argc!=5))	
	{
		cout<<"Usage:"<<endl<<Usage<<endl;
		cout<<endl<<"Example1:"<<endl<<endl<<Example1<<endl<<endl<<"Will generate one svg file without simplification"<<endl;
		cout<<endl<<"Example2:"<<endl<<endl<<Example2<<endl<<endl<<"Will generate one svg file with simplification threshold 0.00000001"<<endl;
		cout<<endl<<"Example3:"<<endl<<endl<<Example3<<endl<<endl<<"Will generate a batch of svg files with simplification threshold range from 0 to 0.01 with step 0.001"<<endl;
		exit(-1);
	}
	string inputFile, outputFile, mode;
	double simplify_threshold=-1.0;;
	bool batchMode;
	if (argc==3)
	{
		inputFile = std::string(argv[1]);
		outputFile = std::string(argv[2]);
		batchMode=false;
	}
	if (argc==5)
	{
		inputFile = std::string(argv[3]);
		outputFile = std::string(argv[4]);
		simplify_threshold=atof(argv[2]);
		mode = std::string(argv[1]);
		if (mode==std::string("-sb"))
			batchMode=true;
		else
			batchMode=false;
	}

	
	cout<<"Reading "<<inputFile<<"..."<<endl;
	if (myReader->readCSVFile(inputFile))
	{
		cout<<"Data File "<<inputFile<<" Read OK"<<endl;
	}
	else
	{
		cout<<"Fail to read data File "<<inputFile<<endl<<"please check the file name and format"<<endl;
		exit(-2);
	}
	cout<<"Preparing SVG output Data Structure..."<<endl;
	myReader->prepareSimplification();
	cout<<"Output Data Structure preparation done"<<endl;
	if (simplify_threshold>0)
	{
		cout<<"Simplifing with threshold..."<<simplify_threshold<<endl;
		if (!batchMode)
			myReader->simplifyPolygons(simplify_threshold);
		else
			myReader->simplifyPolygons(simplify_threshold,&outputFile,true);
		cout<<"Simplification done"<<endl;
	}

	//make sure the output file name is end with .svg
	int extDotPos = outputFile.find_last_of("."); //find the extension file name dot
	string fileExt="";
	if (extDotPos!=outputFile.npos)
		fileExt=outputFile.substr(extDotPos);
		
	if ((fileExt!=".svg")||(fileExt==""))
	{
		outputFile+=".svg";
	}
	cout<<"Generating Final SVG image..."<<endl;
	myReader->generateSVG(outputFile);
	cout<<"SVG image "<<outputFile<<" generated"<<endl;
	cout<<"Thanks for using world-to-svg tool"<<endl;

	return 0;
}
