#ifndef CITYFILE_LOADER_H
#define CITYFILE_LOADER_H
#include <map>
#include <Magick++.h>

#include <vector>
#include <deque>
#include <algorithm>
#include <assert.h>
#include <string>
#include <math.h>
#include <time.h>
#include <iostream>
using namespace std;
using namespace Magick;

#define MAXBUFF_SIZE 50000
#define GLOBAL_MAXBUFF_SIZE 200000
//#define MAXBUFF_SIZE 60000
//#define REALLYMAXBUFF_SIZE 200000
#define EPSILON 0.000001

//structure to store the edge priority for the ordering of simplification
struct S_PriorityEdge
{
	int edge_id;
	double priority;
};
struct S_Parcel //structure of parcel for simplification
{
	int parcel_id; //index of the parcel
	int owner;     //index of the building turtle
	double area;
	int type;
	vector<int> edges;   //list of the the boundary edges, 
						//store the index of the id in the building_turtles[i].patchCoords[j].xcor;global edge list
};

struct S_Vertex	//structure of global vertex list for simplification
{
	int vertex_id;
	double xcoor;
	double ycoor;
	vector<int> edges; //edges which has this vertex as one of the end point.
					   //idx of the global edge list
};

struct S_Edge	//structure of global edge list for simplification
{
	int edge_id;
	int start; //id in the global vertice 
	int end;
	vector<int> owner_parcel; //should be has two at most owners, id from global parcel list
};

struct Color_Tuple
{	
	int R;
	int G;
	int B;
};
struct Square
{
	int size;
	int xcor;  //topleft corner x coor
	int ycor;  //topleft corner y coor
};
struct PatchCoor
{
	int xcor;
	int ycor;
};

struct DoublePatchCoor
{
	double xcor;
	double ycor;
};
struct Rect
{
	PatchCoor TopLeft;
	PatchCoor BottomRight;
};

struct TurtleData
{
	int id;
	int type; //1=residential, 2=commercial. 3=industrial
	int population;
	int heading;
	double density;
	vector<PatchCoor> patchCoords;
	int maxSquareSize; //here we store the max length for all possible squares within the patches belongs to this turtle;
					   //while considering their relative positions to the road. this number will be from 1 to 5
	PatchCoor TopLeftCoor_MaxSquare;
	vector<Square> SplitSquares;
};

struct candidateExtender
	{
		vector< deque<PatchCoor> >::iterator iterSeq;
		deque<PatchCoor>::iterator iterSpliter;
		int direction;  //0 for to extend until sequence finish,
						//1 to extend to the beginning of the sequence
		double weight;  //priority to be chosen
	};
//this structure store the index of our intereting descriptions in the data chunk.
struct GlobalIndex
{
	int screen_edge_x_idx;
	int screen_edge_y_idx;
	int ground_level_idx;
	int water_level_idx;
	
	int average_residential_density_idx;
	int max_residential_density_idx;
	int residential_population_idx;
	
	int average_commercial_density_idx;
	int max_commercial_density_idx;
	int commercial_population_idx;
	
	int average_industrial_density_idx;
	int max_industrial_density_idx;
	int industrial_population_idx;
	
	//for svg road path and water boundary
	int pins_tertiaries_idx;
	int pins_primaries_idx;
	int pins_water_boundaries_idx;
};

struct RoadQueueItem
{
	int index;
	int xcoor;
	int ycoor;
	int direction;
};

struct TurtleIndex
{
	int who_idx;
	int breed_idx;
	int heading_idx;
	int building_zone_idx;
	int parcel_idx;
	int population_idx;
	int population_density_idx;
	int developer_name_idx;
	//for road extender
	int road_segment_idx;
	
};
struct PatchIndex
{
	int pxcor_idx;	
	int pycor_idx;	
	int road_idx;	
	int roadtype_idx;
	int connectivity_idx;
	int is_corner_idx;
	int terrain_idx;
	int elevation_idx;
	int owner_idx;
};
struct PatchData
{
	int xcor;
	int ycor;
	bool isRoad;
	int terrain; //0=land, 1=water
	int elevation;
	int zone;    //0=empty, 1=residential, 2=commercial. 3=industrial
	int owner;	 //the turtle id we need.
	bool road_hack;
	int road_type;
	int connectivity;
	int is_corner;
	vector<int> co_ownership;
	int co_ownership_config;
};

struct CityData //globale data structure
{
	int xmax; //(screen size in x dimension)
	int ymax; //(screen size in y dimension)
	int water_level;
	int size; //we assume the map is a square, so it is the integer length per side.
	int ground_level;
	double max_elevation;
	double min_elevation;
	
	double average_residential_density;
	double max_residential_density;
	long int residential_population;
	
	double average_commercial_density;
	double max_commercial_density;
	long int commercial_population;
	
	double average_industrial_density;
	double max_industrial_density;
	long int industrial_population; 
	
	
	vector<PatchData> patches;
	vector<TurtleData> building_turtles;  //here we only care about the building turtles
	vector<int>PatchMap;				//in this field, we store the patch map by denoting the 
								//patch ID in the slot, PatchMap[i*xmax+j], 
								//if it is a road, it will be -1, other wise it is 0;
};

class cCSVFileReader  //the main class for Tom's city input
{
public:
	cCSVFileReader();
	bool readCSVFile(string fileName);
	void GenerateImage();
	virtual ~cCSVFileReader();
	void generateSVG(string fileName);	
	void prepareSimplification(); //this function prepare all the data structures to start the simplification with
	void simplifyPolygons(double threshold, string *inputFileNamePre=NULL, bool batchmode=false);

	
private:
	void ScaleTheElevation(int scaleFactor);
	void findSplitSquares();
	int calcRoadCost(int x, int y, int size);
	void findMaxPossibleSquare();
	void preparePatchMap();
	inline void getRidOfQuot(char * outStr, char *inStr, long int buffer_size=MAXBUFF_SIZE);
	void extractPatchInfo(vector<PatchCoor> &patches, const char *inStr,int TurtleID);
	int  extractRoadSegmentInfo(deque<PatchCoor> &patches, const char *inStr, bool waterFront=false, bool primaryRoad=false);
	char * getDataAtPos(char * buf, int pos, long int buffer_size=MAXBUFF_SIZE); //the input is already a comma separated data line
											 //what to do here is return the string stores there at 
											 //the given position.
	void getStrBeforeBra(char * outStr, const char *inStr, long int buffer_size=MAXBUFF_SIZE); //substract the str right before the bracket }
															  //used to get info from "{turtle 181}", it will return 	
																//"181", "{breed building}", it will return "building"
	void parseGlobalDescription(char * input);
	void parseTurtleDescription(char * input);
	void parsePatchDescription(char * input);

	void processSequenceSegements(const char * dataStr, vector< deque<PatchCoor> > & output, long int buffer_size, bool waterFront=false, bool primaryRoad=false);

	void parcelDensityRemapper();
	inline int getPatchIndex(int xCoor, int yCoor);
	
	//for svg image generating
	void depthFirstTravelForSecondaryRoad(FILE *svgFile, deque<RoadQueueItem> & RoadQueue);
	void generateRoadCurveInSVG(vector<DoublePatchCoor> & vPatches, char * outStr);
	void movingAverageRoadPatches(deque<PatchCoor> & inputPatches, vector<DoublePatchCoor> & outputPatches, int window_length, bool isPrimary=false);
	void movingAverageRoadPatches2(deque<PatchCoor> & inputPatches, vector<DoublePatchCoor> & outputPatches, int window_length);
	void generateParcelInSVG();
	void generateRoadPathInSVG();
	void generateBuildingSlotInSVG();
	void generateColorStrFromParcel(int idx, char * colorStr);
	
	int generateParcelBoundary(vector<int> & edgeList, vector<double> &boundary);
	
	//for simplification
		int addEdge(double x1,double y1, double x2, double y2, int ownerParcel);
	int sortEdgesInParcel(vector<int> & InputEdges);
		double calcNewAreaIfEdgeContracted(int parcelID, int edgeID);
	double calcMaximEdgeCost(int edgeID);
	void removeRepeat(vector<int> & inputList);
	void performEdgeContraction(int edgeID, vector<int> & affectedEdges);
	bool checkVertexInEdgeList(double xcoor, double ycoor,int edgeID_TOBE_contracted, vector<int> &edges);

	//if we are dragging one of the start_v and end_v vertex to a new position, 
	//calculate the new area
	double calcNewAreaForDraggingVertex(int parcelID, int start_v, int end_v, double new_x, double new_y);
	inline double calcTriangleArea(double x0, double y0, double x1, double y1, double x2, double y2);
	bool PointInParcel(double px, double py, int parcelID);
	void replaceEdge(int dest, int src);
	
	
	//modualize the readin section
 	inline void readInCSVDataItem(const char *input, char *output, int idx, long int str_size=MAXBUFF_SIZE);
	int readGlobalSection(FILE * dataFile);
	int readTurtleSection(FILE * dataFile);
	int readPatchSection(FILE * dataFile);
	
	//fix the empty parcel under road
	void fixEmptyParcelUnderRoad();
	void fixOverlapParcels();
	void fixHolesInParcels();
	void fixRoadIntersection(vector< deque<PatchCoor> > & roads, int range);
	void fixRoadIntersection2(vector< deque<PatchCoor> > & roads, int range);
	void mergeRoadSequence(vector< deque<PatchCoor> > & oldRoads, vector< deque<PatchCoor> > & newRoads, bool isPrimary=false, bool isNormalOrder=true);
	
	void fixRoadAfterMerged(vector< deque<PatchCoor> > &roads, bool isNormalOrder=true);
	bool fixRoadIntersectionHelper(vector< deque<PatchCoor> > & roads, int idx,  int direction);
	
	void offsetPrimaryRoads(vector< deque<PatchCoor> > & roads, double offset);
	
	//building our own topology
	void buildSecondaryRoadTopology(vector< deque<PatchCoor> > &roads);
	void fixFourConnectors(vector< deque<PatchCoor> > &roads);
	void formRoadSegmentsFromDFSTraversal(vector<PatchCoor> &traverseList,vector< deque<PatchCoor> > &roads);
	void DFSRoadBuilder(PatchCoor patch_coor, 
                        vector<bool> & isVisited, 
                        vector<PatchCoor> &traverseList,
                        int xoffset, int yoffset);
	inline bool canGoFurther_DFS(PatchCoor &patch_coor, PatchCoor &patch_coor_togo,vector<bool> &isVisited, bool newEntrance);
	void fixSecondaryPrimaryConnection(vector< deque<PatchCoor> > &secondRoads,
									   const vector< deque<PatchCoor> > &primaryRoads);
	bool checkCanExtendSecondRoad(PatchCoor patch_coor);
	
	
	//for dealing with co-ownership: patches under road
	void prepareSimplify_co_ownership(S_Parcel & parcel_item, int patch_index, int parcel_index);
	void addEdgeToParcelItem(S_Parcel & parcel_item, int edge_idx, int ownerParcel);
	void deleteDuplicateEdge(S_Parcel & parcel_item, int parcel_index);
	void deleteDuplicateEdgePre(S_Parcel & parcel_item, int parcel_index);
	
	void massageVertexCoor();
	
	void fixNotFourConnectedParcel();
public:

	
	CityData *data;
	//Image * MagickImage;
private:
	map<int, Color_Tuple> colorMapping;
	map<int, int>turtleMap;
	GlobalIndex globalHeader;
	TurtleIndex turtleHeader;   //those index strutures store the index of each important data
	PatchIndex patchHeader;
	vector< deque<PatchCoor> > roadSegments_primary;
	vector< deque<PatchCoor> > roadSegments_secondary;
	vector< deque<PatchCoor> > roadSegments_secondary_new;
	vector< deque<PatchCoor> > waterFrontSegments;
	FILE *svgFile;

	vector<S_Parcel> parcel_list;
	vector<S_Edge> edge_list;
	vector<S_Vertex> vertex_list;
	double * parcel_area_orig;
	char *waterFrontStr;
	char *primaryRoadStr;
	char *secondaryRoadStr;
	vector<PatchCoor> roadIntersector;
};

#endif
