breeds [ mice cheese ]

;; A flag that tells us when we are done.
globals [ done? map-data ]

to startup
  load-map
  if "yes" = ( user-choice "Would you like to set the directory where the data files are stored?" [ "no" "yes" ] )
    [ choose-directory ]
end

;; Setup procedure
to setup
  cp ct
  set done? false
  add-cheese
  add-mice
  display-map
end

to display-map
  foreach map-data [ set pcolor-of patch first ? item 1 ? last ? ]
end

to close-all-files
  locals [ countup ]
  set countup 1

  repeat 30
  [
    file-open "mouse_" + countup + "_replay"
    file-close
    set countup countup + 1
  ]
end

to delete-all-files
  locals [ countup ]
  set countup 1

  repeat 30
  [
    if ( file-exists? ("mouse_" + countup + "_replay") )
    [
      file-open "mouse_" + countup + "_replay"
      file-close
      file-delete "mouse_" + countup + "_replay"
    ]
    set countup countup + 1
  ]
end

;; Although this function is never used in the model. It was left
;; to show you how the "File IO Patch Data.txt" file was created.
to save-map
  file-write "File IO Patch Data.txt"
  ask patches
  [
    file-write pxcor
    file-write pycor
    file-write pcolor
  ]
  file-close
end

;; This function loads a pre-existing map of a maze. The format
;; of the file is: pxcor pycor pcolor. You can view the file by
;; opening the file "File IO Patch Data.txt"

;; using a simple text editor.
to load-map

  ;; Check to see if you changed to a directory that doesn't have the file
  ifelse ( file-exists? "File IO Patch Data.txt" )
  [
    set map-data []

    file-open "File IO Patch Data.txt"

    ;; Read in all the data in the file
    while [ not file-at-end? ]
    [
      ;; file-read gives you variables.  In this case numbers.
      ;; We store them in a double list (ex [[1 1 9.9999] [1 2 9.9999] ...
      set map-data sentence map-data (list (list file-read file-read file-read))
    ]

    ;; Done reading in map information.  Close the file.
    file-close
  ]
  [ user-message "There is no File IO Patch Data.txt file in current directory!" ]
end

;; Add the mice to the corner of the screen.
to add-mice
  create-custom-mice 30
  [
    setxy -16 -16
    set heading 0
    set color gray
  ]
end

;; Add the cheese at the end of the maze.
to add-cheese
  create-custom-cheese 1
  [
    setxy 13 -14
    set color yellow
    set shape "box"
  ]
end

;; This is the main function that move the mice.
to go-mice

  every .1
  [
    ;; Highlighting lets you focus on a mouse
    check-highlighting

    ask mice
    [
      ;; This is the mice's simple AI for determining direction.
      determine-heading

      ;; This tracks the mice's movements in the data files.
      track

      ;; Time to actually move the mouse now.
      fd 1

      ;; Check to see if the mouse found the cheese.
      check-cheese
    ]
  ]

  ;; Stop the forever button?
  if ( done? )
    [ stop ]
end

;; You can either highlight a mouse to be red
;; or hide all the other mice.
to check-highlighting
  ifelse ( highlighting-on? )
  [
    ask mice
    [
      ;; Highlight a mouse
      ifelse ( who = highlight-mouse )
      [
        set color red
        set hidden? false
      ]
      [
        set color gray
        set hidden? not show-all-mice?
      ]
    ]
  ]
  [
    ;; If highlighting is off we want to make sure
    ;; we can see them all and they're the correct color.
    ask mice
    [
      set hidden? false
      set color gray
    ]
  ]
end

;; Did someone get the cheese?
to check-cheese ;; Turtle procedure
  if ( any? cheese-here )
  [
    user-message "Mouse " + who + " got the cheese!"
    set done? true
  ]
end

;; This simple function lets the mice determine where they are going.
;; It checks left and right to see if it can turn and it does so randomly
;; or if it runs into a wall.  Otherwise, it just turns around.  Works
;; for 90 degree mazes.
to determine-heading  ;; Turtle Procedure

  ;; Should we go left?
  if ( check-left and
     (  pcolor-of patch-ahead 1 = white or random 3 = 0 ) )
  [ lt 90 ]

  ;; Should we go right?
  if ( check-right and
     (  pcolor-of patch-ahead 1 = white or random 3 = 0 ) )
  [ rt 90 ]

  ;; Should we turn around?
  if ( pcolor-of patch-ahead 1 = white )
  [ lt 180 ]

  ;; Sniff the cheese?
  if ( any? cheese in-radius 3 )
    [ set heading towards one-of cheese ]
end

;; Reports true if there is no wall to the left of the mouse.
to-report check-left
  locals [ ret ]
  ;; We use without-interruption because we don't
  ;; want to see the mouse actually turn.
  without-interruption
  [
    lt 90
    set ret ( pcolor-of patch-ahead 1 = black ) and
            ( pcolor-of patch-ahead 2 = black )
    rt 90
  ]
  report ret
end

;; Reports true if there is no wall to the right of the mouse.
to-report check-right
  locals [ ret ]
  ;; We use without-interruption because we don't
  ;; want to see the mouse actually turn.
  without-interruption
  [
    rt 90
    set ret ( pcolor-of patch-ahead 1 = black ) and
            ( pcolor-of patch-ahead 2 = black )
    lt 90
  ]
  report ret
end

;; Does the user want to choose a new default directory?
to choose-directory
  locals [ path ]

  ;; This pops up a dialog that lets the user choose a directory
  set path user-choose-directory

  ;; Check to see if they canceled
  if ( path != false )
  [
    ;; This changes the directory
    set-current-directory path
  ]
end

to track
  file-open "mouse_" + who + "_replay"
  file-write heading
end

;; This function will replay a single mouse's file
to replay-mouse
  setup
  close-all-files
  ;; We only want one mouse
  ask mice with [ who != mouse-to-replay ]
    [ die ]
  ;; Call the simulation function
  simulate false
end

;; This function will replay all the mice's files
to replay-all
  setup
  close-all-files
  simulate true
end

;; This function will simulate the mice's movements by using
;; data from files.  The all? parameter is a boolean that tells
;; the function if we should only simulate one turtle or all of them.
to simulate [ all? ]

  while [ not done? ]
  [
    every .1
    [
      ;; We don't need highlighting if there is only one mouse.
      if ( all? )
        [ check-highlighting ]

      ask mice
      [
        ifelse ( file-exists? ("mouse_" + who + "_replay") )
        [
          ;; Select the mouse's file for reading.
          file-open "mouse_" + who + "_replay"

          ifelse ( file-at-end? )
          [ set done? true ]
          [
            ;; Read in its next heading
            set heading file-read

            ;; Move!
            fd 1
          ]
        ]
        [ set done? true ]
      ]
    ]
  ]
end

;; Halts the replay instantly
to halt-replay
  set done? true
end
@#$#@#$#@
GRAPHICS-WINDOW
321
10
646
356
17
17
9.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
488
655
583
Command Center
0

BUTTON
74
41
157
74
setup
setup\ndelete-all-files
NIL
1
T
OBSERVER
T
NIL

BUTTON
159
41
243
74
NIL
go-mice
T
1
T
OBSERVER
T
NIL

SLIDER
18
338
185
371
mouse-to-replay
mouse-to-replay
1
30
20
1
1
NIL

BUTTON
190
338
297
371
NIL
replay-mouse
NIL
1
T
OBSERVER
T
NIL

BUTTON
92
111
219
144
NIL
choose-directory
NIL
1
T
OBSERVER
T
NIL

SWITCH
158
202
299
235
show-all-mice?
show-all-mice?
0
1
-1000

BUTTON
190
375
297
408
NIL
halt-replay
NIL
1
T
OBSERVER
T
NIL

BUTTON
18
375
185
408
NIL
replay-all
NIL
1
T
OBSERVER
T
NIL

SLIDER
18
238
296
271
highlight-mouse
highlight-mouse
1
30
20
1
1
NIL

SWITCH
13
202
157
235
highlighting-on?
highlighting-on?
0
1
-1000

TEXTBOX
94
90
253
108
Location of the Data Files

TEXTBOX
56
16
303
34
Warning: This model will create 30 data files

TEXTBOX
107
313
247
331
Replay Last Model Run

TEXTBOX
118
181
240
199
Highlight a Mouse

BUTTON
81
441
230
474
Remove all Data Files
delete-all-files
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This model is a simple simulation of thirty mice running through a maze in a random fashion.  The purpose of this model is to show how to use files to save and replay model executions.  It also shows how to load in complicated patch information such as a maze.  The map information was created beforehand, but the function used to save the patch information was left in the code.  It also lets the user choose which directory the data is saved in. There are also some tools to highlight and focus on certain mice during the run.  This lets you confirm that single mouse replays are being run correctly.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0pre5
@#$#@#$#@
setup
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
