;; A completely random setup of turtles
to setup
  ca
  cct random 40 + 10
  [
    setxy random-float screen-size-x
          random-float screen-size-y
  ]
end

;; This mixes up the turtles more by
;; having them go in random directions
to go
  every .1
  [
    ask turtles
    [
      lt random-float 360
      fd random-float 5
    ]
  ]
end

;; This will load all the turtles from a file
;; that was previously saved from save-all
to load-all
  locals [ file ]

  ;; This will bring up a file dialog for the user
  ;; We don't use user-choose-new-file because we want
  ;; an exisiting file to be chosen.
  set file user-choose-file

  ;; We want to make sure the user didn't press cancel
  if ( file != false )
  [
    ca

    ;; Open the file
    file-open file

    ;; Using file-read to load in turtle information.
    ;; The first number is the turtle count. The rest is
    ;; turtle information for each of the turtles.
    ;; We saved in the format: xcor, ycor, heading, color
    ;; Remember read returns actual variables and not just
    ;; strings.
    cct file-read
    [
      setxy file-read file-read
      set heading file-read
      set color file-read
    ]

    ;; We are done.  Close the file.
    file-close
  ]
end

;; This function will save all the turtles information in a file.
;; It can be later loaded back using load-all.
to save-all
  locals [ file ]

  ;; This brings up a dialog for the user to choose a new file.
  ;; We use user-choose-new-file because the user can choose
  ;; a non-existing file.
  set file user-choose-new-file

  ;; Check to make sure the user didn't hit cancel
  if ( file != false )
  [
    if ( file-exists? file )
    [
      file-delete file
    ]

    ;; Open file for "writing" information out to
    file-open file

    ;; We use file-write because it lets us easily use file-read
    ;; to get the information back as variables and not just strings
    file-write (count turtles)

    ;; Write all the turtles info.  It doesn't really matter which
    ;; order the turtles do it in.
    ask turtles
    [
      file-write xcor
      file-write ycor
      file-write heading
      file-write color
    ]

    ;; Close file.  We are done writing.
    file-close
  ]
end

;; This lets us keep track of who used the model.
;; It uses the file user_list.txt to store the information.
;; Note it stores it in the system's default directory.  This
;; could be changed by using the set-current-directory
;; command.
to list-used-by
  locals [ used-by ]

  ;; We need to check if a file exists just in case someone
  ;; presses the button before anyone enters their name.
  ifelse ( file-exists? "user_list.txt" )
  [
    file-open "user_list.txt"

    ;; We are going to generate a long string here.
    ;; \n is an end-of-line character
    set used-by "People who used this model\n"

    ;; This loops over the file until we read in everything
    ;; from the file into the string used-by.
    ;; Note the use of endoffile.  We don't really know how
    ;; long the file is going to be, but it doesn't matter
    ;; because the file is read in sequentially.
    while [ not file-at-end? ]
    [
      set used-by used-by + file-read-line + "\n"
    ]

    ;; Show the user
    user-message used-by

    ;; Okay we are finished.  Close the file.
    file-close
  ]
  [
    ;; Notify the user
    user-message "There are no previous users"
  ]
end

;; This function will let the user enter his or her name to a
;; list of names in a file that can be later retrieved using
;; the function list-used-by.  Note that the file name is hardcoded so
;; the user has no choice where the information is stored.  This is
;; different than save-all which lets the user choose the file.
to enter-used-by
  locals [ name ]

  ;; We do this check so someone doesn't enter his or her name
  ;; in twice by accident
  ifelse "no" = ( user-choice "Have you entered your name already?" [ "no" "yes" ] )
  [
    ;; We like to add commas between the names, but we don't
    ;; want a comma before the first name, so we assume it is
    ;; the first name if the file doesn't already exist.
    ifelse ( file-exists? "user_list.txt" )
    [
      set name ", "
    ]
    [
      set name ""
    ]

    ;; Prepend the comma
    set name name + user-input "Your Name"

    file-open "user_list.txt"

    ;; Add it to the file.  Notice we use file-type so there is no
    ;; end-of-line character.
    file-type name

    ;; Okay done adding the name.  Close the file.
    file-close
  ]
  [
    user-message "Well, you don't need to enter it more than once."
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
305
10
630
356
17
17
9.0
1
10
1
1
1
0

CC-WINDOW
2
544
676
640
Command Center

BUTTON
41
43
136
77
New
setup
NIL
1
T
OBSERVER
T

BUTTON
143
43
244
76
Randomize
go
T
1
T
OBSERVER
T

BUTTON
25
138
107
171
Save
save-all
NIL
1
T
OBSERVER
T

BUTTON
180
138
260
171
Load
load-all
NIL
1
T
OBSERVER
T

BUTTON
13
310
264
343
People who have used this model
list-used-by
NIL
1
T
OBSERVER
T

BUTTON
36
269
241
302
Enter your name
enter-used-by
NIL
1
T
OBSERVER
T

TEXTBOX
12
112
148
130
Save All Turtles in a File

TEXTBOX
165
112
296
130
Load Turtles from a File

TEXTBOX
110
205
200
223
Model Users

TEXTBOX
62
228
271
265
Warning: You need to have writing permission in the model's folder

@#$#@#$#@
WHAT IS IT?
-----------
This model shows how to use basic file input and output primitives.  The first thing you can do is save and load all the turtles on the screen.  The SETUP and RANDOMIZE buttons will create a random turtle set and scatter them around the screen, while SAVE and LOAD will let you save the turtles into a user-chosen file or load a previously saved file.  This portion of the code demonstrates the use of the primitives file-open, file-read, file-write, file-close, user-choose-file, and user-choose-new-file.

The second thing you can do is enter your name to a list of people who have used the model. This can be done using the button ENTER YOUR NAME.  Clicking on PEOPLE WHO HAVE USED THIS MODEL will bring up a message that lists all the people who have entered their name.  It stores and retrieves all the information from the file 'user_list.txt' in the model's directory.  Note that you need to have write permission in the model's directory for this to work.  You can change the directory using the set-current-directory command.  This porition of the code demonstrates the use of the primitives file-exists?, file-at-end?, file-read-line, and file-type.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1pre2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
