globals
[
  x-cor-list ;list of x-cor in mouse's path (from oldest to newest)
  y-cor-list ;list of y-cor in mouse's path
  time-list  ;list of times when corresponding x-cor and y-cor happen in mouse's path
  index      ;place in list
  int-timer  ;intermediate timer, to keep separate mouse paths plotted consecutively
             ;on the plot, regardless of time passed in between clicks
  dif        ;difference in time between end of one mouse click, and beginning of next mouse click
  plot-index ;minimum y value of plot shown, for use in scrolling
  draw-on    ;this is true if the draw procedure should continue running
  replay-on  ;this is true if the replay procedure is in progress
  clear-on   ;this is true if the setup procedure is in progress
  setup-ever-on ;this is true if the setup procedure has ever been run
  mag-vel-x-list ;list of x magnetude velocities of mouse's path
  mag-vel-y-list ;list of y magnetude velocities of mouse's path
  mark-list  ;list of index when mark has been asked
  mark-dif   ;allows size of mark on plot to vary based on y-value of plot\
  mark-it    ;during replay, lets it know when you should mark, based on in mark button was pushed now,
             ;or if mark button was pushed earlier
  sem
  stop-it
  replay-index
  current-mode ;equals "replay" or "draw" or "nothing"
  last-replay-index
  just-marked
  plabel-index
]

;initializes variables
to setup

  reset-timer
  ca

  ;create empty lists, and initialize size of list
  set x-cor-list []
  set y-cor-list []
  set time-list []
  set mag-vel-x-list []
  set mag-vel-y-list []
  set mark-list []

  ;initalize other variables
  set index -1
  set int-timer 0
  set plot-index 0
  set draw-on 0
  set replay-on 0
  set mark-it 0
  set stop-it 0
  set sem 0
  set replay-index 0
  set current-mode "nothing"
  set just-marked 0
  set plabel-index 0

  ;setup has now run one time
  ;setup executes, once, the first time any of the buttons have been pushed
  set setup-ever-on 1

end

;follows the mouse's path, and collects path in a list
to get-mouse-path

  if ((draw-on = 1) and (replay-on = 0))
  [
    if (current-mode = "replay")
    [
      set int-timer (int-timer + item(last-replay-index) time-list - item(0) time-list + 0.5)
    ]
    set current-mode "draw"

    ;if any of the buttons have never been pushed
    if (setup-ever-on = 0)
    [
      setup
    ]

    ;while the replay button and clear button are not pressed in
    if ((replay-on = 0) and (clear-on = 0))
    [

      ifelse (mouse-down?)
      ;if the mouse is held down, plot.
      [
        ;if this is the beginning of the list, then clear the list
        ;old list may remain from previous mouse click
        ;also, clear old plot
        if (index = -1)
        [
          set x-cor-list []
          set y-cor-list []
          set time-list []
          set mark-list []
          clear-patches
        ]

        ;add items to list
        ifelse (index < 500)
        [
          ;add items to list when list contains less than 500 entries
          set x-cor-list lput mouse-xcor x-cor-list
          set y-cor-list lput mouse-ycor y-cor-list
          set time-list lput timer time-list
          set index index + 1
        ]
        [
          ;list contains less 500 entries
          ;make list contain last 499 entries
          ;add new item to end of list
          ;don't increase index
          set x-cor-list butfirst x-cor-list
          set y-cor-list butfirst y-cor-list
          set time-list butfirst time-list
          set x-cor-list lput mouse-xcor x-cor-list
          set y-cor-list lput mouse-ycor y-cor-list
          set time-list lput timer time-list
        ]
        draw-path-compute-and-plot
      ]
      [
        ;initializes index of list
        set index -1
      ]
    ]

    if (clear-on = 1)
    ;the clear button has been pushed, so stop the draw button
    [
      set draw-on 0
      setup
      set clear-on 0
      stop
    ]
  ]
  if (stop-it = 2)
  [
    set draw-on 0
    set stop-it 0
    stop
  ]

end


to fade-patches
;takes any unblack patch, and fades it slightly, towards black

  locals [ trace-patches ]
  set trace-patches patches with [ (pcolor != black) and (pcolor != 9) ]
  if any? trace-patches
    [ ask trace-patches
      [ set pcolor ( pcolor - 0.3 )
        if (((round pcolor) mod 10) = 0)
         [ set pcolor black ]
      ]
    ]
end

;replay the mouse motion, and resulting graph, of
;last extended mouse click
to replay

;show "mark-it" + mark-it
  if ((replay-on = 1) and (draw-on = 0))
  [

    set current-mode "replay"

    ;if any of the buttons have never been pushed
    if (setup-ever-on = 0)
    [
      setup
    ]

    set draw-on 0
    get-mouse-path

    ;mark-dif tells how big the mark on the plot should be, based on the plot's y-axis
    set mark-dif ((plot-y-max - plot-y-min) * (1 / 35))

    ;while each element of the list has not been drawn and plotted
    ;and the clear button is not pressed
    ifelse ((replay-index < (length x-cor-list)) and (draw-on = 0) and (clear-on = 0))
    [

      ;pause if speed-of-replay is 0
      ;so you can slow down, and stop motion using slider
      if (Speed-of-Replay > 0)
      [
        ;every increment of time, dependant on Speed-of-Replay
        every (((-0.00495) * Speed-of-Replay) + 0.505)
        [
          set index (index + 1)

;show "mark-it " + mark-it

          draw-path-compute-and-plot
          set replay-index (replay-index + 1)
          set last-replay-index replay-index
        ]
      ]
    ]
    [
      set replay-index 0
      set index -1
      set plabel-index 0
      clear-patches
    ]

    if (clear-on = 1)
    [
      ;the clear button has been pushed, so stop the replay button
      setup
      set clear-on 0
      stop
    ]

    set replay-on 0
  ]
  if (stop-it = 1)
  [
    set replay-index 0
    set index -1
    set replay-on 0
    set stop-it 0
    stop
  ]

end


to draw-path-compute-and-plot

  locals
  [
    change-in-xcor
    change-in-ycor
    change-in-timer
    x-avg
    y-avg
    mag-index
  ]

;show "mark-it (draw1)" + mark-it

  ifelse(member? (index) mark-list = true)
  [ set mark-it 1 ]
  [ ifelse (just-marked = 1)
    [ set just-marked 0]
    [ set mark-it 0]
   ; set mark-it 0
  ]

;show "mark-it (draw2)" + mark-it

  ;turn each patch the mouse is determined to be, red
  ask patch-at (item (index) x-cor-list) (item (index) y-cor-list)
  [
    ifelse ((mark-it = 0) and (pcolor != 9))
    [
      set pcolor red
    ]
    [
      ;set pcolor 9
      ;ask patch-at ((item (index) x-cor-list)) ((item (index) y-cor-list))
      ;[
        set plabel-index plabel-index + 1
        set plabel plabel-index
      ;]
    ]
    fade-patches
  ]


  ;let it graph one-click-mouse-path, followed by another, no matter length in wait time
  ;so adjust the time-list entry, to follow on graph
  if (index = 0)
  [
    set dif ((item(0) time-list) - int-timer)
  ]
  if (index = ((length x-cor-list) - 1))
  [
    set int-timer (((item(index) time-list) - dif) + 0.5)
  ]

  ;make graph scroll, in the y direction
  if ((item(index) time-list - dif) > (plot-index + 9))
  [
    set plot-index (plot-index + 1)
  ]
  set-plot-x-range plot-index (plot-index + 10)

  ;if this is the beginning of the list, then clear out the
  ;old magnetude velocity lists
  ;they are initialized with a 0 as the first element, to be
  ;used as a placeholder, since
  ;mag-vel-x-list[index] uses x-cor-list[index] and x-cor-list[index-1]
  if (index = 0)
  [
    set mag-vel-x-list [0]
    set mag-vel-y-list [0]
  ]

;show "mark-it (draw3)" + mark-it

without-interruption [
  ;mark-dif tells how big the mark on the plot should be, based on the plot's y-axis
  set mark-dif ((plot-y-max - plot-y-min) * (1 / 35))

  if (Pink.)
  [
    ;plot x coordinate versus time it was there
    set-current-plot-pen "Pink"
    ifelse (index < 1) [set-plot-pen-color 9.99] [set-plot-pen-color 137]
    plotxy ((item(index) time-list) - dif) (item(index) x-cor-list)

    ;plot a mark, if mark was requested during some replay of this part of the path
    set-current-plot-pen "Mark-Pink"
    if (not(empty? mark-list))
    [
      if (mark-it = 1)
      [
        set-plot-pen-color 9.99
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) x-cor-list - mark-dif)
        set-plot-pen-color 0
        plotxy ((item(index) time-list) - dif + 0.1) (item(index) x-cor-list - mark-dif)
        plotxy ((item(index) time-list) - dif + 0.1) (item(index) x-cor-list + mark-dif)
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) x-cor-list + mark-dif)
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) x-cor-list - mark-dif)
      ]
    ]
  ]

  if (Blue.)
  [
    ;plot y coordinate versus time it was there
    set-current-plot-pen "Blue"
    ifelse (index < 1) [set-plot-pen-color 9.99] [set-plot-pen-color 85]
    plotxy ((item(index) time-list) - dif) (item(index) y-cor-list)

    ;plot a mark, if mark was requested during some replay of this part of the path
    set-current-plot-pen "Mark-Blue"
    if (not(empty? mark-list))
    [
      if (mark-it = 1)
      [
        set-plot-pen-color 9.99
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) y-cor-list - mark-dif)
        set-plot-pen-color 0
        plotxy ((item(index) time-list) - dif + 0.1) (item(index) y-cor-list - mark-dif)
        plotxy ((item(index) time-list) - dif + 0.1) (item(index) y-cor-list + mark-dif)
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) y-cor-list + mark-dif)
        plotxy ((item(index) time-list) - dif - 0.1) (item(index) y-cor-list - mark-dif)
      ]
    ]

  ]

  if ((index < (length time-list)) and (index > 0))
  [
    set change-in-timer ((item (index) time-list) - (item (index - 1) time-list))
    if (Purple.)
    [

      ;add items to list
      ifelse (index < 6)
      [
        ;find x magnetude velocity, store at end of list, when less than 6 entries
        set change-in-xcor ((item (index) x-cor-list) - (item (index - 1) x-cor-list))
        set mag-vel-x-list lput (change-in-xcor / change-in-timer) mag-vel-x-list

      ]
      [
        ;list contains less 6 entries
        ;make list contain last 5 entries
        ;add new item to end of list

        set change-in-xcor ((item (index) x-cor-list) - (item (index - 1) x-cor-list))
        set mag-vel-x-list butfirst mag-vel-x-list
        set mag-vel-x-list lput (change-in-xcor / change-in-timer) mag-vel-x-list
      ]
      ;create average of entire mag-vel-x-list
      if (index = 1)
      [
        set x-avg
          (item (1) mag-vel-x-list)
      ]
      if (index = 2)
      [
        set x-avg
          (((item (1) mag-vel-x-list) +
          (item (2) mag-vel-x-list)) / 2)
      ]
      if (index = 3)
      [
        set x-avg
          (((item (1) mag-vel-x-list) +
          (item (2) mag-vel-x-list) +
          (item (3) mag-vel-x-list)) / 3)
      ]
      if (index = 4)
      [
        set x-avg
          (((item (1) mag-vel-x-list) +
          (item (2) mag-vel-x-list) +
          (item (3) mag-vel-x-list) +
          (item (4) mag-vel-x-list)) / 4)
      ]
      if (index > 4)
      [
        set x-avg
          (((item (1) mag-vel-x-list) +
          (item (2) mag-vel-x-list) +
          (item (3) mag-vel-x-list) +
          (item (4) mag-vel-x-list) +
          (item (5) mag-vel-x-list)) / 5)
      ]

      ;plot average x magnetude velocity
      set-current-plot-pen "Purple"
      ifelse (index < 2) [set-plot-pen-color 9.99] [set-plot-pen-color 117]
      plotxy (item(index)  time-list - dif) x-avg

      ;plot a mark, if mark was requested during some replay of this part of the path
      set-current-plot-pen "Mark-Purple"
      if (not(empty? mark-list))
      [
        if (mark-it = 1)
        [
          set-plot-pen-color 9.99
          plotxy ((item(index) time-list) - dif - 0.1) (x-avg - mark-dif)
          set-plot-pen-color 0
          plotxy ((item(index) time-list) - dif + 0.1) (x-avg - mark-dif)
          plotxy ((item(index) time-list) - dif + 0.1) (x-avg + mark-dif)
          plotxy ((item(index) time-list) - dif - 0.1) (x-avg + mark-dif)
          plotxy ((item(index) time-list) - dif - 0.1) (x-avg - mark-dif)
        ]
      ]
    ]

    if (Green.)
    [

      ;add items to list
      ifelse (index < 6)
      [
        ;find x magnetude velocity, store at end of list, when less than 6 entries
        set change-in-ycor ((item (index) y-cor-list) - (item (index - 1) y-cor-list))
        set mag-vel-y-list lput (change-in-ycor / change-in-timer) mag-vel-y-list

      ]
      [
        ;list contains less 6 entries
        ;make list contain last 5 entries
        ;add new item to end of list

        set change-in-ycor ((item (index) y-cor-list) - (item (index - 1) y-cor-list))
        set mag-vel-y-list butfirst mag-vel-y-list
        set mag-vel-y-list lput (change-in-ycor / change-in-timer) mag-vel-y-list
      ]

      ;create average of entire mag-vel-x-list
      if (index = 1)
      [
        set y-avg
          (item (1) mag-vel-y-list)
      ]
      if (index = 2)
      [
        set y-avg
          (((item (1) mag-vel-y-list) +
          (item (2) mag-vel-y-list)) / 2)
      ]
      if (index = 3)
      [
        set y-avg
          (((item (1) mag-vel-y-list) +
          (item (2) mag-vel-y-list) +
          (item (3) mag-vel-y-list)) / 3)
      ]
      if (index = 4)
      [
        set y-avg
          (((item (1) mag-vel-y-list) +
          (item (2) mag-vel-y-list) +
          (item (3) mag-vel-y-list) +
          (item (4) mag-vel-y-list)) / 4)
      ]
      if (index > 4)
      [
        set y-avg
          (((item (1) mag-vel-y-list) +
          (item (2) mag-vel-y-list) +
          (item (3) mag-vel-y-list) +
          (item (4) mag-vel-y-list) +
          (item (5) mag-vel-y-list)) / 5)
      ]

      ;plot average x magnetude velocity
      set-current-plot-pen "Green"
      ifelse (index < 2) [set-plot-pen-color 9.99] [set-plot-pen-color 65]
      plotxy (item(index)  time-list - dif) y-avg

      ;plot a mark, if mark was requested during some replay of this part of the path
      set-current-plot-pen "Mark-Green"
      if (not(empty? mark-list))
      [
        if (mark-it = 1)
        [
          set-plot-pen-color 9.99
          plotxy ((item(index) time-list) - dif - 0.1) (y-avg - mark-dif)
          set-plot-pen-color 0
          plotxy ((item(index) time-list) - dif + 0.1) (y-avg - mark-dif)
          plotxy ((item(index) time-list) - dif + 0.1) (y-avg + mark-dif)
          plotxy ((item(index) time-list) - dif - 0.1) (y-avg + mark-dif)
          plotxy ((item(index) time-list) - dif - 0.1) (y-avg - mark-dif)
        ]
      ]
    ]
  ]
]

  if(mark-it = 1) [set mark-it 0]

  ;plot a gray, y=0 line when green or purple is displayed
  set-current-plot-pen "Gray"
  ifelse ((index > 1) and ((Green.) or (Purple.)))
    [set-plot-pen-color 5] [set-plot-pen-color 9.99]
  plotxy (item(index) time-list - dif) 0

    if (just-marked = 0) [set mark-it 0]

end

to mark
  if (current-mode = "replay")
  [
    ;set index of place in path, when mark was requested, into a list
    ;sort the list, so if it is replayed, new marks will be included, in order

;show "MARK-IT"
    set mark-list lput index mark-list
    set mark-it 1
    set just-marked 1
;show "mark-it (after mark)" + mark-it
  ]
end

to clear
  set clear-on 1
  if ((replay-on = 0) and (draw-on = 0))
  [
    setup
    set clear-on 0
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
190
15
628
285
47
26
4.51
0
10
1
1
1
0

CC-WINDOW
5
480
637
575
Command Center

BUTTON
105
49
179
83
Draw
ifelse (sem = 0)\n[\n  set sem 1\n  set draw-on 1\n  get-mouse-path \n  set draw-on 0\n  set sem 0\n]\n[ set stop-it 1]
T
1
T
OBSERVER
T
NIL

PLOT
190
295
628
466
Plot
NIL
NIL
0.0
5.0
-10.0
10.0
true
true
PENS
"Pink" 1.0 0 -65413 true
"Blue" 1.0 0 -16776961 true
"Green" 1.0 0 -11352576 true
"Purple" 1.0 0 -8716033 true
"Mark-Pink" 1.0 0 -16777216 false
"Mark-Blue" 1.0 0 -16777216 false
"Mark-Purple" 1.0 0 -16777216 false
"Mark-Green" 1.0 0 -16777216 false
"gray" 1.0 0 -7566196 false

BUTTON
105
94
179
128
Replay
ifelse (sem = 0)\n[\n  set sem 1\n  set replay-on 1\n  replay\n  set replay-on 0\n  set sem 0\n]\n[set stop-it 2]
T
1
T
OBSERVER
T
NIL

SWITCH
89
295
179
328
Pink.
Pink.
0
1
-1000

SWITCH
88
341
178
374
Blue.
Blue.
1
1
-1000

SLIDER
14
139
179
172
Speed-of-Replay
Speed-of-Replay
0
100
100
5
1
percent

BUTTON
104
228
179
263
Clear
clear
NIL
1
T
OBSERVER
T
NIL

SWITCH
88
388
178
421
Purple.
Purple.
1
1
-1000

SWITCH
88
433
178
466
Green.
Green.
1
1
-1000

BUTTON
103
183
179
217
Mark
mark
NIL
1
T
OBSERVER
T
NIL

TEXTBOX
6
49
102
90
Press "Draw" and drag your mouse on the black screen.

TEXTBOX
8
184
99
221
Press "Mark" only if "Replay" is running.

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.

If you want the Replay button to work, make sure the Draw Button is not on.



THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
