globals
[
  ticks              ;ticks of the clock
  create-a-sheep     ;boolean to create an offspring of a student's sheep
  create-a-wolf      ;boolean to create an offspring of a student's wolf
  sheep-body-types   ;list holding name of every sheep shape not in use
  wolf-body-types    ;list holding name of every wolf shape not in use
  num-sheep          ;number of sheep currently alive
  num-wolves         ;number of wolves currently alive
  new-sheep-shape    ;the name of the shape of a student's sheep, for use
                     ;  in creating identical offspring
  new-wolf-shape     ;the name of the shape of a student's wolf, for use
                     ;  in creating identical offspring
  new-x              ;the x-coordinate of a student's sheep, for use
                     ;  in placing offspring nearby
  new-y              ;the y-coordinate of a student's wolf, for use
                     ;  in placing offspring nearby
]

breeds
[
  sheep              ;sheep, are either a "sheep" shape, if not controlled by a student
                     ;  or of a distinct sheep shape, if offspring of a sheep controlled
                     ;  by a student
  wolves             ;wolves, are either a "wolf" shape, if not controlled by a student
                     ;  or of a distinct wolf shape, if offspring of a sheep controlled
                     ;  by a student
  students           ;either sheep or wolves, is controlled by a student
]

turtles-own
[
  energy             ;integer showing amount of energy species have, determines health
  body-name          ;string describing shape of animal in a phrase for display on client
  breed-type         ;whether turtle is a sheep, wolves, ssheep, or swolves
                     ;ssheep are students who control a sheep
                     ;swolves are students who control a wolf
]

sheep-own
[
  grabbed?           ;used to prevent two wolves from eating the same sheep
]

patches-own
[
  countdown          ;used to control time grass has been growing
]

students-own
[
  user-id            ;unique id, input by client when they log in, to identify each student turtle
  grabbed?           ;used to prevent two wolves from eating the same sheep
  step-size          ;size of step turtle, controlled by student, can take
  herd-spacing       ;amount of spacing in the herd controlled by student
]

;initialize variables, and create initial number of turtles
to setup
  hubnet-set-client-interface "COMPUTER" [ "pred-prey-client.nlogo" ]
  hubnet-reset
  ca
  cp
  ct
  clear-output
  set create-a-sheep 0
  set create-a-wolf 0
  set new-sheep-shape " "
  set new-wolf-shape " "
  set new-x 0
  set new-y 0
  set wolf-body-types ["wolfee" "wolfew" "wolfey" "wolfen" "wolfek" "wolfed"
                       "wolfyy" "wolfyk" "wolfyw" "wolfyd" "wolfye" "wolfyn"
                       "wolfnn" "wolfnk" "wolfnw" "wolfny" "wolfne" "wolfnd"
                       "wolfdd"  "wolfrk" "wolfdy" "wolfdw" "wolfdn" "wolfde"
                       "wolfww"  "wolfwk" "wolfwy" "wolfwn" "wolfwe" "wolfwd" ]

  set sheep-body-types ["sheepdd" "sheepdk" "sheepdy" "sheepdw" "sheepde" "sheepdn"
                        "sheepnn" "sheepnk" "sheepny" "sheepnw" "sheepne" "sheepnd"
                        "sheepkk" "sheepky" "sheepkw" "sheepkd" "sheepkg" "sheepke"
                        "sheepee" "sheepek" "sheepey" "sheepew" "sheepen" "sheeped"
                        "sheepyy" "sheepyk" "sheepyw" "sheepyd" "sheepye" "sheepyn"]

  set num-sheep initial-number-sheep
  set num-wolves initial-number-wolves
  set ticks 0

  ask patches [ set pcolor green ]
  if grass? [
      ;; indicates whether the grass switch is on
      ;; if it is true, then grass grows and the sheep eat it
      ;; if it false, then the sheep don't need to eat
    ask patches [
      set countdown random-int-or-float grass-regrowth-time ;; initialize grass grow clocks randoml
      if (random-int-or-float 2) = 0  ;;half the patches start out with grass
          [ set pcolor brown ]
    ]
  ]

  ;create initial number of sheep specified, initialize their variables
  set-default-shape sheep "sheep"
  create-custom-sheep initial-number-sheep
  [
    set color white
    set label-color blue - 2
    set energy random-int-or-float (2 * sheep-gain-from-food)
    setxy random-int-or-float screen-size-x random-int-or-float screen-size-y
    set grabbed? false
    set breed-type "sheep"
  ]

  ;create initial number of wolves specified, initialize their variables
  set-default-shape wolves "wolf"
  create-custom-wolves initial-number-wolves
  [     set color black
    set energy random-int-or-float (2 * wolf-gain-from-food)
    setxy random-int-or-float screen-size-x random-int-or-float screen-size-y
    set breed-type "wolves"
  ]

  display-labels

  do-plot

end

to go

  every 0.1 [

    ;; don't always update the graphics window on the server since it
    ;; can really bog down some clients, making the plots not update
    display
    no-display
    do-plot  ;; plot populations
  ]

  ;; get commands and data from the clients
  listen-clients

  ;ask sheep who are not controlled by students (generic sheep) to operate
  ask sheep with [shape = "sheep"]
  [
    random-move
    if grass? [
      set energy energy - 1  ;; deduct energy for sheep only if grass? switch is on
      eat-grass
    ]
    reproduce-sheep
    death
  ]

  ;ask sheep who are controlled by students to operate
  ask students with [breed-type = "ssheep"]
  [

    if grass? [
      set energy energy - 1  ;; deduct energy for sheep only if grass? switch is on
      eat-grass
    ]
    reproduce-student-sheep
    death
  ]

  ;ask sheep who are offspring of sheep controlled by students to operate
  ask sheep with [shape != "sheep"]
  [
    herd-move
    if grass? [
      set energy energy - 1  ;; deduct energy for sheep only if grass? switch is on
      eat-grass
    ]
    reproduce-sheep
    death
  ]

  ;create offspring of sheep controlled by students
  if create-a-sheep = 1
  [
    create-custom-sheep 1
    [
      set energy random-int-or-float (2 * sheep-gain-from-food)
      setxy new-x new-y
      set grabbed? false
      set shape new-sheep-shape
      set breed-type "ssheep"
    ]
    set create-a-sheep 0
  ]

  ;ask wolves not controlled by students (generic wolves) to operate
  ask wolves with [shape = "wolves" ]
  [
    random-move
    set energy energy - 1  ;; wolves lose energy as they move
    catch-sheep
    reproduce-wolves
    death
  ]

  ;ask wolves controlled by students to operate
  ask students with [breed-type = "swolves"]
  [

    set energy energy - 1
    catch-sheep
    reproduce-student-wolves
    death
  ]

  ;ask wolves who are offspring of wolves controlled by students to operate
  ask wolves with [shape != "wolves" ]
  [
    herd-move
    set energy energy - 1
    catch-sheep
    reproduce-wolves
    death
  ]

  ;create offspring of wolves controlled by students
  if create-a-wolf = 1
  [
    create-custom-wolves 1
    [
      set energy random-int-or-float (2 * wolf-gain-from-food)
      setxy new-x new-y
      set shape new-wolf-shape
      set breed-type "swolves"
    ]
    set create-a-wolf 0
  ]

  if grass? [ ask patches [ grow-grass ] ]

  every 0.5
    [ display-labels ]
  set ticks ticks + 1

  if not any? turtles [ stop ]

  if students != nobody [ask students [send-info-to-clients]]

end

;allows generic turtles to move randomly
to random-move  ;; turtle procedure
  rt random-int-or-float 50 - random-int-or-float 50
  fd 0.2
end

;allows turtles who are offspring of turtles controlled by students to move as a herd
to herd-move
  locals [temp-shape temp-xcor temp-ycor]

  set temp-shape shape    ;shape of offspring
  set temp-xcor xcor      ;xcor of offspring
  set temp-ycor ycor      ;ycor of offspring

  ;find parents of offspring, based on identical shapes
  ;  move offspring closer or away from parent, based on
  ;  a random approximation of the herd-spacing specified by the parent
  ask students with [shape = temp-shape]
  [
    ifelse(temp-xcor > xcor)
    [set temp-xcor (xcor + herd-spacing +
      (random-int-or-float(abs(temp-xcor - xcor))) - (abs(temp-xcor - xcor) / 2))]
    [set temp-xcor (xcor - herd-spacing +
      (random-int-or-float(abs(temp-xcor - xcor))) - (abs(temp-xcor - xcor) / 2))]

    ifelse(temp-ycor > ycor)
    [set temp-ycor (ycor + herd-spacing +
      (random-int-or-float(abs(temp-xcor - xcor))) - (abs(temp-xcor - xcor) / 2))]
    [set temp-ycor (ycor - herd-spacing +
      (random-int-or-float(abs(temp-xcor - xcor))) - (abs(temp-xcor - xcor) / 2))]
  ]

  set xcor temp-xcor     ;new xcor of offspring
  set ycor temp-ycor     ;new ycor of offspring

end

;sheep eat grass, if it is available
to eat-grass    ;; sheep eat grass, turn the patch brown
  if pcolor = green [
    set pcolor brown
    set energy energy + sheep-gain-from-food  ;; sheep gain energy by eating
  ]
end

;generic sheep, and offspring of student's sheep reproduce
to reproduce-sheep

    if random-int-or-float 100 < sheep-reproduce
    [  ;; throw "dice" to see if you will reproduce

      set energy (energy / 2)          ;; divide energy between parent and offspring
      hatch 1 [ rt random-int-or-float 360 fd 1 ]   ;; hatch an offspring and move it forward 1 step
      set num-sheep num-sheep + 1

    ]

end

;sheep controlled by students reproduce
to reproduce-student-sheep

    if (random-int-or-float 100 < sheep-reproduce)
    [  ;; throw "dice" to see if you will reproduce

      set energy (energy / 2)            ;; divide energy between parent and offspring
      set create-a-sheep 1
      set num-sheep num-sheep + 1
      set new-sheep-shape shape
      set new-x xcor + 1
      set new-y ycor + 1
      set heading random-int-or-float 360

    ]
end

;generic wolves, and offspring controlled by students reproduce
to reproduce-wolves
  if random-int-or-float 100 < wolf-reproduce [  ;; throw "dice" to see if you will reproduce
    set energy (energy / 2 )  ;; divide energy between parent and offspring
    hatch 1 [ rt random-int-or-float 360 fd 1 ]  ;; hatch an offspring and move it forward 1 step
    set num-wolves num-wolves + 1
  ]
end

;wolves controlled by students reproduce
to reproduce-student-wolves
  if random-int-or-float 100 < wolf-reproduce [
    set energy (energy / 2)
    set create-a-wolf 1
    set num-wolves num-wolves + 1
    set new-wolf-shape shape
    set new-x xcor + 1
    set new-y ycor + 1
    set heading random-int-or-float 360
  ]
end

;wolves randomly catch sheep or student controlled sheep
to catch-sheep
  locals [prey1 prey2 prey prey3]

  ;gives sheep and student controlled sheep on patch equal chance of being eaten
  set prey1 random-one-of sheep-here with [not grabbed?]
  set prey2 random-one-of students-here with [breed-type = "ssheep" and not grabbed?]
  set prey3 random-int-or-float 2

  if (prey3 = 0)
  [ ifelse (prey1 = nobody) [set prey prey2] [set prey prey1] ]
  if (prey3 = 1)
  [ ifelse (prey2 = nobody) [set prey prey1] [set prey prey2]]
  ;; grab a random sheep
  ;; that no one else is grabbing
  if prey != nobody
    ;; did we get one?  if so,
    [
      set grabbed?-of prey true                 ;; prevent other wolves from grabbing it
      ask prey [ die ]                          ;; kill it
      set num-sheep num-sheep - 1
      set energy energy + wolf-gain-from-food ]     ;; get energy from eating
end

;turtles die, when energy dips below zero
to death

locals
[
  temp-shape      ;shape of turtle controlled by student
  temp-grabbed    ;status of grabbed? from turtle controlled by student
  temp-x          ;xcor of turtle controlled by student
  temp-y          ;ycor of turtle controlled by student
  temp-energy
]

  if (students != nobody and round energy > 0) [ask students [send-info-to-clients]]
  if energy < 0
  [ if breed-type = "sheep"
    [
      set num-sheep num-sheep - 1
      die
    ]
    if breed-type = "wolves"
    [ set num-wolves num-wolves - 1
      die
    ]
    if breed-type = "ssheep"
    [ set num-sheep num-sheep - 1
      ;if there are any more sheep with the same shape as this student-sheep
      ; then kill the offspring sheep, and allow the student-sheep to take on char.
      set temp-shape shape
      ifelse (count sheep with [shape = temp-shape] > 0)
      [
        ask random-one-of sheep with [shape = temp-shape]
        [
          set temp-grabbed grabbed?
          set temp-x xcor
          set temp-y ycor
          set temp-energy energy
          die
        ]
      ]
      [die]
      set grabbed? temp-grabbed
      set xcor temp-x
      set ycor temp-y
      set energy temp-energy
    ]
    if breed-type = "swolves"
    [ set num-wolves num-wolves - 1
      set temp-shape shape
      ifelse (count wolves with [shape = temp-shape] > 0)
      [
        ask random-one-of wolves with [shape = temp-shape]
        [
          set temp-x xcor
          set temp-y ycor
          set temp-energy energy
          die
        ]
      ]
      [die]
      set xcor temp-x
      set ycor temp-y
      set energy temp-energy
    ]
  ]
end

to grow-grass  ;; patch procedure
  ;; countdown on brown patches, if reach 0, grow some grass
  if pcolor = brown [
    ifelse countdown <= 0
      [ set pcolor green
        set countdown grass-regrowth-time ]
      [ set countdown (countdown - 1) ]
  ]
end

to do-plot
  set-current-plot "populations"
  set-current-plot-pen "sheep"
  plot num-sheep
  set-current-plot-pen "wolves"
  plot num-wolves
  if grass? [
    set-current-plot-pen "grass / 4"
    plot count patches with [ pcolor = green ] / 4  ;; divide by four to keep it within similar
                                                ;; range as wolf and sheep populations
  ]
end

to display-labels
  ifelse show-energy?
  [
    ifelse grass?
    [
      ask turtles
      [ set label round energy ]
    ]
    [
      ask wolves
      [ set label round energy ]
      ask sheep
      [ set label no-label ]
    ]
  ]
  [
    ask turtles
    [ set label no-label ]
  ]
end

;; sends the appropriate monitor information back to the client
to send-info-to-clients
  hubnet-send user-id "Located at:" ("(" + pxcor + "," + pycor + ")")
  hubnet-send user-id "You are a:" (body-name)
  ifelse energy < 1
  [hubnet-send user-id "Energy:" ("(" + 0 + ")")]
  [hubnet-send user-id "Energy:" ("(" + round(energy) + ")")]
end

;; Kill the turtle, set its shape, color, and position
;; and tell the node what its turtle looks like and where it is
to remove-student
  ask students with [user-id = hubnet-message-source]
  [
    die
  ]
end

;; Cause the students to move forward step-size in new-heading's heading
to execute-move [new-heading]
  locals [ new-pcor screen-edge]

  ask students with [user-id = hubnet-message-source]
  [
    set heading new-heading

    ;; don't allow students to wrap around the screen
    ;; so use move-amount instead of step-size

  ifelse heading mod 180 = 0
  [
    set screen-edge screen-edge-y
    ifelse heading = 0
    [ set new-pcor pycor + step-size ]
    [ set new-pcor pycor - step-size ]
  ]
  [
    set screen-edge screen-edge-x
    ifelse heading = 90
    [ set new-pcor pxcor + step-size ]
    [ set new-pcor pxcor - step-size ]
  ]
  ;; if we would step past a patch at the edge of the graphics window,
  ;; make us step onto the edge patch
  if abs new-pcor > abs screen-edge
  [ jump step-size - ( abs new-pcor - abs screen-edge ) ]
  jump step-size


    hubnet-send user-id "Located at:" ("(" + pxcor + "," + pycor + ")")
  ]

end

;; Create a turtle, set its shape, color, and position
;; and tell the node what its turtle looks like and where it is
to create-new-student
  create-custom-students 1
  [
    setup-student-vars
    send-info-to-clients
    ;; we want to make sure that the clients all have the same plot ranges,
    ;; so when somebody logs in, set the plot ranges to themselves so that
    ;; everybody will have the same size plots.
    set-plot-y-range plot-y-min plot-y-max
    set-plot-x-range plot-x-min plot-x-max
  ]
end

;; sets the turtle variables to appropriate initial values
to setup-student-vars  ;; turtle procedure
locals [temp temp2]

  set user-id hubnet-message-source

  set temp random-int-or-float 2
  set herd-spacing 0

  ;if there are more students that are sheep, than initial number of sheep
  ;then make the next student be a wolf
  if (count students with [breed-type = "ssheep"] > initial-number-sheep) [ set temp 1 ]
  ;if there are more students that are wolves, than initial number of wolves
  ;then make the next student be a sheep
  if (count students with [breed-type = "swolves"] > initial-number-wolves) [set temp 0]
  ;if there are more students than the initial number of sheep and wolves
  ;requested, then make the next student be randomly assigned a sheep or wolf
  if (count students < (initial-number-sheep + initial-number-wolves)) [set temp random-int-or-float 2]

  if (temp = 0)
  [  ;;pick a random sheep
    if count sheep > 0 [ ask random-one-of sheep [die]  ]

    ;;student is a new sheep
    set breed-type "ssheep"
    set temp2 random-int-or-float length sheep-body-types
    set shape item temp2 sheep-body-types
    set sheep-body-types remove shape sheep-body-types
    set energy random-int-or-float (2 * sheep-gain-from-food)
    set grabbed? false
    set body-name "sheep with a "
    if (substring shape 5 6 = "d") [set body-name word body-name "red "]
    if (substring shape 5 6 = "e") [set body-name word body-name "blue "]
    if (substring shape 5 6 = "k") [set body-name word body-name "black "]
    if (substring shape 5 6 = "n") [set body-name word body-name "green "]
    if (substring shape 5 6 = "y") [set body-name word body-name "gray "]
    set body-name word body-name "blanket "
    if (substring shape 6 7 != substring shape 5 6)
    [
      set body-name word body-name "with a "
      if (substring shape 6 7 = "d") [set body-name word body-name "red "]
      if (substring shape 6 7 = "e") [set body-name word body-name "blue "]
      if (substring shape 6 7 = "k") [set body-name word body-name "black "]
      if (substring shape 6 7 = "n") [set body-name word body-name "green "]
      if (substring shape 6 7 = "y") [set body-name word body-name "gray "]
      if (substring shape 6 7 = "w") [set body-name word body-name "white "]
      set body-name word body-name "spot"
    ]
  ]

  if (temp = 1)
  [
    if count wolves > 0 [ask random-one-of wolves [die] ]
    set breed-type "swolves"
    set temp2 random-int-or-float length wolf-body-types
    set shape item temp2 wolf-body-types
    set wolf-body-types remove shape wolf-body-types
    set energy random-int-or-float (2 * wolf-gain-from-food)
    set body-name "wolf with a "
    if (substring shape 4 5 = "d") [set body-name word body-name "red "]
    if (substring shape 4 5 = "e") [set body-name word body-name "blue "]
    if (substring shape 4 5 = "w") [set body-name word body-name "white "]
    if (substring shape 4 5 = "n") [set body-name word body-name "green "]
    if (substring shape 4 5 = "y") [set body-name word body-name "gray "]
    set body-name word body-name "blanket "
    if (substring shape 5 6 != substring shape 4 5)
    [
      set body-name word body-name "with a "
      if (substring shape 5 6 = "d") [set body-name word body-name "red "]
      if (substring shape 5 6 = "e") [set body-name word body-name "blue "]
      if (substring shape 5 6 = "w") [set body-name word body-name "white "]
      if (substring shape 5 6 = "n") [set body-name word body-name "green "]
      if (substring shape 5 6 = "y") [set body-name word body-name "gray "]
      if (substring shape 5 6 = "k") [set body-name word body-name "black "]
      set body-name word body-name "spot"
    ]
  ]


  setxy random-int-or-float screen-size-x random-int-or-float screen-size-y
  set heading item random-int-or-float 3 [ 0 90 270 ]

end

;; NetLogo knows what each student turtle is supposed to be
;; doing based on the tag sent by the node:
;; step-size - set the turtle's step-size
;; Up    - make the turtle move up by step-size
;; Down  - make the turtle move down by step-size
;; Right - make the turtle move right by step-size
;; Left  - make the turtle move left by step-size
to execute-command [command]
  if command = "step-size"
  [
    ask students with [user-id = hubnet-message-source]
    [ set step-size hubnet-message ]
    stop
  ]
  if command = "herd-spacing"
  [
    ask students with [user-id = hubnet-message-source]
    [ set herd-spacing hubnet-message ]
    stop
  ]

  if command = "Up"
  [ execute-move 0 stop ]
  if command = "Down"
  [ execute-move 180 stop ]
  if command = "Right"
  [ execute-move 90 stop ]
  if command = "Left"
  [ execute-move 270 stop ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Code for interacting with the clients ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; determines which client sent a command, and what the command was
to listen-clients
  while [ hubnet-message-waiting? ]
  [
    hubnet-fetch-message
    ifelse hubnet-enter-message?
    [ create-new-student ]
    [
      ifelse hubnet-exit-message?
      [ remove-student ]
      [ execute-command hubnet-message-tag ]
    ]
  ]

end


;; report the amount we can move in the current direction


;; give the user some information about what the setup button does so they can
;; know whether they want to proceed before actually doing the setup
to setup-prompt
  if user-yes-or-no? ("The SETUP button should only be used when starting "
              + "over with a new group (such as a new set of students) since "
              + "all data is lost.  Use the RE-RUN button for continuing with "
              + "an existing group."
              + "\n\nDo you really want to setup the model?")
  [
    user-message "Before closing this dialog, please do the following:"
                 + "\n  -Have everyone that is currently logged in, log off and "
                 + "then kick all remianing clients with the HubNet Console."
    setup
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
358
11
636
310
5
5
24.4
1
10
1
1
1
0

CC-WINDOW
5
543
645
638
Command Center

SLIDER
3
163
177
196
initial-number-sheep
initial-number-sheep
0
50
3
1
1
NIL

SLIDER
3
200
177
233
sheep-gain-from-food
sheep-gain-from-food
0.0
50.0
4.0
1.0
1
NIL

SLIDER
3
235
177
268
sheep-reproduce
sheep-reproduce
0
20.0
7.0
1.0
1
%

SLIDER
181
163
360
196
initial-number-wolves
initial-number-wolves
0
50
3
1
1
NIL

SLIDER
181
199
346
232
wolf-gain-from-food
wolf-gain-from-food
0.0
100.0
60.0
1.0
1
NIL

SLIDER
181
235
346
268
wolf-reproduce
wolf-reproduce
0.0
20.0
7.0
1.0
1
%

SWITCH
5
100
99
133
grass?
grass?
1
1
-1000

SLIDER
106
101
318
134
grass-regrowth-time
grass-regrowth-time
0
100
33
1
1
NIL

BUTTON
8
41
77
74
setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
90
41
157
74
go
go
T
1
T
OBSERVER
T
NIL

PLOT
8
332
317
529
populations
time
pop.
0.0
100.0
0.0
100.0
true
true
PENS
"sheep" 1.0 0 -16776961 true
"wolves" 1.0 0 -65536 true
"grass / 4" 1.0 0 -11352576 true

MONITOR
87
274
158
323
sheep
count sheep
3
1

MONITOR
162
274
244
323
wolves
count wolves
3
1

MONITOR
248
274
324
323
grass / 4
count patches with [ pcolor = green ] / 4
0
1

TEXTBOX
8
143
148
162
Sheep settings

TEXTBOX
186
143
299
161
Wolf settings

TEXTBOX
9
81
161
99
Grass settings

MONITOR
5
274
83
323
time-ticks
ticks
0
1

SWITCH
167
41
303
74
show-energy?
show-energy?
1
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
This model explores the stability of predator-prey ecosystems. (The construction of this model is described in two papers by Wilensky & Reisman referenced below.) Such systems are called unstable when they tend to result in extinction for one or more species involved.  In contrast, systems are stable when they tend to maintain themselves over time, despite fluctuations in population sizes. There are two main variations to this model.

In the first variation, wolves and sheep wonder randomly around the landscape, while the wolves look for sheep to prey on. Each step costs the wolves energy, and they must eat sheep in order to replenish their energy - when they run out of energy they die. To allow the population to continue, each wolf or sheep has a fixed probability of reproducing at each time step. This variation produces interesting population dynamics, but is ultimately unstable.

The second variation includes grass (green) in addition to wolves and sheep. The behavior of the wolves is identical to the first variation, however this time the sheep must eat grass in order to maintain their energy - when they run out of energy they die. Once grass is eaten it will only regrow after a fixed amount of time. This variation is more complex than the first, but it is generally stable.


HOW TO USE IT
-------------
1. Set the GRASS? switch to TRUE to include grass in the model, or to FALSE to only include wolves (red) and sheep (white).
2. Adjust the slider parameters (see below), or use the default settings.
3. Press the SETUP button.
4. Press the GO button to begin the simulation.
5. View the POPULATIONS plot to watch the populations fluctuate over time
6. View the count-sheep/count-wolves monitors to view current population sizes

Parameters:
INITIAL-NUMBER-SHEEP: The initial size of sheep population
INITIAL-NUMBER-WOLVES: The initial size of wolf population
SHEEP-GAIN-FROM-FOOD: The amount of energy sheep get for every grass patch eaten
WOLF-GAIN-FROM-FOOD: The amount of energy wolves get for every sheep eaten
SHEEP-REPRODUCE: The probability of a sheep reproducing at each time step
WOLF-REPRODUCE: The probability of a wolf reproducing at each time step
GRASS?: Whether or not to include grass in the model
GRASS-REGROWTH-TIME: How long it takes for grass to regrow once it is eaten
SHOW-ENERGY?: Whether or not to show the energy of each animal in the Graphics Window

Notes:
- one unit of energy is deducted for every step a wolf takes
- when grass is included, one unit of energy is deducted for every step a sheep takes

THINGS TO NOTICE
----------------
When grass is not included, watch as the sheep and wolf populations fluctuate. Notice that increases and decreases in the sizes of each population are related. In what way are they related? What eventually happens?

Once grass is added, notice the green line added to the population plot representing fluctuations in the amount of grass. How do the sizes of the three populations appear to relate now? What is the explanation for this?

Why do you suppose that some variations of the model might be stable while others are not?


THINGS TO TRY
-------------
Try adjusting the parameters under various settings. How sensitive is the stability of the model to the particular parameters?

Can you find any parameters that generate a stable ecosystem that includes only wolves and sheep?

Try setting GRASS? to TRUE, but setting INITIAL-NUMBER-WOLVES to 0. This gives a stable ecosystem with only sheep and grass. Why might this be stable while the variation with only sheep and wolves is not?

Notice that under stable settings, the populations tend to fluctuation at a predictable pace. Can you find any parameters that will speed this up or slow it down?

Try changing the reproduction rules -- for example, what would happen if reproduction depended on energy rather than being determined by a fixed probability?


EXTENDING THE MODEL
-------------------
There are a number ways to alter the model so that it will be stable with only wolves and sheep (no grass). Some will require new elements to be coded in or existing behaviors to be changed. Can you develop such a version?


NETLOGO FEATURES
----------------
Note the use of breeds to model two different kinds of "turtles": wolves and sheep. Note the use of patches to model grass.

Note use of the RANDOM-ONE-OF agentset command to select a sheep to be eaten by a wolf.


RELATED MODELS
---------------
Look at Rabbit Grass Weeds for another model of interacting populations with different rules.


CREDITS AND REFERENCES
----------------------
Wilensky, U. & Reisman, K. (1999). Connected Science: Learning Biology through Constructing and Testing Computational Theories -- an Embodied Modeling Approach. International Journal of Complex Systems, M. 234, pp. 1 - 12. (This model is a slightly extended version of the model described in the paper.)

Wilensky, U. & Reisman, K. (in press). Thinking like a Wolf, a Sheep or a Firefly: Learning Biology through Constructing and Testing Computational Theories -- an Embodied Modeling Approach. Cognition & Instruction.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Wolf Sheep Predation model. http://ccl.northwestern.edu/netlogo/models/WolfSheepPredation. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

sheep
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135

sheepdd
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58

sheepde
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Rectangle -16776961 true false 147 83 203 139

sheepdk
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Rectangle -16777216 true false 146 87 204 141

sheepdn
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Rectangle -16711936 true false 151 87 206 139

sheepdw
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Rectangle -1 true false 151 80 206 133

sheepdy
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Rectangle -7566196 true true 149 90 200 135

sheeped
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70
Rectangle -65536 true false 149 89 203 139

sheepee
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70

sheepek
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70
Rectangle -16777216 true false 151 84 205 141

sheepen
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70
Rectangle -16711936 true false 150 87 201 140

sheepew
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70
Rectangle -1 true false 149 82 204 138

sheepey
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16776961 true false 133 70 102 161 225 175 237 71 222 55 199 46 174 44 142 56
Polygon -16776961 true false 201 47 223 53 233 61 236 69 211 70
Rectangle -7566196 true true 152 87 205 141
Rectangle -7566196 true true 159 139 160 139

sheepkd
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54
Rectangle -16776961 true false 150 82 204 135
Rectangle -65536 true false 150 81 206 137

sheepke
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54
Rectangle -16776961 true false 150 82 204 135

sheepkk
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54

sheepkn
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54
Rectangle -16711936 true false 151 85 206 141

sheepkw
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54
Rectangle -1 true false 147 85 202 140

sheepky
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16777216 true false 132 68 100 161 225 177 237 71 222 54 199 45 174 43 143 54
Rectangle -7566196 true true 152 86 203 139

sheepnd
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16711936 true false 132 68 101 160 225 176 237 71 221 53 200 46 174 44 142 55
Polygon -16711936 true false 223 176 102 157 101 160 120 154 103 159
Polygon -16711936 true false 221 175 101 158 101 161
Rectangle -65536 true false 152 83 204 134

sheepne
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16711936 true false 132 68 101 160 225 176 237 71 221 53 200 46 174 44 142 55
Polygon -16711936 true false 223 176 102 157 101 160 120 154 103 159
Polygon -16711936 true false 221 175 101 158 101 161
Rectangle -16776961 true false 151 84 202 137

sheepnk
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16711936 true false 132 68 101 160 225 176 237 71 221 53 200 46 174 44 142 55
Polygon -16711936 true false 223 176 102 157 101 160 120 154 103 159
Polygon -16711936 true false 221 175 101 158 101 161
Rectangle -16777216 true false 150 82 204 132

sheepnn
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Polygon -16711936 true false 134 57 98 162 228 177 238 65 229 54 203 43 175 40 156 47 134 57

sheepnw
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16711936 true false 132 68 101 160 225 176 237 71 221 53 200 46 174 44 142 55
Polygon -16711936 true false 223 176 102 157 101 160 120 154 103 159
Polygon -16711936 true false 221 175 101 158 101 161
Rectangle -1 true false 153 84 202 135

sheepny
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -16711936 true false 132 68 101 160 225 176 237 71 221 53 200 46 174 44 142 55
Polygon -16711936 true false 223 176 102 157 101 160 120 154 103 159
Polygon -16711936 true false 221 175 101 158 101 161
Rectangle -7566196 true true 150 80 203 133

sheepyd
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Rectangle -65536 true false 148 81 204 137

sheepye
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Rectangle -16776961 true false 145 83 204 139

sheepyk
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Rectangle -16777216 true false 153 80 203 134

sheepyn
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Rectangle -16711936 true false 149 83 202 137

sheepyw
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56
Rectangle -1 true false 148 83 200 136

sheepyy
false
0
Circle -1 true false 35 16 98
Circle -1 true false 16 53 64
Circle -1 true false 149 66 119
Circle -1 true false 185 165 64
Circle -1 true false 77 156 68
Circle -1 true false 75 70 111
Circle -1 true false 125 127 81
Circle -1 true false 237 58 34
Rectangle -1 true false 208 225 233 270
Rectangle -1 true false 96 218 121 270
Rectangle -1 true false 96 212 125 270
Rectangle -1 true false 202 216 203 216
Rectangle -1 true false 204 215 232 270
Rectangle -1 true false 96 208 128 270
Circle -1 true false 67 81 106
Circle -1 true false 117 47 135
Rectangle -65536 true false 133 70 132 70
Polygon -65536 true false 133 69 103 161 225 176 236 70 219 54 198 47 172 45 142 58
Polygon -7566196 true true 133 67 101 161 225 176 237 70 223 55 199 46 174 43 139 57
Polygon -7566196 true true 176 43 201 45 224 54 238 69 209 56

wolf
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40

wolfdd
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72

wolfde
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Rectangle -16776961 true false 175 95 220 132
Rectangle -16776961 true false 172 90 225 135

wolfdk
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Rectangle -7566196 true false 173 92 225 136
Rectangle -16777216 true false 174 91 226 136
Rectangle -16777216 true false 173 93 221 135

wolfdn
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Rectangle -16711936 true false 174 96 221 136

wolfdw
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Rectangle -1 true false 172 93 222 135

wolfdy
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Rectangle -16711936 true false 174 96 221 136
Rectangle -7566196 true false 173 95 222 138

wolfed
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71
Rectangle -65536 true false 174 96 227 135

wolfee
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71

wolfek
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71
Rectangle -16777216 true false 173 93 222 136
Rectangle -16777216 true false 22 317 23 317

wolfen
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71
Rectangle -16711936 true false 170 93 227 136

wolfew
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71
Rectangle -1 true false 174 99 226 136

wolfey
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16776961 true false 135 85 155 171 252 156 254 66 244 59 140 71
Rectangle -7566196 true false 171 94 225 136

wolfnd
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70
Rectangle -65536 true false 172 92 225 135

wolfne
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70
Rectangle -16776961 true false 173 93 224 138

wolfnk
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70
Rectangle -16777216 true false 171 93 222 131

wolfnn
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70

wolfnw
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70
Rectangle -1 true false 174 94 224 137

wolfny
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -16711936 true false 136 84 155 171 252 157 254 66 244 58 143 70
Rectangle -7566196 true false 170 89 222 136

wolfwe
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71
Rectangle -16776961 true false 172 90 225 136

wolfwk
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71
Rectangle -16777216 true false 174 91 221 135

wolfwn
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71
Rectangle -16711936 true false 174 91 222 135

wolfwr
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71
Rectangle -65536 true false 172 88 221 135

wolfww
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71

wolfwy
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -1 true false 135 82 155 172 252 157 254 66 240 58 143 71
Rectangle -7566196 true false 171 93 220 136

wolfyd
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82
Rectangle -65536 true false 171 89 224 135

wolfye
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82
Rectangle -16776961 true false 172 88 222 137

wolfyk
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82
Rectangle -16777216 true false 174 89 223 136

wolfyn
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82
Rectangle -16711936 true false 171 92 222 137

wolfyw
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82
Rectangle -1 true false 168 93 223 138

wolfyy
false
2
Polygon -7566196 true false 109 108 132 24 8 93
Polygon -7566196 true false 101 43 72 15 78 60
Polygon -7566196 true false 100 99 46 145 131 129
Polygon -7566196 true false 119 70 140 86 257 72 251 37 282 67 258 71 280 68 254 174 142 175 118 132 104 109
Polygon -7566196 true false 145 176 124 223
Polygon -7566196 true false 141 174 124 237 161 281 145 226 165 175
Polygon -7566196 true false 216 175 204 232 247 270 264 283 225 225 254 174
Polygon -7566196 true false 217 177 221 163 240 169 241 169
Polygon -7566196 true false 222 172 227 186 242 175 236 167
Polygon -7566196 true false 132 83 278 67 155 98
Polygon -7566196 true false 116 132 144 192 252 172
Polygon -7566196 true false 170 182 144 231 150 179
Polygon -7566196 true false 124 238 170 287 146 228
Polygon -7566196 true false 256 68 258 80 279 67
Rectangle -16777216 true false 71 17 71 16
Polygon -16777216 true false 72 15 75 54 5 93 91 106 41 147 116 133 138 182 122 238 172 292 148 228 167 190 213 182 201 233 268 288 227 224 257 170 282 66 247 31 254 67 137 81 122 69 133 23 100 40
Polygon -65536 true false 136 83 156 169 251 156 253 66 244 60 143 72
Polygon -7566196 true false 135 84 154 171 251 156 255 65 244 58 142 70 135 82

@#$#@#$#@
NetLogo 2.1beta3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
