globals [TimeElapsed Depth index AlgaeDead AlgaeHatched]
breeds [alg1 alg2 alg3]
patches-own [irradiance nutrient]
    ; density determines float vs sink / NutContent: growth vs ballast / Biomass
turtles-own [density energy NutContent Biomass]

to setup
ca
set TimeElapsed 0
set-default-shape alg1 "oscillatoria"
cct-alg1 NumAlgae  [randomize]
                           ;   turtles-own variables are set as a distribution of values with mean and standard deviation
ask turtles [set energy random-normal 1.4 0.7 set density random-normal 990 30 set NutContent random-normal 5 2 set Biomass random-normal 7.5 2]
                           ; this patch represents "sediment" at bottom of water column
ask patch 0 (- screen-edge-y) [set pcolor 35  set nutrient SedimentNutrient]
setup-plot
do-plot
end


to go
solar-input
ask alg1
  [photosynthesize
   reproduce
   floatSink
                    ;;   ; algae that hit sediment have 25% probability of dying and releasing nutrient
   if ycor < (- screen-edge-y + 0.4) and (random 100 < 25)
      [set AlgaeDead AlgaeDead + 1 set nutrient-of patch-here nutrient + NutContent die ]
   assimilate ]
set TimeElapsed TimeElapsed + TimeInterval
ask patches [gradient]     ; nutrient diffuses in water column
                           ; colorize gradients (Light & Nutrients)
ask patches [ifelse int TimeElapsed mod 2 = 0
    [if pycor != (- screen-edge-y) [set pcolor 40 + irradiance / 100]  ]
    [if pycor != (- screen-edge-y) [set pcolor 10 + nutrient]  ] ]
do-plot
end


to solar-input                        ; sets solar irradiance at surface
ask patch 0 screen-edge-y
  [ifelse TimeElapsed mod 24 - Daylength < 0
    [set irradiance 1000 * SIN(180 * (TimeElapsed mod 24) / Daylength)]
    [set irradiance 0] ]
                            ; irradiance decays exponentially through water column / algae attenuate also
ask patches
  [if abs pycor !=  screen-edge-y
    [set irradiance irradiance-of (patch 0 (pycor + 1)) * exp (- (Extinction + count turtles-here * AlgalExtinction))] ]
end

to gradient                                    ;; patch procedure -- distributes nutrient through water column
   locals [curChem chemLost]
   set curChem nutrient
   set chemLost nutrient * DiffusionPortion * TimeInterval
   ifelse ((abs pycor) != screen-edge-y) and (pycor != (screen-edge-y - 1))
     [set nutrient curChem + 0.5 * DiffusionPortion * TimeInterval * (nutrient-of patch-at 0 1 + nutrient-of patch-at 0 -1) - chemLost]
     [ifelse pycor = (screen-edge-y - 1)
        [set nutrient curChem + 0.5 * DiffusionPortion * TimeInterval * (nutrient-of patch-at 0 -1) - 0.5 * chemLost]
        [if pycor = (- screen-edge-y)
           [set nutrient curChem + 0.5 * DiffusionPortion * TimeInterval * (nutrient-of patch-at 0 1) - 0.5 * chemLost] ] ]
end


to photosynthesize           ;; turtle procedure -- performs density change, biomass increase
locals [grew densityIncrease]
set grew false
            ; energy level is integrated over a 24 h period (I aver of Stella model)
set energy energy + ((irradiance-of patch-here - energy) * TimeInterval / 24)

            ; grows in light if nutrient-replete / max growth rate is 0.04 per h
if NutContent > 3
    [set Biomass Biomass + 0.12 * TimeInterval * irradiance-of patch-here / (irradiance-of patch-here + 25)
     set grew true]

            ; density increase is function of current irradiance & consumption is basal level plus 24 h light history
            ; if turtle grew, then density increase is only half as much
set densityIncrease ((7.92 * irradiance-of patch-here / (irradiance-of patch-here + 25)) - (1.38 + 0.001 * energy)) * TimeInterval
ifelse grew
   [set density density + 0.5 * densityIncrease]
   [set density density + densityIncrease]
end

to floatSink                             ;; turtle procedure
locals [velocity]
set heading 180                                                ; head S so that sinkers go S and floaters N (fd a negative number)
set velocity 5.23e-06 * (RadiusAlga ^ 2) * (density - 1000)    ;  first term is product of velocity, form and viscoscity (Stella model)
ifelse density < 1000
  [fd max list (velocity * TimeInterval) (ycor - screen-edge-y + 0.51) ]
  [fd min list (velocity * TimeInterval) (ycor + screen-edge-y) ]
end

to reproduce                            ;; turtle procedure
if Biomass > 10
  [set AlgaeHatched AlgaeHatched + 1
   set Biomass 0.5 * Biomass  set NutContent 0.5 * NutContent
   hatch 1 [set Biomass Biomass-of myself set NutContent NutContent-of myself set density (density-of myself + random-normal -0 10) rt 90 fd random-float 1.0 lt 90]
   ]
end

to assimilate                            ;; turtle procedure
locals [uptake]
set uptake AssimEffic * nutrient-of patch-here * TimeInterval
without-interruption
    [set NutContent NutContent + uptake
    ask patch-here [set nutrient (nutrient - uptake)] ]
end

; proc randomizes turtle position and heading
to randomize
  setxy (random-float (screen-size-x) - screen-edge-x) (random-float (screen-size-y) - screen-edge-y )
  rt random-float 360
  if pycor > (screen-edge-y - 1) or pycor < (- screen-edge-y - 1)
    [randomize]
end


to setup-plot ;; set up plotting
  set-current-plot "Nutrient Distribution"
  set-plot-x-range 0 (screen-size-y - 1)
  set-current-plot "Algal Distribution"
  set-plot-x-range 0 (screen-size-y - 1)
  set-current-plot "Biomass Content" plot-pen-reset
  set-current-plot "Nutrient Content" plot-pen-reset
  set-current-plot "Algal Stats"  plot-pen-reset
;  set-current-plot "Movement"
;  set-plot-y-range (- screen-edge-y) screen-edge-y
;  set index 0
;  repeat NumAlgae
;    [create-temporary-plot-pen "turtle" + index
;    set-plot-pen-color 15 + 10 * index
;    set index index + 1]
end


to do-plot
  locals [loc]
  set loc (screen-edge-y)
  set-current-plot "Nutrient Distribution"
  plot-pen-reset
  set-current-plot "Algal Distribution" plot-pen-reset
  repeat (screen-size-y - 1) [
; plots distribution of the chemical in the water column
  set-current-plot "Nutrient Distribution"
  set-current-plot-pen "Nut" plotxy abs (loc - screen-edge-y) nutrient-of patch 0 (loc - 1)
; plots distribution of algae in the water column
  set-current-plot "Algal Distribution"
  set-current-plot-pen "Alg" plotxy abs (loc - screen-edge-y) count turtles-on patch 0 (loc - 1)
   set loc loc - 1 ]
; plots distribution of biomas
  set-current-plot "Biomass Content"
  histogram-from alg1 [Biomass]
  set-current-plot "Nutrient Content"
  histogram-from alg1 [NutContent]
; plots average depth and standard deviation of algae
  set-current-plot "Algal Stats"
  set-current-plot-pen "s2"
  plotxy TimeElapsed ( - standard-deviation values-from turtles [ycor])
  set-current-plot-pen "mean"
  plotxy TimeElapsed mean values-from turtles [ycor] - screen-edge-y
;; plots movement of initial set of turtles
;  set-current-plot "Movement"
;  set index 0
;  repeat NumAlgae
;    [set-current-plot-pen "turtle" + index
;    if turtle index != nobody [plot ycor-of turtle index]
;    set index index + 1]
end

@#$#@#$#@
GRAPHICS-WINDOW
659
10
834
3506
0
10
165.0
1
10
1
1
1
0

CC-WINDOW
5
3520
898
3615
Command Center

BUTTON
9
10
75
43
Setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
82
10
145
43
Go
go
T
1
T
OBSERVER
T
NIL

SLIDER
9
52
181
85
Daylength
Daylength
0
24
12
1
1
hours

PLOT
713
362
889
517
Movement
NIL
NIL
0.0
24.0
0.0
10.0
true
true

SLIDER
10
92
192
125
Extinction
Extinction
0
1
0.45
0.05
1
per meter

SLIDER
447
327
620
360
TimeInterval
TimeInterval
0
1
0.25
0.05
1
hours

SLIDER
9
131
185
164
RadiusAlga
RadiusAlga
0
100
32
1
1
micrometers

MONITOR
715
13
823
62
Dissolved Nutrient
sum values-from patches [nutrient]
1
1

SLIDER
448
366
620
399
DiffusionPortion
DiffusionPortion
0
0.5
0.21
0.01
1
NIL

PLOT
211
169
411
319
Nutrient Distribution
Depth
NIL
0.0
20.0
0.0
20.0
false
false
PENS
"Nut" 1.0 1 -16777216 true

MONITOR
716
68
814
117
Algal Nutrient
sum values-from turtles [NutContent]
0
1

MONITOR
716
120
814
169
Total Nutrient
sum values-from patches [nutrient] + sum (values-from turtles [NutContent])
0
1

SLIDER
447
405
619
438
AssimEffic
AssimEffic
0
0.5
0.15
0.01
1
NIL

MONITOR
717
231
784
280
NIL
AlgaeDead
0
1

MONITOR
717
283
799
332
NIL
AlgaeHatched
0
1

MONITOR
59
175
116
224
Algae
count alg1
0
1

PLOT
421
170
621
320
Biomass Content
NIL
NIL
4.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

PLOT
210
19
410
169
Algal Distribution
Depth
NIL
0.0
100.0
0.0
10.0
false
false
PENS
"Alg" 1.0 1 -11352576 true

SLIDER
448
443
620
476
NumAlgae
NumAlgae
0
100
12
1
1
NIL

PLOT
419
16
619
166
Nutrient Content
NIL
NIL
0.0
10.0
0.0
20.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
447
479
622
512
AlgalExtinction
AlgalExtinction
0
0.03
0.0
0.0010
1
per m

SLIDER
446
514
618
547
SedimentNutrient
SedimentNutrient
0
500
200
10
1
NIL

PLOT
12
331
437
541
Algal Stats
Hours
Mean Depth or St Dev
0.0
100.0
-20.0
0.0
true
true
PENS
"s2" 1.0 0 -16776961 true
"mean" 1.0 0 -65536 true

@#$#@#$#@
Allan Konopka
NetLogo Workshop 2004

WHAT ARE YOU MODELING?
=======================

Aquatic ecosystems (lakes and oceans) -- in the water column are gradients of physical and chemical factors.  The activity of biological organisms may be restricted to certain regions due to the presence / absence of certain factors.  Some organisms' behavior is in response to these gradients; the organisms may also be agents that change the chemical conditions.

This particular model concerns algae that migrate vertically in the water column.  Migration can actively occur by flagellar movement or passively due to the buoyant lift provided by gas vesicles (hollow protein shells in the cell).  Gas-vacuolate algae produce polysaccharide ballast (which makes them denser than water) when they photosynthesize in the light and consume the polysaccharide ballast in the dark (which can make them less dense than water).

There is a gradient of light (highest at the surface) and a gradient of nutrient concentration (highest at the sediment surface).  Both factors are necessary but neither alone is sufficient for growth.  If the algae have nutrients and can photosynthesize they may grow. If there are nutrients in the water, they can acquire them to support growth in the light.


WHO ARE THE AGENTS?
===================

The "world" is a single column of patches.  The top patch is the atmosphere, the next is the surface of the lake, and the bottom patch in the column represents the surface of the lake sediment.

Algae are represented by turtles.  These algae have several properties: density, nutrient content, biomass, energy content.


WHAT ARE THE RULES?
===================

Solar irradiance at the lake surface varies over the course of the day.

There is an exponential reduction in light irradiance as it penetrates the water column.

Nutrients diffuse from patch to patch through the water column.  They can be taken up by algae present in the patch.

If algal filaments are on a patch (depth) where there is light, they photosynthesize.  The product of photosynthesis can be used to make polysaccharide ballast and/or biomass.  Biomass is produced if the organism's nutrient content is above a thrshhold value.  Half the photosynthate goes to biomass synthesis and half to polysaccharide ballast synthesis.  If biomass is not produced (nutrient content too low), all photosynthate goes toward ballast synthesis.

If biomass doubles, the alga "hatches" a new alga.

The density of the alga is evaluated.  If density > water density, it sinks and if density < water density it floats.  The velocity is computed from Stokes' Law -- the difference in density between cell and water and the radius of the particle are the important parameters.

If the alga reaches the sediment bottom, there is a certain probability that it will die and release its nutrient content to the sediment.



WHAT SETTINGS CAN VARY?
=======================
Daylength
Light extinction coefficient through the water column
Radius of the alga
Time interval over which the model is evaluated
Rate of nutrient diffusion through the water column
Rate of nutrient uptake by an alga


RESULTS / OBSERVATIONS
=======================
At this stage, the model generally "behaves" as expected, although some details are a bit bumpy.

That is, individual filaments cycle through the water column -- they float to the surface, produce polysaccharide ballast, become denser than water and sink.  At depth, they consume ballast and eventually become less dense than water and begin to float back to the surface.


FUTURE DIRECTIONS
=================
There are a number of details which should be analyzed:
- When do filaments form a homogeneous population (all carrying out same behavior at same time) vs oscillations out of phase with each other.
- Effect of different lake depths / extinction coefficients / algal radii upon behavior.

The vertical column could be used for many different microbial behaviors, not only in the water column but also with depth in sediment.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

algfilament
true
0
Rectangle -256 true false 31 139 285 144

anabaena
true
0
Circle -11352576 true false 139 141 22
Circle -11352576 true false 156 127 20
Circle -11352576 true false 125 157 20
Circle -11352576 true false 175 127 18
Circle -11352576 true false 107 158 18
Circle -11352576 true false 193 125 21
Circle -11352576 true false 109 176 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

oscillatoria
true
0
Line -7566196 true 45 198 238 90

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
