globals      [
             time                      ;; the model clock
             electtime                 ;; election clock
             social-choice             ;; the vote results for the society
             social-choice-text        ;; vote results in a word for monitor
             choice-e-minus-1          ;; choice at the previous election
             elections                 ;; counter: number of elections
             reversal                  ;; counter of reversed votes
             choice-rule               ;; rule for aggregating individual preferences
             network
             ]

turtles-own  [
             preference                ;; list of each turtle's order of preferences (0=blue, 1=green, 2=yellow)
             localpref                 ;; set of social choice of neighbors
             localwin                  ;; winning majority choice of neighbors
             loser?                    ;; did the turtle's choice prevail in last election?
             ]

patches-own  [
             ]


to setup
    clear-all
    set-default-shape turtles "person"
    crt number
    ask turtles                                                 ;; randomize placement
      [
        setxy random-float screen-size-x
              random-float screen-size-y
        set loser? false
      ]
    assign-preference                                           ;; assign initial preferences
    check-neighbors                                             ;; initialize turtles' knowledge of neighborhood
    color-turtles                                               ;; identify turtle preference by color
    set time 0                                                  ;; initialize globals
    set electtime 0
    set elections 0
    set choice-e-minus-1 0
    set reversal 0
end

to go
  check-neighbors
  change-preference
  color-turtles
  plot-elections
  elect-social-choice
  ask turtles [ set heading random 360 fd 1 ]
  do-plots
  ifelse (time < 5000)
    [ set time time + 1 ]
    [ stop ]
end


to do-plots
  set-current-plot "Preference Totals"

    set-current-plot-pen "gray"
    ppu
    plotxy 0 501
    ppd
    plotxy plot-x-max 501
    ppu
    set-current-plot-pen "bluepref"

    set-current-plot-pen "bluepref"
      plot count turtles with [ (item 0 preference) = 1 ]
    set-current-plot-pen "greenpref"
      plot count turtles with [ (item 0 preference) = 2 ]
    set-current-plot-pen "yellowpref"
      plot count turtles with [ (item 0 preference) = 3 ]

  set-current-plot "Preference Histogram"
    histogram-from turtles [ (item 0 preference) ]

end



to assign-preference                                        ;; turtles have a preference order
   ask turtles [
     set preference (list 1 2 3)                            ;; generate same order for everyone
     set preference shuffle preference                      ;; then everyone shuffles its list randomly
     ]
end

to color-turtles                                            ;; identify turtles' preferences by a unique color
  ask turtles
    [
      if (item 0 preference = 1) [ set color blue ]
      if (item 0 preference = 2) [ set color green ]
      if (item 0 preference = 3) [ set color yellow ]
    ]
end

to check-neighbors

  ask turtles
  [
  ifelse (coalitions? and time >= 10 and loser?)    ;; count up preferences only of neighboring losers, excluding self
    [
      set localpref (list (count turtles in-radius 1 with [self != myself and item 0 preference = 1 and loser?])
                    (count turtles in-radius 1 with [self != myself and item 0 preference = 2 and loser?])
                     (count turtles in-radius 1 with [self != myself and item 0 preference = 3 and loser?])
                    )

      if (item 0 localpref > item 1 localpref) and (item 0 localpref > item 2 localpref)     ;; check to see which choice is the winnner
         [ set localwin 1 ]
      if (item 1 localpref > item 0 localpref) and (item 1 localpref > item 2 localpref)     ;; and set localwin to blue, green, or yellow
         [ set localwin 2 ]
      if (item 2 localpref > item 0 localpref) and (item 2 localpref > item 1 localpref)
         [ set localwin 3 ]
      if (item 0 localpref = 0) and (item 1 localpref = 0) and (item 2 localpref = 0)
         [ set localwin -1 ]
      if (localwin = 0)                                                                      ;; if tie or no local winner
         [ set localwin -1 ]
     ]

     [
     set localpref (list (count turtles in-radius 1 with [self != myself and item 0 preference = 1])           ;; count up neighbors' preferences, excluding self
                    (count turtles in-radius 1 with [self != myself and item 0 preference = 2])
                    (count turtles in-radius 1 with [self != myself and item 0 preference = 3])
                   )

     if (item 0 localpref > item 1 localpref) and (item 0 localpref > item 2 localpref)     ;; check to see which choice is the winnner
        [ set localwin 1 ]
     if (item 1 localpref > item 0 localpref) and (item 1 localpref > item 2 localpref)     ;; and set localwin to blue, green, or yellow
        [ set localwin 2 ]
     if (item 2 localpref > item 0 localpref) and (item 2 localpref > item 1 localpref)
        [ set localwin 3 ]
     if (item 0 localpref = 0) and (item 1 localpref = 0) and (item 2 localpref = 0)
        [ set localwin -1 ]
     if (localwin = 0)                                                                      ;; if tie or no local winner
        [ set localwin -1 ]                                                                 ;; set localwin to -1

    ]
   ]
  end

to change-preference
  ask turtles
  [
    if (localwin != -1)                                                          ;; if there is a local winning preference
      [
      if (localwin != item 0 preference)                                         ;; and I disagree with it
        [
        check-losers-and-winners
        ]
      ]
  ]
end


to elect-social-choice
  ifelse (electtime < 10 )
    [
    set electtime electtime + 1
    ]
    [

    set choice-e-minus-1 social-choice
    ifelse (count turtles with [(item 0 preference) = 1 ] > count turtles with [(item 0 preference) = 2])
      [
      ifelse (count turtles with [(item 0 preference) = 1 ] > count turtles with [(item 0 preference) = 3])
        [set social-choice 1 set social-choice-text "Blue"]
        [set social-choice 3 set social-choice-text "Yellow"]
      ]
      [
      ifelse (count turtles with [(item 0 preference) = 2 ] > count turtles with [(item 0 preference) = 3])
        [set social-choice 2 set social-choice-text "Green"]
        [set social-choice 3 set social-choice-text "Yellow"]
      ]
      if (choice-e-minus-1 != social-choice)                       ;; if the collective choice changess
        [
          if (time != 10)                                          ;; if it isn't the first election
            [
            set reversal reversal + 1                              ;; tally the reversal of collective choice
            plot-reversal
            ]
        ]
    set electtime 1
    set-loser
    set elections elections + 1
    ]
end

to plot-reversal
  if (reversal-pen?)
  [
    set-current-plot "Preference Totals"
    set-current-plot-pen "red"
    ppu
    plotxy time plot-y-max
    if (social-choice = 1 )
      [ set-plot-pen-color blue ]
    if (social-choice = 2 )
      [ set-plot-pen-color green ]
    if (social-choice = 3 )
      [ set-plot-pen-color yellow ]
    ppd
    plotxy time plot-y-min
  ]
end

to set-loser
  ask turtles
    [
      ifelse (social-choice != item 0 preference)
        [set loser? true]
        [set loser? false]
    ]
end

to check-losers-and-winners
   ifelse (sore-losers?) and (loser?)                                              ;; if sore-losers? selected
          [ ]                                                                      ;; if not, do not change preferfences

          [ ifelse (good-winners?)                                                 ;; if good-winners? selected
            [                                                                      ;; make winners more sensitive to losers
            if (random-float (1 / winner-sensitivity) < voter-sensitivity)         ;; check my sensitivity
              [
              if (localwin = item 1 preference)                                    ;; reorder my preferences
                 [
                 set preference (replace-item 1 preference item 0 preference)
                 set preference (replace-item 0 preference localwin)
                 ]
              if (localwin = item 2 preference)
                 [
                 set preference (replace-item 2 preference item 1 preference)
                 set preference (replace-item 1 preference localwin)
                 ]
               ]
            ]
            [
            if (random-float 1 < (voter-sensitivity))                              ;; otherwise check my sensitivity
              [
              if (localwin = item 1 preference)                                    ;; reorder my preferences
                 [
                 set preference (replace-item 1 preference item 0 preference)
                 set preference (replace-item 0 preference localwin)
                 ]
              if (localwin = item 2 preference)
                 [
                 set preference (replace-item 2 preference item 1 preference)
                 set preference (replace-item 1 preference localwin)
                 ]
              ]

            ]
        ]

end

to check-loser-neighbors
    ask turtles
    [ ;; count up preferences only of neighboring losers, excluding self
      set localpref (list (count turtles in-radius 1 with [self != myself and item 0 preference = 1 and loser?])                        (count turtles in-radius 1 with [self != myself and item 0 preference = 2 and loser?])
                     (count turtles in-radius 1 with [self != myself and item 0 preference = 3 and loser?])
                    )

      if (item 0 localpref > item 1 localpref) and (item 0 localpref > item 2 localpref)     ;; check to see which choice is the winnner
         [ set localwin 1 ]
      if (item 1 localpref > item 0 localpref) and (item 1 localpref > item 2 localpref)     ;; and set localwin to blue, green, or yellow
         [ set localwin 2 ]
      if (item 2 localpref > item 0 localpref) and (item 2 localpref > item 1 localpref)
         [ set localwin 3 ]
      if (item 0 localpref = 0) and (item 1 localpref = 0) and (item 2 localpref = 0)
         [ set localwin -1 ]
      if (localwin = 0)                                                                      ;; if tie or no local winner
         [ set localwin -1 ]
     ]
end

to plot-elections
  if (electtime = 10 )
  [ if (election-pen?)
    [
    set-current-plot "Preference Totals"
    set-current-plot-pen "gray"
    ppu
    plotxy time plot-y-max
    ppd
    plotxy time plot-y-min
    ppu
    set-current-plot-pen "bluepref"
    ]
  ]
end

to binary-choice

  ask turtles [
    if (item 0 preference = 3)
       [
       ifelse (random-float 1 < 0.5)
          [set preference (replace-item 0 preference 1)]
          [set preference (replace-item 0 preference 2)]
       ]
    ]
  color-turtles

end
@#$#@#$#@
GRAPHICS-WINDOW
205
89
525
430
15
15
10.0
1
10
1
1
1
0

CC-WINDOW
5
519
906
614
Command Center

BUTTON
41
31
107
64
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
112
31
177
64
NIL
go
T
1
T
OBSERVER
T
NIL

MONITOR
315
29
372
78
blue
count turtles with [(item 0 preference) = 1 ]
0
1

MONITOR
375
29
432
78
green
count turtles with [ (item 0 preference)  = 2 ]
3
1

MONITOR
436
29
493
78
yellow
count turtles with [ (item 0 preference) = 3 ]
3
1

SLIDER
37
110
174
143
number
number
100
1000
1000
1
1
NIL

MONITOR
206
29
263
78
NIL
time
0
1

PLOT
534
90
897
342
Preference Totals
time
totals
0.0
100.0
0.0
550.0
true
false
PENS
"bluepref" 1.0 0 -16776961 true
"greenpref" 1.0 0 -11352576 true
"yellowpref" 1.0 0 -13312 true
"red" 1.0 0 -65536 true
"gray" 1.0 0 -3355444 false

PLOT
632
355
832
505
Preference Histogram
time
% of total
1.0
4.0
0.0
501.0
true
false
PENS
"<default>" 1.0 1 -16777216 false

SLIDER
17
237
189
270
voter-sensitivity
voter-sensitivity
0
1
0.06
0.01
1
NIL

MONITOR
644
31
698
80
Elected
social-choice-text
3
1

MONITOR
700
31
782
80
%-reversed
(reversal / elections) * 100
0
1

TEXTBOX
343
10
470
28
          Preferences

SWITCH
17
274
189
307
good-winners?
good-winners?
1
1
-1000

SWITCH
18
348
190
381
sore-losers?
sore-losers?
0
1
-1000

SLIDER
17
311
189
344
winner-sensitivity
winner-sensitivity
2
5
2
1
1
NIL

SWITCH
37
146
173
179
reversal-pen?
reversal-pen?
1
1
-1000

SWITCH
17
386
190
419
coalitions?
coalitions?
1
1
-1000

TEXTBOX
78
10
168
28
Model Setup

TEXTBOX
67
216
157
234
Voter Options

TEXTBOX
657
10
774
28
Dependent Variables

SWITCH
37
182
173
215
election-pen?
election-pen?
0
1
-1000

TEXTBOX
216
10
306
28
Clock

BUTTON
52
68
170
101
binary-choice
binary-choice
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
David C. Earnest
NetLogo Workshop 2004

WHAT IS IT?
-----------

This model explores how people form and change their political preferences.  It is based upon Condorcet's theorem, which shows that when individuals have more than two choices, the outcome of an election depends upon the rules of election (institutions) rather than on the distribution of preferences.  Condorcet showed that in the absence of such institutions, the collective choice may "cycle," or change regularly between possible societal choices.  This model illustrates how such cycles may emerge from the decentralized and localized decision-making of individual voters.



HOW IT WORKS
------------

The model simulates a Condorcet problem by generating a population of voters with random distributed preferences for three (nominal) policies: blue, green and yellow.  It explores the conditions under which Condorcet-style cycling may occur.  Rather than imposing an election rule, it simply calculates the social choices as the plurality.  The model holds an election every ten time periods, starting at t = 10.

Voters in the model move randomly around the space and may change their preference order.  Their decision to re-order their preferences depends upon two factors: (a) the preferences of their immediate neighbors (that is, all voters within a radius of one); and (b) the outcome of the previous election.  The user can enact different decision rules for voters (called good-winners?, sore-losers?, and coalitions?) by using the swicthes and sliders in the "Voter Options" control area.


HOW TO USE IT
-------------

Click the "Setup" button to generate a population of voters whose preferences and locations are distributed randomly. You can adjust the number of voters using the "number" slider to create a denser or thinner network of voters.  Click "go" to have the voters move and change their preferences according to their immediate neighbors.

When the "good-winners?", "sore-losers?", and "coalitions?" switches are all in the OFF position, a voter has no memory of the previous election.  A voter will change its preferences with a fixed probability only if its neigbors' preferences are different. The user can change the probability of voters changing their preferences with the "voter-sensitivity" slider.  If the slider is set to 0.20, for example, a voter will change its preferences 20 percent of the time (or alternatively, on average about one-fifth of the voters will change their preferences during any given step in the model).  If the slider is set to 1.00, voters will evaluate and, if necessary, change their preferences during each step of the model.

The good-winners?, sore-losers? and coalitions? switches activate different strategies for a subset of the voters.  Each switch allows the voters to remember the outcome of the previous election and to change (or choose not to) their preferences according to a specific decision rule.  When activated, the good-winners? switch tells those voters whose preferences "won" the previous election to be good winners--that is, to be more sensitive to the preferences of their neighbors.  Winners will be two to five times more sensitive than losers according to the value set by the "winner-sensitivity" slider.

The sore-losers? switch acts in a similar way, but makes those voters who lost the previous election perfectly insensitive to other voters.  That is to say, after losing an election the losing voters will not change their preferences until the next election at the earliest.

The coalitions? switch simulates the possibility of losing voters cooperating.  When selected, the switch allows losing voters to change their preference in response to other losing voters but not to the winners.  That is to say, losing voters listen to each other (but not to winning voters) and change their preferences accordingly, while winners listen to everyone, both winners and losers.


THINGS TO NOTICE
----------------

Watch what happens when none of the Voter Options switches are enacted.  Notice how one preference quickly "wins" an election, and how the society locks into this choice.

Watch what happens when you activate the "good-winners?" switch.  Set up and run the model several times to see whether or not good winners make the system more or less "stable"--that is, whether or not society converges around one choice or election cycles emerge.

Watch what happens when you activate the "sore-losers?" switch only.

Watch what happens when you activate the "coalitions?" switch only.


THINGS TO TRY
-------------

Try running the model with the "coalitions?" and "good-winners?" switches activated at the same time.

Try running the model with none of the switches activated.  Then, after four or five elections (40 or 50 on the time counter), activate the "coalitions?" switch (or alternatively the "sore-losers?" switch.


EXTENDING THE MODEL
-------------------

Extensions of this model might include:

1. introducing new collective choice rules, other than the plurality rule.
2. introducing non-local networks of voters, so that voters' preferences depend upon voters who are distant in the model.
3. increasing the time between elections (currently set for an election every 10 ticks).


NETLOGO FEATURES
----------------

None.


RELATED MODELS
--------------

See other voting models in the Models Library, including Rudy Ruckers' voting model adapted by Uri Wilensky and the various Prisoners' Dilemmma models in the unverified folder.


CREDITS AND REFERENCES
----------------------

For information on Condorcet's Theorem and election cycles, see:

James Adams, "Condorcet Efficiency and the Behavioral Model of the Vote," The Journal of Politics, 59, 4 (November 1997): 1252-1263.

David M. Estlund, Jeremy Waldron, Bernard Grofman, and Scott L. Feld, "Democratic Theory and the Public Interest: Condorcet and Rousseau Revisited," The American Political Science Review, 83, 4 (December 1989) 1317-1340.

Kurt Taylor Gaubatz, "Election Cycles and War," The Journal of Conflict Resolution, 35, 1 (June 1991): 212-144.

H. P. Young, "Condorcet's Theory of Voting," The American Political Science Review, 83, 4 (December 1988): 1231-1244.




Model developed by:

David C. Earnest, Ph.D.
Assistant Professor of Political Science
Old Dominion University
BAL 700
Norfolk, VA 23505
dearnest@odu.edu
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
