; add model procedures here

to set-defaults
  set line-up-first? true
  set show-setup? false
  set go-curly? false
  set radius 14
  set curl-ups 2
  set fd-factor 1
  set rt-factor 1
end

to plot-headings
  ask turtles [
    set-current-plot "Headings"
    set-current-plot-pen "heading"
    plotxy who heading
  ]
end

to setup
  ca
  set-default-shape turtles "turtle"
  ask patches with [pycor = 0] [set pcolor green]
  crt 360
  if not show-setup? [ask turtles [ht]]
  ifelse go-curly? [
    ask turtles [fd radius]
  ] [
    if line-up-first? [ask turtles [line-up]]
    repeat curl-ups [ask turtles [curl-up]]
    set radius 0.5 * max values-from turtles [ycor]
  ]
  if not show-setup? [ask turtles [st]]

  ask turtles [
    without-interruption [
      ask other-turtles-here [die]
    ]
  ]

  ask turtles [set size 3]
end

to line-up
  set heading 0
  set xcor who
end

to curl-up
  repeat count turtles - who [cycloid]
end

to go
  ask turtles [cycloid]
end

to cycloid
  fd sin heading * fd-factor ;; now in cycloid
  rt rt-factor
end
@#$#@#$#@
GRAPHICS-WINDOW
321
10
682
393
100
100
1.75
1
10
1
1
1
0

CC-WINDOW
5
551
691
646
Command Center

SLIDER
140
122
313
155
radius
radius
0
100
14
1
1
NIL

BUTTON
10
13
74
47
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
105
12
169
46
NIL
go
T
1
T
OBSERVER
T
NIL

SWITCH
6
56
157
89
line-up-first?
line-up-first?
0
1
-1000

SLIDER
5
226
178
259
fd-factor
fd-factor
0
10
1
1
1
NIL

SLIDER
6
265
179
298
rt-factor
rt-factor
0
90
1
1
1
NIL

SWITCH
164
56
307
89
show-setup?
show-setup?
1
1
-1000

SLIDER
6
189
179
222
curl-ups
curl-ups
0
5
2
1
1
NIL

BUTTON
200
13
299
47
NIL
set-defaults
NIL
1
T
OBSERVER
T
NIL

SWITCH
9
123
130
156
go-curly?
go-curly?
1
1
-1000

PLOT
9
314
209
464
Headings
who
heading
0.0
360.0
0.0
360.0
true
false
PENS
"heading" 1.0 2 -16777216 true

BUTTON
32
503
139
537
NIL
plot-headings
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
TJ Leone
NetLogo Workshop 2004

WHAT IS IT?
-----------
This model shows a circle rolling along a line.  It includes false starts and switches
and plot added to try to understand it.


HOW IT WORKS
------------
I'm still figuring this out.  My plan was to create a circle out of turtles and then
get each turtle travelling in a cycloid.

Here's are sites that show how a point on circle travels in a cycloid and present the
parametric equations that describe the curve:

  http://www.math.bcit.ca/entertainment/coaster/index.shtml
  http://www.xahlee.org/SpecialPlaneCurves_dir/Cycloid_dir/cycloid.html
  http://mathworld.wolfram.com/Cycloid.html

The parametric equations make sense to me.  If the circumference point we're following,
let's call it P, begins a the origin and rolls through t radians, then the circle
touches the ground at r*t, where r is the radius of the circle.

I still need to spend some time going over
the translation of the parametric equations (analytic geometry) into a Logo procedure  (turtle geometry) as presented at

  http://www.cs.nps.navy.mil/people/faculty/rowe/curve.html

The procedure for drawing a cycloid as presented on the site above is:

TO CYCLOID
FORWARD SIN HEADING
RIGHT 1
CYCLOID
END

My original plan was just to arrange turtles in a circle and then have each one travel
along a cycloid.  This didn't work because the cycloids weren't synched up correctly.
They were all of the same amplitude, but the tops and bottoms of arches were different.

Since the cycloid uses the sine of the heading to change its position, I tried changing initial headings in different ways.  I tried the initial heading used in the Turtles Circling model:

  set heading atan ycor (- xcor)

I also tried changing the intial heading by different amounts.  Neither of these seemed to make any difference.

If I set the initial heading of all turtles to the same thing, the whole circle moved
in a cycloid.

I tried laying out the turtles in a line and then having each go through a different number of steps in the cycloid, saving those headings and resetting turtles to those headings after arranging them in a circle.  That didn't work.

Finally, I tried layout out the turtles in a line and then having each run through a different number of cycloid steps--twice.  They arranged themselves in a circle, and the circle rolls nicely.

I create a "set-defaults" buttons to save all the settings that make the circle roll along the line.  I also left the other settings around to play with.

HOW TO USE IT
-------------
To make the circle roll, click "set-defaults", then "setup", then "go".  The "line-up-first?" switch determines if the turtles will be laid out in a line before any curl-ups are run.  Otherwise, the turtles are left at the origin.  The "curl-ups" are runs of cycloid.  The number of runs for each turtle depends on the id number of the turtle.

When the "show-setup" switch is one, the turtles are visible during "setup".  Otherwise, they are hidden until the end of "setup".

The "go-curly" switch is used to reproduce a bug I had in earlier versions that looked pretty cool.  The radius of a circle can be set when the "go-curly" switch is on.  Otherwise, it is always determined by "setup".

The "fd-factor" and "rt-factor" sliders multiply fd and rt turns in the cycloid procedure.

The headings of all turtles can be plotted at any moment by clicking on the "plot-headings" button.


THINGS TO NOTICE
----------------
Like I said above, I'm still trying to figure out why this works.  See if the default settings result in distinctive headings or spacings of turtles.


THINGS TO TRY
-------------
Try running "setup" with the "show-setup?" switch on.  Try starting up with different numbers of curl-ups or with the "line-up-first?" switch off.

See what happens when you have the go-curly? button on.  Play with radius, fd-factor, and rt-factor in these modes.

Try putting a pen down for different turtles to see the cycloids they're tracing.


EXTENDING THE MODEL
-------------------
Use the epicycloid to make a circle that rolls around the outside of another circle.  Use the hypocycloid to make a circle that rolls around the inside of another circle.


NETLOGO FEATURES
----------------
When "setup" is run with the default settings, more turtles are created than needed to complete the circle.  Multiple turtles end up occupying the same space.  This doesn't harm the effect, but I wanted to get rid of turtles in the same spot to make it easier to analyze what was going on.  So, after all turtles are arranged in a circle, I use the following code:

  ask turtles [
    without-interruption [
      ask other-turtles-here [die]
    ]
  ]

The "other-turtles-here" reporter gives me an agent set that includes all the turtles with the same location as "who" except for "who".  The "without-interruption" makes sure that two turtles on the same patch aren't trying to clobber each other at the same time.

I found out I only need 85 turtles when default settings are used, but I kept the initial number of turtles set to 360 and just remove extras as needed give me flexibility as I make further changes to the model.

RELATED MODELS
--------------
This model was inspired by the "Turning Circles" model in the NetLogo Models Library.


CREDITS AND REFERENCES
----------------------
This model was created by

TJ Leone
Leone Learning Systems, Inc.
237 Custer Ave
Evanston, IL 60202

tj@leonelearningsystems.com
http://www.leonelearingsystems.com
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
