

;--------------------------------------------------------
;;ask turtles with [member? 3 areas] [show self]  [member? 3 areas]
;ask turtle 1 [ show areas ]
;ask turtles with [member? 0 areas and member? 1 areas] [show self]


globals [clock halfedge current_area shape_list color_list expired]

breeds [agents officers edges nodes]

nodes-own [out-edges in-edges grabbed? shape_inventory production_limit biases]
edges-own [a b turtle_traffic]

turtles-own []

patches-own [ areas ]


to setup
  ca
  set-default-shape edges "line"

  set shape_list [ "circle" "square" "x" "diamond" "triangle" ]
  set color_list [ yellow blue brown white red violet orange pink gray green ]

  setup-nodes

  ask nodes
  [
     ask nodes with [self != myself]
        [form-edge myself]
  ]

  set clock 0
  set current_area 0
  set expired 0

  ask patches
     [
        set areas[]
     ]

end

to setup-nodes

   locals[initial_bias]

  cct-nodes number_nodes
  [
    set out-edges []
    set in-edges []
    set shape_inventory [ 0 0 0 0 0 ]

    set initial_bias 1
    set initial_bias initial_bias / number_nodes
    set initial_bias precision initial_bias 3
    set biases n-values number_nodes [initial_bias]

      set production_limit prod_limit
      set production_limit production_limit - 1
      set shape first shape_list
      set shape_list lput shape shape_list
      set shape_list butfirst shape_list

      set color first color_list
      set color_list lput color butfirst color_list

    set grabbed? false
;    set color random-one-of [blue]
    set size 10
    forward 50

    if label?
    [
       set label who
       set label-color white
    ]

  ]
end

to-report connected-to? [other-node]  ;; node procedure
  report member? other-node (map [b-of ?] out-edges)
end

to connect-to [other-node]  ;; node procedure
  hatch 1
    [ set breed edges
      set turtle_traffic 0
      set color blue
      set a myself
      set b other-node
      set (out-edges-of a) lput self (out-edges-of a)
      set (in-edges-of b) lput self (in-edges-of b)
;      set label who
      reposition ]
end

to reposition  ;; edge procedure
  setxy (xcor-of a) (ycor-of a)
  set heading towards-nowrap b
  set size distance-nowrap b
  jump (distance-nowrap b) / 2
end

to form-edge [other-node];; node procedure
;  set other-node random-one-of (nodes in-radius-nowrap 150 with [self != myself])
  if (other-node != nobody) and
     (not connected-to? other-node) and
     (not value-from other-node [connected-to? myself])
    [ connect-to other-node ]
end

to disappear  ;; edge procedure
  set (out-edges-of a) remove self (out-edges-of a)
  set (in-edges-of b) remove self (in-edges-of b)
  die
end


;--------------------------------------------------------

to kill_links

  if mouse-down? [
     ask turtles-at round mouse-xcor round mouse-ycor [
           disappear
     ]
  ]

end

;--------------------------------------------------------

to node_drag

locals [num]

set num 0

  while [mouse-down?]
  [
     ask turtles-at round mouse-xcor mouse-ycor [
        if num = 0 [
           set grabbed? true
           set num 1
        ]
     ]

     ask nodes with [ grabbed? ] [
        without-interruption [
           setxy mouse-xcor mouse-ycor
           foreach out-edges [ ask ? [ reposition ] ]
           foreach in-edges [ ask ? [reposition ] ]
        ]
     ]
  ]

  ask nodes [ set grabbed? false ]
  set num 0

end

;--------------------------------------------------------

to box_mouse

locals[startx starty endx endy draw_turtle]

  if mouse-down? [
     set startx mouse-xcor
     set starty mouse-ycor

  while [mouse-down?] [

     cct 1 [ set draw_turtle self ]

     ask draw_turtle [
         set color blue
         set xcor startx
         set ycor mouse-ycor
         set pcolor color
         die
      ]

      cct 1 [ set draw_turtle self ]

      ask draw_turtle [
         set color yellow
         set ycor starty
         set xcor mouse-xcor
         set pcolor color
         die
       ]

       set endx mouse-xcor
       set endy mouse-ycor

  ]

       cct 1 [ set draw_turtle self ]

       ask draw_turtle [
          set color red
          set xcor startx + 1
          set ycor starty - 1

          while [ xcor < endx ] [
                set heading 180
                repeat (starty - endy) [
                   fd 1
                   set areas fput current_area areas
                ]

                set xcor xcor + 1
                set ycor starty - 1
          ]

          die
       ]



       cct 1 [set draw_turtle self]

       ask draw_turtle [
          set color yellow
          set xcor mouse-xcor
          set ycor mouse-ycor
          pd
          set heading 0
          fd starty - endy
          lt 90
          fd endx - startx
          die
       ]

      cct 1 [set draw_turtle self]

      ask draw_turtle [
          set color blue
          set xcor mouse-xcor
          set ycor mouse-ycor
          pd
          set heading 270
          fd endx - startx
          rt 90
          fd starty - endy
          die
       ]

    set current_area current_area + 1

    change_biases

]
end


;--------------------------------------------------------

;to move  ;; node procedure
;
;   locals[to_go move_turtle]
;
;      set move_turtle self
;
;      ask move_turtle [
;
;          set to_go 0
;
;          while [ to_go < 100 ][
;             fd 1
;             set to_go to_go + 1
;          ]
;
;
;      ]
;
;end


to send_random

locals [endx endy s_to s_from order move_turtle quant obj able_to_produce? lottery lottery2 sum_so_far pick]
   if expired = number_nodes [ stop ]

   set s_from random-one-of nodes
;   set s_to random-one-of nodes with [self != s_from]

   set lottery biases-of s_from
   set lottery map [? * 1000] lottery

   set sum_so_far 0
   set lottery2 []
   foreach lottery [set sum_so_far sum_so_far + ? set lottery2 lput sum_so_far lottery2]
   set pick random 1000
   set pick first filter [? > pick] lottery2
   set pick position pick lottery2
       ;return the index of the first item with a value greater than the random number
   set s_to random-one-of nodes with [self != s_from]


   set able_to_produce? true

   set quant num-obj

   ask s_from [
       set obj shape-of self
       ifelse production_limit = 0 [
          set able_to_produce? false ] [
             set able_to_produce? true ]
   ]

   ask s_to  [
       set endx xcor
       set endy ycor

        ifelse (obj = "circle" ) [set shape_inventory replace-item 0 shape_inventory (item 0 shape_inventory + quant) ]
        [ ifelse (obj = "square") [set shape_inventory replace-item 1 shape_inventory (item 1 shape_inventory + quant) ]
        [ ifelse (obj = "x") [set shape_inventory replace-item 2 shape_inventory (item 2 shape_inventory + quant) ]
        [ ifelse (obj = "diamond") [set shape_inventory replace-item 3 shape_inventory (item 3 shape_inventory + quant) ]
        [ set shape_inventory replace-item 4 shape_inventory (item 4 shape_inventory + quant) ] ] ] ]

   ]

while [ quant > 0 and able_to_produce?] [

    ask s_from [

    set production_limit production_limit - 1

    ifelse production_limit = 0 [
        set able_to_produce? false
        set size 5
        set expired expired + 1 ]
        [ set able_to_produce? true ]

          hatch 1 [
             set size 5
             set heading towardsxy-nowrap endx endy
             set move_turtle self
          ]

       ask move_turtle [
          fd distancexy-nowrap endx endy
          find_edge s_from s_to
          die
       ]
   ]

set quant quant - 1

]

end

to find_edge [s_to s_from] ;node procedure

  locals [cur_edge]

  set cur_edge random-one-of edges with [(s_to = a and s_from = b) or (s_to = b and s_from = a)]

  if cur_edge != nobody [
     ask cur_edge [
        set turtle_traffic turtle_traffic + 1
        set label turtle_traffic
     ]
   ]

end

to send_to_from ;; node prodecure

locals [endx endy obj order move_turtle quant]

   set quant num-obj

   ask nodes with [ who = send_from ] [
       set size 20
       wait 1
       set size 10
       set obj shape-of self
   ]

   ask nodes with [ who = send_to ] [
       set size 20
       wait 1
       set size 10
       set endx xcor
       set endy ycor

       ifelse (obj = "circle" ) [set shape_inventory replace-item 0 shape_inventory (item 0 shape_inventory + quant) ]
        [ ifelse (obj = "square") [set shape_inventory replace-item 1 shape_inventory (item 1 shape_inventory + quant) ]
        [ ifelse (obj = "x") [set shape_inventory replace-item 2 shape_inventory (item 2 shape_inventory + quant) ]
        [ ifelse (obj = "diamond") [set shape_inventory replace-item 3 shape_inventory (item 3 shape_inventory + quant) ]
        [ set shape_inventory replace-item 4 shape_inventory (item 4 shape_inventory + quant) ] ] ] ]
   ]


;   foreach values-from nodes with [ who = send_from ] [
;       ask ? [


while [ quant > 0 ] [

    ask nodes with [ who = send_from] [
          hatch 1 [
             set size 5
             set heading towardsxy-nowrap endx endy
             set move_turtle self
          ]

       ask move_turtle [
          fd distancexy-nowrap endx endy
          die
       ]
   ]

set quant quant - 1

]

end

to send_shapes

locals [move_turtle to_go]

  foreach values-from nodes [self] [
    ask ? [

    hatch 1 [
       set size 5
       set heading towardsxy 0 0
       set move_turtle self
    ]

    ask move_turtle [
          set to_go 0

          while [ to_go < 100 ][
             fd 1
             set to_go to_go + 1
          ]
          die
    ]
    ]
  ]

end

to change_biases ; node procedure

locals [in_node]

    ask nodes with [member? 0 areas] [
        set in_node self
        ask nodes with [member? 0 areas and self != myself] [
             set biases replace-item who biases (item who biases * bias)
        ]
    ]

       ;     [set shape_inventory replace-item 3 shape_inventory (item 3 shape_inventory + quant) ]

end
@#$#@#$#@
GRAPHICS-WINDOW
321
10
886
596
55
55
5.0
1
30
1
1
1
0

CC-WINDOW
5
610
895
705
Command Center

BUTTON
6
10
80
43
setup
setup
NIL
1
T
OBSERVER
T
NIL

MONITOR
10
222
67
271
turtles
count turtles
0
1

MONITOR
172
224
222
273
clock
clock
3
1

SWITCH
229
240
319
273
label?
label?
1
1
-1000

BUTTON
7
55
66
88
drag
node_drag
T
1
T
OBSERVER
NIL
NIL

BUTTON
7
94
108
127
box mouse
box_mouse
T
1
T
OBSERVER
T
NIL

MONITOR
80
223
158
272
num areas
current_area
0
1

SLIDER
87
10
205
43
number_nodes
number_nodes
2
50
5
1
1
NIL

BUTTON
209
10
319
43
remove links
kill_links
T
1
T
OBSERVER
T
NIL

BUTTON
9
182
115
215
send across
send_shapes
NIL
1
T
OBSERVER
T
NIL

SLIDER
202
140
294
173
send_to
send_to
0
9
4
1
1
NIL

SLIDER
103
140
195
173
send_from
send_from
0
9
3
1
1
NIL

BUTTON
8
139
96
172
from - to
send_to_from
NIL
1
T
OBSERVER
T
NIL

SLIDER
217
182
322
215
num-obj
num-obj
1
3
3
1
1
NIL

BUTTON
122
182
209
215
send random
send_random
T
1
T
OBSERVER
T
NIL

SLIDER
117
94
209
127
prod_limit
prod_limit
1
100
50
1
1
NIL

SLIDER
217
95
309
128
bias
bias
0
5
3
1
1
NIL

@#$#@#$#@
Nathan Bos
Erik Johnston
Ning Nan
NetLogo Workshop 2004
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

diamond
false
0
Polygon -7566196 true true 150 15 45 150 150 285 255 150

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
