; reproduction of fish
; create 2 breeds-- girl fish and boy fish

globals [ watergood nspawn noldage nbadwater neaten nofood]
turtles-own [fishhealth energy age]
patches-own [foodrot]

;fishhealth needs to include age, energy, water quality, waterquality is related to number of fish, tanksize
;; amount of food provided, amount eaten

breeds [boyfish girlfish]

to startup
  ca
  create-custom-boyfish n-boyfish?
    [
    set color blue
    set shape "fish2"
     setxy random-float screen-size-x
          random-float screen-size-y
          set size 2
          set energy 20
          set age 10 ;; age of mature fish
          ]
  create-custom-girlfish n-girlfish?
    [ set color pink
      set shape "fish2"
      setxy random-float screen-size-x
          random-float screen-size-y
          set size 2
          set energy 30
          set watergood 80
          setplot
          ;;set age 10
    ]
    draw-walls
    set FilterFlow? 300
 end

to go
 ask turtles [
      fd .5  ;; wander around and look for food]
      lt random 30
      rt random 30
      set energy energy - .5
      set age age + .1
      bounce
      ]
     if count turtles = 0
       [
       stop
       ]
      feed
      if random-float 1 < p-spawn  ;; sets probability of spawning based on slider setting on interface
        [
          spawn
         ]
      kickbucket
      monitorwater
      eatsmallfish
      pollutewater
      filterwater
      set-current-plot-pen "Boyfish"
      plot count boyfish
      set-current-plot-pen "Girlfish"
      plot count girlfish
      set-current-plot-pen "W-Qual"
      plot watergood
end


to spawn
  if watergood > 60
    [ask girlfish with [ (energy >= 30) and (size >= 3) ]              ;make sure fish are in good health--energy
      [if any? (boyfish in-radius 2) with [ (energy >= 30) and (size >= 3) ];; and are mature in size
       [ hatch 20
         [set nspawn nspawn + 1
         set age 0
         set shape "fish2"
         set size 1
         set energy 15
         ifelse random 2 = 0 ;; half the time set fish to boyfish
          [
          set breed boyfish
          set color blue
          set shape "fish2"
          ]
          [
          set breed girlfish
          set color pink
          set shape "fish2"
          ]
         lt 45
         fd 3
         ]
        ]
       ]
      ]
end


to kickbucket
  ask turtles
  [
  if (energy <= 0)  ;;or (age > 80)
    [set nofood nofood + 1
    die
    ]
  if (age > 80)
    [set noldage noldage + 1
    die
    ]
  ]
   if watergood < 10
     [
      ifelse  (count turtles >= 3)  ;; if more than 3 fish, randomly pick one, otherwise, kill the rest
       [ask random-n-of 3 turtles
           [
           set nbadwater nbadwater + 1
           die
           ]
       ]
       [
          ask turtles [
          set nbadwater nbadwater + 1
          die
          ]
       ]
   ]
end

to feed ;; feed fish some amount of food at some interval in  random patches
        ;; if fish wander into patch with food, increase energy by 10
   ask random-n-of amount-food? patches
     [if random 100 < 50  ;;set 50% probability
       [if pcolor = black
         [
         set pcolor yellow
         set foodrot 0
         ]
      ]
     ]
    ask turtles
      [
       if pcolor = yellow
        [
        set pcolor black
        set energy energy + 10
          if size < 4
            [set size size + .2]
        ]
      ]
    ;rt random 360
   ; fd  3
end

to monitorwater  ;; this will monitor different waterquality variables
  ask patches
    [
      if pcolor = yellow
      [
       set foodrot foodrot + random-float 1
       if foodrot > 40
         [if watergood > 0
           [
           set watergood watergood - .1
           set foodrot 0
           set pcolor black
           ]
         ]
        ]
       ]
     end

;; improve waterquality by doing water change

to change-water
  set watergood 100
  ask patches
    [
      set foodrot 0
     ]
end

to eatsmallfish  ;; eat fish that are more than .5 smaller than me
locals [foodenergy mysize]
set foodenergy 0
    ask turtles
      [set mysize size
        if any? (turtles-on patch-ahead 2)  with [size <= (mysize - .25)]
        [
        ask turtles-on patch-ahead 2
        [
          set foodenergy size
          set neaten neaten + 1
          die
        ]
       ]
      set energy energy + foodenergy
       set watergood watergood - .1
      ]

end

;; for full grown fish, decrement waterquality by .03
;; for immature fish, decrement by .05

to pollutewater
  locals [bigfish littlefish bigpollute littlepollute]  ; holds count of fish =4 <4
  set bigfish count turtles with [size >= 3.5]
  set littlefish count turtles with [size < 3.5]
  set bigpollute bigfish * .03
  set littlepollute littlefish * .05
  set watergood watergood - (bigpollute + littlepollute)
end

to filterwater
if watergood < 100
  [
  set watergood watergood + filterflow? / 25
  ]
end

to plant
end

to setplot    ;; provides "0" line on graph
set-current-plot-pen "zero-axis"
  auto-plot-off
  plotxy 0 0
  plotxy 1000000000 0
  auto-plot-on
end

to draw-walls
  ask patches with [ abs pycor = screen-edge-y]
    [
    set pcolor sky
    ]
   ask patches with [ abs pxcor = screen-edge-x]
    [
    set pcolor sky
    ]
 end

;; this procedure checks the coordinates and makes the turtles
;; reflect according to the law that the angle of reflection is
;; equal to the angle of incidence
to bounce  ;; turtle procedure
  ; check: hitting left or right wall?
  if (abs pxcor-of patch-ahead 1 = screen-edge-x)
    ; if so, reflect heading around x axis
    [ set heading (- heading) ]
  ; check: hitting top or bottom wall?
  if (abs pycor-of patch-ahead 1 = screen-edge-y)
    ; if so, reflect heading around y axis
    [ set heading (180 - heading) ]
end


@#$#@#$#@
GRAPHICS-WINDOW
233
21
651
460
25
25
8.0
1
10
1
1
1
0

CC-WINDOW
5
491
865
586
Command Center

BUTTON
19
20
96
53
NIL
startup
NIL
1
T
OBSERVER
T
NIL

BUTTON
137
23
200
56
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
26
80
198
113
N-boyfish?
N-boyfish?
0
20
16
1
1
NIL

SLIDER
26
125
198
158
N-girlfish?
N-girlfish?
0
20
12
1
1
NIL

SLIDER
27
172
199
205
p-spawn
p-spawn
0
1
0.3
0.1
1
NIL

PLOT
7
280
228
456
Fish
time
n fish/ water quality
0.0
1000.0
-100.0
150.0
true
true
PENS
"Boyfish" 1.0 0 -16776961 true
"Girlfish" 1.0 0 -65281 true
"W-Qual" 1.0 0 -11352576 true
"zero-axis" 1.0 0 -16777216 false

SLIDER
684
29
856
62
amount-food?
amount-food?
0
50
48
1
1
NIL

BUTTON
687
83
804
116
NIL
change-water
NIL
1
T
OBSERVER
T
NIL

MONITOR
720
428
809
477
Waterquality
watergood
3
1

SLIDER
26
220
198
253
FilterFlow?
FilterFlow?
100
1000
300
10
1
NIL

MONITOR
700
161
820
210
Number spawned
nspawn
0
1

MONITOR
697
223
823
272
Poor Water Deaths
nbadwater
0
1

MONITOR
713
284
813
333
Number eaten
neaten
0
1

MONITOR
711
357
820
406
Old Age Deaths
noldage
0
1

@#$#@#$#@
Cindy E. Hmelo-Silver
Lei Liu
NetLogo Workshop 2004

WHAT IS IT?
-----------
A simulation of fish that swim, eat, grow, reproduce, excrete waste and die


HOW IT WORKS
------------
Two breeds of fish, male and female.  Periodically, some amount of food is added to the fish tank.

As fish swim, they use energy and age.
As fish eat, they gain energy, grow to adulthood (and then stop growing) and add waste to water.
Fish eat smaller fish
Fish die if: energy runs out, they get too old, water quality is poor.
Fish spawn according to a probability if boy fish and girl fish are near each other, are mature, have enough energy, and the water quality is good.

If food is not eaten, it rots and pollutes the water.
The filter cleans waste and pollution from water depending on the flow of the filter
Changing the water is a fast way to improve water quality.

HOW TO USE IT
-------------
Set the number of male and female fish that you would like to start with. Be sure to turn the filter to an amount adequate for the number of fish that you have.  Set the amount of food they will get periodincally.


THINGS TO NOTICE
----------------
Effects of variables on water quality and fish population.
Relation between fish population and water quality.
See relation between amount of food and population range.
Watch the fry (baby fish) grow!

THINGS TO TRY
-------------

Setting different ratio of male and female fish
Changing probability of spawning
Changing amounts of food
Changing efficiency of filter



EXTENDING THE MODEL
-------------------
different types of fish that are more or less tolerant of variations in water quality
size of tank
monitoring when parameters are changed
monitoring # water changes and when they occur
plants
nitrate variable
fish activity and feeding as f (light)
temperature
pH
dissolved oxygen and carbon dioxide


This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
Cindy E. Hmelo-Silver
Lei Liu
2004
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

fish2
true
0
Polygon -7566196 true true 150 16 96 54 75 99 69 152 92 220 124 248 86 288 156 276 212 287 180 248 215 224 240 170 242 128 229 83 189 31
Circle -1 true false 154 31 40
Circle -16777216 true false 164 40 22

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
