patches-own [age landcover new-landcover]  ;; holds the variables for pixel age and LUC
globals [clock houses]  ;; counts runs

to setup
locals [LUdistribute min-distance dwellers]
ca
set clock 0
clear-plot

set LUdistribute 1
repeat 5 [
  ask random-one-of patches [set landcover LUdistribute]
  set LUdistribute LUdistribute + 1
  ]
ask patches [
  if landcover = 0 [
    set min-distance min values-from patches with [landcover != 0] [distance myself]
    set new-landcover landcover-of one-of patches with [landcover != 0 and distance myself = min-distance]
    ]
  ]
ask patches with [ landcover = 0] [ set landcover  new-landcover]

repeat population [                                 ;; create longhouses randomly. # of longhouses = init # of turtles
  ask random-one-of patches
    [set landcover 6 set pcolor cyan]               ;; longhouses are assigned landcover 6
]
ask patches [
  if landcover = 1 [set pcolor green set age 50]       ;; forest
  if landcover = 2 [set pcolor red set age 0]         ;; cultivated
  if landcover = 3 [set pcolor black set age -1]        ;; water
  if landcover = 4 [set pcolor green + 3 set age random 5]   ;; regrowth
  if landcover = 5 [set pcolor white set age -1]       ;; oil palm
  ]
crt population
ask turtles [
  set size 3
  ]
populate  ;; calls the populate procedure
end

to populate
distribute  ;; calls the distribute procedure
ask turtles with [value-from patch-here [landcover] != 6]
 [without-interruption
   [setxy value-from first houses [pxcor]
          value-from first houses [pycor]]     ;; have the turtles go to the patch in the houses list with the lowest population (the first patch in the list)
  distribute  ;; calls the distribute procedure
  ]
end

to distribute
set houses sort-by [count turtles-on ?1 < count turtles-on ?2] values-from patches with [landcover = 6] [self] ;; selects patches with landcover = 6 and sort them based on the number of turtles on the patch and writesthe list to the houses variable
end

to grow
set clock clock + 1
ask patches [
  if landcover = 2 and age >= cultivationperiod [set landcover 4 set age 0]
  if landcover = 1 [set pcolor black set age age + 1]       ;; forest
  if landcover = 2 [set pcolor red set age age + 1]         ;; cultivated
  if landcover = 4 [set pcolor green + 3 set age age + 1]   ;; regrowth
  ]
plot mean values-from patches with [landcover = 4] [age]   ;; reports the average age of the regrowth

end

to cultivate
locals [locate destination]
populate
 ask turtles [
   without-interruption [
     set locate max values-from patches in-radius searchradius with [not any? turtles-here and landcover = 4] [age] ;; finds oldest non-occupied fallow patches
     ;show "locate is: " + locate
     set destination random-one-of patches in-radius searchradius with [not any? turtles-here and locate = age] ;; selects a patch from the world where the age is simular to the locate list
     ;show "destination is :" + destination
     setxy (pxcor-of destination) (pycor-of destination) ;; makes the turtle go to the selected destination
     ]
    if random 100 <= cultivation_probability [set landcover 2 set age 0]
   ]
end

to runall
grow
cultivate
end
@#$#@#$#@
GRAPHICS-WINDOW
512
10
917
436
30
30
6.48
1
10
1
1
1
0

CC-WINDOW
5
560
926
655
Command Center

BUTTON
1
10
67
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
71
10
134
43
NIL
grow
T
1
T
OBSERVER
T
NIL

PLOT
22
396
292
546
regrowth age
NIL
NIL
0.0
100.0
0.0
100.0
true
true
PENS
"regrowth" 1.0 0 -11352576 true

BUTTON
72
47
133
92
NIL
cultivate
T
1
T
OBSERVER
T
NIL

SLIDER
162
76
334
109
population
population
0
100
9
1
1
NIL

MONITOR
23
348
115
397
avg fallow age
mean values-from patches with [landcover = 4] [age]
3
1

MONITOR
113
348
170
397
NIL
clock
3
1

SLIDER
162
10
334
43
cultivationperiod
cultivationperiod
0
10
1
1
1
NIL

SLIDER
162
43
334
76
cultivation_probability
cultivation_probability
0
100
100
1
1
NIL

BUTTON
160
146
226
179
add 5
crt 5\npopulate
NIL
1
T
OBSERVER
T
NIL

MONITOR
165
348
222
397
turtles
count turtles
3
1

BUTTON
232
146
333
179
kill
ask random-one-of turtles [die]
NIL
1
T
OBSERVER
T
NIL

BUTTON
2
46
69
79
NIL
runall
T
1
T
OBSERVER
T
NIL

SLIDER
161
111
333
144
searchradius
searchradius
0
500
8
1
1
NIL

@#$#@#$#@
Martin Rudbeck Jepsen
Thomas Theis Nielsen
NetLogo Workshop 2004
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 0 0 300

circle 2
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1beta4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
