patches-own [food]
turtles-own [fitness
             turniness
             memory-weight
             last-memory
             genome
             gene1 ; 8 bits for each gene in genome are assigned values here
             gene2
             gene3 ; memory-depth

            ]
globals [generation
         age
         anim
         best-turtle
         fitlist
         best-fitness
         avg-fitness
         seed-genomes
         fledgling
         chosen-size
         genome-size
             best-gene1
             best-gene2
             best-gene3
         ]

to setup
    ca
    grow-landscape
    create-animals
end

to grow-landscape
    ask patches [set pcolor black
    set food 0]

    repeat patch-number
    [
        set food-of random-one-of patches 1
    ]
    ask patches [if food > 0 [set pcolor red]]
    repeat patch-size
    [
        ask patches with [ pcolor = red ] [ask neighbors4 [set pcolor red
                                                           set food 1
                                                           ]
                                          ]
    ]
end

to create-animals
    crt population-size
    ask turtles [ ;create genomes of appropriate size
        set genome list (random-int-or-float 2) (random-int-or-float 2)
        repeat 22 [ set genome lput (random-int-or-float 2) genome ]
        set genome-size 24
        rt random-int-or-float 360
        fd random-int-or-float screen-edge-x
        set shape "worm"
        set color blue
        ]
end

to evolve
        set generation (generation + 1)
        if (generation > generations ) [stop]
        compete
        do-best-of-generation-plot
        grow-landscape
        mate-recombine
        mutate
        rebirth
end

to compete
    set age 0
    ask turtles [
        set turniness 0
        set memory-weight 0
        find-gene-values ]
    repeat age-at-death
    [
        ask turtles
        [
            turn
            fd 1
            ifelse (food > 0 )
               [ set fitness (fitness + 1)
                 set last-memory 0
                 ask patch-at 0 0 [set pcolor cyan] ]
               [ set last-memory last-memory + 1 ;increment memory
                 set memory-weight ( last-memory / ( gene3 + 0.0001 ) ); sets memory strength
                 if (memory-weight > 1) [ set memory-weight 1 ] ; keeps weigth below 1
                ]
        ]
        set age age + 1
       ; wait 0.01
    ]
    do-best-of-age-plot
    set fitlist values-from turtles [fitness] ;makes list of fitness values
end

to find-gene-values
    set gene1 item 0 genome
    set gene1 gene1 + ( item 1 genome * 2 )
    set gene1 gene1 + ( item 2 genome * 4 )
    set gene1 gene1 + ( item 3 genome * 8 )
    set gene1 gene1 + ( item 4 genome * 16 )
    set gene1 gene1 + ( item 5 genome * 32 )
    set gene1 gene1 + ( item 6 genome * 64 )
    set gene1 gene1 + ( item 7 genome * 128 )
    set gene1 ( gene1 / 256 * 180)
    set gene2 item 8 genome
    set gene2 gene2 + ( item 9 genome * 2 )
    set gene2 gene2 + ( item 10 genome * 4 )
    set gene2 gene2 + ( item 11 genome * 8 )
    set gene2 gene2 + ( item 12 genome * 16 )
    set gene2 gene2 + ( item 13 genome * 32 )
    set gene2 gene2 + ( item 14 genome * 64 )
    set gene2 gene2 + ( item 15 genome * 128 )
    set gene2 ( gene2 / 256 * 180)
    set gene3 item 16 genome
    set gene3 gene3 + ( item 17 genome * 2 )
    set gene3 gene3 + ( item 18 genome * 4 )
    set gene3 gene3 + ( item 19 genome * 8 )
    set gene3 gene3 + ( item 20 genome * 16 )
    set gene3 gene3 + ( item 21 genome * 32 )
    set gene3 gene3 + ( item 22 genome * 64 )
    set gene3 gene3 + ( item 23 genome * 128 )
    set gene3 ( gene3 )
end

to turn
    set turniness gene1 * ( memory-weight ) + gene2 * ( 1 - memory-weight )
    rt random-int-or-float turniness
    lt random-int-or-float turniness
end

to mate-recombine
    locals [ fst ]
    set seed-genomes []
    set fst 0
    set best-fitness max fitlist
    set fitlist sort fitlist
    set avg-fitness mean values-from turtles [fitness]
    while [ length seed-genomes < population-size / 2 ] [ ;takes best 1/2 of genomes
        ask turtles [
                if (fitness = max fitlist) ;finds best current genome
                [ if (length seed-genomes > population-size / 2) [stop] ;when half of population is full, then cull remainder
                  set seed-genomes lput genome seed-genomes
                  if (fst = 0 ) [
                      set best-gene1 gene1
                      set best-gene2 gene2
                      set best-gene3 gene3 ]
                  die ] ]; die after adding genome, so it won't be chosen again
                set fitlist but-last fitlist ] ; go deeper away from high fitness if half of population-seed is not filled
    set chosen-size length seed-genomes ; will only allow mating between chosen ones of previous generation (not their offspring)
    ask turtles [die]
    while [length seed-genomes < population-size] ;fill genome pool with recombined products of chosen matings
       [
        make-fledgling ;recombine genomes of those who mate
        set seed-genomes lput fledgling seed-genomes ;add recombined genomes to pool
       ]


end

to make-fledgling
    locals [husband
            wife
            choice
            bit ]
    set fledgling []
    set husband item ( random-int-or-float chosen-size ) seed-genomes ;assigns random genome to husband
    set wife husband ;sets wife to husband, to get into while statement below
    ifelse (item 0 seed-genomes = item 1 seed-genomes) []
    [
    while [wife = husband ]
        [
        set wife item ( random-int-or-float length seed-genomes ) seed-genomes
        ] ;gives wife a new genome
    ]
    set bit 0
    repeat genome-size [ ;randomly chooses each gene from parent's genomes
        set choice random-int-or-float 2 ;choose wife/husband gene to add
        ifelse (choice = 1)
            [ set fledgling lput item bit wife fledgling ]  ;add wife's gene
            [ set fledgling lput item bit husband fledgling ]  ;add husband's gene
        set bit bit + 1
    ]
end

to mutate
    locals [genome-m
            bit-m
            switch]
    set genome-m 0
    set bit-m 0
    repeat length seed-genomes [ ; scan all genomes
        set bit-m 0
        repeat genome-size [ ; scan all genes
            if (random-int-or-float 1000 < mutation-rate * 1000 )  ; change bit if mutation rate threshold is met
            [ set switch item bit-m item genome-m seed-genomes
            ifelse (switch = 1)
            [ set seed-genomes (replace-item genome-m seed-genomes
                                    (replace-item bit-m (item genome-m seed-genomes) 0 )) ]
            [ set seed-genomes (replace-item genome-m seed-genomes
                                    (replace-item bit-m (item genome-m seed-genomes) 1 )) ]
            ]
        set bit-m bit-m + 1 ;moves to next bit
        ]
    set genome-m genome-m + 1 ; moves to next genome
    ]

end

to rebirth
    locals [next-contestant
    fst]
    set next-contestant 0
    set fst 0
    no-display
    crt population-size
    repeat population-size [
        ask turtle next-contestant [ ;assigns each new genome
            set genome item next-contestant seed-genomes
            set color blue
            ifelse (fst = 0) [pd
            set color cyan
            set fst 1 ]
            [ rt random-int-or-float 360
            fd random-int-or-float screen-edge-x ]
        ]
        set next-contestant next-contestant + 1
    ]
        display
end

to do-best-of-generation-plot
    set-current-plot "Best-Harvest"
    plot max values-from turtles [fitness]
end

to do-best-of-age-plot
    set-current-plot "Average-Harvest"
    plot mean values-from turtles [fitness]

    set-current-plot "Off Food Turniness"
    plot best-gene1

    set-current-plot "On Food Turniness"
    plot best-gene2

    set-current-plot "Memory Depth"
    plot best-gene3
end
@#$#@#$#@
GRAPHICS-WINDOW
434
31
1047
665
100
100
3.0
1
10
1
1
1

CC-WINDOW
31
543
209
695
Command Center

BUTTON
21
48
102
95
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
117
49
198
95
NIL
evolve
T
1
T
OBSERVER
T

SLIDER
21
103
193
136
population-size
population-size
0
100
30
1
1
NIL

SLIDER
22
144
194
177
generations
generations
0
100
100
1
1
NIL

SLIDER
204
104
376
137
patch-size
patch-size
0
10
10
1
1
NIL

SLIDER
205
143
377
176
patch-number
patch-number
0
100
30
1
1
NIL

MONITOR
214
48
286
97
NIL
generation
3
1

PLOT
24
224
217
379
Best-Harvest
Generation
Harvest
0.0
1.0
0.0
20.0
true
false

PLOT
221
224
413
377
Average-Harvest
Time
Harvest
0.0
2.0
0.0
2.0
true
false

MONITOR
308
46
375
95
NIL
age
3
1

SLIDER
21
184
193
217
age-at-death
age-at-death
0
2000
1783
1
1
NIL

SLIDER
204
184
376
217
mutation-rate
mutation-rate
0
1
0.01
0.0010
1
NIL

PLOT
23
382
217
536
Off Food Turniness
Time
Turniness
0.0
20.0
0.0
100.0
true
false

PLOT
220
382
411
536
On Food Turniness
Time
Turniness
0.0
20.0
0.0
100.0
true
false

PLOT
219
543
419
693
Memory Depth
Time
Depth
0.0
20.0
0.0
100.0
true
false

@#$#@#$#@
This program uses a genetic algorithm to evolve an optimal forager.
The foraging goal is find the most red pixels.

Agent Behavior:
Agents are each given a genome which assigns values to three parameters:
1)off-food turniness
2)on-food turniness and
3)memory depth
Agents decide how much to turn by interpolating between on and off food
turniness, using their memory.  If an agent has just seen food, it will turn
according to a tent function with a width equal to two times the value of the gene.
The tent function is always centered at the animals present heading.  If an
agent hasn't seen food in a long time, then it will do the same using its
off food turniness gene.  Agents use their memory depth gene to determine
how quickly they move from on food behavior to off food behavior.  Agents with
a higher value, will behave as if they are near food for longer than agents
with a low memory depth value.

Algorithm Behavior:

Agents are created for each new generation using a combination of better genomes from the last generation.
The "evolve" button does the following:
	set generation (generation + 1)
        if (generation > generations ) [stop]
        compete
        do-best-of-generation-plot
        grow-landscape
        mate-recombine
        mutate
        rebirth

compete: agents move around and collect resources by bumping into them.
do-best-of-generation-plot: plots the best harvest
grow-landscape: regrows the landscape to prepare it for the next generation
mate-recombine: this procedure only allows the best half of the agents to create offspring.
		the offspring are the recombined genomes of these genomes.
mutate: this goes through genomes and randomly changes genes based on the mutation rate.
rebirth: this assigns genomes to new agents.

I'm sure I forgot a few things in explaining this, but that's basically what it does.  There are
a lot of different assumptions that go into any genetic algorithm; I make different assumptions
on different days.  There are still a few things I would like to fix but have never had the time
to do so.  If you'd like to contact me, please do at thills@mail.utexas.edu.

Created by Thomas Hills.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

worm
true
0
Polygon -7566196 true true 68 75 70 93 87 110 98 113 125 116 134 124 135 143 129 187 139 194 163 210 181 203 221 183 171 186 156 172 156 157 152 127 138 104 113 98 89 94 78 70 75 55 69 65

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
setup
compete
@#$#@#$#@
@#$#@#$#@
