globals [ target ;the target genome
          step
          census
          fitlist ;the fitness list made from the agents
          seed-genomes ;the genomes to be assigned to the next generation
          fledgling ;genome product of recombination
          chosen-size
          best-fitness
          match
          best-genome
          avg-fitness ]
turtles-own [genome
             fitness ]
to setup
    ca
    clear-output
    ask patch-at 0 0 [set pcolor blue] ;sets center 'target' patch
    set seed-genomes []
    set best-genome 0
    set match 0
    create-target
    ask turtles [die]
    crt population-size
    ask turtles [ ;create genomes of appropriate size
        set size 5
        set genome list (random-int-or-float 2) (random-int-or-float 2)
        repeat target-size - 2 [
            set genome lput (random-int-or-float 2) genome
        ]
    ]
end

to evolve
    compare-genomes ;calculates distance from target genome
    ask turtles [
        jump fitness - target-size ;move distance from target

    ]
    do-plots
    display
    no-display
    set fitlist sort fitlist ;puts fitnesses in order
    set seed-genomes []
    mate-recombine ;takes best animals and creates new genomes
    ask turtles [die]
    mutate ; adds mutations to new generation
    rebirth ; gives genomes to new generation
    display
end

to mate-recombine
    locals [ fst ]
    set fst 0
    set best-fitness max fitlist
    set avg-fitness mean values-from turtles [fitness]
    while [ length seed-genomes < population-size / 2 ] [ ;takes best 1/2 of genomes
        ask turtles [
                if (fitness = max fitlist) ;finds best current genome
                [ if (length seed-genomes > population-size / 2) [stop] ;when half of population is full, then cull remainder
                ifelse (fst = 0 ) ; if your good, let'em know it
                  [ set seed-genomes lput genome seed-genomes
                    set best-genome genome
                    set fst 1] ; add genome to seed-genomes
                  [ set seed-genomes lput genome seed-genomes ]
                  die ] ]; die after adding genome, so it won't be chosen again
                set fitlist but-last fitlist ] ; go deeper away from high fitness if half of population-seed is not filled
    set chosen-size length seed-genomes ; will only allow mating between chosen ones of previous generation (not their offspring)
    while [length seed-genomes < population-size] ;fill genome pool with recombined products of chosen matings
       [
        make-fledgling ;recombine genomes of those who mate
        set seed-genomes lput fledgling seed-genomes ;add recombined genomes to pool
       ]
end

to make-fledgling
    locals [husband
            wife
            choice
            bit ]
    set fledgling []
    set husband item ( random-int-or-float chosen-size ) seed-genomes ;assigns random genome to husband
    set wife husband ;sets wife to husband, to get into while statement below
    ifelse (item 0 seed-genomes = item 1 seed-genomes) []
    [
    while [wife = husband ]
        [
        set wife item ( random-int-or-float length seed-genomes ) seed-genomes
        ] ;gives wife a new genome
    ]
    set bit 0
    repeat target-size [ ;randomly chooses each gene from parent's genomes
        set choice random-int-or-float 2 ;choose wife/husband gene to add
        ifelse (choice = 1)
            [ set fledgling lput item bit wife fledgling ]  ;add wife's gene
            [ set fledgling lput item bit husband fledgling ]  ;add husband's gene
        set bit bit + 1
    ]
end




to mutate
    locals [genome-m
            bit-m
            switch]
    set genome-m 0
    set bit-m 0
    repeat length seed-genomes [ ; scan all genomes
        set bit-m 0
        repeat target-size [ ; scan all genes
            if (random-int-or-float 1000 < mutation-rate * 1000 )  ; change bit if mutation rate threshold is met
            [ set switch item bit-m item genome-m seed-genomes
            ifelse (switch = 1)
            [ set seed-genomes (replace-item genome-m seed-genomes
                                    (replace-item bit-m (item genome-m seed-genomes) 0 )) ]
            [ set seed-genomes (replace-item genome-m seed-genomes
                                    (replace-item bit-m (item genome-m seed-genomes) 1 )) ]
            ]
        set bit-m bit-m + 1 ;moves to next bit
        ]
    set genome-m genome-m + 1 ; moves to next genome
    ]
end

to rebirth
    cct population-size [
      set size 5
      set genome item 0 seed-genomes
      set seed-genomes butfirst seed-genomes
    ]
   ; locals [next-contestant]
   ;; set next-contestant 0
;crt population-size
   ; repeat population-size [
    ;    ask turtle next-contestant [ ;assigns each new genome
    ;        set genome item next-contestant seed-genomes
    ;    ]
    ;    set next-contestant next-contestant + 1
   ; ]
end

to compare-genomes
    set census 0
    repeat population-size [
        ask turtle census [
            set step 0
            set fitness 0
            repeat target-size [ ;goes through genome
                if ( step < target-size + 1 ) [
                    if ( item step genome = item step target) ;compares genomes
                        [set fitness fitness + 1 ]
                set step step + 1
                ]
            ]
        ]
    set census census + 1
    ]
    set fitlist values-from turtles [fitness] ;makes list of fitness values
    set step 0
end

to create-target ;generates target genome
     set target list (random-int-or-float 2) (random-int-or-float 2)
     repeat target-size - 2 [
         set target lput (random-int-or-float 2) target
         ]
end

to do-plots
    set-current-plot "Best-Fitness"
        plot best-fitness
    set-current-plot "Average Fitness"
        plot avg-fitness
end
@#$#@#$#@
GRAPHICS-WINDOW
365
10
779
445
50
50
4.0
1
10
1
1
1

CC-WINDOW
367
449
768
570
Command Center

BUTTON
6
158
105
206
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
109
158
214
206
NIL
evolve
T
1
T
OBSERVER
T

SLIDER
7
43
356
76
target-size
target-size
2
100
93
1
1
NIL

MONITOR
8
211
358
260
NIL
target
3
1

SLIDER
7
82
357
115
population-size
population-size
0
100
27
1
1
NIL

SLIDER
7
121
356
154
mutation-rate
mutation-rate
0
1
0.0090
0.0010
1
NIL

MONITOR
217
158
287
207
NIL
best-fitness
0
1

MONITOR
9
268
358
317
best genome so far
best-genome
3
1

PLOT
9
322
187
566
Best-Fitness
Generation
Fitness
0.0
20.0
0.0
30.0
true
false

PLOT
188
322
363
568
Average Fitness
Time
Avg Fitness
0.0
20.0
90.0
100.0
true
false

MONITOR
291
158
357
207
NIL
avg-fitness
3
1

@#$#@#$#@
Elementary Genetic Algorithm

This uses the basic features of genetic algorithms to solve a simple binary matching task.  The 'target-size' sets the length of the binary sequence (zeros and ones) that must be found.  This sequence can be thought of as the optimal genetic information (genome) for a particular environment.  The 'population-size' sets the number of individuals that can try out their own genomes in that environment.  Each generation the genetic algorithm runs through the following sequence of events, much like the process of evolution takes place in a typical ecosystem.

	1.  The individuals test their genomes against the environment ('compare-genomes' in the procedures).
	2.  Individuals with better genomes are more likely to become part of the mating pool for future generations ('mate-recombine' procedure).
	3.  Individuals in the mating pool then randomly recombine their genomes with their partners to generate new offspring genomes (called seed-genomes in 'mate-recombine')
	4.  Once the seed-genomes are constructed, they undergo mutation at a rate set by the 'mutation-rate' ('mutation' procedure).
	5.  Finally, the newly constructed and mutated seed-genomes are assigned to individuals in the next generation who start the process over again.

In this simulation, the fitness is assigned by counting the number of genes (individual zeros or ones in the genome) that match the target genome.  After ranking the fitnesses, the half of the population with higher fitness are then chosen to reproduce.  These individual genomes are then randomly chosen in pairs, who then randomly decide which parent gives their gene at each locus (i.e., the location of an individual gene).  Mutation occurs by choosing a random number at each locus for each genome.  If the random number is lower than the mutation rate, then the gene is switched to its opposite state.

This was created as a preliminary excercise for the ARS-Genetic, by Thomas Hills.
thills@mail.utexas.edu
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
setup
repeat 12 [ evolve ]
compare-genomes
ask turtles [ jump fitness - target-size ]
@#$#@#$#@
@#$#@#$#@
