;;Model Easy Money

globals [updates race-over pause                 ;; controls in the procedure
         can-pay total-can-pay winner winnings   ;; these and remaining globals are defined in INFORMATION
         take takes total-take extras
         odds-0 odds-1 odds-2 odds-3 odds-4 odds-5
         bets-1 bets-2 bets-3 bets-4 bets-5 bets total-bets
         take-1 take-2 take-3 take-4 take-5
         pay-1 pay-2 pay-3 pay-4 pay-5
         pays-1 pays-2 pays-3 pays-4 pays-5
         ]

to setup
   ca crt 6
   ask turtle 0[set heading 0 set shape "wolf" set size 2 set color brown setxy -9 -24]     ;; the hare
   ask turtle 1[set heading 0 set shape "turtle" set size 2 set color black setxy -6 -24]   ;; turtle 1
   ask turtle 2[set heading 0 set shape "turtle" set size 2 set color blue setxy -3 -24]    ;;    "   2
   ask turtle 3[set heading 0 set shape "turtle" set size 2 set color gray setxy 0 -24]     ;;    "   3
   ask turtle 4[set heading 0 set shape "turtle" set size 2 set color magenta setxy 3 -24]  ;;    "   4
   ask turtle 5[set heading 0 set shape "turtle" set size 2 set color red setxy 6 -24]      ;;    "   5
   ask turtles[set label who]                  ;; turtles show ID
   default                                     ;; sets default values of bets
   set take-% 10                               ;;    "      "  value of take-%
   set updates 0                               ;; counts procedure steps
   set pause false                             ;; resets the pause procedure for a zero odds
   ask patches[
   if pxcor <= 9 and pycor <= 25[set pcolor 55]                  ;; makes the green course
   if pxcor >= -9 and pxcor <= 9 and pycor = -25[set pcolor 45]  ;; makes the yellow start line
   if pxcor = -9 and pycor >= 20[set pcolor 54.99]]              ;; makes a hazard for the hare
end

to default
   set bet-1 5                                 ;; default value of the bet
   set bet-2 15                                ;;    "      "    "  "   "
   set bet-3 10                                ;;    "      "    "  "   "
   set bet-4 25                                ;;    "      "    "  "   "
   set bet-5 20                                ;;    "      "    "  "   "
end

to go
   ifelse updates >= 50[stop][                 ;; stops the run at 50 steps
   bet-random                                  ;; makes typical values of bets
   calc                                        ;; calculates the odds
   ifelse pause = true[stop][                  ;; stops go for zero odds
   wait 0.5
   pay
   set total-bets total-bets + bets
   wait 0.5
   pays
   wait 0.5
   plot-odds
   set updates updates + 1]]                   ;; counts procedure steps
end

to bet-random                                  ;; makes typical values of bets
   if random 40 = 1 [set bet-1 (5 * (1 + random 10))]
   if random 40 = 2 [set bet-2 (5 * (1 + random 10))]
   if random 40 = 3 [set bet-3 (5 * (1 + random 10))]
   if random 40 = 4 [set bet-4 (5 * (1 + random 10))]
   if random 40 = 5 [set bet-5 (5 * (1 + random 10))]
end

to calc                                        ;; calculates the odds
   set bets bet-1 + bet-2 + bet-3 + bet-4 + bet-5
   set take bets * take-% / 100                ;; calculates the bookie's take
   set can-pay bets - take                     ;; calculates how much left to pay the winners
   set odds-1 int ((can-pay / bet-1) - 1)      ;; calculates odds for the bet
   if odds-1 = 0[set pause true]               ;; the zero odds discriminant
   set odds-2 int ((can-pay / bet-2) - 1)
   if odds-2 = 0[set pause true]
   set odds-3 int ((can-pay / bet-3) - 1)
   if odds-3 = 0[set pause true]
   set odds-4 int ((can-pay / bet-4) - 1)
   if odds-4 = 0[set pause true]
   set odds-5 int ((can-pay / bet-5) - 1)
   if odds-5 = 0[set pause true]
end

to reset                                       ;; resets the odds calculation after an odds = 0 found
   set bets 0 set take 0 set take-% 0
   set can-pay 0 set odds-1 0
   set odds-2 0 set odds-3 0
   set odds-4 0 set odds-5 0
   set pause false                             ;; resets the odds zero discriminant
end

to step-run
   go                                          ;; runs go once
   set take-% 10                               ;; sets take-% back to 10
   default                                     ;; sets bets to default values
end

to pay                                         ;; calculates the pay for each bet plus stake return
   set pay-1 (bet-1 * odds-1) + bet-1
   set pay-2 (bet-2 * odds-2) + bet-2
   set pay-3 (bet-3 * odds-3) + bet-3
   set pay-4 (bet-4 * odds-4) + bet-4
   set pay-5 (bet-5 * odds-5) + bet-5
end

to pays                                        ;; adds the bets for each turtle
   set pays-1 pays-1 + pay-1
   set pays-2 pays-2 + pay-2
   set pays-3 pays-3 + pay-3
   set pays-4 pays-4 + pay-4
   set pays-5 pays-5 + pay-5
   set total-can-pay total-can-pay + can-pay   ;; adds up all available for paying
   set takes takes + take                      ;; adds up all the bookmakers take
end

to plot-odds
   set-current-plot "betting on turtles"
   set-current-plot-pen "turtle-1"
   plot odds-1
   set-current-plot-pen "turtle-2"
   plot odds-2
   set-current-plot-pen "turtle-3"
   plot odds-3
   set-current-plot-pen "turtle-4"
   plot odds-4
   set-current-plot-pen "turtle-5"
   plot odds-5
end

to race
   ifelse race-over = true[stop][
   ask turtle random 6[ifelse color-ahead = 45[if winner = 0[set winner who win]][ ;; racing the turtles
   ifelse color-ahead = 54.99[fd 0.001 * random 2][fd 0.001 * random 3]]]]             ;; slowing the hare
end

to-report color-ahead                       ;; used to stop the turtles at the winning line and slow the hare
   report pcolor-of patch-at dx dy
end

to win                                      ;; pays the winning bets and the bookmakers take plus extras
   if winner = 1[set winnings pays-1]
   if winner = 2[set winnings pays-2]
   if winner = 3[set winnings pays-3]
   if winner = 4[set winnings pays-4]
   if winner = 5[set winnings pays-5]
   set extras total-bets - takes - winnings ;; calculates the extras
   set total-take takes + extras
   set race-over true
end





@#$#@#$#@
GRAPHICS-WINDOW
629
10
787
437
9
25
7.8
1
10
0
1

CC-WINDOW
2
380
132
441
Command Center

SLIDER
1
346
199
379
take-%
take-%
0
20
10
5
1
NIL

MONITOR
7
283
69
332
NIL
total-bets
0
1

MONITOR
260
298
340
347
NIL
total-can-pay
2
1

MONITOR
201
348
258
397
NIL
take
2
1

SLIDER
1
48
140
81
bet-1
bet-1
5
50
20
5
1
NIL

SLIDER
1
99
138
132
bet-2
bet-2
5
50
40
5
1
NIL

SLIDER
2
148
138
181
bet-3
bet-3
5
50
30
5
1
NIL

SLIDER
2
196
138
229
bet-4
bet-4
5
50
35
5
1
NIL

SLIDER
2
246
138
279
bet-5
bet-5
5
50
25
5
1
NIL

MONITOR
201
47
258
96
NIL
pay-1
2
1

MONITOR
202
97
259
146
NIL
pay-2
2
1

MONITOR
201
147
258
196
NIL
pay-3
2
1

MONITOR
201
197
258
246
NIL
pay-4
2
1

MONITOR
201
247
258
296
NIL
pay-5
2
1

BUTTON
20
10
124
43
setup
setup
NIL
1
T
OBSERVER

MONITOR
142
47
200
96
NIL
odds-1
0
1

MONITOR
142
97
200
146
NIL
odds-2
0
1

MONITOR
142
147
200
196
NIL
odds-3
0
1

MONITOR
141
197
199
246
NIL
odds-4
0
1

MONITOR
140
247
198
296
NIL
odds-5
0
1

MONITOR
332
350
393
399
NIL
extras
2
1

MONITOR
76
283
133
332
NIL
bets
0
1

MONITOR
259
47
314
96
NIL
pays-1
2
1

MONITOR
260
97
315
146
NIL
pays-2
2
1

MONITOR
260
147
315
196
NIL
pays-3
2
1

MONITOR
260
197
314
246
NIL
pays-4
2
1

MONITOR
260
247
314
296
NIL
pays-5
2
1

MONITOR
202
297
258
346
NIL
can-pay
2
1

MONITOR
261
349
330
398
NIL
takes
2
1

BUTTON
512
403
617
436
run race
race
T
1
T
OBSERVER

MONITOR
396
350
463
399
total take
total-take
2
1

PLOT
318
10
625
263
betting on turtles
updates
odds
0.0
50.0
0.0
20.0
true
true
PENS
"turtle-1" 1.0 2 -16777216 true
"turtle-2" 1.0 2 -16776961 true
"turtle-3" 1.0 2 -7566196 true
"turtle-4" 1.0 2 -65281 true
"turtle-5" 1.0 2 -65536 true

MONITOR
395
299
462
348
NIL
winnings
0
1

MONITOR
533
352
583
401
NIL
winner
0
1

MONITOR
395
401
462
450
NIL
total-bets
0
1

BUTTON
177
10
283
43
run/pause
go
T
1
T
OBSERVER

TEXTBOX
635
437
777
467
0      1      2      3      4      5

MONITOR
531
264
585
313
updates
updates
0
1

BUTTON
152
401
252
434
reset odds
reset
NIL
1
T
OBSERVER

BUTTON
267
402
373
435
step run
step-run
NIL
1
T
OBSERVER

@#$#@#$#@
MODEL EASY MONEY
---------------

This model was made to illustrate the apparent ease with which a "book" might be made to provide
a guaranteed take, that is, if the Bookmaker follows these simple rules:

          RULE 1. No bet should be accepted until the set of proposed bets of which it is a
                  member have had the procedure run to determine the ODDS for the bets and no
                  ODDS is zero. The ODDS will be zero if any bet in the set is too high to be
                  covered by the other bets in the set.

          RULE 2. If a bet is not covered the Bookmaker may reduce the take* and the take should
                  be restored upwards as soon as conditions permit. Alternatively the bet should
                  be refused at that time or perhaps put aside until it can be associated with a
                  set** of proposed bets that will cover it.***

          RULE 3. In calculating the ODDS for a bet always round down to the integer.

  *Reducing the take, even to zero, is a good option because much of the total take comes from
   the extras made by rounding down to the integer in the ODDS calculation.
 **Waiting for a set of bets to cover each other is the simplest of methods guaranteeing a take;
   also the delay involved, very little in a busy office, can probably be made acceptable to most
   gamblers.
***In this model these actions take place automatically when RESET ODDS and STEP RUN are pressed
   in sequence to clear a zero ODDS condition.

THE CALCULATIONS
----------------

To gain clarity the model has been kept simple. For example, it has been made to accept only bets
to win, (no place bets); also a set of bets, on all turtles, is made at each step of the procedure;
also ODDS less than one are not considered.

The algebra of the procedure in this model is:
 (for a SINGLE RACE with 5 turtles and 1 hare competing;
  also it uses turtle-1 where necesary as the example;
  also no one bets on the hare, a known loser.).

                    bet-1 = value of a bet made on turtle 1.
                   odds-1 = odds for "  "    "   "    "   "
                     bets = the sum of bets proposed but not yet accepted,
                          = bet-1 + bet-2 + bet-3 + bet-4 + bet-5 in this model.
               total-bets = the sum of ALL BETS ON THIS RACE TO THIS TIME, also see below.
                     take = bets x (take-% / 100), (preceeds the odds calculation.)
                  can-pay = bets - take, (preceeds the odds calculation.)
                   odds-1 = integer((can-pay/bet-1) - 1), (the - 1 allows for return of the stake;
                            also rounding down to the integer makes extras for the Bookmaker.)
                    pay-1 = (bet-1 x odds-1) + bet-1, (the + bet-1 is return of the stake for a winning bet.)                     pays-1 = addition of pay-1's.
            total-can-pay =     "     " can-pay's.
                    takes =     "     " take's.
                   extras = gain from rounding down to the integer in the odds calculation.
               total take = takes + extras.
               total-bets = winnings + total take (after the race this checks the overall arithmetic.)

USING THE MODEL
---------------

      SETUP will clear all and bring the turtles and hare to the starting gate and set to default
      values the BETS and TAKE-%
      RUN/PAUSE will illustrate 50 steps of betting with random changes of bet values; also the odds
      change in accordance with the rule that high values of bets will reduce the odds, including to
      zero, in which case the procedure automatically freezes.
      RESET ODDS will unfreeze the procedure and set take-% to zero (temporarily), so that the high
      value bet is covered.
      STEP RUN will run the procedure for one step to check that the freezing condition has been cleared
      and it will reset take-% and the bets to the default values.
      RUN/PAUSE will continue the run.
      PAUSE will pause the procedure to enable experimental manual setting of the bet and take sliders.
      RUN RACE will run the race and complete the calculations to reveal the winner plus the winnings.

      Readers of fables will no doubt note that the HARE with ID 0, lives up to its reputation.

FURTHER READING
---------------

The author has no experince of gambling or bookmaking and consequently is not in a position to make
suggestions under this heading. However the author would be pleased to receive comments and information
that would remedy this situation.

The author Derek Rush may be contacted by Email at derekrush@beeb.net         June 2003.
















@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 1.3
@#$#@#$#@
setup
@#$#@#$#@
@#$#@#$#@
