turtles-own [
    individual-strategy-list     ;; list of strategies per agent
    individual-choices-list      ;; list of choices per agent
    active-predictor             ;; reference of the used strategy
    forecast                     ;; forecast of each agent
    choice                       ;; choice according to the forecast
    strategies-scores            ;; list of scores of each strategy
    position-min-score           ;; position of the most accurate strategy
    agent-score                  ;; each agent's score

    step                         ;; loop tool
    ]

globals [count-list              ;; list of agents attending the bar
         count-list-go           ;; list of agents forecasting the bar is closed
         history                 ;; list of opening of the bar (1 if opened, 0 if not)
         strategies              ;; list of available strategies
         open-level              ;; level of opening of the bar (60 in the standard version)
         value                   ;; loop tool
         ]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;; Setting Up ;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  setting-up                ;; preparing the simulation
  patches-setup             ;; set up the patches
  turtles-setup             ;; set up the turtles
  strategy-setting          ;; create the strategy list
  distribute-strategies     ;; distribute the strategies to the agents
end

to setting-up
 ca
 set-default-shape turtles "person"
 set count-list []
 set count-list-go []
end

to patches-setup
ask patches [set pcolor blue - 3]
ask random-n-of Number-of-Agents patches [
  sprout 1 [
    set open-level (round (0.6 * count turtles))
    set agent-score 0.5
    ifelse (random-int-or-float 1.0 < .5)
      [set choice 1]
      [set choice 0]
    ]
 ]
end

to turtles-setup
ask turtles
      [ifelse choice = 1
      [set color pink]
      [set color white]]
set count-list lput count turtles with [choice = 1] count-list
set count-list-go lput ((count turtles with [active-predictor > open-level]) * 100 / count turtles) count-list-go
ifelse (last count-list > open-level) [set history 0] [set history 1]
end

to strategy-setting
set strategies n-values Strategy-Space [?]
 repeat ((length strategies) - Strategy-Space)[
 set strategies remove-item (random (length strategies)) strategies]
 set value (Memory - 1)
 repeat (20 - Memory)
 [set strategies remove-item value strategies
 set strategies remove-item ((2 * value) - 1) strategies
 set strategies remove-item ((3 * value) - 2) strategies
 set strategies remove-item ((4 * value) - 3) strategies
 set strategies remove-item ((5 * value) - 4) strategies
 set strategies remove-item ((6 * value) - 5) strategies
 set strategies remove-item ((7 * value) - 6) strategies
 set strategies remove-item ((8 * value) - 7) strategies
 set strategies remove-item ((9 * value) - 8) strategies
]
end

to distribute-strategies
ask turtles [
set strategies-scores n-values Strategies-per-agent [0]
set individual-strategy-list []
while [length  (remove-duplicates individual-strategy-list) < strategies-per-agent ]
  [set individual-strategy-list lput (random-one-of strategies) individual-strategy-list
   set individual-strategy-list remove-duplicates individual-strategy-list]
]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;; Go !!!!!!! ;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go
;;according to last weeks attendence, each strategy determines a new choice
update-individual-choices

ask turtles
[;;each agent chooses the his active-predictor, the most "accurate"
;; ie : the lowest aggregate error or score out of the vailable strategies
mapping-score-strategy

;;each agent selects an action (to go or not)
select-action]

turtles-setup
update-agent-score
update-scores-agent-list
do-plot
end

to update-individual-choices
ask turtles [
   set individual-choices-list n-values Strategies-per-agent [0]
   set step 0
   repeat (strategies-per-agent) [forecasts]]
end

to forecasts
if item step individual-strategy-list <= 19
  [TitForTat
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 19
  and item step individual-strategy-list <= 39
  [Mirror
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 39
  and item step individual-strategy-list <= 59
  [Fixed
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 69
  and item step individual-strategy-list <= 89
  [Trend
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 89
  and item step individual-strategy-list <= 99
  [OppositeTrend
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 99
  and item step individual-strategy-list <= 119
  [Trend2
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 119
  and item step individual-strategy-list <= 139
  [MovingAverage
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 139
  and item step individual-strategy-list <= 159
  [OppositeMovingAverage
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 159
  and item step individual-strategy-list <= 179
  [Trend3
   set individual-choices-list (replace-item step individual-choices-list forecast)]
if item step individual-strategy-list > 179
  and item step individual-strategy-list <= 199
  [OppositeTrend3
   set individual-choices-list (replace-item step individual-choices-list forecast)]
set step (step + 1)
end

to mapping-score-strategy
set position-min-score position (min strategies-scores) strategies-scores
set active-predictor item position-min-score individual-choices-list
end

to select-action
  ifelse (active-predictor <= open-level)
  [set choice 1]
  [set choice 0]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;; Scores ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to update-agent-score
ask turtles [
if (choice = 1 and history = 0) [set agent-score agent-score + 0.4]
if (choice = 0) [set agent-score agent-score + 0.5]
if (choice = 1 and history = 1) [set agent-score agent-score + 0.6]
]
end

to update-scores-agent-list
ask turtles [
  set step 0
  repeat (strategies-per-agent)[
       set strategies-scores replace-item step strategies-scores
        ((Lambda * (item step strategies-scores))
        + ((1 - Lambda) * (abs (last count-list - (item step individual-choices-list)))))
        set step (step + 1)]
  ]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;; Strategies ;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to TitForTat
;The same as N (up to 20) weeks ago
ifelse (length count-list < (item step individual-strategy-list) + 1)
  [set forecast (random count turtles)]
  [set forecast (item ((length count-list)
                - (item step individual-strategy-list) - 1) count-list)]
end

to Mirror
;A mirror image around 50% of N (up to 20) weeks ago
ifelse (length count-list < (item step individual-strategy-list) + 1 - 20)
  [set forecast (random count turtles)]
  [set forecast (count turtles - (item ((length count-list)
                - (item step individual-strategy-list) - 1 + 20) count-list))]
end

to Fixed
;Allways choose the same attendence (5%, 10%, 15%, 20%, 25% ... or 100%)
set forecast (round
  ((((item step individual-strategy-list) - 40 + 1) * 5 / 100) * count turtles))
end

to Trend
;A N (up to 20 weeks ago) dated 2 day trend applied to the last attendence
ifelse (length count-list < (item step individual-strategy-list) + 2 - 60)
  [set forecast (random count turtles)]
  [set forecast (last count-list +
  (item ((length count-list) - (item step individual-strategy-list) - 1 + 60) count-list)
  - (item ((length count-list) - (item step individual-strategy-list) - 2 + 60) count-list))
  if forecast > count turtles [set forecast count turtles]
  if forecast < 0 [set forecast 0]]
end

to OppositeTrend
;A N (up to 20 weeks ago) dated 2 day opposite trend applied to the last attendence
ifelse (length count-list < (item step individual-strategy-list) + 2 - 80)
  [set forecast (random count turtles)]
  [set forecast (last count-list -
  (item ((length count-list) - (item step individual-strategy-list) - 1 + 80) count-list)
  + (item ((length count-list) - (item step individual-strategy-list) - 2 + 80) count-list))
  if forecast > count turtles [set forecast count turtles]
  if forecast < 0 [set forecast 0]]
end

to Trend2
;A N (up to 20 weeks ago) dated 2 day (3 day spaced) trend applied to the last attendence
ifelse (length count-list < (item step individual-strategy-list) + 4 - 100)
  [set forecast (random count turtles)]
  [set forecast (last count-list +
  (item ((length count-list) - (item step individual-strategy-list) - 1 + 100) count-list)
  - (item ((length count-list) - (item step individual-strategy-list) - 4 + 100) count-list))
  if forecast > count turtles [set forecast count turtles]
  if forecast < 0 [set forecast 0]]
end

to MovingAverage
;A N (up to 20 weeks ago) 5 day Moving average
ifelse (length count-list < (item step individual-strategy-list) + 5 - 120)
  [set forecast (random count turtles)]
  [set forecast (round (
0.2 * (item ((length count-list) - (item step individual-strategy-list) - 1 + 120) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 2 + 120) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 3 + 120) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 4 + 120) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 5 + 120) count-list)
  ))]
end

to OppositeMovingAverage
;A N (up to 20 weeks ago) opposite 5 day Moving average
ifelse (length count-list < (item step individual-strategy-list) + 5 - 140)
  [set forecast (random count turtles)]
  [set forecast (Count turtles - round (
0.2 * (item ((length count-list) - (item step individual-strategy-list) - 1 + 140) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 2 + 140) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 3 + 140) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 4 + 140) count-list)
+ 0.2 * (item ((length count-list) - (item step individual-strategy-list) - 5 + 140) count-list)
  ))]
end

to Trend3
;A N (up to 20 weeks ago) dated 2 day relative trend applied to the last attendence
ifelse (length count-list < (item step individual-strategy-list) + 2 - 160)
  [set forecast (random count turtles)]
  [set forecast (last count-list +
  (((item ((length count-list) - (item step individual-strategy-list) - 1 + 160) count-list)
  - (item ((length count-list) - (item step individual-strategy-list) - 2 + 160) count-list)) / last count-list)
  * 100)
  set forecast round (forecast)
  if forecast > count turtles [set forecast count turtles]
  if forecast < 0 [set forecast 0]]
end

to OppositeTrend3
;A N (up to 20 weeks ago) dated 2 day relative trend applied to the last attendence
ifelse (length count-list < (item step individual-strategy-list) + 2 - 160)
  [set forecast (random count turtles)]
  [set forecast (last count-list -
  (((item ((length count-list) - (item step individual-strategy-list) - 1 + 160) count-list)
  - (item ((length count-list) - (item step individual-strategy-list) - 2 + 160) count-list)) / last count-list)
  * 100)
  set forecast round (forecast)
  if forecast > count turtles [set forecast count turtles]
  if forecast < 0 [set forecast 0]]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;; Plotting ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to do-plot
 locals [Gone Averag
         min-score max-score avg-score
         ]

 set Gone count turtles with [Choice = 1]
 set-current-plot "People going to El Farol"
 set-plot-y-range 0 count turtles
 set-current-plot-pen "Present"
 set-plot-y-range 0 count turtles
 plot (Gone)
 set Averag mean count-list
 set-current-plot-pen "Mean"
 plot (Averag)
 set-current-plot "Attendance-Histogram"
 set-plot-pen-mode 1
 set-plot-x-range 0 count turtles
 auto-plot-on
 histogram-list count-list

 set min-score min values-from turtles [agent-score]
 set max-score max values-from turtles [agent-score]
 set avg-score mean values-from turtles [agent-score]
 set-current-plot "Agent's scores"
 set-current-plot-pen "min-score"
 plot (min-score / (length count-list))
 set-current-plot-pen "max-score"
 plot (max-score / (length count-list))
 set-current-plot-pen "avg-score"
 plot (avg-score / (length count-list))
end
@#$#@#$#@
GRAPHICS-WINDOW
209
10
414
206
6
5
15.0
1
10
1
1
1
0

CC-WINDOW
2
606
749
722
Command Center

SLIDER
7
128
203
161
Strategies-per-agent
Strategies-per-agent
3
12
6
1
1
NIL

BUTTON
140
10
202
43
Go
Go
T
1
T
OBSERVER
T
NIL

PLOT
419
10
719
186
People going to El Farol
Week
Attendence
0.0
100.0
0.0
100.0
true
false
PENS
"Present" 1.0 0 -16776961 true
"Mean" 1.0 0 -65281 true

MONITOR
8
262
64
311
Present
count turtles with [choice = 1]
0
1

MONITOR
65
262
120
311
Mean
mean count-list
0
1

MONITOR
8
211
176
260
History
(list item ((length count-list) - 8) count-list\n item ((length count-list) - 7) count-list\n item ((length count-list) - 6) count-list\n item ((length count-list) - 5) count-list\n item ((length count-list) - 4) count-list\n item ((length count-list) - 3) count-list\n item ((length count-list) - 2) count-list\n item ((length count-list) - 1) count-list)
3
1

MONITOR
121
262
176
311
St Dev
standard-deviation count-list
2
1

BUTTON
6
10
68
43
NIL
Setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
7
93
203
126
Strategy-Space
Strategy-Space
6
200
200
1
1
NIL

BUTTON
71
10
136
43
Go Once
Go
NIL
1
T
OBSERVER
T
NIL

SLIDER
110
163
203
196
Lambda
Lambda
0
0.9
0.5
0.1
1
NIL

MONITOR
65
314
120
363
<= 60%
100 - mean count-list-go
0
1

SLIDER
7
59
203
92
Number-of-Agents
Number-of-Agents
0
143
100
1
1
NIL

PLOT
419
188
719
364
Agent's scores
Week
Scores
0.0
100.0
0.45
0.55
true
false
PENS
"min-score" 1.0 0 -16776961 true
"avg-score" 1.0 0 -11352576 true
"max-score" 1.0 0 -65536 true

PLOT
210
210
413
363
Attendance-Histogram
Present
Turtles
0.0
100.0
0.0
100.0
true
false

SLIDER
7
163
99
196
Memory
Memory
5
20
20
1
1
NIL

MONITOR
8
314
63
363
> 60%
100 - mean count-list-go
0
1

@#$#@#$#@
WHAT IS IT?
-----------
El Farol is a bar in Santa Fe, New Mexico, at which a band plays Irish music on Thursday evenings. The bar is enjoyable only if it is not too crowded (say more than 60% of the agents). Trying to decide whether or not to go to El Farol on Thursday nights is what Arthur's, 1994, "El Farol bar problem" is about.

Each agent (out of "Number-of-Agents") has to decide whether they should go or not by making an independent decision. In other words, each agent enjoys going to El Farol to listen to the music, but not if the bar is going to be too crowded.

In order to optimise ones utility, each agent has to try and predict what everybody else will do. However, the problem is set up so that any model of the problem that is shared by most of the agents is what Schelling refers to as a self-negating prophecy:
"If all believe few will go, all will go and vice versa".


HOW IT WORKS
------------
Arthur modelled this situation by randomly giving each agent "Strategies-per-agent" potentially suitable predictors, given past data (e.g. the past "D" weeks attendance figures, called "Memory"). For example, the number of people turning up over the past 10 weeks might be: 15, 67, 84, 34, 45, 76, 40, 56, 23 and 35. Each person then independently employs a prediction procedure to estimate how many people will appear at the bar in the coming week. Typical predictors might be for example:

-the same number as last week (35),
-a mirror image around 50% of last week's attendance,
-a (rounded) average of attendances over the past four weeks (39).
-the same as 2 weeks ago (2-period cycle predictor),
-etc. ...

Each week each agent evaluates these models against the past data and chooses the one that was the best predictor on this data and then uses this to predict the number who will go this time (the active predictor). It will go if this prediction is less than 60% and not if it is more than 60%.

Because agents have "Strategies-per-agent" different suitable predictors in their sets, some will turn up at the bar, while others will not. A new attendance figure is available and everyone updates the accuracies of all their predictors.


HOW TO USE IT
-------------
1 - The Sliders
---------------
The "Number-of-Agents" slider is straight-forward. It determines the number of agents in the game (set at 100 in the standard game).

The "Strategy-Space" slider will determine the size "S" of possible strategies (S = 6 to 200, set at 200).

The "Strategies-per-agent" slider will distribute randomly the strategy soups (3, 4, ..., or 12 out of S pre-designed ones, set at 6) to each agent.

The "Memory" slider will determine the length of the past data the agents use (5 to 20, set at 20).

The "Lambda" slider is used to compute the performance of each strategy. A low "Lambda" uses gives more importance to current performance while a high "Lambda" uses the past. (0 to 0.9).


2 - Buttons
-----------
Two buttons, "SETUP" and "GO", control execution of the model.

The "SETUP" button resets the system, preparing the model to be run. The number of strategies per agent is determined by the slider.

The "GO" button, a forever button, will then run the model.

The "GO ONCE" button is the same as GO except the agents only take one step.

3 - Plot
--------
People Going to El Farol - The total attendance at the bar and the average vs. time.

Agent's scores - The best, worst and average score vs. time.

Attendance-Histogram - Histogram of the attendance at the bar.

**The agents have a binary choice, to go or to stay, respectively coloured in pink and white.


STRATEGIES
----------
The strategies used in this simulation are the following.

TitForTat: This family of strategies predicts next week's attendance by using the same as N weeks ago, with N from 0 to "Memory" (20 possibilities).
It could be for example N = 0 for last weeks attendance or N = 4 for 5 weeks ago's one.
Mirror: This is a family of strategies using a mirror image around 50% of N (up to 20) weeks ago as prediction.

Fixed: The "Fixed" strategy always chooses the same attendance (5%, 10%, 15%, 20%, 25% ... or 100%).

Trend: A N (up to 20) dated 2 day trend applied to the last attendance.

OppositeTrend: A N (up to 20) dated 2 day opposite trend applied to the last attendance.

Trend2: A N (up to 20) dated 2 day (3 day spaced) trend applied to the last attendance.

MovingAverage: A N (up to 20) 5 day Moving average.

OppositeMovingAverage: A N (up to 20) opposite 5 day Moving average.

Trend3: A N (up to 20) dated 2 day relative trend applied to the last attendance.

OppositeTrend3: A N (up to 20) dated 2 day relative trend applied to the last attendance.


THINGS TO NOTICE
----------------
In the case of absence of collusion or prior communication between agents, this model, of limited resources, shows how unstable behaviour is yielded in this repeated game, and two interesting patterns are revealed.

First, the number of people who attend the bar fluctuates around an average level of 60%, with on average 60% predicting not to go and 40% to go, which constitutes the Nash Equilibria, which is selected through the computer simulations. It is learning that allows the bounded-rational agents to select this equilibrium strategy.

Second, the dynamics seem random, despite that no random component determines the dynamics of how many people go. In fact, although at a microscopic level each agent is applying a different predictor at any one time, with varying degrees of success, when viewed globally, they seem pretty indistinguishable. The only random part of Arthur's model is the setting-up.
Therefore once the game is set the output is completely deterministic since the rule update is stable.


CREDITS AND REFERENCES
----------------------
This model was based upon Brian Arthur's article:
Arthur, Brian, (1994), "Inductive Reasoning and Bounded Rationality: The El Farol Bar Problem", American Economic Review, 84(2), pp.406-411.

Information can be found on the web at:
http://www.santafe.edu/arthur/Papers/El_Farol.html


AUTHOR
------
The Author, Mark Garofalo, may be contacted by Email at: mark_garofalo@hotmail.com
http://www.markgarofalo.com/
January 2004.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.1pre3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
