turtles-own [
  flockmates         ;; agentset of nearby turtles
  nearest-neighbor   ;; closest one of our flockmates
]
globals [
  mate_sizes          ;; list of number of flockmates for each turtle
  group_lists        ;; list of turtle ids in each extended group
  group_sizes        ;; list of number in each extended group
  plot_legend
]

to setup
  ca
  cct population
    [ rt random-float 360.0
      setxy (random-float screen-size-x) (random-float screen-size-y)
      set color round (heading / 360 * 14) * 10 + 5
    ]
  histograms
  set-plot-pen-mode 1
end

to go
  ;; the model runs faster if we don't update the screen
  ;; until after all turtles have moved
  no-display
  ask turtles [ flock ]
  group-set  ;; new
  display
  histograms ;;new
 end

to histograms
  ifelse toggle_plot
     [ ;set-current-plot "Histogram of number of flock mates"
       ;set-plot-xlabel "Number of flockmates (neighbots)"
       set plot_legend "Histogram of number of flock mates"
       set-plot-x-range 0 population
       set-plot-y-range 0 population
       set-histogram-num-bars population
       histogram-list mate_sizes
     ]
     [ ;set-current-plot "Histogram of direction"
       ;set-plot-xlabel "Direction of turtle"
       set plot_legend "Histogram of direction (0 & 360 is up)"
       set-plot-x-range 0 360
       set-plot-y-range 0 population
       set-histogram-num-bars 18
       histogram-from turtles [heading]
     ]
end
to group-set ;; find list of groups in flockmates  - returns: mate_sizes group_lists group_sizes
locals  [ temp
          fm      ;; temp list
          t-list  ;; list of who = [ 0 1 2 ... ]
          a-list  ;; list of "who" that is in flockmate agentsets
          i-list  ;; list of agents that have been inserted
          g-list  ;; list of agents in each group
          t-loop
          a-loop
          flocktmp;; list of flockmates for current agent
          n-list
          again   ;; flag to continue seach within a group for new members
         ]

set t-list []
set g-list []
set group_lists []
set i-list []
set t-list values-from turtles [who]
set a-list []

repeat population [set a-list fput [] a-list] ; set up list with one entry per agent (t-list is convienent)
foreach t-list ;; create a list equivalent to agentset flockmate - called "a-list"
 [ set fm value-from (turtle ?) [ flockmates] ;; returns agentset (not list as suggested in manual)
   set temp values-from fm [who] ; list of flockmates for ?
   if length temp > 0 [set a-list replace-item ? a-list (sentence ? temp)]
 ]

set n-list filter [length item ? a-list > 0] t-list

while [ length n-list > 0 ]
[

set g-list item (item 0 n-list) a-list
set again true

while [again]
[ set again false

foreach t-list  ;; Loop over all agents
 [ set t-loop ?
   set flocktmp item t-loop a-list
   if (not member? t-loop i-list and not empty? flocktmp)
     [ foreach flocktmp                   ;; insert flock mates into a-list
         [ set a-loop ?
           if (not member? a-loop i-list and length item a-loop a-list > 1 and member? a-loop g-list)  ;; check if this agent's flockmates have been inserted?
           [ set g-list replace-item (position a-loop g-list) g-list (item a-loop a-list) ;; replace with list
             set i-list lput a-loop i-list
             set g-list reduce [sentence ?1 ?2 ] g-list  ;; flattens list of lists
             set g-list sort remove-duplicates g-list    ;; remove duplicates
             set again true
           ]
         ]  ;; end foreach over g-list - at end - have a complete group list
     ]
 ] ;; end loop over t-list and if test
;print g-list
] ;; loop over while again - continue until no new agents added

 set group_lists lput g-list group_lists
 set temp reduce [sentence ?1 ?2 ] group_lists
 set n-list filter [not member? ? temp] t-list
 set n-list filter [length item ? a-list > 0] n-list
] ; loop over while n-list
; print list "group_lists" group_lists
set group_sizes sort map [ length ? ] group_lists
;print list "group sizes:" group_sizes
set mate_sizes map [ length item ? a-list] t-list
end

to flock  ;; turtle procedure
  find-flockmates
  if any? flockmates
    [ find-nearest-neighbor
      ifelse distance nearest-neighbor < minimum-separation
        [ separate ]
        [ align
          cohere ] ]
  fd 1.0
  set color hsb (heading / 360) 1. 1.
end

to find-flockmates  ;; turtle procedure
  set flockmates (turtles in-radius vision) with [self != myself]
end

to find-nearest-neighbor ;; turtle procedure
  set nearest-neighbor min-one-of flockmates [distance myself]
end

;;; SEPARATE

to separate  ;; turtle procedure
  turn-away (heading-of nearest-neighbor) max-separate-turn
end

;;; ALIGN

to align  ;; turtle procedure
  turn-towards average-flockmate-heading max-align-turn
end

to-report average-flockmate-heading  ;; turtle procedure
  ;; We can't just average the heading variables here.
  ;; For example, the average of 1 and 359 should be 0,
  ;; not 180.  So we have to use trigonometry.
  report atan mean values-from flockmates [sin heading]
              mean values-from flockmates [cos heading]
end

;;; COHERE

to cohere  ;; turtle procedure
  turn-towards average-heading-towards-flockmates max-cohere-turn
end

to-report average-heading-towards-flockmates  ;; turtle procedure
  ;; "towards myself" gives us the heading from the other turtle
  ;; to me, but we want the heading from me to the other turtle,
  ;; so we add 180
  report atan mean values-from flockmates [sin (towards myself + 180)]
              mean values-from flockmates [cos (towards myself + 180)]
end

;;; HELPER PROCEDURES

to turn-towards [new-heading max-turn]  ;; turtle procedure
  turn-at-most (subtract-headings new-heading heading) max-turn
end

to turn-away [new-heading max-turn]  ;; turtle procedure
  turn-at-most (subtract-headings heading new-heading) max-turn
end

;; turn right by "turn" degrees (or left if "turn" is negative),
;; but never turn more than "max-turn" degrees
to turn-at-most [turn max-turn]  ;; turtle procedure
  ifelse abs turn > max-turn
    [ ifelse turn > 0
        [ rt max-turn ]
        [ lt max-turn ] ]
    [ rt turn ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was originally created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo (and was changed in the process to
; be an alternate version of the model) as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2002.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Flocking model.
; http://ccl.northwestern.edu/netlogo/models/Flocking.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
250
10
704
485
18
18
12.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
674
713
769
Command Center
0

BUTTON
31
10
112
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
121
11
202
44
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
10
52
233
85
population
population
1
200
50
1
1
NIL

SLIDER
5
181
238
214
max-align-turn
max-align-turn
0.0
20.0
5.0
0.25
1
degrees

SLIDER
5
215
238
248
max-cohere-turn
max-cohere-turn
0
20
3.0
0.25
1
degrees

SLIDER
5
249
238
282
max-separate-turn
max-separate-turn
0
20
1.5
0.25
1
degrees

SLIDER
10
99
233
132
vision
vision
0
10
3.0
0.5
1
patches

SLIDER
10
133
233
166
minimum-separation
minimum-separation
0
5
1.0
0.25
1
patches

PLOT
5
489
412
660
NIL
NIL
number in bin
0.0
25.0
0.0
20.0
true
true
PENS
"default" 1.0 1 -12225100 false

SWITCH
6
421
125
454
Toggle_plot
Toggle_plot
1
1
-1000

MONITOR
1
330
245
379
Group sizes
group_sizes
0
1

MONITOR
6
454
245
503
Plot being displayed
plot_legend
0
1

@#$#@#$#@
21 OCT 2003

*** The standard flocking example (Flocking) supplied with Ver 1.3 of NetLogo is modified to include changes:

1. Turtles are now color-coded by the direction they are heading.  This makes the flocking behavior much more obvious to the eye: all the agents in a flock have a similar color.

2. A subroutine GROUP-SET is added that finds from the agentset FLOCKMATES the lists of turtles in extended groups implied by the individual neighbor lists.  For example, if your neighbor has a neighbor that is not in your list, then the three of you will be in the same extended group list.

To use the subroutine, just copy it into your program, define GLOBAL variables mate_size, group_lists and group_sizes, call GROUP-SET appropriately, provide an agentset with the neighbor lists (you can rename FLOCKMATES to whatever is your neighbor list).  You may wish to comment out mate_size if you do not require it - it is included here for output.

The output of GROUP-SET is three lists:
 - group-lists: a list of lists of the form:
 [ (list of turtle numbers in a extended group) (list of turtle number in another extended group) ...]
The essential property is that all the sets of extended groups are disjoint in any combination (they have no common members). The order of the groups is not meaningful, and an extended group is only 2 or more turtles.
 - group-sizes: an ordered list of the group sizes (you may wish to remove the sorting to retain correspondence between the two lists).
 - mate_size: the number of neighbors for each turtle

Despite the calculation being at worst an N^2 problem (where N is the number of turtles), the code typically does not consider all N^2 combinations. But you will notice a speed penalty for large N.

3. Three diagnostics are included:
 - Histogram of the directions of turtles
 - Histogram of the numbers of flockmates
 - Monitor of the group sizes

*** THINGS TO OBSERVE ***:
- look at the histogram of the directions to follow the coherence formation and the different extended groups that self-organize
- compare the histogram of the number of flockmates and the group size: this illustrates how the extended group is an emergent property of the system
- examine the dynamics of these diagnostics as you change the parameters (like the vision range) during a run. How quickly do the extended groups form, breakup, reform?

*** AN EXAMPLE: What determines what final direction of the flock after a some time?

If you repeat a simulation over and over, it is difficult to predict the final direction from visually observing the initial distribution of directions in the graphics. This illustrates the sensitivity of the final solution (the direction of an entire flock) on details of the initial distribution.

* Try this experiment:
 - Toggle the plot to show the histogram of direction.
 - Set vision to the maximum (10) and the minimum distance to minumum (0).
 - Click SETUP, and then click GO. After the flocks align, click Go to stop.
 - Repeat the last step a couple of times.
Note how the flock quickly aligns to one direction (observe in the histogram all individuals will be in one or two bins) because everyone sees almost everyone else (observe the extended group size). Can you predict from the initial histogram what will be the final direction?

* Now try this experiment:
 - Repeat the first three steps from the previous example.
 - While running, slide the mimimun distance to both extremes.  Observe the histogram.
When the minimum distance is maximum, every one tries to avoid everyone else and the system will start to randomize. But note how the distribution in the histogram takes time to populate other angles.  When you reduce the minimum distance to zero, the flock will quickly reorient to the original direction of the flocks.  How long do you have to let the simulation run before it "forgets" the previous collective angle? Remember that the turtles have no individual memory except their direction.  The answer might surprise you.  Can you predict how long is long enough by looking at the histogram?

*** A suggestion: To better see the formation and interactions of flock mates, try reducing the speed of the agents by replacing "fd 1.0" with "fd 0.5"

Norman Johnson
Theoretical Division
Los Alamos National Laboratory
norman@lanl.gov

The following is the standard introduction:
-------------------------------------------

WHAT IS IT?
-----------
This model is an attempt to mimic the flocking of birds.  (The resulting motion also resembles schools of fish.)  The flocks that appear in this model are not created or led in any way by special leader birds.  Rather, each bird is following exactly the same set of rules, from which apparent flocks emerge.

The birds follow three rules: "alignment", "separation", and "cohesion".  "Alignment" means that a bird tends to turn so that it is moving in the same direction that nearby birds are moving.  "Separation" means that a bird will turn to avoid another bird which gets too close.  "Cohesion" means that a bird will move towards other nearby birds (unless another bird is too close).  When two birds are too close, the "separation" rule overrides the other two, which are deactivated until the minimum separation is achieved.

The three rules affect only the bird's heading.  Each bird always moves forward at the same constant speed.


HOW TO USE IT
-------------
First, determine the number of birds you want in the simulation and set the POPULATION slider to that value.  Press SETUP to create the birds, and press GO to have them start flying around.

The default settings for the sliders will produce reasonably good flocking behavior.  However, you can play with them to get variations:

Three TURN-ANGLE sliders control the maximum angle a bird can turn as a result of each rule.

VISION is the distance that each bird can see 360 degrees around it.


THINGS TO NOTICE
----------------
Central to the model is the observation that flocks form without a leader.

There are no random numbers used in this model, except to position the birds initially.  The fluid, lifelike behavior of the birds is produced entirely by deterministic rules.

Also, notice that each flock is dynamic.  A flock, once together, is not guaranteed to keep all of its members.  Why do you think this is?

After running the model for a while, all of the birds have approximately the same heading.  Why?

Sometimes a bird breaks away from its flock.  How does this happen?  You may need to slow down the model or run it step by step in order to observe this phenomenon.


THINGS TO TRY
-------------
Play with the sliders to see if you can get tighter flocks, looser flocks, fewer flocks, more flocks, more or less splitting and joining of flocks, more or less rearranging of birds within flocks, etc.

You can turn off a rule entirely by setting that rule's angle slider to zero.  Is one rule by itself enough to produce at least some flocking?  What about two rules?  What's missing from the resulting behavior when you leave out each rule?

Will running the model for a long time produce a static flock?  Or will the birds never settle down to an unchanging formation?  Remember, there are no random numbers used in this model.


EXTENDING THE MODEL
-------------------
Currently the birds can "see" all around them.  What happens if birds can only see in front of them?

Is there some way to get V-shaped flocks, like migrating geese?

What happens if you put walls around the edges of the screen that the birds can't fly into?

Can you get the birds to fly around obstacles in the middle of the screen?

What would happen if you gave the birds different velocities?  For example, you could make birds that are not near other birds fly faster to catch up to the flock.  Or, you could simulate the diminished air resistance that birds experience when flying together by making them fly faster when in a group.

Are there other interesting ways you can make the birds different from each other?  There could be random variation in the population, or you could have distinct "species" of bird.


NETLOGO FEATURES
----------------
Notice the need for special procedures for subtracting two headings, and for averaging groups of headings.  Just subtracting the numbers, or averaging the numbers, doesn't give you the results you'd expect, because of the discontinuity where headings wrap back to 0 once they reach 360.


CREDITS AND REFERENCES
----------------------
This model is inspired by the Boids simulation invented by Craig Reynolds.  The algorithm we use here is roughly similar to the original Boids algorithm, but it is not the same.  The exact details of the algorithm tend not to matter very much -- as long as you have alignment, separation, and cohesion, you will usually get flocking behavior resembling that produced by Reynolds' original model.  Information on Boids is available at http://www.red3d.com/cwr/boids/.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Flocking model. http://ccl.northwestern.edu/netlogo/models/Flocking. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 30 30 240

circle 2
false
0
Circle -7566196 true true 16 16 270
Circle -16777216 true false 46 46 210

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7566196 true true 30 30 240

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11030470 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11030470 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11030470 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6525626 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11030470 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11030470 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11030470 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11030470 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11030470 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0pre1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
