patches-own [ elevation        ; elevation of the patch
            ]
globals     [ highest          ; maximum patch elevation
              lowest           ; minimum patch elevation
              d                ; distance
              direction        ; direction in which walker is looking
              height           ; elevation where walker is
              walks            ; number of climbs and descents
              x                ; distance input to plot
              ]

to setup-patches                                 ; creates a terrain of
   ca                                            ; random elevations
   ask patches
     [set elevation (random-int-or-float 10000)               ; A slider variable 'smoothness'
     ]                                           ; sets how rough the terrain becomes
   repeat smoothness[diffuse elevation 1.0 ]     ; during diffusion of 'elevation'.
   ask patches
     [set pcolor scale-color brown
      elevation 10000 1000                       ; colors general ground
     ]
   set highest max values-from patches [elevation]
   set lowest min values-from patches [elevation]
   ask patches
     [if (elevation > (highest - 400))           ; colors hills
     [set pcolor grey
     ]
      if (elevation > (highest - topping))       ; colors tops of hills
     [set pcolor 7
     ]
      if (elevation < (lowest + 300))            ; colors valleys
     [set pcolor green
     ]
     ]                                           ; below:- colors one highest patch
                                                 ; in each quadrant
    set pcolor-of max-one-of patches with [(pxcor > 0) and (pxcor != 50)
        and (pycor > 0) and (pycor != 50)][elevation] 7.1
    set pcolor-of max-one-of patches with [(pxcor < 0) and (pxcor != -50)
        and (pycor > 0) and (pycor != 50)][elevation] 7.1
    set pcolor-of max-one-of patches with [(pxcor > 0) and (pxcor != 50)
        and (pycor < 0) and (pycor != -50)][elevation] 7.1
    set pcolor-of max-one-of patches with [(pxcor < 0) and (pxcor != -50)
        and (pycor < 0) and (pycor != -50)][elevation] 7.1
end

to setup
   clear-marks
   clear-plot
   ct
   crt 1
   ask turtle 0
     [set color blue
      setxy 0 0
      set heading random-int-or-float 360
      set direction heading
      set height elevation-of patch-here
      set d 0
      set walks 0
      set x 0
     ]
end

to go
   if (walks = 8) or (height = 0)[stop]
   if (walks = 0) or (walks = 2) or (walks = 4) or (walks = 6)[climb]
   if (walks = 1) or (walks = 3) or (walks = 5) or (walks = 7)[descend]
end

to climb
   ask turtle 0
     [ifelse (d > 140)                           ; limits distance of look
      or (abspx = 50) or (abspy = 50)            ; and stops wrap
     [increment-heading                          ; increments direction of look
     ]
     [ifelse (color-ahead = 7.1)                 ; looks for hilltop patch color
     [walk print height wait 1                   ; walks towards patch of that color
      set pcolor white set walks walks + 1       ; also leaves mark on arrival
     ]
     [increment-d                                ; increments range of look
     ]
     ]
     ]
end

to walk
   repeat d [fd 1 set d (d - 1) increment-x do-plot wait 0.2]
end

to descend
   ask turtle 0
     [ifelse (d > 140)                           ; limits distance of look
      or (abspx = 50) or (abspy = 50)            ; and stops wrap
     [increment-heading                          ; increments direction of look
     ]
     [ifelse (color-ahead = green) and (d <= 50) ; looks for green of a valley
     [walk print height wait 1                   ; up to distance 50
      set pcolor yellow set walks walks + 1      ; also leaves mark on arrival
     ]
     [increment-d
     ]
     ]
     ]
end

to increment-heading                             ; rotates direction of look (heading) by 0.5 degree steps
   set d 0                                       ; (sets d 0 to start new distance search)
   set heading heading + 0.5
   set direction heading                         ; reports direction to monitor
end

to-report abspx
   report abs pxcor-of patch-at (d * dx) (d * dy); reports absolute x cordinates
end

to-report abspy
   report abs pycor-of patch-at (d * dx) (d * dy); reports absolute y coordinates
end

to increment-d
   set d d + 1
end

to-report color-ahead                            ; the guidance "radar"
   report pcolor-of patch-at (d * dx) (d * dy)
end

to increment-x                                    ; sets up 'x' plot
   set x x + 1
end

to do-plot
   set height elevation-of patch-here             ; defines 'y' plot
   if (walks = 0) or (walks = 2) or (walks = 4) or (walks = 6)[set-plot-pen-color red]
   if (walks = 1) or (walks = 3) or (walks = 5) or (walks = 7)[set-plot-pen-color green]
   plotxy x height
end

to clear-marks
   ask patches
     [if (pcolor = white) [set pcolor 7.1]
      if (pcolor = yellow) [set pcolor green]
     ]
end























@#$#@#$#@
GRAPHICS-WINDOW
374
10
788
445
50
50
4.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
497
797
592
Command Center
0

SLIDER
378
450
585
483
smoothness
smoothness
15.0
30.0
30.0
0.5
1
NIL

BUTTON
187
42
274
90
setup / clear
setup
NIL
1
T
OBSERVER
T
NIL

MONITOR
282
42
360
91
highest hill
highest
0
1

MONITOR
12
42
92
91
lowest valley
lowest
0
1

BUTTON
100
42
180
90
setup terrain
setup-patches
NIL
1
T
OBSERVER
T
NIL

SLIDER
589
449
777
482
topping
topping
200
350
250
1
1
NIL

MONITOR
68
321
129
370
distance
d
0
1

PLOT
3
92
359
319
height  and cross-section of the terrain
distance
height
0.0
50.0
4000.0
6500.0
true
false

TEXTBOX
58
118
148
148
    red climb\ngreen descent

BUTTON
309
323
364
421
start
go
T
1
T
OBSERVER
T
NIL

MONITOR
4
372
63
421
direction
direction
1
1

MONITOR
4
320
64
369
height
height
0
1

MONITOR
69
373
128
422
NIL
walks
0
1

@#$#@#$#@
HILLWALKER is a model inspired by the Tutorial Model 3 and it uses the random terrain creating code
of that model, but a little modified. The sliders 'smoothness' and 'topping' plus the 'setup terrain'
button create the terrain of green valleys, brown hills and grey tops.

An offwhite color 7 patch appears as a 'topping' on some hills to simulate greater height --- snow if
you like. Also in each quadrant of the screen there is a single patch of color 7.1, usually "invisible"
being impossible to see if against offwhite.
The "invisible" patch is used as the peak that the walker sees and aims for during the climb but first
the walker must look around and across the terrain to find it. The walker's search begins from a random
heading but the look around proceeds clockwise.
The 'climb' procedure uses sub-procedures 'increment-heading', 'increment-distance', 'color-ahead'
and 'walk'.
The 'setup / clear' button followed by the 'start' button runs these procedures which starts with the
walker in the center of the display. The walker will walk to the top of one hill and mark the hilltop
white before descending to a nearby valley where it marks yellow. The walking continues to three more
hilltops and valleys. Each sector counts as a walk, 8 in all. The descent to a valley procedure is
similar to the climb but looks for a valley closeby up to distance 50.
The plot shows the profile of the climbs in red and the descents in green.
If a terrain proves to be particularly interesting, the walker may be returned to the display center
and the model may be cleared of plots and marks by the 'setup / clear' button. Then the model may be run
again using the 'start' button.
The model produces a print of the heights of the 8 marks in the Command Center.
The work was first done to investigate a "guidance procedure" for NetLogo. The procedures include code
to avoid wrap. It is possible that models such as this may have some value in sharing procedures among
the NetLogo Users and for this reason it is offered.

The Author Derek Rush may be contacted by Email at derekrush@beeb.net   10th June 2002





@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7500403 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7500403 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7500403 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7500403 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7500403 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7500403 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7500403 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7500403 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7500403 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -1184463 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7500403 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7500403 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7500403 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7500403 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7500403 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7500403 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7500403 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

circle
false
0
Circle -7500403 true true 34 34 230

person
false
0
Circle -7500403 true true 155 20 63
Rectangle -7500403 true true 158 79 217 164
Polygon -7500403 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7500403 true true 216 83 267 123 248 143 215 107
Polygon -7500403 true true 167 163 145 234 183 234 183 163
Polygon -7500403 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7500403 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7500403 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7500403 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8630108 true false 195 75 195 120 240 120 240 75
Polygon -8630108 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7500403 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8630108 true false 90 210 105 225 120 210
Polygon -8630108 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7500403 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8630108 true false 210 210 195 225 180 210
Polygon -8630108 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7500403 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6459832 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6459832 true true 87 80 79 55 76 79
Polygon -6459832 true true 81 75 70 58 73 82
Polygon -6459832 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6459832 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6459832 true true 116 140 114 189 105 137
Rectangle -6459832 true true 109 150 114 192
Rectangle -6459832 true true 111 143 116 191
Polygon -6459832 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6459832 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6459832 true true 214 134 203 168 192 148
Polygon -6459832 true true 204 151 203 176 193 148
Polygon -6459832 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6459832 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6459832 true true 201 99 214 69 215 99
Polygon -6459832 true true 207 98 223 71 220 101
Polygon -6459832 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6459832 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6459832 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6459832 true true 48 143 58 141
Polygon -6459832 true true 46 136 68 137
Polygon -6459832 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6459832 true true 38 138 66 149
Polygon -6459832 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6459832 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 3.0pre5
@#$#@#$#@
setup-patches
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
