to go
ask turtles
 [if (not (who = 0 or who = (numdots - 1))) ;don't move the endpoint dots
   [set heading towardsxy-nowrap ;move each dot toward the dot before it and the dot after it
               (mean (list (xcor-of (turtle (who - 1))) (xcor-of (turtle (who + 1)))))
               (mean (list (ycor-of (turtle (who - 1))) (ycor-of (turtle (who + 1)))))
   set heading (heading + (random-int-or-float noise) - (random-int-or-float noise)) ;also add some noise
   fd movement-rate
    if (pcolor-of patch-here = green) ;uh oh, the dot moved on to an obstacle
      [back movement-rate ;so let's take back the move
       if pcolor-of patch-here = green [back movement-rate]] ;move back some more if needed
    ]]
end

to simulated-annealing
 set noise 360 ;start with a high level of noise
 repeat 359
   [if mouse-down? [stop]
   repeat (11 - speed-for-simulated-annealing) [go]
   set noise noise - 1] ;gradually reduce the amount of noise
end

to setup
 ca
 crt numdots
 ask turtles [set label who
             set color red]
 randomize
end

to randomize
    ask turtles
        [setxy (random-int-or-float screen-size-x) (random-int-or-float screen-size-y)
        if who = 0
         [set color blue
          set xcor 0
          set ycor screen-edge-y * -1]
        if who = (numdots - 1)
         [set color blue
          set xcor 0
          set ycor screen-edge-y]]
end

to draw
ask patches
 [if mouse-down?
   [if (distancexy mouse-xcor mouse-ycor) < pensize
     [set pcolor green]]]
end

to erase
ask patches
 [if mouse-down?
   [if (distancexy mouse-xcor mouse-ycor) < pensize
     [set pcolor black]]]
end







@#$#@#$#@
GRAPHICS-WINDOW
321
10
899
609
35
35
8.0
1
10
1
1
1

CC-WINDOW
4
464
319
584
Command Center

BUTTON
15
100
96
133
NIL
draw
T
1
T
OBSERVER
T

BUTTON
123
100
204
133
NIL
erase
T
1
T
OBSERVER
T

BUTTON
123
56
254
89
Randomize dots
randomize
NIL
1
T
OBSERVER
T

BUTTON
15
55
96
88
Set up
setup
NIL
1
T
OBSERVER
T

SLIDER
57
253
229
286
pensize
pensize
0
8
3
1
1
NIL

SLIDER
57
298
229
331
numdots
numdots
0
40
11
1
1
NIL

SLIDER
60
342
233
375
noise
noise
0
360
112
1
1
NIL

SLIDER
59
381
232
414
movement-rate
movement-rate
0
8
1.0
0.2
1
NIL

BUTTON
14
146
106
180
Step once
go
NIL
1
T
OBSERVER
T

BUTTON
122
147
255
180
Go continuously
go
T
1
T
OBSERVER
T

BUTTON
49
203
228
236
Try simulated annealing
Simulated-annealing
NIL
1
T
OBSERVER
T

SLIDER
24
424
265
457
speed-for-simulated-annealing
speed-for-simulated-annealing
1
10
8
1
1
NIL

@#$#@#$#@
(Written by Dr. Robert Goldstone, Department of Psychology,
Indiana University.  http://cognitrn.psych.indiana.edu/.  Email: rgoldsto@indiana.edu)

This simulation shows how close-to-optimal paths can be found around obstacles with a decentralized system.  In traditional Artificial Intelligence, the search for a good solution is typically viewed as a process of adding steps to a solution and backtracking when dead-ends are found.  The alternative method pursued here is to have simple agents locally influence each others position.  They globally form a path even though no agent by itself represents an entire solution.

The method used here for creating good solutions is called "simulated annealing."  In actual annealing, strong metal structures are formed by gradually cooling the metal.  At high temperatures, the molecules of the metal move freely with respect to one another.  If the metal is cooled slowly, thermal mobility is lost.  The atoms are often able to line themselves up and form a pure crystal that is completely ordered, hence increasing the strength of the structure.  The analogy in "simulated annealing" is gradually reduce the randomess in a system.  Early on, randomness helps the system sample different candidate solutions.  later on, stability helps the system settle down to a single strong solution.  In computational applications of simulated annealing, the probability that a system will move from a state with goodness/energy E1 to a state with goodness E2 is set to exp[-(E2-E1)/kT], where a low E value means a BETTER solution (low energy is good), k is a constant, and T is the temperature or randomness in the system.  As such, if temperature if very low, then the system will almost always move in the direction of lower E.  However, if T is high, then the system may move toward the state with a higher E.  Although it may seem counter-productive to increase the energy of the system if one wants to have the least energy possible, it helps the system in avoiding LOCAL MINIMA.  Local minima are states such that the current E value is lower than all E values that are obtained by neighboring states, yet E is not the lowest possible value that the system could obtain.  In order to find global rather than local minima, it is often necessary that the energy in the system momentarily increases.  As an analogy, if a dog want to get a bone on the other side of a fence, he may actually have to walk further away from the bone (momentarily) so as to walk around the fence in order to eventually be close to the bone.  The local minimum in this case would be the dog salivating right at the fence with the bone a mere two feet away on the other side.  It is a local minimum because the dog cannot move without increasing his distance from the bone.  However, there is a global minimum that would have the dog right at the bone, but this could only be found if the dog jostles himself out of the local minima.  High values of T allow this to happen.

The individual agents (dots) in this simulation follow the following simple rules:

1. Each dot is assigned two neighbors, making a set of dots arranged from first to last.  One of the fixed blue dots is the neighbor of the first dot.  The other blue dot is the neighbor of the last dot.

2. Each dot will move toward the average position of its two neighbors, and will also move with a bit of randomness.

3. If the location that a dot would move to is painted green, then it does not move.



HOW TO USE IT
-------------
The dots will always try to form a path between the two blue dots on the top and bottom of the screen.

Click the SET UP button to erase the screen, and create a number of dots stipulated by the num_dots slider.

Click the RANDOMIZE button to preserve the green obstacles but place the dots in new locations.

The NOISE slider sets the amount of randomness in the movements of the dots.  A high value of noise corresponds to a high temperature T.

The MOVEMENT-RATE slider sets the amount of movement that each dot makes on every step.

To try out a simulated annealing strategy for your problem, click SIMULATED-ANNEALING.  The program will start off with a high level of noise and gradually reduce it with time.  How quickly gradually noise is reduced is set by the SPEED-FOR-SIMULATED-ANNEALING slider.

Click DRAW and ERASE to draw and erase the green obstacle on the screen.  If you are drawing, simply click the mouse on the area where you would like an obstacle.  The size of your obstacle will be defined by the slider PEN-SIZE.



THINGS TO NOTICE
----------------
Draw a single medium-sized obstacle in the middle of the screen.  If NOISE is set to zero, then a good path is not created by the dots between the two blue points.  In this case, each dot will always move closer to their neighboring dots whenever possible, and thus they will easily fall into local minima where no dot can move without falling on a green patch.

If NOISE is set very high, then a good path is also not found.  In this case, the dots randomly move around without settling on any path.

Setting NOISE to a small value often suffices to find a good solution, but notice that the dots still jiggle around.  Also, this solution will not work if the obstacles are fairly complex.  In these cases, you may be able to find paths using simulated annealing (gradually reducing the noise) that you would not be able to find otherwise.


EXPLORATIONS
------------
Experiment with different ways of adjusting the amount of randomness and amount of movement of the balls.  What technique does the best job of usually finding good pathways?  According to formula for simulated annealing mentioned above, how should you gradually reduce the noise in the system?

What is the best number of balls for a maze?  Why?  Does the best number of balls depend on the maze?

What kinds of mazes can and cannot usually be solved?

If the balls have trouble finding a pathway, what can you do to help them?

How can you improve the simulated annealing schedule for reducing noise that is used here?

Is there a better method than simulated annealing for efficiently finding pathways?

@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
setup
repeat 75 [ go ]
ask turtles [ set size 6 ]
@#$#@#$#@
@#$#@#$#@
