globals [
    steps         ; Number of steps the model has made (think hours for peers)
    graphs        ; List of connected sub graphs in peer network
    maxgraph      ; The ID of the graph with the largest number of nodes
    maxgraphsize  ; The number of nodes is has
]
turtles-own [
    inEdges       ; List of IDs of nodes pointing to me
    outEdges      ; List of IDs of nodes I point to
    numOutEdges   ; Number of out edges I'm supposed to maintain
    nhood         ; The nodes in my neighborhood
    ttl           ; The number of steps until I go off the air
    graph         ; The subnet I currently belong to
]
to-report edgeCount
    report length outEdges + length inEdges
end
to setup
    clear-all
    set-default-shape turtles "circle"
    set steps 0
    create-custom-turtles (screen-size-x * screen-size-y) [
        set ycor (screen-edge-y - floor (who / screen-size-x))
        set xcor (who mod screen-size-x) - screen-edge-x
        set color gray
        set inEdges []
        set outEdges []
        set ttl 0
        set numOutEdges 0
        ifelse drawLabels? [set label  0 + " "][set label no-label]
    ]
    ask turtles [ ; need second pass for nhood: all turtles created & in place
        set nhood turtles with [
            (distance myself <= radiusRange) and (who != who-of myself)
        ]
    ]
end
to step
    set steps steps + 1
    ask turtles [
        ifelse (ttl = 0) [
            if ( random-float 100 < birthProbability ) [createNode]
        ] [
            set ttl ttl - 1
            if (ttl = 0) [destroyNode]
        ]
    ]
    ; nodes may need to add links anytime they are reduced
    ; by deletions or because there were previously too
    ; few "on" neighbors available
    ask turtles with [numOutEdges > length outEdges] [
        addOutEdges (numOutEdges - length outEdges)
    ]
    calcGraphs
    ask turtles [
        ;set edgeCount length outEdges + length inEdges
        ifelse (ttl = 0) [hideturtle][
            set color (position graph graphs) * 10 + gray
            ifelse drawLabels?
                [set label (length outEdges + length inEdges) + " "]
                [set label no-label]
            showturtle
        ]
    ]
    doPlots
end
to createNode
    set ttl 1 + random lifeRange
    set numOutEdges 1 + random edgesRange
end
to destroyNode
    locals [myID]
    set ttl 0
    set numOutEdges 0
    set myID who
    ask nhood [
        set inEdges remove myID inEdges
        set outEdges remove myID outEdges
    ]
    set inEdges []
    set outEdges []
end
to addOutEdges [numEdges]
    locals [myID theirID them]
    set myid who
    repeat numEdges [
        set them random-one-of nhood with [
            not (member? myID outEdges or member? myID inEdges or ttl = 0)
        ]
        if (them != nobody) [
            ask them [
                set inEdges fput myID inEdges
                set theirID who
            ]
            set outEdges fput theirID outEdges
        ]
    ]
end
to calcGraphs
    locals[stack node edges graphsize]
    set graphs [] ; reset list of graphs
    set maxgraphsize 0
    ask turtles [set graph -1] ; reset which graph each turtle belongs to
    ask turtles with [ttl > 0] [
        ; important! must be sequential due to state kept in turtles
        without-interruption [
            if (graph = -1) [ ; if this turtle not yet in a graph, make root of next graph
                set stack [] ; stack of current batch of turtles to be processed
                set stack fput turtle who stack
                set graph who
                set graphs lput who graphs
                set graphsize 0
                while [length stack > 0] [
                    set node first stack
                    set stack but-first stack
                    set graphsize graphsize + 1
                    set edges sentence inEdges-of node outEdges-of node
                    while [length edges > 0] [
                        set node turtle first edges
                        set edges but-first edges
                        if (graph-of node = -1) [
                            set graph-of node who
                            set stack lput node stack
                        ]
                    ]
                    if (graphsize > maxgraphsize) [
                        set maxgraph who
                        set maxgraphsize graphsize
                    ]
                ]
            ]
        ]
    ]
    set graphs sentence maxgraph remove maxgraph graphs
end
to doPlots
    set-current-plot "Nodes"
    set-current-plot-pen "One"
        plot count turtles with [edgeCount = 1]
    set-current-plot-pen "Two"
        plot count turtles with [edgeCount = 2]
    set-current-plot-pen "Three"
        plot count turtles with [edgeCount = 3]
    set-current-plot-pen "Four"
        plot count turtles with [edgeCount = 4]
    set-current-plot-pen "Five"
        plot count turtles with [edgeCount = 5]
    set-current-plot-pen "Six"
        plot count turtles with [edgeCount = 6]
    set-current-plot-pen ">Six"
        plot count turtles with [edgeCount > 6]
    set-current-plot "EdgeHistogram"
    set-histogram-num-bars 10
    histogram-from turtles with [ttl > 0] [edgeCount]
end
@#$#@#$#@
GRAPHICS-WINDOW
368
10
798
461
10
10
20.0
1
12
1
1
1
0
1
1
1

CC-WINDOW
5
602
807
697
Command Center
0

BUTTON
6
43
87
76
NIL
setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
153
79
325
112
lifeRange
lifeRange
1
10
4
1
1
NIL

BUTTON
6
78
87
111
NIL
step
NIL
1
T
OBSERVER
T
NIL

SLIDER
153
117
325
150
radiusRange
radiusRange
1
5
2
1
1
NIL

SLIDER
153
42
325
75
birthProbability
birthProbability
1
100
20
1
1
NIL

SLIDER
153
153
326
186
edgesRange
edgesRange
1
8
2
1
1
NIL

BUTTON
6
112
87
145
go
step
T
1
T
OBSERVER
T
NIL

MONITOR
91
43
148
92
NIL
steps
3
1

SWITCH
6
147
130
180
drawLabels?
drawLabels?
1
1
-1000

PLOT
0
358
328
588
Nodes
Step
NodeCount
0.0
50.0
0.0
50.0
true
true
PENS
"One" 1.0 0 -8630108 true
"Two" 1.0 0 -13345367 true
"Three" 1.0 0 -2674135 true
"Four" 1.0 0 -11221820 true
"Five" 1.0 0 -13840069 true
"Six" 1.0 0 -6459832 true
">Six" 1.0 0 -16777216 true

PLOT
3
186
329
355
EdgeHistogram
Edges
Nodes
1.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -2674135 true

MONITOR
91
95
150
144
maxnet
maxgraphsize
3
1

@#$#@#$#@
The model creates a peer network who's network nodes/peers come alive (i.e. boot and joint the network) for a fixed time, then turn off again, dropping out of the network.  The fixed time is chosen randomly between 1 and a maximum life (lifeRange).

When a peer joins the net, she receives a number of outward pointing connections/edges (numOutEdges) she attempts to maintain with other peers.  She keeps a list of her current outward edges/connections (outEdges) and incoming connections/edges (inEdges) from other peers who's outEdges point to her.  Whenever her numOutEdges is larger than the actual outEdge list, she creates new connections within her neighborhood.  The new connection cannot be an existing in or out edge.

A peer's neighborhood is determined at birth to be the peers within a radius randomly chosen between 1 and a maxium radius (radiusRange).  Similarly, numOutEdges is chosen between 1 and a maximum number of edges (edgesRange).

Every step of the simulation, think hours for example, decreases the time to live (ttl) of the peer node.  When it dies, it stays off for one cycle, then rejoins with a fixed probability (birthProbability).

At the end of a step, a global calculation of disjoint subgraphs is made .. the peer groups that are complete by them selves but cannot "see" the other peer groups. The largest peer group is grey, the others various colors.  The size of the largest group is displayed as "maxnet".

Usage

First set the four sliders to the values of interest.  An interesting set is:
	birthProbability: 20 (come to life roughtly one in five chances)
	lifeRange: 4 (stay alive from 1 to 4 hours/steps)
	radiusRange: 2 (use a neighborhood of radius between 1 to 2 patches)
	edgesRange: 2 (have between 1 and 2 outgoing edges)
This set typically has more than one peer group, which can be seen by the multiple colored subnets.

Realistically, the edge and radius ranges would be larger, and would show the point of this model: peer nets tend to "close" rather naturally, avoiding disjoint subnets of isolated peer groups.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7500403 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7500403 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7500403 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7500403 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7500403 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7500403 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7500403 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7500403 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7500403 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -1184463 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7500403 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7500403 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7500403 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7500403 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7500403 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7500403 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7500403 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

circle
false
0
Circle -7500403 true true 35 35 230

person
false
0
Circle -7500403 true true 155 20 63
Rectangle -7500403 true true 158 79 217 164
Polygon -7500403 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7500403 true true 216 83 267 123 248 143 215 107
Polygon -7500403 true true 167 163 145 234 183 234 183 163
Polygon -7500403 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7500403 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7500403 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7500403 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8630108 true false 195 75 195 120 240 120 240 75
Polygon -8630108 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7500403 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8630108 true false 90 210 105 225 120 210
Polygon -8630108 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7500403 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8630108 true false 210 210 195 225 180 210
Polygon -8630108 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7500403 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6459832 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6459832 true true 87 80 79 55 76 79
Polygon -6459832 true true 81 75 70 58 73 82
Polygon -6459832 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6459832 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6459832 true true 116 140 114 189 105 137
Rectangle -6459832 true true 109 150 114 192
Rectangle -6459832 true true 111 143 116 191
Polygon -6459832 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6459832 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6459832 true true 214 134 203 168 192 148
Polygon -6459832 true true 204 151 203 176 193 148
Polygon -6459832 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6459832 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6459832 true true 201 99 214 69 215 99
Polygon -6459832 true true 207 98 223 71 220 101
Polygon -6459832 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6459832 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6459832 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6459832 true true 48 143 58 141
Polygon -6459832 true true 46 136 68 137
Polygon -6459832 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6459832 true true 38 138 66 149
Polygon -6459832 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6459832 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 3.0pre5
@#$#@#$#@
setup
repeat 20 [ step ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
