;; Written by: Joe Glessner (jglessner@usip.edu) under the direction of Dr. James Johnson
;; University of the Sciences in Philadelphia
;; 2004

globals [
         numTrav
         rand-infect
         isset
         time
         continue
         interval
         xcoord
         ycoord
         radius
         dead
         file
        ]

turtles-own[
            weak
            recover
           ]

to startup


    ask patches[set pcolor white]
    import-ppm "Welcome.ppm"

end

;;--------------------------------------------------------------------------------------------------------

to Setup
  ca
  ;;random-seed 311
  disp-back
  setup-turtles
  set time 0
  file-close

end

;;--------------------------------------------------------------------------------------------------------

to Go
  set continue true
  init-recover

  loop[
    ifelse continue and count turtles > 0[
      ask turtles [
        change-dir
        move
        infect
        reproduce
        age
      ]
      set time time + 1
      do-plots

    ]
    [
     stop]
  ]
end

;;--------------------------------------------------------------------------------------------------------

to Halt
  set continue false
end

;;--------------------------------------------------------------------------------------------------------
to import-ppm [pathname]
  ;; USER-CHOOSE-FILE returns false if the user cancels, so this
  ;; check is here to handle that case
  if not is-string? pathname
    [ stop ]
  carefully [
    file-open pathname
    import-ppm-contents
  ]
  []
  file-close
end
to import-ppm-contents
  ;; check magic number
  if read-pgm-entry != "P3" [
    user-message "This is not a valid ASCII format PGM file (magic number is not P3)"
    stop
  ]
  ;; get width, height, and white value
  let width read-from-string read-pgm-entry
  let height read-from-string read-pgm-entry
  let max-value read-from-string read-pgm-entry
  ;; read the actual pixel values
  let x 0
  let y 0
  while [y < height] [
    set x 0
    while [x < width][

       let value1 (read-from-string read-pgm-entry / max-value)
       let value2 (read-from-string read-pgm-entry / max-value)
       let value3 (read-from-string read-pgm-entry / max-value)
        ;; convert from image coordinates to patch coordinates
        ;; (in patch coordinates, the origin is in the center,
        ;; not the top left, and y coordinates increase going up,
        ;; not going down)
        let px x - screen-edge-x
        let py screen-edge-y - y

        ;; make sure we're not out of bounds
        if (abs px <= screen-edge-x) and (abs py <= screen-edge-y) [
          ;; actually color the patch
          ask patch px py [
            set pcolor (rgb value1 value2 value3) ;scale-color white value 0 max-value
          ]
        ]
        set x x + 1
      ]
    set y y + 1
  ]
end
;; reads the next entry in a pgm file
;; - an error occurs if there are no more entries
;; - entries are separated by arbitrary whitespace
;; - characters on a line after "#" are comments
to-report read-pgm-entry
  ;; get next character
  let c file-read-characters 1
  ;; ignore leading whitespace
  while [whitespace-char? c]
    [ set c file-read-characters 1 ]
  ;; skip comments
  if c = "#" [
    ;; first skip the comment itself
    while [c != "\n" and c != "\r"] [
      set c file-read-characters 1
    ]
    ;; then skip linefeeds and/or newlines at end of comment
    while [c = "\n" or c = "\r"] [
      set c file-read-characters 1
    ]
  ]
  ;; read the entry
  let str ""
  while [not whitespace-char? c]
    [ set str str + c
      set c file-read-characters 1 ]
  report str
end
;; reports true if c is a single whitespace character
to-report whitespace-char? [c]
  report c = " " or
         c = "\t" or   ;; tab
         c = "\n" or   ;; newline
         c = "\r"      ;; linefeed
end
                                                                                ;;(C) 2004 Uri Wilensky.
;;==============================================================

to init-recover
  ask turtles[
   if recover = true
     [set color blue + (color - red)
      set size size + .001
      set weak false
      set recover false]
  ]
end

;;--------------------------------------------------------------------------------------------------------

to disp-back
 locals[width]
 ask patches [set pcolor white]
  if not(Background = 1) [
    ifelse (substring Background 1 2) = "A"
        [set width 10]
        [ifelse (substring Background 1 2) = "B"
          [set width 5]
          [ifelse (substring Background 2 3) = "A"
            [set width 10]
            [set width 5]
          ]
       ]
  if (substring Background 0 1 = "2") [
    ask patches [
        if (pxcor = 0) and (abs(pycor) < screen-edge-x - (width * 2))
          [set pcolor black]
        if (abs(pycor) = screen-edge-x - (width * 2)) and (abs(pxcor) < screen-edge-x - (width * 2))
          [set pcolor black]
    ]
  ]
  if ((substring Background 0 1) = "3") [
    ask patches [
      if (abs(pxcor) + abs(pycor) = screen-edge-x - width) and (abs(pycor) > width)
        [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "4") [
    ask patches [
      if (abs(pxcor) = abs(pycor)) and (abs(pxcor) < screen-edge-x - width)
        [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "5") [
    ask patches [
      if (pxcor = 0) and ((abs(pycor) < screen-edge-x - 40) or (abs(pycor) > screen-edge-x - (40 - width)))
        [set pcolor black]
      if (abs(pxcor) = abs(pycor)) and (abs(pxcor) < screen-edge-x - width)
        [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "6") [
    ask patches [
      if (abs(pxcor) + abs(pycor) = 70) and (abs(pycor) > (width / 2) and not(abs(pxcor) > 20 and abs(pxcor) < (20 + width))) or
         ((abs(pxcor) + abs(pycor) = 90) and (abs(pycor) < (90 - width)) and not((abs(pxcor) > 70 and abs(pxcor) < (70 + width))  or (abs(pxcor) > 28 and abs(pxcor) < (30 + width)))) or
         (abs(pxcor) = abs(pycor) and abs(pycor) < 36) or
         (abs(pxcor) = abs(pycor) and abs(pycor) > 45 + width)
        [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "7") [
    ask patches [
      if (abs(pxcor) = 50 and not(abs(pycor) < (width / 2)) and abs(pycor) < screen-edge-x - width) or
         (pxcor = 0 and pxcor < screen-edge-x - width and not(abs(pycor) > 10 and abs(pycor) < 15)) or
         (abs(pycor) = 10 and abs(pxcor) < screen-edge-x - width) or
         (abs(pycor) = 32  and not(abs(pxcor) < (50 + width) and abs(pxcor) > (50 - width))) or
         (abs(pycor) = 55 and abs(pxcor) < screen-edge-x - width and not(abs(pxcor) < width)) or
         (abs(pycor) = 80 and not(abs(pxcor) < (50 + width) and abs(pxcor) > (50 - width)))
           [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "8") [
    ask patches [
       if (abs(pxcor) = 0 and pycor < screen-edge-x - width) or
          (abs(pxcor) < 10 and abs(pxcor) > 10 and pycor = 80) or
          (abs(pxcor) = 10 and pycor > (75 + width)) or
          (abs(pxcor) < 20 and abs(pxcor) > 0 and pycor = 70) or
          (abs(pxcor) = 20 and pycor < screen-edge-x - width and pycor > 69) or
          (abs(pxcor) < 30 and abs(pxcor) > width and pycor = 60) or
          (abs(pxcor) = 30 and pycor > 59) or
          (abs(pxcor) < 40 and abs(pxcor) > 0 and pycor = 50) or
          (abs(pxcor) = 40 and pycor < screen-edge-x - width and pycor > 49) or
          (abs(pxcor) = 50 and pycor > 39) or
          (abs(pxcor) < 50 and abs(pxcor) > width and pycor = 40) or
          (abs(pxcor) < 60 and abs(pxcor) > 0 and pycor = 30) or
          (abs(pxcor) = 60 and pycor > 29 and pycor < screen-edge-x - width) or
          (abs(pxcor) = 70 and pycor > 19) or
          (abs(pxcor) < 70 and abs(pxcor) > width and pycor = 20) or
          (abs(pxcor) < 80 and abs(pxcor) > 0 and pycor = 10) or
          (abs(pxcor) = 80 and pycor > 9 and pycor < screen-edge-x - width) or
          (abs(pxcor) = screen-edge-x - 10 and pycor > 1) or
          (abs(pxcor) < screen-edge-x - 10 and abs(pxcor) > width and pycor = 2)
         [set pcolor black]
    ]
  ]
  if (substring Background 0 1 = "9") [
    set xcoord width
    set ycoord 85 + width
    repeat 16 - width
      [back-diag]
    set xcoord 1
    set ycoord 70
    repeat 32 - width
      [back-diag]
    set xcoord width
    set ycoord 50 + width
    repeat 51 - width
      [back-diag]
    set xcoord 1
    set ycoord 30
    repeat 73 - width
      [back-diag]
    set xcoord width
    set ycoord 6 + width
    repeat 95 - width
      [back-diag]
    set xcoord 17
    set ycoord 2
    repeat 82 - width
      [back-diag]
    set xcoord 35 + width
    set ycoord width
    repeat 66 - width
      [back-diag]
    set xcoord 58
    set ycoord 2
    repeat 44 - width
      [back-diag]
    set xcoord 75 + width
    set ycoord width
    repeat 26 - width
      [back-diag]
    ask patches [
      if (pycor = 1 and abs(pxcor) < screen-edge-x - 8) or
         (pxcor = 0 and pycor < screen-edge-x - 8)
        [set pcolor black]
    ]
  ]
  if (substring Background 0 2 = "10")[
   ask patches [
       if (abs(pxcor) = 0 and pycor < screen-edge-x - width) or
          (abs(pxcor) < 10 and abs(pxcor) > 10 and abs(pycor) = 80) or
          (abs(pxcor) = 10 and abs(pycor) > (75 + width)) or
          (abs(pxcor) < 20 and abs(pxcor) > 0 and abs(pycor) = 70) or
          (abs(pxcor) = 20 and abs(pycor) < screen-edge-x - width and abs(pycor) > 69) or
          (abs(pxcor) < 30 and abs(pxcor) > width and abs(pycor) = 60) or
          (abs(pxcor) = 30 and abs(pycor) > 59) or
          (abs(pxcor) < 40 and abs(pxcor) > 0 and abs(pycor) = 50) or
          (abs(pxcor) = 40 and abs(pycor) < screen-edge-x - width and abs(pycor) > 49) or
          (abs(pxcor) = 50 and abs(pycor) > 39) or
          (abs(pxcor) < 50 and abs(pxcor) > width and abs(pycor) = 40) or
          (abs(pxcor) < 60 and abs(pxcor) > 0 and abs(pycor) = 30) or
          (abs(pxcor) = 60 and abs(pycor) > 29 and abs pycor < screen-edge-x - width) or
          (abs(pxcor) = 70 and abs(pycor) > 19) or
          (abs(pxcor) < 70 and abs(pxcor) > width and abs(pycor) = 20) or
          (abs(pxcor) < 80 and abs(pxcor) > 0 and abs(pycor) = 10) or
          (abs(pxcor) = 80 and abs(pycor) > 9 and abs pycor < screen-edge-x - width) or
          (abs(pxcor) = screen-edge-x - 10 and abs(pycor) > (width / 2))
         [set pcolor black]
    ]
   ]
  ]
end

;;--------------------------------------------------------------------------------------------------------

to back-diag
  ask patch xcoord ycoord [set pcolor black]
  ask patch (- xcoord) ycoord [set pcolor black]
  set xcoord xcoord + 1
  set ycoord ycoord + 1
end

;;--------------------------------------------------------------------------------------------------------
to setup-turtles
    crt Travelers_Left + Travelers_Right + Meanderers_Left + Meanderers_Right
    set numTrav (Travelers_Left + Travelers_Right)
    ask turtles[set weak false
                set size 5]
    init-infect
    separate-move
end

;;--------------------------------------------------------------------------------------------------------

to check-approp
    loop[
       ifelse ((pcolor-of (patch-ahead 1) = black) or
         (pcolor-of (patch-left-and-ahead (heading - (90 * (quad))) 1) = black and
          pcolor-of (patch-right-and-ahead (90 - (heading - (90 * (quad)))) 1) = black)) or
          pxcor-of (patch-ahead 1) = (- pxcor-of (patch-here)) or
          pycor-of (patch-ahead 1) = (- pycor-of (patch-here)) or
         ((isset = 0) and (Background = 1) and (pxcor-of (patch-ahead 1) = 0))
           [change-dir]
           [stop]
    ]
end

;;--------------------------------------------------------------------------------------------------------

to change-dir
  set heading (random 360)
end

;;--------------------------------------------------------------------------------------------------------

to-report quad
  if heading >= 0 and heading < 90
     [report 0]
  if heading >= 90 and heading < 180
     [report 1]
  if heading >= 180 and heading < 270
     [report 2]
  if heading >= 270 and heading < 360
     [report 3]
end

;;--------------------------------------------------------------------------------------------------------

to init-infect
ask turtles
      [set color ((.001 * (random Healthy_Lifespan)) + blue)
       if color >= (blue + (.001 * Procreation_Age))
         [set color color + 3]
       if who >= numTrav
         [set shape "circle"]
      ]
    repeat ((Travelers_Left + Meanderers_Left) * (%_Infected_Left / 100))
      [set rand-infect (random (Travelers_Left + Meanderers_Left))
       if rand-infect >= Travelers_Left
         [set rand-infect (rand-infect + Travelers_Right)]
       check-already-infect-l
       become-infected
      ]
    repeat ((Travelers_Right + Meanderers_Right) * (%_Infected_Right / 100))
      [set rand-infect (random (Travelers_Right + Meanderers_Right))
       ifelse rand-infect < (Travelers_Right)
         [set rand-infect (rand-infect + Travelers_Left)]
         [set rand-infect (rand-infect + Travelers_Left + Meanderers_Left)]
       check-already-infect-r
       become-infected
      ]
end

;;--------------------------------------------------------------------------------------------------------

to become-infected
       ifelse color-of turtle rand-infect < 106
         [if (color-of turtle rand-infect - blue) < (.001 * Susceptible_Below) or (color-of turtle rand-infect - blue) > (.001 * Susceptible_Above)
           [set weak-of turtle rand-infect true]]
         [if (color-of turtle rand-infect - blue) < 3 + (.001 * Susceptible_Below) or (color-of turtle rand-infect - blue) > 3 + (.001 * Susceptible_Above)
           [set weak-of turtle rand-infect true]]
       set color-of turtle rand-infect (((color-of turtle rand-infect) - blue) + red)
       set size-of turtle rand-infect (size-of turtle rand-infect + (.001 * (random Susceptible_Infection_Lifespan)))
       if random 100 < Deadly_Mutations
         [set color-of turtle rand-infect (color-of turtle rand-infect + .010)]
       ifelse random 100 < Percent_Recovery
         [set recover-of turtle rand-infect true]
         [set recover-of turtle rand-infect false]

end

;;--------------------------------------------------------------------------------------------------------
to check-already-infect-l
loop[
     ifelse (shade-of? red (color-of turtle rand-infect))
       [set rand-infect (random (Travelers_Left + Meanderers_Left))
        if rand-infect >= Travelers_Left
          [set rand-infect (rand-infect + Travelers_Right)]]
       [stop]
    ]
end

;;--------------------------------------------------------------------------------------------------------

to check-already-infect-r
loop[
     ifelse (shade-of? red (color-of turtle rand-infect))
       [set rand-infect (random (Travelers_Right + Meanderers_Right))
       ifelse rand-infect < (Travelers_Right)
         [set rand-infect (rand-infect + Travelers_Left)]
         [set rand-infect (rand-infect + Travelers_Left + Meanderers_Left)]]
       [stop]
    ]
end

;;--------------------------------------------------------------------------------------------------------

to separate-move
  ask turtles [
      set isset 0
      ifelse who < Travelers_Left or
        (who < (Travelers_Left + Travelers_Right + Meanderers_Left) and who >= (Travelers_Left + Travelers_Right))
          [if Left_Quadrant = "Top Outside"
             [set heading -90
              fd 75
              set heading 0
              fd 75]
           if Left_Quadrant = "Top Center"
             [set heading -90
              fd 25
              set heading 0
              fd 75]
           if Left_Quadrant = "High Outside"
             [set heading -90
              fd 75
              set heading 0
              fd 25]
           if Left_Quadrant = "High Center"
             [set heading -90
              fd 25
              set heading 0
              fd 25]
           if Left_Quadrant = "Low Outside"
             [set heading -90
              fd 75
              set heading 180
              fd 25]
           if Left_Quadrant = "Low Center"
             [set heading -90
              fd 25
              set heading 180
              fd 25]
           if Left_Quadrant = "Bottom Outside"
             [set heading -90
              fd 75
              set heading 180
              fd 75]
           if Left_Quadrant = "Bottom Center"
             [set heading -90
              fd 25
              set heading 180
              fd 75]
           set heading (random 360)
           repeat (random (100 - Density_Left))
             [check-approp
              fd 1]]
          [if Right_Quadrant = "Top Outside"
             [set heading 90
              fd 75
              set heading 0
              fd 75]
           if Right_Quadrant = "Top Center"
             [set heading 90
              fd 25
              set heading 0
              fd 75]
           if Right_Quadrant = "High Outside"
             [set heading 90
              fd 75
              set heading 0
              fd 25]
           if Right_Quadrant = "High Center"
             [set heading 90
              fd 25
              set heading 0
              fd 25]
           if Right_Quadrant = "Low Outside"
             [set heading 90
              fd 75
              set heading 180
              fd 25]
           if Right_Quadrant = "Low Center"
             [set heading 90
              fd 25
              set heading 180
              fd 25]
           if Right_Quadrant = "Bottom Outside"
             [set heading 90
              fd 75
              set heading 180
              fd 75]
           if Right_Quadrant = "Bottom Center"
             [set heading 90
              fd 25
              set heading 180
              fd 75]
           set heading (random 360)
           repeat (random (100 - Density_Right))
             [check-approp
              fd 1]]
    ]
    set isset 1
end

;;--------------------------------------------------------------------------------------------------------

to infect
  set radius Infection_Radius
  check-radius
  if shade-of? blue color and (Contact_Infection > (random 100)) and
     any? turtles in-radius-no-wrap radius with [shade-of? red color] and
     not(size <= 5 + .001 + (.001 * Recover_Immune_Time) and size > 5)
     [ifelse color < 106
         [if (color - blue) < (.001 * Susceptible_Below) or (color - blue) > (.001 * Susceptible_Above)
           [set weak true]]
         [if (color - blue) < 3 + (.001 * Susceptible_Below) or (color - blue) > 3 + (.001 * Susceptible_Above)
           [set weak true]]
      set color red + (color - blue)  ;; add current age
      set size 5
      if random 100 < Deadly_Mutations
       [set color (color + .010)]
      if random 100 < Percent_Recovery
       [set color blue + (color - red)
        set size size + .001
        set weak false]
      ]
end

;;--------------------------------------------------------------------------------------------------------

to check-radius
loop[
      ifelse any? patches in-radius-no-wrap radius with [pcolor = black]
        [set radius radius - 1]
        [stop]
    ]
end

;;--------------------------------------------------------------------------------------------------------

to reproduce
locals[
       mat-blue-trav
       mat-blue-meand
       nei-mat-blue-trav
       nei-mat-blue-meand
       mat-red-trav
       mat-red-meand
       nei-mat-red-trav
       nei-mat-red-meand
       myWho ;; any? turtles in-radius-no-wrap includes calling turtle
      ]
  if radius > 5
    [set radius 5]
  set myWho who
  if ((random 100) < 25) and  ;; 50% chance of running into opposite sex also, no reproduction 2x a time interval
       count turtles in-radius-no-wrap radius < Carrying_Capacity and
       count turtles in-radius-no-wrap radius > 1
    [  set mat-blue-trav (shade-of? blue color and color >= blue + (.001 * Procreation_Age) and shape = "default")
       set mat-blue-meand (shade-of? blue color and color >= blue + (.001 * Procreation_Age) and shape = "circle")
       set nei-mat-blue-trav any? turtles in-radius-no-wrap radius with
         [not (who = myWho) and shade-of? blue color and color >= blue + (.001 * Procreation_Age) and shape = "default"]
       set nei-mat-blue-meand any? turtles in-radius-no-wrap radius with
         [not (who = myWho) and shade-of? blue color and color >= blue + (.001 * Procreation_Age) and shape = "circle"]
       set mat-red-trav (shade-of? red color and color >= (red + (.001 * Procreation_Age)) and shape = "default")
       set mat-red-meand (shade-of? red color and color >= (red + (.001 * Procreation_Age)) and shape = "circle")
       set nei-mat-red-trav any? turtles in-radius-no-wrap radius with
         [not (who = myWho) and shade-of? red color and color >= red + (.001 * Procreation_Age) and shape = "default"]
       set nei-mat-red-meand any? turtles in-radius-no-wrap radius with
         [not (who = myWho) and shade-of? red color and color >= red + (.001 * Procreation_Age) and shape = "circle"]
       ifelse mat-blue-meand and nei-mat-blue-meand and (random 1000) < (Healthy_Meanderer_Fertility * Healthy_Meanderer_Fertility)
           [hatch Healthy_Birth_Number
             [set color blue
              set weak false
              set size 5]]
       [ifelse mat-blue-trav and nei-mat-blue-trav and (random 1000) < (Healthy_Traveler_Fertility * Healthy_Traveler_Fertility)
           [hatch Healthy_Birth_Number  ;; repetitive but nested ifelse the only way to do case
             [set color blue
              set weak false
              set size 5]]
       [ifelse ((mat-blue-meand and nei-mat-red-meand) or (nei-mat-blue-meand and mat-red-meand)) and (random 1000) < (Healthy_Meanderer_Fertility * Infected_Meanderer_Fertility)
         [ifelse (random 100) < 50
            [hatch Healthy_Birth_Number
              [set color blue
               set weak false
               set size 5]]
            [hatch Infected_Birth_Number
              [set color red
              set weak true
              set size 5
              if random 100 < Percent_Recovery
                [set color blue + (color - red)
                 set size size + .001
                 set weak false]
              if random 100 < Deadly_Mutations
                 [set color (color + .010)]]]]
       [ifelse ((mat-blue-trav and nei-mat-red-trav) or (nei-mat-blue-trav and mat-red-trav)) and (random 1000) < (Healthy_Traveler_Fertility * Infected_Traveler_Fertility)
         [ifelse (random 100) < 50
            [hatch Healthy_Birth_Number
              [set color blue
               set weak false
               set size 5]]
            [hatch Infected_Birth_Number
              [set color red
              set weak true
              set size 5
              if random 100 < Percent_Recovery
                [set color blue + (color - red)
                 set size size + .001
                 set weak false]
              if random 100 < Deadly_Mutations
                 [set color (color + .010)]]]]
       [ifelse mat-red-meand and nei-mat-red-meand and (random 1000) < (Infected_Meanderer_Fertility * Infected_Meanderer_Fertility)
         [hatch Infected_Birth_Number
           [set color red
           set weak true
           set size 5
           if random 100 < Percent_Recovery
                [set color blue + (color - red)
                 set size size + .001
                 set weak false]
            if random 100 < Deadly_Mutations
             [set color (color + .010)]]]
       [if mat-red-trav and nei-mat-red-trav and (random 1000) < (Infected_Traveler_Fertility * Infected_Traveler_Fertility)
         [hatch Infected_Birth_Number
           [set color red
           set weak true
           set size 5
           if random 100 < Percent_Recovery
                [set color blue + (color - red)
                 set size size + .001
                 set weak false]
            if random 100 < Deadly_Mutations
             [set color (color + .010)]]]]]]]]

    ]
end

;;--------------------------------------------------------------------------------------------------------

to move
  ifelse shape = "default"
    [repeat Traveler_Moves ;; how many steps per move
      [check-approp
       fd 1]]
    [repeat Meanderer_Moves
      [check-approp
       fd 1]]
end

;;--------------------------------------------------------------------------------------------------------

to age
  set color (color + .001)
  if (color >= (blue + (.001 * Procreation_Age)) and color < 106) or
     (color >= (red + (.001 * Procreation_Age))  and color < 16)
     [set color color + 3]
  ifelse shade-of? blue color
    [if size > 5 [set size (size + .001)]
     ifelse color < 106
         [if color >= (blue + (.001 * Healthy_Lifespan))
           [set dead dead + 1
            die]]
         [if color >= ((blue + 3) + (.001 * Healthy_Lifespan))
           [set dead dead + 1
            die]]]
     [set size (size + .001)
     ifelse color < 16
       [if color >= (red + (.001 * Healthy_Lifespan))
         [set dead dead + 1
         die]]
       [if color >= ((red + 3) + (.001 * Healthy_Lifespan))
         [set dead dead + 1
          die]]
     ifelse weak
       [if size >=  (5 + (.001 * Susceptible_Infection_Lifespan))
         [set dead dead + 1
          die]]
       [if size >=  (5 + (.001 * Infection_Lifespan))
         [set dead dead + 1
          die]]
    ]
end

;;--------------------------------------------------------------------------------------------------------

to do-plots
  set-current-plot "Healthy/ Infected"
  set-current-plot-pen "Healthy"
  plot count turtles with [shade-of? blue color]
  set-current-plot-pen "Infected"
  plot count turtles with [shade-of? red color]
  ;;----------------------------------------------
  set-current-plot "Travelers (Arrows)"
  set-current-plot-pen "Healthy Adult"
  plot count turtles with [color > 106 and shape = "default"]
  set-current-plot-pen "Healthy Young"
  plot count turtles with [shade-of? blue color and color < 106 and shape = "default"]
  set-current-plot-pen "Infected Adult"
  plot count turtles with [shade-of? red color and color > 16 and shape = "default"]
  set-current-plot-pen "Infected Young"
  plot count turtles with [shade-of? red color and color < 16 and shape = "default"]
  ;;----------------------------------------------
  set-current-plot "Meanderers (Circles)"
  set-current-plot-pen "Healthy Adult"
  plot count turtles with [color > 106 and shape = "circle"]
  set-current-plot-pen "Healthy Young"
  plot count turtles with [shade-of? blue color and color < 106 and shape = "circle"]
  set-current-plot-pen "Infected Adult"
  plot count turtles with [shade-of? red color and color > 16 and shape = "circle"]
  set-current-plot-pen "Infected Young"
  plot count turtles with [shade-of? red color and color < 16 and shape = "circle"]
end

;;--------------------------------------------------------------------------------------------------------

to setup-file
locals[line]
  ifelse file-exists? "C:/WINDOWS/Desktop/Pestilence/Pestilence Data.csv"
    [set file "C:/WINDOWS/Desktop/Pestilence/Pestilence Data.csv"] ;;Because file locations will vary from system to system, you may want to change this to aviod continuous file selection.
    [ifelse file-exists? "C:/Documents and Settings/All Users/Desktop/Pestilence/Pestilence Data.csv"
      [set file "C:/Documents and Settings/All Users/Desktop/Pestilence/Pestilence Data.csv"]
      [set file user-choose-file]
    ]
  file-open file
  set line file-read-line ;; You need to close the Excel SS if you get an error here.
  file-close
  if line = "first"
    [file-delete file]
  file-open file
  if line = "first"
    [file-print "This file MUST be closed while running the pestilence model. Do NOT save when prompted."
     file-print "To format: 1) Open 'Template' Excel document(Enable Macros) 2) View this window 3)Press Alt + F8 4) Select Template.xls!Macro1 and Run"
    ]
  if line = "first"
    [user-message "File output will be found on the Desktop in the Pestilence folder (The image with a,). This Excel file must be closed while running the Pestilence model. Do NOT save when prompted."]
  file-print "___P_____,____E____,____S____,____T___,____I__,____L____,____E____,____N__,____C___,_____E___,_________,_________"
  file-close
end

;;--------------------------------------------------------------------------------------------------------

to disp-header
  file-type "Initial Conditions"
  file-type ",Density Left: "                file-type Density_Left
  file-type ",Density Right: "               file-type Density_Right
  file-type ",Travelers Left: "              file-type Travelers_Left
  file-type ",Travelers Right: "             file-type Travelers_Right
  file-type ",Meanderers Left: "             file-type Meanderers_Left
  file-type ",Meanderers Right: "            file-type Meanderers_Right
  file-type ",% Infected Left: "             file-type %_Infected_Left
  file-type ",% Infected Right: "            file-type %_Infected_Right
  file-type ",Background: "                  file-type Background
  file-type ",Left Quadrant: "               file-type Left_Quadrant
  file-type ",Right Quadrant: "              file-print Right_Quadrant

  file-type "Susceptible Infection Lifespan: " file-type Susceptible_Infection_Lifespan
  file-type ",Susceptible Below: "              file-type Susceptible_Below
  file-type ",Susceptible Above: "              file-type Susceptible_Above
  file-type ",Percent Recovery: "               file-type Percent_Recovery
  file-type ",Recover Immune Time: "            file-type Recover_Immune_Time
  file-type ",Infection Lifespan: "             file-type Infection_Lifespan
  file-type ",Deadly Mutations: "               file-type Deadly_Mutations
  file-type ",Healthy Traveler Fertility: "     file-type Healthy_Traveler_Fertility
  file-type ",Healthy Meanderer Fertility: "    file-type Healthy_Meanderer_Fertility
  file-type ",Infected Traveler Fertility: "    file-type Infected_Traveler_Fertility
  file-type ",Infected Meanderer Fertility: "   file-type Infected_Meanderer_Fertility
  file-type ",Procreation Age: "                file-print Procreation_Age

  file-type "Healthy Birth Number: "        file-type Healthy_Birth_Number
  file-type ",Infected Birth Number: "        file-type Infected_Birth_Number
  file-type ",Healthy Lifespan: "             file-type Healthy_Lifespan
  file-type ",Meanderer Moves: "              file-type Meanderer_Moves
  file-type ",Traveler Moves: "               file-type Traveler_Moves
  file-type ",Carrying Capacity: "           file-type Carrying_Capacity
  file-type ",Contact Infection: "           file-type Contact_Infection
  file-type ",Infection Radius: "             file-type Infection_Radius

  file-print ""
  file-type "Time,Total,% Healthy,% Infected,Healthy Adult Travelers,Healthy Young Travelers,"
  file-type "Infected Adult Travelers,Infected Young Travelers,Healthy Adult Meanderers,"
  file-type "Healthy Young Meanderers,Infected Adult Meanderers,"
  file-print "Infected Young Meanderers"
end

;;--------------------------------------------------------------------------------------------------------

to update-file
  file-type time file-type ","
  file-type count turtles file-type ","
  ifelse count turtles > 0
    [file-type precision ((count turtles with [shade-of? blue color] / count turtles) * 100) 3 file-type ","
     file-type precision ((count turtles with [shade-of? red color] / count turtles) * 100) 3 file-type ","
    ]
    [file-type 0 file-type "," file-type 0 file-type ","]
  file-type count turtles with [color > 106 and shape = "default"] file-type ","
  file-type count turtles with [shade-of? blue color and color < 106 and shape = "default"] file-type ","
  file-type count turtles with [shade-of? red color and color > 16 and shape = "default"] file-type ","
  file-type count turtles with [shade-of? red color and color < 16 and shape = "default"] file-type ","
  file-type count turtles with [color > 106 and shape = "circle"] file-type ","
  file-type count turtles with [shade-of? blue color and color < 106 and shape = "circle"] file-type ","
  file-type count turtles with [shade-of? red color and color > 16 and shape = "circle"] file-type ","
  file-type count turtles with [shade-of? red color and color < 16 and shape = "circle"] file-print ""
end
@#$#@#$#@
GRAPHICS-WINDOW
432
10
912
511
100
100
2.34
1
5
1
1
1
0

CC-WINDOW
5
645
1026
740
Command Center

BUTTON
366
47
434
84
NIL
Go
NIL
1
T
OBSERVER
T
NIL

BUTTON
366
10
434
47
NIL
Setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
0
412
212
445
Healthy_Lifespan
Healthy_Lifespan
0
100
80
1
1
NIL

SLIDER
0
181
212
214
Infection_Lifespan
Infection_Lifespan
0
50
20
1
1
NIL

SLIDER
212
313
367
346
Procreation_Age
Procreation_Age
0
30
16
1
1
NIL

SLIDER
0
445
212
478
Contact_Infection
Contact_Infection
0
100
83
1
1
NIL

SLIDER
0
76
128
109
Meanderers_Left
Meanderers_Left
0
500
85
5
1
NIL

SLIDER
128
43
271
76
Travelers_Right
Travelers_Right
0
500
100
5
1
NIL

SLIDER
128
76
271
109
Meanderers_Right
Meanderers_Right
0
500
325
5
1
NIL

SLIDER
0
109
128
142
%_Infected_Left
%_Infected_Left
0
100
5
1
1
NIL

SLIDER
128
109
271
142
%_Infected_Right
%_Infected_Right
0
100
10
1
1
NIL

SLIDER
0
379
212
412
Infected_Birth_Number
Infected_Birth_Number
0
15
1
1
1
NIL

PLOT
-1
511
308
631
Healthy/ Infected
Time (years)
Number
0.0
100.0
0.0
100.0
true
true
PENS
"Healthy" 1.0 0 -16776961 true
"Infected" 1.0 0 -65536 true

BUTTON
366
84
434
124
Stop
Halt
NIL
1
T
OBSERVER
T
NIL

MONITOR
366
169
434
218
Total
count turtles
3
1

MONITOR
912
72
999
121
Healthy Adult
count turtles with [color > 106 and shape = "default"]
3
1

MONITOR
912
120
999
169
Healthy Young
count turtles with [shade-of? blue color and color < 106 and shape = "default"]
3
1

SLIDER
0
313
212
346
Infected_Traveler_Fertility
Infected_Traveler_Fertility
0
100
19
1
1
NIL

SLIDER
212
346
366
379
Carrying_Capacity
Carrying_Capacity
0
5
3
1
1
NIL

PLOT
308
511
654
631
Travelers (Arrows)
Time (years)
Number
0.0
100.0
0.0
100.0
true
true
PENS
"Healthy Adult" 1.0 0 -6710785 true
"Healthy Young" 1.0 0 -16776961 true
"Infected Adult" 1.0 0 -26215 true
"Infected Young" 1.0 0 -65536 true

MONITOR
366
121
434
170
Time (years)
time
3
1

MONITOR
366
413
434
462
% Infected
(count turtles with [shade-of? red color] / count turtles) * 100
3
1

MONITOR
366
364
434
413
% Healthy
(count turtles with [shade-of? blue color] / count turtles) * 100
3
1

MONITOR
912
167
999
216
Infected Adult
count turtles with [shade-of? red color and color > 16 and shape = "default"]
3
1

MONITOR
912
216
999
265
Infected Young
count turtles with [shade-of? red color and color < 16 and shape = "default"]
3
1

MONITOR
912
324
999
373
Healthy Adult
count turtles with [color > 106 and shape = "circle"]
3
1

MONITOR
912
370
999
419
Healthy Young
count turtles with [shade-of? blue color and color < 106 and shape = "circle"]
3
1

MONITOR
912
416
999
465
Infected Adult
count turtles with [shade-of? red color and color > 16 and shape = "circle"]
3
1

MONITOR
912
462
999
511
Infected Young
count turtles with [shade-of? red color and color < 16 and shape = "circle"]
3
1

PLOT
654
511
999
631
Meanderers (Circles)
Time (years)
Number
0.0
100.0
0.0
100.0
true
true
PENS
"Healthy Adult" 1.0 0 -6710785 true
"Healthy Young" 1.0 0 -16776961 true
"Infected Adult" 1.0 0 -26215 true
"Infected Young" 1.0 0 -65536 true

MONITOR
366
218
434
267
Healthy
count turtles with [shade-of? blue color]
3
1

MONITOR
366
266
434
315
Infected
count turtles with [shade-of? red color]
3
1

SLIDER
212
445
367
478
Infection_Radius
Infection_Radius
0
15
3
1
1
NIL

CHOOSER
271
55
367
100
Left_Quadrant
Left_Quadrant
"Top Outside" "Top Center" "High Outside" "High Center" "Low Outside" "Low Center" "Bottom Outside" "Bottom Center"
5

CHOOSER
271
97
367
142
Right_Quadrant
Right_Quadrant
"Top Outside" "Top Center" "High Outside" "High Center" "Low Outside" "Low Center" "Bottom Outside" "Bottom Center"
1

SLIDER
163
10
274
43
Density_Right
Density_Right
0
100
86
1
1
NIL

TEXTBOX
927
10
1017
28
Travelers

TEXTBOX
922
265
1012
283
Meanderers

SLIDER
0
280
212
313
Healthy_Traveler_Fertility
Healthy_Traveler_Fertility
0
100
87
1
1
NIL

SLIDER
212
214
367
247
Percent_Recovery
Percent_Recovery
0
100
5
1
1
NIL

SLIDER
212
280
367
313
Deadly_Mutations
Deadly_Mutations
0
100
7
1
1
NIL

SLIDER
212
247
367
280
Recover_Immune_Time
Recover_Immune_Time
0
100
15
1
1
NIL

SLIDER
0
148
212
181
Susceptible_Infection_Lifespan
Susceptible_Infection_Lifespan
0
50
10
1
1
NIL

SLIDER
212
148
367
181
Susceptible_Below
Susceptible_Below
0
100
15
1
1
NIL

SLIDER
212
181
367
214
Susceptible_Above
Susceptible_Above
0
100
71
1
1
NIL

MONITOR
366
315
434
364
Susceptible
count turtles with [weak = true]
3
1

MONITOR
912
278
999
327
Total
count turtles with [shape = "circle"]
3
1

MONITOR
912
23
999
72
Total
count turtles with [shape = "default"]
3
1

MONITOR
366
462
434
511
Dead
dead
3
1

SLIDER
0
214
212
247
Healthy_Meanderer_Fertility
Healthy_Meanderer_Fertility
0
100
96
1
1
NIL

SLIDER
0
247
212
280
Infected_Meanderer_Fertility
Infected_Meanderer_Fertility
0
100
18
1
1
NIL

SLIDER
0
346
212
379
Healthy_Birth_Number
Healthy_Birth_Number
0
15
1
1
1
NIL

CHOOSER
271
10
366
55
Background
Background
1 "2A" "2B" "3A" "3B" "4A" "4B" "5A" "5B" "6A" "6B" "7A" "7B" "8A" "8B" "9A" "9B" "10A" "10B"
17

SLIDER
212
379
367
412
Meanderer_Moves
Meanderer_Moves
0
100
20
1
1
NIL

SLIDER
212
412
367
445
Traveler_Moves
Traveler_Moves
0
100
40
1
1
NIL

TEXTBOX
0
10
90
42
Initial\nConditions

SLIDER
0
43
128
76
Travelers_Left
Travelers_Left
0
500
97
1
1
NIL

SLIDER
59
10
163
43
Density_Left
Density_Left
0
100
50
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This program models the spread of an infectious disease.


HOW IT WORKS
------------
Organisms move randomly about. Travelers are meant to move more than meanderers. Move_Factor allows the user to select how many times farther travelers move than meanderers. When a healthy organism moves onto a patch within Infection_Radius patches of another organism that is infected there is a certain chance that it will also become infected ("Contact_Infection"). Various backgrounds provide the effect of natural boundaries between populations.


HOW TO USE IT
-------------
Initial Conditions
Density_Left: The density that organisms on the left will start.
Density_Right: The density that organisms on the right will start.
Travelers_Left: The number of triangle shapes present at the setup of the program on 			the left side of the screen.
Travelers_Right: The number of triangle shapes present at the setup of the program on 			 the right side of the screen.
Meanderers_Left: The number of circle shapes present at the setup of the program on 			 the left side of the screen.
Meanderers_Right: The number of circle shapes present at the setup of the program on 			  the right side of the screen.
%_Infected_Left: The exact percent of organisms that are infected on the left at the 			 start of the program.
%_Infected_Right: The exact percent of organisms that are infected on the right at the 			  start of the program.
Background: The boundary setup. The increasing number loosely represents a more 		    restrictive boundary.
Left_Quadrant: Position for organisms on the left to start.
Right_Quadrant: Position for organisms on the right to start.

Susceptible_Infection_Lifespan: The number of years before the organism dies once 					infected during the susceptible period (Above/Below).
Susceptible_Below: The age below which organisms are more susceptible to 				   the disease.
Susceptible_Above: The age above which organisms are more susceptible to the disease.
Percent_Recovery: The percent chance that an infected organism will become healthy.
Recover_Immune_Time: The time an organism is immune to the disease after recovery.
Infection_Lifespan: The number of years before the organism dies once infected. If its 			    age exceeds the Healthy_Lifespan it will die regardless.
Deadly_Mutations: The percent chance that an infected organism will experience a deadly 	          mutation that will take ten years off its life.
_ _Fertility: The percent chance that an organism will successfully 				      reproduce given the chance. If an infected organism breeds with a healthy 	      one, the fertility of each are multiplied together to produce an overall 		      probability.
Procreation_Age: The age at which an organism can reproduce. (marked by a change to a 			 lighter color)
Healthy_Birth_Number: The number of offspring per procreation event. (assuming both 		              partners are healthy)
Infected_Birth_Number: The number of offspring per procreation event. (assuming both 		               partners are infected) There is a 50% chance that the 				       Infected_Birth_Number will be produced in the case of a   			       healthy/infected reproduction.
Healthy_Lifespan: The number of years (Time) an organism will live assuming it is 			  healthy (blue).
Carrying_Capacity: If this number of organisms is met in a radius of five (pixels), no 		           reproduction will take place.
Meanderer_Moves: How many "steps" meanderers take during the period of a year. 	                         (relative to pixel size)
Traveler_Moves: How many "steps" travelers take during the period of a year.
Contact_Infection: The percent chance that a healthy organism will be infected if it 			   comes in contact with an infected organism.
Infection_Radius: The radius that a infected organism must be in of a healthy organism 		  to infect it. Notice that contact infection plays a role.
File_Output: To output values to an excel spreadsheet.
Clear_File: Deletes previous information instead of appending to past results.
File_Time_Interval: The interval that values are sent to the file.


THINGS TO NOTICE
----------------
One important thing to understand is that infections do not proliferate themselves exclusively by reproducing. Their main sustenance is preying upon healthy individuals. Keep in mind that even though the infection radius may increase, barriers will prevent those on the other side of an infected organism even though they may be in the radius. Also, deadly mutations and recovery occur at the time of infection so that it only happens once even though this would happen at random times but the effect is the same. The varying fertilities allow the inclusion of the observation that meanderers often procreate more than travelers and to realize its affect on disease propagation.


THINGS TO TRY
-------------
Try different backgrounds while keeping everything else the same. Observe how the different variables lead to a rapid or steady spread of disease. Be sure to keep all the variables in mind because one extreme value can offset the effect of the others drastically. Try any combination and examine the different equilibriums or lack thereof. The graphs also reveal a lot about trends. You can even utilize the spreadsheet formatted output for further graphical analysis.


EXTENDING THE MODEL
-------------------
The study of the spread of infectious disease can have a countless number of variables such as the latency of the disease or the contamination time of a certain area. Most variables were included that could reasonably fit on the screen. Algorithms for infection radius were limited by the in-radius function not being able to recognize boundaries.


NETLOGO FEATURES
----------------
The graphical tools allow an immediate analysis of trends.

RELATED MODELS
--------------
Virus is a more basic model. The HIV model is also based on a similar idea.

CREDITS AND REFERENCES
----------------------
Produced by Joe Glessner (jglessner@usip.edu) under the direction of Dr. James Johnson
University of the Sciences in Philadelphia
2004
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
