
globals [clock halfedge current_area shape_list color_list expired

  t agent-id rnd crnd
  orders ordervalues
  costs inventory
  relationship trade-q msg-clr del-clr requests color-incr
  personality weight prod_limit bias shape_need money
  c_c c_t t_c t_t c_inv t_inv current_round
  run_c_c run_c_t run_t_c run_t_t  run_c_inv run_t_inv ; these values are the running totals for the different statistics

  travel-condition   ; this variable is needed to shift the conditions to travelers
  double-condition   ; this variable is needed to shif thte conditions to double

  number_nodes      ; initially a slider variable, has now been fixed at 10

  distracted_col     ; this is an agentset of all the collocators get distracted

  stat-flag ; set stat-flag to true when there is a statistically significant result between the conditions

  cc-ct-stat
  cc-tc-stat
  cc-tt-stat
  ct-tc-stat
  ct-tt-stat
  tc-tt-stat
  col-tel-stat

  id_list

]

breeds [agents officers edges nodes delayed-requests round-data]

round-data-own [col-col col-tel tel-col tel-tel col_inventory tel_inventory cc-list ct-list tc-list tt-list cinv-list tinv-list]

nodes-own [out-edges in-edges grabbed? shape_inventory production_limit biases person_type my_need inbox delay id
          sell-to]
edges-own [a b turtle_traffic]

turtles-own []

delayed-requests-own [delay messages]

patches-own [ areas ]

;-------------- beginning of setup -------------------------------
;purpose: set up the initial configuration of the experiment.
;when called: before an experimental play
;called by: the "setup standare" "setup travel" and "setup double" buttons
;receives: none
;returns: none
;remarks: it initiates the configuration of the experiment by calling the procedure "initiate-round."

to setup
  ca ;clear everything on the screen
  set number_nodes 10
  set environ "in-out"
  set travel-condition false
  set double-condition false
  set current_round 1 ;set the first round as round 1
  initiate-first-round ;go to the "initiate-round" procedure to set up the initial configuration of the experiment.
end

to setup-travel
  ca ;clear everything on the screen
  set number_nodes 10
  set environ "in-out"
  set double-condition false
  set travel-condition true
  set current_round 1 ;set the first round as round 1
  initiate-first-round ;go to the "initiate-round" procedure to set up the initial configuration of the experiment.
end

to setup-double
  ca ;clear everything on the screen
  set number_nodes 10
  set double-condition true
  set travel-condition false
  set environ "doubles"
  set current_round 1 ;set the first round as round 1
  initiate-first-round ;go to the "initiate-round" procedure to set up the initial configuration of the experiment.
end

;--------------- end of setup -----------------------------

;----------------beginning of setup-nodes----------------------------
;purpose: set up the initial configuration of the agents (subjects of the experiment)
;when called: during the play, in the beginning of each round
;called by: the "initiate-round" procedure
;receives: none
;returns: none
;remarks:

to setup-nodes

  locals [i]


  cct-nodes number_nodes
  [
    set out-edges []
    set in-edges [] ;out-edges and in-edges are for the lines connecting each pair of agents

    set shape_inventory [ 0 0 0 0 0 ] ;the shapes bought by each agent will be recorded by this variable
    set inbox [] ;all the incoming requests will be stored by this variable
    set delay 1 ;a variable reflecting the time delay
    set sell-to [] ;a variable recording how many shapes each agent sold to others
    set i 0
    while [i < number_nodes][
    set sell-to lput 0 sell-to
    set i i + 1
    ]


    set production_limit prod_limit ;set the production limit of each agent.

    set id first id_list
    set id_list butfirst id_list

    set shape first shape_list
    set shape_list lput shape shape_list
    set shape_list butfirst shape_list ;assign a shape to an agent

    set color first color_list
    set color color
    set color_list lput color butfirst color_list ;assign a color to an agent

    set my_need first shape_need
    set shape_need lput my_need butfirst shape_need ;my_need is the variable reflecting what shapes
    ;and how many each agent need to fill their orders.

    set grabbed? false

    set size 8
;    if environ = "doubles" [ set heading heading + 12 ]
    forward 50 ;define the size and location of each agent on the screen.
;  show "who" + who
;  show "id" + id
  ]
end
;--------------end of setup-nodes

;--------------beginning of go----------------------
;purpose: start the experiment and keep it running
;when called: in the beginning of an experiment
;called by: the "go" button
;receives: none
;returns: none
;remarks:

to go
  turtle-trading
  if (current_round = rounds)[stop]
end

to go-traveler
  set travel-condition true
  turtle-trading
  if (current_round = rounds)[stop]
end

to go-double
  set double-condition true
  turtle-trading
  if (current_round = rounds)[stop]
end

;--------------end of go----------------------------

;--------------beginning of turtle-trading----------
;purpose: implement the trading behaviors among the subjects
;when called: during the play, in the beginning of each time period
;called by: the "go" procedure
;receives: none
;returns: none
;remarks: this procedure is also called by itself to iterate through all the agents and called by
;         "initiate-round" at the beginning of each round
;         Note: there is a bug in printing log file.  The data of the last round of an experiment
;         will be written the total-number-of-rounds times. For example, if we run 2 rounds in an
;         experiment, the data for the second round will be written twice in the log file.

to turtle-trading

  locals [condition]; condition reflects what treatments the agents are in.

    if (travel-condition) [ if (current_round >= 3) [set environ "travelers"] ]    ; sets up the travelers condition
    if (double-condition) [set environ "doubles"]  ; sets up the doubles condition

    set expired count nodes with [production_limit <= 0] ; count how many agents have sold out all the shapes
    if (expired = number_nodes)[ ;when all the agents are sold out, record their inventories in the log file.

    ask nodes [show sell-to]

    if logfile? [ do-logfile ]

    if (current_round mod 5 = 0 and statistics?)
        [ run-exhaustive-t-tests ]    ; run t-test on every condition

    ifelse on-screen-data? [ log-round-data ] [update-data-summary]
        update-totals
        do_plots_rt
    ifelse (current_round = rounds)
    [stop] ;if all the rounds have been run through, stop the experiment.
    [
        set current_round current_round + 1

        initiate-round

        ] ;if the experiment has not gone through all the rounds, go to a new round.
   ]

  set t t + 1 ;increment the time period

  if (any? delayed-requests with [delay = t])
  [send-delayed-requests delayed-requests with [delay = t]]; if there are delayed requests scheduled to be sent in this time period, send them out.

  while [any? nodes with [delay = t]][
  set agent-id random-one-of nodes with [delay = t] ;find out all the agents that can act in this period, and randomize their order to act.

  ifelse (empty? inbox-of agent-id or production_limit-of agent-id <= 0)
  [turtle-buy agent-id] ;if an agent's inbox is empty or has sold out all the shapes, it can send out a buying request.
  [turtle-sell agent-id] ;if an agent has requests in its inbox and it still has shapes to sell, fulfill one of the requests.
  ; Note: an agent can only buy OR sell, not buy and sell, in each time period.
     ifelse (empty? areas-of agent-id) ; decide in which future period an agent will act.

       [set delay-of agent-id t + 1 + cost_of_telecommuting]
       [
         ifelse (member? who-of agent-id distracted_col)
         [set delay-of agent-id t + 1 + time_of_distraction]
         [set delay-of agent-id t + 1]
         ]


];closing the "while" function
turtle-trading ;repeat this procedure till all the agents have sold out.
end
;-------------------end of turtle-trading---------------------------------------

;-------------------beginning of turtle-buy----------------------------------
;purpose: implement the buying behavior of agents (subjects in an experiment)
;when called: during the play
;called by: the "turtle-trading" procedure
;receives: the ID of the agent who is in action
;returns: none
;remarks:

to turtle-buy [buyer_id]

  locals [
     shape_to_request quantity potential_seller i j buyer_need long
  ]

    set i 3
    set j 0 ;i & j are used to control the iteration through an agent's shape need (the my_need variable)
    set shape_list [ "circle" "box" "x" "diamond" "triangle" ]
    set potential_seller nodes with [shape = "cross"];"cross" is an nonexistant shape. It is used to make the potential_seller set empty.
    ask buyer_id [set buyer_need my_need]
    set quantity 0
    set shape_to_request ""


while [not empty? buyer_need][
  set long length buyer_need
  set i random long ;position of the shape
  set j item i buyer_need ;quantity
  if (j > 0)[set shape_to_request item i shape_list]
  set potential_seller nodes with [shape = shape_to_request and production_limit > 0]
  set quantity j
  ifelse (count potential_seller > 0) [
    set buyer_need []
  ]
  [set buyer_need remove-item i buyer_need]
]



if (count potential_seller = 0) [stop];if no agent can sell the shape, stop the turtle-buy procedure, and go back to turtle-trading.

  send-request buyer_id potential_seller shape_to_request quantity ;if some agents can sell the shape, send a request by calling the "send-request" procedure.

end
;-------------------end of turtle-buy-------------------------------


;-------------------beginning of send-request----------------------
;purpose: send a request from the buyer to a seller
;when called: during the play
;called by: the "turtle-buy" procedure
;receives: the ID of the buyer, the set of the potential sellers (those who can sell the shape), the shape to request, and the amount to request.
;returns: none
;remarks: the favoritism in requesting is reflected by the algorithm to pick a seller out of the potential seller set

to send-request [agent-from potential_seller shape_to_request quantity]

  locals [condition collocator agent-to request buyer]

  set shape_list [ "circle" "box" "x" "diamond" "triangle" ]
  set condition areas-of agent-from ;find out whether the buyer is a collocator or a telecommuter
  set collocator nobody

  ask agent-from[
    set buyer who
  ]

  ifelse favoritism
  [ ; start the favoritism rule
  if not (empty? condition) [
  set collocator random-one-of nodes with [shape = shape_to_request
                                            and areas = condition and production_limit > 0]

  ] ; if the buyer is a collocator, pick a seller from the collocating potential sellers


  ifelse (collocator = nobody)
  [set agent-to random-one-of potential_seller] ;if no collocators can sell, randomly pick a seller among all the potential sellers
  [set agent-to collocator]
  ] ;end the favoritism rule
  [set agent-to random-one-of potential_seller] ; else, the first-come-first-serve rule

  if (agent-to = nobody)
  [stop] ; if no agent in the experiment can sell, stop the turtle-request procedure, and go back to turtle-trading


  set request word quantity buyer ; compose the request. Each request is a string of two numbers.  The first number is the amount of
  ;                                 shapes to request, and the second number is the ID of the buyer.

  ifelse (cost_of_telecommuting = 0)
  [ask agent-to[
    set inbox lput request inbox
    ] ; if there is no time delay, send out the request immediately.
  ]
  [;else start

  if (not empty? areas-of agent-from and not empty? areas-of agent-to)[
      ask agent-to[
      set inbox lput request inbox
      ] ; if there is time delay, but both the seller and the buyer are collocators, send out the request immediately.
  ]

  if (not empty? areas-of agent-from and empty? areas-of agent-to)[ ; if the buyer is a collocator but the seller is a telecommuter,
  ;                                                                   schedule the request to be sent in a later time period

    save-delayed-requests request agent-to

  ]

  if (empty? areas-of agent-from)[ ; if the buyer is a telecommuter, schedule the request to be sent in a later period.

    save-delayed-requests request agent-to

  ]

 ];else end
end
;----------------end of send-request-----------------------------

;----------------beginning of save-delayed-request---------------
;purpose: send requested delayed from previous periods
;when called: during the play
;called by: the "turtle-trading" procedure
;receives: the set of requests scheduled to be sent in the current time period
;returns: none
;remarks:

to save-delayed-requests [request agent-to]

  locals [temp i j my_message]

  set i id-of agent-to

  ifelse (any? delayed-requests with [delay = t + 1 + cost_of_telecommuting])
      [set temp random-one-of delayed-requests with [delay = t + 1 + cost_of_telecommuting]
       ask temp [
         set my_message item i messages
         set my_message lput request my_message
         set messages replace-item i messages my_message
         ]
      ]
      [cct-delayed-requests 1
        [
          set delay t + 1 + cost_of_telecommuting
          set messages []
          while [j < number_nodes][
          set messages lput [] messages
          set j j + 1
          ]
          set my_message item i messages
         set my_message lput request my_message
         set messages replace-item i messages my_message
        ]
      ]


end


;----------------end of save-delayed-request---------------------

;----------------beginning of send-delayed-requests--------------
;purpose: send requested delayed from previous periods
;when called: during the play
;called by: the "turtle-trading" procedure
;receives: the set of requests scheduled to be sent in the current time period
;returns: none
;remarks:

to send-delayed-requests [agent-requests]

  locals [temp i]

  ask random-one-of agent-requests[
    set temp messages ; retrieve all the requests scheduled to be sent in the current period
    die
  ]

  ask nodes [
    set i id
    set inbox sentence inbox item i temp
  ]


end
;-------------------end of send-delayed-requests-------------------------

;------------------beginning of turtle-sell----------------------------
;purpose: implement the selling behavior of the agents
;when called: during the play
;called by: the "turtle-trading" procedure
;receives: the ID of the seller.
;returns: none
;remarks: the favoritism is reflected by how the seller pick a request to fulfill

to turtle-sell [seller_id]

  locals [request quantity buyer shape_to_deliver
    my_inbox my_area n demand
    endx endy move_turtle
  ]

  set shape_list [ "circle" "box" "x" "diamond" "triangle" ]

  set request []

  ask seller_id[
    set my_inbox reverse inbox
    set shape_to_deliver position shape shape_list
  ]

  set my_area areas-of seller_id

  ifelse (favoritism)
  [ ; start the favoritism selling rule


  foreach my_inbox [
      set buyer read-from-string but-first ?

      if (areas-of turtle buyer = my_area and not empty? my_area)
      [set request ? ; if the seller is a collocator, pick the first request sent by another collocator
       set n position ? reverse my_inbox

      ]
  ]

  if (empty? request)[ ; if there is no request from other collocators, or if the seller is a telecommuter, pick the first request in the seller's inbox.
  set my_inbox reverse my_inbox
  set request first my_inbox
  set n 0
  ]
  ] ; end of the favoritism selling rule
  [
  set my_inbox reverse my_inbox
  set request first my_inbox
  set n 0
  ];end else, the f-c-f-s rule



  set quantity read-from-string first request ; retrieve the amount to sell from the request
  set buyer read-from-string butfirst request ; retrieve the buyer ID from the  request

  ask turtle buyer[
    set demand item shape_to_deliver my_need
  ]

  if (quantity > demand)
  [set quantity demand] ; if the buyer does not need as many shapes as it requests (the buyer might have got the same shape from the
;                         the other producer before the current seller responds), set the amount to sell as the current need of the buyer.
;                         This is to prevent any agent from buying more shapes than they need.

  if (quantity <= 0) [
  ask seller_id [set inbox filter [? != request] inbox] ; if the buyer does not need the shape anymore, remove all the buyer's requests from the seller's
  ;                                               inbox and stop the turtle-sell procedure.
  stop]

  ask seller_id[

    if (quantity > production_limit)
    [set quantity production_limit] ; if the buyer requests more than the seller can produce, set the amount to sell to the number of shapes left of the seller.
    set production_limit production_limit - quantity ; deduct the amount sold from the seller's production limit
    set inbox filter [? != request] inbox ; remove the fulfilled request from the seller's inbox

    if production_limit = 0 [set color color - 2] ; if the seller is sold out, shrink its size on the screen.
    if sum shape_inventory = 8 [ set color 75]
  ]

  ask turtle buyer[ ; update the buyer's inventory and shape needed variable
    set shape_inventory replace-item shape_to_deliver shape_inventory (item shape_to_deliver shape_inventory + quantity)
    set size 5 + (sum shape_inventory)
    set my_need replace-item shape_to_deliver my_need (item shape_to_deliver my_need - quantity)
    set endx xcor
    set endy ycor
  ]

  ifelse (empty? areas-of turtle buyer) ; update the total shapes bought by collocators and telecommuters.
  [set t_inv t_inv + quantity]
  [set c_inv c_inv + quantity]

  if (empty? areas-of seller_id and empty? areas-of turtle buyer) ; update the 2 x 2 who-is-selling-to-whom table on the screen
    [set t_t t_t + quantity]
  if (empty? areas-of seller_id and not empty? areas-of turtle buyer)
    [set t_c t_c + quantity]
  if (not empty? areas-of seller_id and empty? areas-of turtle buyer)
    [set c_t c_t + quantity]
  if (not empty? areas-of seller_id and not empty? areas-of turtle buyer)
    [set c_c c_c + quantity]

   ask seller_id[ ; update the sell-to variable of the seller
    set sell-to replace-item (id-of turtle buyer) sell-to (item (id-of turtle buyer) sell-to + quantity)

  ]

  while [quantity > 0][ ; demonstrate the delivery of shapes on the screen
  ask seller_id[
    hatch 1 [
             set size 5
             set heading towardsxy-nowrap endx endy
             set move_turtle self
          ]
    ask move_turtle [
          fd distancexy-nowrap endx endy
          find_edge seller_id turtle buyer
          die
       ]

  ]
  set quantity quantity - 1
]

do_plots ; update the plots on the interface.

end

;--------------------------end of turtle-sell------------------------

;--------------------------beginning of initiate-round--------------
;purpose: set up initial configuration in the beginning of a round
;when called: during the play
;called by: the "setup" procedure in the beginning of an experiment and the "turtle-trading" procedure during the play
;receives: none
;returns: none
;remarks:

to initiate-round
locals [temp i]

 ask nodes [die]
 ask edges [die]
 ask delayed-requests [die]
 cp

 set-current-plot "inventory"
 clear-plot

  set t 0
  set prod_limit 6
  set clock 0
  set current_area 0
  set expired 0
  set c_inv 0
  set t_inv 0
  set t_t 0
  set t_c 0
  set c_t 0
  set c_c 0
  set i 0

  set shape_need [[0 3 3 1 1][1 0 3 3 1][1 1 0 3 3][3 1 1 0 3][3 3 1 1 0]]

  set-default-shape edges "line"

  set shape_list [ "circle" "box" "x" "diamond" "triangle" ]
  set color_list [ yellow blue brown white red violet orange pink gray green ]

  set id_list []
  while [i < number_nodes][
  set id_list lput i id_list
  set i i + 1
  ]

  setup-nodes ; call the "setup-nodes" procedure to assign initial configuration to the agents

  ask nodes
  [
     ask nodes with [self != myself and shape != shape-of myself]
        [form-edge myself]
  ] ; draw the lines connecting each pair of agents

set-environment

  set distracted_col random-n-of #_of_distraction nodes with [not empty? areas]
  set distracted_col values-from distracted_col [who]

end



;----------------------end of initiate-round---------------------------------

;--------------------------beginning of initiate-first-round--------------
;purpose: set up initial configuration in the beginning of a round
;when called: during the play
;called by: the "setup" procedure in the beginning of an experiment and the "turtle-trading" procedure during the play
;receives: none
;returns: none
;remarks:

to initiate-first-round
locals [temp i]

 ct
 cp
 clear-all-plots

  set-variables

  set shape_need [[0 3 3 1 1][1 0 3 3 1][1 1 0 3 3][3 1 1 0 3][3 3 1 1 0]]

  set-default-shape edges "line"

  set shape_list [ "circle" "box" "x" "diamond" "triangle" ]
  set color_list [ yellow blue brown white red violet orange pink gray green ]

  set i 0
  set id_list []
  while [i < number_nodes][
  set id_list lput i id_list
  set i i + 1
  ]

  setup-nodes ; call the "setup-nodes" procedure to assign initial configuration to the agents

  ask nodes
  [
     ask nodes with [self != myself and shape != shape-of myself]
        [form-edge myself]
  ] ; draw the lines connecting each pair of agents

set-environment

  set distracted_col random-n-of #_of_distraction nodes with [not empty? areas]
  set distracted_col values-from distracted_col [who]

if (statistics?) [ make-summary-data-turtle ]

end

;-------------------------end of initiate-first-round--------------------

;--------------------------beginning of set-variables--------------
;purpose: set up initial values of varialbes
;when called: during the play
;called by: the "initiate-first-round" procedure in the beginning of an experiment
;receives: none
;returns: none
;remarks:

to set-variables

  set t 0
  set prod_limit 6
  set clock 0
  set current_area 0
  set expired 0
  set c_inv 0
  set t_inv 0
  set t_t 0
  set t_c 0
  set c_t 0
  set c_c 0
  set run_c_c 0
  set run_c_t 0
  set run_t_t 0
  set run_t_c 0
  set stat-flag false
  set cc-ct-stat false
  set cc-tc-stat false
  set cc-tt-stat false
  set ct-tc-stat false
  set ct-tt-stat false
  set tc-tt-stat false
  set col-tel-stat false

end

;----------------------end of set-variables---------------------------------

;---------------------beginning of plot procedures----------------------------------
;purpose: plot the "inventory" and "running_average" charts on the screen
;when called: during the play
;called by: the "turtle-sell" procedure
;receives: none
;returns: none
;remarks:

to do_plots

  set-current-plot "inventory"
  auto-plot-on
  set-current-plot-pen "col"

  plot c_inv ; the red line shows the total number of shapes bought by collocators

  set-current-plot-pen "tel"

  plot t_inv ; the black line shows the total number of shapes bought by telecommuters

end

to do_plots_rt

  set-current-plot "running-averages"
  auto-plot-on

  set-current-plot-pen "col_col"
  plot (run_c_c / (current_round)) ; the red line shows the total number of shapes bought by collocators

  set-current-plot-pen "col_tel"
  plot (run_c_t / (current_round)) ; the black line shows the total number of shapes bought by telecommuters

  set-current-plot-pen "tel_col"
  plot (run_t_c / (current_round)); the red line shows the total number of shapes bought by collocators

  set-current-plot-pen "tel_tel"
  plot (run_t_t / (current_round)); the black line shows the total number of shapes bought by telecommuters

  set-current-plot "average-inventory"
  auto-plot-on

  set-current-plot-pen "col"
  plot (run_c_inv / (current_round)) ; the red line shows the average number of shapes bought by cols

  set-current-plot-pen "tel"
  plot (run_t_inv / (current_round)) ; the black line shows the average number of shapes bought by tels

end

;--------------------- end of plot procedures------------------------------------------

;--------------------- start log information -----------------------------------
;purpose: write to the log files
;when called: during the play
;called by: the "turtle-trading" procedure
;receives: none
;returns: none
;remarks:

to do-logfile

locals [condition]

    ask nodes[
    ifelse (empty? areas-of self)
    [set condition 1]
    [set condition 0]
      file-open "logfile"
    file-print (word date-and-time "\t" current_round "\t" favoritism "\t"
      cost_of_telecommuting "\t" #_of_distraction "\t" time_of_distraction "\t"
      condition "\t" who "\t" item 0 shape_inventory "\t"
      item 1 shape_inventory "\t" item 2 shape_inventory "\t"
      item 3 shape_inventory "\t" item 4 shape_inventory "\t"
      sum shape_inventory "\t" sell-to)
    ]
    file-close

    file-open "roundlog"
    file-print (word date-and-time "\t" current_round "\t" favoritism "\t" cost_of_telecommuting
                "\t" #_of_distraction "\t" time_of_distraction
                "\t" c_inv "\t" t_inv "\t" c_c "\t" c_t "\t" t_t "\t" t_c)
    file-close

end

;--------------------- end log information -------------------------------------


to find_edge [s_to s_from] ;node procedure

  locals [cur_edge lines]

  set cur_edge random-one-of edges with [(s_to = a and s_from = b) or (s_to = b and s_from = a)]

  set lines ["line" "line1" "line2" "line3" "line4"]

  if cur_edge != nobody [
     ask cur_edge [
        set shape item turtle_traffic lines
        set color 96
        set turtle_traffic turtle_traffic + 1
        set label turtle_traffic

     ]
   ]

end

to-report connected-to? [other-node]  ;; node procedure
  report member? other-node (map [b-of ?] out-edges)
end

to connect-to [other-node]  ;; node procedure
  hatch 1
    [ set breed edges
      set turtle_traffic 0
      set color 81.7; 109.9
      set a myself
      set b other-node
      set (out-edges-of a) lput self (out-edges-of a)
      set (in-edges-of b) lput self (in-edges-of b)
;      set label who
      reposition ]
end

to reposition  ;; edge procedure
  setxy (xcor-of a) (ycor-of a)
  set heading towards-nowrap b
  set size distance-nowrap b
  jump (distance-nowrap b) / 2
end

to form-edge [other-node];; node procedure
;  set other-node random-one-of (nodes in-radius-nowrap 150 with [self != myself])
  if (other-node != nobody) and
     (not connected-to? other-node) and
     (not value-from other-node [connected-to? myself])
    [ connect-to other-node ]
end

to disappear  ;; edge procedure
  set (out-edges-of a) remove self (out-edges-of a)
  set (in-edges-of b) remove self (in-edges-of b)
  die
end


;--------------------------------------------------------

to kill_links

  if mouse-down? [
     ask turtles-at round mouse-xcor round mouse-ycor [
           disappear
     ]
  ]

end

;--------------------------------------------------------

to node_drag

locals [num]

set num 0

  while [mouse-down?]
  [
     ask turtles-at round mouse-xcor mouse-ycor [
        if num = 0 [
           set grabbed? true
           set num 1
        ]
     ]

     ask nodes with [ grabbed? ] [
        without-interruption [
           setxy mouse-xcor mouse-ycor
           foreach out-edges [ ask ? [ reposition ] ]
           foreach in-edges [ ask ? [reposition ] ]
        ]
     ]
  ]

  ask nodes [ set grabbed? false ]
  set num 0

end

;--------------------------------------------------------

to box_mouse

locals[startx starty endx endy draw_turtle]

;  ask patches
;     [
;        set areas[]
;
;    if ((pycor = (- screen-edge-y + 10) or pycor = screen-edge-y) and (pxcor >= (0 - 10) and pxcor <= screen-edge-x)) [
;    set pcolor black
;    ]
;    if ((pycor >= (- screen-edge-y + 10) and pycor <= screen-edge-y) and (pxcor = screen-edge-x or pxcor = (0 - 10) ))[
;    set pcolor black
;    ]
;
;  ]

  if mouse-down? [
     set startx mouse-xcor
     set starty mouse-ycor

    while [mouse-down?] [

     cct 1 [ set draw_turtle self ]

     ask draw_turtle [
         set color green
         set xcor startx
         set ycor mouse-ycor
         set pcolor color
         die
      ]

      cct 1 [ set draw_turtle self ]

      ask draw_turtle [
         set color red
         set ycor starty
         set xcor mouse-xcor
         set pcolor color
         die
       ]

       set endx mouse-xcor
       set endy mouse-ycor

  ]

       cct 1 [ set draw_turtle self ]

       ask draw_turtle [
          set color red
          set xcor startx + 1
          set ycor starty

          while [ xcor < endx ] [
                set heading 180
                repeat (starty - endy) [
                   fd 1
                   set areas fput current_area areas

                   ifelse pcolor = 0 [set pcolor 59] [set pcolor pcolor + 20]
                ]

                set xcor xcor + 1
                set ycor starty
          ]

          die
       ]

       cct 1 [set draw_turtle self]

       ask draw_turtle [
          set color yellow
          set xcor mouse-xcor
          set ycor mouse-ycor
          pd
          set heading 0
          fd starty - endy
          lt 90
          fd endx - startx
          die
       ]

      cct 1 [set draw_turtle self]

      ask draw_turtle [
          set color blue
          set xcor mouse-xcor
          set ycor mouse-ycor
          pd
          set heading 270
          fd endx - startx
          rt 90
          fd starty - endy
          die
       ]

    set current_area current_area + 1

   ; change_biases

]
end

to send_shapes

locals [move_turtle to_go]

  foreach values-from nodes [self] [
    ask ? [

    hatch 1 [
       set size 5
       set heading towardsxy 0 0
       set move_turtle self
    ]

    ask move_turtle [
          set to_go 0

          while [ to_go < 100 ][
             fd 1
             set to_go to_go + 1
          ]
          die
    ]
    ]
  ]

end

to change_biases ; node procedure

locals [in_node]

    ask nodes with [member? 0 areas] [
        set in_node self
        ask nodes with [member? 0 areas and self != myself] [
             set biases replace-item who biases (item who biases * bias)
        ]
    ]

       ;     [set shape_inventory replace-item 3 shape_inventory (item 3 shape_inventory + quant) ]

end

to set-environment

  if environ = "in-out" [

    cp
    ask patches
     [
        set areas[]
    if (pycor >= (- screen-edge-y + 10) and pycor <= screen-edge-y and pxcor >= (0 - 10) and (pxcor <= screen-edge-x - 12))
    [set areas[0] set pcolor pcolor + 61]
    if ((pycor = (- screen-edge-y + 10) or pycor = screen-edge-y) and (pxcor >= (0 - 10) and (pxcor <= screen-edge-x - 12))) [
    set pcolor red ]
    if ((pycor >= (- screen-edge-y + 10) and pycor <= screen-edge-y) and ((pxcor = screen-edge-x - 12) or pxcor = (0 - 10) ))[
    set pcolor red ] ; draw the collocating red box on the screen

    if on-screen-data? [       ; this is a short proceedure to draw a line around the area where the data is
    if (pxcor = (- screen-edge-x + 10 ))
      [ set pcolor 45] ]

    ]
]

  if environ = "travelers" [
  cp
  ask patches
     [

    set areas[]

    if ((pycor >= (- screen-edge-y + 55) and pycor <= screen-edge-y) and ((pxcor <= screen-edge-x - 12) and pxcor >= (- screen-edge-x + 12)))
    [set areas[0] set pcolor pcolor + 11.1 ] ; colors the patches
    if ((pycor = (- screen-edge-y + 55) or pycor = screen-edge-y) and (((pxcor <= screen-edge-x - 12) and pxcor >= (- screen-edge-x + 12)) or pxcor = (- screen-edge-x + 12 ))) [
    set pcolor blue ] ; draw top line and bottom line
    if ((pycor >= (- screen-edge-y + 55) and pycor <= screen-edge-y) and ((pxcor = screen-edge-x - 12) or pxcor = (- screen-edge-x + 12 )))[
    set pcolor blue ] ; draw left and right lines
    ; draw the collocators blue box on the screen

    if on-screen-data? [       ; this is a short proceedure to draw a line around the area where the data is
    if (pxcor = (- screen-edge-x + 10 ))
      [ set pcolor 45] ]
  ]
 ]


  if environ = "doubles" [

  cp

  ask turtles with [shape = "circle"] [

    set heading heading + 180
    fd 25
    without-interruption [
      foreach out-edges [ ask ? [ reposition ] ]
      foreach in-edges [ ask ? [reposition ] ]
    ]
  ]

  ask patches
     [

    set areas[]

    if (pycor >= (- screen-edge-y + 20) and pycor <= screen-edge-y and pxcor >= (0 - 10) and pxcor <= (screen-edge-x - 12))
    [set areas[0] set pcolor pcolor + 101]
    if ((pycor = (- screen-edge-y + 20) or pycor = screen-edge-y) and (pxcor >= (0 - 10) and pxcor <= (screen-edge-x - 12))) [
    set pcolor green ]
    if ((pycor >= (- screen-edge-y + 20) and pycor <= screen-edge-y) and ((pxcor = screen-edge-x - 12) or pxcor = (0 - 10) ))[
    set pcolor green ] ; draw the collocating red box on the screen

    if on-screen-data? [       ; this is a short proceedure to draw a line around the area where the data is
    if (pxcor = (- screen-edge-x + 10 ))
      [ set pcolor 45] ]

  ]
]

set current_area current_area + 1


end


to update-totals

  set run_c_c run_c_c + c_c      ; this procedures updates the running totals from the current experiment
  set run_c_t run_c_t + c_t
  set run_t_c run_t_c + t_c
  set run_t_t run_t_t + t_t
  set run_c_inv run_c_inv + c_inv
  set run_t_inv run_t_inv + t_inv

end

to log-round-data

  cct-round-data 1
    [
      set shape "box"
      set color ((current_round * 10) + 5)
      set size 5
      set xcor ( - screen-edge-x + 7)
      set ycor ( - screen-edge-y + 5 + (current_round * 5 ))
      set col-col c_c
      set col-tel c_t
      set tel-col t_c
      set tel-tel t_t
      set col_inventory c_inv
      set tel_inventory t_inv
      set label current_round
    ]

    update-data-summary

end

to update-data-summary

 ask round-data with [label = "S"]       ;;; this code will append to the summary turtle the data from the current round
   [
     set cc-list lput c_c cc-list
     set ct-list lput c_t ct-list
     set tc-list lput t_c tc-list
     set tt-list lput t_t tt-list
     set cinv-list lput c_inv cinv-list
     set tinv-list lput t_inv tinv-list
   ]

end

;--------------begninning of t-test procedure--------------
;purpose: do t-test
;when called: during the play
;called by: the "run-exhaustive-t-tests" procedure
;receives: two lists of data
;returns: none
;remarks:

to ttest [y1list y2list]

  ; the purpose of this proceedure is to return whether there is a significant difference between two resuts.

  locals [s1 s2 y1bar y2bar n1 n2 s sd tvalue df ttable critical]

  set ttable [100 12.706 4.303 3.182 2.776 2.571 2.447 2.365 2.306 2.262
              2.228 2.201 2.179 2.160 2.145 2.131 2.120 2.110 2.101 2.093
              2.086 2.08 2.074 2.069 2.064 2.06 2.056 2.052 2.048 2.045
              2.042 2.021 2 1.98 1.96] ; two-tail critical values at 0.05

  set s1 variance y1list
  set s2 variance y2list
  set y1bar mean y1list
  set y2bar mean y2list
  set n1 length y1list
  set n2 length y2list
  set df (n1 - 1) + (n2 - 1)
  set s ((n1 - 1) * s1 + (n2 - 1) * s2) / ((n1 - 1) + (n2 - 1))
  set sd sqrt (s * (n1 + n2) / (n1 * n2))
  set tvalue abs (y1bar - y2bar) / sd

  if (df <= 30)
    [set critical item df ttable
     if (tvalue > critical)
       [set stat-flag true

       ]

    ]

  if (df > 30 and df <= 40)
    [set critical item 31 ttable
     if (tvalue > critical)
       [set stat-flag true
;        show "Statistical significance found"
       ]
;       [show "Not Statistically significant"]
    ]

  if (df > 40 and df <= 60)
    [set critical item 32 ttable
     if (tvalue > critical)
       [set stat-flag true
;        show "Statistical significance found"
       ]
;       [show "Not Statistically significant"]
    ]

  if (df > 60 and df <= 120)
    [set critical item 33 ttable
     if (tvalue > critical)
       [set stat-flag true

       ]

    ]

  if (df > 120)
    [set critical item 34 ttable
     if (tvalue > critical)
       [set stat-flag true

       ]

    ]

end

;---------------------end of t-test procedure---------------------------

to run-exhaustive-t-tests

    locals [flag? test-list-1 test-list-2 cc-temp ct-temp tc-temp tt-temp cinv-temp tinv-temp]

    set flag? false

; set up four local list with the infomation from the summary stat turtle, this will be easier to manipulate

    ask round-data with [label = "S"]
    [
      set cc-temp cc-list
      set ct-temp ct-list
      set tc-temp tc-list
      set tt-temp tt-list
      set cinv-temp cinv-list
      set tinv-temp tinv-list
    ]

; run the six different test conditions and if any of them are significant, set significance reporter to true

    ttest cc-temp ct-temp
    set cc-ct-stat false
    if (stat-flag)                  ;; this little if statment is just a kluge to set the individual flags
      [
        set flag? true
        set cc-ct-stat true
        set stat-flag false
      ]

    ttest cc-temp tc-temp
    set cc-tc-stat false
    if (stat-flag)
      [
        set flag? true
        set cc-tc-stat true
        set stat-flag false
      ]

    ttest cc-temp tt-temp
    set cc-tt-stat false
    if (stat-flag)
      [
        set flag? true
        set cc-tt-stat true
        set stat-flag false
      ]

    ttest ct-temp tc-temp
    set ct-tc-stat false
    if (stat-flag)
      [
        set flag? true
        set ct-tc-stat true
        set stat-flag false
      ]

    ttest tt-temp ct-temp
    if (stat-flag)
      [
        set flag? true
        set ct-tt-stat true
        set stat-flag false
      ]

    ttest tt-temp tc-temp
    set tc-tt-stat false
    if (stat-flag)
      [
        set flag? true
        set tc-tt-stat true
        set stat-flag false
      ]

    ttest cinv-temp tinv-temp
    set col-tel-stat false
      if (stat-flag)
        [
          set flag? true
          set col-tel-stat true
          set stat-flag false
        ]

      if (flag?) [set stat-flag true]

end

to make-summary-data-turtle

    ; the purpose of this function is to get information from the data-turtles to use for the t-test
  cct-round-data 1
    [
      set shape "Circle"
      set color 133
      set size 8
      set xcor ( - screen-edge-x + 5)
      set ycor ( - screen-edge-y + 4)
      set cc-list []
      set ct-list []
      set tc-list []
      set tt-list []
      set cinv-list []
      set tinv-list []
      set label "S"

    ]

end

@#$#@#$#@
GRAPHICS-WINDOW
496
10
1241
596
73
55
5.0
1
30
1
1
1
0

CC-WINDOW
5
610
1250
705
Command Center

BUTTON
6
38
122
71
setup standard
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
138
86
236
119
go
go
NIL
1
T
OBSERVER
T
NIL

BUTTON
366
193
488
226
remove links
kill_links
T
1
T
OBSERVER
T
NIL

BUTTON
366
153
487
186
Node Reposition
node_drag
T
1
T
OBSERVER
T
NIL

BUTTON
366
233
489
266
add environment
box_mouse
T
1
T
OBSERVER
T
NIL

SLIDER
6
176
167
209
cost_of_telecommuting
cost_of_telecommuting
0
100
0
1
1
NIL

MONITOR
96
332
153
381
col_tel
c_t
0
1

MONITOR
96
385
153
434
tel_tel
t_t
0
1

TEXTBOX
4
373
94
391
seller

TEXTBOX
81
314
171
332
buyer

MONITOR
36
332
93
381
col_col
c_c
0
1

MONITOR
36
385
93
434
tel_col
t_c
0
1

PLOT
8
463
179
587
inventory
time
parts
0.0
30.0
0.0
35.0
true
true
PENS
"col" 1.0 0 -65536 true
"tel" 1.0 0 -16777216 true

SLIDER
8
86
122
119
rounds
rounds
1
50
5
1
1
NIL

CHOOSER
366
37
480
82
environ
environ
"in-out" "travelers" "doubles"
0

BUTTON
366
87
478
120
change enviornment
set-environment
NIL
1
T
OBSERVER
T
NIL

BUTTON
248
37
351
70
setup double
setup-double
NIL
1
T
OBSERVER
T
NIL

BUTTON
137
37
234
70
setup travel
setup-travel
NIL
1
T
OBSERVER
T
NIL

SWITCH
191
154
340
187
on-screen-data?
on-screen-data?
0
1
-1000

PLOT
164
303
354
433
running-averages
Rounds
Average sold betewen
0.0
10.0
0.0
10.0
true
true
PENS
"col_col" 1.0 0 -65536 true
"col_tel" 1.0 0 -11352576 true
"tel_col" 1.0 0 -65281 true
"tel_tel" 1.0 0 -16776961 true

SWITCH
191
194
342
227
logfile?
logfile?
1
1
-1000

TEXTBOX
148
10
298
28
Setup Conditions

TEXTBOX
62
123
152
141
Variables

TEXTBOX
383
132
483
150
Interaction with environment

TEXTBOX
56
294
146
317
ROUND DATA

TEXTBOX
214
128
318
146
EXPERIMENT DATA

SWITCH
27
142
145
175
favoritism
favoritism
0
1
-1000

SLIDER
6
211
168
244
#_of_distraction
#_of_distraction
0
5
0
1
1
NIL

SLIDER
6
246
169
279
time_of_distraction
time_of_distraction
0
10
0
1
1
NIL

MONITOR
365
378
422
427
cc-ct
cc-ct-stat
0
1

MONITOR
430
379
487
428
tc-tt
tc-tt-stat
3
1

TEXTBOX
398
308
488
364
Statistical Significance for In-group Formation

SWITCH
190
233
343
266
statistics?
statistics?
0
1
-1000

PLOT
193
463
365
588
average-inventory
Rounds
parts
0.0
10.0
0.0
35.0
true
true
PENS
"col" 1.0 0 -65536 true
"tel" 1.0 0 -16777216 true

TEXTBOX
399
463
489
519
Statistical Significance for Performance Difference

MONITOR
393
526
476
575
col-tel
col-tel-stat
0
1

@#$#@#$#@
WHAT IS IT?
-----------

This is a computer simulation of a lab experiment, Shape Factory. Shape Factory is a game for studying the collaboration patterns among distributed teams. In this game there are ten 'players', each with a different color, and each capable of producing one of five specialty shapes. The shapes represent work that agents can do for each other, and the five different shapes represents different 'skills'. In the default versions of the game (as run with human subjects) there are ten players and five shapes, so each shape is produced by two players.Players request shapes from each other to fulfill the shape orders they receive at the beginning of a round. Each tries to collect as many shapes as they can to fulfill its orders.

Each player also receives requests and decides how its shapes will be distributed. Players can only produce a limited number of shapes (6) every round of the game, which mimics the realistic situation that people have limited time and attention, and must choose how to allocate it.The players can be placed in different configurations of being together (collocated) or separated (remote). This flexible arrangement allows us to study what happens to collaborating teams when they are in different geographic configurations. Findings from real subjects show that location is a strong determiner of collaboration patterns, and that remote people are sometimes, but not always at a disadvantage in these collaborations.This model was built to first replicate findings made with real subjects, and then extend the human subject data by A) allowing us to break down the causal mechanisms that may explain findings and B) allowing us to run new configurations that have never been run with real subjects.

HOW IT WORKS
------------

Understanding the Model:

Location is defined by a shaded box on the screen. Players within the same box are viewed as being in the same location. Players not inside any box are remote to everyone including each other (also sometimes called 'telecommuters').

The three setup buttons create configurations that mimic the three Shape Factory experimental conditions run to date. Location boxes can also be drawn manually with the Add Environment tool, by selecting the tool and then dragging a box. The players (nodes) and Links between players (links) can also be manipulated manually if the interaction button is selected (remember to unselect the interaction buttons before running the model).

Sequence of events each round:

First, a random player who is not delayed this round looks to see if they can produce any more shapes in this round, or if the production limit of 6 has been reached. If they still have shapes to send, the player will check to see if it has received requests from other players. If they have, they will fill one and only one request. If 'favoritism' is turned on, the collocated players will first try to fill the request of another player who is collocated with them; otherwise requests are filled in the order received. The transfer of a shape from one player to another is animated on the screen (sometimes the overall animation speed of the simulation must be reduced to see the shapes being delivered).

In a given round, if there are no pending requests for shapes, the selected player will send out one request of their own. It will first decide on one shape to try to acquire that round. If Favoritism is turned on, the player will first try to request the shape from the collocated producers of that shape. Otherwise, the player will randomly choose from all producers of that shape and send them a request.

Action repeats until all players have filled their production limits. When a player has reached its limit, this is indicated on the screen by enlarging the player icon. When all players have given away six shapes, one round of the game is complete. A Shape Factory game usually consists of multiple rounds. The number of rounds can be set by the "rounds" slider in the model interface. Each round, players receive new orders to try and fulfill and players are again able to sell an amount of objects equal to the production limit.

Understanding delay in the model:

When people are remote from each other in the real world, they must use Computer-mediated communications rather than informal verbal interactions. Computer-Mediate Communications are usually slower. Electronic mail messages, for example, take longer to write and usually take longer to get a response than do verbal messages. Even faster media, such as phone or Instant Messaging, still slow down communications when compared to face-to-face collaboration. 

To simulate this, a delay factor is included in this model, and is set by the cost_of_telecommuting slider. When delay is turned on, all requests to and from remote players take longer to arrive at their destination. Players cannot take any other actions while their messages are in transit. Only players collocated with each other can send requests that will arrive in the next time period. For all other transactions-- collocated to remote, remote to collocated, and remote to remote-- requests are delayed by the specified number of periods.

Understanding distraction in the model:

Collocated players can communicate with each other verbally, but sometimes this hurts as much as it helps! Real-world 'telecommuters' often report that they can work more efficiently without the distraction of the office. In experiments with real subjects we found that collocated groups sometimes engaged in distractive conversations that decreased their performance in comparison to other groups.
Distraction can be added into the Shape Factory model. A distraction takes players out of the action, and causes them to miss turns. (Note that 'Turns' are smaller units than 'Rounds'- one Round usually includes multiple Turns.) Sliders can be used to set the number of Collocated players who will be distracted on any given turn, and to set the amount of time they will be distracted.

Game Outcomes:

Success in this game is measured for each playerby how many parts they were able to acquire. Game statistics track how many parts each agent acquired, and from whom. These statistics are gathered in the monitors of the model, and each rounds data can be examined in the rounds "data turtle" if the on-screen-statistics switch is turned on.

HOW TO USE IT
-------------

Set up the experiment that you wish to see and then select go. The standard experiment is five rounds. At the end of each five round set, a series of statistical tests are run to show if there have been significant results at that point.

Preset configurations:

There are three preset experimental configurations in this model: standard, travelers, and doubles. These three correspond to experimental conditions run previously with human subjects. Other configurations can be created manually.

Standard - ten players are created with ten different colors and two each of five shapes. One area box is drawn to include five players, one of every shape. These are simulated collocators. The other five players are outside the box, and so are simulated telecommuters.

Travel- in this configuration, players simulate travel by changing places after three rounds of the game. Actually the collocated box is moved rather than moving the players but the effect is the same. (Note: location will always change after round 3, even if experiment is longer than 5 rounds.)

Doubles- in this configuration, the collocators and remote groups are unbalanced. Both circle players are in the collocated group, and both triangles are remotes. The groups are no longer self sufficient.

Variables:You can experiment with several experimental variables that may affect the trading patterns.

Favoritism- when favoritism is on, players who are in the same area (collocated) preferentially send requests to each other and fill each other's orders. When favoritism is in effect, collocated agents will only send requests to remote playerswhen the preferred collocator is out of parts, and will only fill requests from remote players after any requests from collocators have been filled. Players who are remote have no such preference and treat all agents equally.

Cost_of_telecommuting- using this slider you can set the amount of delay whenever agents communicate between locations

#_of_distractions- sometimes some of the agents located together are distracted by each others' conversation. This slider changes the number of agents that will be distracted in any given round.

time_of_distraction- this slider changes the length of time agents are distracted

Alternative setups:

Currently, it is not possible to run multi-round games with different setups besides the three presets. It is possible to run single-round games with alternative setups.

Node Reposition allows the user to select and move shape nodes. Click on the hollow circle in the very center of a node to grab it, and drag it to a new position. Its associated links will move along with it. (Note: sometimes it is difficult to find the center of the hollow circle. Be patient and eventually the node will be grabbed. Drag the node to the desired location and release it with a mouse click.)

Remove Links allows the user to delete green link lines between nodes. Click on the red circle in the center of a link to select it. Add Environment allows the user to create new box locations. Click where the top left corner of the box should go and drag to the position of the bottom right corner. Nodes count as inside the box (and thus collocated with each other) if the center of the node falls inside.

RESULT REPRESENTATIONS
-------------
Transaction Traces

In the interface, you can view the traces of shape deliveries. Delivery traces are in blue. They show the formation of social exchange networks. The number of shapes delivered is displayed next to the delivery traces. The thicker the traces, the more shapes have been sent between two agents. In this sense, the thicker traces mean stronger relationships.

Data displays

These real-time displays are built into the model.

ROUND DATA The 2x2 matrix below ROUND DATA shows the number of shapes exchanged in the current round between agent types. If there is an in-group bias, which there usually is, the col_col number will be higher than the col_tel, and the tel_tel number will be higher than the tel_col.

RUNNING-AVERAGE To the right of the matrix is a line graph showing the average numbers of shapes exchanged between agent types across multiple rounds. A difference between the red and the green lines means that collocated agents sold unequal amounts of shapes to other collocators than to telecommuters. At the same time, distance between the blue and the pink lines means that telecommuters sold different amounts of shapes to telecommuters than to collocators.

INVENTORY On the bottom left of the screen, the inventory shows how many total parts collocators and telecommuters are able to acquire in the current round as the round is in process. The number of parts acquired is the measure of success in this game (more parts = better). A difference between the red and black lines would mean that one or the other group is performing better in the round.

AVERAGE-INVENTORY This graph shows final inventory average of previous rounds. A difference between the red and black lines would mean that one or the other group is performing better in the game.

Statistical Significance:

Two graphs are built in to do on-the-fly statistical tests on the two main outcome variables of this research when the statistics? switch is turned on. The following statistics are run after every five rounds. 

STATISTICAL TEST OF INGROUP FORMATION performs t-tests to see whether groups are preferentially selling parts to each other.

cc-ct performs a t-test to see whether there is a significant difference between the number of parts collocators sold to other collocators and the number of part they sold to telecommuters.

tc-tt performs a t-test to see whether there is a significant difference between the number of parts collocators sold to other collocators and the number of part they sold to telecommuters.

STATISTICAL SIGNIFICANCE FOR PERFORMANCE DIFFERENCE performs a t-test to see whether there is a significant difference in the performance (inventory levels) of collocators and telecommuters.

Data Turtles

When On-screen-data? is switched to on, then two type of data turtles are presented in the environment. A data turtle is a type of turtle that has no purpose other than to store information about the simulation. The purple turtle in the lower left of the environment is the summary statistics turtle that maintains information about the performance of the simulation over many rounds. The square turtles that appear above the purple turtle each represent the results of a specific round. The number on the square corresponds with the round it is representing. To examine an individual turtle, select the turtle and inspect the turtle.

Logging

Two log files will be created after each run of the model. Both are saved in the same folder as this model. The file named "logfile" captures agent specific data across all rounds. The file called "roundlog" stores round summary data. If there are ten players and ten rounds of a model simulation, one hundred records will appear in "logfile" while ten records will show in "roundlog". There are 15 fields in the "logfile." They are the date and time of the run, current round, whether favoritism on, cost_of_telecommuting, #_of_distraction, time_of_distraction, col or tel (0:col, 1:tel), agent id, number of circle bought, number of square bought, number of X'sbought, number of diamond bought, number of triangle bought, total shapes bought (inventory), and how many did I sell to other agents. There are 12 fields in the "roundlog" file. They are date and time of the run, current round, whether favoritism on, cost_of_telecommuting, #_of_distraction, time_of_distraction, total inventory of collocators, total inventory of telecommuters, number of shapes sold from col to col, number of shapes sold from col to tel, number of shapes sold from tel to tel and number of shapes sold from tel to col.

THINGS TO NOTICE
----------------
Turn on "favoritism" and set cost_of_telecommuting, #_of_distraction and time_of_distraction to zero, see whether in-groups are formed within collocated and remote agents. Then, turn off "favoritism" and set cost_of_telecommuting to one, see whether the pattern of in-group formation changes. In this way, you can isolate the effect of favoritism or cost of telecommuting on the formation of in-group among remote agents. From the model, we find that it is the exclusion from collocated agents rather than communication delay that drives remote agents into in-groups.

Turn on "favoritism" and set cost_of_telecommuting, #_of_distraction and time_of_distraction to zero, see whether collocated or remote agents perform better in terms of inventory. Then, turn off "favoritism" and set cost_of_telecommuting to one, see whether their relative performance changes. This way, you can observe the isolated impact of favoritism or cost_of_telecommuting on performance. To us, the most interesting yet counter-intuitive result from the model is that favoritism has a harmful effect on the performance of collocated agents.

Turn on "favoritism," set cost_of_telecommuting to zero, #_of_distraction to five and time_of_distraction to one, to see whether there is a difference between the performance of collocated and remote agents. In our lab experiment, when the three variables (in-group favoritism, communication delay and distraction) were present, we often see equal performance between collocated and remote players.

THINGS TO TRY
-------------
For those who are interested, you can extract more details of a model run from the on screen data turtles. Later, you can export the log files to a spreadsheet and do various statistical tests.

MODEL CREATORS
-------------
Johnston, E., Ning, N., Bos, N., Olson, J.

For more information on Shapefactory Research, please visit http://www.crew.umich.edu//shapefactory/index.php

FURTHER READING
---------------
Arrow, H., Berdahl, J.L., Bouas, K.S., Craig, K.M., Cummings, A., Lebie, L., McGrath, J.E., O'Connor, K.M., Rhoades, J.A. and Ann Time, S. Technology and groups: An integration. Computer Supported Cooperative Work, 4, 1996, 253-261.2

BeLanger, F. Communication patterns in distributed work groups: A network analysis. IEEE Transactions on Professional Communication, 42, 4 (1999), 261-275.3.

Bos, N., Sadat Shami, N., Olson, J.S., Cheshin, A. and Nan, N. In-group/out-group effects in distributed teams: An experimental simulation. In Proc. CSCW 2004, ACM Press (2004), 429-436.4.

Dobbs, M., and Crano, W. D. Outgroup accountability in the minimal group paradigm: Implications for aversive discrimination and social identity theory. Personality and Social Psychology Bulletin, 27, 3 (2001), 355-364.5.

Epstein, J.M. and Axtell, R. Growing Artificial Societies: Social Science from the Bottom up. Brookings Institution Press, Washington, D. C., 1996.6. Finn, K.E., Sellen, A.J., and Wilbur, S.B. Video-Mediated Communication. Lawrence Erlbaum Associates, Mahway, NJ, 1997.7.

Hinds, P.J. and Bailey, D.E. Out of sight, out of sync: Understanding conflict in distributed teams. Organization Science, 14, 6 (2003), 615-632.8.

Mas-Colell, A, Whinston, M.D. and Green, J.R. Microeconomic Theory, Oxford University Press, Oxford, New York, 2002.9.

NetLogo website. http://ccl.northwestern.edu/netlogo/10. 

Ning, N., Johnston, E., Olson, J., and Bos, N., Beyond Being in the Lab: Using Agent Based Simulations to Isolate Competing Hypotheses. Conference Proceedings CHI 2005.

Rosenthal, E. Social networks and team performance. Team Performance Management, 3, 4 (1997). 288.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
false
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45
Circle -16777216 true false 135 135 30

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230
Circle -16777216 true false 135 135 30

diamond
false
0
Polygon -7566196 true true 150 45 45 121 150 285 256 121
Circle -16777216 true false 135 135 30

line
true
0
Line -7566196 true 150 2 150 302
Circle -65536 true false 148 148 4

line1
true
0
Line -7566196 true 150 2 150 302
Circle -65536 true false 148 148 4
Rectangle -7566196 true true 149 1 151 302

line2
true
0
Line -7566196 true 150 2 150 302
Circle -65536 true false 148 148 4
Rectangle -7566196 true true 149 1 151 302
Rectangle -7566196 true true 149 2 153 302

line3
true
0
Line -7566196 true 150 2 150 302
Circle -65536 true false 148 148 4
Rectangle -7566196 true true 149 1 151 302
Rectangle -7566196 true true 149 2 153 302
Rectangle -7566196 true true 149 1 149 15
Rectangle -7566196 true true 147 2 154 302

line4
true
0
Line -7566196 true 150 2 150 302
Circle -65536 true false 148 148 4
Rectangle -7566196 true true 149 1 151 302
Rectangle -7566196 true true 149 2 153 302
Rectangle -7566196 true true 149 1 149 15
Rectangle -7566196 true true 147 2 154 302
Rectangle -7566196 true true 147 0 156 300

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

triangle
false
0
Polygon -7566196 true true 150 15 15 240 285 240
Circle -16777216 true false 135 135 30

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

x
false
0
Polygon -7566196 true true 90 32 31 91 210 272 271 210
Polygon -7566196 true true 211 33 30 211 91 272 271 91
Circle -16777216 true false 135 135 30

@#$#@#$#@
NetLogo 2.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
