; Stack machine simulator
; Written by Teresa Carrigan
; April 2004

globals [top step numOnStack console]

breeds [ node stack]

to startup
  setup
end

; initialize the stack and global variables
to setup
  ca
  set console "initializing stack"
  set top (- screen-size-y * .4)
  set step 4
  set numOnStack 0
  cct-stack 1 [
    set shape "rectangle"
    setxy -3 top
    set size 5.0
    ]
  cct-stack 1 [
    set shape "rectangle"
    setxy 3 top
    set size 5.0
    ]
  cct-stack 1 [
    set shape "rectangle"
    setxy 0 top
    set size 5.0
    set label "STACK"
    ]
  set top (top + step)
end

; create a new node, place it on the top of the stack
to push
  locals [ name ]
  if not any? stack
     [ setup ]
  set name user-input "What should I push?"
  set console "push " + name
  cct-node 1 [
    set shape "circle"
    set color red
    setxy 0 top
    set size 5.0
    set label name
    ]
  set top (top + step)
  set numOnStack (numOnStack + 1)
end

; remove the node at the top of the stack
to pop
  if not any? stack
     [ setup ]
  ifelse any? node-at 0 (top - step)
    [ask node-at 0 (top - step)
      [ set console ("pop result = " + label )
        die
      ]
    set top (top - step)
    ]
    [
      set console  "Stack is empty - can't pop!"
    ]
  set numOnStack (numOnStack - 1)
end

; Remove the top two items from the stack, add them, and then push their sum onto the stack
to add
  locals [ save ]
  if not any? stack
     [ setup ]
  ifelse any? node-at 0 (top - step - step)
  [
    set console "add"
    ask node-at 0 (top - step)
      [  set save label
         die
      ]
    set top (top - step)
    ask node-at 0 (top - step)
       [  set label ( "(" + label + "+" + save + ")" )
       ]
   ]
   [  set console  "Two items needed to ADD" ]
  set numOnStack (numOnStack - 1)
end

; Remove the top two items from the stack, multiply them, and then push their product onto the stack.
to mul
  locals [ save ]
  if not any? stack
     [ setup ]
  ifelse any? node-at 0 (top - step - step)
  [
    set console "mul"
    ask node-at 0 (top - step)
      [  set save label
         die
      ]
    set top (top - step)
    ask node-at 0 (top - step)
       [  set label ( "(" + label + "*" + save + ")" )
       ]
   ]
   [  set console "Two items needed to MUL" ]
  set numOnStack (numOnStack - 1)
end

; Remove the top two items from the stack, subtract them, and then push their difference onto the stack.
to sub
  locals [ save ]
  if not any? stack
     [ setup ]
  ifelse any? node-at 0 (top - step - step)
  [
    set console "sub"
    ask node-at 0 (top - step)
      [  set save label
         die
      ]
    set top (top - step)
    ask node-at 0 (top - step)
       [  set label ( "(" + label + "-" + save + ")" )
       ]
   ]
   [  set console "Two items needed to SUB" ]
  set numOnStack (numOnStack - 1)
end

; Remove the top two items from the stack, divide them, and then push their quotient onto the stack.
to div
  locals [ save ]
  if not any? stack
     [ setup ]
  ifelse any? node-at 0 (top - step - step)
  [
    set console "div"
    ask node-at 0 (top - step)
      [  set save label
         die
      ]
    set top (top - step)
    ask node-at 0 (top - step)
       [  set label ( "(" + label + "/" + save + ")" )
       ]
   ]
   [  set console "Two items needed to DIV" ]
 set numOnStack (numOnStack - 1)
end

; push a random item onto the stack
to push-random
  locals [ name ]
  set name (random-one-of ["a" "b" "c" "d" "e" "f" "g"])
  cct-node 1 [
    set shape "circle"
    set color red
    setxy 0 top
    set size 5.0
    set label name
    ]
  set top (top + step)
  set numOnStack (numOnStack + 1)
  set console "push " + name
end

; do a random arithmetic operation on the top two nodes of the stack
to oper-random
  locals [oper]
  set oper (random-one-of ["add" "sub" "mul" "div"])
  run oper
end

; Generate a single random line of a stack machine program, and then execute it.
; Uses oper-random and push-random
to demo
   if not any? stack
     [ setup ]
   ifelse (numOnStack > 4)
     [ oper-random ]
     [ ifelse (numOnStack < 1)
       [push-random ]
       [ ifelse (numOnStack > 1) and (random 100 < 50)
         [ oper-random ]
         [ ifelse (numOnStack = 1) and (random 100 < 25)
             [pop]
             [push-random ]
          ]
        ]
      ]
end

; *** NetLogo Model Copyright Notice ***
;
; Copyright 2004 by Teresa W. Carrigan.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Teresa W. Carrigan.
; Contact Teresa W. Carrigan for appropriate licenses for redistribution
; for profit.
;
; To refer to this model in academic publications, please use:
; Carrigan, T. (2004).  Simple Stack Machine model.
; Blackburn College, Carlinville IL.
;
; In other publications, please use:
; Copyright 2004 by Teresa W. Carrigan.  All rights reserved.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
321
10
699
409
11
11
16.0
1
20
1
1
1

CC-WINDOW
9
335
306
402
Command Center

BUTTON
51
68
114
101
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
8
107
71
140
NIL
push
NIL
1
T
OBSERVER
T

BUTTON
8
141
71
174
NIL
pop
NIL
1
T
OBSERVER
T

BUTTON
73
107
136
140
NIL
add
NIL
1
T
OBSERVER
T

BUTTON
138
107
201
140
NIL
mul
NIL
1
T
OBSERVER
T

BUTTON
74
142
137
175
NIL
sub
NIL
1
T
OBSERVER
T

BUTTON
138
142
201
175
NIL
div
NIL
1
T
OBSERVER
T

BUTTON
48
176
177
209
random command
demo
NIL
1
T
OBSERVER
T

BUTTON
114
68
177
101
reset
ca\nclear-output
NIL
1
T
OBSERVER
T

MONITOR
4
214
318
263
NIL
console
0
1

@#$#@#$#@
WHAT IS IT?
-----------
This is a model of a generic zero-address computer, also known as a stack machine.  This model does not show the numeric output of a program, but rather the standard infix expression for that output.  This allows students to test their stack machine programs to see if the programs correctly implement the assigned expression.


HOW IT WORKS
------------
A generic stack machine assembly language has only six operations: Push, Pop, Add, Sub, Mul, and Div.  Push adds an item to the top of the stack, then increments the stack pointer.  The items on the stack do not operate by themselves in this model; everything happens because of button clicks.  This is inherent in any stack model, because only the top of the stack is allowed to be processed.


HOW TO USE IT
-------------
First press the Setup button, which builds the base of the stack.  Now push at least two items onto the stack, and then try pressing one of the operation buttons.  If you have written a Stack Machine program that you wish to test, then for each line of the program simply press the appropriate button.  To see a demonstration of a random program, repeatedly press the "Random command" button.

Setup - Initializes variables and builds the base of the stack.

Push - Adds an item to the top of the stack.  This requires the user to input the item to be added.

Pop - Removes the top item from the stack, storing it somewhere.  In this simulation, it is simply displayed in the console.

Add - Removes the top two items from the stack, adds them, and then pushes their sum onto the stack.

Sub - Removes the top two items from the stack, subtracts them, and then pushes their difference onto the stack.

Mul - Removes the top two items from the stack, multiplies them, and then pushes their product onto the stack.

Div - Removes the top two items from the stack, divides them, and then pushes their quotient onto the stack.

Random command - Generates a single random line of a stack machine program, and then executes it.  This is handy for demonstrating how the stack machine works when you do not have a program written yet.

Reset - Clears the command center window and also the graphics window.  You must press Setup to build the stack again.


THINGS TO NOTICE
----------------
Every time you Push an item, the stack gets one more item on top.  For every time you Pop, Add, Sub, Mul, or Div, the stack loses one item.

What happens when you try to Pop and the stack is empty?  What happens when you have a single item on the stack, and you try to Add?  When you press Sub, how does the stack determine which item is subtracted?


THINGS TO TRY
-------------
Convert an arithmetic expression to Reverse Polish Notation (also known as postfix).  Enter this RPN expression as a series of stack commands.  The final result popped will be the normal infix equivalent of the expression.  For example,

C 4 + 3 A - *

would be entered as Push C, Push 4, Add, Push 3, Push A, Sub, Mul, Pop.  What is popped should be (C + 4) * (3  A).


EXTENDING THE MODEL
-------------------
Add another operator to the stack, such as exponentiation, or sin.

Add a memory bank, where variables may be pushed from, or popped to.

Change the stack machine so that instead of simply changing the label of an item, items always have numeric values and the expressions are evaluated.

Modify push to limit the number of items that can be pushed on the stack (to prevent wrapping).


NETLOGO FEATURES
----------------
Several interesting NetLogo features were used in this model.  The oper-random procedure uses random-one-of to select the name of a procedure to run, and then uses the run command to call the procedure.  This is much cleaner than a series of nested ifelse statements to call the procedure chosen.

Each procedure that removes an item from the stack uses "ifelse any? node-at x y" to check that there is a node (the name of the breed used for items) at position x y.


CREDITS AND REFERENCES
----------------------
This model was written by Teresa W. Carrigan, April 2004.

Permission to use, modify or redistribute this model is hereby granted,
provided that both of the following requirements are followed:
	a) this copyright notice is included.
	b) this model will not be redistributed for profit without permission
	   from Teresa W. Carrigan.
Contact Teresa W. Carrigan for appropriate licenses for redistribution
for profit.

To refer to this model in academic publications, please use:
Carrigan, T. (2004). Simple Stack Machine model.
Blackburn College, Carlinville IL.

In other publications, please use:
Copyright 2004 by Teresa W. Carrigan.  All rights reserved.

For more information on Stack Machines, see one of these textbooks:
[1] Stallings, W. "Computer Organization and Architecture: Designing for Performance", Sixth Edition.  Prentice Hall. pages 371-376.
[2] Tanenbaum, A.  "Structured Computer Organization", Fourth Edition.  Prentice Hall.  pages 338-341.
[3] Null, L. and Lobur, J. "Essentials of Computer Architecture", First Edition.  Jones & Bartlett.  pages 204-207.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

rectangle
false
9
Rectangle -16776961 true false 0 46 299 257

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
