breeds [ reactants products ]
turtles-own [ reacting? velocity ]
patches-own [ patch-temperature ]
globals [ambient-temperature]

;*******************************************************************
;* setup the model
;*******************************************************************
to setup
  ca
  set ambient-temperature starting-temp

  if (setups = "DEFAULT")[set-default]
  if (setups = "EXPLOSION")[set-explosion]
  if (setups = "SPONTANEOUS")[set-spontaneous]
  if (setups = "ENDOTHERMIC")[set-endothermic]
  if (setups = "LIMITED REACTANT")[set-limited-reactant]

  ask patches [
    set patch-temperature ambient-temperature
  ]
  heat-color-patches

  create-custom-reactants type-1-reactants
    [ set shape "oxygen"
      setxy random-float screen-size-x
      random-float screen-size-y
      set reacting? false
      ]
  create-custom-reactants type-2-reactants
    [ set shape "hydrogen"
      setxy random-float screen-size-x
      random-float screen-size-y
      set reacting? false
      ]

  ask turtles [ set heading random 360 ]
end

to set-default

  ;; advanced variables
  set system-insulation 10
  set heat-diffusion 0.9
  set color-change-sensitivity 4

  ;; simple variables
  set reactants-threshold 0.5
  set energy-released 300
  set type-1-reactants 200
  set type-2-reactants 200
  set mouse-temp-effect 500
  set starting-temp 20
  set coloration "RED AND BLUE"

end

to set-endothermic

  ;; set to default
  set-default

  ;; set explosions
  set reactants-threshold 1.0
  set energy-released -700
  set type-1-reactants 200
  set type-2-reactants 200
  set mouse-temp-effect 1000
  set starting-temp 50

end

to set-spontaneous

  ;; set to default
  set-default

  ;; set explosions
  set reactants-threshold 0.2
  set energy-released 500
  set type-1-reactants 200
  set type-2-reactants 200
  set mouse-temp-effect 1000
  set coloration "RAINBOW 1"

end

to set-explosion

  ;; set to default
  set-default

  ;; set explosions
  set reactants-threshold 0.5
  set energy-released 1000
  set type-1-reactants 500
  set type-2-reactants 500
  set mouse-temp-effect 500
  set coloration "RAINBOW 1"

end

to set-limited-reactant

  ;; set to default
  set-default

  set type-1-reactants 500
  set type-2-reactants 50
  set coloration "RAINBOW 1"

end

to go

  ;*******************************************************************
  ;* set velocity of turtle relative to the heat of patch underneath
  ;*******************************************************************
  ask turtles
    [
      set velocity ( patch-temperature / 100 )
      fd velocity
    ]

  ;*******************************************************************
  ;* as patches cool to ambient temperature, raise ambient temperature
  ;*******************************************************************
  ask patches
    [
      if (patch-temperature > ambient-temperature) [
        set patch-temperature patch-temperature - 1
        set ambient-temperature ( ambient-temperature +  ( (system-insulation / 10) * (1 / count patches) ) )
      ]
      if (patch-temperature < ambient-temperature) [
        set patch-temperature patch-temperature + 1
        set ambient-temperature ( ambient-temperature -  ( (system-insulation / 10) * (1 / count patches) ) )
      ]
    ]


  ask reactants [ react-forward ]

  if (mouse-down?) [mouse-change-temperature]

  diffuse patch-temperature heat-diffusion
  heat-color-patches
  do-plot

  ;*******************************************************************
  ;* STOP if zero of either reactants
  ;*******************************************************************
   if (count reactants with [ shape = "hydrogen" ] = 0) [stop]
   if (count reactants with [ shape = "oxygen" ] = 0) [stop]

end

;*******************************************************************
;* procedure allows the mouse to change the temperature of a patch
;*******************************************************************
to mouse-change-temperature
     locals[current-patch-temperature]
     set current-patch-temperature ( patch-temperature-of patch-at mouse-xcor mouse-ycor )
     set patch-temperature-of patch-at mouse-xcor mouse-ycor current-patch-temperature + mouse-temp-effect
     ask patches [ if ( patch-temperature < 0 ) [ set patch-temperature 0 ] ]
end


;*******************************************************************
;* procedure makes reactants reactant with other reactants
;*******************************************************************
to react-forward
  locals [partner
          vert-force-part horz-force-part
          vert-force-my horz-force-my
          vert-force-total horz-force-total
          force-angle
          force-total
          vcm
         ]

  if (any? other-reactants-here)
    [ set reacting? true ;; prevent other molecules from reacting
      set partner (random-one-of other-reactants-here)
      ; show stuff
      if(not (shape = shape-of partner))[
      if not reacting?-of partner
        [ set reacting?-of partner true ;; prevent other reactions with partner

          ; set the vertical and horizontal forces of particles
          set vert-force-my (velocity * sin (heading))
          set horz-force-my (velocity * cos (heading))
          set vert-force-part (velocity-of partner * sin (heading-of partner))
          set horz-force-part (velocity-of partner * cos (heading-of partner))

          set vert-force-total abs ( vert-force-part - vert-force-my )
          set horz-force-total abs ( horz-force-part - horz-force-my )
          set force-total sqrt ((vert-force-total ^ 2 ) + ( horz-force-total ^ 2))

          if(force-total >= reactants-threshold) [
            set breed-of partner products
            set patch-temperature patch-temperature + energy-released
            set heading-of partner random 360
            set color-of partner red
            set shape-of partner "water"
            die
          ]

         ]
        ]
      set reacting? false
   ]
end


;*******************************************************************
;* plots the variables
;*******************************************************************
to do-plot
  set-current-plot "Variables"
  set-current-plot-pen "react1"
  plot count reactants with [ shape = "oxygen" ]
  set-current-plot-pen "react2"
  plot count reactants with [ shape = "hydrogen" ]
  set-current-plot-pen "prod"
  plot count products

  set-current-plot "temperature"
  plot ambient-temperature
end

;*******************************************************************
;* changes the color of the patches relative to the ambient temperature
;* this code was adapted from Art/Diffusion Graphics
;*******************************************************************
to heat-color-patches
  locals [ heat-list min-heat max-heat]
  set heat-list values-from patches [ patch-temperature ]
  set min-heat min heat-list
  set max-heat max heat-list
  if(max-heat - ambient-temperature < (color-change-sensitivity * 100))[set max-heat ambient-temperature + (color-change-sensitivity * 100)]

  if(coloration = "RAINBOW 1")[
      ask patches [
        set pcolor hsb
                   ((1.5 * (patch-temperature - ambient-temperature)) / (max-heat - ambient-temperature + 0.0000000001)) ; hue
                    1.0                                                 ; saturation
                   ((3.0 * (patch-temperature - ambient-temperature)) / (max-heat - ambient-temperature)) ; brightness
        ]
  ]

  if(coloration = "RAINBOW 2")[
      ask patches [
        set pcolor hsb
                   ((1.5 * (patch-temperature)) / (max-heat)) ; hue
                    1.0                                                 ; saturation
                   ((3.0 * (patch-temperature)) / (max-heat)) ; brightness
        ]
  ]

  if(coloration = "RED AND BLUE")[
    ask patches [
      if(patch-temperature >= ambient-temperature)[
      set pcolor scale-color red patch-temperature ambient-temperature max-heat
      ]
      if(patch-temperature < ambient-temperature)[
      set pcolor scale-color violet ( (ambient-temperature - patch-temperature) / ambient-temperature ) 0 (2 / color-change-sensitivity)
      ]
    ]
  ]
end

@#$#@#$#@
GRAPHICS-WINDOW
8
10
459
356
24
17
9.0
1
10
1
1
1

CC-WINDOW
191
390
441
524
Command Center

BUTTON
689
165
752
198
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
690
203
753
236
NIL
go
T
1
T
OBSERVER
T

SLIDER
448
489
604
522
starting-temp
starting-temp
1
500
20
1
1
NIL

SLIDER
450
389
606
422
reactants-threshold
reactants-threshold
0.1
5
0.5
0.1
1
NIL

SLIDER
610
390
777
423
type-1-reactants
type-1-reactants
0
500
500
10
1
NIL

SLIDER
450
425
606
458
energy-released
energy-released
-1000
1000
800
10
1
NIL

SLIDER
12
389
181
422
heat-diffusion
heat-diffusion
0
1
0.9
0.1
1
NIL

SLIDER
12
425
180
458
system-insulation
system-insulation
0
10
10
1
1
NIL

SLIDER
608
489
782
522
mouse-temp-effect
mouse-temp-effect
-1000
1000
500
100
1
NIL

TEXTBOX
478
469
595
487
SYSTEM VARIABLES

TEXTBOX
548
369
684
387
REACTION VARIABLES

TEXTBOX
637
469
727
487
USER VARIABLES

PLOT
472
10
751
160
Variables
time
number (temp)
0.0
100.0
0.0
500.0
true
true
PENS
"react1" 1.0 0 -11352576 true
"react2" 1.0 0 -65536 true
"prod" 1.0 0 -16776961 true

SLIDER
12
462
180
495
color-change-sensitivity
color-change-sensitivity
1
10
4
1
1
NIL

SLIDER
610
425
777
458
type-2-reactants
type-2-reactants
0
500
500
1
1
NIL

TEXTBOX
35
368
181
386
Advanced Variables

PLOT
471
164
684
314
temperature
NIL
NIL
0.0
100.0
0.0
200.0
false
true
PENS
"default" 1.0 0 -65536 false

CHOICE
470
318
622
363
setups
setups
"DEFAULT" "USER" "EXPLOSION" "SPONTANEOUS" "ENDOTHERMIC" "LIMITED REACTANT"
2

MONITOR
689
264
751
313
temp
ambient-temperature
3
1

CHOICE
627
318
751
363
coloration
coloration
"RED AND BLUE" "RAINBOW 1" "RAINBOW 2"
1

@#$#@#$#@
WHAT IS IT?
------------------------------------------------------------------
RELATIONSHIP BETWEEN TEMPERATURE AND KINETIC ENERGY OF MOLECULES

This model demonstrates the connection between chemical reactions and the free energy or heat of the system.  Chemical reactions often need an initial input of energy to start the reaction.  While the reactants of a reaction may be plentiful and often collide, they may not collide with enough force to trigger the reaction.   The Kinetic Energy (or speed) of molecules and therefore the force of a collision can be increased by increasing the temperature of the surroundings (ambient temperature).

This model demonstrates this concept.  The molecules move at a speed proportional to their surroundings and collisions are calculated based on molecular speed and the angles of their trajectory (i.e. to particles colliding head on do so with more fore than particles that scrape by each other).


EXOTHERMIC AND ENDOTHERMIC REACTIONS

When reactants come together with enough Kinetic Energy they will react to form product(s).  In most reactions, heat is released into the system (Exothermic) or absorbed from the system (Endothermic) to create the product.  This in turn will gradually raise or lower the ambient temperature of the system.

This model reflects this and the system will heat or cool depending on whether the reaction is Exothermic or Endothermic.   The model does not immediately heat the entire system the temperature change is first local and then dissipates into the surroundings gradually heating the system.  As a result, there will exist local hot and cold pockets in our model.


MORE ABOUT HEAT IN A SYSTEM

Heat tends to escape from a system.  A perfectly insulated system will keep all of the energy released within the system.  Conversely, a perfectly non-insulated system will lose all of the energy released within the system.



HOW IT WORKS
------------------------------------------------------------------

Two reactants will move within the system and react according to user's set parameters and mouse actions.  The program will plot the number of reactants, products and the heat of the system.


HOW TO USE IT
------------------------------------------------------------------

BASIC USE
-------------

setups: Preprogrammed setups for the models.  See "THINGS TO TRY" section for description.  If user deviates from preprogrammed setups, (s)he must select "USER" in this menu.

setup: Sets up the model based on the user defined variables or selected "setup".

go: Starts and Stops the model.

mouse:  The user can heat or cool localized temperature by clicking down on an area with the mouse.  Holding the mouse down will continue to affect the system as will a mouse drag.  This function is used in combination with the mouse-temp-effect variable.

mouse-temp-effect (slider): Sets the amount of heating or cooling the mouse can do on the system.

MORE ADVANCED USE
-----------------

ambient-temperature (slider): Sets the ambient temperature of the system.  It will increase on its own in a system reacting exothermically.

reactant-number (2 sliders): Sets the initial number of reactants in the system.

reactant-threshold (slider): Sets the force (energy) threshold of colliding reactants  needed to react and become a product.

energy-released:  Sets the amount of heat absorbed or released into the system.  Setting this greater than zero creates Exothermic Reactions.  Setting this less than zero creates Endothermic Reactions.

EVEN MORE ADVANCED USE
----------------------

system-insulation: Sets the insulation of a system.  The maximum value of 10 will keep all of the heat release by reactions or mouse clicks in the system.  A model set to 0 will never heat or cool.

heat-diffusion (slider):  Sets the rate of diffusion of heat into the system.

color-sensitivity (slider): Sets the heat coloring function's sensitivity to the average temperature.  A high value will make the coloring less sensitive relative to the ambient temperature.

coloration: "RED AND BLUE" is the best selection for noticing individual reactions-- especially if you plan to display endothermic reactions, which are nearly unnoticeable in the "RAINBOW 1" and "RAINBOW 2" algorithms.  However, the "RAINBOW" coloration schemes could be more interesting as models that show heat dispersion.  "RAINBOW 1" AND "RAINBOW 2" are nearly identical, however the background "RAINBOW 2" is not normalized to the ambient temperature and therefore changes color as the ambient temperature increases.


THINGS TO NOTICE
------------------------------------------------------------------
Molecules move noticeably faster in hotter patches and will slow again in cooler patches.

Heat patches with the mouse and watch as the molecules seem to flee this area.  They are really just speeding up.

Calculations of the force of impact between two molecules are based on the speed and angle of the molecules.

The molecules are modeled after the chemical reaction of hydrogen and oxygen to form water.  However, the molecules are not named for this reaction because the chemical reaction shown is not balanced and can be endothermic.

Cooling is less noticeable: we can only make an area so cold (0).  So localized effects, which are dissipated, are not as visual as heating which can create temperature differences several hundred degrees hotter than the average temperature.

The program will color the local environment (roygbiv scale: red = hot; blue = cold) of the system relative to the average temperature for the "RAINBOW" coloration models.  This means that a red-hot patch in a system with an average temperature of 5 will be less than a cool-green/blue patch in a system with an average temperature of 100.  The relative nature of the color scale can be altered by the color-sensitivity slider.


THINGS TO TRY
------------------------------------------------------------------

setups
------------

USER: User defined setup.  Use this choice to alter predefined setups.

DEFAULT: This model is a good place to start when experimenting with "USER" setup and mouse temperature effects.  Its variables are set to benign levels that will do little without user input.

EXPLOSION: The model is  an explosion waiting to happen.  Use your mouse to set it off.  Observe the reaction and especially the plots-- this setup creates a great curve.

SPONTANEOUS: The model is similar to the defaul model, but the reaction-threshold is set lower and therefore we will see more reactions.

ENDOTHERMIC: Perhaps the most interesting model.  Set to model an endothermic reaction, the reaction needs a catalyst or booster (in the form of mouse heating) to start the reaction.  It makes for an interesting combination of hot and cold effects.  NOTE: this model should be used with "RED AND BLUE" for best results.

LIMITED REACTANT: Model shows the importance of both reactants in the reaction.  Good younger students.


EXTENDING THE MODEL
------------------------------------------------------------------
The colorations scheme could be added to and improved.
Add to the setups.
Increase the radius of the mouse effect.
Create reactions with >2 reactants and/or more than 1 product.


RELATED MODELS
--------------
Other Chemistry sample models especially those in the Chemical Reactions folder.


CREDITS AND REFERENCES
----------------------
Thanks to Mitchell Hull for his creation of this model

@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

arrowoutline
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150
Line -1 false 150 0 -1 150
Line -1 false 104 151 -2 151
Line -1 false 195 151 299 151
Line -1 false 151 1 300 150
Line -1 false 104 151 106 292
Line -1 false 106 292 195 291
Line -1 false 195 291 195 152
Line -16777216 false 150 2 2 149
Line -16777216 false 2 149 106 149
Line -16777216 false 106 149 108 288
Line -16777216 false 108 288 193 288
Line -16777216 false 193 288 192 148
Line -16777216 false 192 147 294 147
Line -16777216 false 294 147 150 2

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

hydrogen
true
7
Rectangle -1 true false 90 150 90 172
Circle -1 true false 29 89 121
Circle -1 true false 150 90 120
Rectangle -1 true false 122 135 172 162

oxygen
true
0
Circle -65536 true false 2 77 147
Circle -65536 true false 151 76 148
Rectangle -65536 true false 129 134 172 166

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

water
true
0
Circle -65536 true false 74 74 152
Circle -1 true false 30 44 90
Circle -1 true false 180 44 90

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
