breeds [wolves moose]
wolves-own [packmates nearest-wolf]
globals [surrounded steps patches-colored]

to setup
  ca
  set patches-colored false
  set-default-shape wolves "wolf-face"
  set-default-shape moose "moose-face"
  set surrounded 0
  set steps 0
  create-custom-wolves population
    [ set color yellow - 2 + random 7  ;; random shades look nice
      setxy (random-float screen-size-x)
            (random-float screen-size-y)
      rt random-float 360 ]
  ask wolves [set packmates wolves with [self != myself]]
  create-custom-moose 1
    [ set color blue
      setxy (random-float screen-size-x)
            (random-float screen-size-y)
      rt random-float 360 ]
  set-plot-pen-interval 1
  auto-plot-on
end

to color-patches ;; patch procedure
ifelse patches-colored
  [ask patches [set pcolor 0]]
  [ask patches [ifelse (pxcor + pycor) mod 2 = 1
     [set pcolor 0]
     [set pcolor 4]]]
  set patches-colored not patches-colored
end

to go
  ask moose [ move-moose ]
  plot surrounded
  if surrounded = guards-needed [stop]
  ask wolves [ move-wolf ]
  set steps steps + 1
  wait delay / 20.
end

to move-moose  ;; turtle procedure
  set surrounded count neighbors  with [count wolves-here > 0 ]
  if surrounded >= guards-needed [stop]
  turn-towards towards min-one-of wolves [distance myself] + 180
  fd random-float moose-speed
end

to move-wolf
locals [goal-vector] ;; This is a vector of two elements: <length, bearing>
  set nearest-wolf min-one-of packmates [distance myself]
  set goal-vector  vector-add
    list distance one-of moose towards one-of moose
    list (repulsion * distance nearest-wolf) (180 + towards nearest-wolf)
  turn-towards last goal-vector + random wolf-noise
  fd random-float wolf-speed
end

;;; HELPER PROCEDURES

to-report vector-add [v1 v2] ;; a vector is a list of two elements: length and angle
  locals [x-sum y-sum]
  set x-sum ((first v1) * sin (last v1)) + ((first v2) * sin (last v2))
  set y-sum ((first v1) * cos (last v1)) + ((first v2) * cos (last v2))
  report list (sqrt ((x-sum * x-sum) + (y-sum * y-sum))) atan x-sum y-sum
end

to turn-towards [new-heading ]  ;; turtle procedure
  rt (subtract-headings new-heading heading)
end

to turn-away [new-heading ]  ;; turtle procedure
  rt (subtract-headings heading new-heading)
end


; *** NetLogo Model Copyright Notice ***
;
; Portions of this model were borrowed from the "flocking" model copyright
; 1998 by Uri Wilensky. All Rights Reserved. In its present form the model
; is Copyright 2004 by H. Van Dyke Parunak. All Rights Reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky and Van Parunak.
; Contact the authors for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; H.V.D. Parunak.  Wolf-Moose Capture Model.
; http://www.erim.org/~vparunak/models/WolfMoose.nlogo . Altarum Institute,
; Ann Arbor, MI, 2003.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
335
11
975
672
10
10
30.0
1
10
1
1
1

CC-WINDOW
6
464
250
546
Command Center

BUTTON
6
12
62
45
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
71
13
126
46
Run
go
T
1
T
OBSERVER
T

SLIDER
9
88
186
121
population
population
2
20
10
1
1
NIL

SLIDER
6
218
233
251
repulsion
repulsion
0.0
2.0
0.7
0.1
1
NIL

SLIDER
9
135
232
168
wolf-speed
wolf-speed
0
10
3.5
0.5
1
patches

SLIDER
9
169
232
202
moose-speed
moose-speed
0
10
2.5
0.5
1
patches

BUTTON
152
14
207
47
Step
go
NIL
1
T
OBSERVER
T

SLIDER
6
256
178
289
wolf-noise
wolf-noise
0
180
10
2
1
NIL

MONITOR
254
133
311
182
NIL
steps
0
1

SLIDER
199
87
330
120
guards-needed
guards-needed
1
8
6
1
1
NIL

PLOT
23
300
311
450
guards
steps
occupied
0.0
100.0
0.0
4.0
true
false
PENS
"nbhd-size" 1.0 0 -65536 false

MONITOR
251
194
329
243
NIL
surrounded
0
1

BUTTON
246
14
326
47
NIL
color-patches
NIL
1
T
OBSERVER
T

SLIDER
9
53
185
86
delay
delay
0
10
2
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
Wolves are able to catch moose, deer, and other animals much larger and more powerful than an individual wolf. They overcome their prey by surrounding them. The wolves in front of the prey keep its attention by menacing it, enabling one of the wolves behind the prey to jump on its back and bring it down.

How do the wolves coordinate their actions so as to surround the prey? This problem was introduced to the Distributed AI research community in the late 1980's by Miro Benda, and stimulated a series of studies that assumed wolves could communicate with one another and exchange beliefs and intentions. These unrealistic assumptions were shown to be unnecessary by Korf in 1991 [1] (summarized in [3]). He showed that if the prey always moves away from the nearest wolf, and if each wolf moves toward the prey and away from the wolf nearest to it, the wolves will always catch the prey on a hexagonal lattice, as long as the prey moves slower than the wolves do.

This model recreates Korf's solution. Unfortunately, NetLogo does not give an easy way to work with hexagonal places, but the basic dynamics still work in spite of the anisotropies of the square place lattice. The prey is considered surrounded when a specified number (guards-needed) of the cells in its 8-neighborhood are occupied by wolves.

HOW TO USE IT
-------------

Buttons:
setup.--Creates wolves and the moose.
Run.--Runs the model in a continuous loop.
Step.--Runs one step of the model.
color-patches.--Toggles coloring of the individual patches so that you can see which patches are occupied by wolves and the moose.

Variables:
delay.--Slows down the simulation so that you can observe the movement of the wolves and moose more clearly.
population.--The number of wolves.
guards-needed.--The number of cells in the moose's 8-neighborhood that the wolves must occupy to capture the moose. When this condition is satisfied, the model stops execution.
wolf-speed.--The maximum speed, in places per time step, of a wolf. The actual distance the wolf moves is selected randomly with this as the upper limit. Thus a wolf's average speed will be half of this value.
moose-speed.--The maximum speed, in places per time step, of the moose. The actual distance the moose  moves is selected randomly with this as the upper limit. Thus the moose's average speed will be half of this value.
repulsion.--How much a wolf is repelled by its nearest neighboring wolf, compared with its attraction to the moose.
wolf-noise.--The upper limit of a random angle to add to each wolf's final direction when it moves, useful to keep the wolves from aligning themselves.

Reporters:
steps.--The current number of steps through which the model has run.
surrounded.--The current number of places in the moose's 8-neighborhood that are occupied by wolves. When surrounded = guards-needed, the model will stop.
guards.--A plot of surrounded over time.

First, determine the wolf population and the number of guards needed to capture the moose. (It helps if the wolf population is at least as large as the required number of guards.) Press setup.

Set the relative speed of wolves and moose, and press Run or Step to watch them move.

When the wolves manage to surround the moose, toggle patch coloring on with the "color-patches" button to verify that they have indeed occupied the required number of neighboring patches.

THINGS TO NOTICE
----------------
If the moose is faster than the wolves, it tends to stay on the edge of the wolf population and can evade it.

If the moose is slower than the wolves, they quickly surround it. The model then goes through three phases:
* The Chase, in which the wolves are getting themselves distributed around the moose;
* Herding, in which the mass of wolves with the moose at their center jostles around until the wolves manage to occupy the required number of places;
* The Kill, when surrounded = guards-needed.

THINGS TO TRY
-------------
Experiment with different settings of the parameters, and try to correlate them with  the number of steps it takes the wolves to surround the moose.

Pay particular attention to the effect of the wolf-noise parameter. What is it about the basic rules that makes a little noise helpful? What happens if this parameter is too high? What is the "right amount" of wolf noise?

Similarly, what is the "right amount" of repulsion between wolves? Can the wolves dislike one another so much that they would rather disperse than catch dinner?

EXTENDING THE MODEL
-------------------
Add the ability to have more than one moose. How do the wolves' decision rules have to change to enable the pack to harvest the most moose?

This simple model does not observe the physical constraint that two things cannot occupy the same space at the same time. In particular, wolves and the moose can pass through each other. Add code to prevent a wolf or moose from moving to or through a place if it is already occupied. How does this exclusion principle affect the behavior of the model? Does the wolves' ability to block the moose's movements shorten the herding time? Or does the restriction on the wolves' ability to move around extend it?

Adjust the wolf-noise and repulsion parameters dynamically as the system runs. For example, when wolves are far from the moose, these parameters might be lower to enable the wolves to get near the moose as quickly as possible. As they draw nearer to the moose, it becomes more important for them to spread out in order to surround the moose. Compare the behavior of the adaptive model with the model with static parameters.

NETLOGO FEATURES
----------------
This model uses breeds to distinguish the wolves and the moose.

I had to implement my own vector addition function to combine wolf-wolf repulsion with
wolf-moose attraction. Did I miss a NetLogo primitive here? Is this a candidate for a new primitive?

CREDITS AND REFERENCES
----------------------
This model is based on the Flocking model in the NetLogo distribution. The algorithm is that proposed in ref. [1]. Ref [2] offers discussion on the limitations of this model.

[1]	R. E. Korf. A Simple Solution to Pursuit Games. In Proceedings of Eleventh International Workshop on Distributed Artificial Intelligence, pages 183-194, 1992.

[2]	M. Manela and J. A. Campbell. Designing Good Pursuit Problems as Testbeds for Distributed AI:  A Novel Application of Genetic Algorithms. In Proceedings of 5th European Workshop on Modeling Autonomous Agents in a Multi-Agent World, MAAMAW 93, pages 231-252, Springer, 1995.

[3]	H. V. D. Parunak. Go to the Ant: Engineering Principles from Natural Agent Systems. Annals of Operations Research, 75:69-101, 1997. Available at http://www.erim.org/~vparunak/gotoant.pdf.

To refer to this model in academic publications, please use:
H.V.D. Parunak.  Wolf-Moose Capture Model. http://www.erim.org/~vparunak/models/WolfMoose.nlogo . Altarum Institute, Ann Arbor, MI, 2003.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

moose-face
false
0
Circle -7566196 true true 101 110 95
Circle -7566196 true true 111 170 77
Polygon -7566196 true true 135 243 140 267 144 253 150 272 156 250 158 258 161 241
Circle -16777216 true false 127 222 9
Circle -16777216 true false 157 222 8
Circle -1 true false 118 143 16
Circle -1 true false 159 143 16
Polygon -7566196 true true 106 135 88 135 71 111 79 95 86 110 111 121
Polygon -7566196 true true 205 134 190 135 185 122 209 115 212 99 218 118
Polygon -7566196 true true 118 118 95 98 95 98 69 84 23 76 8 35 27 19 27 40 38 47 48 16 55 23 55 23 58 41 71 35 75 15 90 19 86 38 100 49 111 76 117 99
Polygon -7566196 true true 167 112 190 96 221 84 263 74 276 30 258 13 258 35 244 38 240 11 230 11 226 35 212 39 200 15 192 18 195 43 169 64 165 92

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-face
false
0
Polygon -7566196 true true 93 80 115 44 130 77 167 77 185 45 203 78 222 105 205 110 209 131 178 148 164 201 135 201 121 146 91 131 95 109 79 104
Circle -16777216 true false 124 97 11
Circle -7566196 true true 161 101 3
Circle -7566196 true true 165 100 3
Circle -16777216 true false 162 97 10

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
set population 200
setup
repeat 200 [ go ]
@#$#@#$#@
@#$#@#$#@
