globals [ attendre        ;; pour que l'appli ne plante pas si l'on reinit sans couper la simulation
          cx              ;; coordonne en x du centre
          cy              ;; coordonne en y du centre
          stable?          ;; on test la stabilit du cercle
          stabilite        ;; cycle de stabilit du cercle
          seuil1      ;;de changement de couleur
          seuil2
        ]

turtles-own[
            ancien-x
            ancien-y
            ]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; phase d'initialisation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to startup
  set rayon 30
  set agents 20
  set distance-perception 12
  set force-repulsion 3
  set seuil1 0.0008
  set seuil2 0.00007
  set stable? false
  set stabilite 0

end

to initialiser

  cp ct
  ca
  set  attendre true
  set cx 0
  set cy 0
  set stable? false
  set stabilite 0
  set-default-shape turtles "circle"
  set seuil1 0.0005
  set seuil2 0.00007

  ;;init-fond
  init-agents
   set  attendre false
end


to init-fond
  set (pcolor-of patch cx cy) white
end


to init-agents
    crt agents
    cct 1 [
    set color white
    set size 1.3
    ]
    wait 0.2

  ask turtles with [color != white ] [
    set size  2
    set color red
    setxy random-float screen-size-x
          random-float screen-size-y

  ]
  wait 0.5
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;phase d'execution
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to lancer
  if mouse-down? [changer-centre]
  ifelse( is-boolean? attendre )               ;; on evite de faire planter l'applet si l'on reinitialise sans couper le moteur
    [if (not attendre)[
      set stable? true
      orienter-agents-cercle              ;;on va attirer les agents vers le bord du cercle
      avancer-vers-cercle
      fuir-autres-agents                  ;; les agents ont peur de leurs voisins et s'en loigne s'il sont trop prs
      tester-stabilite
      ifelse (stabilite < 20)[ask turtles with [color != white ][set color red ]]
      [ifelse (stabilite < 90)[ask turtles with [color != white ][set color yellow - 1  ]]
      [ask turtles with [color != white ][set color green ]]
      ]

      ]
    ]
    [initialiser]
end


to orienter-agents-cercle         ;; on oriente les agents de telle sorte qu'ils regardent le centre du cercle
  ask turtles with [color != white ][
    set ancien-x xcor
    set ancien-y ycor
    set heading towardsxy cx cy
  ]
end


to avancer-vers-cercle
ask turtles with [color != white ][
   fd (( (distancexy cx cy ) - rayon) / 2  );; si les agents sont trop loin, on les rapproche du centre, sinon, on les loigne
   ]
end



to fuir-autres-agents
locals [individu liste liste-indiv]
   ask turtles with [color != white ][
   set liste-indiv turtles with [color != white ] in-radius distance-perception    ;;on genere la liste des individus dans un perimetre autour d'un agent
   set individu  who
   if (count liste-indiv > 1)
    [
    set liste values-from liste-indiv [who]        ;; dans le perimetre de repulsion, on retire l'agent central de la liste
    set liste filter [? != who] liste              ;; en effet, in-radius inclus l'agent demandeur de l'information
    without-interruption[
    foreach liste [
         ask turtle ? [eloigner(individu)]          ;; on fait s'eloigner les individus du perimetre de l'individu central
      ]
    ]
    ]

    ]
  end


to eloigner[agent]
    locals [offset dist]
 set heading towards turtle agent                        ;; l'agent qui va fuir fait face  son concurrent
  set dist distance turtle agent
     ifelse (dist = 0)
       [set offset 0.1]
       [set offset 0]
 fd ( - (force-repulsion ) / (dist + offset) )   ;; ...et recule d'autant que la force de rpulsion est leve

 end

to tester-stabilite
locals [seuil]

  ifelse (stabilite < 50)[set seuil seuil1][set seuil seuil2]

  set stable? true

  ask turtles with [color != white ][

    if ( (  ( abs(ancien-x - xcor) / (2 * screen-edge-x) ) > seuil) or (  ( abs(ancien-y - ycor) / (2 * screen-edge-y) ) > seuil))
    [set stable? false]
  ]
  if is-boolean? stable?[
  ifelse stable?[set stabilite (stabilite + 1)][set stabilite (stabilite / 1.75)]
 ]


end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; phase de changement de paramtres (temps rel)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to changer-centre                                          ;; on change la position du centre du cercle
if mouse-down? [
    ;;ask patches [ set pcolor black ]
    set cx (round mouse-xcor)
    set cy (round mouse-ycor)
    ask turtles with [color = white ] [setxy cx cy]

    ;;set (pcolor-of patch-at cx cy) white
  ]
end

to ajouter                                                ;;on ajout un agent, plac alatoirement
ask one-of turtles [
  hatch 1 [
            set color red
            setxy random-float screen-size-x
                  random-float screen-size-y]

]
end

to retirer                                              ;; on retire un agent de la liste
if (count turtles > 4)[
  ask one-of turtles with [color != white ] [
    die

  ]
]
end

@#$#@#$#@
GRAPHICS-WINDOW
319
10
771
483
55
55
3.982
1
0
1
1
1
0
1
1
1

CC-WINDOW
5
497
780
592
Command Center
0

TEXTBOX
18
238
314
477
Mode d'emploi :\n1. Cliquez sur action/pause.\nVous pouvez changer interactivement le nombre d'agents en cliquant sur ajouter ou retirer.\n2. Pour interrompre/reprendre la simulation, cliquez sur action/pause.\n3. Pour effectuer une nouvelle simulation, choisissez un nombre d'agents avec le curseur agents, puis cliquez sur validation.\n4. A tout moment, vous pouvez jouer sur le rayon du cercle avec le curseur rayon.\n5. Vous pouvez jouer sur les interactions entre agents avec les curseurs distance-perception et force-repulsion.

BUTTON
164
94
312
127
validation
initialiser
NIL
1
T
OBSERVER
T
NIL

BUTTON
4
13
100
83
action/pause
lancer
T
1
T
OBSERVER
T
NIL

SLIDER
4
163
161
196
rayon
rayon
5
50
30
1
1
NIL

BUTTON
106
13
202
46
ajouter agent
ajouter
NIL
1
T
OBSERVER
T
NIL

BUTTON
106
50
202
83
retirer agent
retirer
NIL
1
T
OBSERVER
T
NIL

SLIDER
165
137
312
170
distance-perception
distance-perception
1
40
12
1
1
NIL

SLIDER
165
174
312
207
force-repulsion
force-repulsion
1
30
3
1
1
NIL

MONITOR
207
22
312
71
nombre d'agents
max list 0 ((count turtles) - 1)
1
1

SLIDER
3
94
161
127
agents
agents
3
100
20
1
1
NIL

TEXTBOX
325
14
767
34
        Formation du cercle

@#$#@#$#@
DE QUOI S'AGIT IL ?
-------------------

Cette applet montre la capacit qu'ont des agents, placs alatoirement,  s'organiser seuls pour former un cercle, en se rpartissant rgulirement autour de celui-ci.
Les agents, rouges  l'origine, deviendront verts quand ils seront dans une situation relativement stable.


QUEL EST LE PRINCIPE ?
----------------------

Les agents (points rouges  l'cran) n'ont que deux comportements trs simples :
	* S'loigner ou se rapprocher d'un point de rfrence (le centre du cercle, le point blanc  l'cran) pour s'en maintenir  une certaine distance.
	* Fuir ses voisins s'ils sont trop prs.

Avec la premire action, les agents vont se placer  une certaine distance du point de rfrence, sans se proccuper de la position des autres agents. Du coup, tous les agents peuvent se retrouver au mme endroit!

Avec la seconde action, les agents vont se repousser les uns des autres s'ils sont trop proches. La rpulsion sera alors d'autant plus forte que les agents sont prs. les agents vont donc chercher  maintenir une distance minimale avec leurs voisins.

La combinaison des deux actions va alors entrainer la formation du cercle.
Les agents vont commencer  se rapprocher du bord du cercle, et du coup, de leurs voisins. Ils vont vouloir s'en loigner, ce qui va les repousser un peu plus loin que l'endroit qu'ils visaient.
De proche en proche, les agents vont d'abord se rapprocher du bord du cercle, puis s'loigner de leurs voisins, et ainsi de suite. Les agents vont ainsi avoir tendance  "glisser" le long du cercle pour pouvoir se placer  un endroit plus favorable:
  *o ils ne sont plus gns par leurs voisins,
  *ou alors, o ils seront "coincs" entre deux voisins et ne pourront plus bouger.

Au final, les agents formeront un cercle, en ayant un minimum d'espace entre eux.


COMMENT AGIR SUR LE PROGRAMME ?
-------------------------------

Avant toute chose, il faut initialiser le programme avec un certain nombre d'agent.
Le curseur  droite du bouton "Initialiser" permet de dfinir le nombre d'agent souhait au dpart.

Des agents (en rouge) vont apparatre  l'cran. Un point blanc au centre indique la position dsir du centre du cercle.
Ce point peut tre dplac en cliquant sur la surface de la simulation (zone noire).

La simulation peut commencer, il suffit pour cel de cliquer sur le bouton "lancer".
Le programme va alors tourner sans fin jusqu' ce que l'on appuye  nouveau sur le bouton "lancer"; ou alors en appuyant sur initialiser, qui provoquera la rinitialisation de la position des agents, leur nombre, et la position du centre.

Il est aussi possible d'agir pendant la simulation sur diffrents aspects:
	* On peut changer le centre du cercle  tout instant en cliquant dans la zone noire,
	* On peut modifier la distance souhaite entre ce centre et les agents, en dplaant le curseur "rayon",
	* On peut choisir la distance pour laquelle les agents s'estiment trop prs de leurs voisins, et vont donc chercher  s'en loigner (curseur "distance-perception"),
	* On peut dterminer avec quelle force les agents vont fuir leurs voisins. c'est  dire,  quelle vitesse ils vont partir dans la direction oppose (curseur "force-repulsion"),
	* Enfin, On peut ajouter ou supprimer des agents  tout moment, en cliquant sur les deux boutons baptiss "ajouter agent" et "retirer agent".


POINTS A NOTER
--------------

Tout d'abord, le monde des agents est sphrique. Comme sur notre plante, si l'on avance toujours tout droit, on revient sur nos pas. Pour les agents, il en va de mme, s'il sortent par un bord de l'cran, ils rapparaitront de l'autre cot. En plaant le centre du cercle suffisamment au bord de la zone de simulation, le cercle form commencera sur un bord, et finira sur l'autre.

Les agents peuvent aussi s'enfuir trs vite! En effet, un agent en fuite peut se retrouver subitement trs proche d'un autre agent en fuite. De ce fait, ils seront tellement proche que la force de rpulsion (qui, rappelons-le, et d'autant plus leve que les agents sont proches) va tre trs importante! On pourra alors voir des agents qui sembleront "sauter" loin d'un autre.


QUELQUES TRUCS A ESSAYER
------------------------

Que se passe t'il quand on met de nombreux agents sur un cercle trs petit?
Que se passe t'il quand on met une distance de rpulsion leve avec une force de rpulsion leve?
Comment jouer sur les curseur de distance de rpulsion et de force pour stabiliser le cercle sans que des agents sautent partout?
De mme, comment obtenir un cercle rgulier selon le rayon de cercle choisit?

Il est possible de changer plusieurs paramtres  la fois. Toutefois, afin d'observer ce que l'on souhaite, il est souvent plus sage de regarder l'influence des paramtres un par un.

Vous pouvez commencer par choisir un rayon, pour lequel vous allez menez des expriences. Ensuite, vous pourrez essayer d'abord l'influence de la distance de rpulsion, puis de la force, et enfin du nombre d'agent. Enfin, quand vous commencerez  deviner les comportements, essayez de prdire les paramtres qui donneront de bons rsultats avec un nouveau nombre d'agent.


CREDITS ET REFERENCES
---------------------

Cette applet a t dvelopp avec NetLogo par Aurlien SAINT-DIZIER pour le site www.interstices.info
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7500403 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7500403 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7500403 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7500403 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7500403 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7500403 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7500403 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7500403 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7500403 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -1184463 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7500403 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7500403 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7500403 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7500403 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7500403 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6459832 true false 150 32 157 162
Polygon -13345367 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7500403 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7500403 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7500403 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

circle
false
0
Circle -7500403 true true 35 35 230

person
false
0
Circle -7500403 true true 155 20 63
Rectangle -7500403 true true 158 79 217 164
Polygon -7500403 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7500403 true true 216 83 267 123 248 143 215 107
Polygon -7500403 true true 167 163 145 234 183 234 183 163
Polygon -7500403 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7500403 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7500403 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7500403 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8630108 true false 195 75 195 120 240 120 240 75
Polygon -8630108 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7500403 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8630108 true false 90 210 105 225 120 210
Polygon -8630108 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7500403 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8630108 true false 210 210 195 225 180 210
Polygon -8630108 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7500403 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7500403 true true 15 105 105 165
Rectangle -7500403 true true 45 90 105 105
Polygon -7500403 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7500403 true true 105 120 263 195
Rectangle -7500403 true true 108 195 259 201
Rectangle -7500403 true true 114 201 252 210
Rectangle -7500403 true true 120 210 243 214
Rectangle -7500403 true true 115 114 255 120
Rectangle -7500403 true true 128 108 248 114
Rectangle -7500403 true true 150 105 225 108
Rectangle -7500403 true true 132 214 155 270
Rectangle -7500403 true true 110 260 132 270
Rectangle -7500403 true true 210 214 232 270
Rectangle -7500403 true true 189 260 210 270
Line -7500403 true 263 127 281 155
Line -7500403 true 281 155 281 192

wolf-left
false
3
Polygon -6459832 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6459832 true true 87 80 79 55 76 79
Polygon -6459832 true true 81 75 70 58 73 82
Polygon -6459832 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6459832 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6459832 true true 116 140 114 189 105 137
Rectangle -6459832 true true 109 150 114 192
Rectangle -6459832 true true 111 143 116 191
Polygon -6459832 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6459832 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6459832 true true 214 134 203 168 192 148
Polygon -6459832 true true 204 151 203 176 193 148
Polygon -6459832 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6459832 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6459832 true true 201 99 214 69 215 99
Polygon -6459832 true true 207 98 223 71 220 101
Polygon -6459832 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6459832 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6459832 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6459832 true true 48 143 58 141
Polygon -6459832 true true 46 136 68 137
Polygon -6459832 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6459832 true true 38 138 66 149
Polygon -6459832 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6459832 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 3.0pre5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
