 globals[moves                 ; number of moves so far
         winner?               ; false until someone wins
         PLAYER-1              ; a player and colored red
         PLAYER-2              ; the other player and CONNIE both colored blue
         current-player        ; used to control the order of play
         next-player?          ; shows next player

                               ; the following are parts of CONNIE's (Connect4 Informed Engine) brain
                               ; x & y are the built-in Netlogo patch coordinates
         h                     ; positive number horizontal coordinate h = x+4
         p                     ; single number patch coordinate p = x+7y+18
         n                     ; counts steps in blue-check and red-check
         winb                  ; counts the checkers in a line of 4 patches during blue-check
         winr                  ;    "    "     "     "  "  "   "  "    "       "   red    "
         pblue                 ; signals the detection of a line of 3 blue checkers
         pred                  ;    "     "      "      " "   "   " 3 red      "
         block                 ; closes and opens the red and blue check results to CONNIE
         redlist               ; single number 'p' patch coordinate list of red checkers
         bluelist              ; single number 'p' patch coordinate list of blue checkers
         allist                ; list of all 'p' used to adjust the column lists as play proceeds
         column1               ; a list of the checkers in each column by 'p' number
         column2               ; "   "   "  "      "     "   "     "    "  "     "
         column3               ; "   "   "  "      "     "   "     "    "  "     "
         column4               ; "   "   "  "      "     "   "     "    "  "     "
         column5               ; "   "   "  "      "     "   "     "    "  "     "
         column6               ; "   "   "  "      "     "   "     "    "  "     "
         column7               ; "   "   "  "      "     "   "     "    "  "     "
         ]

patches-own [anyone?              ; is there a checker here?
            ]

to setup                          ; setting up the game board and the program for some of CONNIE's brain
   ca                             ; clear all
   set allist [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 ; list of all 'p' showing
               24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 0] ; patches unused
   set column1 [1 8 15 22 29 36 0]             ; 7 lists of the checkers on the board by 'p' number
   set column2 [2 9 16 23 30 37 0]             ; with the inital condition set to full, plus 0 at the top
   set column3 [3 10 17 24 31 38 0]            ; initial condition
   set column4 [4 11 18 25 32 39 0]            ;    "        "
   set column5 [5 12 19 26 33 40 0]            ;    "        "
   set column6 [6 13 20 27 34 41 0]            ;    "        "
   set column7 [7 14 21 28 35 42 0]            ;    "        "
   set bluelist []                ; list of the blue checkers by 'p' number
   set redlist []                 ;   "   "  "   red     "     "  "     "
   set block false                ; initial condition
   set next-player? "PLAYER-1"    ;    "        "
   set winner? false              ;    "        "
   text-1                         ; shows the 'p' values at the patches on the game board
   ask patches[ifelse pycor + screen-edge-y = 0[
   set pcolor grey                ; fills the un-needed row
   set anyone? false
   ][
   set pcolor yellow              ; colors the game-board
   set anyone? false
   sprout 1[
   set shape "circle"             ; shapes the game-board
   set color 9.999
   ]]]
end

to text-1                         ; shows the 'p' values at the patches on the game board
   crt 1
   ask turtles [setxy 3 3 plc set plabel
  "36               37               38               39              40               41               42"]
   ask turtles [setxy 3 2 plc set plabel
  "29               30               31               32              33               34               35"]
   ask turtles [setxy 3 1 plc set plabel
  "22               23               24               25              26               27               28"]
   ask turtles [setxy 3 0 plc set plabel
  "15               16               17               18              19               20               21"]
   ask turtles [setxy 3 -1 plc set plabel
  "08               09               10               11              12               13               14"]
   ask turtles [setxy 3 -2 plc set plabel
  "01               02               03               04              05               06               07"]
end

to plc                 ; sets text color
   ht set plabel-color 0
end

to add-piece [x]       ; the button column numbers or xcor's read L to R -3 -2 -1 0 1 2 3
                       ; which after the transformation h=x+4 becomes     1  2  3 4 5 6 7
   locals [
   move-ready          ; shows that checker is ready to be placed
   listing             ; the list of ycor's
   y                   ; vertical coordinate of a patch
          ]
   ifelse winner? = true [stop][                          ; don't continue if the game is over
   set move-ready false
   set listing [-2 -1 0 1 2 3]                            ; lists the ycor's or row numbers
   if value-from patch x 3 [anyone?] = false [            ; used to see if there are any legal sites
   cct 1 [set shape "circle"                              ; to make a checker
   without-interruption [
   foreach listing [
   ask patch x ? [if anyone? = false and move-ready = false [
   set y pycor                                            ; y coordinate of the checker
   set anyone? true                                       ; this patch is now in use
   set move-ready true
   ]]]]
   ifelse remainder (moves + 2) 2 = 0 [                   ; creates 1's and 0's to
   set color red set current-player "PLAYER-1"][          ; sets the order of play
   set color blue set current-player "PLAYER-2 & CONNIE"] ; player-2 or CONNIE is second to move
   ifelse color = blue[set next-player? "PLAYER-1"][
   set next-player? "PLAYER-2 or CONNIE"]
   setxy x y                                              ; coordinates of the checker
   set moves moves + 1                                    ; the checker is placed

   ; CONNIE's brain starts in this short section here
   ; CONNIE's alternative positive number horizontal coordinate is h = x + 4
   ; and her alternative single number patch coordinate p = x + 7y + 18, hence the following code:
   set h x + 4  set p x + (7 * y) + 18
   ; also for h from p she uses, ifelse remainder (p + 7) 7 = 0[set h 7][set h remainder (p + 7) 7]
   ; checkers are listed according to position 'p' and by color, then checked for winning sets of 4
   ; and this must be completed for each move, (a checker placed), first for red and then for blue
   ; but if CONNIE is ON, for blue only after she has made her checks, also because she uses parts of
   ; the same procedure, 'set block true' in the code below temporarily isolates CONNIE from this code
   if color = red[
   set block true set redlist sort fput p redlist redcheck set block false]    ; checks for a red win
   if color = blue[
   set block true set bluelist sort fput p bluelist bluecheck set block false] ; checks for a blue win
   ]]]

   ; this section is not CONNIE code
   ifelse winner? = true [stop][
   ifelse moves = 42 [set moves 0 set next-player? "PLAYER-1"
   print " No More Moves Left - Its a Draw!" print " Hit START PLAY to re-start" stop][

   ; CONNIE continues here with the code that removes 'p' numbers from allist as checkers are placed
   foreach allist[if member? ? redlist = true[set allist remove ? allist]]
   foreach allist[if member? ? bluelist = true[set allist remove ? allist]]
   ; then uses allist to remove 'p' numbers of checkers from column lists so that
   foreach column1[if member? ? allist = false[set column1 remove ? column1]]
   foreach column2[if member? ? allist = false[set column2 remove ? column2]]
   foreach column3[if member? ? allist = false[set column3 remove ? column3]]
   foreach column4[if member? ? allist = false[set column4 remove ? column4]]
   foreach column5[if member? ? allist = false[set column5 remove ? column5]]
   foreach column6[if member? ? allist = false[set column6 remove ? column6]]
   foreach column7[if member? ? allist = false[set column7 remove ? column7]]
   ; the first 'p' number of a column is available for CONNIE to use to
   ; test for a blue line of 3 to make into a blue line of 4 (to make an immediate win)
   if CONNIE = true and remainder (moves + 2) 2 != 0[
   if moves = 1[consplay]                ; to implement a set play for blue's first move
   if (first column1) != 0[
   set bluelist fput (first column1) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column2) != 0[
   set bluelist fput (first column2) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column3) != 0[
   set bluelist fput (first column3) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column4) != 0[
   set bluelist fput (first column4) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column5) != 0[
   set bluelist fput (first column5) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column6) != 0[
   set bluelist fput (first column6) bluelist bluecheck set bluelist remove-item 0 bluelist]
   if (first column7) != 0[
   set bluelist fput (first column7) bluelist bluecheck set bluelist remove-item 0 bluelist]
   ifelse pblue = true[consplay][

   ; or to test for a red line of 3 to STOP red with a blue checker {best defence)
   if random (hard-play-easy) = 0[ ; changes the availability of the next 14 lines of code
   if (first column1) != 0[
   set redlist fput (first column1) redlist redcheck set redlist remove-item 0 redlist]
   if (first column2) != 0[
   set redlist fput (first column2) redlist redcheck set redlist remove-item 0 redlist]
   if (first column3) != 0[
   set redlist fput (first column3) redlist redcheck set redlist remove-item 0 redlist]
   if (first column4) != 0[
   set redlist fput (first column4) redlist redcheck set redlist remove-item 0 redlist]
   if (first column5) != 0[
   set redlist fput (first column5) redlist redcheck set redlist remove-item 0 redlist]
   if (first column6) != 0[
   set redlist fput (first column6) redlist redcheck set redlist remove-item 0 redlist]
   if (first column7) != 0[
   set redlist fput (first column7) redlist redcheck set redlist remove-item 0 redlist]]
   ifelse pred = true [consplay][
   ]]
   consplay]
   ]]
end

to consplay
   ifelse remainder (moves + 2) 2 = 0[stop][            ; CONNIE must wait her turn
   ifelse moves = 1[ifelse (p = 4) = true[              ; a set play for move 1
   add-piece 1][add-piece 0]                            ; "  "    "   "    "  "
   if view-brain = true[print " a set play "]][         ; "  "    "   "    "  "
   ifelse pblue = true or pred = true[add-piece con-h][ ; CONNIE adds a checker in play
   add-piece -3 + random 7 if view-brain = true[        ; random if CONNIE has no better play
   print " p="+ p +" random-play "]]]]                  ; prints data for view-brain
   flag                                                 ;    "     "   "    "     "
   set pblue false set pred false                       ; resets conquer and conquest
end

to conquer                                         ; finds the 'p' number for CONNIE from blue-check
   set p item 0 bluelist set pblue true stop
end

to conquest                                        ; finds the 'p' number for CONNIE from red-check
   set p item 0 redlist set pred true stop
end

to-report con-h                                    ; transforms p to h and back to x
   ifelse remainder (p + 7) 7 = 0[set h 7][set h remainder (p + 7) 7]
   if h = 1 [report -3] if h = 2[report -2] if h = 3[report -1]
   if h = 4[report 0] if h = 5[report 1] if h = 6[report 2] if h = 7[report 3]
end

to flag                                            ; prints some data for view-brain
   if view-brain = true and pblue = true[print " p="+ p +" a blue win"]
   if view-brain = true and pred = true[print " p="+ p +" stops a red 4"]
end

;  more brain below, but some of it is equivalent to the check-for-wins code in pre-CONNIE models.

to swinb                                           ; counts blue checkers in a line
   set winb winb + 1
end

to swinr                                           ; counts red checkers in a line
   set winr winr + 1
end

to winsb                                           ; detect a blue win and in-line conditions for
   set n n + 1                                     ; CONNIE's 'conquer' reporter of bluecheck
   ifelse winb = 4[
   ifelse CONNIE = true[
   if n >= 1 and n <= 69[if view-brain = true and pblue = true[
   print " n="+ n +" a blue 4"]                        ; prints data for view-brain
   ifelse block = false[conquer set winb 0][won]]][won]][
   set winb 0]
end

to winsr                                           ; detect a red win and in-line conditions
   set n n + 1                                     ; CONNIE's 'conquest' reporter of redcheck
   ifelse winr = 4[
   ifelse CONNIE = true[
   if n >= 70 and n <= 138[if view-brain = true[
   print " n="+ n +" just in time"]                         ; prints data for view-brain
   ifelse block = false[conquest set winr 0][won]]][won]][
   set winr 0]
end

to won
   set winner? true print " " + current-player + " won the game in " + moves + " moves! "
   set moves 0 set winb 0 set winr 0 set next-player? "PLAYER-1" stop
end

;The 'P' diagram        36  37  38  39  40  41  42
;                       29  30  31  32  33  34  35
;                       22  23  24  25  26  27  28
;                       15  16  17  18  19  20  21
;                       08  09  10  11  12  13  14
;                       01  02  03  04  05  06  07
to bluecheck
   set n 0
   foreach[01 08 15 22][if member? ? bluelist = true[swinb]]winsb    ; n = 1 up/down
   foreach[08 15 22 29][if member? ? bluelist = true[swinb]]winsb
   foreach[15 22 29 36][if member? ? bluelist = true[swinb]]winsb
   foreach[02 09 16 23][if member? ? bluelist = true[swinb]]winsb
   foreach[09 16 23 30][if member? ? bluelist = true[swinb]]winsb    ; n = 5
   foreach[16 23 30 37][if member? ? bluelist = true[swinb]]winsb
   foreach[03 10 17 24][if member? ? bluelist = true[swinb]]winsb
   foreach[10 17 24 31][if member? ? bluelist = true[swinb]]winsb
   foreach[17 24 31 38][if member? ? bluelist = true[swinb]]winsb
   foreach[04 11 18 25][if member? ? bluelist = true[swinb]]winsb    ; n = 10
   foreach[11 18 25 32][if member? ? bluelist = true[swinb]]winsb
   foreach[18 25 32 39][if member? ? bluelist = true[swinb]]winsb
   foreach[05 12 19 26][if member? ? bluelist = true[swinb]]winsb
   foreach[12 19 26 33][if member? ? bluelist = true[swinb]]winsb
   foreach[19 26 33 40][if member? ? bluelist = true[swinb]]winsb    ; n = 15
   foreach[06 13 20 27][if member? ? bluelist = true[swinb]]winsb
   foreach[13 20 27 34][if member? ? bluelist = true[swinb]]winsb
   foreach[20 27 34 41][if member? ? bluelist = true[swinb]]winsb
   foreach[07 14 21 28][if member? ? bluelist = true[swinb]]winsb
   foreach[14 21 28 35][if member? ? bluelist = true[swinb]]winsb    ; n = 20
   foreach[21 28 35 42][if member? ? bluelist = true[swinb]]winsb

   foreach[01 02 03 04][if member? ? bluelist = true[swinb]]winsb    ; across
   foreach[02 03 04 05][if member? ? bluelist = true[swinb]]winsb
   foreach[03 04 05 06][if member? ? bluelist = true[swinb]]winsb
   foreach[04 05 06 07][if member? ? bluelist = true[swinb]]winsb    ; n = 25
   foreach[08 09 10 11][if member? ? bluelist = true[swinb]]winsb
   foreach[09 10 11 12][if member? ? bluelist = true[swinb]]winsb
   foreach[10 11 12 13][if member? ? bluelist = true[swinb]]winsb
   foreach[11 12 13 14][if member? ? bluelist = true[swinb]]winsb
   foreach[15 16 17 18][if member? ? bluelist = true[swinb]]winsb    ; n = 30
   foreach[16 17 18 19][if member? ? bluelist = true[swinb]]winsb
   foreach[17 18 19 20][if member? ? bluelist = true[swinb]]winsb
   foreach[18 19 20 21][if member? ? bluelist = true[swinb]]winsb
   foreach[22 23 24 25][if member? ? bluelist = true[swinb]]winsb
   foreach[23 24 25 26][if member? ? bluelist = true[swinb]]winsb    ; n = 35
   foreach[24 25 26 27][if member? ? bluelist = true[swinb]]winsb
   foreach[25 26 27 28][if member? ? bluelist = true[swinb]]winsb
   foreach[29 30 31 32][if member? ? bluelist = true[swinb]]winsb
   foreach[30 31 32 33][if member? ? bluelist = true[swinb]]winsb
   foreach[31 32 33 34][if member? ? bluelist = true[swinb]]winsb    ; n = 40
   foreach[32 33 34 35][if member? ? bluelist = true[swinb]]winsb
   foreach[36 37 38 39][if member? ? bluelist = true[swinb]]winsb
   foreach[37 38 39 40][if member? ? bluelist = true[swinb]]winsb
   foreach[38 39 40 41][if member? ? bluelist = true[swinb]]winsb
   foreach[39 40 41 42][if member? ? bluelist = true[swinb]]winsb    ; n = 45

   foreach[21 27 33 39][if member? ? bluelist = true[swinb]]winsb    ; bk/diagonal
   foreach[14 20 26 32][if member? ? bluelist = true[swinb]]winsb
   foreach[20 26 32 38][if member? ? bluelist = true[swinb]]winsb
   foreach[07 13 19 25][if member? ? bluelist = true[swinb]]winsb
   foreach[13 19 25 31][if member? ? bluelist = true[swinb]]winsb    ; n = 50
   foreach[19 25 31 37][if member? ? bluelist = true[swinb]]winsb
   foreach[06 12 18 24][if member? ? bluelist = true[swinb]]winsb
   foreach[12 18 24 30][if member? ? bluelist = true[swinb]]winsb
   foreach[18 24 30 36][if member? ? bluelist = true[swinb]]winsb
   foreach[05 11 17 23][if member? ? bluelist = true[swinb]]winsb    ; n = 55
   foreach[11 17 23 29][if member? ? bluelist = true[swinb]]winsb
   foreach[04 10 16 22][if member? ? bluelist = true[swinb]]winsb

   foreach[15 23 31 39][if member? ? bluelist = true[swinb]]winsb    ; fd/diagonal
   foreach[08 16 24 32][if member? ? bluelist = true[swinb]]winsb
   foreach[16 24 32 40][if member? ? bluelist = true[swinb]]winsb    ; n = 60
   foreach[01 09 17 25][if member? ? bluelist = true[swinb]]winsb
   foreach[09 17 25 33][if member? ? bluelist = true[swinb]]winsb
   foreach[17 25 33 41][if member? ? bluelist = true[swinb]]winsb
   foreach[02 10 18 26][if member? ? bluelist = true[swinb]]winsb
   foreach[10 18 26 34][if member? ? bluelist = true[swinb]]winsb    ; n = 65
   foreach[18 26 34 42][if member? ? bluelist = true[swinb]]winsb
   foreach[03 11 19 27][if member? ? bluelist = true[swinb]]winsb
   foreach[11 19 27 35][if member? ? bluelist = true[swinb]]winsb
   foreach[04 12 20 28][if member? ? bluelist = true[swinb]]winsb    ; n = 69
end

to redcheck
   set n 69
   foreach[01 08 15 22][if member? ? redlist = true[swinr]]winsr     ; n = 70 up/down
   foreach[08 15 22 29][if member? ? redlist = true[swinr]]winsr
   foreach[15 22 29 36][if member? ? redlist = true[swinr]]winsr
   foreach[02 09 16 23][if member? ? redlist = true[swinr]]winsr
   foreach[09 16 23 30][if member? ? redlist = true[swinr]]winsr
   foreach[16 23 30 37][if member? ? redlist = true[swinr]]winsr     ; n = 75
   foreach[03 10 17 24][if member? ? redlist = true[swinr]]winsr
   foreach[10 17 24 31][if member? ? redlist = true[swinr]]winsr
   foreach[17 24 31 38][if member? ? redlist = true[swinr]]winsr
   foreach[04 11 18 25][if member? ? redlist = true[swinr]]winsr
   foreach[11 18 25 32][if member? ? redlist = true[swinr]]winsr     ; n = 80
   foreach[18 25 32 39][if member? ? redlist = true[swinr]]winsr
   foreach[05 12 19 26][if member? ? redlist = true[swinr]]winsr
   foreach[12 19 26 33][if member? ? redlist = true[swinr]]winsr
   foreach[19 26 33 40][if member? ? redlist = true[swinr]]winsr
   foreach[06 13 20 27][if member? ? redlist = true[swinr]]winsr     ; n = 85
   foreach[13 20 27 34][if member? ? redlist = true[swinr]]winsr
   foreach[20 27 34 41][if member? ? redlist = true[swinr]]winsr
   foreach[07 14 21 28][if member? ? redlist = true[swinr]]winsr
   foreach[14 21 28 35][if member? ? redlist = true[swinr]]winsr
   foreach[21 28 35 42][if member? ? redlist = true[swinr]]winsr     ; n = 90

   foreach[01 02 03 04][if member? ? redlist = true[swinr]]winsr     ; across
   foreach[02 03 04 05][if member? ? redlist = true[swinr]]winsr
   foreach[03 04 05 06][if member? ? redlist = true[swinr]]winsr
   foreach[04 05 06 07][if member? ? redlist = true[swinr]]winsr
   foreach[08 09 10 11][if member? ? redlist = true[swinr]]winsr     ; n = 95
   foreach[09 10 11 12][if member? ? redlist = true[swinr]]winsr
   foreach[10 11 12 13][if member? ? redlist = true[swinr]]winsr
   foreach[11 12 13 14][if member? ? redlist = true[swinr]]winsr
   foreach[15 16 17 18][if member? ? redlist = true[swinr]]winsr
   foreach[16 17 18 19][if member? ? redlist = true[swinr]]winsr     ; n = 100
   foreach[17 18 19 20][if member? ? redlist = true[swinr]]winsr
   foreach[18 19 20 21][if member? ? redlist = true[swinr]]winsr
   foreach[22 23 24 25][if member? ? redlist = true[swinr]]winsr
   foreach[23 24 25 26][if member? ? redlist = true[swinr]]winsr
   foreach[24 25 26 27][if member? ? redlist = true[swinr]]winsr     ; n = 105
   foreach[25 26 27 28][if member? ? redlist = true[swinr]]winsr
   foreach[29 30 31 32][if member? ? redlist = true[swinr]]winsr
   foreach[30 31 32 33][if member? ? redlist = true[swinr]]winsr
   foreach[31 32 33 34][if member? ? redlist = true[swinr]]winsr
   foreach[32 33 34 35][if member? ? redlist = true[swinr]]winsr     ; n = 110
   foreach[36 37 38 39][if member? ? redlist = true[swinr]]winsr
   foreach[37 38 39 40][if member? ? redlist = true[swinr]]winsr
   foreach[38 39 40 41][if member? ? redlist = true[swinr]]winsr
   foreach[39 40 41 42][if member? ? redlist = true[swinr]]winsr

   foreach[21 27 33 39][if member? ? redlist = true[swinr]]winsr     ; n =115 bk/diagonal
   foreach[14 20 26 32][if member? ? redlist = true[swinr]]winsr
   foreach[20 26 32 38][if member? ? redlist = true[swinr]]winsr
   foreach[07 13 19 25][if member? ? redlist = true[swinr]]winsr
   foreach[13 19 25 31][if member? ? redlist = true[swinr]]winsr
   foreach[19 25 31 37][if member? ? redlist = true[swinr]]winsr     ; n = 120
   foreach[06 12 18 24][if member? ? redlist = true[swinr]]winsr
   foreach[12 18 24 30][if member? ? redlist = true[swinr]]winsr
   foreach[18 24 30 36][if member? ? redlist = true[swinr]]winsr
   foreach[05 11 17 23][if member? ? redlist = true[swinr]]winsr
   foreach[11 17 23 29][if member? ? redlist = true[swinr]]winsr     ; n = 125
   foreach[04 10 16 22][if member? ? redlist = true[swinr]]winsr

   foreach[15 23 31 39][if member? ? redlist = true[swinr]]winsr     ; fd/diagonal
   foreach[08 16 24 32][if member? ? redlist = true[swinr]]winsr
   foreach[16 24 32 40][if member? ? redlist = true[swinr]]winsr
   foreach[01 09 17 25][if member? ? redlist = true[swinr]]winsr     ; n = 130
   foreach[09 17 25 33][if member? ? redlist = true[swinr]]winsr
   foreach[17 25 33 41][if member? ? redlist = true[swinr]]winsr
   foreach[02 10 18 26][if member? ? redlist = true[swinr]]winsr
   foreach[10 18 26 34][if member? ? redlist = true[swinr]]winsr
   foreach[18 26 34 42][if member? ? redlist = true[swinr]]winsr     ; n = 135
   foreach[03 11 19 27][if member? ? redlist = true[swinr]]winsr
   foreach[11 19 27 35][if member? ? redlist = true[swinr]]winsr
   foreach[04 12 20 28][if member? ? redlist = true[swinr]]winsr     ; n = 138
end

















@#$#@#$#@
GRAPHICS-WINDOW
366
10
775
440
3
3
57.0
1
10
1
1
1

CC-WINDOW
9
278
357
441
Command Center

BUTTON
374
391
429
424
1
wait 0.3 \nadd-piece -3
NIL
1
T
OBSERVER
T

BUTTON
430
391
485
424
2
wait 0.3\nadd-piece -2
NIL
1
T
OBSERVER
T

BUTTON
486
390
541
423
3
wait 0.3\nadd-piece -1
NIL
1
T
OBSERVER
T

BUTTON
543
390
598
423
4
wait 0.3\nadd-piece 0
NIL
1
T
OBSERVER
T

BUTTON
600
389
655
422
5
wait 0.3\nadd-piece 1
NIL
1
T
OBSERVER
T

BUTTON
657
389
712
422
6
wait 0.3\nadd-piece 2
NIL
1
T
OBSERVER
T

BUTTON
714
389
769
422
7
wait 0.3\nadd-piece 3
NIL
1
T
OBSERVER
T

BUTTON
249
63
359
96
START PLAY
setup
NIL
1
T
OBSERVER
T

TEXTBOX
25
164
219
246
PLAYER-1 always goes first and\ncan play against CONNIE by switching CONNIE to ON.\nPLAYER-2 can get CONNIE's help by switching her ON for a while.

TEXTBOX
250
99
359
148
PLAYER-1 is RED\nPLAYER-2 and\nCONNIE are BLUE.

TEXTBOX
23
11
212
164
CONNECT FOUR   Click buttons\n1 to 7 to get checkers in line on a row, column or diagonal.\n4 in line is a win\n5 in line gives 2 wins \n6 in line gives 3 wins\n7 in line gives 4 wins \nALWAYS click START PLAY to \nstart or restart a game.

MONITOR
217
10
360
59
NEXT PLAYER
next-player?
0
1

MONITOR
179
63
243
112
next move
moves + 1
0
1

SWITCH
252
197
360
230
view-brain
view-brain
0
1
-1000

TEXTBOX
24
243
227
277
   !!!! view-brain slows CONNIE !!!!     p = patch number   n = line number                  \n

SWITCH
251
155
360
188
CONNIE
CONNIE
0
1
-1000

SLIDER
251
239
361
272
hard-play-easy
hard-play-easy
0
2
0
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This model is a Netlogo version of the classic Milton Bradley/Hasbro game "Connect Four: The Vertical Four-in-a-Row Checkers Game."  It is intended to be a two-player game, and is entirely for fun!

The rules are simple.  Each player places a colored-checker in one of the seven slots.  The first player to get four in a row, column, or on a diagonal wins.


HOW IT WORKS
------------
SETUP creates the game board, with seven columns and six rows.  The grey space at the top of the screen is necessary because Netlogo's graphic canvas won't permit the exact dimensions of the game space.  The white "holes" are turtles that serve as place-holders for the colored checkers.  They have no other function in the game.

ADD-PIECE [x] is called by the seven buttons on the bottom of the screen. They are used by the players to add each checker to the board.  ADD-PIECE [x] finds the first "empty" space in the relevant column, and inserts a checker of the player's color.  This procedure uses a list to evaluate possible y coordinates to find the first empty space in a column.  Once placed on the board, the new turtle sets its color to that of the player, and uses SET-POSSIBLE-HEADINGS to set its own list of possible headings for evaluating each move.

CHECK-FOR-WINNER is only run after seven moves have been completed.  This procedure uses the two lists (single-headings and double-headings) to look in each of the legal directions to determine if the most recently-played checker has won the game.  The procedure looks first to see if the latest piece is at the end of a chain of 4 like-checkers (single-headings), and then looks to see if it is inside (not on the end) of a string of four checkers.  This analysis involves looking forward two checkers on a heading, and then "turning around" (by adding 180 degrees to the heading) to look at the two checkers in the opposite direction.

When the game is over, the command window prints which side won, and in how many steps.

To play again, either use the "PLAY AGAIN?" button (calling SETUP), or hit any one of the seven button on the bottom of the screen.


HOW TO USE IT
-------------
It's pretty self-explanatory.  Find a friend, and start seeing who can win the game first.  Its the 1970's version of outwit, outlast, and outplay!


THINGS TO NOTICE
----------------
The code for this model is remarkably short, with just three meaningful procedures. (STARTUP loads the model but isn't necessary).
Notice how the ADD-PIECES and CHECK-FOR-WINNERS procedures enable the game to work.  That is the heart of the model.

The game will only run under Netlogo 2.0 and later because of its reliance on the "ask patch-at-heading-and-distance" primitive.

The model is not really "agent-based" in the traditional sense of the word, but it illustrates the robustness of the Netlogo environment and its ease of use.  And it is just plain fun to play!


EXTENDING THE MODEL
-------------------
One possible extension would be to build a procedure that would provide a "Help" mode - either pointing out existing patterns of three like-checkers or warning when a player is
"in check."

Another extension would be to have the checkers appear at the top of the screen, and then "drop" into place in each column.

A one-player version of the game (with a computer opponent) is probably the most challenging extension to the model, and would certainly take longer than the initial model to code, but it would be an amazing accomplishment!


RELATED MODELS
--------------
This model is related to the game models in the Netlogo Models Library.  Unlike Frogger and Pac-Man, which require the awkward use of buttons to replicate the original joysticks, CONNECT FOUR plays completely like the original - with the one exception of having the ability to make all the checkers drop to the table.


CREDITS AND REFERENCES
----------------------
This model was inspired by the Hasbro game "CONNECT FOUR" and is dedicated to Nicholas Gizzi, an enthusiastic six year-old who is fascinated by the original Connect Four, and is now learning to move virtual turtles around a computer screen!  He dutifully beta-tested this model.

The model was created by Michael C. Gizzi with assistance from Boyce Baker and Richard Vail. The model is copyright 2003 by the Center for Agent-Based Modeling, Mesa State College, 2508 Blichman Avenue, Grand Junction, CO 81505.  http://www.modelingcomplexity.org.

The original board game CONNECT FOUR is copyright Hasbro 1998.

THE MODEL IS INTENDED SOLELY FOR EDUCATIONAL AND ENTERTAINMENT PURPOSES.  IT MAY NOT BE REPRODUCED FOR COMMERCIAL USE WITHOUT VIOLATING EITHER OF THE COPYRIGHTS LISTED HERE.

NOTES ON connect4 and CONNIE
--------------------------
This model was extended and changed as a compliment to Michael for his original work
in giving us connect4, which is on the Netlogo Community Models site. Also it was
to respond to his suggestion that "A one-player version of the game (with a computer opponent) is probably the most challenging extension to the model". HOW RIGHT HE WAS!!!

I have added a different CHECK-FOR-WINNER procedure; also CONNIE who plays a fair game,
adjustable between 'hard' and 'easy', for those who like to play against a machine.

CONNIE exhibits no intelligence, in fact a large proportion of her play is random because this avoids some of the repetition of machine play. Also it was discovered that the spatial relations of the game demand very complicated procedures to provide a greater level of play. The prime reason for this is the ease with which twofold, threefold, etc lines of 3 with common members can be formed and the extreme difficulty of anticipating the formation.

The Procedures section includes detailed explanations of the code and some of its working can be viewed in the Command Center.

Derek Rush who can be contacted on derekrush@beeb.net February 2004.


















@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
