; ###################################################
; ########  DECLARATIONS ###########################3
; ################################################

globals [
; GLOBAL CONSTANTS ############################
; numbers which are constants throughout the whole code, but which could be given another value
  start-euros         ; amount of euros each turtle starts with
  min-invest-energy   ; the minimum energy a turtle needs in order to be able to do an investment
  min-invest-balance  ; the minimum balance a turtle needs in order to be able to do an investment
  no-food-needed      ; having x energy, I don't need to buy more food.
  avg-food-trans-amnt ; average amount for a food transaction
  spend-all-on-food   ; if my energy is lower than x, I have a tendency to spend all my money on food, if i get the opportunity
  diamond-eagerness   ; after having bought a diamond, it is still sensible to have x euro's left (cahs + savings) for daily food.
  diamonds-per-partij ; amount of diamonds in a partij (shipping)
  repay-loan-limit    ; if more then x euro's in cash or savings, I want to repay my loans.
  withdraw-limit      ; if less then x euro's cash, I want to do a withdrawal from savings
  withdrawal          ; the amount of cash in a withdrawal from savings. Also for deposits and repayment of loans.
  deposit-limit       ; if more than x euro's cash, I want to make a deposit to savings
  bankrupt-limit      ; if more than x on open-loans, I go bankrupt
  bankrupt-left-over  ; this is the amount of cash I am allowed to keep, after going bankrupt
  bankrupt-time       ; after x plotpoints of time being bankrupt, all is forgiven and forgotten, and I can start over again
  profit-share-limit  ; profits of the bank will not be shared when smaller than x euro's
  payment-cash-left   ; after doing a large payment, I want x euro's to keep in my pocket as cash. The rest goes to/from savings.
  loan-limit          ; people having less than x euro's of money, cannot get a loan from the bank. Too poor.
  loan-factor         ; The maximum loan I can get is X times the money I have at the moment. The bank does not want it to be too risky.
  invest-eagerness    ; after having done an investment, I want to have x euro's of cash (+ savings) left for daily food.
  mineworkers-needed  ; the amount of mineworkers needed to get raw diamonds from an investment. These people need to be paid.
  mineworker-limit    ; if I have more than x of money (cash + savings), I don't want to do this dirty work in the mines anymore.
  invest-plot-freq    ; update the investment plot every x plotpoint rounds (so others plots are updated x times more often)
; GENERAL ############################
  ntransactions       ; keeps count of the total number of transactions done
  dStrade 			      ; difference of Sum in trade. for drawing graph 1: trade per plotpoint
  plotpoints          ; total number of rounds, or x-axis points in the plots
; INVESTMENT GLOBALS ############################
  dStradeDiamonds     ; as dStrade, but for diamonds
  Sinvested      		  ; total sum of euro's used for investment
  diamondProduction   ; total diamonds produced
  SeOut 			        ; The total sum of euro's flowing out of the community
  SeIn 				        ; total sum of euro's flowing into the community from outside
  maxROI              ; for scaling of the ROIcurve
; BANK GLOBALS ################################
  bank-loans          ; sum of all bank loans
  bank-reserves       ; that which should stay in the bank
  bank-deposits       ; sum of all bank savings
  bank-loan-available ; amount available for loans
  bank-profit         ; revenues of the bank, earned from for example interest.
  shared              ; total amount of shares given to savings account holders
]

turtles-own [
  member 			        ; true if member. One can only use local currency units if member
  euros 			        ; balance of euros in pocket
  units 			        ; balance of units
  Strade			        ; total sum of trade: euros and units ever earned by this turtle
  energy 			        ; money is used to buy food; if no food, no luxury
  energycolor			    ; internal property used to give a hungry turtle a paler color
; INVESTMENT PROPERTIES ####################################
  invested      		  ; total sum of euro's used for investment by this turtle
  RawDiamonds         ; number of Diamonds the turtle offers for selling
  Diamonds            ; number of bought diamonds
  Diamonds-value      ; value of bought diamonds
  Rawat               ; list of raw diamond investment dates
  Returns             ; list of earnings from sold diamonds
  Diamonds-sold       ; number of sold diamonds
; BANK PROPERTIES ##############################################
  savings             ; savings account
  org-loans           ; original loan amount
  open-loans          ; real debt = original loan + interest (interest% is given per plotpoint)
  loan-time           ; time in plotpoints when loan started
  Sinterest           ; all interest paid
  Bankrupt            ; is turtle bankrupt
]

; #################################################
; ########## INIT #########################
; #################################################

to set-constants
; all constants which for some or another reason could ever need adjusting go here, so it is easy to adjust them.
  set start-euros         (50) ; amount of euros each turtle starts with
  set min-invest-energy   (40) ; the minimum energy a turtle needs in order to be able to do an investment
  set min-invest-balance  (75) ; the minimum balance a turtle needs in order to be able to do an investment
  set no-food-needed      (45) ; having x energy, I don't need to buy more food.
  set avg-food-trans-amnt (10) ; average amount for a food transaction
  set spend-all-on-food   (15) ; if my energy is lower than x, I have a tendency to spend all my money on food, if i get the opportunity
  set diamond-eagerness   (50) ; after having bought a diamond, it is still sensible to have x euro's left (cahs + savings) for daily food.
  set diamonds-per-partij (10) ; amount of diamonds in a partij (shipping)
  set repay-loan-limit    (35) ; if more than x euro's cahs + savings, repay the loans (repay everything above this)
  set withdraw-limit      (20) ; if less than x euro's cash, I want to do a withdrawal from savings
  set withdrawal          (10) ; the amount of cash in a withdrawal from savings. Also for deposits and repayment of loans.
  set deposit-limit       (50) ; if more than x euro's cash, I want to make a deposit to savings
  set bankrupt-limit      (600) ; if more than x on open-loans, I go bankrupt
  set bankrupt-left-over  (10) ; this is the amount of cash I am allowed to keep, after going bankrupt
  set bankrupt-time       (250) ; after x plotpoints of time being bankrupt, all is forgiven and forgotten, and I can start over again
  set profit-share-limit  (100) ; profits of the bank will not be shared when smaller than x euro's
  set payment-cash-left   (20) ; after doing a large payment, I want x euro's to keep in my pocket as cash. The rest goes to/from savings.
  set loan-limit          (75) ; people having less than x euro's of money, cannot get a loan from the bank. Too poor.
  set loan-factor         (2) ; The maximum loan I can get is X times the money I have at the moment. more is too risky.
  set invest-eagerness    (50) ; after having done an investment, I want to have x euro's of cash (+ savings) left for daily food.
  set mineworkers-needed  (15) ; the amount of mineworkers needed to get raw diamonds from an investment. These people need to be paid.
  set mineworker-limit    (35) ; if I have more than x of money (cash + savings), I don't want to do this dirty work in the mines anymore.
  set invest-plot-freq    (4) ; update the investment plot every x plotpoint rounds (so others plots are updated x times more often)
end

to setup
  ca
  set-constants
  set-default-shape turtles "person"
  crt 100
  ask turtles [
    fd 15
    set rawat []
    set returns []
    set bankrupt false
    ifelse who < 11                 ; todo: slider startmembers
      [set member true set color red]
      [set member false set color green]
    lt random 360
    fd (random-normal 50 10)
    set euros start-euros
;    spread-over-patches
    set energy (random-normal 24 4)
    if energy > 50 [set energy 50]
    if energy < 10 [set energy 10]
    showhunger
  ]
  setup-plots
end

to spread-over-patches ; spreads all turtles equally over patches, for testing purposes
  set xcor ((floor (who / 10)) * 3) - 14
  set ycor ((who mod 10) * 3) - 14
  set heading 0
end

to showhunger
;this procedure provides the color of the turtles: the less energy, the paler they are.
;Energyvalues above 25 are considered healthy, hence no pale color then.
;Energycolor is only used for setting the color for energy's < 25
   ifelse energy > 25
      [set energycolor 25]
      [set energycolor energy]
   if (color >= 50) and (color < 60)
      [set color scale-color green energycolor 50 0]
   if (color >= 10) and (color < 20)
      [set color scale-color red energycolor 55 0 ]
end


; #################################################
; ########## MOVING  #########################
; #################################################

to go
  repeat 100 [
    ask turtles [ move ]
    ask turtles [ without-interruption [trade] ]
    ask turtles [bank-traffic]
    update-bank
    ask turtles [invest]
  ]
  ask turtles [recalc-loans]
  share-profits
  update-plots
end

to test-go ;; a very controlled go for testing purposes
  ask turtle 0 [ fd 3 ]
  ask turtles [without-interruption [trade]]
  ask turtle 0 [fd 1]
end

to move
;this procedure lets a turtle move 4 steps. But if a turtle is hungry, it
;will move slower.
   if (bankrupcy) [stop]
   let speed 4
   rt random 10
   lt random 10
   if energy < 0 [set energy 0]
   if energy < 25
      [set speed (0.875 + (0.125 * energy))]
   decreaseenergy speed
   fd speed
end

to decreaseenergy [speed]
;this procedure takes care that with each step, energy is decreased
;the lower the speed, the less energy it costs, of course
   let energydecrease 0.261			; this is the ideal value: the expected value for just playing equal
 ; set for lower speeds, try 0.005 and 0.005 for an alternative, or 0.00333 and 0.01
   if speed < 3
      [set energydecrease (((0.5 * speed) + 0.5) * energydecrease / 2)]
 ; finally: setenergy
   set energy (energy - energydecrease)
   showhunger
end


; #################################################
; ########## TRADING #########################
; #################################################

to trade
  if (euros <= 0) [stop]
  if (bankrupcy) [stop]
  ifelse (                                                   ; if rich enough, and enough food, and diamonds available, buy diamonds
    (energy > min-invest-energy) and                         ; has enough food
    ((euros + savings) > min-invest-balance) and             ; rich enough
    (count other-turtles-here with [RawDiamonds > 0] > 0)    ; someone with diamonds is around
  )
    [buy-diamonds]                                           ; buy the diamonds
    [buy-food]                                               ; else, buy food
end

to buy-food
  if (energy > no-food-needed) [stop]                        ; I won't buy food when I don't need it
  let seller 0
; determine amount
  let amount -1
  while [amount < 0] [
    set amount (random-normal avg-food-trans-amnt 1)         ; amount is something around 10,
    if (amount > euros) [                                    ; unless I have euro's short,
      ifelse (energy < spend-all-on-food)                    ; in that case, if I have not too much energy
        [set amount euros]                                   ; use all my money on food
        [set amount 0]                                       ; or, if I do have enough energy, just keep the money and do nothing
    ]
  ]
  if (amount = 0) [stop]
  set seller random-one-of other-turtles-here with [bankrupt = false]
  if seller != nobody [
    if (random 2) = 0   [                   ;; 50% chance of trading with customer
      set euros-of seller (euros-of seller + amount)
      set euros (euros - amount)
      set Strade-of seller (Strade-of seller + amount)
      set dStrade (dStrade + amount)
      set energy energy + amount
      if energy > 50 [set Energy 50]
      set ntransactions ntransactions + 1
    ]
  ]
end

to buy-diamonds
  let seller 0
  set seller random-one-of other-turtles-here with [RawDiamonds > 0]
  if seller != nobody [
    let amount (euros + savings - diamond-eagerness)
  ; payment
    if ((pay-from amount) < 0) [stop]
    set euros-of seller (euros-of seller + amount)
    set Strade-of seller (Strade-of seller + amount)
  ; diamond transfer
    set diamonds diamonds + 1
    set RawDiamonds-of seller (RawDiamonds-of seller - 1)
  ; global counters
    set dStrade (dStrade + amount)
    set dStradeDiamonds (dStradeDiamonds + amount)
    set ntransactions ntransactions + 1
  ; turtle counters
    set diamonds-value diamonds-value + amount
    let partij Floor ((diamonds-sold-of seller) / diamonds-per-partij )  ; to which shipping does it belong??
    set diamonds-sold-of seller (diamonds-sold-of seller + 1)
    let templist (returns-of seller)
    let presentReturn item partij templist
    set presentReturn (presentReturn + amount)
    if (presentReturn > maxROI) [set maxROI ceiling (presentReturn - investmentThreshold)]
    set templist replace-item partij templist presentReturn
  ; update the list
    set returns-of seller templist
  ]
end

; #############################################################3
; ######################  BANKING  ########################
; #############################################################3

to bank-traffic
; procedure brings surplus cash to the bank after each move
; or gets cash from the savings account in case of shortage
; or pays off a loan in case of debts
;
; first check if I want to repay a loan
  if ((open-loans > 0) and ((euros + savings) > repay-loan-limit)) [
    let repayamount (euros + savings - repay-loan-limit)
    repay-loan repayamount
  ]
; check if I want to withdraw cash
  if (euros < withdraw-limit) [
    let cashToGet withdrawal
    if (savings < cashToGet) [set cashToGet savings]
    if (savings > 0) [withdraw-from-savings cashToGet]
  ]
; check if I want to deposit cash
  if (euros > deposit-limit) [
    ifelse (open-loans > 0)
      [repay-loan withdrawal]                    ; not a possible option, but anyways, leave it
      [deposit-to-savings withdrawal]
  ]
end

to update-bank
  without-interruption [
    set bank-deposits (sum values-from turtles [savings])
    set bank-loans (sum values-from turtles [org-loans])
    set bank-reserves ((bankreserves / 100) * bank-deposits)
    set bank-loan-available (bank-deposits + bank-profit - bank-reserves - bank-loans)
  ]
end

to recalc-loans   ; this procedure adds interest to the loans
  if (open-loans < 0) [stop]
  let base (1 + (interest% / 100))                ; interest% per day (= plotpoint)
  let newloan (open-loans * base)
  let calculated-interest (newloan - open-loans)
  set open-loans newloan
  set-bankrupcy
end

to share-profits      ; this procedure shares the bank profits with holders of savings accounts
  if (bank-profit > profit-share-limit) [                ; no sharing when profits are small
    without-interruption [
      let sumsavings (sum values-from turtles [savings])   ; how much savings are there?
      let sumtoshare (profit-share-% * bank-profit / 100)  ; how much will be shared?
      let shareInterest (100 * sumtoshare / sumsavings)    ; % of interest everybody will get on savings
      ask turtles with [savings > 0] [
        let bonus (savings * shareInterest / 100)          ; the bonus per turtle
        set savings (savings + bonus)                      ; give it to them
      ]
      set bank-profit (bank-profit - SumtoShare)           ; decrease profit
      set shared (shared + sumtoShare)                     ; increase counter
    ]
  ]
end


; ################### BASIC BANK FUNCTIONS ###############

; This function takes care that if you have to pay something, it is
; taken from your cash and/or savings in the right configuration
; meaning: 20 left on cash, the rest from savings.
; if there is not enough, then it returns a -1; if everything is ok,
; it returns a 0
; usage:   if ((pay-from amount) < 0) [stop]
to-report pay-from [amount]
  if ((savings + euros) < amount) [report -1]                     ; no payment possible if not enough money
  set savings (savings - (amount - (euros - payment-cash-left)))  ; take that much, so that x is left in cash, and take rest from savings
  ifelse (savings < 0) [                                          ; but if that means that savings goes negative, correct this
    set euros (payment-cash-left + savings)
    set savings 0
  ] [
    set euros payment-cash-left
  ]
  report 0
end

to withdraw-from-savings [amount]
  if (amount > savings) [set amount savings]
  set euros (euros + amount)
  set savings (savings - amount)
end

to deposit-to-savings [amount]
  if (amount > euros) [set amount euros]        ; in fact, not a necessary check, unless someone would put constants at inconsistent values
  set euros (euros - amount)
  set savings (savings + amount)
end

to-report get-loan [amount]
  set amount (test-loan amount)               ; how much loan can I get?
  set open-loans (open-loans + amount)        ; open-loans keeps the real value of the loan, so including interest
  set org-loans (org-loans + amount)          ; org-loans keeps the original value of the loan, so this stays equal until paid off
  set loan-time plotpoints                    ; the time at getting the loan, needed to calculate interest
  set euros (euros + amount)
; add the loan to the returns of the diamond partij it belongs to. Then, the return of the investment is :
; income from selling - invested amount + original loan received - actual loan remaining yet - paid interest so far.
; now add the received money in a new item in returns list, as you get the loan before doing the investment, there is no item yet.
  set returns lput amount returns
  update-bank
  report amount
end

to-report test-loan [amount] ; reports how much loan I can get
  if ((euros + savings) < loan-limit) [report 0]      ; we do not provide loans for poors
  if (open-loans > 0) [report 0]                      ; if you allready have a loan, first pay it off. More than one loan not allowed
; maximum loan is loan-factor (=2) times your money; more would be too risky
  if (((euros + savings) * loan-factor) < amount) [set amount ((euros + savings) * loan-factor)]
; the bank must have it available, and the savings of this turtle do not count for this availability
  if ((bank-loan-available - savings) < amount) [set amount 0]    ; if not available, then nothing! The bank doesn't provide partial loans,
  report amount                                                   ; because it wants to keep some safety range
end

to repay-loan [amount]
  if (open-loans < amount) [set amount open-loans]
  if ((pay-from amount) < 0) [stop]                                ; if I don't have enough money, then stop
  let remaining amount                                             ; remaining is the part which is paid on interest. here it is initialized
  if (org-loans > 0) [                                             ; keep track of original loan sum, to know how much bank has available
    set remaining (amount - org-loans)                             ; remaining is the sum left after repaying the original loan sum
    if (remaining < 0) [set remaining 0]                           ; remaining is 0 if the original loan sum isn't paid back completely
    set org-loans (org-loans - amount)
    if (org-loans < 0) [set org-loans 0]                           ; paid off original amount!
  ]
  set open-loans (open-loans - amount)                             ; decrease open loan
  set bank-profit (bank-profit + remaining)                        ; everything which is in remaining is pure profit for the bank
  if (remaining > 0) [                                             ; if any interest paid, it should be distracted from the diamond returns
    let index (position loan-time RawAt)                           ; first get the correct index number from RawAt
    let oldreturn (item index returns)
    set returns replace-item index returns (oldreturn - remaining) ; now distract the paid interest (= remaining) from the correct returns list item
  ]
  if (open-loans <= 0) [                                           ; congratulations, paid it all back including interest.
    set euros (euros - open-loans)
    set open-loans 0
    set org-loans 0
    set loan-time 0
  ]
  set Sinterest Sinterest + remaining                              ; adjust the interest counters
  update-bank
end

; this procedure checks if the turtle should be declared bankrupt
to set-bankrupcy
  if (open-loans > bankrupt-limit) [  ; go bankrupt
  ; all damage is on the account of the bank, and goes from its profit. This might mean that no more loans will be available to others.
    set bank-profit (bank-profit - bankrupt-left-over + euros + savings - open-loans)
  ; reset almost everything for this turtle
    set bankrupt true
    set euros bankrupt-left-over
    set org-loans 0
    set open-loans 0
    set DiamondProduction (DiamondProduction - RawDiamonds)
    set RawDiamonds 0
    set Diamonds 0
    set Diamonds-value 0
    set Diamonds-sold (diamonds-per-partij * (length RawAt))  ;consider all diamonds as sold, existing diamonds are destroyed/lost
    set savings 0
    set loan-time plotpoints      ; to keep the time how long the turtle is bankrupt
  ]
end

; procedure checks if turtle is bankrupt, and does the necessities if so
; returns true if bankrupt
to-report bankrupcy
  ifelse (bankrupt = true) [
    if (plotpoints > (bankrupt-time + loan-time)) [      ; if bean bankrupt for x rounds, all is forgotten and forgiven
      set bankrupt false
      set loan-time 0
      report false
    ]
    report true
  ] [
    report false
  ]
end


; #############################################################3
; ###################### INVESTMENTS ########################
; #############################################################3
to invest
  without-interruption [
    let fromLoan (InvestmentThreshold * %BankFinancing / 100)                          ; this is what someone could get for a loan
    set fromLoan (test-loan fromLoan)                                                  ; but can I get it??
    if ((euros + savings) > (InvestmentThreshold + invest-eagerness - fromLoan)) [     ; only if I'm rich enough
    ; finances
      set fromLoan (get-loan fromLoan)                                                 ; get the loan
      if ((pay-from InvestmentThreshold) < 0) [stop]                                   ; pay it. Though not needed, function tests again
    ; update properties
      set RawDiamonds (RawDiamonds + diamonds-per-partij)                              ; get the diamonds
      set invested (invested + InvestmentThreshold)
      set rawat lput plotpoints rawat                                                  ; put the date (in plotpoints) in the list RawAt
      ifelse (length rawAt = length returns) [                                         ; corresponding item for returns already exists
        let lastitem ((length returns) - 1)                                            ; the index of the lastitem
        let return (item lastitem returns)                                             ; get that last item
        set returns replace-item lastitem returns (return - InvestmentThreshold)       ; distract the invested amount from it
      ] [                                                                              ; in this case a new returns item should be created
      set returns lput (0 - InvestmentThreshold) returns                               ; containing minus invested amount
      ]
    ; give the invested money to 25 random turtles with less then 35 euro's
      let spentlocal (InvestmentThreshold * (100 - %InvestOut) / 100)                  ; payment for mineworkers
      let remaining spentlocal                                                         ; in case there are not enough poors to spend it on
      repeat mineworkers-needed [
        let mineworker random-one-of turtles with [(euros + savings) <= mineworker-limit]
        if (mineworker != nobody) [
          set (euros-of mineworker) ((euros-of mineworker) + (spentlocal / mineworkers-needed))
          set (STrade-of mineworker) ((STrade-of mineworker) + (spentlocal / mineworkers-needed))
          set remaining (remaining - (spentlocal / mineworkers-needed))
        ]
      ]
    ; update globals
      set SeOut (SeOut + InvestmentThreshold + remaining - spentlocal)                 ; if everybody's too rich, they don't want to work
      set dSTrade (dSTrade + spentlocal - remaining)                                   ; in the mine. So then mineworkers are from outside
      set Sinvested (Sinvested + InvestmentThreshold)                                  ; the community, so that money flows out
      set diamondProduction (diamondProduction + diamonds-per-partij)                  ; adjust production counter
    ]
    update-bank
  ]
end

;################################################################
;############# PLOTTING #####################################
;##############################################################

to setup-plots
; here goes any initializing of plots, if needed
  update-energy-plot
  update-lorenz-and-gini-plots
end

to update-plots
  update-trade-plot
  update-wealth-distribution
  update-lorenz-and-gini-plots
  update-energy-plot
  update-investment-plot
  update-wealth-time-plot
  update-debts-time-plot
  set plotpoints plotpoints + 1                  ; keep the time
  set dStrade 0                                  ; reset global counters
  set dStradeDiamonds 0
end

to update-trade-plot
  set-current-plot "Trade / Time"
  set-current-plot-pen "total"
  plot dStrade
  set-current-plot-pen "luxury"
  plot dStradeDiamonds
  set-current-plot-pen "food"
  plot (dStrade - dStradeDiamonds)
end

to update-lorenz-and-gini-plots
  set-current-plot "Lorenz Curve"
  clear-plot
; draw a straight line from lower left to upper right
  set-current-plot-pen "equal"
  plot 0
  plot 100
  set-current-plot-pen "lorenz"
  set-plot-pen-interval 100 / count turtles
  plot 0
  let sorted-wealth sort values-from turtles [euros + diamonds-value + savings - open-loans]
  let total-wealth sum sorted-wealth
  if (total-wealth <= 0) [stop]
  let wealth-sum-so-far 0
  let index 0
  let gini-index-reserve 0
  let lorenz-y 0
  let minimum-lorenz 0              ; correction of gini for negative lorenz values
; now actually plot the Lorenz curve -- along the way, we also calculate the Gini index
  repeat count turtles [
    set wealth-sum-so-far (wealth-sum-so-far + item index sorted-wealth)
    set lorenz-y ((wealth-sum-so-far / total-wealth) * 100)
    plot lorenz-y
    if (minimum-lorenz > lorenz-y) [set minimum-lorenz lorenz-y]
    set index (index + 1)
    set gini-index-reserve
      gini-index-reserve +
      (index / count turtles) -
      (wealth-sum-so-far / total-wealth)
  ]
; correction for negative y axis
  let extra-area 0
  if (minimum-lorenz < 0) [set extra-area ((0 - minimum-lorenz) / 100)]
; plot Gini Index
  set-current-plot "Gini-Index / Time"
  plot ((gini-index-reserve / count turtles) / (area-of-equality-triangle + extra-area))
end

to-report area-of-equality-triangle
; not really necessary to compute this when num-people is large;
; if num-people is large, could just use estimate of 0.5
  report ((count turtles * (count turtles - 1) / 2) / (count turtles ^ 2))
end

to update-wealth-distribution
  set-current-plot "Wealth Distribution"
  clear-plot
  set-current-plot-pen "all"
  histogram-from turtles [euros + diamonds-value + savings - open-loans]
  set-current-plot-pen "euros"
  histogram-from turtles [euros + savings - open-loans]
end

to update-energy-plot
  set-current-plot "energy"
  plot mean values-from turtles [energy]
end

to update-investment-plot
  if (plotpoints mod invest-plot-freq != 0) [stop]             ; only plot once out of invest-plot-freq rounds (= 4)
  set-current-plot "Return of Investments"
  clear-plot
  if (plotpoints >= plot-x-max) [set-plot-x-range 0 ceiling (1.20 * plotpoints)]
  if (maxROI >= plot-y-max) [set-plot-y-range (0 - InvestmentThreshold) ceiling (1.10 * maxROI)]
  set-current-plot-pen "zero"
  set-plot-pen-interval ceiling (plotpoints * 1.25)
  plot 0
  plot 0
  set-current-plot-pen "ROI"
  set-plot-pen-interval 1
; get lists: each item in the list represents a turtle
  let listRawAt values-from turtles with [length rawat > 0] [rawat]                  ; list with lists with investment dates
  let listEarned values-from turtles with [length rawat > 0] [returns]               ; list with lists with investment returns
  let listLoans values-from turtles with [length rawat > 0] [open-loans]             ; list with values of open-loans
  let index 0                                                                        ; index for the lists of turtle properties
  foreach listRawAt [                                                                ; run thru main list with invest dates
    let listX ?                                  ; RawAt is a list of lists. ListX is one item, with invest dates of one turtle
    let indexinner 0                                                                 ; index for the lists inside the lists
    foreach listX [                                                                  ; run thru this list of dates of one turtle
      let x 0                                                                        ; x-value of the plot
      set x (plotpoints - (item indexinner listX))                                   ; x is time since doing the investment
      let y (0 - InvestmentThreshold)                                                ; starting value of y
      if (length listEarned > index) [                                               ; if there are revenues for this turtle
        let listY (item index listEarned)                                            ; put the list of revenues in ListY
        if (length listY > indexinner) [                                             ; and if this list has revenues for this investment
; y = income from selling - invested amount + original loan received - actual loan remaining yet - paid interest so far.
; invested amount, org loan received and paid interest are allready included in y, so only actual open loans remains
          let loan-open (item index listLoans)
          set y ((item indexinner listY) - loan-open)
        ]
      ]
      plotxy x y                                                                     ; finally, plot x and y
      set indexinner indexinner + 1                                                  ; and increase the inner index
    ]
    set index index + 1                                                              ; and increase the outer index
  ]
end

to update-wealth-time-plot
  set-current-plot "Wealth / Time"
  plot round (sum values-from turtles [euros + diamonds-value + savings - open-loans] / 1000 )
end

to update-debts-time-plot
  set-current-plot "debts / time"
  set-current-plot-pen "open"
  plot sum values-from turtles [open-loans]
  set-current-plot-pen "org"
  plot sum values-from turtles [org-loans]
  set-current-plot-pen "fund"
  plot bank-loan-available
end
@#$#@#$#@
GRAPHICS-WINDOW
383
12
679
329
20
20
6.98
1
10
1
1
1
0

CC-WINDOW
5
633
975
728
Command Center

BUTTON
791
12
854
45
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
888
13
959
46
NIL
go
T
1
T
OBSERVER
NIL
NIL

TEXTBOX
13
10
60
28
Numbers:

MONITOR
6
37
65
86
persons
count turtles
0
1

MONITOR
6
97
63
146
hungry
count turtles with [energy <= 10]
0
1

MONITOR
6
157
64
206
members
count turtles with [member = 1]
0
1

TEXTBOX
91
10
132
28
Totals:

MONITOR
88
36
145
85
Trade
round (sum values-from turtles [Strade] / 1000)
0
1

MONITOR
88
96
145
145
trans.
round (ntransactions / 1000)
0
1

MONITOR
87
157
143
206

sum values-from turtles [euros + savings]\n+ bank-profit
0
1

MONITOR
86
268
143
317
 max
max values-from turtles \n[euros + savings - open-loans]
0
1

TEXTBOX
245
10
303
40
Medians:

MONITOR
240
36
297
85
energy
median values-from turtles [energy]
1
1

MONITOR
240
97
297
146

median values-from turtles \n[euros + savings - open-loans]
1
1

PLOT
704
469
959
619
Trade / Time
time
Trade
0.0
10.0
0.0
10.0
true
true
PENS
"food" 1.0 0 -7566196 true
"luxury" 1.0 0 -16711681 true
"total" 1.0 0 -16776961 true

PLOT
161
338
321
458
Lorenz Curve
% People
% Wealth
0.0
100.0
0.0
100.0
true
false
PENS
"lorenz" 1.0 0 -16711936 true
"equal" 100.0 0 -16777216 true

PLOT
216
468
416
618
Gini-Index / Time
NIL
NIL
0.0
10.0
0.0
1.0
true
true
PENS
"default" 1.0 0 -16711738 false

PLOT
326
338
521
458
Wealth Distribution
poor <--------> rich
n (turtles)
0.0
300.0
0.0
40.0
false
true
PENS
"all" 10.0 1 -11352576 true
"euros" 10.0 0 -256 true

PLOT
552
337
712
457
energy
NIL
NIL
0.0
10.0
0.0
10.0
true
true
PENS
"default" 1.0 0 -6524078 false

MONITOR
165
36
225
85
invested
Round (SInvested / 1)
0
1

MONITOR
165
158
224
207
sold
sum values-from turtles [diamonds]
0
1

MONITOR
165
96
223
145
prod.
diamondProduction
0
1

MONITOR
166
214
223
263
price
(mean values-from turtles [diamonds-value])\n/ (mean values-from turtles [diamonds])
2
1

PLOT
8
467
208
617
Return of Investments
time since production
Earned
0.0
10.0
-200.0
0.0
false
true
PENS
"ROI" 1.0 2 -16776961 false
"zero" 1.0 0 -16777216 false

MONITOR
86
324
143
373
richest
max values-from turtles [euros + \nsavings + diamonds-value - open-loans]
0
1

SLIDER
791
71
963
104
InvestmentThreshold
InvestmentThreshold
200
500
400
10
1
NIL

TEXTBOX
167
10
220
28
Diamonds

SLIDER
793
114
965
147
%InvestOut
%InvestOut
0
100
0
5
1
NIL

MONITOR
87
211
143
260
 out
round SeOut
0
1

PLOT
722
338
882
458
Wealth / time
time
Wealth
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 0 -16776961 true

TEXTBOX
317
10
359
28
Banking:

MONITOR
311
36
365
85
Savings
sum values-from turtles [savings]
0
1

SLIDER
793
155
965
188
bankreserves
bankreserves
0
100
25
1
1
NIL

SLIDER
794
196
966
229
%bankFinancing
%bankFinancing
0
100
50
1
1
NIL

MONITOR
311
96
364
145
loanfund
bank-loan-available
0
1

PLOT
427
469
666
618
debts / time
time
debts
0.0
10.0
0.0
10.0
true
true
PENS
"open" 1.0 0 -65536 true
"org" 1.0 0 -44544 true
"fund" 1.0 0 -8716033 true

SLIDER
794
236
966
269
interest%
interest%
0
10
4.1
0.1
1
NIL

MONITOR
312
153
364
202
loans
sum values-from turtles [open-loans]
0
1

MONITOR
313
265
363
314
profit
bank-profit
0
1

MONITOR
7
211
65
260
bankrupt
count turtles with [bankrupt = true]
0
1

MONITOR
312
209
364
258
interest
sum values-from turtles [Sinterest]
0
1

MONITOR
256
264
308
313
shared
shared
0
1

SLIDER
794
278
966
311
profit-share-%
profit-share-%
0
20
10.0
0.2
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------

The game is a simulation game of a small economic community in a poor country. The persons walking around are turtles. As they meet each other, they do economic transactions. Basically, they buy food for euros. If they are getting richer, their interest is going more to buying luxury goods (symbolized by diamonds).



HOW IT WORKS
------------






HOW TO USE IT
-------------
Start the simulation by clicking setup. When clicking go, the turtles start moving and trading. The keep on doing so as long as the go-button remains 'pressed'. Turtles cannot die from hunger if they do not trade. However, they can come to a standstill if they go bankrupt.

The game allows you to monitor the economic success via a graph and several counters. Will sliders you can increase or decrease several parameters to see their effect on the economy or the distribution of wealth.



Short explanation on the basic elements:
----------------------------------

black square with thingies
This is the community, and here the turtles move around. Red turtles are members, greens are non-members, but for the moment this doesn't have any meaning. The more hungry they are, the paler they look. You can slow down the simulation by using the slider on top of the black area.

buttons: Setup
resets everthing to initial values, and releases 100 turtles in the wild.

buttons: Go
if clicked, the turtles start moving. When they meet another, they might trade. But not always, sometimes they just don't have enough money, don't trust each other, etc etc.

Monitors:
In the left of the screen are all the monitors. They monitor certain counters and variables. For details, see "Things to notice"

Sliders:
In the right of the screen, below the setup and go buttons, are the sliders. They allow you to manipulate certain parameters. For detailed description, see "To manipulate".

Plots:
At the bottom of the screen there are various graphs. For details, see "things to notice".



THINGS TO NOTICE
----------------

MONITORS

Numbers: Persons
This is the total number of turtles.

Numbers: Hungry
This is the total number of turtles being hungry. A turtle is hungry when his energy falls below 10. It has no further consequences for the turtle, except that turtles who are short on food, will of course not buy luxury, but only food. They will not invest either. The less energy the turtle has, the paler its color is.

Numbers: members
without any meaning yet

Numbers: bankrupt
A turtle who cannot pay back the debts he made, and who's debt is running out of hand, will be declared bankrupt. He looses everything but 10 euro's of pocket money, and cannot move or trade anymore. After several rounds (default 100) the bankrupcy expires, and the turtle is allowed to move and trade again.

Totals: trade
The sum of all traded euro's since the setup of the simulation. This monitor gives the number in thousands of euro's.

Totals: trans
The total number of transactions done, in thousands, since the setup of the simulation.

Totals: 
The total amount of euro's in the community. That is: in the pockets of turtles, on their bank savings accounts, and in the bank's profit.

Totals:  out
The total sum of euro's which left the community, because of payments to persons from outside. For the time being, the only way a euro can leave the community, is because it is paid to a foreign mineworker.

Totals:  max
The maximum amount of euro's a turtle has.

Totals: richest
the wealth of the richest turtle. Wealth is defined as the sum of euro's in pocket and in savings, minus the debts, plus the value of the luxury goods (diamonds) it bought. The value of stock (raw diamonds) is not counted, as they might be unsellable.

Diamonds: invested
Diamonds symbolize luxury goods. Before they can be sold, they must be produced. They are produced by turtles who do an investment. An investment is done by paying a certain amount of money, and receiving 10 raw diamonds for it. These diamonds can be sold to other turtles, with profit (or loss). This monitor shows the total amount of euro's which has been invested yet.

Diamonds: produced
This is the total amount of diamonds which have been produced as raw diamonds. Per investment, 10 raw diamonds are produced.

Diamonds: sold
Diamonds need to be sold as luxury goods, otherwise the investors will loose their investment. This is the total amount of diamonds which were sold to other turtles.

Diamonds: price
This is the average price for a diamond. Diamonds don't have fixed prices, as they are luxury goods, they are always wanted, as long as there is surpluss money available for it. The customer pays whatever he has available, but keeps a certain buffer for not running the risk of getting hungry. When the price falls below a certain limit, the producers will not sell anymore.

Medians: energy
the median energy of turtles

Medians: euro's
the median of the amount of euro's a turtle has. This is cash + savings - debts.

Banking: savings
A turtle which has more than a certain limit of euro's in cash, goes to the bank and deposits this in his savings account. This monitor is the sum of all savings accounts. The money in savings is available for loans.

Banking: loanfund
The loanfund is the amount of money available for loans. This is the money available in savings minus the bank reserve + the bank profit.

Banking: loans
The sum of all open loans (including interest).

Banking: interest
All interest paid up to now.

Banking: profit
The profit from the bank comes from interest payments. However, the profit is used to cover losses from bankrupcies, and a certain percentage can be shared amongst holders of savings accounts with positive balance.

Banking: shared
This is the summed amount of money which has been shared amongst holders of savings accounts. So it is the profit for savings account holders.

Graphs: Lorentz curve
This graph shows the (in)equity of the distribution of wealth. Both wealth and the percentage of people owning that wealth are ranked, and plotted against each other. So, for example, if the poorest 10% of the people all together own 5% of all the wealth, then this is represented in the graph by the point (10,5).
If there is a 100% equity amongst the people, and there is exactly 100 euro's in the population, to be spread amongst 100 persons, then this curve would then 1% would own 1%, 2% would own 2%, 10% would own 10%, etc etc. So this curve would follow the ideal black line, which goes from the lower left corner to the upper right corner.
As soon as there is some form of inequity, the curve falls below the black line. The bigger the area between the curve and the line, the more inequity there is. Total inequity, where one person would own all 100 euro's, would show a curve in the shape of a mirrored L (always following the x-axis, until x=100, and then jumping up to y=100).

Graphs: Wealth Distribution
The graph shows two histograms of the wealth distribution: the green bars are the total wealth (= cash + savings + value of bought diamonds - open debts); the yellow line is the total amount of euro's (= cash + savings - open debts).

Graphs: engery
This graph shows the median energy over time. Time graph x-axisses are always in "plotpoints", where one plotpoint is 100 turtle moves.

Graphs: Wealth / time
The development of wealth over time. Wealth is defined as cash + savings + value of bought diamonds - open debts; Time graph x-axisses are always in "plotpoints", where one plotpoint is 100 turtle moves.

Graphs: Return of Investments
This graph plots each investment done for its returns. As soon as an investment is done, it appears in the graph as a dot. The x-axis shows the time since the investment was done. The y-axis shows the overall balance for this investment. This is defined as the euro's earned by selling the diamonds which came from this investment, minus the initial amount invested, minus the paid interest over the loan for the investment. The black horizontal line in the plot is representing the zero level: as soon as a dot appears above this line, there is a profit.

Graphs: Gini-index / time
This graph shows the inequity developing over time. Used is the Gini index, which follows from the Lorenz curve. The Gini index is the area between the curve and the line in the Lorenz graph, divided by the total area below the black line of perfect equality (this total area below the black line is always 0.5). The extreme values for the Gini index are 0 in case of perfect equity (the curve and the line would be the same, so the area between them would be 0, so the Gini index would be 0), and 1 in case of complete inequity (the mirrored L, resulting in a Gini index of 0.5/0.5 = 1).
So the Gini index is a measurement for inequity, being 0 when there is maximum equity, and being 1 when there is maximum inequity.

Graphs: debts / time
This graph shows the development of debts over time. Three lines are plotted:
* red: the total of open debts (so including interest)
* orange: the original debt amounts without interest. When a debt is repaid, first the original amount is paid back.
* violet: the total loan fund available for loans

Graphs: Trade / time
this is the basic graph depicting economic development in our community. Don't let yourself be fooled by huge variations: that is the way economy is. Look at the overall trends. When there is economic growth, you will recognize it for sure in the graph, after some time.



TO MANIPULATE
-------------

Slider: investmentThreshold:
this shows the costs of investment. When doing an investment, a turtle will have to pay this amount. In practice, a turtle also wants to keep so pocket money for food, so this means that a turtle having an amount of euros above this number + 50, will start investing it in his business, thus spending most of his money, but improving his chances to earn more later on. A good value to start with would be 400.

Slider: %InvestOut:
When investing, the money is spend on producing raw diamonds from the mines. In fact, the money is paid to mineworkers. This slider determines how many of these mineworkers are coming from outside of our own community. If it is set on 40%, it means that 40% of the mineworkers are from outside; so 40% of the money is gone, and 60% flows back and is redistributed amongst turtles of the own community.

Slider: BankReserves:
This is the bank reserve in percentages. If a bank wants to offer loans, it normally uses money which has been deposited in savings accounts. However, this money is owned by its holders of savings accounts, and must be paid to them as soon as the holder wants to withdraw it. Still, the bank uses it, in the knowledge that not all of the account holders will want to withdraw their money at the same time. So money deposited in savings accounts is usually reallocated, by lending it out several times to other people. In this way, money is created by the bank.
But for protection of the account holders, the governments usually sets a percentage, which the bank is obliged to keep. If this slider is 25%, it means that the bank is allowed to use 75% of all the money in savings accounts for providing loans, and 25% must be kept in cash, in case the account holders come and get their money.
A good value for this slider might be 25%.

Slider: %BankFinancing:
In our community, the bank is only a savings and investment bank; it does not provide loans for consumption, only for investments. This slider determines which percentage of an investment can be financed via the bank. If this slider is on 30%, it means that the turtle doing an investment, must get 70% of the amount from its own money, and 30% of the money can be obtained via a loan. A good value for this slider might be 50%.

Slider: interest%:
This slider shows the percentage of interest over loans. It is the interest percentage per plotpoint (= 100 turtle moves).

Slider: profit-share-%:
This slider shows to what extend the profits of the bank are shared amongst holders of savings accounts with a positive balance. When profits are above a certain safety limit, they can be shared amongst holders of a positive balance. In such a case, x % of the profit is put apart each plotpoint (= 100 turtle moves), and distributed over the holders of savings accounts with positive balances, in such a way that everybody receives an equal percentage over his current balance.




THINGS TO TRY
-------------

You can run the model with the following settings:

1) Normal:
investmentThreshold = 400
%InvestedOut = 0
Bankreserves = 25
%BankFinancing = 50
Interest% = 2.0
profit-share-% = 10.0

This is the economy under normal circumstances. Run the model and see the following happening:
- over time, there is a quite a strong growth in wealth (curve wealth / time)
- gini index slightly drops, so as everybody is getting more rich, there is less inequality
- investments are done, and loans over them are paid back (curve debts / time)
- investments have positive returns (curve Return of Investments)


2) No investment:
investmentThreshold = 500
%InvestedOut = 0
Bankreserves = 25
%BankFinancing = 0
Interest% = 2.0
profit-share-% = 10.0

By putting the investmentThreshold on maximum, and the %BankFinancing on 0, we can be sure that there will be no investments. Note that the model is in a steady state: there is no economic growth, nor economic shrinking.
- the amount of euro's in the economy does not change (monitor Totals: )
- The distribution of wealth almost immediately stabilizes around a value just above 0.5 (curve gini index).


3) The effect of bank reserves and investments
After running the model for a while with the settings of item 2), do the following:
- Try and move the bank reserve slider to 100%: the loanfund drops immediately to 0.
- Move back the bank reserve slider to 25%, and move the %BankFinancing slider to 50%. Soon there will be open loans in the simulation. As soon as there will be an open loan, the total amount of euro's in the simulation will vary, and wealth will increase. The more open loans, the more euro's available, and the more wealth developing.
- This can be sped up by decreasing the bank reserves. You may also notice a slight economic growth in the growth / time curve, as a result of this.


4) The effect of outflowing money
Continue with the same situation, but now move the slider "%InvestOut" to the maximum. You can also try this by starting anew with the settings of 1), but %InvestOut on 100. The result of this, is that all of the invested money is flowing out of the community. The effects of this are dramatic.
- total sum of euro's decreasing (monitor: totals: )
- gini goes up again, so more inequality
- Return of Investments drops, leading to bankrupcies
- wealth curves tops of, and drops
- economic growth drops
- energy drops, hunger rises.
- investments stop; no more investments. If you try this from the start with a fresh setup, you will see a wave of first investments, which soon stops, and no investments will be done anymore.
This crisis is because all of the money invested is flowing out of the community. The investors let the money flow out, but are only focusing on the internal market, which is in stagnation because of a shortage of money.

5) the effect of too high interest rates
investmentThreshold = 400
%InvestedOut = 0
Bankreserves = 25
%BankFinancing = 50
Interest% = 10.0
profit-share-% = 10.0

See for yourself: from a debt crisis with several bankrupcies, a real economic crisis results. The most dramatic effects can be found when running the model for some time with normal settings (1), and then increasing the interest rate to extreme values.
Try and find out what is the maximum interest percentages which seems to result in a still stable economy. Which other sliders are influencing this?



EXTENDING THE MODEL
-------------------
* market:
in the determination of prices, there is no market. The model could be greatly improved if the prices would be determined by supply and demand. Also costs of interest should be calculated into the prices.

* sensible decision making and experiences:
at this moment, the turtles are not learning anything from their own and other's experiences. For example: a turtle just tries and gets a loan, even if the interest rates are outrageous, and everyone else getting a loan has gone bankrupt.

* more realistic money flows inside and outside the community
Yet, there is only one possibility to let money flow to outside: paying mineworkers from outside. This is of course not at all realistic: as wealth rises (or drops), turtles might have more reasons to go and buy goods from outside, or even to start investing outside (outflowing money).
At the moment, there can never be any money flowing in. This is even more unrealistic: turtles might find a job outside, or might sell their products to the outside world.

* introduction of an alternative money system


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.


@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7566196 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7566196 true true 150 285 285 225 285 75 150 135
Polygon -7566196 true true 150 135 15 75 150 15 285 75
Polygon -7566196 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7566196 true true 96 182 108
Circle -7566196 true true 110 127 80
Circle -7566196 true true 110 75 80
Line -7566196 true 150 100 80 30
Line -7566196 true 150 100 220 30

butterfly
true
0
Polygon -7566196 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7566196 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7566196 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7566196 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7566196 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7566196 true true 47 195 58
Circle -7566196 true true 195 195 58

circle
false
0
Circle -7566196 true true 30 30 240

circle 2
false
0
Circle -7566196 true true 16 16 270
Circle -16777216 true false 46 46 210

cow
false
0
Polygon -7566196 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7566196 true true 73 210 86 251 62 249 48 208
Polygon -7566196 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7566196 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7566196 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7566196 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7566196 true true 60 15 75 300
Polygon -7566196 true true 90 150 270 90 90 30
Line -7566196 true 75 135 90 135
Line -7566196 true 75 45 90 45

flower
false
0
Polygon -11352576 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7566196 true true 85 132 38
Circle -7566196 true true 130 147 38
Circle -7566196 true true 192 85 38
Circle -7566196 true true 85 40 38
Circle -7566196 true true 177 40 38
Circle -7566196 true true 177 132 38
Circle -7566196 true true 70 85 38
Circle -7566196 true true 130 25 38
Circle -7566196 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -11352576 true false 189 233 219 188 249 173 279 188 234 218
Polygon -11352576 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7566196 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7566196 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7566196 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7566196 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7566196 true 150 0 150 300

pentagon
false
0
Polygon -7566196 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7566196 true true 110 5 80
Polygon -7566196 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7566196 true true 127 79 172 94
Polygon -7566196 true true 195 90 240 150 225 180 165 105
Polygon -7566196 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7566196 true true 135 90 165 300
Polygon -7566196 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7566196 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7566196 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7566196 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7566196 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7566196 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7566196 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7566196 true true 30 30 270 270

square 2
false
0
Rectangle -7566196 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7566196 true true 60 270 150 0 240 270 15 105 285 105
Polygon -7566196 true true 75 120 105 210 195 210 225 120 150 75

target
false
0
Circle -7566196 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7566196 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7566196 true true 120 120 60

tree
false
0
Circle -7566196 true true 118 3 94
Rectangle -6524078 true false 120 195 180 300
Circle -7566196 true true 65 21 108
Circle -7566196 true true 116 41 127
Circle -7566196 true true 45 90 120
Circle -7566196 true true 104 74 152

triangle
false
0
Polygon -7566196 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7566196 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7566196 true true 4 45 195 187
Polygon -7566196 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7566196 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7566196 false true 24 174 42
Circle -7566196 false true 144 174 42
Circle -7566196 false true 234 174 42

turtle
true
0
Polygon -11352576 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -11352576 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -11352576 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -11352576 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -11352576 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7566196 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7566196 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7566196 true 150 285 150 15
Line -7566196 true 15 150 285 150
Circle -7566196 true true 120 120 60
Line -7566196 true 216 40 79 269
Line -7566196 true 40 84 269 221
Line -7566196 true 40 216 269 79
Line -7566196 true 84 40 221 269

x
false
0
Polygon -7566196 true true 270 75 225 30 30 225 75 270
Polygon -7566196 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 2.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
