globals [biomass-unit days dollars grazecountdown]
breeds [cow]
patches-own [biomass]
turtles-own [energy] ; as Mcal

to setup
    ca
    set biomass-unit 2.5 ; each increment of biomass (0-10) is valued at this many lbs. dry weight
    ifelse start-patches-even?
        [ask patches [set biomass 4]]
        [ask patches [set biomass (random-float 4.0) * 2]]
    if update-patches?
    [ask patches [update-patches]]
    setup-plot
    setup-histogram
end

to go
   ask patches [grow-grass-and-weeds
               cycle-plants]
   if update-patches?
   [ask patches [update-patches]]
   plot-biomass
   histo-patches
   set days days + 1
   set grazecountdown grazecountdown - 1
   wait 0.5
   if any? turtles
       [ ask cow [graze] ]
end

to cycle-plants ;; patch procedure representing mortality and new establishment
      if biomass <= .3 [
      if ( random 100 ) < 5
        [ set biomass 0 ]
                       ]

      if biomass = 0 [
      if ( random 100 ) < 3
        [ set biomass 1 ]
                     ]
end

to update-patches ;; patch procedure

    if biomass <= 2 [set pcolor 68]
    if biomass > 2 [set pcolor 66]
    if biomass > 6 [set pcolor 64]
    if biomass > 8 [set pcolor 44]
    if biomass <= 0 [set pcolor brown]
end

to cattle-in
    set-default-shape cow "cow"
    create-custom-cow num-cows  ;; create the cows, then initialize their variables
          [
    set color black
    set energy 0
    setxy (random screen-size-x) (random screen-size-y) ; spreads them around
    rt random 360
          ]
    set grazecountdown grazeperiod
end

to cattle-out
    ask turtles [die]
    set grazecountdown 0
end

;;;;; GRAZING PROCEDURES

to graze  ;; daily cow procedure
   if grazecountdown <= 0 [die]
   repeat (.4 / bitesize) [move bite]
   check-fill
   gain
end

to move
    rt random 50 - random 50 fd 1
    repeat selectivity
        [
    if biomass < 1 or biomass > 8 [fd 3]
        ]
end

to bite
       ifelse biomass < 1 or biomass > 8
       [  set energy energy + (biomass * biomass-unit * bitesize * .4)] ; less energy: .4 Mcal per lb.
       [  set energy energy + (biomass * biomass-unit * bitesize * .7)] ; more energy: .7 Mcal per lb.
       set biomass (biomass * (1 - bitesize)) ; patch loses biomass in proportion to bitesize
end

to check-fill
    repeat (1.6 / bitesize) [if energy < 20 [move bite]] ; attempt to harvest 20 Mcal
end

to gain
    set dollars dollars + (((energy - 5) / 7) * .4)
    ;assumes 5 Mcal for maintenance
    ;assumes 7 Mcal needed for pound of $.40 gain
    set energy 0
end

;;;;; PLANT GROWTH

to grow-grass-and-weeds ;; patch procedure
    if biomass < 7.5 ;; stage at which growth slows
    [set biomass biomass + (((precision (biomass ^ .667) 2) / 40) * grassrate)]
    ; gives about 2.5% daily growth with grassrate 1.0
    if biomass >= 7.5
    [set biomass biomass + (((precision ((10 - biomass) ^ .667) 2) / 40) * grassrate)]
end

;;;; PLOTTING PROCEDURES

to setup-plot
    set-current-plot "ecology and economy"
    set-current-plot-pen "avg-biomass"
    set-plot-y-range 0 10
    set-current-plot-pen "dollars per week"
    set-plot-y-range 0 10
end

to setup-histogram
    set-current-plot "patch biomass histogram"
    ;set-plot-x-range 0 10
    set-plot-y-range 0 100
    set-histogram-num-bars 11
end

to plot-biomass
    set-current-plot "ecology and economy"
    set-current-plot-pen "avg-biomass"
    plot mean values-from patches [biomass]
    set-current-plot-pen "dollars per week"
    plot dollars / (days + .001) * 7
end

to histo-patches
    set-current-plot "patch biomass histogram"
    histogram-from patches [ biomass ]  ; using the default plot pen
end

; ***NetLogo Model Copyright Notice***

; Copyright 2003 by Peter Donovan. All rights reserved.

; Permission to use, modify, or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Peter Donovan.
; Contact Peter Donovan (peter@managingwholes.com) for appropriate licenses
; for redistribution for profit.

; To refer to this model in academic publications, please use:
; Donovan, Peter. (2003).  NetLogo Grazing Model # 1.
; http://managingwholes.com/models/graze1.htm.
@#$#@#$#@
GRAPHICS-WINDOW
324
10
674
381
8
8
20.0
1
10
1
1
1

CC-WINDOW
324
570
664
700
Command Center

BUTTON
20
33
85
66
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
89
34
149
67
run
go
T
1
T
OBSERVER
T

MONITOR
159
35
216
84
days
days
0
1

SLIDER
20
176
154
209
num-cows
num-cows
0
100
10
1
1
NIL

BUTTON
20
70
129
103
advance 1 day
go
NIL
1
T
OBSERVER
T

BUTTON
19
212
122
259
cattle in
cattle-in
NIL
1
T
OBSERVER
T

PLOT
19
416
664
566
ecology and economy
days
NIL
0.0
365.0
0.0
10.0
true
true
PENS
"avg-biomass" 1.0 0 -8716033 true
"dollars per week" 1.0 0 -65536 true

MONITOR
225
35
310
84
dollars
dollars
2
1

PLOT
19
270
311
407
patch biomass histogram
<= seedlings      dormant =>
# patches
0.0
10.0
0.0
100.0
true
false
PENS
"default" 1.0 1 -16777216 true

SLIDER
19
106
153
139
grassrate
grassrate
0
3.0
1.0
0.1
1
NIL

SLIDER
159
176
311
209
grazeperiod
grazeperiod
0
100
10
1
1
NIL

MONITOR
237
211
311
260
recovery
0 - grazecountdown
0
1

BUTTON
126
212
233
259
cattle out
cattle-out
NIL
1
T
OBSERVER
T

MONITOR
225
86
310
135
$ per day
dollars / days
2
1

SWITCH
324
382
467
415
update-patches?
update-patches?
0
1
-1000

SWITCH
473
382
664
415
start-patches-even?
start-patches-even?
1
1
-1000

SLIDER
159
140
311
173
selectivity
selectivity
0
4
1
1
1
NIL

SLIDER
19
141
154
174
bitesize
bitesize
0.1
0.4
0.3
0.1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------

This is a simple pasture management or grazing model. There is NO predictive value to this model. Its purpose is to help you experiment and see what happens when relatively simple behaviors interact in complex ways over time. Suggestions for improvements would be appreciated; peter@managingwholes.com. In the future ManagingWholes.com hopes to have grazing models that mimic both grass and weeds, the herding behavior of livestock, the effects of cross-fencing, and multiple species of grazing animals (e.g. cows and goats).


HOW TO USE IT
-------------

Click the SETUP button to start. This gives you a small pasture to manage, with plants in a variety of vegetative growth stages. The only livestock available are cattle, and a generalized grass plant grows on the patches. In this simplified ecosystem, water and sunlight are evenly available from day to day. To simulate drought, rainfall, or seasonal variations, you may adjust the GRASSRATE slider at any time. Click RUN to start the simulation. The RUN button turns black when the simulation is running; click again to pause the simulation. To start over, stop RUN and click SETUP.

Select the stocking rate and duration with the NUM-COWS and GRAZEPERIOD sliders. Slider handles can be dragged, or click to right or left of the handle to fine-tune. CATTLE IN will start grazing. Should you wish to cease grazing before your grazeperiod runs out, hit CATTLE OUT. (Sometimes CATTLE IN doesn't "catch" so keep an eye on the monitors in case you need to hit it again.)

The UPDATE-PATCHES? switch allows you to turn off the flickering color updates of the patches in the pasture. The START-PATCHES-EVEN? switch allows you to experiment with patches at an even starting biomass of 4. When this switch is off, starting biomass is a random distribution between 0 and 7.9.


THINGS TO TRY
-------------

1. See if you can make more than $0.90 a day from this pasture using a GRASSRATE of 1.0.

2. Can you reproduce the pattern often seen in continuously grazed pastures of a mosaic of wolfy, dormant, or overrested plants plus short regrowth? In the histogram this would be a U-shape, representing poor energy flow. How do you turn this around into a hump or normal curve? Try changing BITESIZE and SELECTIVITY as well as NUM-COWS and GRAZEPERIOD.


HOW IT WORKS
------------

PLANT GROWTH. Plant growth is divided into four color-coded stages: early, vegetative, going to seed (progressively darker shades of green), and dormant (tan). Each day, plants grow and increase their biomass, which is recorded on a scale of 0 to 10. (In this simplified model, each increment represents 2.5 pounds of dry matter, so that a patch can grow up to 25 dry-weight pounds of biomass, roots included).

Biomass is a measure of volume. Plants in early and vegetative states increase daily roughly in proportion to their area (biomass to the two-thirds power). In the later stages of growth (beyond biomass 7.5 in this model), they increase daily according to (10 - biomass)^.667. Thus growth slows markedly from biomass 7.5 to 10, to represent the combined effects of plant maturity, competition, and shading. At random, some patches with low biomass die out and turn dark brown, indicating bare ground, simulating the effect of overgrazing plants. At random, seedlings establish in some of the bare patches, returning them to productivity.

CATTLE GRAZING. In building this model, it was assumed that each animal needs 5.0 megacalories (Mcal) daily for maintenance, and that 7.0 Mcal above the maintenance requirement was sufficient to put on a pound of gain (worth $0.40 in this model). The forage harvested by the cow has energy values of .4 or .7 Mcal per pound of dry matter. Patches with biomass between 2 and 8 yield the higher energy value. Each time a patch is grazed, its biomass is reduced by 10 to 40 percent, depending on the BITESIZE slider. If gain turns negative, it will cost you.

MOVING AND SELECTIVITY. With BITESIZE set at .4 (40% removal of biomass with grazing) and SELECTIVITY set to 1, cows move a minimum of 1 times and a maximum of 4 times each day to a different patch on the model, which is a lattice of 289 patches. Each move starts out with the cow moving randomly forward 1 square. If SELECTIVITY is 1, she evaluates the patch she lands on. If the biomass is between 1 and 8.5 (vegetative plants), she grazes. Otherwise she moves forward again, and grazes the patch she lands on regardless. If after 1 such moves she has not harvested 20 Mcal worth of forage, she makes up to 3 additional moves in an attempt to do so. The SELECTIVITY slider controls how many moves she will make in an attempt to find a vegetative patch to graze. Lowering BITESIZE or increasing SELECTIVITY will increase the daily movement.

PLOTS. The red pen plots dollars per week; the violet pen plots average biomass. Should you make a lot of money in a hurry, or lose lots of money, the vertical scale may increase to the point where average biomass becomes difficult to gauge. The histogram shows the distribution of patches with a given biomass or stage of growth.


WHAT THIS MODEL DOES NOT REPRESENT
----------------------------------

1. The ability of pastures to improve, grow more forage with good management, the effects of animal impact on pasture productivity, or any of the myriad factors relating to soil health.

2. Seasonal variations in the patterns of plant growth (some of these could be simulated with the GRASSRATE slider, but not photoperiod effects). In effect, patches on the lattice grow biomass independently of any season or photoperiod, nutrient availability, or energy storage.

3. Subleties of grazing behavior, for example varying "bites" on plants of different growth stages, or the variation of bite depending on numbers of animals.

4. Varying maintenance requirements of cattle owing to reproductive cycles, cold temperatures, stress, travel needed to fill rumen, or weight gain or loss.

5. Natural biodiversity within species and among species. This is a monoculture of generic cattle and generic plants, both of which "convert energy" in linear fashion. The irregularities that emerge result from the accumulation of random, probabilistic interactions between linear automata.


CREDITS AND REFERENCES
----------------------
Diven, Dick. (2001). Low Cost Cow-Calf Livestock Nutrition manual.

Savory, Allan, with Jody Butterfield. (1999). Holistic Management: A New Framework for Decision Making. Washington, DC: Island Press.

The NetLogo manual. See http://ccl.northwestern.edu/netlogo/

To refer to this model, please use: Donovan, Peter. (2003).  NetLogo Grazing Model # 1. http://managingwholes.com/models/graze1.htm.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

cow
false
0
Rectangle -7566196 true true 58 119 285 224
Polygon -7566196 true true 283 225 262 292 220 292 244 259 210 219
Polygon -7566196 true true 59 219 59 219 54 288 85 287 105 218
Polygon -7566196 true true 59 121 27 205 6 203 6 268 40 274 63 225

cow-shape
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

grass-shape
false
0
Polygon -7566196 true true 66 262 188 17 129 260
Polygon -7566196 true true 12 116 90 254 115 254

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
