globals [
    steps         ; Number of steps the model has made
    edge_list     ; list of all edges
    tol           ; tolerance measure
    inertia       ; decimal version
    terminate     ; the main control repetition terminate flag
    polterminate  ; the policy action subroutine termination flag
    short_l       ; the average of L for all vertices
    clust_coeff   ; netowrks av. clustering coeff
    polsteps      ; no of iterations to policy agreement
    exit_flag
    pathl_min_list ; pathl at min p val
    pathl_p1_list ; path length at p=1
    pathllist ; the list for output of path lengths
    cc_min_list ; cc at min p val
    cc_p1_list ; clust coeff at p=1
    cclist ; the list for output of clustering coeffs
    polsteps_p1_list ; number of its to pol agree at p=1
    polsteps_min_list ; polsteps at min p val
    polstepslist ; the list for output of the number of iterations in policy sim
    ploglist ; list for output of the plog values
    nactorlist ; list for output of the number of actors
    num_actors
    p_rand_log
    shortcuts
    randval
    repcounter ; repetitions counter
    ]
breeds [actors policies envideas] ; this is mostly importqnt for policy simulation
actors-own [
    edges        ; List of edges belonging to actor
    inArcs       ; List of IDs of nodes pointing to me
    outArcs      ; List of IDs of nodes I point to
    policylist   ; list of policies hatched by the turtle
    temp         ; a temp variable
    policy       ; the agreement with policy
    oldpolicy    ; stored version of policy for that round
    mat          ; the full matrix
    short        ; shortest path lengths
    clustc       ; cluser coefficient (Watts Strogatz)

]
policies-own [
   p-target       ; the target actor of the policy
   p-source       ; the source actor of the policy
             ]

;************
; highest level control
;************

to setup
clear-all
set repcounter 1
set pathllist []
set cclist []
set polstepslist []
set pathl_p1_list []
set cc_p1_list []
set polsteps_p1_list []
set pathl_min_list []
set cc_min_list []
set polsteps_min_list []
set ploglist []
set nactorlist []
end

to step            ; a step is one repetition of the plog-loop within actor-loop i.e. 'one_run'
init_one_run
while [terminate = 0]
[one_run]
set repcounter repcounter + 1
end

;************
; the control of one_run, the randomisation values p-loop, within number of actors loop
;************

to init_one_run
set terminate 0
set num_actors num_actors_start - 5
set p_rand_log p_rand_log_end
end

to one_run
ifelse (p_rand_log = p_rand_log_end)               ; if last p val for this num_actors
[set p_rand_log p_rand_log_start                   ; then reset the p val
 set num_actors num_actors + 5]                    ; and increase the actors by 5
[set p_rand_log precision (p_rand_log + 0.2) 1]    ; else increase the p_rand_log by 0.2

if (p_rand_log = p_rand_log_start)                 ; if this is first p value for this num_actor value
  [p1step]                                         ; call the procedure to do the p=1 run early

ifelse (p_rand_log = 0)                            ; if p_rand_log is 0 (i.e. p=1)
[if pathlen? [set short_l last pathl_p1_list]      ; put in the values we calculated in the special p1 run at start
 if clustcoeff? [set clust_coeff last cc_p1_list]
 if pol_action? [set polsteps last polsteps_p1_list]
]
[setnet                                            ; else set up the network
 setactors                                         ; initialise the actors
 calcs                                             ; and run the calulations
]
set nactorlist lput num_actors nactorlist          ; record the actor number
set ploglist lput p_rand_log ploglist              ; and the p value

if (p_rand_log = p_rand_log_start)                 ; if this is first p value for this num_actor value
  [first_p_net]                                    ; run the procedures that calculate/record p0 (min) values

if clustcoeff? [set cclist lput clust_coeff cclist]            ; record the outputs of the calcs
if pathlen? [set pathllist lput short_l pathllist]
if pol_action? [set polstepslist lput polsteps polstepslist]

doplots                                            ; do the plots

if ((num_actors >= num_actors_end) and (p_rand_log >= p_rand_log_end))
[set terminate 1]                                  ; set the terminate flag if on both last num_actors value and last _ value
end

;************
; the heart of the program called within the inner p-values loop
;************

; this is done for the first p value each time within actor loop
to first_p_net
if pathlen? [set pathl_min_list lput (short_l) pathl_min_list]   ;sets the supposed 'minimums' for the plotting
if clustcoeff? [set cc_min_list lput (clust_coeff) cc_min_list]  ;post-run analysis should of course use the correct figures
if pol_action? [set polsteps_min_list lput (polsteps) polsteps_min_list]
end

; this is the procedure done for each plog value in the p-loop
to p1step
locals [stored_p_val]
set stored_p_val p_rand_log     ; this is a bit of necessary housekeeping
set p_rand_log 0
setnet      ; create the network according to the randomisation, p, value
calcs
if pathlen? [set pathl_p1_list lput (short_l) pathl_p1_list]      ;put the required outputs into their lists
if clustcoeff? [set cc_p1_list lput (clust_coeff) cc_p1_list]
if pol_action? [set polsteps_p1_list lput (polsteps) polsteps_p1_list]
set p_rand_log stored_p_val
end

to setnet        ; creates the network in an agent based way
locals [myid]
clear-turtles
clear-patches
set-default-shape actors "circle"
set steps 0
set tol 1
set terminate 0
create-custom-actors (num_actors) [
   set ycor 0
   set xcor 0
   set size 5
   set myid who
   fd ((0.9 * screen-edge-x) - ((myid mod 2) * 20))  ; set up the actors in a concentric circle pattern - nice
   set color green
   set edges []   ; give the actors their empty lists
   set edge_list []
   set policylist []
   ]
ask actors [without-interruption[ makeregularnet    ; get the actors to create their regular network
            ] ]
randomnet    ; then randomise to the extent necessary
if drawnet? [drawlinks]
ask actors
[
ifelse drawLabels?
 [set label 0 + " "]
 [set label no-label]
 ]
end

;************
; the calculation control area
;************

to calcs     ; subroutine that runs the calculations
create_mat   ; creates a matrix
if pathlen? [shortest_paths]   ;call the actual calculation subroutines
if clustcoeff? [cluster_coeff]
if pol_action? [
setactors   ; initiate the actors for the policy simulation
pol_action  ; do the policy simulation
]
set exit_flag 1
end

; creates the matrix to do some of the calcs on
; n.b. it would be preferable to do them all by actor based methods, but hey
to create_mat
locals [i myid]
ask actors
 [without-interruption
  [set myid who
   set i 0
   set mat []
   set short []
   repeat num_actors
    [
     ifelse (member? i edges)
      [set mat lput 1 mat
       set short lput 1 short]
      [set mat lput 0 mat
       set short lput 99999 short]
     set i i + 1
    ]
   set short replace-item myid short 0
   ]
   ]
end

;************
; the agent based policy simulation bits
;************

; for the policy simulation give actors their initial conditions
to setactors
locals [n]
set n 0
while [(count actors with [policy = 1]) < (start_policy_% * num_actors / 100)]
[
ask turtle n
 [update_policy (1)]
set n n + 1
]
color_actors  ; distinguish them graphically
end

;the actual meat of the policy simulation
to pol_action
locals [myid theirsupp]
set polterminate 0       ; the finish condition flag
set polsteps 0           ; the iteration counter
while [polterminate = 0]
[ask actors [set oldpolicy policy]   ; store the old value as we want to test convergence later

 ask actors [without-interruption    ; get all the actors to do their thing
  [set myid who

;**** the interaction rule *********
   set theirsupp ((sum values-from (turtles with [member? myid edges]) [policy]) / (length edges) )
   update_policy (((1 - inertia) * theirsupp) + (inertia * oldpolicy))

;***********************************
   ]]
color_actors        ; update the actors 'opinions' based on some rules
check_convergence
set polsteps polsteps + 1 ; update the iteration counter
dopolplots                ; plot something to let us know the simulation is alive
]
end

; convergence happens either when all actors stop moving or when they all think the same thing (all green or all red)
to check_convergence
set tol max values-from actors [policy] - min values-from actors [policy]
if ((tol < 0.01) or (count actors with [color = green] = 0) or (count actors with [color = red] = 0))
 [set polterminate 1] ; set the terminate flag if we have convergence
end

; the colour of actors shows what they think
; this allows some nice tuning possibilities
to color_actors
ask actors
 [
   ifelse (policy > 0.5)
     [set color red]
     [set color green]
  ]
end

;
to update_policy [newval]
 set policy newval
 ifelse (drawlabels?)
 [ set label " " + precision newval 3]
 [ set label no-label]
end

;********
; small worlds tests subroutines
;********

to shortest_paths
; ************************
; floyd warshall algorithm
; ************************
locals [k i j]
ask actors [set temp short]
set k 0
repeat num_actors
[
 ask actors [without-interruption[
   set i who
   set j 0
   repeat num_actors
    [
    set temp replace-item j temp (min list (item j short) ( (item k short) + (value-from turtle k [item j short]) ) )
    set j j + 1
    ]
   ]]
 set k k + 1
 ask actors [set short temp]
]
set short_l 0       ; short_l (that is an 'el', not a 'one', is the global shortest path value
ask actors [without-interruption
 [set short_l (short_l + (sum short))]]
set short_l (short_l / (num_actors * num_actors))
end

; ************************
; calculation of clustering coefficient as described by Watts and Strogatz
; ************************
to cluster_coeff
locals [tot_links neighbours myedges  i j]
ask actors [without-interruption[
  set neighbours length edges
  set myedges edges
  set tot_links 0
  set i 0
  repeat neighbours
   [
    set j i + 1
    repeat neighbours - (i + 1)
     [ask turtle (item i myedges)
      [if (member? (item j myedges) edges)
       [ set tot_links tot_links + 1]
      ]
      set j j + 1
     ]
    set i i + 1
   ]
  ifelse (tot_links > 0)
  [set clustc (tot_links / (neighbours * (neighbours - 1) / 2))]
  [set clustc 0]
 ]]
set clust_coeff ((sum values-from actors [clustc]) / num_actors)
end

;*********
; section that deals with getting the actors to create the networks
;*********

; a regular network where each actor is neighbours with actors on either side
to makeregularnet
locals [myid relative destination]
set myid who
set relative 0 - ( range )
repeat (range * 2) + 1 [
  set destination (myid + relative + num_actors) mod (num_actors)
  if (destination != myid) [addEdge (destination)]
  set relative relative + 1
  ]
end

; rewiring the edges of a regular network according to a probablity factor p
; again see Watts and Strogatz for the maths
to randomnet
locals [fromid toid n new_toid shortlist]
set n 0
set shortcuts 0
set shortlist sort-by [random-int-or-float (length edge_list) < random-int-or-float (length edge_list)] edge_list
repeat int((length edge_list) * (10 ^ p_rand_log))
 [
   set shortcuts shortcuts + 1
   set fromid item 0 (item n shortlist)
   set toid item 1 (item n shortlist)
   ask turtle fromid
    [without-interruption[
    set new_toid (random-int-or-float num_actors)
    while [(new_toid = fromid) or (member? new_toid edges)]
     [set new_toid (random-int-or-float num_actors)]
    ]]
   replace_edge (list (list fromid toid) (list fromid new_toid))
   set n (n + 1)
 ]
end

;used to get actors to manipulate their network variables consistently
; according to a list 'params' that is self explanatory from the code below
; this code was stolen from someone, but I cannot remember who - let me know if it was you and I will reference you

to replace_edge [params]
locals [fromid new_fromid toid new_toid]
set fromid item 0 (item 0 params)
set toid item 1 (item 0 params)
set new_fromid item 0 (item 1 params)
set new_toid item 1 (item 1 params)
ask turtle fromid
    [
     set edges remove toid edges
    ]
ask turtle new_fromid
    [
    set edges lput new_toid edges
    ]
ask turtle toid
    [
    set edges remove fromid edges
    ]
ask turtle new_toid
    [
    set edges lput new_fromid edges
    ]
set edge_list (replace-item (position (list fromid toid) edge_list) edge_list (list new_fromid new_toid))
end

to addEdge [toactor]
locals [myID theirID them pair]
set myid who
set them turtle toactor
if (them != nobody)
  [
   ask them
     [
      set theirID who
      if (not member? myID edges)
         [
          set edges fput myID edges
          set pair list myID theirID
          set edge_list fput pair edge_list
         ]
      ]
    if (not member? theirID edges)
      [set edges fput theirID edges]
   ]
end

;****************
; graphical representation subroutines
;****************

to drawlinks
locals [dest_actor myid]
ask actors [without-interruption[
             set temp 0
             repeat (length edges)
              [
               set dest_actor (item temp edges)
               hatch_small_blue_policy (dest_actor)
               set temp (temp + 1)
              ]
           ]]
; this makes it all happen so quickly!
ask policies [without-interruption[move_to_dest_draw]]
ask policies [without-interruption[die]]
end

to hatch_small_blue_policy [destination]
locals [myid]
set myid who
hatch 1 [
         set breed policies
         set color blue
         set label ""
         set p-target destination
         set p-source myid
         set heading (towards-nowrap turtle p-target)]
end

to move_to_dest_draw
pd
while [(distance turtle p-target) > 5]
[fd 3]
pu
end

to move_policies
ifelse ((distance turtle p-target) > (distance turtle p-source))
[fd (0.5 * ( 1 / link_length) * (distance turtle p-target)) + 1]
[fd (distance turtle p-target)]
end

to doPlots
if clustcoeff? [set-current-plot "Nodes"
    set-current-plot-pen "clustcoeff"
    set-plot-pen-mode 2
        plotxy p_rand_log ( ((last cclist) - (last cc_p1_list)) / ((last cc_min_list) - (last cc_p1_list)) )]
if pathlen?   [     set-current-plot "Nodes"
    set-current-plot-pen "pathlen"
      set-plot-pen-mode 2
        plotxy p_rand_log ( ((last pathllist) - (last pathl_p1_list)) / ((last pathl_min_list) - (last pathl_p1_list)) )]
if pol_action?  [  set-current-plot "Nodes"
    set-current-plot-pen "polsteps"
    set-plot-pen-mode 2
        plotxy p_rand_log ( ((last polstepslist) - (last polsteps_p1_list)) / ((last polsteps_min_list) - (last polsteps_p1_list)) )]
end


to dopolplots
set-current-plot "poldev"

            plotxy polsteps 100 * (count actors with [color = red]) / num_actors
end
@#$#@#$#@
GRAPHICS-WINDOW
490
10
801
342
150
150
1.0
1
10
1
1
1

CC-WINDOW
151
10
479
295
Command Center

BUTTON
4
10
85
43
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
4
45
85
78
NIL
step
NIL
1
T
OBSERVER
T

BUTTON
4
79
85
112
go
while [repcounter <= reps]\n[step]
NIL
1
T
OBSERVER
T

MONITOR
7
526
64
575
NIL
steps
3
1

SWITCH
595
350
719
383
drawLabels?
drawLabels?
1
1
-1000

PLOT
151
305
481
584
Nodes
log p
norm var
-3.0
0.2
0.0
1.0
true
true
PENS
"clustcoeff" 1.0 2 -8716033 true
"pathlen" 1.0 2 -65413 true
"polsteps" 1.0 2 -11352576 true
"Four" 1.0 0 -16711681 true
"Five" 1.0 0 -16711936 true
"Six" 1.0 0 -6524078 true
">Six" 1.0 0 -16777216 true

SLIDER
3
246
148
279
num_actors_start
num_actors_start
5
150
25
5
1
NIL

SLIDER
3
319
146
352
p_rand_log_start
p_rand_log_start
-4
0
-3.0
0.2
1
NIL

SWITCH
491
350
589
383
drawnet?
drawnet?
0
1
-1000

SLIDER
3
396
145
429
link_length
link_length
1
5
1
1
1
NIL

MONITOR
74
526
131
575
tol
tol
3
1

SLIDER
3
434
144
467
range
range
0
50
4
1
1
NIL

MONITOR
600
389
657
438
av l(p)
short_l
3
1

MONITOR
625
442
682
491
clustc
clust_coeff
3
1

SLIDER
3
281
147
314
num_actors_end
num_actors_end
5
150
35
5
1
NIL

SLIDER
3
357
143
390
p_rand_log_end
p_rand_log_end
-4
0
0.0
0.2
1
NIL

MONITOR
814
10
891
59
NIL
num_actors
3
1

MONITOR
815
67
865
116
p
10 ^ p_rand_log
4
1

SLIDER
4
472
143
505
start_policy_%
start_policy_%
0
100
60
2
1
NIL

PLOT
690
393
990
586
poldev
iter
%_pol_agree
0.0
20.0
0.0
100.0
true
false

MONITOR
488
533
565
582
red actors
count actors with [color = red]
0
1

MONITOR
579
533
648
582
NIL
polsteps
0
1

SWITCH
490
388
596
421
pathlen?
pathlen?
0
1
-1000

SWITCH
490
442
613
475
clustcoeff?
clustcoeff?
0
1
-1000

SWITCH
489
497
617
530
pol_action?
pol_action?
0
1
-1000

MONITOR
905
235
968
284
NIL
shortcuts
3
1

MONITOR
815
123
950
172
NIL
100 * (10 ^ p_rand_log)
3
1

MONITOR
815
179
874
228
NIL
randval
3
1

MONITOR
899
10
949
59
edges
num_actors * range
3
1

MONITOR
815
235
900
284
exp. shortcuts
(10 ^ p_rand_log) * num_actors * range
3
1

MONITOR
815
291
929
340
NIL
length edge_list
3
1

MONITOR
877
67
950
116
NIL
p_rand_log
3
1

SLIDER
5
145
147
178
reps
reps
0
100
3
1
1
NIL

MONITOR
36
182
108
231
repetition
repcounter
0
1

@#$#@#$#@
Small world parameters and political activity in a network

Network regular and randomized according to rules in Watts and Strogatz 1998

somewhere in between the regular and randomised networks we should see a small world, if parameters are right, this is recognised by the large drop in global path length withpout a corresponding fall in clustering.

We can examine alongside this an actor based simulation of political activity in networks. At present this is a (not very good) rule that looks towards convergence of opinion on a policy, where actors are influenced by their inetractions with each other. I have a paper in progresson this subject. I can make drafts available if people are interested.

What we hope to see is that some kind of believable model of policy formation in a network is affected by the small world properties eing present.

Things to note
1) depending on the size of your network/ expected number of shortcuts you can boost up the minimum p-log value and save a load of worthless iterations
2) the poldev graph is not much use (at the momnet), it just shows that the policy simulation is alive and kicking
3 output is done by issuing the SHOW command in the control window with the desired lists, that is always ploglist and nactorlist for the independent variables and any selection from pathllist, cclist and polstepslist fotr the dependent variables. Easy to copy, paste and turn text into columns in excel.
4) although the repetitions work, the repetition counter seems not to show the right value. I hope and pray that this is the only bug at the moment
5) I'm not going to dicuss or document the policy making simulation as I'm not happy with it and it will be covered in my paper.

Comments welcome

Stuart Astill
s.j.astill@lse.ac.uk
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
setup
init_one_run
one_run
@#$#@#$#@
@#$#@#$#@
