patches-own [psugar maxpsugar]
turtles-own [vision sugar initSugar metabolism age maxage tags neighborhood
    sex parentStep fertileStart fertileEnd]

; -- The experiments are setup by manipulating the values of the global variables. --
; Rather than a GUI element for each one, a choice box contains an experiment
; label to run.  A setup procedure derived from that label is called to set
; the globals where they differ from the defaults.
globals [
    visionRange metabolismRange sugarRange ageRange sexRange fertileRanges
    steps growBackRate includeMyPatch?
    scapeColor geometryScape? randAgents? agentLayout cultureTags
    useM? useR? useK? useS? male female
    ]
to globals-defaults
   ;set numTurtles      400       ; Turtles not declared or set; done by slider
    set useM?           true      ; Use the M (Motion) rule?
    set useR?           false     ; Use the R (Replacement) rule?
    set useS?           false     ; Use the S (Sexual reproduction) rule?
    set useK?           false     ; Use the K (Culture tags) rule?

    set growBackRate    1         ; The G (growback) rule's rate
    set metabolismRange [1 4]     ; lo/hi range for metabolism random value
    set visionRange     [1 6]     ; lo/hi range for vision random value
    set sugarRange      [5 25]    ; lo/hi range for sugar/wealth random value
    set ageRange        [60 100]  ; lo/hi range for age random value
    ; male/female values for begin/end of fertility ages as lo/hi ranges for random values.
    ; This sets both male/female start ages between 12 & 15, and male end age in range 50-60
    ; while female end age is between 40-50.
    set fertileRanges   [ [[12 15][50 60]] [[12 15][40 50]] ]

    ; culture tags: the list size determines the number of tags while the value determines
    ; the values at position n as "random item n culturtags".  With 2's, the tags are
    ; randomly 0 or 1.
    set cultureTags     [2 2 2 2 2 2 2 2 2 2 2]
    set includeMyPatch? true      ; whether to include me in the search neighborhood.
    set agentLayout     0         ; 0 = whole scape, 1 = lower left, 2 = lower left and top right
    set scapeColor      orange    ; orange or other dark color makes scape clearer. Yellow is usual.
    set geometryScape?  false     ; Whether to use a geometrically derived scape or a "map"
    set randAgents?     true      ; Whether or not to randomize "ask turtles" in go procedure
    set steps           0         ; The number of steps taken in the simulation thus far
    set male            0         ; Constant for "male"
    set female          1         ; Constant for "female"
end
to experiment0 ; Put "manual" experimental setup over-rides here
    set includeMyPatch? false
    set agentLayout     2
    set geometryScape?  true
    set scapeColor      yellow
    set metabolismRange [1 1]
    set visionRange     [5 10]
    set-globals 0 true false true false  ; 0 => use slider's value for number of turtles
end
to experiment1 set-globals 400 true false false false set growBackRate 100 end
to experiment2 set-globals 400 true false false false end
to experiment3 set-globals 250 true true false false end
to experiment4
    set agentLayout 1
    set includeMyPatch? false
    ;set geometryScape?  true
    ;set scapeColor      yellow
    set metabolismRange [1 1]
    set visionRange     [5 10]
    set-globals 0 true false false false  ; 0 => use slider's value for number of turtles
end
to experiment5a set-globals 400 true false true false end
to experiment5b experiment5a set fertileRanges [ [[12 15][40 50]] [[12 15][30 40]] ] end
to experiment5c experiment5a set sugarRange [10 40] end
to experiment6 set-globals 400 true false false true end

to setup-globals                  ; utility to setup the globals for this experiment
    globals-defaults
    run "experiment" + substring experiment 0 position ":" experiment
end
to set-globals [n M? R? S? K?]    ; utility used by the experiments
    if n > 0 [set numTurtles n]
    set useM? M?  set useR? R?  set useS? S?  set useK? K?
    ifelse useS? [set sugarRange [50 100]][set sugarRange [5 25]]
end
to set-color                      ; utility to change the color of the agents.  feel free to modify!
    if useS? [
        if age > fertileend [set color grey]
        if steps = parentStep [set color yellow]
        if (steps = parentStep + 1) or (steps = 0) or (steps > 2 and age = 1)
            [ifelse (sex = male) [set color red] [set color blue]]
        if steps > 1 and age = 0 [set color cyan]
    ]
    if useK? [
        ifelse (sum tags > length tags / 2) [set color red] [set color blue]
    ]
end

; -- The setup and go button's procedures --
to setup
    clear-all
    setup-globals
    set-default-shape turtles "circle"
    create-turtles numTurtles
    setup-plots
    setup-patches
    ask turtles [setup-turtles]
end
to go
    set steps steps + 1
    no-display
        without-interruption [
            ifelse randAgents?
                [foreach shuffle-agents turtles [ask ? [ Step ]]]
                [ask turtles [ Step ]]
            ask turtles [set-color]
        ]
        ask patches [ rule-G growBackRate ]
    display
    if count turtles = 0 [stop]
    do-plots
end
to Step
    if useM? [rule-M]
    if (sugar <= 0 or age > maxage) [if useR? [rule-R] die]
    if useK? [rule-K]
    if useS? [rule-S]
    set-color
end
to do-plots   ; Plotting procedure called each step
    locals [l ll i]
    set-current-plot "Agents"
    plot count turtles
    set-current-plot "Averages"
    set-current-plot-pen "Vision"
    plot mean values-from turtles [vision]
    set-current-plot-pen "Metabolism"
    plot mean values-from turtles [metabolism]
    if (steps = 1 or steps mod 10 = 0) [
        set-current-plot "Histogram"
        clear-plot
        ifelse useK? [
            set l []
            set i 0
            repeat length cultureTags [
                set ll remove 0 (values-from turtles [item i tags])
                set l sentence l map [? * i] ll
                set i i + 1
            ]
            set-plot-x-range 0 length cultureTags
            set-plot-y-range 0 count turtles
            set-histogram-num-bars length cultureTags
            histogram-list l
        ] [
            set-plot-x-range 0 (int ((max values-from turtles [sugar] + 9) / 10) * 10)
            set-histogram-num-bars 10
            histogram-from turtles [sugar]
        ]
    ]
end

; -- The SugarScape growth and motion procedures. --
to rule-M    ; Motion rule (page 25)
    locals [ps p v d]
    set ps (patches at-points neighborhood) with [count turtles-here = 0]
    if (count ps > 0) [
        set v psugar-of max-one-of ps [psugar]              ; v is max sugar w/in vision
        set ps ps with [psugar = v]                         ; ps is legal sites w/ v sugar
        set d distance min-one-of ps [distance myself]      ; d is min dist from me to ps agents
        set p random-one-of ps with [distance myself = d]   ; p is one of the min dist patches
        if (psugar >= v and includeMyPatch?) [set p patch-here]
        setxy pxcor-of p pycor-of p                         ; jump to p
        set sugar sugar + psugar-of p                       ; consume its sugar
        ask p [setpsugar 0]                                 ; .. setting its sugar to 0
    ]
    set sugar sugar - metabolism    ; eat sugar (metabolism)
    set age age + 1
end
to rule-R    ; Replacement rule
    ask patch-here [sprout 1 [setup-turtles]]
end
to rule-S    ; Sexual reproduction rule
    locals [ts t ps p]
    if (fertile?) [
        set t self
        set ts []
        ask neighbors4 [ask turtles-here with [sex != sex-of t and fertile?] [set ts fput self ts]]
        set ts sort-by [random 2 = 0] ts
        foreach ts [
            ask ? [set ps values-from (neighbors4 with [count turtles-here = 0]) [self]]
            set ps sentence ps values-from (neighbors4 with [count turtles-here = 0]) [self]
            if length ps != length remove-duplicates ps [show "dups!"]
            if (not empty? ps) [
                set p item (random length ps) ps
                ask p [ sprout 1 [
                    set sugar-of ? sugar-of ? - (initSugar-of ? / 2)
                    set sugar-of t sugar-of t - (initSugar-of t / 2)
                    set parentStep-of ? steps
                    set parentStep-of t steps
                    setup-turtle ((initSugar-of ? / 2) + (initSugar-of t / 2))
                                 (item random 2 list vision-of ? vision-of t)
                                 (item random 2 list metabolism-of ? metabolism-of t)
                                 list pxcor pycor
                ]]
            ]
            if (not fertile?) [stop]
        ]
    ]
end
to rule-K   ; Culture rule
    locals [i t myTags]
    set myTags tags
    ask neighbors4 [ask turtles-here [
        set i random length tags
        set t item i myTags
        set tags replace-item i tags t
        ;ifelse (sum tags > length tags / 2) [set color red] [set color blue]
    ]]
end
to rule-G [n]    ; Growback rule (page 23)
    setpsugar psugar + n
end
to-report fertile? ; Utility to report whether or not I'm fertile
    report age >= fertileStart and age <= fertileEnd and sugar > initSugar
end


; -- setup/initialization procedures --
to setup-turtles
    setup-turtle get-param sugarRange get-param visionRange get-param metabolismRange []
end
to setup-turtle [s v m xy]
    set sugar        s
    set initSugar    sugar
    set vision       v
    set neighborhood make-neighborhood vision
    set metabolism   m

    set age          0
    ifelse (useR? or useS?) [set maxage get-param ageRange][set maxage 1000000]
    set sex          random 2
    set parentStep   0
    set fertileStart get-param item 0 item sex fertileRanges
    set fertileEnd   get-param item 1 item sex fertileRanges
    set tags         map [random ?] cultureTags

    ifelse (who = 0) [set color black] [set color red] ; default
    set-color ; let user override
    ifelse (empty? xy) [setupxy] [setxy item 0 xy item 1 xy]
end
to setupxy
    locals [s x y]
    ifelse agentLayout = 0 [set s screen-size-x] [
    ifelse agentLayout = 1 [set s ceiling sqrt (numTurtles + 1)]
                           [set s ceiling sqrt ((numTurtles / 2) + 1)]
    ]
    set x random s
    set y random s
    if (agentLayout = 2 and random 2 = 0) [set x x - s set y y - s ]
    setxy x - screen-edge-x y - screen-edge-y
    if (any? other-turtles-here) [setupxy]
end
to setup-plots
    set-current-plot "Agents"
    set-plot-y-range 0 count turtles
    set-current-plot "Averages"
    set-plot-y-range 0 max list last visionRange last metabolismRange
    set-current-plot "Histogram"
    ifelse useK? [
        set-plot-x-range 0 length cultureTags
        set-histogram-num-bars length cultureTags
    ] [
        set-histogram-num-bars 10
    ]
end
to setup-patches
    ifelse (geometryScape?) [setup-patches-geom] [setup-patches-map]
end
to setup-patches-map
    ; From RePast's sugarscape simulation.  See its scape.pgm file.
    ; Right col & botom row added for NetLogo's odd-number grid
    locals [s]
    set s
        "000000000000000000111111111222222222222222222222211" +
        "000000000000000000111111111222222222333333222222221" +
        "000000000000000000111111112222222233333333332222222" +
        "000000000000000001111111122222223333333333333322222" +
        "000000000000000001111111122222233333333333333332222" +
        "000000000000000001111111222222333333333333333333222" +
        "000000000000000011111111222222333333344443333333222" +
        "000000000000000111111111222223333334444444433333322" +
        "000000000000000111111112222223333344444444443333322" +
        "000000000000001111111112222233333344444444443333332" +
        "000000000000011111111112222233333444444444444333332" +
        "000000000000111111111122222233333444444444444333332" +
        "000000000001111111111122222233333444444444444333332" +
        "000000000011111111111122222233333444444444444333332" +
        "000000000111111111111122222233333344444444443333332" +
        "000000011111111111111222222233333344444444443333322" +
        "000001111111111111112222222233333334444444433333322" +
        "111111111111111111122222222233333333344443333333222" +
        "111111111111111111222222222223333333333333333333222" +
        "111111111111111222222222222223333333333333333332222" +
        "111111111122222222222222222222333333333333333322222" +
        "111111112222222222222222222222233333333333332222222" +
        "111111222222222222222222222222222333333333222222222" +
        "111122222222222222222222222222222222222222222222211" +
        "111222222222222222222222222222222222222222222222111" +
        "112222222222333333333222222222222222222222222221111" +
        "122222222233333333333332222222222222222222222111111" +
        "122222223333333333333333222222222222222222111111111" +
        "222222233333333333333333322222222222221111111111111" +
        "222222333333333333333333322222222222211111111111111" +
        "222222333333344443333333332222222222111111111111110" +
        "222223333334444444433333332222222222111111111110000" +
        "222223333344444444443333332222222211111111111100000" +
        "222233333344444444443333332222222211111111110000000" +
        "222233333444444444444333332222221111111111000000000" +
        "222233333444444444444333332222221111111111000000000" +
        "222233333444444444444333332222221111111110000000000" +
        "222233333444444444444333332222211111111100000000000" +
        "222233333344444444443333332222211111111000000000000" +
        "222223333344444444443333322222211111111000000000000" +
        "222223333334444444433333322222111111110000000000000" +
        "222222333333344443333333222222111111100000000000000" +
        "222222333333333333333333222222111111100000000000000" +
        "222222233333333333333332222221111111100000000000000" +
        "122222223333333333333322222211111111100000000000000" +
        "122222222233333333332222222211111111100000000000000" +
        "112222222222333333222222222111111111100000000000000" +
        "111222222222222222222222221111111111000000000000000" +
        "111122222222222222222222211111111111000000000000000" +
        "111111222222222222222221111111111111000000000000000" +
        "111111112222222222222111111111111110000000000000000"
    ask patches [
        setmaxpsugar read-from-string item ((25 - pycor) * 51 + (pxcor + 25)) s
    ]
end
to setup-patches-geom
    locals [c r d c1 r1]
    set c screen-edge-x / 2             ; center coords .. assume square layout
    set r 4 + (c * 1.415)               ; radius to center for largest circle
    set d r / 4                         ; delta between layers

    ask patches [setmaxpsugar 0]

    fillcircle c c r 1          fillcircle neg c neg c r 1
    filldiamond -9 9 5 1        filldiamond 9 -9 5 1

    set r r - d
    fillcircle c c r 2          fillcircle neg c neg c r 2
    filldiamond -3 3 8 2        filldiamond 3 -3 8 2

    set r r - d set c1 c - 3 set r1 r - 2.5
    fillcircle c1 c1 r1 3       fillcircle neg c1 neg c1 r1 3
    fillcircle c c r 3          fillcircle neg c neg c r 3

    set r r - d
    fillcircle c c r 4          fillcircle neg c neg c r 4
end

; --- Utilities ---
to fillcircle [x y r s]   ; Geom utility to fill a circle
    ask patches with [distancexy-nowrap x y <= r] [setmaxpsugar s]
end
to filldiamond [x y h s]   ; Geom utility to fill a diamond shape
    ask patches with [abs (pxcor - x) + abs (pycor - y) <= h] [setmaxpsugar s]
end

to setpsugar [s]    ; Utility to set this patches psugar to s, checking s for neg and maxsugar
    set psugar max list 0 (min list s maxpsugar)
    set pcolor scapeColor + 4 - psugar
end
to setmaxpsugar [s] ; Utility to set this patches maxpsugar to s
    set maxpsugar s
    setpsugar s
end

to-report shuffle-agents [a] ; Utility from Seth for randomizing an agent set
    locals [shuffled]
    set shuffled values-from a [self]
    set shuffled sort-by [random 2 = 0] shuffled
    report shuffled
end

to-report neg [n]     ; Utility for unary negation
    report 0 - n
end

to-report get-param [p] ; Utility to return a random number between values in a 2 element list
    ;ifelse ( is-list? p ) [report first p + random (last p - first p + 1)] [report p]
    report first p + random (last p - first p + 1)
end

to-report make-neighborhood [n]
    locals [i l]
    set i 1
    set l []
    while [ i <= n ] [
        set l lput (list 0 i) l
        set l lput (list i 0) l
        set l lput (list 0 (0 - i)) l
        set l lput (list (0 - i) 0) l
        set i i + 1
    ]
    report l
end


@#$#@#$#@
GRAPHICS-WINDOW
329
10
696
398
25
25
7.0
1
10
1
1
1
0

CC-WINDOW
2
606
726
722
Command Center

BUTTON
7
10
70
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
77
10
140
43
NIL
go
T
1
T
OBSERVER
T
NIL

BUTTON
146
10
209
43
step
go
NIL
1
T
OBSERVER
T
NIL

PLOT
4
107
307
240
Agents
Time
Count
0.0
100.0
0.0
100.0
true
false

PLOT
1
245
307
395
Averages
Time
Value
0.0
100.0
0.0
10.0
true
true
PENS
"Vision" 1.0 0 -16777216 true
"Metabolism" 1.0 0 -65536 true

PLOT
3
417
309
567
Histogram
Wealth or Tags
Turtles
0.0
500.0
0.0
100.0
true
false
PENS
"default" 10.0 1 -65536 true

SLIDER
6
55
139
88
numTurtles
numTurtles
100
800
400
50
1
NIL

CHOICE
154
52
315
97
Experiment
Experiment
"0:Manual" "1:Ginf,M-P26" "2:G1,M-P28" "3:G1,MR-P32" "4:Migration-P42" "5a:G1,MS-P56" "5b:G1,MS-P64" "5c:G1,MS-P65" "6:G1,MK-P72"
1

@#$#@#$#@
WHAT IS IT?
-----------
This is a version of SugarScape, as presented in "Growing Artificial Societies" by Epstein and Axtell.  It is preliminary, having only the rules: G M R S K.  Note: we use a darker color than the book's yellow in order to see the regions of the scape a bit more clearly.  It is easly set to yellow, see globals below. Note that we used a 51x51 scape rather than the book's 50x50 due to the NetLogo coordinate system having a 0 center, thus an odd number total width/height.  Likely not a problem.


HOW IT WORKS
------------
Patches represent a toroidal world with "sugar" (representing wealth) deposited in two roughly circular regons or "hills" of increasing sugar density.  Agents obey rules for harvesting sugar and reproducing, while the sugar patches obey a growback rule. See book for details.


HOW TO USE IT
-------------
Contrary to typical simulations, this does NOT use lots of GUI elements for each parameter of the model.  This is done to avoid cluttered spaces and to make it easier to set several parameters at once, simplifying the various simulations and avoiding errors.

The parameters are all globals and can be individually set.  See the globals portion of the procedures section for details.  Note that experiment0 lets one individually set each parameter, providing a means of complete generalization.  Insights from users much, much appreciated here!


THINGS TO NOTICE
----------------
We generally get reasonably close match with the book but in one case, the "waves" of migration experiment (labeled: "4:Migration-P42"), we cannot achieve the clustering and migration over time shown in the book and movie.  This is likely an artifact of my implementation, but it would be great if we could reproduce the sustained waves.  Similarly, I had difficulty reproducing the four sexual reproduction curves (5a-5c).


THINGS TO TRY
-------------
Setup your own experiment0, changing parameters as you'd like.  These parameters let you try new behaviors from the initial sugarscape model:
    set includeMyPatch? true      ; whether to include me in the search neighborhood.
    set agentLayout     0         ; 0 = whole scape, 1 = lower left, 2 = lower left and top right
    set scapeColor      orange    ; orange or other dark color makes scape clearer
    set geometryScape?  false     ; Whether to use a geometrically derived scape or a "map"


EXTENDING THE MODEL
-------------------
We need to complete the book's rules.  Volunteers appreciated!


NETLOGO FEATURES
----------------
Seth's "shuffle" routine is quite nifty.  I also like the brevity of
	set tags map [random ?] cultureTags
and other powerful list/agentset methods.  NetLogo team: Congrats!


RELATED MODELS
--------------
The Wealth Distribution model, in the Social Science section, is modeled pretty
closely on Sugarscape.


CREDITS AND REFERENCES
----------------------
http://www.brook.edu/press/books/ARTIFSOC.HTM
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.1pre3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
