__extensions [ "sound.jar" ]

globals [first-game? current-game
L  all-positions win-loose-tie-positions
L9 L9-win L9-loose L9-tie
computer-win-messages first-message?
setup-finished?]

to setup-field
   ifelse (pxcor + pycor) mod 2 = 0
      [set pcolor violet]
      [set pcolor pink]
end

to generate-all-positions
   locals [index temp]
   set index n-values 19683 [?]
   set L []
   foreach index
         [set temp convert-base3 ?
         if not invalid? temp [set L lput temp L]]
end

to-report convert-base3 [Z]
   locals [digit base3 base10]
   set base3 [0 0 0 0 0 0 0 0 0]
   set base10 Z
   set digit 0
   while [base10 > 0]
          [set base3 (replace-item digit base3 (base10 mod 3))
          set base10 int (base10 / 3)
          set digit (digit + 1)]
   report base3
end

to-report invalid? [Z]
   locals [Z0 the-report]
   set the-report false
   set Z0 remove 0 Z
   ifelse length Z0 mod 2 = 0
      [if 2 * sum Z0 != 3 * length Z0 or winning? 1 Z
         [set the-report true]]
      [if 2 * sum Z0 + 1 != 3 * length Z0 or winning? 2 Z
         [set the-report true]]
   report the-report
end

to-report my-sublist [indices some-list]
   locals [temp]
   set temp []
   foreach indices
     [set temp lput item ? some-list temp]
   report temp
end

to-report test? [a list-of-lists some-list] ; a = 1 or a = 2
; helps to report if player a has already won
   locals [temp]
   set temp false
   foreach list-of-lists
      [if sum my-sublist ? some-list = 3 * a and not member? 0 my-sublist ? some-list [set temp true]]
   report temp
end

to-report winning? [a some-list] ; a = 1 or a = 2
; reports if player a has already won
   locals [check-list]
   set check-list [
   [0 1 2] [3 4 5] [6 7 8]
   [0 3 6] [1 4 7] [2 5 8]
   [0 4 8] [2 4 6] ]
   ifelse test? a check-list some-list
      [report true]
      [report false]
end

to-report select-list-of-size [a]
   locals [some-position]
   set some-position []
   foreach L
      [if length remove 0 ? = a [set some-position lput ? some-position]]
   report some-position
end

to partition-positions-by-length
   set all-positions []
   foreach [1 2 3 4 5 6 7 8 9]
      [set all-positions lput select-list-of-size ? all-positions]
end

to-report next-move [a old-position]; a = 1 or a = 2
   locals [temp new-positions]
   set temp []
   set new-positions []
   foreach [0 1 2 3 4 5 6 7 8]
      [set temp replace-item ? old-position a
       if length remove 0 temp > length remove 0 old-position
         [set new-positions lput temp new-positions]
      ]
   report new-positions
end

to-report win-loose-tie [a  old-position win loose tie]
   locals [my-report]
   ;set my-report "loose"
   ; reports if player a has a winning move, at least a tie move, or only loosing moves
   foreach next-move a old-position
      [if member? ? win [set my-report "win"]]
   if my-report = "win" [report "win"]
   foreach next-move a old-position
      [if member? ? tie [set my-report "tie"]]
   if my-report = "tie" [report "tie"]
   report "loose"
end

to-report win-loose-tie-start
   set L9-win []
   set L9-loose []
   set L9-tie []
   foreach item 8 all-positions
      [ifelse winning? 1 ?
         [set L9-win lput ? L9-win]
         [set L9-tie lput ? L9-tie]]
   report (list L9-loose L9-win L9-tie)
end

to win-loose-tie-induction
   locals [length-k-pos b c previous-win-loose-tie win loose tie]
   set win-loose-tie-positions fput win-loose-tie-start []
   foreach [7 6 5 4 3 2 1 0]
      [set length-k-pos item ? all-positions
       set b ((? mod 2) + 1)
       set c ((? + 1) mod 2 + 1)
       set previous-win-loose-tie item 0 win-loose-tie-positions
       set win item 0 previous-win-loose-tie
       set loose item 1 previous-win-loose-tie
       set tie item 2 previous-win-loose-tie
       set win-loose-tie-positions fput win-loose-tie-transition b c length-k-pos win loose tie win-loose-tie-positions ]
end

to-report win-loose-tie-transition [b c Lk win loose tie]
   locals [response win-1 loose-1 tie-1]
   set win-1 [] set loose-1 [] set tie-1 []
   foreach Lk
   ; c is 1 if b is 2 and 2 if b is 1
   ; win-1 contains all positions from which player c can win
   ; loose-1 contains all positions from which player c should loose or already lost
   ; tie-1 contains all positions from which player c can at best get a tie
      [ifelse winning? b ?
         [set loose-1 lput ? loose-1] ; player c has already won
         [set response win-loose-tie c ? loose win tie
          if response = "win" [set win-1 lput ? win-1]
          if response = "loose" [set loose-1 lput ? loose-1]
          if response = "tie" [set tie-1 lput ? tie-1]]
       ]
    report (list win-1 loose-1 tie-1)
end

to setup-game
   ifelse is-number? setup-finished?
   [
   set setup-finished? false
   ask patches [setup-field]
   set current-game [0 0 0 0 0 0 0 0 0]
   generate-all-positions
   partition-positions-by-length
   win-loose-tie-induction
   set setup-finished? true
   ]
   [
   user-message
   "You only need to set up the game once. Just click the play-game button for another game"
   ]
end

to cross
   locals [my-turn? x y]
   set my-turn? true
   while [my-turn?]
   [if mouse-down?
      [ask patch-at mouse-xcor mouse-ycor [
           sprout 1 [set shape "cross" set color black]
           set my-turn? false
           set x pxcor set y pycor
           ]
      ]
   ]
   set current-game replace-item this-patch x y current-game 1
   make-cross-sound
end

to circle
   locals [my-turn? x y]
   set my-turn? true
   while [my-turn?]
   [if mouse-down?
      [ask patch-at mouse-xcor mouse-ycor [
           sprout 1 [set shape "circle" set color black]
           set my-turn? false
           set x pxcor set y pycor
           ]
      ]
   ]
   set current-game replace-item this-patch x y current-game 2
   make-circle-sound
end

to-report this-patch [x y]
   report 4 + x - y * 3
end

to-report x-cor-game [z]
   report (z mod 3) - 1
end

to-report y-cor-game [z]
   report (3 + (z mod 3) - z) / 3
end

to play-one-move
   (foreach [0 1 2 3 4 5 6 7 8] current-game
      [
      ask patch-at x-cor-game ?1 y-cor-game ?1 [
  if not any? turtles-here [
     if ?2 = 1 [sprout 1 [set shape "cross" set color black] make-cross-sound]
     if ?2 = 2 [sprout 1 [set shape "circle" set color black] make-circle-sound]
  ]
     ] ])
end

to set-computer-messages
  set computer-win-messages
     [
      "You are no match against me!"
      "Got you!"
      "That's my winning move!"
     ]
end


to-report winning-move [a win tie loose]
   locals [great-moves okay-moves]
   set great-moves []
   set okay-moves []
   foreach next-move a current-game
      [if member? ? win [set great-moves lput ? great-moves]]
   ifelse length great-moves != 0
         [if first-message?
            [print random-one-of computer-win-messages
             set first-message? false]
          report random-one-of great-moves]
         [foreach next-move a current-game
           [if member? ? tie [set okay-moves lput ? okay-moves]]]
   ifelse length okay-moves != 0
         [;print "tie move"
          report random-one-of okay-moves]
         [print "loosing move" ; this never happens
          report random-one-of loose]
end

to make-cross-sound
   play-drum "LOW WOOD BLOCK" 127
end

to make-circle-sound
   play-drum "HI WOOD BLOCK" 127
end

to play-game1
   locals [temp win loose tie]
   ask turtles [die]
   set current-game [0 0 0 0 0 0 0 0 0]
   foreach [0 2 4 6 8]
      [set temp item ? win-loose-tie-positions
       set win item 0 temp
       set loose item 1 temp
       set tie item 2 temp
       set current-game winning-move 1 loose tie win
       play-one-move display
       if winning? 1 current-game or ? = 8 [stop]
       circle wait 1]
end

to play-game2
   locals [temp win loose tie]
   ask turtles [die]
   display
   set current-game [0 0 0 0 0 0 0 0 0]
   foreach [1 3 5 7]
      [cross wait 1
       set temp item ? win-loose-tie-positions
       set win item 0 temp
       set loose item 1 temp
       set tie item 2 temp
       set current-game winning-move 2 loose tie win
       play-one-move display
       if winning? 2 current-game [stop]
       ]
   if not winning? 2 current-game [cross]
end

to play-game
  if not setup-finished?
     [user-message "Please wait until the game is ready to play" stop]
  clear-output
  set-computer-messages
  set first-message? true
  ifelse computer-first?
     [play-game1]
     [play-game2]
end
@#$#@#$#@
GRAPHICS-WINDOW
321
10
631
341
1
1
100.0
1
10
1
1
1
0

CC-WINDOW
5
355
640
450
Command Center

BUTTON
33
16
130
49
NIL
setup-game
NIL
1
T
OBSERVER
T
NIL

BUTTON
34
66
123
99
NIL
play-game
NIL
1
T
OBSERVER
T
NIL

SWITCH
150
16
291
49
computer-first?
computer-first?
0
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
This is the familiar Tic Tac Toe game programmed in NetLogo where you can play against the computer.

HOW IT WORKS
------------
The program collects all possible Tic Tac Toe positions in a list and then rearranges the list by the number of moves that have been made. There are at most 9 moves possible in a game. After 9 moves it is easy to determine who won or whether or not it was a tie. The program then partitions all positions of length k, k=1,2,...,8 (that is k moves have been made) into three lists: a list of winning positions, a list of loosing positions, and a list of tie positions. The list of winning positions of length k consists of all positions of length k where the player whose move it is next can force a win. The list of loosing positions of length k consists of all positions of length k where the player whose move it is next either lost with the k-th move or the other player can force a win. If such a partitioning has been established for positions of length k then it can be established for positions of length k-1 as follows. The program assigns each position of length k-1 to the winning, loosing or tie list depending on what happens with every possible next move. Note that for each given position there are only a few possible next moves. The program proceeds inductively working its way from k=8 to k=1.

After all positions have been partitioned into winning, loosing, and tie, the game can begin. The computer first checks if there is a winning move. If there is none, it picks a tie move. This is only possible because the lists of tie moves of length 1 and length 2 are not empty. Since the list of winning moves of length 1 is empty, neither the first nor the second player can force a win.


HOW TO USE IT
-------------
First click on the setup-game button. This may take a while, but you only have to do it once. Next click on the play-game button. If the switch "computer-first?" is on then the computer will make the first move. Otherwise the computer waits for your move. In order to make a move simply click on the playing field of your choice. You need to give the computer about 1 second to make a move, otherwise you may encounter an error.


THINGS TO NOTICE
----------------
The computer never looses.


CREDITS AND REFERENCES
----------------------
This program was written by Bertram Zinner, 2004.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

cross
false
0
Polygon -7566196 true true 30 59 61 28 270 226 240 270
Polygon -7566196 true true 210 28 30 241 60 270 258 59

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.1beta3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
