

#include "cCSVFileReader.h"



cCSVFileReader::cCSVFileReader()
{
	data=new CityData;
	
	Color_Tuple colorRGB;
	colorRGB.R=255;
	colorRGB.G=255;
	colorRGB.B=0;
	colorMapping[1]=colorRGB; 	//yellow for res
	
	colorRGB.R=255;
	colorRGB.G=0;
	colorRGB.B=0;
	colorMapping[2]=colorRGB;	//red for com
	
	colorRGB.R=0;
	colorRGB.G=0;
	colorRGB.B=255;
	colorMapping[3]=colorRGB;	//blue for ind
	
	colorRGB.R=128;
	colorRGB.G=128;
	colorRGB.B=128;
	colorMapping[4]=colorRGB;	//gray for road
	
	//mix Com and REs 
	colorRGB.R=255;
	colorRGB.G=128;
	colorRGB.B=0;
	colorMapping[6]=colorRGB;
	
	colorRGB.R=0;
	colorRGB.G=255;
	colorRGB.B=255;
	colorMapping[50] =colorRGB;	
	
	colorRGB.R=64;
	colorRGB.G=64;
	colorRGB.B=64;
	colorMapping[8] =colorRGB;	//black for intersection
	
	colorRGB.R=0;
	colorRGB.G=0;
	colorRGB.B=0;
	colorMapping[9] =colorRGB;	//black for intersection
	
	colorRGB.R=255;
	colorRGB.G=255;
	colorRGB.B=255;
	colorMapping[10] =colorRGB;	//white for corner
	//everything else
	colorRGB.R=0;
	colorRGB.G=255;
	colorRGB.B=0;
	colorMapping[7] =colorRGB;
	
	primaryRoadStr= new char[GLOBAL_MAXBUFF_SIZE];
	memset(primaryRoadStr,0,GLOBAL_MAXBUFF_SIZE);
	secondaryRoadStr = new char [GLOBAL_MAXBUFF_SIZE];
	memset(secondaryRoadStr,0,GLOBAL_MAXBUFF_SIZE);
	waterFrontStr = new char [GLOBAL_MAXBUFF_SIZE];
	memset(waterFrontStr,0,GLOBAL_MAXBUFF_SIZE);
	
}

cCSVFileReader::~cCSVFileReader()
{
	data->patches.clear();
	data->building_turtles.clear();
	data->PatchMap.clear();
	delete data;
}
bool cCSVFileReader::readCSVFile(string fileName)
{
	char buffer[MAXBUFF_SIZE];
	char dataStr[MAXBUFF_SIZE], tmpBuf[MAXBUFF_SIZE], tmpStr[MAXBUFF_SIZE]; 
	FILE * dataFile=NULL;
	
	
	vector<PatchCoor> patchCoorItem;
	
	double temp_max_elevation=-1;
	double temp_min_elevation=256;
	if ((dataFile=fopen(fileName.c_str(), "r"))==0)
		return false;
	//now we can start reading the CSV file

	//first try to find the global values, set the screen size;
	while (!feof(dataFile))
	{		
		fgets(buffer,MAXBUFF_SIZE-1,dataFile);		
		//printf("%s",buffer);
		if (strncmp(buffer, "\"GLOBALS\"\n",9)==0)
		{
			fgets(buffer,MAXBUFF_SIZE-1,dataFile);  //the line to denote the meaning
			//printf("%s",buffer);
			parseGlobalDescription(buffer);
			
			//now we can parse the global section
			readGlobalSection(dataFile);
			
			break;
		}
	}
	//printf("OK read global section\n");
	
	//MagickImage = new Image(Geometry(data->size,data->size),"white");
  	//MagickImage->type( GrayscaleType );
  	//MagickImage->type(TrueColorType);
	
	//after that (break out from the previous loop)
	//we need to search for turtles
	fgets(buffer,MAXBUFF_SIZE-1,dataFile);
	while (!feof(dataFile))
	{
		fgets(buffer,MAXBUFF_SIZE-1,dataFile);
		if (strncmp(buffer, "\"TURTLES\"",9)==0) //now we come to the patches part
		{
			fgets(buffer,MAXBUFF_SIZE-1,dataFile); //pass the line of description
			parseTurtleDescription(buffer);
			break;
		}
	}
	//in the section of turtles, we are only caring about the building turtles.
	//so we need only record the population and area it occupied.
	
	readTurtleSection(dataFile);					
	//printf("OK read turtle section\n");
	//after that (break out from the previous loop)
	//we need to search for patches
	//fgets(buffer,MAXBUFF_SIZE-1,dataFile);
	
	while (!feof(dataFile))
	{
		fgets(buffer,MAXBUFF_SIZE-1,dataFile);
		if (strncmp(buffer, "\"PATCHES\"", 8)==0) //now we come to the patches part
		{
			
			fgets(buffer,MAXBUFF_SIZE-1,dataFile); //pass the line of description
			//here we need to parse the description line to store the mapping of words and order
			parsePatchDescription(buffer);
			break;
		}
	}
	
	readPatchSection(dataFile);
	
	
												 
	//end of building our own topology*/
	
	//parcelDensityRemapper();
	//findMaxPossibleSquare();
	//findSplitSquares();
	
	printf("Parsing Primary Roads...\n");
	processSequenceSegements(primaryRoadStr, roadSegments_primary,GLOBAL_MAXBUFF_SIZE,false,true);
	//printf("primary road parsing OK!\n");

	//processSequenceSegements(secondaryRoadStr, roadSegments_secondary,GLOBAL_MAXBUFF_SIZE,false);

	//in this function we are going to build the topology on secondary road
	//from the raster grid using the DFS
	printf("Parsing Secondary Roads...\n");
	buildSecondaryRoadTopology(roadSegments_secondary_new);
	//printf("secondary road parsing ok!\n");
	
	 //we don't want to excessive wiggliness
	printf("Parsing Water Front...\n");
	processSequenceSegements(waterFrontStr, waterFrontSegments,GLOBAL_MAXBUFF_SIZE,true,false);
	//printf("waterfront parsing OK!\n");
	
	//fixOverlapParcels();
	
	printf("Fixing holes, patches under road and overlap patches...\n");
	fixOverlapParcels();
	fixEmptyParcelUnderRoad(); //need to revisit this function later
	fixHolesInParcels();
	fixNotFourConnectedParcel();
	
	printf("Fixing roads interconnections...\n");
	fixRoadIntersection2(roadSegments_primary,2); //works fine for now, may need some finer tweak
	
	fixSecondaryPrimaryConnection(roadSegments_secondary_new, roadSegments_primary);

	
	int i;
	
	
	int xcoor;
	int ycoor;
	int fUp,fDown,fLeft,fRight;
	
	int patchIdx;
	PatchCoor patchCoor_road;
	for (i=0; i<data->patches.size();i++)
	{
		//printf("%d",i);
		fUp=0;
		fDown=0;
		fLeft=0;
		fRight=0;
		xcoor = data->patches[i].xcor;
		ycoor = data->patches[i].ycor;
		if (!data->patches[i].isRoad)
			continue;
		if ((patchIdx =getPatchIndex(xcoor-1,ycoor))!=-1)
		{
			//printf("\tleft: %d", patchIdx);
			if ((data->patches[patchIdx].isRoad)&&(data->patches[patchIdx].road_type==3))
				fLeft=1;
		}
		if ((patchIdx =getPatchIndex(xcoor+1,ycoor))!=-1)
		{
			//printf("\tRight: %d", patchIdx);
			if ((data->patches[patchIdx].isRoad)&&(data->patches[patchIdx].road_type==3))
				fRight=1;
		}
		if ((patchIdx =getPatchIndex(xcoor,ycoor-1))!=-1)
		{
			//printf("\tUp: %d", patchIdx);
			if ((data->patches[patchIdx].isRoad)&&(data->patches[patchIdx].road_type==3))
				fUp=1;
		}
		if ((patchIdx =getPatchIndex(xcoor,ycoor+1))!=-1)
		{
			//printf("\tDown: %d", patchIdx);
			if ((data->patches[patchIdx].isRoad)&&(data->patches[patchIdx].road_type==3))
				fDown=1;
		}
		
		if ((fUp+fDown+fLeft+fRight)==4)
		{
			patchCoor_road.xcor = xcoor;
			patchCoor_road.ycor = ycoor;
			roadIntersector.push_back(patchCoor_road);
			//printf("\tRoad Intersection");
		}
		//printf("\n");
		
	}
	if (secondaryRoadStr!=NULL)
		delete [] secondaryRoadStr;
	if (primaryRoadStr!=NULL)
		delete [] primaryRoadStr;
	if (waterFrontStr!=NULL)
		delete [] waterFrontStr;
	fclose(dataFile);
	return true;
}

inline void cCSVFileReader::readInCSVDataItem(const char *input, char *output, int idx, long int buffer_size)
{
	//given input string, and idx, give out output str
	//long int buffer_size = strlen(input)+1;
	//printf("%d\n", buffer_size);
	char *tmpBuf = new char[buffer_size];
	memset(output,0,buffer_size);
	memset(tmpBuf,0,buffer_size);
	strcpy(tmpBuf,input);
	getRidOfQuot(output,getDataAtPos(tmpBuf, idx, buffer_size), buffer_size);
	delete [] tmpBuf;
}

int cCSVFileReader::readGlobalSection(FILE * dataFile)
{
	//the assumption is the global section contains a very long line;
	char buffer[GLOBAL_MAXBUFF_SIZE];
	char dataStr[GLOBAL_MAXBUFF_SIZE];
	
	
	memset(buffer,0,GLOBAL_MAXBUFF_SIZE);
	fgets(buffer,GLOBAL_MAXBUFF_SIZE-1,dataFile);	//the very first line of global data
	
	//now we come the global value definition part
	//here we should read the window size x,y in 
	//prepare for data string
		
	readInCSVDataItem(buffer,dataStr,globalHeader.screen_edge_x_idx,GLOBAL_MAXBUFF_SIZE);
	data->xmax=atoi(dataStr);
	//xmax is the max possible x cordinate
			
	readInCSVDataItem(buffer,dataStr,globalHeader.screen_edge_y_idx,GLOBAL_MAXBUFF_SIZE);	
	data->ymax=atoi(dataStr);
	//ymax is the max possible y cordinate
			
	preparePatchMap();
	
	//elevation	
	readInCSVDataItem(buffer,dataStr,globalHeader.ground_level_idx,GLOBAL_MAXBUFF_SIZE);
	data->ground_level=atoi(dataStr);
	
	readInCSVDataItem(buffer,dataStr,globalHeader.water_level_idx,GLOBAL_MAXBUFF_SIZE);
	data->water_level=atoi(dataStr);
			
			
	//residential:	
	readInCSVDataItem(buffer,dataStr,globalHeader.average_residential_density_idx,GLOBAL_MAXBUFF_SIZE);
	data->average_residential_density=atof(dataStr);
			
	readInCSVDataItem(buffer,dataStr,globalHeader.max_residential_density_idx,GLOBAL_MAXBUFF_SIZE);
	data->max_residential_density=atof(dataStr);
	
	readInCSVDataItem(buffer,dataStr,globalHeader.residential_population_idx,GLOBAL_MAXBUFF_SIZE);
	data->residential_population=atoi(dataStr);
	
		
	//commercial
				
	readInCSVDataItem(buffer,dataStr,globalHeader.average_commercial_density_idx,GLOBAL_MAXBUFF_SIZE);
	data->average_commercial_density=atof(dataStr);
				
	readInCSVDataItem(buffer,dataStr,globalHeader.max_commercial_density_idx,GLOBAL_MAXBUFF_SIZE);
	data->max_commercial_density=atof(dataStr);
			
	readInCSVDataItem(buffer,dataStr,globalHeader.commercial_population_idx,GLOBAL_MAXBUFF_SIZE);
	data->commercial_population=atoi(dataStr);
	
	
	//industrial
				
	readInCSVDataItem(buffer,dataStr,globalHeader.average_industrial_density_idx,GLOBAL_MAXBUFF_SIZE);		
	data->average_industrial_density=atof(dataStr);
		
	readInCSVDataItem(buffer,dataStr,globalHeader.max_industrial_density_idx,GLOBAL_MAXBUFF_SIZE);
	data->max_industrial_density=atof(dataStr);

	readInCSVDataItem(buffer,dataStr,globalHeader.industrial_population_idx,GLOBAL_MAXBUFF_SIZE);
	data->industrial_population=atoi(dataStr);
	


	//road and lake front parser for svg generation
	readInCSVDataItem(buffer,dataStr,globalHeader.pins_tertiaries_idx,GLOBAL_MAXBUFF_SIZE);
	//printf("\n%s\n", dataStr);
	//memset(secondaryRoadStr,0,GLOBAL_MAXBUFF_SIZE);
	strcpy(secondaryRoadStr,dataStr);
	
				
	readInCSVDataItem(buffer,dataStr,globalHeader.pins_primaries_idx,GLOBAL_MAXBUFF_SIZE);
	//memset(primaryRoadStr,0,GLOBAL_MAXBUFF_SIZE);
	strcpy(primaryRoadStr,dataStr);
	

			
	readInCSVDataItem(buffer,dataStr,globalHeader.pins_water_boundaries_idx,GLOBAL_MAXBUFF_SIZE);
	//printf("\n%s\n", dataStr);
	//memset(waterFrontStr,0,GLOBAL_MAXBUFF_SIZE);
	strcpy(waterFrontStr,dataStr);


	return 0;
}

int cCSVFileReader::readTurtleSection(FILE * dataFile)
{
	char buffer[MAXBUFF_SIZE];
	char dataStr[MAXBUFF_SIZE];
	char tmpStr[MAXBUFF_SIZE];
	map<int,int> turtle_class_map; //from class turtle id to RCI
	int turtle_class_idx;
	TurtleData turtleItem;
	do 
	{	
		memset(buffer,0,MAXBUFF_SIZE);
		fgets(buffer,MAXBUFF_SIZE-1,dataFile);
		if (strlen(buffer)<=2)
			{
				break;				//an empty line consist one two character 10 13 or on windows 10
			}						//good for now, may need to review later
		//printf("OK Road!\n");
		memset(dataStr,0,MAXBUFF_SIZE);					
		readInCSVDataItem(buffer,tmpStr,turtleHeader.breed_idx,MAXBUFF_SIZE);
		getStrBeforeBra(dataStr,tmpStr);
		
		if (!strcmp(dataStr,"developer-classes"))
		{
			//find the 	
			int idx;
			readInCSVDataItem(buffer,dataStr,turtleHeader.who_idx,MAXBUFF_SIZE);	
			idx = atoi(dataStr);
			readInCSVDataItem(buffer,dataStr,turtleHeader.developer_name_idx,MAXBUFF_SIZE);
			if (!strcmp(dataStr,"\"\"residential\"\""))
				turtle_class_map[idx]=1;
			if (!strcmp(dataStr,"\"\"commercial\"\""))
				turtle_class_map[idx]=2;
			if (!strcmp(dataStr,"\"\"industrial\"\""))
				turtle_class_map[idx]=3;
			continue;
		}
		
		//printf("%s\n", dataStr);
		//if (strcmp(dataStr,"buildings") && strcmp(dataStr,"parks"))				//the turtle is not what we need.
		//	continue;
	
		if (!strcmp(dataStr,"buildings"))
		{
			readInCSVDataItem(buffer,dataStr,turtleHeader.building_zone_idx,MAXBUFF_SIZE);
		
			sscanf(dataStr,"{turtle %d}",&turtle_class_idx);
			//printf("%d\n", turtle_class_idx);
			turtleItem.type=turtle_class_idx;  //defer the actual type mapping until we read all
											   //the turtle information since at this moment the mapping
											   //may not be ready
			
			readInCSVDataItem(buffer,dataStr,turtleHeader.heading_idx,MAXBUFF_SIZE);
			turtleItem.heading = atoi(dataStr);
		
 
			readInCSVDataItem(buffer,dataStr,turtleHeader.population_idx,MAXBUFF_SIZE);
			turtleItem.population = atoi(dataStr);
		 
			readInCSVDataItem(buffer,dataStr,turtleHeader.population_density_idx,MAXBUFF_SIZE);
			turtleItem.density = atof(dataStr);
			
				//clamp the density to its max
			if (turtleItem.type==1 && turtleItem.density>data->max_residential_density)
			{
				turtleItem.population = (int)(turtleItem.population * data->max_residential_density/turtleItem.density);
				turtleItem.density = data->max_residential_density;
			}
			
			
			if (turtleItem.type==2 && turtleItem.density>data->max_commercial_density)
			{
				turtleItem.population = (int)(turtleItem.population * data->max_commercial_density/turtleItem.density);
				turtleItem.density = data->max_commercial_density;
			}
			
			if (turtleItem.type==3 && turtleItem.density>data->max_industrial_density)
			{
				turtleItem.population = (int)(turtleItem.population * data->max_industrial_density/turtleItem.density);
				turtleItem.density = data->max_industrial_density;
			}
		}
		else if (!strcmp(dataStr,"parks"))
			{
				turtleItem.type=-1;
				turtleItem.density=0;
				turtleItem.population=0;
				turtleItem.heading=0;
			}
			else continue;
		
		//turtle's id is where we can uniquely identify it, 
		//since patch will store the turtle id as its owner
		//we need a map turtleMap to quickly access turtle data.
		readInCSVDataItem(buffer,dataStr,turtleHeader.who_idx,MAXBUFF_SIZE);	
		turtleItem.id = atoi(dataStr);

			
		//read in the patches belong to this parcel		
		readInCSVDataItem(buffer,dataStr,turtleHeader.parcel_idx,MAXBUFF_SIZE);
		extractPatchInfo(turtleItem.patchCoords,dataStr, turtleItem.id);
		
		//printf("%d\t%d\t%d\t%f\n",turtleItem.id,turtleItem.type,turtleItem.population,turtleItem.density);
		data->building_turtles.push_back(turtleItem);  //store it into the city data
		turtleMap[turtleItem.id] = data->building_turtles.size()-1; //record the index map
	}while (1);	
	
	//do the actual type mapping;
	for (int k=0; k<data->building_turtles.size(); k++)
	{
		if (data->building_turtles[k].type==-1)
			data->building_turtles[k].type=4; //park
		else
		{
			data->building_turtles[k].type = turtle_class_map[data->building_turtles[k].type];
		}
	}		
	return 0;
}

int cCSVFileReader::readPatchSection(FILE * dataFile)
{
	char buffer[MAXBUFF_SIZE];
	char dataStr[MAXBUFF_SIZE];
	char tmpStr[MAXBUFF_SIZE];
	int i;
	PatchData patchItem;
	for (i=0; i<(2*data->xmax+1)*(2*data->ymax+1);i++)
	{	
		memset(buffer, 0, MAXBUFF_SIZE);
		fgets(buffer,MAXBUFF_SIZE-1,dataFile); //this line is the data line	
		
		//x,y coordinate
		readInCSVDataItem(buffer,dataStr,patchHeader.pxcor_idx,MAXBUFF_SIZE);		
		patchItem.xcor=atoi(dataStr)+data->xmax;
			
		readInCSVDataItem(buffer,dataStr,patchHeader.pycor_idx,MAXBUFF_SIZE);
		//patchItem.ycor=(atoi(dataStr)+data->ymax);
		patchItem.ycor=-(atoi(dataStr)-data->ymax);
		
		//road
		readInCSVDataItem(buffer,dataStr,patchHeader.road_idx,MAXBUFF_SIZE);
		if (strcmp(dataStr,"false"))	
			patchItem.isRoad=true;
		else patchItem.isRoad=false; 
		
		//update the patchMap with road information
		if (patchItem.isRoad)
			data->PatchMap[patchItem.xcor*data->size+patchItem.ycor]=-1;

		readInCSVDataItem(buffer,dataStr,patchHeader.roadtype_idx,MAXBUFF_SIZE);
		patchItem.road_type=atoi(dataStr);
		
		readInCSVDataItem(buffer,dataStr,patchHeader.terrain_idx,MAXBUFF_SIZE);
		if (!strcmp(dataStr,"\"\"land\"\""))
			patchItem.terrain=0;
		if (!strcmp(dataStr,"\"\"water\"\""))
			patchItem.terrain=1;
		
		
		readInCSVDataItem(buffer,dataStr,patchHeader.elevation_idx,MAXBUFF_SIZE);
		patchItem.elevation=atoi(dataStr);
		//in simcity visualization, we should set non water patch elevate to ground elevation
		if (patchItem.terrain==0)
			patchItem.elevation = data->ground_level;
		
		readInCSVDataItem(buffer,tmpStr,patchHeader.owner_idx,MAXBUFF_SIZE);
		if (!strcmp(tmpStr,"nobody"))
			patchItem.owner=-1;
		else
		{
			memset(dataStr,0,MAXBUFF_SIZE);
			getStrBeforeBra(dataStr,tmpStr);
			patchItem.owner=atoi(dataStr);	
		}
		
		//connectivity
		readInCSVDataItem(buffer,dataStr,patchHeader.connectivity_idx,MAXBUFF_SIZE);
		patchItem.connectivity=atoi(dataStr);
		
		readInCSVDataItem(buffer,dataStr,patchHeader.is_corner_idx,MAXBUFF_SIZE);
		if (!strcmp(dataStr,"true"))
			patchItem.is_corner=1;
		if (!strcmp(dataStr,"false"))
			patchItem.is_corner=0;
	
		data->patches.push_back(patchItem);	

		//printf("%d\t",patchItem.xcor);
		//printf("%d\n",patchItem.ycor);
		/*printf("%d\t",patchItem.isRoad);
		printf("%d\t",patchItem.road_type);
		printf("%d\t",patchItem.connectivity);
		printf("%d\t",patchItem.is_corner);
		printf("%d\t",patchItem.terrain);
		printf("%d\t",patchItem.elevation);
		printf("%d\n",patchItem.owner);
		*/

	}
	
}


//build our own topology
//based on the patch map.
void cCSVFileReader::buildSecondaryRoadTopology(vector< deque<PatchCoor> > &roads)
{
	
	vector<PatchCoor> traverseList;
	PatchData patchItem;
	
	PatchCoor patch_coor;

	vector<bool> isVisited; //the boolean array for all the patches.


	int xcoor, ycoor; //the current coordinate to be operated on
	for (int i=0; i<data->size; i++)
	{
		for (int j=0; j<data->size; j++)
		{
			isVisited.push_back(false);
		}
	}
	
	int patchIndex; //in the 1D array
	
	//try to find a seed
	for (int i=0; i<data->size; i++)
	{
		for (int j=0; j<data->size; j++)
		{
			
				xcoor=i;
				ycoor=j;
				//we are first trying to find a seed to begin with
				if ((patchIndex = getPatchIndex(xcoor, ycoor))!=-1)
				{
					//now we get a valid patchIndex;
					if (isVisited[patchIndex]) 
						continue;
					//else isVisited[patchIndex]=true;
						
					patchItem=data->patches[patchIndex];
					if (!((patchItem.isRoad) && (patchItem.road_type==3))) continue; //3 secondary Road, 1 primary road
					// now we are sure this patch is a valid one to seed the depth first search
					
					patch_coor.xcor = xcoor;
					patch_coor.ycor = ycoor;
					//printf("Start DFS\n");
					DFSRoadBuilder(patch_coor, isVisited, traverseList,0,0);
					//break;
				} //end of finding seed
				
		} //end of j
	}  //end of i 
	formRoadSegmentsFromDFSTraversal(traverseList,roads);
	//fixFourConnectors(roads);
	//printf("Done with fixing secondary and primary road connections\n");
}
inline bool cCSVFileReader::canGoFurther_DFS(PatchCoor &patch_coor, 
									  PatchCoor &patch_coor_togo,
                                      vector<bool> &isVisited,
									  bool newEntrance)
{
	//if neighbour unvisited, is road, not consecutive primary road then we can run DFS on it
	
	//proceed the edge patch_coor to 
	PatchData patchItem;
	int xcoor = patch_coor_togo.xcor;
	int ycoor = patch_coor_togo.ycor;
	int patchIndex;
	int prev_road_type=data->patches[getPatchIndex(patch_coor.xcor,patch_coor.ycor)].road_type;	
	if ((patchIndex = getPatchIndex(xcoor, ycoor))!=-1)
	{
		patchItem=data->patches[patchIndex];
		
		if (patchItem.isRoad)
		{
			//if (patchItem.connectivity>=3)
			//	return true;
			if (!isVisited[patchIndex])
			{
				//return true; 
				if ((patchItem.road_type==3))// && (prev_road_type==1))
				{
						return true;
				}
			}
			else
			{
				if (newEntrance)
					return true;
			}
		}
			
	}
	return false;				
}
void cCSVFileReader::DFSRoadBuilder(PatchCoor patch_coor, 
                                    vector<bool> & isVisited, 
                                    vector<PatchCoor> &traverseList,
                                    int xoffset, int yoffset)
{
	//process this one
	PatchCoor placeholder_coor;
	placeholder_coor.xcor = -1;
	placeholder_coor.ycor = -1;
	deque<PatchCoor> prev_connect;
	bool newEntrance;
	/*if (traverseList.size()>2)
	{
		vector<PatchCoor>::iterator iter_prev = traverseList.end()-1;
		int count=0;
	
		while ((iter_prev!=traverseList.begin())&&((iter_prev->xcor!=-1)||(iter_prev->ycor!=-1))&&(count<4))
		{
			prev_connect.push_front(*iter_prev);
			count++;
			iter_prev--;
		}
	}*/
	int neighbour_cango_count=0;  //how many neighbour can proceed
	int patchIndex;
	if ((patchIndex=getPatchIndex(patch_coor.xcor,patch_coor.ycor))!=-1)
	{
		newEntrance = !isVisited[patchIndex];
		//printf("(%d,%d)--> ", patch_coor.xcor, patch_coor.ycor);
		isVisited[patchIndex]=true;
		traverseList.push_back(patch_coor);
	}
	
	prev_connect.push_back(patch_coor);
	
	PatchCoor neighbour_coor;
	//for each 4 direction process espectively
	for (int i=1; i>=-1; i--)
	{
		for (int j=1; j>=-1; j--)
		{
			if ((i==-xoffset) && (j==-yoffset))
				continue; //we don't want this road segment go backwards.
			
			if ((abs(i)+abs(j))==1) //4 connected neighbour
			{
				neighbour_coor.xcor=patch_coor.xcor+i;
				neighbour_coor.ycor=patch_coor.ycor+j;
				
				if (canGoFurther_DFS(patch_coor, neighbour_coor, isVisited, newEntrance))
				{
					if (neighbour_cango_count>0)
					{	
						for (int k=0; k<prev_connect.size(); k++)
							traverseList.push_back(prev_connect[k]);
					}
					DFSRoadBuilder(neighbour_coor,isVisited,traverseList, i,j);
					neighbour_cango_count++;
				}
			}
		}
	}
	
	/*for (int i=1; i>=-1; i--)
	{
		for (int j=1; j>=-1; j--)
		{
			if ((abs(i)+abs(j))==2) //8 connected neighbour
			{
				neighbour_coor.xcor=patch_coor.xcor+i;
				neighbour_coor.ycor=patch_coor.ycor+j;
				
				if (canGoFurther_DFS(patch_coor, neighbour_coor, isVisited,newEntrance))
				{
					if (neighbour_cango_count>0)
					{	
						for (int k=0; k<prev_connect.size(); k++)
							traverseList.push_back(prev_connect[k]);
					}
					DFSRoadBuilder(neighbour_coor,isVisited,traverseList,i,j);
					neighbour_cango_count++;
				}
			}
		}
	}
	*/
	if (neighbour_cango_count==0)
	{
		//end of one branch
		traverseList.push_back(placeholder_coor);
		//printf("done\n");
	}
}

void cCSVFileReader::formRoadSegmentsFromDFSTraversal(vector<PatchCoor> &traverseList,
													  vector< deque<PatchCoor> > &roads)
{
	vector<PatchCoor>::iterator iter_traverse;

	iter_traverse = traverseList.begin();
	
	deque<PatchCoor> roadSeg;
	deque<PatchCoor> roadSeg_after_conn_fix;
	PatchCoor curr_coor = *iter_traverse;
	int count=0;
	int patchIndex;
	while(iter_traverse!=traverseList.end())
	{			
		//if the current one is not 4 connected with previous one
		while  ((curr_coor.xcor!=-1) && (curr_coor.ycor!=-1))
		{		
			//printf("(%d,%d)-->", curr_coor.xcor, curr_coor.ycor);
			roadSeg.push_back(curr_coor);			
			iter_traverse++;
			curr_coor = *iter_traverse;	
		}
		if (roadSeg.size()>=3) //long enough 
		{
			for (deque<PatchCoor>::iterator iter_conn=roadSeg.begin();
				 iter_conn!=roadSeg.end();
				 iter_conn++)
			{
				if (data->patches[getPatchIndex(iter_conn->xcor,iter_conn->ycor)].connectivity>=3)
				{
					roadSeg_after_conn_fix.push_back(*iter_conn);
					roadSeg_after_conn_fix.push_back(*iter_conn);
					//roadSeg_after_conn_fix.push_back(*iter_conn);
				}
				roadSeg_after_conn_fix.push_back(*iter_conn);
			}
			//printf("done\n");
			roads.push_back( roadSeg_after_conn_fix);
		}
		roadSeg.clear(); 
 		roadSeg_after_conn_fix.clear();
		iter_traverse++; 
		if (iter_traverse!=traverseList.end())
			curr_coor = *iter_traverse;	
	}
}

bool cCSVFileReader::checkCanExtendSecondRoad(PatchCoor patch_coor)
{
	//do we really want this end to be extended?
	//depends if it is an end we want it to be extended, 
	//in it is an intersection of more than 1 secondary roads we don't want 
	int xcoor, ycoor;
	xcoor = patch_coor.xcor;
	ycoor = patch_coor.ycor;
	int patch_index=getPatchIndex(xcoor,ycoor);
	PatchData patch_item;
	if (patch_index!=-1)
	{
		patch_item = data->patches[patch_index];
	}
	
	//if (patch_item.connectivity<3) //it is not an even connector
	//	return true;
	
	bool ret=true;
	int count_secondary=0;
	//search out for it 4 connected neighbours
	//if more than 1(>1) connected neighbour are secondary road_type=3
	//then DONT extend it
	
	if ((patch_index=getPatchIndex(xcoor-1,ycoor))!=-1)
	{
		if (data->patches[patch_index].road_type==3)
			count_secondary++;
	}
	if ((patch_index=getPatchIndex(xcoor+1,ycoor))!=-1)
	{
		if (data->patches[patch_index].road_type==3)
			count_secondary++;
	}
	if ((patch_index=getPatchIndex(xcoor,ycoor+1))!=-1)
	{
		if (data->patches[patch_index].road_type==3)
			count_secondary++;
	}
	if ((patch_index=getPatchIndex(xcoor,ycoor-1))!=-1)
	{
		if (data->patches[patch_index].road_type==3)
			count_secondary++;
	}
	if (count_secondary<=2)
		return true;
	else
		return false;
	//return true;
}
void cCSVFileReader::fixSecondaryPrimaryConnection(vector< deque<PatchCoor> > &secondRoads,
												   const vector< deque<PatchCoor> > &primaryRoads)
{
	//this function will fix the connection of secondary road and primary road
	vector< deque<PatchCoor> >::iterator iterCurRoad = secondRoads.begin();
	deque<PatchCoor>::iterator iterCurPatch;
	int xcoor,ycoor;
	int prev_xcoor, prev_ycoor;
	int x_diff,y_diff;
	int patchIndex, probePatchIndex;
	PatchData patchItem, probePatchItem;
	PatchCoor probePatchCoor;
	vector<PatchCoor> candidateSet;
	const int extend_threshold=3;
	int radius;
	bool not_found;
	
	
	while(iterCurRoad!=secondRoads.end())  //iterate for all the secondary roads
	{
		//for each secondary road segement, if it is an end then looking for near by primary patches
		//the begin end;
		iterCurPatch = iterCurRoad->begin();	
		xcoor = iterCurPatch->xcor;
		ycoor = iterCurPatch->ycor;		
		patchIndex = getPatchIndex(xcoor,ycoor);
		patchItem = data->patches[patchIndex];
		
		
		
		//do we really want this end to be extended?
		//depends if it is an end we want it to be extended, 
		//in it is an intersection of more than 1 secondary roads we don't want 
		//bool canExtends=checkCanExtendSecondRoad(*iterCurPatch);
		if(checkCanExtendSecondRoad(*iterCurPatch))
		{
			iterCurPatch++; //the previous one 
			prev_xcoor =(iterCurPatch)->xcor;
			prev_ycoor =(iterCurPatch)->ycor; 			
			x_diff = xcoor - prev_xcoor;
			y_diff = ycoor - prev_ycoor;
			
			//now we are going to search for the near by 
			
			not_found=true;
			candidateSet.clear();
			radius=1;
			while ((radius<=extend_threshold) && (not_found))
			{
				for (int i=radius; i>=-radius; i--)
				{	
					for(int j=radius; j>=-radius; j--)
					{
						if (i==0 && j==0) continue;
						
						if ((probePatchIndex = getPatchIndex(xcoor+i, ycoor+j))!=-1)
						{
							if (data->patches[probePatchIndex].road_type==1) //it is primary road;
							{
								probePatchCoor.xcor = xcoor+i;
								probePatchCoor.ycor = ycoor+j;
								for (vector< deque<PatchCoor> >::const_iterator iter_primary_road=primaryRoads.begin();
									 iter_primary_road != primaryRoads.end(); iter_primary_road++)
								{
									for (deque<PatchCoor>::const_iterator iter_primary_road_segments=iter_primary_road->begin();
									 iter_primary_road_segments != iter_primary_road->end(); iter_primary_road_segments++)
									{
										//it is a match
										if ((probePatchCoor.xcor==iter_primary_road_segments->xcor) && 
											(probePatchCoor.ycor==iter_primary_road_segments->ycor))
										{
											candidateSet.push_back(probePatchCoor);
											not_found=false;
										}
									}
								}
							}
						}
					}
				}
				radius++;
			}
			//picking from those candidate
			if (!not_found)
			{
				double min_dist=1000000.0;
				double dist;
				int norm_x;
				int norm_y;
				vector<PatchCoor>::iterator iter_min = candidateSet.end();
				for (vector<PatchCoor>::iterator iter_candidate= candidateSet.begin();
					  iter_candidate!= candidateSet.end();
					  iter_candidate++)
				{
					dist = (iter_candidate->xcor-x_diff-xcoor) * (iter_candidate->xcor-x_diff-xcoor) +
						   (iter_candidate->ycor-y_diff-ycoor) * (iter_candidate->ycor-y_diff-ycoor) ;
					if ((iter_candidate->xcor-xcoor)==0)
						norm_x=0;
					else
					{
						norm_x=(iter_candidate->xcor-xcoor)/abs(iter_candidate->xcor-xcoor);
					}
					if ((iter_candidate->ycor-ycoor)==0)
						norm_y=0;
					else
					{
						norm_y=	(iter_candidate->ycor-ycoor)/abs(iter_candidate->ycor-ycoor);
					}
					if ((norm_x==x_diff) && (norm_y==y_diff))					
						dist=0;
					if (dist<min_dist)
					{
						min_dist =dist;
						iter_min = iter_candidate;
					}
				}
				if (iter_min!=candidateSet.end())
				{
					// found one;
					
					iterCurRoad->push_front(*iter_min);
					iterCurRoad->push_front(*iter_min);
					iterCurRoad->push_front(*iter_min);
				}
			}
			
			
		}
	
		//now we need to find our candidate 
		
		//the other end 
		iterCurPatch = iterCurRoad->end()-1;	
		xcoor = iterCurPatch->xcor;
		ycoor = iterCurPatch->ycor;		
		patchIndex = getPatchIndex(xcoor,ycoor);
		patchItem = data->patches[patchIndex];
		if(checkCanExtendSecondRoad(*iterCurPatch))
		{
			iterCurPatch--; //the second to the last one 
			prev_xcoor =(iterCurPatch)->xcor;
			prev_ycoor =(iterCurPatch)->ycor; 			
			x_diff = xcoor - prev_xcoor;
			y_diff = ycoor - prev_ycoor;
			
			not_found=true;
			candidateSet.clear();
			radius=1;
			while ((radius<=extend_threshold) && (not_found))
			{
				for (int i=radius; i>=-radius; i--)
				{	
					for(int j=radius; j>=-radius; j--)
					{
						if (i==0 && j==0) continue;
						
						if ((probePatchIndex = getPatchIndex(xcoor+i, ycoor+j))!=-1)
						{
							if (data->patches[probePatchIndex].road_type==1) //it is primary road;
							{
								probePatchCoor.xcor = xcoor+i;
								probePatchCoor.ycor = ycoor+j;
								for (vector< deque<PatchCoor> >::const_iterator iter_primary_road=primaryRoads.begin();
									 iter_primary_road != primaryRoads.end(); iter_primary_road++)
								{
									for (deque<PatchCoor>::const_iterator iter_primary_road_segments=iter_primary_road->begin();
									 iter_primary_road_segments != iter_primary_road->end(); iter_primary_road_segments++)
									{
										//it is a match
										if ((probePatchCoor.xcor==iter_primary_road_segments->xcor) && 
											(probePatchCoor.ycor==iter_primary_road_segments->ycor))
										{
											candidateSet.push_back(probePatchCoor);
											not_found=false;
										}
									}
								}
							}
						}
					}
				}
				radius++;
			}
			//picking from those candidate
			if (!not_found)
			{
				double min_dist=1000000.0;
				double dist;
				int norm_x;
				int norm_y;
				vector<PatchCoor>::iterator iter_min = candidateSet.end();
				for (vector<PatchCoor>::iterator iter_candidate= candidateSet.begin();
					  iter_candidate!= candidateSet.end();
					  iter_candidate++)
				{
					dist = (iter_candidate->xcor-x_diff-xcoor) * (iter_candidate->xcor-x_diff-xcoor) +
						   (iter_candidate->ycor-y_diff-ycoor) * (iter_candidate->ycor-y_diff-ycoor) ;
					if ((iter_candidate->xcor-xcoor)==0)
						norm_x=0;
					else
					{
						norm_x=(iter_candidate->xcor-xcoor)/abs(iter_candidate->xcor-xcoor);
					}
					if ((iter_candidate->ycor-ycoor)==0)
						norm_y=0;
					else
					{
						norm_y=	(iter_candidate->ycor-ycoor)/abs(iter_candidate->ycor-ycoor);
					}
					if ((norm_x==x_diff) && (norm_y==y_diff))					
						dist=0;
					if (dist<min_dist)
					{
						min_dist =dist;
						iter_min = iter_candidate;
					}
				}
				if (iter_min!=candidateSet.end())
				{
					// found one;
					
					iterCurRoad->push_back(*iter_min);
					iterCurRoad->push_back(*iter_min);
					iterCurRoad->push_back(*iter_min);
				}
			}
			
			
		
		}
		iterCurRoad++;
	}
	
} 
void cCSVFileReader::fixFourConnectors(vector< deque<PatchCoor> > &roads)
{
	vector< deque<PatchCoor> >::iterator iterCurRoad = roads.begin();
	deque<PatchCoor>::iterator iterCurPatch;
	int xcoor,ycoor;
	int prev_xcoor, prev_ycoor;
	int x_diff,y_diff;
	int patchIndex, probePatchIndex;
	PatchData patchItem, probePatchItem;
	PatchCoor probePatchCoor;
	vector<PatchCoor> candidateSet;
	
	const int extend_threshold=2;
	int radius;
	bool not_found;
	while(iterCurRoad!=roads.end())
	{
		//here we are focusing on the connection between end of the secondary roads and 
		//
		iterCurPatch = iterCurRoad->begin();
		
		xcoor = iterCurPatch->xcor;
		ycoor = iterCurPatch->ycor;		
		patchIndex = getPatchIndex(xcoor,ycoor);
		patchItem = data->patches[patchIndex];
		radius =1;
		
		if(checkCanExtendSecondRoad(*iterCurPatch))  //end point can extend
		{
			iterCurPatch++; //the one 
			prev_xcoor =(iterCurPatch)->xcor;
			prev_ycoor =(iterCurPatch)->ycor; 			
			x_diff = xcoor - prev_xcoor;
			y_diff = ycoor - prev_ycoor;
			
			not_found=true;
			candidateSet.clear();
			while ((radius<=extend_threshold) && (not_found))
			{
				for (int i=radius; i>=-radius; i--)
				{	
					for(int j=radius; j>=-radius; j--)
					{
						if (!((i*i+j*j)>(radius-1)*(radius-1) && ((i*i+j*j)<=radius*radius)))
							 continue;
						if (i==0 && j==0) continue;
						if ((probePatchIndex = getPatchIndex(xcoor+i, ycoor+j))!=-1)
						{
							probePatchItem = data->patches[probePatchIndex];
							if (probePatchItem.road_type==1)
							{
								probePatchCoor.xcor = xcoor+i;
								probePatchCoor.ycor = ycoor+j;
								candidateSet.push_back(probePatchCoor);
								not_found=false;
							}
						}
					}
				}
					
				radius++;
			}
			if (!not_found)
			{
				double min_dist=1000000.0;
				double dist;
				int norm_x;
				int norm_y;
				vector<PatchCoor>::iterator iter_min = candidateSet.end();
				for (vector<PatchCoor>::iterator iter_candidate= candidateSet.begin();
					  iter_candidate!= candidateSet.end();
					  iter_candidate++)
				{
					dist = (iter_candidate->xcor-x_diff-xcoor) * (iter_candidate->xcor-x_diff-xcoor) +
						   (iter_candidate->ycor-y_diff-ycoor) * (iter_candidate->ycor-y_diff-ycoor) ;
					if ((iter_candidate->xcor-xcoor)==0)
						norm_x=0;
					else
					{
						norm_x=(iter_candidate->xcor-xcoor)/abs(iter_candidate->xcor-xcoor);
					}
					if ((iter_candidate->ycor-ycoor)==0)
						norm_y=0;
					else
					{
						norm_y=	(iter_candidate->ycor-ycoor)/abs(iter_candidate->ycor-ycoor);
					}
					if ((norm_x==x_diff) && (norm_y==y_diff))					
						dist=0;
					if (dist<min_dist)
					{
						min_dist =dist;
						iter_min = iter_candidate;
					}
				}
				if (iter_min!=candidateSet.end())
				{
					// found one;
					
					iterCurRoad->push_front(*iter_min);
					iterCurRoad->push_front(*iter_min);
					//iterCurRoad->push_front(*iter_min);
					//iterCurRoad->push_front(*iter_min);
					//iterCurRoad->push_front(*iter_min);
					probePatchCoor.xcor = iter_min->xcor + iter_min->xcor - xcoor;
					probePatchCoor.ycor = iter_min->ycor + iter_min->ycor - ycoor;
					iterCurRoad->push_front(probePatchCoor);
					iterCurRoad->push_front(probePatchCoor);
					//probePatchCoor.xcor +=(iter_min->xcor - xcoor);
					//probePatchCoor.ycor +=(iter_min->ycor - ycoor);
					//iterCurRoad->push_front(probePatchCoor);
					//iterCurRoad->push_front(probePatchCoor);
					
					//iterCurRoad->push_front(probePatchCoor);
				}
			}
		}
		

		//another end
		iterCurPatch = iterCurRoad->end()-1;
		
		xcoor = iterCurPatch->xcor;
		ycoor = iterCurPatch->ycor;		
		patchIndex = getPatchIndex(xcoor,ycoor);
		patchItem = data->patches[patchIndex];
		radius=1;
		
		if(checkCanExtendSecondRoad(*iterCurPatch))  //end point can extend
		{
			iterCurPatch--; //the one 
			prev_xcoor =(iterCurPatch)->xcor;
			prev_ycoor =(iterCurPatch)->ycor; 			
			x_diff = xcoor - prev_xcoor;
			y_diff = ycoor - prev_ycoor;
			
			not_found=true;
			candidateSet.clear();
			while ((radius<=extend_threshold) && (not_found))
			{
				for (int i=radius; i>=-radius; i--)
				{	
					for(int j=radius; j>=-radius; j--)
					{
						if (!((i*i+j*j)>(radius-1)*(radius-1) && ((i*i+j*j)<=radius*radius)))
							 continue;
						if (i==0 && j==0) continue;
						if ((probePatchIndex = getPatchIndex(xcoor+i, ycoor+j))!=-1)
						{
							probePatchItem = data->patches[probePatchIndex];
							if (probePatchItem.road_type==1)
							{
								probePatchCoor.xcor = xcoor+i;
								probePatchCoor.ycor = ycoor+j;
								candidateSet.push_back(probePatchCoor);
								not_found=false;
							}
						}
					}
				}
					
				radius++;
			}
			if (!not_found)
			{
				double min_dist=1000000.0;
				double dist;
				int norm_x;
				int norm_y;
				vector<PatchCoor>::iterator iter_min = candidateSet.end();
				for (vector<PatchCoor>::iterator iter_candidate= candidateSet.begin();
					  iter_candidate!= candidateSet.end();
					  iter_candidate++)
				{
					dist = (iter_candidate->xcor-x_diff-xcoor) * (iter_candidate->xcor-x_diff-xcoor) +
						   (iter_candidate->ycor-y_diff-ycoor) * (iter_candidate->ycor-y_diff-ycoor) ;
					if ((iter_candidate->xcor-xcoor)==0)
						norm_x=0;
					else
					{
						norm_x=(iter_candidate->xcor-xcoor)/abs(iter_candidate->xcor-xcoor);
					}
					if ((iter_candidate->ycor-ycoor)==0)
						norm_y=0;
					else
					{
						norm_y=	(iter_candidate->ycor-ycoor)/abs(iter_candidate->ycor-ycoor);
					}
					if ((norm_x==x_diff) && (norm_y==y_diff))					
						dist=0;
					if (dist<min_dist)
					{
						min_dist =dist;
						iter_min = iter_candidate;
					}
				}
				if (iter_min!=candidateSet.end())
				{
					// found one;
					
					iterCurRoad->push_back(*iter_min);
					iterCurRoad->push_back(*iter_min);
					//iterCurRoad->push_back(*iter_min);
					//iterCurRoad->push_back(*iter_min);
					//iterCurRoad->push_back(*iter_min);
					probePatchCoor.xcor = iter_min->xcor + iter_min->xcor - xcoor;
					probePatchCoor.ycor = iter_min->ycor + iter_min->ycor - ycoor;
					iterCurRoad->push_back(probePatchCoor);
					iterCurRoad->push_back(probePatchCoor);
					/*probePatchCoor.xcor +=(iter_min->xcor - xcoor);
					probePatchCoor.ycor +=(iter_min->ycor - ycoor);
					iterCurRoad->push_back(probePatchCoor);
					iterCurRoad->push_back(probePatchCoor);
					*/
					//iterCurRoad->push_back(probePatchCoor);
				}
			}
		}		
		iterCurRoad++;
	}
}

void cCSVFileReader::fixRoadAfterMerged(vector< deque<PatchCoor> > &roads, bool isNormalOrder)
{
	
	
	int i,j,k,m;
	int xcoor, ycoor;
	PatchCoor push_tobe_item;
	
	int xDiff=0, yDiff=0;
	int patchIdx;
	
	vector< deque<PatchCoor> >::iterator iterCur = roads.begin();
	vector< deque<PatchCoor> >::iterator iterLastAppended;
	vector< deque<PatchCoor> >::iterator iterSearch;
	deque<PatchCoor>::iterator iterSearchInsideSeq;  //iterator inside the sequence we are searching now
	deque<PatchCoor>::iterator iterPrev, iterNext;
	deque<PatchCoor> currentRoadSeq;
	deque<PatchCoor> remainingHalf;  //once we contacated half of the sequence we need to push the remaining half
									 //at the end of the queue and waiting to process later.
	int curSeqSize;
	bool found;
	
	float prevWeight, nextWeight; //to see where should we go
	bool not_anymore=false; //can we further extend?
	//the pointer to the currently being processed one
	int count; //we need to go up a little bit
	
	candidateExtender extender;
	vector<candidateExtender> candidateSet;
	bool changed;
	while (iterCur!=roads.end())
	{
		//for each road sequence we need to find out how can we extend it to form a longer sequence
		//we don't care about the too small road patches.
		if ((curSeqSize=iterCur->size())<3)
			{
				//iterCur->clear();
				iterCur++;
				continue;
			}
		//assign value to the new RoadSeq
		currentRoadSeq.clear();
		for (i=0; i<iterCur->size(); i++)
				currentRoadSeq.push_back((*iterCur)[i]);
		iterLastAppended = iterCur;
		
		changed=false;
		
		candidateSet.clear();
		count=0;
		
		//do a search here to concatenate the sequence with other road sequence
			//first get the corrent coordinate 
			
			if (isNormalOrder)
			{
				xcoor = currentRoadSeq[currentRoadSeq.size()-1].xcor;
				ycoor = currentRoadSeq[currentRoadSeq.size()-1].ycor;
				
				//our direction indicator
				xDiff=0;
				yDiff=0;
				count = currentRoadSeq.size()-2;
				while (xDiff==0 && yDiff==0)
				{	
					xDiff = xcoor-currentRoadSeq[count].xcor;
					yDiff = ycoor-currentRoadSeq[count].ycor;
					count--;
				}
			}
			else
			{
				xcoor = currentRoadSeq[0].xcor;
				ycoor = currentRoadSeq[0].ycor;
				
				//our direction indicator
				xDiff=0;
				yDiff=0;
				count = 1;
				while (xDiff==0 && yDiff==0)
				{	
					xDiff = xcoor-currentRoadSeq[count].xcor;
					yDiff = ycoor-currentRoadSeq[count].ycor;
					count++;
				}
			}
			remainingHalf.clear();
			
			prevWeight = -1000;
			nextWeight = -1000;
			found=false;
			
			for (iterSearch=roads.begin(); iterSearch!=roads.end(); iterSearch++)
			{
				if (iterSearch==iterCur) continue;
				if (iterSearch->size()<3) continue;
				for (iterSearchInsideSeq=iterSearch->begin(); 
					 iterSearchInsideSeq!=iterSearch->end();
					 iterSearchInsideSeq++)
				{
					//try to find the same one 	
					if ((abs(iterSearchInsideSeq->xcor-xcoor)<=1) && (abs(iterSearchInsideSeq->ycor-ycoor)<=1))				
					{
						//bingo we find one candidate
						extender.iterSeq = iterSearch;
						extender.iterSpliter = iterSearchInsideSeq;
						
						iterNext=iterSearchInsideSeq+1;
						iterPrev=iterSearchInsideSeq-1;
						if (iterNext!=iterSearch->end())
						{
							//which means iterNext is valid we can go that direction
							
							extender.direction=0;
							nextWeight=(iterNext->xcor-xcoor-2*xDiff)*(iterNext->xcor-xcoor-2*xDiff)+
								       (iterNext->ycor-ycoor-2*yDiff)*(iterNext->ycor-ycoor-2*yDiff)+
									   3*(iterSearchInsideSeq->xcor-xcoor-xDiff)*(iterSearchInsideSeq->xcor-xcoor-xDiff)+
									   3*(iterSearchInsideSeq->ycor-ycoor-yDiff)*(iterSearchInsideSeq->ycor-ycoor-yDiff);
							extender.weight=nextWeight;
							candidateSet.push_back(extender);
							//printf("Next (%d,%d)--(%d,%d)-->(%d, %d)\t(%d,%d):\t%f\n", xcoor,ycoor,xDiff,yDiff,iterSearchInsideSeq->xcor, iterSearchInsideSeq->ycor, iterNext->xcor,iterNext->ycor,extender.weight);
							//printf("%d %d %d %d\n", (iterNext->xcor-xcoor-2*xDiff), (iterNext->ycor-ycoor-2*yDiff), (iterNext->xcor-xcoor-2*xDiff)*(iterNext->xcor-xcoor-2*xDiff), (iterNext->ycor-ycoor-2*yDiff)*(iterNext->ycor-ycoor-2*yDiff));
						}
						
						if (iterSearchInsideSeq!=iterSearch->begin())
						{
							//which means iterPrev is valid
							extender.direction=1;
						
							prevWeight=(iterPrev->xcor-xcoor-2*xDiff)*(iterPrev->xcor-xcoor-2*xDiff)+
								       (iterPrev->ycor-ycoor-2*yDiff)*(iterPrev->ycor-ycoor-2*yDiff)+
									   3*(iterSearchInsideSeq->xcor-xcoor-xDiff)*(iterSearchInsideSeq->xcor-xcoor-xDiff)+
									   3*(iterSearchInsideSeq->ycor-ycoor-yDiff)*(iterSearchInsideSeq->ycor-ycoor-yDiff);
						
						
							extender.weight=prevWeight;
							//printf("Prev (%d,%d)--(%d,%d)-->(%d, %d)\t(%d,%d):\t%f\n", xcoor,ycoor,xDiff,yDiff,iterSearchInsideSeq->xcor, iterSearchInsideSeq->ycor, iterPrev->xcor,iterPrev->ycor,extender.weight);

							candidateSet.push_back(extender);
						}
					}
				}
			}
			//ok we searched all the candidate connector 
			
			if (candidateSet.empty())
			{
				
				not_anymore=true;
			}
			else
			{	
				//now we need to do the extending and splitting
				double maxweight=10000000;
				int maxIdx=-1;
				int dir;
				deque<PatchCoor>::iterator curIter;
				for (int p=0; p<candidateSet.size(); p++)
				{
					if (candidateSet[p].weight<maxweight)
					{
						maxweight=candidateSet[p].weight;
						maxIdx=p;
					}
				}
				
				dir = candidateSet[maxIdx].direction;
				curIter = candidateSet[maxIdx].iterSpliter;
				remainingHalf.clear();
				//what we do now is according to the xDiff and yDiff to extend the road sequence 
				iterPrev = curIter;
				iterNext = curIter;
				count=0;
				if (dir==0)
				{
					//then we need to append following the normal order
					while((iterNext!=candidateSet[maxIdx].iterSeq->end())&&(count<=1))
					{
						
						if(isNormalOrder)
							iterCur->push_back(*iterNext);
						else
							iterCur->push_front(*iterNext);
						iterNext++;
						count++;
					}
				}
				else
				{
					//then we need to append in reverse order
					while((iterPrev!=candidateSet[maxIdx].iterSeq->begin())&&(count<=1))
					{
						
						if (isNormalOrder)
							iterCur->push_back(*iterPrev);
						else
							iterCur->push_front(*iterPrev);
						iterPrev--;
						count++;
					}
					
				}
				//candidateSet[maxIdx].iterSeq->clear();
				changed=true;
			}
		
		
		iterCur++;
	}			
}
void cCSVFileReader::mergeRoadSequence(vector< deque<PatchCoor> > & oldRoads, vector< deque<PatchCoor> > & newRoads, bool isPrimary, bool isNormalOrder)
{
	//this function takes the previous old roads as input
	//try to merge those small road segments as greedy as possible to form longer
	//roads
	
	
	int i,j,k,m;
	int xcoor, ycoor;
	PatchCoor push_tobe_item;
	
	int xDiff=0, yDiff=0;
	int patchIdx;
	
	vector< deque<PatchCoor> >::iterator iterCur = oldRoads.begin();
	vector< deque<PatchCoor> >::iterator iterLastAppended;
	vector< deque<PatchCoor> >::iterator iterSearch;
	deque<PatchCoor>::iterator iterSearchInsideSeq;  //iterator inside the sequence we are searching now
	deque<PatchCoor>::iterator iterPrev, iterNext;
	deque<PatchCoor> currentRoadSeq;
	deque<PatchCoor> remainingHalf;  //once we contacated half of the sequence we need to push the remaining half
									 //at the end of the queue and waiting to process later.
	int curSeqSize;
	bool found;
	
	float prevWeight, nextWeight; //to see where should we go
	bool not_anymore=false; //can we further extend?
	//the pointer to the currently being processed one
	int count; //we need to go up a little bit
	
	candidateExtender extender;
	vector<candidateExtender> candidateSet;
	bool changed;
	while (iterCur!=oldRoads.end())
	{
		//for each road sequence we need to find out how can we extend it to form a longer sequence
		//we don't care about the too small road patches.
		if ((curSeqSize=iterCur->size())<3)
			{
				iterCur->clear();
				iterCur++;
				continue;
			}
		//assign value to the new RoadSeq
		currentRoadSeq.clear();
		for (i=0; i<iterCur->size(); i++)
				currentRoadSeq.push_back((*iterCur)[i]);
		iterLastAppended = iterCur;
		not_anymore=false;
		changed=false;
		while(!not_anymore)
		{	
			candidateSet.clear();
			
			//do a search here to concatenate the sequence with other road sequence
			//first get the corrent coordinate 
			
			if (isNormalOrder)
			{
				xcoor = currentRoadSeq[currentRoadSeq.size()-1].xcor;
				ycoor = currentRoadSeq[currentRoadSeq.size()-1].ycor;
				
				//our direction indicator
				xDiff=0;
				yDiff=0;
				count = currentRoadSeq.size()-2;
				while (xDiff==0 && yDiff==0)
				{	
					xDiff = xcoor-currentRoadSeq[count].xcor;
					yDiff = ycoor-currentRoadSeq[count].ycor;
					count--;
				}
			}
			else
			{
				xcoor = currentRoadSeq[0].xcor;
				ycoor = currentRoadSeq[0].ycor;
				
				//our direction indicator
				xDiff=0;
				yDiff=0;
				count = 1;
				while (xDiff==0 && yDiff==0)
				{	
					xDiff = xcoor-currentRoadSeq[count].xcor;
					yDiff = ycoor-currentRoadSeq[count].ycor;
					count++;
				}
			}
			remainingHalf.clear();
			
			prevWeight = -1000;
			nextWeight = -1000;
			found=false;
			
			for (iterSearch=oldRoads.begin(); iterSearch!=oldRoads.end(); iterSearch++)
			{
				if (iterSearch==iterLastAppended) continue;
				if (iterSearch->size()<3) continue;
				for (iterSearchInsideSeq=iterSearch->begin(); 
					 iterSearchInsideSeq!=iterSearch->end();
					 iterSearchInsideSeq++)
				{
					//try to find the same one 	
					if ((abs(iterSearchInsideSeq->xcor-xcoor)<=1) && (abs(iterSearchInsideSeq->ycor-ycoor)<=1))				
					{
						//bingo we find one candidate
						extender.iterSeq = iterSearch;
						extender.iterSpliter = iterSearchInsideSeq;
						
						iterNext=iterSearchInsideSeq+1;
						iterPrev=iterSearchInsideSeq-1;
						if (iterNext!=iterSearch->end())
						{
							//which means iterNext is valid we can go that direction
							
							extender.direction=0;
							nextWeight=(iterNext->xcor-xcoor-2*xDiff)*(iterNext->xcor-xcoor-2*xDiff)+
								       (iterNext->ycor-ycoor-2*yDiff)*(iterNext->ycor-ycoor-2*yDiff)+
									   3*(iterSearchInsideSeq->xcor-xcoor-xDiff)*(iterSearchInsideSeq->xcor-xcoor-xDiff)+
									   3*(iterSearchInsideSeq->ycor-ycoor-yDiff)*(iterSearchInsideSeq->ycor-ycoor-yDiff);
							extender.weight=nextWeight;
							candidateSet.push_back(extender);
							//printf("%d %d %d %d\n", (iterNext->xcor-xcoor-2*xDiff), (iterNext->ycor-ycoor-2*yDiff), (iterNext->xcor-xcoor-2*xDiff)*(iterNext->xcor-xcoor-2*xDiff), (iterNext->ycor-ycoor-2*yDiff)*(iterNext->ycor-ycoor-2*yDiff));
						}
						
						if (iterSearchInsideSeq!=iterSearch->begin())
						{
							//which means iterPrev is valid
							extender.direction=1;
						
							prevWeight=(iterPrev->xcor-xcoor-2*xDiff)*(iterPrev->xcor-xcoor-2*xDiff)+
								       (iterPrev->ycor-ycoor-2*yDiff)*(iterPrev->ycor-ycoor-2*yDiff)+
									   3*(iterSearchInsideSeq->xcor-xcoor-xDiff)*(iterSearchInsideSeq->xcor-xcoor-xDiff)+
									   3*(iterSearchInsideSeq->ycor-ycoor-yDiff)*(iterSearchInsideSeq->ycor-ycoor-yDiff);
						
						
							extender.weight=prevWeight;

							candidateSet.push_back(extender);
						}
					}
				}
			}
			//ok we searched all the candidate connector 
			
			if (candidateSet.empty())
			{
				
				not_anymore=true;
			}
			else
			{	
				//now we need to do the extending and splitting
				double maxweight=10000000;
				int maxIdx=-1;
				int dir;
				deque<PatchCoor>::iterator curIter;
				for (int p=0; p<candidateSet.size(); p++)
				{
					if (candidateSet[p].weight<maxweight)
					{
						maxweight=candidateSet[p].weight;
						maxIdx=p;
					}
				}
				
				if (maxIdx==-1)//incase it fails 
				{
					
					not_anymore=true;
					break;
				}
				dir = candidateSet[maxIdx].direction;
				iterLastAppended = candidateSet[maxIdx].iterSeq;
				curIter = candidateSet[maxIdx].iterSpliter;
				remainingHalf.clear();
				//what we do now is according to the xDiff and yDiff to extend the road sequence 
				iterPrev = curIter;
				iterNext = curIter;
				if (dir==0)
				{
					//then we need to append following the normal order
					while(iterNext!=iterLastAppended->end())
					{
						if(isNormalOrder)
							currentRoadSeq.push_back(*iterNext);
						else
							currentRoadSeq.push_front(*iterNext);
						iterNext++;
					}
							
					//now we need to take care of the remaining half.
							
					while(iterPrev!=iterLastAppended->begin())
					{
						if (isNormalOrder)
							remainingHalf.push_back(*iterPrev);
						else
							remainingHalf.push_front(*iterPrev);
						iterPrev--;
					}
					remainingHalf.push_back(*(iterLastAppended->begin()));
					if (remainingHalf.size()>=4) //worth displaying
					{
						iterPrev=curIter;
						count=0;
						while((iterPrev!=iterLastAppended->end())&&(count<=1))
						{
							if (isNormalOrder)
					 			remainingHalf.push_front(*iterPrev);
							else
								remainingHalf.push_back(*iterPrev);
							iterPrev++;
							count++;
						}
					}
				}
				else
				{
					//then we need to append in reverse order
					while(iterPrev!=iterLastAppended->begin())
					{
						if (isNormalOrder)
							currentRoadSeq.push_back(*iterPrev);
						else
							currentRoadSeq.push_front(*iterPrev);
						iterPrev--;
					}
					currentRoadSeq.push_back(*(iterLastAppended->begin()));
							
					//now we need to take care of the remaining half.
					while(iterNext!=iterLastAppended->end())
					{
						if (isNormalOrder)
							remainingHalf.push_back(*iterNext);
						else
							remainingHalf.push_front(*iterNext);
						iterNext++;
					}
					if (remainingHalf.size()>=4)
					{
						iterNext=curIter;
						count=0;
						while((iterNext!=iterLastAppended->begin())&&(count<=1))
						{
							if (isNormalOrder)
								remainingHalf.push_front(*iterNext);
							else
								remainingHalf.push_back(*iterNext);
							iterNext--;
							count++;
						}
					}
				}
				if (remainingHalf.size()>=4)
							oldRoads.push_back(remainingHalf);
				candidateSet[maxIdx].iterSeq->clear();
				changed=true;
			}
		}//end of while (!not_anymore)
		
		//this currentRoadSeq is done we need to push it to the newRoads
		newRoads.push_back(currentRoadSeq);
		//if (changed)
		//	oldRoads.push_back(currentRoadSeq);
		iterCur->clear();
		iterCur++;
		not_anymore=false;
	}			
}


void cCSVFileReader::fixEmptyParcelUnderRoad()
//assign the empty patches underneath road to the parcels around,
//then there will be no empty green by the road.
{
	int i;
	int leftIdx;
	int rightIdx;
	int upIdx,downIdx;
	int leftupIdx, leftdownIdx, rightupIdx, rightdownIdx;
	int xcoor;
	int ycoor;
	PatchData patchItem;
	int newOwner;
	bool flagDone;
	//once we update 
	int j,k;
	int owner,owner_before;
	int patchIdx;
	PatchCoor currentPatch;
	bool nearRoad=false;
	int direction;
	int co_ownership_config=0;
	srand ( time(NULL) );
	//first pass deal with near road but not road ones.
	
	int* ownerMap = new int[data->patches.size()];
	int* zoneMap = new int[data->patches.size()];
	
	for (int m=0; m<data->patches.size(); m++)
	{
		ownerMap[m] = data->patches[m].owner;
		zoneMap[m] = data->patches[m].zone;
	}
	for (i=0; i<data->patches.size(); i++)
	{
		patchItem = data->patches[i];
		xcoor = patchItem.xcor;
		ycoor = patchItem.ycor;
		
		flagDone = false;
		nearRoad=false;
		direction=0;
		//we only care about empty spots.
		if (patchItem.owner!=-1)
			continue;
		//if (patchItem.isRoad)
		//	continue;
		leftIdx = getPatchIndex(xcoor-1,ycoor);
		rightIdx = getPatchIndex(xcoor+1,ycoor);
		upIdx = getPatchIndex(xcoor,ycoor-1);
		downIdx = getPatchIndex(xcoor,ycoor+1);
		
		
		if (patchItem.isRoad)
			nearRoad=true;
		vector<int> co_owner;
		co_owner.clear();
		co_ownership_config=0;
		data->patches[i].co_ownership_config=0;
		data->patches[i].co_ownership.clear();
		if (nearRoad) //for now we only care about road patches
		{
			//we actually have couple choices to extend the patches nearby to cover this road patch
			if((leftIdx!=-1) && (data->patches[leftIdx].owner!=-1))// && (!data->patches[leftIdx].isRoad))
					{
						co_owner.push_back(leftIdx);	
						//printf("Left ");
						co_ownership_config+=1000;
					}
					
			if((rightIdx!=-1) && (data->patches[rightIdx].owner!=-1))// && (!data->patches[rightIdx].isRoad))
					{
						co_owner.push_back(rightIdx);	
						//printf("Right ");
						co_ownership_config+=100;
					}
					
			if((upIdx!=-1) && (data->patches[upIdx].owner!=-1))// && (!data->patches[upIdx].isRoad))
					{
						co_owner.push_back(upIdx);
						//printf("Up ");
						co_ownership_config+=10;
					}
					
			if((downIdx!=-1) && (data->patches[downIdx].owner!=-1))// && (!data->patches[downIdx].isRoad))
					{
						co_owner.push_back(downIdx);
						//printf("Down ");
						co_ownership_config+=1;
					}
			data->patches[i].co_ownership_config=co_ownership_config;
			//if (co_owner.size()!=0)
			//	flagDone=true;
			if (co_owner.size()==1)
			{
				flagDone=true;
				//printf("%d ", data->patches[co_owner[0]].owner);
				//data->patches[i].owner = data->patches[co_owner[0]].owner;
				//data->patches[i].zone = data->patches[co_owner[0]].zone;
				
				ownerMap[i] = data->patches[co_owner[0]].owner;
				zoneMap[i] = data->patches[co_owner[0]].zone;
			}
			
			if (co_owner.size()>1)
			{
				//randomize to choose one
				//printf("How Many: %d: \t",co_owner.size());
				//for (int p=0; p<co_owner.size();p++)
				//{
				//	printf("%d ",data->patches[co_owner[p]].owner);
				//}
				
				int prevOwner=data->patches[co_owner[0]].owner;
				int k=1;
				while (k<co_owner.size()) //every neighbour belongs to the same parcel owner
				{
					if (data->patches[co_owner[k]].owner!=prevOwner)
					{
						break;
					}
					k++;
				}
				if(k==co_owner.size())
				{
					//data->patches[i].owner = prevOwner;
					//data->patches[i].zone = data->patches[co_owner[0]].zone;
					ownerMap[i] = data->patches[co_owner[0]].owner;
					zoneMap[i] = data->patches[co_owner[0]].zone;
					flagDone=true;
				}
				/*
				if (!flagDone) //for 3 and 4 choose the morjority
				{
					//count the majority of the neighbour owner.
					if (co_owner.size()==3)
					{
						if (data->patches[co_owner[0]].owner == data->patches[co_owner[1]].owner)
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[0]].owner;
							//data->patches[i].zone = data->patches[co_owner[0]].zone;
							ownerMap[i] = data->patches[co_owner[0]].owner;
							zoneMap[i] = data->patches[co_owner[0]].zone;							
						}
						if (data->patches[co_owner[0]].owner == data->patches[co_owner[2]].owner)
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[0]].owner;
							//data->patches[i].zone = data->patches[co_owner[0]].zone;	
							ownerMap[i] = data->patches[co_owner[0]].owner;
							zoneMap[i] = data->patches[co_owner[0]].zone;
						}
						if (data->patches[co_owner[1]].owner == data->patches[co_owner[2]].owner)
						{
							flagDone=true;
							ownerMap[i] = data->patches[co_owner[1]].owner;
							zoneMap[i] = data->patches[co_owner[1]].zone;
							//data->patches[i].owner = data->patches[co_owner[1]].owner;
							//data->patches[i].zone = data->patches[co_owner[1]].zone;	
						}
					}
					if (co_owner.size()==4)
					{
						if ((data->patches[co_owner[0]].owner == data->patches[co_owner[1]].owner)
						     && (data->patches[co_owner[0]].owner == data->patches[co_owner[2]].owner))
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[0]].owner;
							//data->patches[i].zone = data->patches[co_owner[0]].zone;
							ownerMap[i] = data->patches[co_owner[0]].owner;
							zoneMap[i] = data->patches[co_owner[0]].zone;							
						}
						if ((data->patches[co_owner[0]].owner == data->patches[co_owner[1]].owner)
						     && (data->patches[co_owner[0]].owner == data->patches[co_owner[3]].owner))
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[0]].owner;
							//data->patches[i].zone = data->patches[co_owner[0]].zone;
							ownerMap[i] = data->patches[co_owner[0]].owner;
							zoneMap[i] = data->patches[co_owner[0]].zone;							
						}
						if ((data->patches[co_owner[0]].owner == data->patches[co_owner[2]].owner)
						     && (data->patches[co_owner[0]].owner == data->patches[co_owner[3]].owner))
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[0]].owner;
							//data->patches[i].zone = data->patches[co_owner[0]].zone;
							ownerMap[i] = data->patches[co_owner[0]].owner;
							zoneMap[i] = data->patches[co_owner[0]].zone;							
						}
						
						if ((data->patches[co_owner[1]].owner == data->patches[co_owner[2]].owner)
						     && (data->patches[co_owner[1]].owner == data->patches[co_owner[3]].owner))
						{
							flagDone=true;
							//data->patches[i].owner = data->patches[co_owner[1]].owner;
							//data->patches[i].zone = data->patches[co_owner[1]].zone;
							ownerMap[i] = data->patches[co_owner[1]].owner;
							zoneMap[i] = data->patches[co_owner[1]].zone;							
						}
					}
				}
				if (!flagDone) 
				{
					if (co_owner.size()>=3)
					{
						int candidate=rand()%co_owner.size();	
					    //printf("%d\t",candidate);
				        //data->patches[i].owner = data->patches[co_owner[candidate]].owner;
				        //data->patches[i].zone = data->patches[co_owner[candidate]].zone;
						
						ownerMap[i] = data->patches[co_owner[candidate]].owner;
						zoneMap[i] = data->patches[co_owner[candidate]].zone;
						flagDone=true;						
					}
				}*/
 				//int candidate=rand()%co_owner.size();	
				//printf("%d\t",candidate);
				//data->patches[i].owner = data->patches[co_owner[candidate]].owner;
				//data->patches[i].zone = data->patches[co_owner[candidate]].zone;			
			}
			if (flagDone)
			{	//printf("good\n");
				currentPatch.xcor = xcoor;
				currentPatch.ycor = ycoor;
				int turtleIndex=turtleMap[ownerMap[i]];
				
					//we found the owner
				for (k=0; k<data->building_turtles[turtleIndex].patchCoords.size(); k++)
					{
						if ((xcoor==data->building_turtles[turtleIndex].patchCoords[k].xcor) &&
							(ycoor==data->building_turtles[turtleIndex].patchCoords[k].ycor))
						break;
					}
				if (k==data->building_turtles[turtleIndex].patchCoords.size())
						data->building_turtles[turtleIndex].patchCoords.push_back(currentPatch);
			}
			else
			{
				int turtleIndex;
				currentPatch.xcor = xcoor;
				currentPatch.ycor = ycoor;
				for (int p=0; p<co_owner.size(); p++)
				{
					data->patches[i].co_ownership.push_back(ownerMap[co_owner[p]]);
					//push those co owning patches into the turtles' patch list
					turtleIndex=turtleMap[ownerMap[co_owner[p]]];
				
					//we found the owner
					for (k=0; k<data->building_turtles[turtleIndex].patchCoords.size(); k++)
					{
						if ((xcoor==data->building_turtles[turtleIndex].patchCoords[k].xcor) &&
							(ycoor==data->building_turtles[turtleIndex].patchCoords[k].ycor))
						break;
					}
					if (k==data->building_turtles[turtleIndex].patchCoords.size())
						data->building_turtles[turtleIndex].patchCoords.push_back(currentPatch);
				}
				//printf("bad %d\n", co_ownership_config);
			}
		}
		
		/*if((nearRoad))
		{
			
			switch(direction)
			{
				int newParcelIdx;
				case 10:  //road is to the left, we need find the right parcel
					if ((newParcelIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 1:  //road is to the up, we need find the down parcel
					if ((newParcelIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 2:  //road is to the down, we need find the up parcel
					if ((newParcelIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 3:  //road is to the up down, we need find the down parcel
					if ((newParcelIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;
				case 20:  //road is to the right, we need find the left parcel
					if ((newParcelIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 11:  //road is to the left and up, we need find the right and down parcel
					if ((newParcelIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;
				case 12:  //road is to the left and down, we need find the right and up parcel
					if ((newParcelIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;
				case 21:  //road is to the right and up, we need find the left and down parcel
					if ((newParcelIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;
				case 22:  //road is to the right and down, we need find the left and up parcel
					if ((newParcelIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;	
				case 13:  //road is to the left and down and up, we need find the right parcel
					if ((newParcelIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 23:  //road is to the right and down and up, we need find the left parcel
					if ((newParcelIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 30:  //road is to the left right, we need find the down parcel
					if ((newParcelIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					if (!flagDone)
					{
						if ((newParcelIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
						{
							data->patches[i].owner = data->patches[newParcelIdx].owner;
							data->patches[i].zone = data->patches[newParcelIdx].zone;
							if (data->patches[i].owner!=-1)
							{
								flagDone = true;
							}
						}
					}
					break;
				case 31:  //road is to the left right and up, we need find the down parcel
					if ((newParcelIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;
				case 32:  //road is to the left right and down , we need find the up parcel
					if ((newParcelIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
					{
						data->patches[i].owner = data->patches[newParcelIdx].owner;
						data->patches[i].zone = data->patches[newParcelIdx].zone;
						if (data->patches[i].owner!=-1)
						{
							flagDone = true;
						}
					}
					break;	
				default:
				{
					if((leftIdx!=-1) && (data->patches[leftIdx].owner!=-1))// && (!data->patches[leftIdx].isRoad))
					{
						data->patches[i].owner = data->patches[leftIdx].owner;
						data->patches[i].zone = data->patches[leftIdx].zone;
						flagDone = true;
					}
					
					if((!flagDone) && (rightIdx!=-1) && (data->patches[rightIdx].owner!=-1))// && (!data->patches[rightIdx].isRoad))
					{
						data->patches[i].owner = data->patches[rightIdx].owner;
						data->patches[i].zone = data->patches[rightIdx].zone;
						
						flagDone = true;
					}
					
					if((!flagDone) && (upIdx!=-1) && (data->patches[upIdx].owner!=-1))// && (!data->patches[upIdx].isRoad))
					{
						data->patches[i].owner = data->patches[upIdx].owner;
						data->patches[i].zone = data->patches[upIdx].zone;
						
						flagDone = true;
					}
					
					if((!flagDone) && (downIdx!=-1) && (data->patches[downIdx].owner!=-1))// && (!data->patches[downIdx].isRoad))
					{
						data->patches[i].owner = data->patches[downIdx].owner;
						data->patches[i].zone = data->patches[downIdx].zone;
						
						flagDone = true;
					}
				}					
			}
			
			if (flagDone)
			{
				currentPatch.xcor = xcoor;
				currentPatch.ycor = ycoor;
				int turtleIndex=turtleMap[data->patches[i].owner];
				
					//we found the owner
				for (k=0; k<data->building_turtles[turtleIndex].patchCoords.size(); k++)
					{
						if ((xcoor==data->building_turtles[turtleIndex].patchCoords[k].xcor) &&
							(ycoor==data->building_turtles[turtleIndex].patchCoords[k].ycor))
						break;
					}
				if (k==data->building_turtles[turtleIndex].patchCoords.size())
						data->building_turtles[turtleIndex].patchCoords.push_back(currentPatch);
			}
		}//end of if near road*/
	}
	for (int m=0; m<data->patches.size(); m++)
	{
		data->patches[m].owner = ownerMap[m];
		data->patches[m].zone = zoneMap[m];
	}
	delete [] ownerMap;
	delete [] zoneMap;

	//touch it to keep it 4 connected especially when we have an empty patch in the middle 
	//of patches of the same kind
	/*int owner1,owner2,owner3,owner4;
	for (i=0; i<data->patches.size(); i++)
	{
		xcoor = data->patches[i].xcor;
		ycoor = data->patches[i].ycor;

		owner1=owner2=owner3=owner4=-1;
		if (data->patches[i].owner==-1)
		{
			if ((patchIdx=getPatchIndex(xcoor-1,ycoor))!=-1)
			{
				if (data->patches[patchIdx].owner!=-1)
					owner1 = data->patches[patchIdx].owner;
			}
			if ((patchIdx=getPatchIndex(xcoor+1,ycoor))!=-1)
			{
				if (data->patches[patchIdx].owner!=-1)
					owner2 = data->patches[patchIdx].owner;
			}
			if ((patchIdx=getPatchIndex(xcoor,ycoor-1))!=-1)
			{
				if (data->patches[patchIdx].owner!=-1)
					owner3 = data->patches[patchIdx].owner;
			}
			if ((patchIdx=getPatchIndex(xcoor,ycoor+1))!=-1)
			{
				if (data->patches[patchIdx].owner!=-1)
					owner4 = data->patches[patchIdx].owner;
			}
		}
		
		if ((owner1!=-1) && (owner1==owner2) && (owner2==owner3) && (owner3==owner4)  )
		{
			data->patches[i].owner=owner1;
		}
	}*/
}
bool cCSVFileReader::fixRoadIntersectionHelper(vector< deque<PatchCoor> > & roads, int idx, int direction)
{
	//direction 0 push_front 1 push_front
	//extend 0, evaluate the end it self, extend 1: extrapolate with in the range
	int j,k;
	bool found =false;
	int xcoor, ycoor;
	int xDiff=0, yDiff=0, xDiff_plus=0, yDiff_plus=0, xDiff_minus=0, yDiff_minus=0;
	PatchCoor push_tobe_item;
	if (direction==0) //push_front
	{
		
		/*xcoor = roads[idx][0].xcor;
		ycoor = roads[idx][0].ycor;
		if (roads[idx].size()>=2)
		{
			xDiff = xcoor - roads[idx][1].xcor;
			yDiff = ycoor - roads[idx][1].ycor;
		}
		
		for (j=0; j<roads.size(); j++)
		{
			if(j==idx) continue;
				
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					
					//cool we find one
					if (k<=1) // no choice we can go forward
					{
						
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[idx].push_front(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
					
					else if (k>=roads[j].size()-2)//no choice we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[idx].push_front(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
						
					//tough choice when we can choose both way
					if ((k>1) && (k<roads[j].size()-2))
					{
						xDiff_plus = roads[j][k+1].xcor -xcoor;
						yDiff_plus = roads[j][k+1].ycor -ycoor;
						
						xDiff_plus = roads[j][k-1].xcor -xcoor;
						yDiff_plus = roads[j][k-1].ycor -ycoor;
						
						if ((abs(xDiff-xDiff_plus) + abs(yDiff-yDiff_plus)) <= 
							(abs(xDiff-xDiff_minus) + abs(yDiff-yDiff_minus)))
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
							k++;
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						else
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
							k--;
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
				}
			}
			if (found)
				break;
			
		}/*buggy*/
		
		xcoor = roads[idx][0].xcor;
		ycoor = roads[idx][0].ycor;
		if (roads[idx].size()>=2)
		{
			xDiff = xcoor - roads[idx][1].xcor;
			yDiff = ycoor - roads[idx][1].ycor;
		}
		
		for (j=0; j<roads.size(); j++)
		{
			if(j==idx) continue;
				
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					
					//cool we find one
					if (k<=1) // no choice we can go forward
					{
						
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[idx].push_front(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
					
					else if (k>=roads[j].size()-2)//no choice we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[idx].push_front(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
						
					//tough choice when we can choose both way
					if ((k>1) && (k<roads[j].size()-2))
					{
						xDiff_plus = roads[j][k+1].xcor -xcoor;
						yDiff_plus = roads[j][k+1].ycor -ycoor;
						
						xDiff_plus = roads[j][k-1].xcor -xcoor;
						yDiff_plus = roads[j][k-1].ycor -ycoor;
						
						if ((abs(xDiff-xDiff_plus) + abs(yDiff-yDiff_plus)) <= 
							(abs(xDiff-xDiff_minus) + abs(yDiff-yDiff_minus)))
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
							k++;
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						else
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
							k--;
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
				}
			}
			if (found)
				break;
			
			
		}
		
		
	}
	else //push back
	{
		xcoor = roads[idx][roads[idx].size()-1].xcor;
		ycoor = roads[idx][roads[idx].size()-1].ycor;
		if (roads[idx].size()>=2)
		{
			xDiff = xcoor - roads[idx][roads[idx].size()-2].xcor;
			yDiff = ycoor - roads[idx][roads[idx].size()-2].ycor;
		}
		
		for (j=0; j<roads.size(); j++)
		{
			if(j==idx) continue;
				
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					
					//cool we find one
					if (k<=1) // no choice we can go forward
					{
						
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[idx].push_back(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_back(push_tobe_item);
						}
						found=true;
						break;
					}
					
					else if (k>=roads[j].size()-2)//no choice we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[idx].push_back(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_back(push_tobe_item);
						}
						found=true;
						break;
					}
						
					//tough choice when we can choose both way
					if ((k>1) && (k<roads[j].size()-2))
					{
						xDiff_plus = roads[j][k+1].xcor -xcoor;
						yDiff_plus = roads[j][k+1].ycor -ycoor;
						
						xDiff_plus = roads[j][k-1].xcor -xcoor;
						yDiff_plus = roads[j][k-1].ycor -ycoor;
						
						if ((abs(xDiff-xDiff_plus) + abs(yDiff-yDiff_plus)) <= 
							(abs(xDiff-xDiff_minus) + abs(yDiff-yDiff_minus)))
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_back(push_tobe_item);
							k++;
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[idx].push_back(push_tobe_item);
						}
						else
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_back(push_tobe_item);
							k--;
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[idx].push_back(push_tobe_item);
						}
						found=true;
						break;
					}
				}
			}
			if (found)
				break;
			
		}
	}
	return found;
}
void cCSVFileReader::fixRoadIntersection(vector< deque<PatchCoor> > & roads, int range)
{
	
	//for the end point and start point, search for the the appearence in the other road segment, then extend them
	int i,j,k,m;
	int xcoor, ycoor;
	PatchCoor push_tobe_item;
	bool found=false;
	int xDiff, yDiff;
	int patchIdx;
	bool push_in=false;
	for (i=0; i<roads.size(); i++)
	{
		//printf("starting %d\n", i);
		//first for end point
		
		if (roads[i].size()<4) continue;
		found=false;
		xcoor = roads[i][0].xcor;
		ycoor = roads[i][0].ycor;
		for (j=0; j<roads.size(); j++)
		{
			if(j==i) continue;
			if (roads[j].size()<4) continue;	
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					//printf("road:%d start --- road:%d, %d\n", i,j,k);
					//cool we find one
					if (k!=roads[j].size()-1) //we can go forward
					{
						
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[i].push_front(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[i].push_front(push_tobe_item);
						}
						
						found=true;
						break;
					}
					
					else if (k!=0)//no we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[i].push_front(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[i].push_front(push_tobe_item);
						}
						
						found=true;
						break;
					}
						
					
				}
			}
			if (found)
				break;
		}
		
		if (!found) //then extend along the end gradient direction
		{
			push_in=false;
			
			if (roads[i].size()>=1)
				{
					xDiff = xcoor - roads[i][1].xcor;
					yDiff = ycoor - roads[i][1].ycor;
					//for (k=1; k<range; k++)
					
					push_tobe_item.xcor = xcoor+xDiff;
					push_tobe_item.ycor = ycoor+yDiff;
						
					if ((patchIdx=getPatchIndex(push_tobe_item.xcor, push_tobe_item.ycor))!=-1)
						{
							if (data->patches[patchIdx].isRoad) //since we didn't find previously, this must be a primary road
							{
								roads[i].push_front(push_tobe_item);
								found = true;
								//push_in=true;
							}
							/*push_tobe_item.xcor += xDiff;
							push_tobe_item.ycor += yDiff;
							if ((patchIdx=getPatchIndex(push_tobe_item.xcor, push_tobe_item.ycor))!=-1)
							{
								if (data->patches[patchIdx].isRoad) //since we didn't find previously, this must be a primary road
								{
									roads[i].push_front(push_tobe_item);
									found = true;
								//push_in=true;
								}
							}*/
						}
					
				}	
		}
	}

	//then end point
	for (i=0; i<roads.size(); i++)
	{	
		found=false;
		if (roads[i].size()<4) continue;
		//end point
		xcoor = roads[i][roads[i].size()-1].xcor;
		ycoor = roads[i][roads[i].size()-1].ycor;
		for (j=0; j<roads.size(); j++)
		{
			if(j==i) continue;
			if (roads[j].size()<4) continue;	
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					printf("road:%d end --- road:%d, %d\n", i,j,k);
					//cool we find one
					if (k!=roads[j].size()-1) //we can go forward
					{
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[i].push_back(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[i].push_back(push_tobe_item);
						}
					
						found=true;
						break;
					}
					
					else if (k!=0)//no we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[i].push_back(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[i].push_back(push_tobe_item);
						}
						
						found=true;
						break;
					}
						
					
				}
			}
			if (found)
				break;
			
		}
		if (!found) //then extend along the end gradient direction
		{
			push_in = false;
			
				if (roads[i].size()>=1)
				{
					xDiff = xcoor - roads[i][roads[i].size()-2].xcor;
					yDiff = ycoor - roads[i][roads[i].size()-2].ycor;
					
					push_tobe_item.xcor = xcoor+xDiff;
					push_tobe_item.ycor = ycoor+yDiff;
					
					if ((patchIdx=getPatchIndex(push_tobe_item.xcor, push_tobe_item.ycor))!=-1)
					{
							if (data->patches[patchIdx].isRoad)
							{
								roads[i].push_back(push_tobe_item);
								found = true;
								//push_in = true;
								/*if ((patchIdx=getPatchIndex(push_tobe_item.xcor, push_tobe_item.ycor))!=-1)
								{
									if (data->patches[patchIdx].isRoad)
									{
										push_tobe_item.xcor += xDiff;
										push_tobe_item.ycor += yDiff;
										roads[i].push_back(push_tobe_item);
									}
								}*/
							}
					}
					
				}
		}
	}
	/*old solution*/
}

void cCSVFileReader::offsetPrimaryRoads(vector< deque<PatchCoor> > & roads, double offset)
{
	for (vector< deque<PatchCoor> >::iterator iter_road = roads.begin();
		 iter_road!=roads.end();
		 iter_road++)
	{
		for (deque<PatchCoor>::iterator iter_patch=iter_road->begin();
			 iter_patch!=iter_road->end();
			 iter_patch++)
		{
			iter_patch->xcor+=(int)offset;
			iter_patch->ycor+=(int)offset;
			if (iter_patch->xcor<0) 
				iter_patch->xcor=0;
			if (iter_patch->ycor<0) 
				iter_patch->ycor=0;
			if (iter_patch->xcor>data->xmax*2)
				iter_patch->xcor=data->xmax*2;
			if (iter_patch->ycor>data->ymax*2)
				iter_patch->ycor=data->ymax*2;
		}
	}
}

//this is only for primary road connector fix 
void cCSVFileReader::fixRoadIntersection2(vector< deque<PatchCoor> > & roads, int range)
{
	
	//for the end point and start point, search for the the appearence in the other road segment, the extend them
	int i,j,k;
	int xcoor, ycoor;
	PatchCoor push_tobe_item;
	bool found=false;
	int xDiff, yDiff;
	int patchIdx;
	bool push_in=false;
	int m,n;
	bool isCenterOK;
	PatchCoor center;
	for (i=0; i<roads.size(); i++)
	{
		//printf("starting %d\n", i);
		//first for end point
		
		if (roads[i].size()<4) continue;
		found=false;
		xcoor = roads[i][0].xcor;
		ycoor = roads[i][0].ycor;
		for (j=0; j<roads.size(); j++)
		{
			if(j==i) continue;
			if (roads[j].size()<4) continue;	
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					//cool we find one
					if (k!=roads[j].size()-1) //we can go forward
					{
						
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[i].push_front(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[i].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
					
					else if (k!=0)//no we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[i].push_front(push_tobe_item);
						
						k--;
						if (k!=0)
						{
					push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[i].push_front(push_tobe_item);
						}
						found=true;
						break;
					}
						
					
				}
			}
			if (found)
				break;
		}
		
	
		if (!found) //then extend along the end gradient direction
		{
			
			//then we will do a exhausting search for neighbouring primary connector backbone
			//first extend the road a bit and search 
			
			
			if (roads[i].size()>=1)
				{
					xDiff = xcoor - roads[i][1].xcor;
					yDiff = ycoor - roads[i][1].ycor;
					//for (k=1; k<range; k++)
				
					center.xcor = xcoor+xDiff;
					center.ycor = ycoor+yDiff;
						
					if ((patchIdx=getPatchIndex(center.xcor, center.ycor))!=-1)
						{
							isCenterOK = true;
							
						}
					else isCenterOK = false;					
				}	
			if (isCenterOK)
			{
				for (m=-1; m<=1; m++)
				{
					for (n=-1; n<=1; n++)
					{
						xcoor = center.xcor + m;
						ycoor = center.ycor + n;
						if ((getPatchIndex(xcoor, ycoor))!=-1) //valid
						{
							for (j=0; j<roads.size(); j++) //we want to put in one more points
							{
								if(j==i) continue;
								{
									for (k=0; k<roads[j].size(); k++)
									{
										if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
										{
											
											//cool we find one
											//now we need to push back the (xcoor,ycoor)
											push_tobe_item.xcor = xcoor;
											push_tobe_item.ycor = ycoor;
											roads[i].push_front(push_tobe_item);
											if (k<roads[j].size()-2) //we can go forward
											{
												push_tobe_item.xcor = roads[j][k+1].xcor;
												push_tobe_item.ycor = roads[j][k+1].ycor;
												roads[i].push_front(push_tobe_item);
												k++;
												push_tobe_item.xcor = roads[j][k+1].xcor;
												push_tobe_item.ycor = roads[j][k+1].ycor;
												roads[i].push_front(push_tobe_item);
											}
											else if (k>=2)
											{
												push_tobe_item.xcor = roads[j][k-1].xcor;
												push_tobe_item.ycor = roads[j][k-1].ycor;
												roads[i].push_front(push_tobe_item);
												k--;
												push_tobe_item.xcor = roads[j][k-1].xcor;
												push_tobe_item.ycor = roads[j][k-1].ycor;
												roads[i].push_front(push_tobe_item);
											}	
											found = true;
											break;											
										}
									}
								}
								if (found)
									break; //break j
							}
						}
						if (found)
							break;
					}//end of n
					if (found)
						break;
				}//end of m	
			}//end of isCenterOK
		}	//end of (!found) 		
		
	}
	//then end point
	for (i=0; i<roads.size(); i++)
	{	
		found=false;
		if (roads[i].size()<4) continue;
		//end point
		xcoor = roads[i][roads[i].size()-1].xcor;
		ycoor = roads[i][roads[i].size()-1].ycor;
		for (j=0; j<roads.size(); j++)
		{
			if(j==i) continue;
			if (roads[j].size()<4) continue;	
			for (k=0; k<roads[j].size(); k++)
			{
				if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
				{
					
					//cool we find one
					if (k!=roads[j].size()-1) //we can go forward
					{
						push_tobe_item.xcor = roads[j][k+1].xcor;
						push_tobe_item.ycor = roads[j][k+1].ycor;
						roads[i].push_back(push_tobe_item);
						k++;
						if (k!=roads[j].size()-1)
						{
							push_tobe_item.xcor = roads[j][k+1].xcor;
							push_tobe_item.ycor = roads[j][k+1].ycor;
							roads[i].push_back(push_tobe_item);
						}
						found=true;
						break;
					}
					
					else if (k!=0)//no we have to backward
					{
						push_tobe_item.xcor = roads[j][k-1].xcor;
						push_tobe_item.ycor = roads[j][k-1].ycor;
						roads[i].push_back(push_tobe_item);
						
						k--;
						if (k!=0)
						{
							push_tobe_item.xcor = roads[j][k-1].xcor;
							push_tobe_item.ycor = roads[j][k-1].ycor;
							roads[i].push_back(push_tobe_item);
						}
						
						found=true;
						break;
					}
						
					
				}
			}
			if (found)
				break;
			
		}
		if (!found) //then extend along the end gradient direction
		{
			
			//then we will do a exhausting search for neighbouring primary connector backbone
			//first extend the road a bit and search 
			
			
			if (roads[i].size()>=1)
				{
					xDiff = xcoor - roads[i][roads[i].size()-2].xcor;
					yDiff = ycoor - roads[i][roads[i].size()-2].ycor;
					//for (k=1; k<range; k++)
				
					center.xcor = xcoor+xDiff;
					center.ycor = ycoor+yDiff;
						
					if ((patchIdx=getPatchIndex(center.xcor, center.ycor))!=-1)
						{
							isCenterOK = true;
							
						}
					else isCenterOK = false;					
				}	
			if (isCenterOK)
			{
				for (m=-1; m<=1; m++)
				{
					for (n=-1; n<=1; n++)
					{
						xcoor = center.xcor + m;
						ycoor = center.ycor + n;
						if ((getPatchIndex(xcoor, ycoor))!=-1) //valid
						{
							for (j=0; j<roads.size(); j++) //we want to put in one more points
							{
								if(j==i) continue;
								{
									for (k=0; k<roads[j].size(); k++)
									{
										if ((roads[j][k].xcor == xcoor) && (roads[j][k].ycor == ycoor))
										{
											
											//cool we find one
											//now we need to push back the (xcoor,ycoor)
											push_tobe_item.xcor = xcoor;
											push_tobe_item.ycor = ycoor;
											roads[i].push_back(push_tobe_item);
											if (k<roads[j].size()-2) //we can go forward
											{
												push_tobe_item.xcor = roads[j][k+1].xcor;
												push_tobe_item.ycor = roads[j][k+1].ycor;
												roads[i].push_back(push_tobe_item);
												k++;
												push_tobe_item.xcor = roads[j][k+1].xcor;
												push_tobe_item.ycor = roads[j][k+1].ycor;
												roads[i].push_back(push_tobe_item);
											}
											else if (k>=2)
											{
												push_tobe_item.xcor = roads[j][k-1].xcor;
												push_tobe_item.ycor = roads[j][k-1].ycor;
												roads[i].push_back(push_tobe_item);
												k--;
												push_tobe_item.xcor = roads[j][k-1].xcor;
												push_tobe_item.ycor = roads[j][k-1].ycor;
												roads[i].push_back(push_tobe_item);
											}	
											found = true;
											break;											
										}
									}
								}
								if (found)
									break; //break j
							}
						}
						if (found)
							break;
					}//end of n
					if (found)
						break;
				}//end of m	
			}//end of isCenterOK
		}	//end of (!found) 		
		
	}
	/*old solution*/
}
void cCSVFileReader::fixNotFourConnectedParcel()
{
	bool found;
	vector<PatchCoor> tobe_deleted;
	int owner_this;
	PatchCoor patchItem;
	int xcoor,ycoor;
	int otherSideIdx;
	for (int i=0; i<data->building_turtles.size(); i++)
	{
	
		owner_this = data->building_turtles[i].id;
		tobe_deleted.clear();
		for (int j=0; j<data->building_turtles[i].patchCoords.size(); j++)
		{
			//for all the patches inside this parcel
			xcoor = data->building_turtles[i].patchCoords[j].xcor;
			ycoor = data->building_turtles[i].patchCoords[j].ycor;
			
			found = false;
			int patch_index;
			if ((patch_index=getPatchIndex(xcoor, ycoor))!=-1)
			{
				if (data->patches[patch_index].co_ownership.size()>0)
				{
					continue;
				}
			}
			
			if ((otherSideIdx=getPatchIndex(xcoor-1,ycoor))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner==owner_this) //the other side patch is belong to the different parcel
				{
					found=true;
					continue;
				}
			}
			
			
			if ((otherSideIdx=getPatchIndex(xcoor,ycoor+1))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner==owner_this) //the other side patch is belong to the different parcel
				{
					found=true;
					continue;
				}
			}
			
			
			if ((otherSideIdx=getPatchIndex(xcoor+1,ycoor))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner==owner_this) //the other side patch is belong to the different parcel
				{
					found=true;
					continue;
				}
			}
			
			if ((otherSideIdx=getPatchIndex(xcoor,ycoor-1))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner==owner_this) //the other side patch is belong to the different parcel
				{
					found=true;
					continue;
				}
			}
			if (!found) //not 4 connected;
			{
				patchItem.xcor =xcoor;
				patchItem.ycor =ycoor;
				tobe_deleted.push_back(patchItem);
			}
		}
		for (int k=0; k<tobe_deleted.size(); k++)
		{
			for (vector<PatchCoor>::iterator iter_patch = data->building_turtles[i].patchCoords.begin();
				iter_patch!= data->building_turtles[i].patchCoords.end(); iter_patch++)
			{
				if ((iter_patch->xcor == tobe_deleted[k].xcor) && (iter_patch->ycor == tobe_deleted[k].ycor) )
				{
					data->building_turtles[i].patchCoords.erase(iter_patch);
					break;
				}
			}
		}
	}
}
//this function is called to make sure there is no hole any of the parcel
void cCSVFileReader::fixHolesInParcels()
{
	int i,j;
	int patch_index;
	PatchData patch_item;
	PatchCoor patch_coor;
	int owner_id1,owner_id2,owner_id3,owner_id4;
	int orig_patch_index;
	for (i=0; i<data->size; i++)
	{
		for (j=0; j<data->size; j++)
		{
			if ((patch_index=getPatchIndex(i,j))!=-1)
			{
				// check to see if it is empty
				owner_id1=-1;
				owner_id2=-1;
				owner_id3=-1;
				owner_id4=-1;
				orig_patch_index=patch_index;
				patch_coor.xcor=i;
				patch_coor.ycor=j;
				if (data->patches[patch_index].owner==-1)
				{
					if ((patch_index=getPatchIndex(i-1,j))!=-1)
					{ 
					    //if (data->patches[patch_index].owner!=-1)
							owner_id1=data->patches[patch_index].owner;
					}
					if ((patch_index=getPatchIndex(i+1,j))!=-1)
					{
						//if (data->patches[patch_index].owner!=-1)
							owner_id2=data->patches[patch_index].owner;
					}
					if ((patch_index=getPatchIndex(i,j-1))!=-1)
					{
						//if (data->patches[patch_index].owner!=-1)
							owner_id3=data->patches[patch_index].owner;
					}
					if ((patch_index=getPatchIndex(i,j+1))!=-1)
					{
						//if (data->patches[patch_index].owner!=-1)
							owner_id4=data->patches[patch_index].owner;
					}
					
					//we came here then this the we want.
					if ((owner_id1!=-1)&&(owner_id1==owner_id2)&&(owner_id2==owner_id3)
						&&(owner_id3==owner_id4))
					{
						data->patches[orig_patch_index].owner=owner_id1;
						int turtle_id = turtleMap[owner_id1];
						data->building_turtles[turtle_id].patchCoords.push_back(patch_coor);
					}
				}
			}
		}
	}
		
}


//there are some overlap parcels, we don't want 
//to get rid of them, we need to go back the patch raster map
//to find if the owner of this parcel is indeed the id of this turtle
void cCSVFileReader::fixOverlapParcels()
{
	int i;
	int patchIdx;
	int xcoor,ycoor;
	int turtle_id;
	vector<PatchCoor>::iterator patchIter;
	vector<int> deleteList;
	for (i=0; i<data->building_turtles.size(); i++)
	{
		turtle_id = data->building_turtles[i].id;
		for(patchIter=data->building_turtles[i].patchCoords.begin(); 
			patchIter<data->building_turtles[i].patchCoords.end();
			patchIter++)
		{
			xcoor = patchIter->xcor;
			ycoor = patchIter->ycor;
			
			if ((patchIdx=getPatchIndex(xcoor,ycoor))!=-1)
			{
				if (data->patches[patchIdx].owner!=turtle_id) //belong to someone else
				{
					//data->building_turtles[i].patchCoords.erase(patchIter);
					deleteList.push_back(i);
					
					//data->building_turtles[i].patchCoords.clear();
					break;					
				}
			}
		}
	}
	
	for (i=0; i<deleteList.size(); i++)
	{
		printf("turtle %d deleted\n", deleteList[i]);
		data->building_turtles[deleteList[i]].patchCoords.clear();
	}
	for (i=0; i<data->patches.size(); i++)
	{
		data->patches[i].owner=-1;
	}
	for (i=0; i<data->building_turtles.size(); i++)
	{
		turtle_id = data->building_turtles[i].id;
		for(patchIter=data->building_turtles[i].patchCoords.begin(); 
			patchIter<data->building_turtles[i].patchCoords.end();
			patchIter++)
		{
			xcoor = patchIter->xcor;
			ycoor = patchIter->ycor;
			
			if ((patchIdx=getPatchIndex(xcoor,ycoor))!=-1)
			{
				data->patches[patchIdx].owner = turtle_id;
			}
		}
	}
}
char * cCSVFileReader::getDataAtPos(char * buf, int pos, long int buffer_size)
{
	/*int i=0;
	char seps[]   = ",\n";
	char * token;

	token = strtok( buf, seps );
	while( (token != NULL) && (i<pos))
	{
  
      token = strtok( NULL, seps );
	  i++;
	}
	return token;
	*/
	int i=0;
	
	char * ret=new char[buffer_size];
	memset(ret,0, buffer_size);
	char *ptr = buf;
	
	bool done= false;
	//printf("OK:\n");
	int j=0;
	while (!done)
	{
		//printf("%d",ptr);
		while ((*ptr==',')||(*ptr==0))
			{	
				ptr++;
				i++;
			}

		if (i==pos)
		{
			ret[j] = *ptr;
		}
		
		if (i>=(pos+1))
		{
			//printf("New i:%d, %s\n",i,ret);
			done = true;
		}
		ptr++;
		if (i==pos)
			j++;
		
	}
	return ret;
}

inline void cCSVFileReader::getRidOfQuot(char * outStr, char *inStr, long int buffer_size)
{
	//printf("\n\n%s", inStr);
	char *tmpStr= new char[buffer_size];
	memset(tmpStr,0,buffer_size);
	strcpy(tmpStr,inStr);
	strncpy(outStr,tmpStr+1,strlen(tmpStr)-2);
	delete []inStr;
	delete [] tmpStr;
}


void cCSVFileReader::getStrBeforeBra(char * outStr, const char *inStr, long int str_size)
{
	char *tmpStr =  new char[str_size];
	memset(tmpStr,0,str_size);
	strcpy(tmpStr,inStr);
	char *token;
	token = strtok(tmpStr," "); //find where is the space, which is right before the information we need
	token = strtok( NULL, " ");
	strncpy(outStr,token,strlen(token)-1);
	delete [] tmpStr;
}
int cCSVFileReader::extractRoadSegmentInfo(deque<PatchCoor> &patches, const char *inStr, bool waterFront, bool primaryRoad)
{
	//printf("OK in\n");
	PatchCoor patch_coor_item;
	char seps[] = "{}";
	char tmpStr[MAXBUFF_SIZE];
	memset(tmpStr,0,MAXBUFF_SIZE);
	strcpy(tmpStr,inStr);
	char *token;
	patches.clear();
	token = strtok( tmpStr, seps );
	char tempStr[20];
	int i=0;
	int patchIndex;
	int patchOwner;
	int prevXCoor, prevYCoor; //coordinate of previous pushed_in patches
	
	while ((token!=NULL)&&(strcmp(token,"]")))			//since the last one we got will be "}"
	{
		token = strtok(NULL,seps);
		sscanf(token,"%s %d %d",tempStr, &patch_coor_item.xcor,&patch_coor_item.ycor);
		patch_coor_item.xcor += data->xmax;
		//patch_coor_item.ycor += data->ymax;
		patch_coor_item.ycor = -(patch_coor_item.ycor-data->ymax);
		if (i!=0)
		{
			if ((abs(patch_coor_item.xcor-prevXCoor)>(data->size/4)) ||
				(abs(patch_coor_item.ycor-prevYCoor)>(data->size/4)) )
			{
				//wrap around
				continue;
			}
		}
		
		if (!waterFront)
		{
			/*if ((patchIndex=getPatchIndex(patch_coor_item.xcor,patch_coor_item.ycor))!=-1)
			{
				//printf("hehe%d\n",patchIndex);
				if (data->patches[patchIndex].terrain == 1)
				{
					printf("road on water\n");
					return i;
				}
				if ((patchOwner = data->patches[patchIndex].owner)!=-1)
				{
					//it belong to a building turtle;
				}
				else
				{
					//we probably want to fill it with surrounding parcels
				}
			}*/
		}
		/*if (primaryRoad)
		{
			patch_coor_item.xcor+=10;
			if (patch_coor_item.xcor>2*data->xmax) patch_coor_item.xcor=2*data->xmax-1;
			patch_coor_item.ycor+=10;
			if (patch_coor_item.ycor>2*data->ymax) patch_coor_item.ycor=2*data->ymax-1;	
		}*/
		if (//(patch_coor_item.xcor!=0) && (patch_coor_item.xcor!=2*data->xmax) &&
			//(patch_coor_item.ycor!=0) && (patch_coor_item.ycor!=2*data->ymax) &&
		    ((prevXCoor!=patch_coor_item.xcor) ||(prevYCoor!=patch_coor_item.ycor))
		   )
		{
			patches.push_back(patch_coor_item);
			if ((patchIndex=getPatchIndex(patch_coor_item.xcor, patch_coor_item.ycor))!=-1)
			{
				if (data->patches[patchIndex].connectivity>=3)
				{
					patches.push_back(patch_coor_item);
					patches.push_back(patch_coor_item);
					//patches.push_back(patch_coor_item);
					//patches.push_back(patch_coor_item);
					//patches.push_back(patch_coor_item);
					//patches.push_back(patch_coor_item);
				}
			}
			prevXCoor = patch_coor_item.xcor;
			prevYCoor = patch_coor_item.ycor;		
			i++;
		}
		else
		{
			if (waterFront)
			{
				//if (patch_coor_item.xcor==0) patch_coor_item.xcor=1;
				//if (patch_coor_item.ycor==0) patch_coor_item.ycor=1;	
				//if (patch_coor_item.xcor==2*data->xmax) patch_coor_item.xcor=2*data->xmax-1;	
				//if (patch_coor_item.ycor==2*data->ymax) patch_coor_item.ycor=2*data->ymax-1;
				patches.push_back(patch_coor_item);	
				prevXCoor = patch_coor_item.xcor;
				prevYCoor = patch_coor_item.ycor;		
				i++;
			}
		}
		token = strtok(NULL,seps);
	}
	//printf("OK out\n");
	return 0;
}

void cCSVFileReader::processSequenceSegements(const char * dataStr,vector< deque<PatchCoor> > & output, long int buffer_size, bool waterFront, bool primaryRoad)
{
	//the input string is in the format of [[{patch x y} ...] [...] ...]
	//if  can convert each road string in to the format [{patch x y} ...]
	//then I can call the above function to get one sequence
	char *tmpStr = new char[buffer_size];
	memset(tmpStr,0,buffer_size);
	strcpy(tmpStr, dataStr);
	char *ptLeft, *ptRight, *curPt;
	curPt =strchr(dataStr,'[');
	curPt++; //we don't need the first [
	//printf("\n%s\n", dataStr);
	deque<PatchCoor> sequence_item;
	int breakIdx;
	do
	{	
		ptLeft = strchr(curPt,'[');		
		curPt=ptLeft;
		if (ptLeft==NULL)   //if we cannot find open [, stops here
			break;
		ptRight = strchr(curPt, ']');
		curPt=ptRight;
		if ((ptLeft!=NULL) && (ptRight!=NULL) )//&& ((ptRight-ptLeft)>2))
		{
			memset(tmpStr,0, buffer_size);
			strncpy(tmpStr,ptLeft, (ptRight-ptLeft+1));
			//printf("%s\n", tmpStr);
			if (strlen(tmpStr)<=4)
				continue;
			sequence_item.clear();
			if ((breakIdx=extractRoadSegmentInfo(sequence_item, tmpStr, waterFront,primaryRoad))!=0)
			{
				//wrap around
				//more work to be done to get the second half.
			}
			
			if ((breakIdx==0) || (breakIdx>5)) 
			{
				/*for (int p=0; p<sequence_item.size(); p++)
				{
					printf("(%d,%d)->",sequence_item[p].xcor, sequence_item[p].ycor);
				}
				printf("\n");*/
				output.push_back(sequence_item);
			}
		}
	}while(ptLeft != NULL);
	delete [] tmpStr;
	
}


void cCSVFileReader::extractPatchInfo(vector<PatchCoor> &patches, const char *inStr, int TurtleID)
{
	PatchCoor patch_coor_item;
	char seps[] = "[]";
	char tmpStr[MAXBUFF_SIZE];
	memset(tmpStr,0,MAXBUFF_SIZE);
	strcpy(tmpStr,inStr);
	char *token;
	patches.clear();
	token = strtok( tmpStr, seps );
	int minSum=65535;
	int minX=65535;
	int minIndex=-1;
	int i=0;
	while ((token!=NULL)&&(strcmp(token,"}")))			//since the last one we got will be "}"
	{
		token = strtok(NULL,seps);
		sscanf(token,"%d %d",&patch_coor_item.xcor,&patch_coor_item.ycor);
		patch_coor_item.xcor += data->xmax;
		//patch_coor_item.ycor += data->ymax;
		patch_coor_item.ycor = -(patch_coor_item.ycor-data->ymax);
		if (minSum>(patch_coor_item.xcor+patch_coor_item.ycor))
		{
			minSum=patch_coor_item.xcor+patch_coor_item.ycor;
			minX = patch_coor_item.xcor;
			minIndex= i;
		}
		if ((minSum==(patch_coor_item.xcor+patch_coor_item.ycor))&&(minX<patch_coor_item.xcor))
		{
			minX = patch_coor_item.xcor;
			minIndex= i;
		}
		
		patches.push_back(patch_coor_item);

		//update PatchMap
		data->PatchMap[patch_coor_item.xcor*data->size + patch_coor_item.ycor] = TurtleID;
		i++;
		token = strtok(NULL,seps);
	}
	int tempX=patches[0].xcor;
	int tempY=patches[0].ycor;

	patches[0].xcor =patches[minIndex].xcor;
	patches[0].ycor =patches[minIndex].ycor;
	patches[minIndex].xcor = tempX;
	patches[minIndex].ycor = tempY;
	
}

void cCSVFileReader::preparePatchMap()
{
	int i,j,size;
	size = 2*data->xmax+1;
	for (i=0;i<size;i++)
		for (j=0;j<size;j++)
		{
			data->PatchMap.push_back(0);
		}
	data->size=size;
}

void cCSVFileReader::findMaxPossibleSquare()
{

	bool flag=true;
	bool flag_PossibleTopLeft =true;
	int sideLength = data->size;
	int tempSize = 1;//we should start with size 1;
	int tempSize_MAX=0; //that's the max size of all possible square, the return value 
	PatchCoor tempCoor, //possible topleft point coor
              tempCoor_MAX, //return value, the topleft point we pick for the MAX size, 
			  blockerCoor;  
	int curID;
	int m;
	vector<Rect> DeletedRegion;
	Rect region_item;
	//we should run this algorithm for all the turtles
	int i,j;
	vector<TurtleData>::iterator turtle_iter;
	vector<PatchCoor>::iterator patch_iter;
	vector<Rect>::iterator region_iter;
	
	int currRoadCost=0;  //we want our patch to be the side of the road, so we need to calc the nearby road patch num;
	int tempRoadCost=0;

	for (turtle_iter=data->building_turtles.begin();
		turtle_iter<data->building_turtles.end();
		turtle_iter++)
		{
			curID = turtle_iter->id;	//turtle's ID;
			tempSize_MAX=0;
			DeletedRegion.clear();
			//but we need check all other possible topleft
			patch_iter = turtle_iter->patchCoords.begin();
			do
			{
				tempCoor.xcor = patch_iter->xcor;
				tempCoor.ycor = patch_iter->ycor;
				flag_PossibleTopLeft = true;
				if (!DeletedRegion.empty())
				{
					
					region_iter = DeletedRegion.begin();
					do
					{
						//within any of the deleted region?
						if ((tempCoor.xcor>=region_iter->TopLeft.xcor) &&
							(tempCoor.ycor>=region_iter->TopLeft.ycor) &&
							(tempCoor.xcor<=region_iter->BottomRight.xcor) &&
							(tempCoor.ycor<=region_iter->BottomRight.ycor) 
							)
						{
							flag_PossibleTopLeft =false;
							break;
						}
					region_iter ++;
					}while (region_iter!=DeletedRegion.end());
				}
				
				if (flag_PossibleTopLeft)
				{
					tempSize = 0;
					currRoadCost =0;
					//this while loop find the max possible square size for the particular topleft coord
					
					while (tempSize<5)
					{
						flag =true;

						//loop for all the edge points;
						for (i=tempCoor.xcor;i<=tempCoor.xcor+tempSize;i++)
							for (j=tempCoor.ycor;j<=tempCoor.ycor+tempSize;j++)
							{
								//compare the patch id with the map to see if it is still inside this turtle
						 
								if (curID!=data->PatchMap[i*sideLength+j])
								{
									flag=false;
									blockerCoor.xcor = i;
									blockerCoor.ycor = j;
									break;
								}
							}
						//if the flag remain true that means for this size, it can fit, then we should increase the size
						//to try;
						if (flag) tempSize++;
						else break;
					}
					//if the size is greater than our current size we need to replace it
					if (tempSize > tempSize_MAX)
					{
						tempSize_MAX = tempSize;
						tempCoor_MAX.xcor = tempCoor.xcor;
						tempCoor_MAX.ycor = tempCoor.ycor;
						currRoadCost = calcRoadCost(tempCoor.xcor,tempCoor.ycor,tempSize);
					}
					
					if (tempSize == tempSize_MAX) 
					{
						tempRoadCost = 	calcRoadCost(tempCoor.xcor,tempCoor.ycor,tempSize);
						if (tempRoadCost > currRoadCost)
						{
							tempSize_MAX = tempSize;
							tempCoor_MAX.xcor = tempCoor.xcor;
							tempCoor_MAX.ycor = tempCoor.ycor;
							currRoadCost = tempRoadCost;
						}

					}

					if (tempSize<5)  //that means there are patches blocked our 5*5 square, we now know the blocker
					{
						region_item.TopLeft.xcor = tempCoor.xcor;
						region_item.TopLeft.ycor = tempCoor.ycor;
						region_item.BottomRight.xcor = blockerCoor.xcor;
						region_item.BottomRight.ycor = blockerCoor.ycor;

					//	DeletedRegion.push_back(region_item);
					}
					

				}	//end of if (flag_PossibleTopLeft)
				patch_iter++;
			}while (patch_iter!=turtle_iter->patchCoords.end());
			

			turtle_iter->maxSquareSize= tempSize_MAX;
			turtle_iter->TopLeftCoor_MaxSquare.xcor = tempCoor_MAX.xcor;
			turtle_iter->TopLeftCoor_MaxSquare.ycor = tempCoor_MAX.ycor;
		}	
}

//this function remap the population of each building turtle
//void cCSVFileReader::reMapPopulation()
//this function tries to find the possible space division by 2*2 patches
void cCSVFileReader::findSplitSquares()
{
	int curID;
	//we should run this algorithm for all the turtles
	int i,j;
	Square square_item;
	int sideLength = data->size;
	bool flag; //to indicate if it is a possible topleft corner 
	PatchCoor tempCoor; //possible topleft point coor
	vector<TurtleData>::iterator turtle_iter;
	vector<PatchCoor>::iterator patch_iter;
	
	vector<int> PatchMap=data->PatchMap;

	for (turtle_iter=data->building_turtles.begin();
		turtle_iter<data->building_turtles.end();
		turtle_iter++)
		{
			turtle_iter->SplitSquares.clear();

			curID = turtle_iter->id;	//turtle's ID;
			
			patch_iter = turtle_iter->patchCoords.begin(); //from the first one
			/* then loop for all possible topleft corner point*/
			do
			{
				flag = true;
				tempCoor.xcor = patch_iter->xcor;
				tempCoor.ycor = patch_iter->ycor;
				//first check to see if it is a valid top left corner for a 2*2 patch.
				//if it is, then put that square into the splitsquares collection, and mark the region used.
				//if it is not, keep going until we use up all the possible patches.
				for (i=tempCoor.xcor;i<=tempCoor.xcor+1;i++)
					for (j=tempCoor.ycor;j<=tempCoor.ycor+1;j++)
						{
								//compare the patch id with the map to see if it is still inside this turtle
						 
								if (curID!=PatchMap[i*sideLength+j])
								{
									flag=false;
									break;
								}
						}

				/* this is the right topleft corner*/
				if (flag) {
						square_item.size = 2;
						square_item.xcor = tempCoor.xcor;
						square_item.ycor = tempCoor.ycor;
						turtle_iter->SplitSquares.push_back(square_item);
						//mark it is used;
						PatchMap[square_item.xcor*sideLength+square_item.ycor]=-2;
						PatchMap[square_item.xcor*sideLength+square_item.ycor+1]=-2;
						PatchMap[(square_item.xcor+1)*sideLength+square_item.ycor]=-2;
						PatchMap[(square_item.xcor+1)*sideLength+square_item.ycor+1]=-2;
					}

				patch_iter++;
			}while (patch_iter!=turtle_iter->patchCoords.end());
		}
}
//this function calculate the number of road patch just outside of the given patch
//x,y is the coordinate of the top left corner, size is the side length of the given patch.
int cCSVFileReader::calcRoadCost(int x, int y, int size)
{
	int i,j;
	int cost=0;
	int sideLength = data->size;
	//first do the two horizontal lines
	if(x>=1) {
		i=x-1;
		for (j=y;j<y+size;j++)
		{
			if (data->PatchMap[i*sideLength+j]==-1)
				cost++;
		}
	}

	if(x<sideLength-size)
	{
		i=x+size;
		for (j=y;j<y+size;j++)
		{
			if (data->PatchMap[i*sideLength+j]==-1)
				cost++;
		}
	}

	//then two vertical lines
	if (y>=1) {
		j=y-1;
		for (i=x; i<x+size; i++)
		{
			if (data->PatchMap[i*sideLength+j]==-1)
				cost++;
		}

	}
	if (y<sideLength-size)
	{
		j=y+size;
		for (i=x; i<x+size; i++)
		{
			if (data->PatchMap[i*sideLength+j]==-1)
				cost++;
		}
	}
	return cost;
	
}

void cCSVFileReader::ScaleTheElevation(int scaleFactor)
{
	int i,j;
	int sideLength = data->size;
	double downScale,upScale;
	downScale = (double)scaleFactor/(data->ground_level - data->min_elevation);
	upScale = (double)scaleFactor/(data->max_elevation - data->ground_level);
	double temp;
	for (i=0;i<sideLength;i++)
		for(j=0;j<sideLength;j++)
		{
			temp = data->patches[i*sideLength+j].elevation;
			if ((data->patches[i*sideLength+j].elevation-data->ground_level) > EPSILON)
			{
				data->patches[i*sideLength+j].elevation = (int)(data->ground_level + upScale * (temp-data->ground_level));
				temp = data->patches[i*sideLength+j].elevation;
			}

			else if ((data->patches[i*sideLength+j].elevation-data->ground_level) < -EPSILON)
			{
				data->patches[i*sideLength+j].elevation = (int)(data->ground_level + downScale * (temp-data->ground_level));
			}
		}
}

void cCSVFileReader::GenerateImage()
{
	int i,type_idx,xCor,yCor,turtle_idx;
	int r,g,b;
	for (i=0; i<data->patches.size(); i++)
	{
		xCor = data->patches[i].xcor;
		yCor = data->patches[i].ycor;
		type_idx=-1;
		if (data->patches[i].owner!=-1)
		{
			//it belongs to some building turtle
			turtle_idx = turtleMap[data->patches[i].owner];
			type_idx = data->building_turtles[turtle_idx].type;
		}
		
		if (data->patches[i].owner==-1)
			type_idx = 7;
		if (data->patches[i].terrain==1) //this water
			type_idx = 50;	
		if (data->patches[i].isRoad)
		{
			/*if (data->patches[i].road_type==3)
				type_idx = 4;
			else type_idx = 2;
			*/
			type_idx = 4;
		}
		if (data->patches[i].connectivity==3)
		{
			type_idx = 8;
		}
		if (data->patches[i].connectivity==4)
		{
			type_idx = 9;
		}
		
		if (data->patches[i].is_corner)
		{
			type_idx = 10;
		}
		r=colorMapping[type_idx].R;
		g=colorMapping[type_idx].G;
		b=colorMapping[type_idx].B;
		//if (type_idx!=-1)
			//MagickImage->pixelColor(xCor,yCor,ColorRGB(r/255.0,g/255.0,b/255.0)); 
		
	}
	//MagickImage->display();
	//MagickImage->write("tom.png");
}

inline int cCSVFileReader::getPatchIndex(int xCoor, int yCoor)
//return -1 which means the input coord is not valid
{
	//this function return the patch index in the data->patches
	int numSize =data->size; //usually 201
	//check boundary condition
	if ((xCoor<0) || (yCoor<0) || (xCoor>=data->size) || (yCoor>=data->size))
		return -1;
	
	//int ret= (xCoor+((numSize-1)-yCoor)*numSize);
	int ret = (xCoor+yCoor*numSize);
	return ret;
}

void cCSVFileReader::generateSVG(string fileName)
{
	int ratio =10;
	int radius=ratio/2;
	svgFile = fopen(fileName.c_str(),"w");
	fprintf(svgFile,"<?xml version=\"1.0\" standalone=\"no\"?>\n<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \n\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
	fprintf(svgFile,"<svg width=\"%fcm\" height=\"%fcm\" viewBox=\"%d %d %d %d\"\nxmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\">\n",(data->size)*ratio/100.0,(data->size)*ratio/100.0,ratio,ratio,(data->size-1)*ratio,(data->size-1)*ratio);
	
	fprintf(svgFile,"<style type=\"text/css\"><![CDATA[\n");
	fprintf(svgFile,".ResidentialParcel { fill:#ffff33; stroke:black; stroke-width:1 }\n");
	fprintf(svgFile,"]]></style>\n");	
	
	fprintf(svgFile,"<style type=\"text/css\"><![CDATA[\n");
	fprintf(svgFile,".CommercialParcel { fill:#e31a1c; stroke:black; stroke-width:1 }\n");
	fprintf(svgFile,"]]></style>\n");
	
	fprintf(svgFile,"<style type=\"text/css\"><![CDATA[\n");
	fprintf(svgFile,".IndustrialParcel { fill:#377db8; stroke:black; stroke-width:1 }\n");
	fprintf(svgFile,"]]></style>\n");
	
	fprintf(svgFile,"<style type=\"text/css\"><![CDATA[\n");
	fprintf(svgFile,".ParkParcel { fill:#00ee00; stroke:black; stroke-width:1 }\n");
	fprintf(svgFile,"]]></style>\n");
	
	//background rect
	fprintf(svgFile,"<g fill=\"#4daf4a\" >\n");
	fprintf(svgFile, "<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\"/>\n", data->size*ratio,data->size*ratio);
	fprintf(svgFile,"</g>\n");
	
	printf("\tGenerating parcels in SVG\n");
	generateParcelInSVG();
	printf("\tGenerating road networks in SVG\n");
	generateRoadPathInSVG();

	fprintf(svgFile,"</svg>");
	fclose(svgFile);
}
void cCSVFileReader::generateColorStrFromParcel(int type, char * colorStr)
{
	//int type=data->building_turtles[idx].type;
	//int density = (int) (data->building_turtles[idx].density);
	/*if (type==1 && density==1)
		sprintf(colorStr,"bbbb00");
	if (type==1 && density==2)
		sprintf(colorStr,"dddd00");
	if (type==1 && density==3)
		sprintf(colorStr,"ffff00");
	if (type==2 && density==1)
		sprintf(colorStr,"bb0000");
	if (type==2 && density==2)
		sprintf(colorStr,"dd0000");
	if (type==2 && density==3)
		sprintf(colorStr,"ff0000");
	if (type==3 && density==1)
		sprintf(colorStr,"0000bb");
	if (type==3 && density==2)
		sprintf(colorStr,"0000dd");
	if (type==3 && density==3)
		sprintf(colorStr,"0000ff");*/
	
	if (type==1)
    	sprintf(colorStr,"ResidentialParcel");
  // comm
  	if (type==2)
    	sprintf(colorStr,"CommercialParcel");
  // indus
  	if (type==3)
    	sprintf(colorStr,"IndustrialParcel");
	
	//if (type==1)
	//	sprintf(colorStr,"ffff00");
	//if (type==2)
	//	sprintf(colorStr,"ff0000");
	//if (type==3)
	//	sprintf(colorStr,"0000ff");
	if (type==4)
		sprintf(colorStr,"ParkParcel");
}
void cCSVFileReader::generateBuildingSlotInSVG()
{
	int ratio =10;
	int radius=ratio/2;
	int i,j,k;
	int parcelSize;
	int xcoor, ycoor;
	char colorStr[7];
	
	for (i=0; i<1300; i++)//data->building_turtles.size(); i++)
	{
		parcelSize = data->building_turtles[i].maxSquareSize;
		xcoor=data->building_turtles[i].TopLeftCoor_MaxSquare.xcor;
		ycoor=data->building_turtles[i].TopLeftCoor_MaxSquare.ycor;
		memset(colorStr,0,7);
		generateColorStrFromParcel(i,colorStr);
		fprintf(svgFile,"<g fill=\"#%s\">\n", colorStr);
		fprintf(svgFile, "<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\"/>\n", xcoor*ratio, ycoor*ratio, ratio*parcelSize,ratio*parcelSize);
		fprintf(svgFile,"</g>\n");
	}
}

int cCSVFileReader::generateParcelBoundary(vector<int> & edges, vector<double> &boundary)
{
	//this functions takes a list of edge index and try to link them together and generate a boundary
	//coordinate
	//return 0 for success, return -1 for failure
	vector<int>::iterator prevEdge, currentEdge, nextEdge;
	currentEdge=edges.begin();
	prevEdge=edges.end()-1;
	nextEdge=currentEdge+1;
	boundary.clear();
	while((edge_list[*currentEdge].edge_id==-1) && currentEdge!=edges.end())
	{
		currentEdge++;
	}
	assert(currentEdge!=edges.end());
	while((edge_list[*nextEdge].edge_id==-1) && nextEdge!=edges.end())
	{
		nextEdge++;
	}
	assert(nextEdge!=edges.end());
	while((edge_list[*prevEdge].edge_id==-1) && (prevEdge!=edges.begin()))
	{
		prevEdge--;
	}
	assert(prevEdge!=edges.begin());
	
	
	//now let's begin;
	bool finished=false;
	bool find;
	while(!finished)
	{
		find=false;
		if ((edge_list[*prevEdge].end == edge_list[*currentEdge].start) || 
			(edge_list[*prevEdge].start == edge_list[*currentEdge].start) ||
			(edge_list[*nextEdge].end == edge_list[*currentEdge].end)||
		    (edge_list[*nextEdge].start == edge_list[*currentEdge].end))
		{
			find=true;
			//push current start back
			boundary.push_back(vertex_list[edge_list[*currentEdge].start].xcoor);
			boundary.push_back(vertex_list[edge_list[*currentEdge].start].ycoor);
		}
		if (!find)
		if ((edge_list[*prevEdge].end == edge_list[*currentEdge].end) || 
			(edge_list[*prevEdge].start == edge_list[*currentEdge].end) ||
			(edge_list[*nextEdge].end == edge_list[*currentEdge].start)||
		    (edge_list[*nextEdge].start == edge_list[*currentEdge].start))
		{
			find=true;
			//push current end back
			boundary.push_back(vertex_list[edge_list[*currentEdge].end].xcoor);
			boundary.push_back(vertex_list[edge_list[*currentEdge].end].ycoor);
		}
		//if (!find) return -1;
		if (!find)
		{
			//there must be some problem in the simplification step,
			//anyhow we will hide it by push the mid point of that edge
			boundary.push_back((vertex_list[edge_list[*currentEdge].start].xcoor+
			                  vertex_list[edge_list[*currentEdge].end].xcoor)/2.0);
			boundary.push_back((vertex_list[edge_list[*currentEdge].start].ycoor+
			                  vertex_list[edge_list[*currentEdge].end].ycoor)/2.0);
		}
		//update the 3 iterators
		prevEdge=currentEdge;
		
		currentEdge++;
		if (currentEdge == edges.end()) 
		{
			finished=true;
			continue;
		}
		
		while(currentEdge!=edges.end()&&(edge_list[*currentEdge].edge_id==-1))
		{
			currentEdge++;
		}
		if (currentEdge == edges.end()) 
		{
			finished=true;
			continue;
		}
		
		nextEdge=currentEdge+1;
		if (nextEdge == edges.end()) 
			nextEdge=edges.begin();	
		while(nextEdge!=edges.end()&&(edge_list[*nextEdge].edge_id==-1))
		{
			nextEdge++;
		}
		if (nextEdge == edges.end()) 
			nextEdge=edges.begin();	
		
	}
	return 0;
}
void cCSVFileReader::generateParcelInSVG()
{
	int ratio =10;
	int radius=ratio/2;
	int i,j,k;
	int colorStep = 999999 / data->building_turtles.size();
	int color;
	char colorStr[7], tmpStr[7];
	char styleStr[20];
	int xCor, yCor;
	int dif;
	/*
	//begin of show original raster pixel in parcels
	for (i=0; i<data->building_turtles.size(); i++)
	//for (i=21; i<22; i++)
	{
		//if(i==21)
		//	continue;
		memset(tmpStr,0,7);
		color=colorStep*i;
		sprintf(tmpStr, "%d", color);
		//if (color!=185328)
		//	continue;
		dif=6-strlen(tmpStr);
		memset(colorStr,0,7);
		for (k=0; k<dif; k++)
		{
			sprintf(colorStr,"0");
		}
		strcat(colorStr,tmpStr);
		//generateColorStrFromParcel(data->building_turtles[i].type,colorStr);
		for (j=0; j<data->building_turtles[i].patchCoords.size(); j++)
		{
			xCor = data->building_turtles[i].patchCoords[j].xcor;
			yCor = data->building_turtles[i].patchCoords[j].ycor;
			fprintf(svgFile,"<g fill=\"#%s\">\n", colorStr);
			fprintf(svgFile, "<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\"/>\n", xCor*ratio, yCor*ratio, ratio,ratio);
			fprintf(svgFile,"</g>\n");
			
		}
	}//end of just output rect*/
	
	//now we need to display real stuff, generating svg shapes using simplified polygons
	for (i=0; i<parcel_list.size(); i++)
	{
		memset(tmpStr,0,7);
		color=colorStep*i;
		//if (color!=654192)
		//	continue;
		//printf("index:%d\n",i);
		//if ((i!=12) && (i!=15))
		//	continue;
		sprintf(tmpStr, "%d", color);
		dif=6-strlen(tmpStr);
		memset(colorStr,0,7);
		for (k=0; k<dif; k++)
		{
				sprintf(colorStr,"0");
		}
		strcat(colorStr,tmpStr);
		strcpy(tmpStr,colorStr);
		int prevIndex;
		
		//finish generating color string
		//if (i!=2145 && i!=2155 && i!=2098)
		//printf("%d\n", parcel_list[i].type);
		//generateColorStrFromParcel(parcel_list[i].type,colorStr);
		memset(styleStr,0,20);
		generateColorStrFromParcel(parcel_list[i].type,styleStr);
		//printf("%d\t",parcel_list[i].type);
		
		/*if (i==1242)
		{
			for (j=0; j<parcel_list[i].edges.size(); j++)
			{
				if (edge_list[parcel_list[i].edges[j]].edge_id==-1)
					continue;
				printf("Edge:%d V%d(%f,%f)--V%d(%f,%f)--> \n", parcel_list[i].edges[j], edge_list[parcel_list[i].edges[j]].start,
														 vertex_list[edge_list[parcel_list[i].edges[j]].start].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[j]].start].ycoor,
				edge_list[parcel_list[i].edges[j]].end,
				vertex_list[edge_list[parcel_list[i].edges[j]].end].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[j]].end].ycoor);
			}
		}
		//end of debug code*/
		vector<double> poly_boundry;
		poly_boundry.clear();
		int result=generateParcelBoundary(parcel_list[i].edges, poly_boundry);
		//now fill the poly boundry according to parcel_list[i].edges
		
		/*bool isFinish =false;
		bool visited_start =false;
		int startIndex;
		int targetIndex;
		j=0; //start from begining
		while( edge_list[parcel_list[i].edges[j]].edge_id==-1) //not already deleted edge
		{
			j++;
		}
		int startEdgeIndex = j;
		startIndex	= edge_list[parcel_list[i].edges[startEdgeIndex]].start;
		targetIndex = edge_list[parcel_list[i].edges[startEdgeIndex]].end;
		poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[startEdgeIndex]].start].xcoor*ratio);
		poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[startEdgeIndex]].start].ycoor*ratio);
		j++;
		
		while((!isFinish) && (j<parcel_list[i].edges.size()))
		{
			if (edge_list[parcel_list[i].edges[j]].edge_id==-1) //it is already deleted
			{
				j++;
				continue; 
			}
			if (targetIndex == edge_list[parcel_list[i].edges[j]].start)
			{
				poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[j]].start].xcoor*ratio);
				poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[j]].start].ycoor*ratio);
				targetIndex = edge_list[parcel_list[i].edges[j]].end;		
			}
			else
			{
				if (targetIndex == edge_list[parcel_list[i].edges[j]].end)
				{
					poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[j]].end].xcoor*ratio);
					poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[j]].end].ycoor*ratio);
					targetIndex = edge_list[parcel_list[i].edges[j]].start;
				}
				else
				{
					if (poly_boundry.size()==2)
					{
						if (!visited_start)
					
						{
							poly_boundry.clear();
							poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[startEdgeIndex]].end].xcoor*ratio);
							poly_boundry.push_back(vertex_list[edge_list[parcel_list[i].edges[startEdgeIndex]].end].ycoor*ratio);
							j=startEdgeIndex;
							targetIndex = edge_list[parcel_list[i].edges[startEdgeIndex]].start;
							startIndex = edge_list[parcel_list[i].edges[startEdgeIndex]].end;
							visited_start=true;
						}
						else
						{
							printf("something is really wrong! %d\n",i);
							for (int k=0; k<parcel_list[i].edges.size(); k++)
								{
									if (edge_list[parcel_list[i].edges[k]].edge_id==-1)
									//continue;
									printf("Delete--");
				printf("Edge:%d V%d(%f,%f)--V%d(%f,%f)--> \n", parcel_list[i].edges[k], 
									                           edge_list[parcel_list[i].edges[k]].start,
														       vertex_list[edge_list[parcel_list[i].edges[k]].start].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[k]].start].ycoor,
				edge_list[parcel_list[i].edges[k]].end,
				vertex_list[edge_list[parcel_list[i].edges[k]].end].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[k]].end].ycoor);
								}
							exit(-1);
							
							
						}
					}
				}
			}
			if (targetIndex==startIndex)
				isFinish=true;
			j++;
		}
		poly_boundry.push_back(vertex_list[targetIndex].xcoor*ratio);
		poly_boundry.push_back(vertex_list[targetIndex].ycoor*ratio);
*/
		if ((result==-1)||(poly_boundry.size()<=4)) 
		{
			printf("failure of generating parcel boundary\n");
			printf("Parcel num: %d\n", i);
			for (int k=0; k<parcel_list[i].edges.size(); k++)
								{
									if (edge_list[parcel_list[i].edges[k]].edge_id==-1)
									//continue;
									printf("Delete--");
				printf("Edge:%d V%d(%f,%f)--V%d(%f,%f)--> \n", parcel_list[i].edges[k], 
									                           edge_list[parcel_list[i].edges[k]].start,
														       vertex_list[edge_list[parcel_list[i].edges[k]].start].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[k]].start].ycoor,
				edge_list[parcel_list[i].edges[k]].end,
				vertex_list[edge_list[parcel_list[i].edges[k]].end].xcoor,
				vertex_list[edge_list[parcel_list[i].edges[k]].end].ycoor);
								}
			continue; //it is not a valid polygon then
		}
			
		//fprintf(svgFile, "<polygon fill=\"#%s\" stroke=\"black\" stroke-width=\"1\" points=\"", colorStr);
		fprintf(svgFile, "<polygon class=\"%s\" points=\"", styleStr);

		for (j=0;j<poly_boundry.size()/2; j++)
		{
			fprintf(svgFile,"%f,%f ", poly_boundry[2*j]*ratio, poly_boundry[2*j+1]*ratio);
		}
		
		fprintf(svgFile,"\" />");
		/*for (j=0; j<parcel_list[i].edges.size(); j++)
		{
			fprintf(svgFile,"<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#s\" opacity=\"0.5\"/>\n", vertex_list[edge_list[parcel_list[i].edges[j]].start].xcoor*ratio,
							  vertex_list[edge_list[parcel_list[i].edges[j]].start].ycoor*ratio, 1.2, tmpStr);
			fprintf(svgFile,"<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#s\" opacity=\"0.5\"/>\n", vertex_list[edge_list[parcel_list[i].edges[j]].end].xcoor*ratio,
							  vertex_list[edge_list[parcel_list[i].edges[j]].end].ycoor*ratio, 1.2, tmpStr);	
		}*/
			
	}
	
	//end of showing the parcels after simplification preparation*/
		//show point
		/*fprintf(svgFile, "<polygon fill=\"#%s\" points=\"", colorStr);
		prevIndex=edge_list[parcel_list[i].edges[0]].end; //for starting edge
		fprintf(svgFile,"<g fill=\"#ff0000\">\n");
				fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[edge_list[parcel_list[i].edges[j]].start].xcoor*ratio, 
				vertex_list[edge_list[parcel_list[i].edges[j]].start].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
		for (j=1; j<parcel_list[i].edges.size(); j++)
		{
			if (prevIndex == edge_list[parcel_list[i].edges[j]].start)
			{
				fprintf(svgFile,"<g fill=\"#ff0000\">\n");
				fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[edge_list[parcel_list[i].edges[j]].start].xcoor*ratio, 
				vertex_list[edge_list[parcel_list[i].edges[j]].start].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
								prevIndex = edge_list[parcel_list[i].edges[j]].end;
			}
			else {
				fprintf(svgFile,"<g fill=\"#ff0000\">\n");
				fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[edge_list[parcel_list[i].edges[j]].end].xcoor*ratio, 
				vertex_list[edge_list[parcel_list[i].edges[j]].end].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");							  
				prevIndex = edge_list[parcel_list[i].edges[j]].start;
			}
		}
		//fprintf(svgFile,"\" />");
		
	}/*end of for all parcels, show polygon svg*/
	
	/*fprintf(svgFile,"<g fill=\"#ff0000\">\n");
		fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[211].xcoor*ratio, 
				vertex_list[211].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
		fprintf(svgFile,"<g fill=\"#00ffff\">\n");
		fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[240].xcoor*ratio, 
				vertex_list[240].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
		fprintf(svgFile,"<g fill=\"#0000ff\">\n");
		fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[202].xcoor*ratio, 
				vertex_list[202].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
		fprintf(svgFile,"<g fill=\"#ffff00\">\n");
		fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%d\"/>\n", 
				vertex_list[203].xcoor*ratio, 
				vertex_list[203].ycoor*ratio, radius);
				fprintf(svgFile,"</g>\n");
	*/
}
//the following function will parse the road network to generate SVG path
void cCSVFileReader::generateRoadPathInSVG()
{
	int i=0,type_idx,xCor,yCor,turtle_idx;
	int ratio =10;
	int radius=ratio/2;
	int j;
	deque<RoadQueueItem> RoadQueue;
	RoadQueueItem queueItem;
	RoadQueueItem curQueueItem;
		
	
	//now in side this function we need to turn the road into curve
	/*i=data->patches.size()-1;
	while ((i>0) && (data->patches[i].connectivity!=4))
	{	
		i--;
	}
	if (i<data->patches.size())
	{
		xCor = data->patches[i].xcor;
		yCor = data->patches[i].ycor;
		//now we find a 4 connector, push this into a queue, with its xCor, yCor, 
		//check for its 4 neighbour
		//#0 xCor, yCor-1;
		//#1 xCor+1, yCor;
		//#2 xCor, yCor+1;
		//#3 xCor-1, yCor;
		if (yCor>=1)
			
		{	//#0 xCor, yCor-1;
			queueItem.index = i;
			queueItem.xcoor =xCor;
			queueItem.ycoor =yCor;
			queueItem.direction = 0;
			RoadQueue.push_back(queueItem);
		}
		if (xCor<data->size)
		{
			//then we should xCor+1 yCor,
			queueItem.index = i;
			queueItem.xcoor =xCor;
			queueItem.ycoor =yCor;
			queueItem.direction = 1;
			RoadQueue.push_back(queueItem);			
		}
		if (yCor<data->size)
		{
			//then we should go xCor yCor+1,
			queueItem.index = i;
			queueItem.xcoor =xCor;
			queueItem.ycoor =yCor;
			queueItem.direction = 2;
			RoadQueue.push_back(queueItem);
		}
		if (xCor>=1)
		{
			//then we should xCor-1 yCor,
			queueItem.index = i;
			queueItem.xcoor =xCor;
			queueItem.ycoor =yCor;
			queueItem.direction = 3;
			RoadQueue.push_back(queueItem);
		}
	}*/
  	//finish initialize the queue;
	//now depth first travesal algorithm to find all secondary road.
	//depthFirstTravelForSecondaryRoad(svgFile, RoadQueue);
	/*
	for (i=0; i<data->patches.size(); i++)
	{
		xCor = data->patches[i].xcor;
		yCor = data->patches[i].ycor;
		type_idx=-1;
		if (data->patches[i].owner!=-1)
		{
			//it belongs to some building turtle
			turtle_idx = turtleMap[data->patches[i].owner];
			type_idx = data->building_turtles[turtle_idx].type;
		}
		
		if (data->patches[i].owner==-1)
			type_idx = 7;
		if (data->patches[i].terrain==1) //this water
			type_idx = 50;	
		if (data->patches[i].isRoad)
		{
			type_idx = 4;
		}
		if (data->patches[i].connectivity==3)
		{
			type_idx = 8;
		}
		if (data->patches[i].connectivity==4)
		{
			type_idx = 9;
		}
		
		//if (data->patches[i].is_corner)
		//{
		//	type_idx = 10;
		//}
		if ((data->patches[i].road_type==1)&&(!data->patches[i].is_corner))
		{
			fprintf(svgFile,"<g fill=\"#222222\">\n");
			fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", xCor*ratio, yCor*ratio, radius);
			fprintf(svgFile,"</g>\n");
		}
		if ((data->patches[i].road_type==3)&&(!data->patches[i].is_corner))
		{
			fprintf(svgFile,"<g fill=\"#aaaaaa\">\n");
			fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", xCor*ratio, yCor*ratio, radius);
			fprintf(svgFile,"</g>\n");
		}
		if ((type_idx == 9)&&(!data->patches[i].is_corner))
		{
			fprintf(svgFile,"<g fill=\"#00ff00\">\n");
			fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", xCor*ratio, yCor*ratio, radius);
			fprintf(svgFile,"</g>\n");
		}
		if ((type_idx == 8)&&(!data->patches[i].is_corner))
		{
			fprintf(svgFile,"<g fill=\"#0000ff\">\n");
			fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", xCor*ratio, yCor*ratio, radius);
			fprintf(svgFile,"</g>\n");
		}
		if (data->patches[i].is_corner)
		{
			fprintf(svgFile,"<g fill=\"#ff0000\">\n");
			fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", xCor*ratio, yCor*ratio, radius);
			fprintf(svgFile,"</g>\n");
		}
		
	}
	
	*/
	int idx;
	char pathStr[60000];
	vector<DoublePatchCoor> roadseg;
	for (i=0; i<waterFrontSegments.size(); i++)
	{
		movingAverageRoadPatches(waterFrontSegments[i], roadseg, 3);
		memset(pathStr,0,60000);
		
		generateRoadCurveInSVG(roadseg, pathStr);
		//printf("%s\n",pathStr);
		fprintf(svgFile,"<path d=\"%s z\"\n fill=\"#00ffff\" stroke=\"#888888\" stroke-width=\"4\"/>\n",pathStr);   

	}/*end of the water front*/
	//printf("after generating water front in SVG\n");
	
	//fprintf(svgFile, "<g tranform = \"translate(-%d, -%d)\">\n",50,50);
	for (i=0; i<roadSegments_primary.size(); i++)
	{
		if (roadSegments_primary[i].size()>=5)
		{
			movingAverageRoadPatches(roadSegments_primary[i], roadseg, 3, true);
			memset(pathStr,0,60000);
		
			generateRoadCurveInSVG(roadseg, pathStr);
			//printf("%s\n",pathStr);
			fprintf(svgFile,"<path d=\"%s \"\n fill=\"none\" stroke=\"#888888\" stroke-width=\"%f\"/>\n",pathStr,2.828*ratio);   
		}	
        //fprintf(svgFile,"\tstroke=\"gray\" stroke-width=\"%d\" />\n",10);

		
		//for (j=0;j<roadSegments[i].size();j++)
		//{
		//	fprintf(svgFile,"<g fill=\"#ff0000\">\n");
		//	fprintf(svgFile, "<circle cx=\"%d\" cy=\"%d\" r=\"%d\"/>\n", roadSegments[i][j].xcor*ratio, roadSegments[i][j].ycor*ratio, radius);
		//	fprintf(svgFile,"</g>\n");
		//
		//}
		
	}
	//fprintf(svgFile,"</g>\n");
	//printf("after generating primary road in SVG\n");
	/*end of primary road*/
	
	for (i=0; i<roadSegments_secondary_new.size(); i++)
	{
		
		if (roadSegments_secondary_new[i].size()>=4)
		{
			movingAverageRoadPatches(roadSegments_secondary_new[i], roadseg, 5);
			memset(pathStr,0,60000);
			generateRoadCurveInSVG(roadseg, pathStr);
			//printf("%s\n",pathStr);
			fprintf(svgFile,"<path d=\"%s \"\n fill=\"none\" stroke=\"#888888\" stroke-width=\"%f\"/>\n",pathStr,1.2*ratio);   
			
		}
		
	}
	for (i=0; i<roadSegments_secondary_new.size(); i++)
	{
		if (roadSegments_secondary_new[i].size()<4)
			continue;
		for (j=roadSegments_secondary_new[i].size()-1;j<roadSegments_secondary_new[i].size();j++)
		{
	
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#888888\" />\n", (roadSegments_secondary_new[i][j].xcor+0.5)*ratio, (roadSegments_secondary_new[i][j].ycor+0.5)*ratio, 1.0*radius);
		}
		for (j=0;j<1;j++)
		{
			
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#888888\" />\n", (roadSegments_secondary_new[i][j].xcor+0.5)*ratio, (roadSegments_secondary_new[i][j].ycor+0.5)*ratio, 1.0*radius);
		
		}
	}
	//the end points*/
	//printf("after generating secondary road in SVG\n");
	/*fprintf(svgFile,"<g opacity=\"0.5\">\n");
	for (i=0; i<roadSegments_primary.size(); i++)
	{
		for (j=roadSegments_primary[i].size()-1;j<roadSegments_primary[i].size();j++)
		{
	
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#888888\" opacity=\"0.5\"/>\n", (roadSegments_primary[i][j].xcor+0.5)*ratio, (roadSegments_primary[i][j].ycor+0.5)*ratio, radius*0.8);
		}
		for (j=0;j<1;j++)
		{
			
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#888888\" opacity=\"0.5\"/>\n", (roadSegments_primary[i][j].xcor+0.5)*ratio, (roadSegments_primary[i][j].ycor+0.5)*ratio, radius*0.8);
		
		}
	}
	fprintf(svgFile,"</g>\n");
	/*primary road end points*/
	//fprintf(svgFile,"<g opacity=\"0.5\">\n");
	
	/*for (i=0; i<roadIntersector.size(); i++)
	{
		fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#888888\" />\n", (roadIntersector[i].xcor+0.5)*ratio, (roadIntersector[i].ycor+0.5)*ratio, radius*1.5);
	}
	connectors*/
	
	
	//fprintf(svgFile,"</g>\n");
	
	//fprintf(svgFile,"<g opacity=\"0.5\">\n");
	/*for (i=0; i<roadSegments_secondary.size(); i++)
	{
		for (j=roadSegments_secondary[i].size()-2;j<roadSegments_secondary[i].size();j++)
		{
	
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#ff0000\" opacity=\"0.5\"/>\n", (roadSegments_secondary[i][j].xcor+0.5)*ratio, (roadSegments_secondary[i][j].ycor+0.5)*ratio, radius*0.9);
		}
		for (j=0;j<2;j++)
		{
			
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#0000ff\" opacity=\"0.5\"/>\n", (roadSegments_secondary[i][j].xcor+0.5)*ratio, (roadSegments_secondary[i][j].ycor+0.5)*ratio, radius*0.9);
		
		}
	}
	//fprintf(svgFile,"</g>\n");
	
	
	/*secondary road end points*/
	/*for (i=0; i<data->patches.size(); i++)
	{
		if (data->patches[i].isRoad)
		{
			fprintf(svgFile,"<g fill=\"#00ffff\" opacity=\"0.2\">\n");
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\"/>\n", (data->patches[i].xcor+0.5)*ratio, (data->patches[i].ycor+0.5)*ratio, radius*0.8);
			fprintf(svgFile,"</g>\n");
		}		
	}*/
	
	
	/*for (i=0; i<data->patches.size(); i++)
	{
		if ((data->patches[i].isRoad)&&(data->patches[i].road_type==1))
		{
			fprintf(svgFile,"<g fill=\"#00ffff\" opacity=\"0.2\">\n");
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\"/>\n", (data->patches[i].xcor+0.5)*ratio, (data->patches[i].ycor+0.5)*ratio, radius*0.8);
			fprintf(svgFile,"</g>\n");
		}		
	}/*primary road patch points*/
	
	
	/*for (i=0; i<roadSegments_primary.size(); i++)
	{
		for (j=0;j<roadSegments_primary[i].size();j++)
		{
	
			fprintf(svgFile, "<circle cx=\"%f\" cy=\"%f\" r=\"%f\" fill=\"#ff0000\" opacity=\"0.5\"/>\n", (roadSegments_primary[i][j].xcor+0.5)*ratio, (roadSegments_primary[i][j].ycor+0.5)*ratio, radius*0.8);
		}
	}/*all primary control points*/
	
}

void cCSVFileReader::movingAverageRoadPatches(deque<PatchCoor> & inputPatches, vector<DoublePatchCoor> & outputPatches, int window_length, bool isPrimary)
{
	outputPatches.clear();
	DoublePatchCoor outputItem;
	int i,j;
	int halfWindow = (int)(floor((double)window_length/2.0));
	vector<PatchCoor> tempPatchCoor;
	for (i=0; i<halfWindow; i++) tempPatchCoor.push_back(inputPatches[0]);
	for (i=0; i<inputPatches.size(); i++) 	
	{
		//printf("%d,%d\t", inputPatches[i].xcor, inputPatches[i].ycor);
		tempPatchCoor.push_back(inputPatches[i]);
	}
	//printf("\n\n");
	if (halfWindow!=0)
	{
		outputItem.xcor = inputPatches[0].xcor;
		outputItem.ycor = inputPatches[0].ycor;
		outputPatches.push_back(outputItem);
	}
	
	for (i=0; i<halfWindow; i++) tempPatchCoor.push_back(inputPatches[inputPatches.size()-1]);	
	
	double sumX, sumY;
	for (i=0; i<inputPatches.size(); i++)
	{
		sumX=0.0;
		sumY=0.0;
		for(j=0; j<window_length; j++)
		{
			sumX += tempPatchCoor[i+j].xcor;
			sumY += tempPatchCoor[i+j].ycor;
		}
		//printf("%f,%f\n", sumX, sumY);
		
		outputItem.xcor = sumX / window_length;
		outputItem.ycor = sumY / window_length;
		
		//assert(outputItem.xcor>=0);
		//assert(outputItem.ycor>=0);
		//assert(outputItem.xcor<=data->xmax*2);
		//assert(outputItem.ycor<=data->ymax*2);
		
		outputPatches.push_back(outputItem);
		//printf("%f,%f\t", outputItem.xcor,outputItem.ycor);
	}
	if (halfWindow!=0)
	{
		outputItem.xcor = inputPatches[inputPatches.size()-1].xcor;
		outputItem.ycor = inputPatches[inputPatches.size()-1].ycor;
		outputPatches.push_back(outputItem);
	}
	if (isPrimary) //since the primary road description is not that acturate, we need somehow compensate it
	{
		for (i=0; i<outputPatches.size(); i++)
		{
			outputPatches[i].xcor+=0.5;
			outputPatches[i].ycor-=0.5;
		}
	}
}
void cCSVFileReader::generateRoadCurveInSVG(vector<DoublePatchCoor> & vPatches, char * outStr)
{
	int ratio=10;
	int i;
	int j;
	for (j=0; j<vPatches.size(); j++)
	{
		vPatches[j].xcor+=0.5;
		vPatches[j].ycor+=0.5;
	}
	double xCoor, yCoor, prevTurnXCoor, prevTurnYCoor, prevXCoor, prevYCoor, xDiff, yDiff, prevXDiff, prevYDiff;
	prevTurnXCoor=prevXCoor = xCoor = vPatches[0].xcor;
	prevTurnYCoor=prevYCoor = yCoor = vPatches[0].ycor;
	
	i=1;
	xCoor = vPatches[i].xcor;
	yCoor = vPatches[i].ycor;
	
	xDiff = prevXDiff = xCoor - prevXCoor;
	yDiff = prevYDiff = yCoor - prevYCoor;
	
	
	double xTemp=(prevXCoor-0.5)-xDiff;
	double yTemp=(prevYCoor-0.5)-yDiff;
	int tmpIndex;
	bool flag=false;
	if ((tmpIndex=getPatchIndex((int)xTemp, (int)yTemp))!=-1)
	{
		if (data->patches[tmpIndex].isRoad)
		{
		//	sprintf(outStr,"M %f %f L", (prevXCoor-2.5*xDiff)*ratio, (prevYCoor-2.5*yDiff)*ratio);
			flag=true;
		}
	}
	//printf("%f %f\n", xDiff,xDiff);
	//if(!flag)
	//sprintf(outStr,"M %f %f L", (prevXCoor-0.5*xDiff)*ratio, (prevYCoor-0.5*yDiff)*ratio);
	sprintf(outStr,"M %f %f L", (prevXCoor)*ratio, (prevYCoor)*ratio);
	flag=false;
	//sprintf(outStr,"M %f %f L", (prevXCoor)*ratio, (prevYCoor)*ratio);
	while (i<vPatches.size())
	{	
		while((fabs(yDiff-prevYDiff)<0.2) && (fabs(xDiff-prevXDiff)<0.2) && (i<vPatches.size()-1)) //moving in the same direction
		{
			i++;
			xDiff = vPatches[i].xcor-xCoor;
			yDiff = vPatches[i].ycor-yCoor;
			prevXCoor = xCoor;
			prevYCoor = yCoor;
			xCoor = vPatches[i].xcor;
			yCoor = vPatches[i].ycor;			
		}			
		//now preXCoor and prevYCoor is new turning point
		//output 
		//printf("%f %f %f %f %f %f %f %f %f %f\n", xCoor, yCoor, prevXCoor, prevYCoor, xDiff, yDiff, prevXDiff, prevYDiff, prevTurnXCoor, prevTurnYCoor);
		
		sprintf(outStr,"%s %f,%f C %f,%f %f,%f", outStr,(prevXCoor+prevTurnXCoor)*ratio/2.0,
												(prevYCoor+prevTurnYCoor)*ratio/2.0,		
											   prevXCoor*ratio,prevYCoor*ratio, 
											   prevXCoor*ratio,prevYCoor*ratio);
		
		prevTurnXCoor=prevXCoor; //update the turning point
		prevTurnYCoor=prevYCoor;
		if (i==vPatches.size()-1)
		{	sprintf(outStr,"%s %f,%f",outStr,(xCoor+prevTurnXCoor)*ratio/2.0,
												(yCoor+prevTurnYCoor)*ratio/2.0);
			//sprintf(outStr,"%s %f,%f",outStr,xCoor*ratio,yCoor*ratio);
			break; //last one
		}
		
		i++;		
		prevXCoor = xCoor;
		prevYCoor = yCoor;
		xCoor = vPatches[i].xcor;
		yCoor = vPatches[i].ycor;
		prevXDiff = xDiff;
		prevYDiff = yDiff;
		xDiff = xCoor - prevXCoor;
		yDiff = yCoor - prevYCoor;
		
		//printf("%d %d %d %d %d %d %d %d %d %d\n", xCoor, yCoor, prevXCoor, prevYCoor, xDiff, yDiff, prevXDiff, prevYDiff, prevTurnXCoor, prevTurnYCoor);
	}
	
	
	xTemp=(xCoor-0.5)+xDiff;
	yTemp=(yCoor-0.5)+yDiff;
	if ((tmpIndex=getPatchIndex((int)xTemp, (int)yTemp))!=-1)
	{
		if (data->patches[tmpIndex].isRoad)
		{
			//sprintf(outStr,"%s L %f,%f", outStr,(xCoor+2.5*xDiff)*ratio,(yCoor+2.5*yDiff)*ratio);
			flag=true;
		}
	}
	//if(!flag)
	//sprintf(outStr,"%s L %f,%f", outStr,(xCoor+0.5*prevXDiff)*ratio,(yCoor+0.5*prevYDiff)*ratio);
	sprintf(outStr,"%s L %f,%f", outStr,(xCoor)*ratio,(yCoor)*ratio);
	//sprintf(outStr,"%s L %f,%f", outStr,(xCoor)*ratio,(yCoor)*ratio);
}
void cCSVFileReader::depthFirstTravelForSecondaryRoad(FILE *svgFile, deque<RoadQueueItem> & RoadQueue)
{
	int ratio=10;
	int nextPatchIndex;
	int curXCoor, curYCoor, nextXCoor, nextYCoor;
	RoadQueueItem queueItem;
	RoadQueueItem curQueueItem;
	bool has_way=false;
	int i,j;
	vector<int> connectorVisited;
	connectorVisited.push_back(RoadQueue[0].index);
	while (!RoadQueue.empty())
	{
		//now we need to process this 
		curQueueItem = RoadQueue[0];
		curXCoor = curQueueItem.xcoor;
		curYCoor = curQueueItem.ycoor;
		nextXCoor = curXCoor;
		nextYCoor = curYCoor;
		
		//#0 xCor, yCor-1;
		//#1 xCor+1, yCor;
		//#2 xCor, yCor+1;
		//#3 xCor-1, yCor;
		if (curQueueItem.direction==0)
		{
			if ((curYCoor!=0)&&(data->patches[getPatchIndex(curXCoor, curYCoor-1)].isRoad))
			{
				//keep going xCor, yCor-1;
				nextXCoor = curXCoor;
				nextYCoor = curYCoor-1;
				nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				//nextYCoor>=1, is secondary road, not connector or corner
				while ((nextYCoor>=1) && (data->patches[nextPatchIndex].road_type==3) &&
						(!(data->patches[nextPatchIndex].is_corner)) &&  //not corner
						(data->patches[nextPatchIndex].connectivity<3)
					   ) //gurrantee it is straight road running this way;
				{
					nextYCoor --;
					data->patches[nextPatchIndex].road_type=0;
					nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				}
			}
		}
		
		if (curQueueItem.direction==1)
		{
			if ((curXCoor!=data->size-1)&&(data->patches[getPatchIndex(curXCoor+1, curYCoor)].isRoad))
			{
				//keep going xCor+1, yCor;			
				nextXCoor = curXCoor+1;
				nextYCoor = curYCoor;
				nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				//nextYCoor>=1, is secondary road, not connector or corner
				while ((nextXCoor<data->size) && (data->patches[nextPatchIndex].road_type==3) &&
						(!(data->patches[nextPatchIndex].is_corner)) &&  //not corner
						(data->patches[nextPatchIndex].connectivity<3)
					   ) //gurrantee it is straight road running this way;
				{
					nextXCoor ++;
					data->patches[nextPatchIndex].road_type=0;
					nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				}
			}
		}
		
		if (curQueueItem.direction==2)
		{
			//keep going xCor, yCor+1;
			if ((curYCoor!=data->size-1)&&(data->patches[getPatchIndex(curXCoor, curYCoor+1)].isRoad))			
			{
				nextXCoor = curXCoor;
				nextYCoor = curYCoor+1;
				nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				//nextYCoor>=1, is secondary road, not connector or corner
				while ((nextYCoor<data->size) && (data->patches[nextPatchIndex].road_type==3) &&
						(!(data->patches[nextPatchIndex].is_corner)) &&  //not corner
						(data->patches[nextPatchIndex].connectivity<3)
					   ) //gurrantee it is straight road running this way;
				{
					nextYCoor ++;
					data->patches[nextPatchIndex].road_type=0;
					nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				}
			}
		}
		
		
		if (curQueueItem.direction==3)
		{
			//keep going xCor-1, yCor;	
			if ((curXCoor!=0)&&(data->patches[getPatchIndex(curXCoor-1, curYCoor)].isRoad))			
			{
				nextXCoor = curXCoor-1;
				nextYCoor = curYCoor;
				nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				
				//nextYCoor>=1, is secondary road, not connector or corner
				while ((nextXCoor>=1) && (data->patches[nextPatchIndex].road_type==3) &&
						(!(data->patches[nextPatchIndex].is_corner)) &&  //not corner
						(data->patches[nextPatchIndex].connectivity<3)
					   ) //gurrantee it is straight road running this way;
				{
					nextXCoor --;
					data->patches[nextPatchIndex].road_type=0;
					nextPatchIndex = getPatchIndex(nextXCoor, nextYCoor);
				}
			}
		}
		if ((nextXCoor!=curXCoor) || (nextYCoor!=curYCoor))
		{
			has_way =true; 
			fprintf(svgFile,"<path d=\"M %d %d L %d %d\"\n",curXCoor*ratio,curYCoor*ratio, nextXCoor*ratio, nextYCoor*ratio);   
        	fprintf(svgFile,"\tstroke=\"gray\" stroke-width=\"%d\" />\n",10);
		}		
		//now for corner and 3-connector, 4-connector, push in RoadQueue
		if ((has_way) && 
			(
		 		(data->patches[nextPatchIndex].is_corner) ||
				(data->patches[nextPatchIndex].connectivity>=3)
			)
		    )
		{
			queueItem.index=nextPatchIndex;
			queueItem.xcoor = nextXCoor;
			queueItem.ycoor = nextYCoor;
			
				//we still need to check to not go to the same route again and again
				//if we have already visited this connector, we mark it visited
				
			for (i=0;i<connectorVisited.size(); i++)
			{
					if (queueItem.index==connectorVisited[i])
						break;
			}
			if (i==connectorVisited.size()) //not in the list
				{
					connectorVisited.push_back(queueItem.index);
					printf("%d\n", queueItem.index);
					for (j=0;j<4;j++)				//for all the direction other than this current one
					{
						if (abs(j-curQueueItem.direction)!=2)
						{
							queueItem.direction=j;					
							RoadQueue.push_back(queueItem);
						}
					}
				}
		}
		//which direction?
		RoadQueue.pop_front();
		//printf("%d\n",RoadQueue.size());		
	}
}
void cCSVFileReader::parseGlobalDescription(char * input)
{
	int idx=0;
	char seps[]   = ",\n";
	char * token;
	char buffer[100];
	token = strtok( input, seps );

	while (token != NULL) 
	{ 
      memset(buffer,0,100);
	  strcpy(buffer,token);
	 // printf("%s\n", token);
	  if (!strcmp(buffer,"\"screen-edge-x\""))
		  globalHeader.screen_edge_x_idx = idx;
	  if (!strcmp(buffer,"\"screen-edge-y\""))
		  globalHeader.screen_edge_y_idx = idx;
	  if (!strcmp(buffer,"\"ground-level\""))
		  globalHeader.ground_level_idx = idx;
	  if (!strcmp(buffer,"\"water-level\""))
		  globalHeader.water_level_idx = idx;
	 
	  if (!strcmp(buffer,"\"average-residential-density\""))
		  globalHeader.average_residential_density_idx = idx;
	  if (!strcmp(buffer,"\"max-residential-density\""))
		  globalHeader.max_residential_density_idx = idx;
	  if (!strcmp(buffer,"\"residential-population\""))
		  globalHeader.residential_population_idx = idx;
	  
	  if (!strcmp(buffer,"\"average-commercial-density\""))
		  globalHeader.average_commercial_density_idx = idx;
	  if (!strcmp(buffer,"\"max-commercial-density\""))
		  globalHeader.max_commercial_density_idx = idx;
	  if (!strcmp(buffer,"\"commercial-population\""))
		  globalHeader.commercial_population_idx = idx;
	  
	  
	  if (!strcmp(buffer,"\"average-industrial-density\""))
		  globalHeader.average_industrial_density_idx = idx;
	  if (!strcmp(buffer,"\"max-industrial-density\""))
		  globalHeader.max_industrial_density_idx = idx;
	  if (!strncmp(buffer,"\"industrial-population\"", 23))
		  globalHeader.industrial_population_idx = idx;

	  if (!strcmp(buffer,"\"pins-tertiaries\""))
		  globalHeader.pins_tertiaries_idx = idx;
	  if (!strcmp(buffer,"\"pins-primaries\""))
		  globalHeader.pins_primaries_idx = idx;
	  if (!strcmp(buffer,"\"pins-water-boundaries\""))
		  globalHeader.pins_water_boundaries_idx = idx;
	
	  token = strtok( NULL, seps );	
	  idx++;
  	}
	//printf("idx: %d\n",globalHeader.industrial_population_idx);
	printf("Reading Global Parameters...\n");
}

//the parse description functions are trying to find out the index of each elements in the different section.
void cCSVFileReader::parsePatchDescription(char * input)
{
	int idx=0;
	char seps[]   = ",\n";
	char * token;
	char buffer[100];
	token = strtok( input, seps );
 	
	while (token != NULL) 
	{ 
      memset(buffer,0,100);
	  strcpy(buffer,token);
	  if (!strcmp(buffer,"\"pxcor\""))
		  patchHeader.pxcor_idx = idx;
	  if (!strcmp(buffer,"\"pycor\""))
		  patchHeader.pycor_idx = idx;
	  if (!strcmp(buffer,"\"road\""))
		  patchHeader.road_idx = idx;
	  if (!strcmp(buffer,"\"road-type\""))
		  patchHeader.roadtype_idx = idx;
	  if (!strcmp(buffer,"\"connectivity\""))
		  patchHeader.connectivity_idx = idx;
	  if (!strcmp(buffer,"\"is-corner\""))
		  patchHeader.is_corner_idx = idx;
	  if (!strcmp(buffer,"\"terrain\""))
		  patchHeader.terrain_idx = idx;
	  if (!strcmp(buffer,"\"elevation\""))
		  patchHeader.elevation_idx = idx;
	  if (!strcmp(buffer,"\"owner\""))
		  patchHeader.owner_idx = idx;
	  token = strtok( NULL, seps );	  
	  idx++;
	}
	printf("Reading Patch Information...\n");
}

void cCSVFileReader::parseTurtleDescription(char * input)
{
	int idx=0;
	char seps[]   = ",\n";
	char * token;
	char buffer[100];
	token = strtok( input, seps );
	
	while (token != NULL) 
	{ 
      memset(buffer,0,100);
	  strcpy(buffer,token);
	  if (!strcmp(buffer,"\"who\""))
		turtleHeader.who_idx = idx;
	  if (!strcmp(buffer,"\"breed\""))
		turtleHeader.breed_idx = idx;
	  if (!strcmp(buffer,"\"heading\""))
		turtleHeader.heading_idx = idx;
	  if (!strcmp(buffer,"\"building-class\""))
		turtleHeader.building_zone_idx = idx;
	  if (!strcmp(buffer,"\"parcel\""))
		turtleHeader.parcel_idx = idx;
	  if (!strcmp(buffer,"\"population\""))
		turtleHeader.population_idx = idx;
	  if (!strcmp(buffer,"\"population-density\""))
		turtleHeader.population_density_idx = idx;
	  if (!strcmp(buffer,"\"road-segment\""))
		  turtleHeader.road_segment_idx = idx;
	  if (!strcmp(buffer,"\"name\""))
		  turtleHeader.developer_name_idx = idx;
	  token = strtok( NULL, seps );
	  idx++;
	}
	printf("Reading Turtle Information...\n");
}

//in this function we try to remap the density of each parcel to 3 categories
//for simcity, the are low, med and high densities so the question here is how 
//can we map those density to the categories.
void cCSVFileReader::parcelDensityRemapper()
{
	vector<TurtleData>::iterator turtle_iter;
	vector<PatchCoor>::iterator patch_iter;

	vector<int> PatchMap=data->PatchMap;
	
	vector<double> ResSplit;
	vector<double> ComSplit;
	vector<double> IndSplit;
	
	double ResMax = -1.0;
	double ComMax = -1.0;
	double IndMax = -1.0;
	
	ResSplit.clear();
	ComSplit.clear();
	IndSplit.clear();
	

	
	for (turtle_iter=data->building_turtles.begin();
		turtle_iter<data->building_turtles.end();
		turtle_iter++)
	{
		if (turtle_iter->type==1) //residential
		{
			if (turtle_iter->density > ResMax)
			{
				ResMax = turtle_iter->density;
			}
		}
		
		if (turtle_iter->type==2) //residential
		{
			if (turtle_iter->density > ComMax)
			{
				ComMax = turtle_iter->density;
			}
		}
		
		if (turtle_iter->type==3) //residential
		{
			if (turtle_iter->density > IndMax)
			{
				IndMax = turtle_iter->density;
			}
			
		}
	}
	
	int i;
	
	for (i=0; i<=5; i++)
	{
		//ResSplit.push_back((double)i*ResMax/5.0);
		ComSplit.push_back((double)i*ComMax/5.0);
		IndSplit.push_back((double)i*IndMax/5.0);
	}
	ResSplit.push_back(0.0);

	ResSplit.push_back(3.0*ResMax/5.0);

	ResSplit.push_back(ResMax);

	ResSplit.push_back(10*ResMax/5.0);
	
	ResSplit.push_back(100*ResMax/5.0);
	
	for (turtle_iter=data->building_turtles.begin();
		turtle_iter<data->building_turtles.end();
		turtle_iter++)
	{
		if (turtle_iter->type==1) //residential
		{
			i=0;
			while ((i<5) && (turtle_iter->density>ResSplit[i]))
			{
				i++;
			}
			turtle_iter->density = i;
			printf("%f\t", turtle_iter->density);
		}
		
		if (turtle_iter->type==2) //Com		
		{	
			i=0;
			while ((i<5) && (turtle_iter->density>ComSplit[i]))
			{
				i++;
			}
			turtle_iter->density = i;
		}
		
		if (turtle_iter->type==3) //Ind
		{
			i=0;
			while ((i<5) && (turtle_iter->density>IndSplit[i]))
			{
				i++;
			}
			turtle_iter->density = i;			
		}
		//printf("%f\t", turtle_iter->density);
	}
	
	
}
void cCSVFileReader::prepareSimplify_co_ownership(S_Parcel & parcel_item, int patch_index, int parcel_index)
{
	//two things to do:
	//1. remove the adjacent edges if it exist in this parcel_item
	//2. add cut and additional boundry 
	
	//for task 1 we need check the 4 edges 
	//printf("Calling co-ownership resolver %d, (%d, %d)\n",parcel_item.owner,data->patches[patch_index].co_ownership[0],data->patches[patch_index].co_ownership[1]);
	vector<int>::iterator edge_iter;
	double xcoor = data->patches[patch_index].xcor;
	double ycoor = data->patches[patch_index].ycor;
	int config = data->patches[patch_index].co_ownership_config;


	double edge_xcoor, edge_ycoor;
	
	int type_position; //0 left, 1 right, 2 up, 3 down
	bool finished=false;
	while(!finished) //we need to make sure the patch has been processed from all possible direction.
	{
	type_position=-1;
	if (((config==1100) || (config==1010) || (config==1001)) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[0]))
		{
			type_position=0;
			data->patches[patch_index].co_ownership[0]=-1;
		}
	
	if ((type_position==-1)&&(((config==1100) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[1]))
		|| (((config==110) || (config==101)) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[0]))))
		{
			type_position=1;
			if (config==1100)
			{
				data->patches[patch_index].co_ownership[1] = -1;
			}
			else
			{
				data->patches[patch_index].co_ownership[0] = -1;
			}
		}
	
	if ((type_position==-1)&&(((config==11) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[0]))
		|| (((config==110) || (config==1010)) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[1]))))
		{	
			type_position=2;
			if (config==11)
			{
				data->patches[patch_index].co_ownership[0] = -1;
			}
			else
			{
				data->patches[patch_index].co_ownership[1] = -1;
			}
		}
	
	
	if ((type_position==-1)&&(((config==1001) || (config==101) || (config==11)) && (parcel_item.owner ==  data->patches[patch_index].co_ownership[1])))
		{
			type_position=3;
			data->patches[patch_index].co_ownership[1] = -1;
		}
	
	if ((type_position==-1)&&((config==1111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[0])||
		(config==1101 && parcel_item.owner ==  data->patches[patch_index].co_ownership[0])||
		(config==1110 && parcel_item.owner ==  data->patches[patch_index].co_ownership[0])||
		(config==1011 && parcel_item.owner ==  data->patches[patch_index].co_ownership[0])))
		{
			type_position=0;
			data->patches[patch_index].co_ownership[0]=-1;
		}
	
	if ((type_position==-1)&&((config==1111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[1])||
		(config==1101 && parcel_item.owner ==  data->patches[patch_index].co_ownership[1])||
		(config==1110 && parcel_item.owner ==  data->patches[patch_index].co_ownership[1])||
		(config==111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[0])))
		{
			type_position=1;
			if (config==111)
			{
				data->patches[patch_index].co_ownership[0] = -1;
			}
			else
			{
				data->patches[patch_index].co_ownership[1] =-1;
			}
		}
	
	if ((type_position==-1)&&((config==1111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[2])||
		(config==1110 && parcel_item.owner ==  data->patches[patch_index].co_ownership[2])||
		(config==1011 && parcel_item.owner ==  data->patches[patch_index].co_ownership[1])||
		(config==111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[1])))
		
		{
			type_position=2;
			if (config==1111 || config==1110)
			{
				data->patches[patch_index].co_ownership[2] =-1;
			}
			else
			{
				data->patches[patch_index].co_ownership[1] =-1;
			}
		}
	
	if ((type_position==-1)&&((config==1111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[3])||
		(config==1011 && parcel_item.owner ==  data->patches[patch_index].co_ownership[2])||
		(config==1101 && parcel_item.owner ==  data->patches[patch_index].co_ownership[2])||
		(config==111 && parcel_item.owner ==  data->patches[patch_index].co_ownership[2])))
		{
			if (config==1111)
			{
				data->patches[patch_index].co_ownership[3] =-1;
			}
			else
			{
				data->patches[patch_index].co_ownership[2]=-1;
			}
			type_position=3;
		}
	//printf("Calling co-ownership resolver %d, (%d, %d), (%d,%d), %d, %d\n",
	//         parcel_item.owner,data->patches[patch_index].co_ownership[0],
	//         data->patches[patch_index].co_ownership[1], xcoor, ycoor, config, type_position);
	
	edge_xcoor=xcoor-1;
	edge_ycoor=ycoor;
	vector<int>::iterator parcel_in_edge_iter;
	if (type_position==0)
	for (edge_iter = parcel_item.edges.begin(); 
		 edge_iter != parcel_item.edges.end();
		 edge_iter++)
	{
		if ((((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor)) && 
		   ((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor+1))) ||
		
		(((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor)) && 
		   ((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor+1))))
		{
			
			parcel_in_edge_iter=find(edge_list[*edge_iter].owner_parcel.begin(),
		                         edge_list[*edge_iter].owner_parcel.end(),parcel_index);
			if (parcel_in_edge_iter!=edge_list[*edge_iter].owner_parcel.end())
				edge_list[*edge_iter].owner_parcel.erase(	parcel_in_edge_iter);	
			parcel_item.edges.erase(edge_iter);
			//printf("delete: (%d,%d) --> (%d,%d)\n", xcoor, ycoor, xcoor, ycoor+1); 
			break;
		}
	}
	
	edge_xcoor=xcoor+1.0;
	edge_ycoor=ycoor;
	if (type_position==1)
	for (edge_iter = parcel_item.edges.begin(); 
		 edge_iter != parcel_item.edges.end();
		 edge_iter++)
	{
		if ((((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor+1)) && 
		   ((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor))) ||
		
		(((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor+1)) && 
		   ((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor))))
		{
			parcel_in_edge_iter=find(edge_list[*edge_iter].owner_parcel.begin(),
		                         edge_list[*edge_iter].owner_parcel.end(),parcel_index);
			if (parcel_in_edge_iter!=edge_list[*edge_iter].owner_parcel.end())
				edge_list[*edge_iter].owner_parcel.erase(	parcel_in_edge_iter);	
			parcel_item.edges.erase(edge_iter);
			//printf("delete: (%d,%d) --> (%d,%d)\n", xcoor+1, ycoor+1, xcoor+1, ycoor); 
			break;
		}
	}
	
	edge_xcoor=xcoor;
	edge_ycoor=ycoor-1.0;
	if (type_position==2)
	for (edge_iter = parcel_item.edges.begin(); 
		 edge_iter != parcel_item.edges.end();
		 edge_iter++)
	{
		if ((((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor)) && 
		   ((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor))) ||
		
		(((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor)) && 
		   ((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor))))
		{
			parcel_in_edge_iter=find(edge_list[*edge_iter].owner_parcel.begin(),
		                         edge_list[*edge_iter].owner_parcel.end(),parcel_index);
			if (parcel_in_edge_iter!=edge_list[*edge_iter].owner_parcel.end())
				edge_list[*edge_iter].owner_parcel.erase(	parcel_in_edge_iter);	
			parcel_item.edges.erase(edge_iter);
			//printf("delete: (%d,%d) --> (%d,%d)\n", xcoor+1, ycoor, xcoor, ycoor); 
			break;
		}
	}
	
	edge_xcoor=xcoor;
	edge_ycoor=ycoor+1.0;
	if (type_position==3)
	for (edge_iter = parcel_item.edges.begin(); 
		 edge_iter != parcel_item.edges.end();
		 edge_iter++)
	{
		if ((((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor+1)) && 
		   ((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor+1))) ||
		
		(((vertex_list[edge_list[*edge_iter].end].xcoor == xcoor) && 
		   (vertex_list[edge_list[*edge_iter].end].ycoor == ycoor+1)) && 
		   ((vertex_list[edge_list[*edge_iter].start].xcoor == xcoor+1) && 
		   (vertex_list[edge_list[*edge_iter].start].ycoor == ycoor+1))))
		{
			parcel_in_edge_iter=find(edge_list[*edge_iter].owner_parcel.begin(),
		                         edge_list[*edge_iter].owner_parcel.end(),parcel_index);
			if (parcel_in_edge_iter!=edge_list[*edge_iter].owner_parcel.end())
				edge_list[*edge_iter].owner_parcel.erase(	parcel_in_edge_iter);	
			parcel_item.edges.erase(edge_iter);
			//printf("delete: (%d,%d) --> (%d,%d)\n", xcoor, ycoor+1, xcoor+1, ycoor+1); 
			break;
		}
	}
	
	//task1 finished
	//now task2
	
	
	switch(config)
	{
		case 1100:
		//it is the the left and right configuration
		if (type_position==0)		
		{
			//this turtle is to the left
			//three edges to be added:
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1, xcoor+0.5,ycoor+1,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+0.5,ycoor, xcoor+0.5,ycoor+1,parcel_index), parcel_index);
			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+0.5,ycoor,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor+1, xcoor+0.5,ycoor+1,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+0.5,ycoor, xcoor+0.5,ycoor+1,parcel_index));*/
		}
		else if (type_position==1)		
		{
			//this turtle is to the right
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+0.5,ycoor, xcoor+1.0,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+0.5,ycoor+1, xcoor+1.0,ycoor+1,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+0.5,ycoor, xcoor+0.5,ycoor+1,parcel_index), parcel_index);
	
			/*parcel_item.edges.push_back(addEdge(xcoor+0.5,ycoor, xcoor+1.0,ycoor,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+0.5,ycoor+1, xcoor+1.0,ycoor+1,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+0.5,ycoor, xcoor+0.5,ycoor+1,parcel_index));*/
		}
			break;
		case 11:
		//it is the up down configiration
		if (type_position==2)		
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1,ycoor, xcoor+1,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+0.5, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
	
			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor,ycoor+0.5,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+1,ycoor, xcoor+1,ycoor+0.5,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor+0.5, xcoor+1.0,ycoor+0.5,parcel_index));*/
			//this turtle is to the up
		}
		else if (type_position==3)		
		{
			//this turtle is to the down
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+0.5, xcoor,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1,ycoor+0.5, xcoor+1,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+0.5, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
	
			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor+0.5, xcoor,ycoor+1.0,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+1,ycoor+0.5, xcoor+1,ycoor+1.0,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor+0.5, xcoor+1.0,ycoor+0.5,parcel_index));*/
		}
		
			break;
		
		case 1010:
		//it is the left and up configiration
		if (type_position==0)		
		{
			//this turtle is to the left
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);

			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+1.0,parcel_index));*/
		}
		else if (type_position==2)		
		{
			//this turtle is to the up
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);

			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor+1.0,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));*/
		}
			break;
		case 1001:
		//it is the left and down configiration
		if (type_position==0)		
		{
			//this turtle is to the left
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor,parcel_index), parcel_index);

			//parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index));
			//parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor,parcel_index));
		}
		else if (type_position==3)		
		{
			//this turtle is to the down
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);

			//parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index));
			//parcel_item.edges.push_back(addEdge(xcoor+1.0,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));
		}
			break;
		case 110:
		//it is the right and up configiration
		if (type_position==1)		
		{
			//this turtle is to the right
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);

			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+1.0,parcel_index));*/
		}
		else if (type_position==2)		
		{
			//this turtle is to the up
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor,ycoor+1.0,parcel_index), parcel_index);

			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor,ycoor+1.0,parcel_index));*/
		}
			break;
		case 101:
		//it is the right and down configiration
		if (type_position==1)		
		{
			//this turtle is to the right
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor,parcel_index), parcel_index);

			/*parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));
			parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor,parcel_index));*/
		}
		else if (type_position==3)		
		{
			//this turtle is to the down
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor,ycoor+1.0,parcel_index), parcel_index);

			//parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor+1.0,ycoor+1.0,parcel_index));
			//parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor,ycoor+1.0,parcel_index));
		}
		break;		
		case 1110:
		if (type_position==0)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
		}
		else if (type_position==1)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
		}
		else if (type_position==2)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+0.5,ycoor+1.0,parcel_index), parcel_index);
		}
		break;
		case 1101:
		if (type_position==0)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+0.5,ycoor,parcel_index), parcel_index);
		}
		else if (type_position==1)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+0.5,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+0.5,ycoor,parcel_index), parcel_index);
		}	
		else if (type_position==3)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+0.5,ycoor,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+0.5,ycoor,parcel_index), parcel_index);
		}
		break;
		case 1011:
		if (type_position==0)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
		}
		else if (type_position==2)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
		}
		else if (type_position==3)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+1.0,ycoor+0.5,parcel_index), parcel_index);
		}
		break;
		case 111:
		if (type_position==1)
		{
			int edge_idx;
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor+1.0,ycoor, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx,xcoor+1.0,ycoor, xcoor,ycoor+0.5);
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor+1.0,ycoor+1.0, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx,xcoor+1.0,ycoor+1.0, xcoor,ycoor+0.5);
		}
		else if (type_position==2)
		{
			int edge_idx;
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor,ycoor, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx, xcoor,ycoor, xcoor,ycoor+0.5);
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor+1.0,ycoor, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx,xcoor+1.0,ycoor, xcoor,ycoor+0.5);
		}
		else if (type_position==3)
		{
			int edge_idx;
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor,ycoor+1.0, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx,xcoor,ycoor+1.0, xcoor,ycoor+0.5);
			addEdgeToParcelItem(parcel_item, edge_idx=addEdge(xcoor+1.0,ycoor+1.0, xcoor,ycoor+0.5,parcel_index), parcel_index);
			//printf("%d add edge_idx:%d (%f,%f)-->(%f,%f)\n", parcel_index, edge_idx,xcoor+1.0,ycoor+1.0, xcoor,ycoor+0.5);
		}
		break;	
		
		case 1111:
		if (type_position==0)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
		}
		else if (type_position==1)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
		}
		else if (type_position==2)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
		}
		else if (type_position==3)
		{
			addEdgeToParcelItem(parcel_item, addEdge(xcoor,ycoor+1.0, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
			addEdgeToParcelItem(parcel_item, addEdge(xcoor+1.0,ycoor+1.0, xcoor+0.5,ycoor+0.5,parcel_index), parcel_index);
		}
		break;
	}
	vector<int>::iterator parcel_iter = find(data->patches[patch_index].co_ownership.begin(), 
	data->patches[patch_index].co_ownership.end(), parcel_item.owner);
	if (parcel_iter == data->patches[patch_index].co_ownership.end())
		finished=true;
	} //end of while
}

void cCSVFileReader::prepareSimplification()
//(x,y) coordinate in the raster image
//[x,y] coordinate in the vector image
{
	int i, j, otherSideIdx;
	int xcoor,ycoor; //patch coor in raster images 
	int owner_this;  //the id of turtle 
	S_Parcel parcel_item;
	S_Parcel vertex_item;
	S_Edge edge_item;
	int index_parcel=0;
	int type;
	//parcel_area_orig.clear();
	//printf("size %d\n",data->building_turtles.size());
	for (i=0; i<data->building_turtles.size(); i++)
	{
		
		//printf("%d\n",i);
		//printf("%d Starting\n ",i);
		//for each of the building turtle:
		//we should be able to find out the all the vertex and edges which belong to this parcel
		//patch:(0,0) --> contains vertice (0,0), (1,0), (0,1), (1,1)
		owner_this = data->building_turtles[i].id;
		parcel_item.owner = owner_this;
		parcel_item.edges.clear();
		type=data->building_turtles[i].type;

		for (j=0; j<data->building_turtles[i].patchCoords.size(); j++)
		{
			//for all the patches inside this parcel
			xcoor = data->building_turtles[i].patchCoords[j].xcor;
			ycoor = data->building_turtles[i].patchCoords[j].ycor;
			int patch_index;
			if ((patch_index=getPatchIndex(xcoor, ycoor))!=-1)
			{
				if (data->patches[patch_index].co_ownership.size()>0)
				{
					continue;
				}
			}
			//if ( index_parcel==31)
			//	printf("Patches: (%d,%d)\n",xcoor,ycoor);
			//now we need to figure out which one for its 4 edges should be in the edge list
			//make sure the other side patch is valid
			
			//first [xcoor,ycoor]->[xcoor,ycoor+1],
			//the other side patch coor (xcoor-1,ycoor)
			if ((otherSideIdx=getPatchIndex(xcoor-1,ycoor))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner!=owner_this) //the other side patch is belong to the different parcel
				{
					//this is an outside edge;
					parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor,ycoor+1,index_parcel));
				}
			}
			/*else
			{
				parcel_item.edges.push_back(addEdge(xcoor,ycoor, xcoor,ycoor+1,index_parcel));
			}*/
			
			//second [xcoor,ycoor+1]->[xcoor+1,ycoor+1],
			//the other side patch coor (xcoor,ycoor+1)
			
			if ((otherSideIdx=getPatchIndex(xcoor,ycoor+1))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner!=owner_this) //the other side patch is belong to the different parcel
				{
					//this is an outside edge;
					parcel_item.edges.push_back(addEdge(xcoor,ycoor+1, xcoor+1,ycoor+1,index_parcel));
				}
			}
			/*else
			{
				parcel_item.edges.push_back(addEdge(xcoor,ycoor+1, xcoor+1,ycoor+1,index_parcel));
			}*/
			
			//third [xcoor+1,ycoor+1]->[xcoor+1,ycoor],
			//the other side patch coor (xcoor+1,ycoor)
			
			if ((otherSideIdx=getPatchIndex(xcoor+1,ycoor))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner!=owner_this) //the other side patch is belong to the different parcel
				{
					//this is an outside edge;
					parcel_item.edges.push_back(addEdge(xcoor+1,ycoor+1, xcoor+1,ycoor,index_parcel));
				}
			}
			/*else
			{
				parcel_item.edges.push_back(addEdge(xcoor+1,ycoor+1, xcoor+1,ycoor,index_parcel));
			}*/
			
			//fourth [xcoor+1,ycoor]->[xcoor,ycoor],
			//the other side patch coor (xcoor,ycoor-1)
			
			if ((otherSideIdx=getPatchIndex(xcoor,ycoor-1))!=-1) //valid
			{
				if (data->patches[otherSideIdx].owner!=owner_this) //the other side patch is belong to the different parcel
				{
					//this is an outside edge;
					parcel_item.edges.push_back(addEdge(xcoor+1,ycoor, xcoor,ycoor,index_parcel));
				}
			}
			/*else
			{
				parcel_item.edges.push_back(addEdge(xcoor+1,ycoor, xcoor,ycoor,index_parcel));
			}*/
		}//end of parce this 
		
		//now it is time for us to deal with co-ownership ones
		for (j=0; j<data->building_turtles[i].patchCoords.size(); j++)
		{
			//for all the patches inside this parcel
			xcoor = data->building_turtles[i].patchCoords[j].xcor;
			ycoor = data->building_turtles[i].patchCoords[j].ycor;
			int patch_index;
			if ((patch_index=getPatchIndex(xcoor, ycoor))!=-1)
			{
				if (data->patches[patch_index].co_ownership.size()>0)
				{
					prepareSimplify_co_ownership(parcel_item, patch_index, index_parcel);
					//printf("end of call\n");
				}
			}
		}
		//printf("end of this parcel %d\n", index_parcel);
		
		//this will arrange the edges in the counter-clock wire order.
		//this will get called when all the parcel patches are 4-connected.
		
		/*{printf("\nParcel: %d initially\n", parcel_item.owner);
			for (int i=0; i<parcel_item.edges.size(); i++)
			{
				printf("%d:  %d(%f,%f)-->%d(%f,%f)\n", parcel_item.edges[i],
				       edge_list[parcel_item.edges[i]].start, 
				       vertex_list[edge_list[parcel_item.edges[i]].start].xcoor,
					   vertex_list[edge_list[parcel_item.edges[i]].start].ycoor,
				      edge_list[parcel_item.edges[i]].end, 
				       vertex_list[edge_list[parcel_item.edges[i]].end].xcoor,
					   vertex_list[edge_list[parcel_item.edges[i]].end].ycoor);
			}
		}*/
		deleteDuplicateEdgePre(parcel_item, index_parcel);
		/*if (index_parcel==31)
		{
			printf("%d\n",parcel_item.edges.size());
			for (j=0; j<parcel_item.edges.size(); j++)
			{
				printf("%d(%f, %f)->%d(%f, %f)\n", 
	edge_list[parcel_item.edges[j]].start, vertex_list[edge_list[parcel_item.edges[j]].start].xcoor,
								   	   vertex_list[edge_list[parcel_item.edges[j]].start].ycoor,
	edge_list[parcel_item.edges[j]].end, vertex_list[edge_list[parcel_item.edges[j]].end].xcoor,
								   	   vertex_list[edge_list[parcel_item.edges[j]].end].ycoor);
			}
		}*/
		if ((sortEdgesInParcel(parcel_item.edges)==-1)||(parcel_item.edges.empty()))
		{
			//this parcel is wrong(wrapped around?);
			/*printf("\nParcel: %d Wrong\n", parcel_item.owner);
			for (int i=0; i<parcel_item.edges.size(); i++)
			{
				printf("%d:  %d(%f,%f)-->%d(%f,%f)\n", 
					   parcel_item.edges[i],
				       edge_list[parcel_item.edges[i]].start, 
				       vertex_list[edge_list[parcel_item.edges[i]].start].xcoor,
					   vertex_list[edge_list[parcel_item.edges[i]].start].ycoor,
				      edge_list[parcel_item.edges[i]].end, 
				       vertex_list[edge_list[parcel_item.edges[i]].end].xcoor,
					   vertex_list[edge_list[parcel_item.edges[i]].end].ycoor);
			}*/
			continue;
			//exit(-1);
		}
		
		//deleteDuplicateEdge(parcel_item, index_parcel);
		
		/*if (index_parcel==31)
		{
			cout<<"after sorting"<<endl;
			printf("%d\n",parcel_item.edges.size());
			for (j=0; j<parcel_item.edges.size(); j++)
			{
				printf("%d(%f, %f)->%d(%f, %f)\n", 
	edge_list[parcel_item.edges[j]].start, vertex_list[edge_list[parcel_item.edges[j]].start].xcoor,
								   	   vertex_list[edge_list[parcel_item.edges[j]].start].ycoor,
	edge_list[parcel_item.edges[j]].end, vertex_list[edge_list[parcel_item.edges[j]].end].xcoor,
								   	   vertex_list[edge_list[parcel_item.edges[j]].end].ycoor);
			}
		}*/
		//parcel_item.parcel_id = i;
		parcel_item.parcel_id = index_parcel;
		parcel_item.owner = owner_this;
		parcel_item.type = type;
		parcel_item.area = data->building_turtles[i].patchCoords.size();
		//parcel_area_orig.push_back(parcel_item.area);
		parcel_list.push_back(parcel_item);  
		//after this, this parcel have 
		index_parcel++;
	}
	parcel_area_orig = new double[index_parcel];
	for (i=0; i<index_parcel; i++)
	{
		parcel_area_orig[i]=parcel_list[i].area;
	}

}

void cCSVFileReader::addEdgeToParcelItem(S_Parcel & parcel_item, int edge_idx, int ownerParcel)
{
	//add the newly generated edge: edge_idx to parcel_item;
	
	vector<int>::iterator edge_iter;
	vector<int>::iterator parcel_in_edge_iter;
	edge_iter=find(parcel_item.edges.begin(), parcel_item.edges.end(), edge_idx);
	if (edge_iter!=parcel_item.edges.end())
	{
		parcel_item.edges.erase(edge_iter);
		parcel_in_edge_iter=find(edge_list[edge_idx].owner_parcel.begin(),
		                         edge_list[edge_idx].owner_parcel.end(),ownerParcel);
		if (parcel_in_edge_iter!=edge_list[edge_idx].owner_parcel.end())
			edge_list[edge_idx].owner_parcel.erase(	parcel_in_edge_iter);	
	}
	else	
	{
		parcel_item.edges.push_back(edge_idx);
	}
}

int cCSVFileReader::addEdge(double x1,double y1, double x2, double y2, int ownerParcel)
//this function will add the input point pair into vertex list first
//then register this pair in the global edge list
//finally return this edge index back to its owner 
{
	//if (ownerParcel==2)
	//printf("Calling addEdge (%f %f) (%f %f), %d\n", x1, y1, x2, y2, ownerParcel);
	int pt1_idx=-1;
	int pt2_idx=-1;
	int i,j;
	int edge_idx;
	S_Vertex vertex_item;
	S_Edge edge_item;
	//first find if the two input points are already in the vertex list
	i=vertex_list.size()-1;
	while (i>=0)
	{
		if ((fabs(x1-vertex_list[i].xcoor)<0.000001) && 
			(fabs(y1-vertex_list[i].ycoor)<0.000001))
		{
			pt1_idx = i;
			break;
		}
		
		i--;
	}
	i=vertex_list.size()-1;
	while (i>=0)
	{
		if ((fabs(x2-vertex_list[i].xcoor)<0.000001) && 
			(fabs(y2-vertex_list[i].ycoor)<0.000001))
		{
			pt2_idx = i;
			break;
			//found pt2;
		}
		i--;
	}	
	if (pt1_idx==-1)
	{
		//no point 1 in the vertex list
		//push_it back and assign pt1_idx = list.size();
		vertex_item.vertex_id = pt1_idx = vertex_list.size();
		vertex_item.xcoor = x1;
		vertex_item.ycoor = y1;
		//postpone assign edge index for now: vertex_item.edges
		vertex_item.edges.clear();
		vertex_list.push_back(vertex_item);		
	}
	if (pt2_idx==-1)
	{
		//no point 1 in the vertex list
		//push_it back and assign pt1_idx = list.size();
		vertex_item.vertex_id = pt2_idx = vertex_list.size();
		vertex_item.xcoor = x2;
		vertex_item.ycoor = y2;
		//postpone assign edge index for now: vertex_item.edges
		vertex_item.edges.clear();
		vertex_list.push_back(vertex_item);		
	}
	
	//now we should add this edge

	for (i=edge_list.size()-1; i>=0; i--)
	{
		if (((edge_list[i].start == pt1_idx) && (edge_list[i].end == pt2_idx)) ||
			((edge_list[i].end == pt1_idx) && (edge_list[i].start == pt2_idx)))
		{
			edge_idx = i;
			break;
		}
	}
	if (i==-1) //didn't find it
	{
		//need to create a new entry for the edge
		edge_item.edge_id = edge_idx = edge_list.size();	
		edge_item.start = pt1_idx;
		edge_item.end = pt2_idx;
		edge_item.owner_parcel.push_back(ownerParcel);	
		edge_list.push_back(edge_item);
		
		//since we are adding a new edge now we need to go back to update the vertex_list
		vertex_list[pt1_idx].edges.push_back(edge_item.edge_id);
		vertex_list[pt2_idx].edges.push_back(edge_item.edge_id);
	}
	else 
	{
		//we have this edge in the list
		vector<int>::iterator parcel_iter = find(edge_list[i].owner_parcel.begin(),
		edge_list[i].owner_parcel.end(), ownerParcel);
		if (parcel_iter==edge_list[i].owner_parcel.end())
			edge_list[i].owner_parcel.push_back(ownerParcel);
		edge_idx = i;
	}
	//if (ownerParcel==0)
	//	printf(" %d\n",edge_idx);
	return edge_idx;
}
void cCSVFileReader::deleteDuplicateEdgePre(S_Parcel & parcel_item, int parcel_index)
{
	//preventing the duplicated half edge from emerging
	//for each possible bad edge search for the duplicate edge
	vector<int> &InputEdges = parcel_item.edges;
	vector<int>::iterator edge_iter;
	vector<int>::iterator duplicate_edge_iter;
	vector<int> tobe_erased;
	vector<int> tobe_added;
	double x1,y1,x2,y2,new_x1,new_y1,new_x2,new_y2;
	double candidate_x1, candidate_y1,candidate_x2, candidate_y2,
	       repair_x1, repair_x2, repair_y1, repair_y2;
	bool found;
	//printf("In Parcel %d:\n", parcel_item.owner);
	for (edge_iter = InputEdges.begin();
		edge_iter!= InputEdges.end();
	    edge_iter++)
	{
		x1 = vertex_list[edge_list[*edge_iter].start].xcoor;
		y1 = vertex_list[edge_list[*edge_iter].start].ycoor;
		x2 = vertex_list[edge_list[*edge_iter].end].xcoor;
		y2 = vertex_list[edge_list[*edge_iter].end].ycoor;
		found =false;
		if ((x1==x2 && fabs(y1-y2)==0.5) ||
			(y1==y2 && fabs(x1-x2)==0.5))
		{
			//now it is the trick.
				//we need to find out.
				if (x1==x2)
				{
					candidate_x1 = x1;
					candidate_x2 = x1;
					repair_x1 =x1;
					repair_x2 =x1;
					//it is in y direction
					if (floor(y1)!=y1)
					{
						if (y1>y2)
						{
							candidate_y1 = y2;
							candidate_y2 =y2+1.0;
							repair_y1 =y1;
							repair_y2 =y2+1.0;
						}
						else
						{
							candidate_y1 = y2;
							candidate_y2 =y2-1.0;
							repair_y1 =y1;
							repair_y2 =y2-1.0;
						}
					}
					else //y2 is 0.5
					{
						if (y1>y2)
						{
							candidate_y1 = y1;
							candidate_y2 = y1-1.0;
							repair_y1 =y2;
							repair_y2 =y1-1.0;
						}
						else
						{
							candidate_y1 = y1;
							candidate_y2 = y1+ 1.0;
							repair_y1 =y2;
							repair_y2 =y1+1.0;
						}
					}
				}
				else
				{
					candidate_y1 = y1;
					candidate_y2 = y1;
					repair_y1 =y1;
					repair_y2 =y1;
					//it is in y direction
					if (floor(x1)!=x1)
					{
						if (x1>x2)
						{
							candidate_x1 = x2;
							candidate_x2 =x2+1.0;
							repair_x1 =x1;
							repair_x2 =x2+1.0;
						}
						else
						{
							candidate_x1 = x2;
							candidate_x2 =x2-1.0;
							repair_x1 =x1;
							repair_x2 =x2-1.0;
						}
					}
					else //x2 is 0.5
					{
						if (x1>x2)
						{
							candidate_x1 = x1;
							candidate_x2 = x1-1.0;
							repair_x1 =x2;
							repair_x2 =x1-1.0;
						}
						else
						{
							candidate_x1 = x1;
							candidate_x2 = x1+ 1.0;
							repair_x1 =x2;
							repair_x2 =x1+1.0;
						}
					}
				}
			//printf("To be evaluated: %d:  %d(%f, %f) --> %d(%f, %f)\n", *edge_iter, edge_list[*edge_iter].start,x1, y1, edge_list[*edge_iter].end,x2, y2);	
			for (duplicate_edge_iter = InputEdges.begin();
				 duplicate_edge_iter != InputEdges.end();
			     duplicate_edge_iter++
				)
			{
				//search for the possible duplicate edges
				if (duplicate_edge_iter==edge_iter) continue;
				new_x1 = vertex_list[edge_list[*duplicate_edge_iter].start].xcoor;
				new_y1 = vertex_list[edge_list[*duplicate_edge_iter].start].ycoor;
				new_x2 = vertex_list[edge_list[*duplicate_edge_iter].end].xcoor;
				new_y2 = vertex_list[edge_list[*duplicate_edge_iter].end].ycoor;
				
				if (((new_x1 == candidate_x1) && 
				   (new_y1 == candidate_y1) &&
				   (new_x2 == candidate_x2) &&
				   (new_y2 == candidate_y2)) ||
				   ((new_x1 == candidate_x2) && 
				   (new_y1 == candidate_y2) &&
				   (new_x2 == candidate_x1) &&
				   (new_y2 == candidate_y1)))
				{
					//bingo
					//now we need remove this one and push back the new repair one
					//printf("Erase: %d:  %d(%f, %f) --> %d(%f, %f)\n", *duplicate_edge_iter, edge_list[*duplicate_edge_iter].start, new_x1, new_y1, edge_list[*duplicate_edge_iter].end,new_x2, new_y2);
					//printf("To be added: (%f, %f) --> (%f, %f)\n", repair_x1, repair_y1, repair_x2, repair_y2);	
					tobe_erased.push_back(*duplicate_edge_iter);
					tobe_erased.push_back(*edge_iter);
					
					//InputEdges.erase(duplicate_edge_iter);
					//InputEdges.erase(edge_iter);
					int new_edge=addEdge(repair_x1,repair_y1, repair_x2,repair_y2, parcel_index);
					//addEdgeToParcelItem(parcel_item,new_edge,parcel_index);
					//printf("To be added: %d:  %d(%f, %f) --> %d(%f, %f)\n", new_edge, edge_list[new_edge].start, 
					//vertex_list[edge_list[new_edge].start].xcoor, vertex_list[edge_list[new_edge].start].ycoor, 
					//edge_list[new_edge].end, vertex_list[edge_list[new_edge].end].xcoor, vertex_list[edge_list[new_edge].end].ycoor);
					
					tobe_added.push_back(new_edge);
					found=true;
					break;
				}
			}
		}
	}
	vector<int>::iterator erase_edge_iter;
	vector<int>::iterator parcel_in_edge_iter;
	for (int i=0; i<tobe_erased.size(); i++)
	{
		erase_edge_iter = find(InputEdges.begin(),InputEdges.end(),tobe_erased[i]);
		//printf("Erase:%d...\n", tobe_erased[i]);
		if (erase_edge_iter!=InputEdges.end())
			InputEdges.erase(erase_edge_iter);
		
		//we also need to erase the parcel_item in that edge's affected parcel_list
		parcel_in_edge_iter=find(edge_list[tobe_erased[i]].owner_parcel.begin(),
		                         edge_list[tobe_erased[i]].owner_parcel.end(),parcel_index);
		if (parcel_in_edge_iter!=edge_list[tobe_erased[i]].owner_parcel.end())
			edge_list[tobe_erased[i]].owner_parcel.erase(	parcel_in_edge_iter);	
	}
	for (int i=0; i<tobe_added.size(); i++)
	{
		addEdgeToParcelItem(parcel_item,tobe_added[i],parcel_index);
	}
}
void cCSVFileReader::deleteDuplicateEdge(S_Parcel & parcel_item, int parcel_index)
{
	vector<int> &InputEdges = parcel_item.edges;
	deque<int> tempEdgeList;
	
	vector<int>::iterator edge_iter = InputEdges.begin();
	double prevCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
	double prevCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
	edge_iter++;
	double curStartCoorX,curStartCoorY, curEndCoorX,curEndCoorY;
	
	int prevEnd = edge_list[*edge_iter].end;
	
	tempEdgeList.clear();
	tempEdgeList.push_back(*edge_iter);
	edge_iter++;
	vector<int> edge_tobe_removed;
	edge_tobe_removed.clear();

	for (;edge_iter!=InputEdges.end();edge_iter++)
	{
		
		if (edge_list[*edge_iter].start == prevEnd)
		{	
			curStartCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
			prevEnd = edge_list[*edge_iter].end;
			
		}
		else
		{
			curStartCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
			prevEnd = edge_list[*edge_iter].start;
		}
		
		/*if  ( (( prevCoorX ==curStartCoorX && curStartCoorX==curEndCoorX)||
			   ( prevCoorY ==curStartCoorY && curStartCoorY==curEndCoorY)
			  )
			&&(((prevCoorX == (curStartCoorX + curEndCoorX) /2.0) && 
		     (prevCoorY == (curStartCoorY + curEndCoorY) /2.0) )
			||((curEndCoorX == (curStartCoorX + prevCoorX) /2.0) && 
		     (curEndCoorY == (curStartCoorY + prevCoorY) /2.0) )))
		*/
		if (( (fabs(prevCoorX-curStartCoorX)<0.00000001 && fabs(curStartCoorX-curEndCoorX)<0.00000001) && 
			  (
			   (fabs(prevCoorY*2.0-(curStartCoorY + curEndCoorY))<0.0000001)||
		       (fabs(curEndCoorY*2.0 -(curStartCoorY + prevCoorY))<0.0000001)
		      )
			 ) ||
			( (fabs(prevCoorY-curStartCoorY)<0.00000001 && fabs(curStartCoorY-curEndCoorY)<0.00000001) && 
			  (
			   (fabs(prevCoorX*2.0-(curStartCoorX + curEndCoorX))<0.0000001)||
		       (fabs(curEndCoorX*2.0 -(curStartCoorX + prevCoorX))<0.0000001)
		      )
			 )
		   )
			{
				printf("deleting (%f,%f),(%f,%f), (%f,%f)\n", prevCoorX, prevCoorY,curStartCoorX, curStartCoorY, curEndCoorX, curEndCoorY);
				tempEdgeList.pop_back();
				edge_tobe_removed.push_back(*edge_iter);
				edge_tobe_removed.push_back(*(edge_iter-1));
				tempEdgeList.push_back(addEdge(prevCoorX, prevCoorY, curEndCoorX, curEndCoorY,parcel_index));
				prevCoorX = curEndCoorX;
				prevCoorY = curEndCoorY;
			}
		else
		{
			tempEdgeList.push_back(*edge_iter);
			prevCoorX = curStartCoorX;
			prevCoorY = curStartCoorY;
		}
		
	}
	if (edge_iter == InputEdges.end())
	{
		edge_iter = InputEdges.begin();
		if (edge_list[*edge_iter].start == prevEnd)
		{	
			curStartCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
		}
		else
		{
			curStartCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
		}
		
		/*if  ( (( prevCoorX ==curStartCoorX && curStartCoorX==curEndCoorX)||
			   ( prevCoorY ==curStartCoorY && curStartCoorY==curEndCoorY)
			  )
			&&(((prevCoorX == (curStartCoorX + curEndCoorX) /2.0) && 
		     (prevCoorY == (curStartCoorY + curEndCoorY) /2.0) )
			||((curEndCoorX == (curStartCoorX + prevCoorX) /2.0) && 
		     (curEndCoorY == (curStartCoorY + prevCoorY) /2.0) )))
	*/
		if (( (fabs(prevCoorX-curStartCoorX)<0.00000001 && fabs(curStartCoorX-curEndCoorX)<0.00000001) && 
			  (
			   (fabs(prevCoorY*2.0-(curStartCoorY + curEndCoorY))<0.0000001)||
		       (fabs(curEndCoorY*2.0 -(curStartCoorY + prevCoorY))<0.0000001)
		      )
			 ) ||
			( (fabs(prevCoorY-curStartCoorY)<0.00000001 && fabs(curStartCoorY-curEndCoorY)<0.00000001) && 
			  (
			   (fabs(prevCoorX*2.0-(curStartCoorX + curEndCoorX))<0.0000001)||
		       (fabs(curEndCoorX*2.0 -(curStartCoorX + prevCoorX))<0.0000001)
		      )
			 )
		   )
		
			{
				printf("deleting (%f,%f),(%f,%f), (%f,%f)\n", prevCoorX, prevCoorY,curStartCoorX, curStartCoorY, curEndCoorX, curEndCoorY);
				tempEdgeList.pop_front();
				tempEdgeList.pop_back();
				tempEdgeList.push_back(addEdge(prevCoorX, prevCoorY, curEndCoorX, curEndCoorY,parcel_index));
			}
	}
	
	/*{
		prevCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
		prevCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
		edge_iter = InputEdges.end()-1;
		prevEnd=edge_list[*edge_iter].start;
		if (edge_list[*edge_iter].start == prevEnd)
		{	
			curStartCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
			prevEnd = edge_list[*edge_iter].end;
			
		}
		else
		{
			curStartCoorX = vertex_list[edge_list[*edge_iter].end].xcoor;
			curStartCoorY = vertex_list[edge_list[*edge_iter].end].ycoor;
			curEndCoorX = vertex_list[edge_list[*edge_iter].start].xcoor;
			curEndCoorY = vertex_list[edge_list[*edge_iter].start].ycoor;
			prevEnd = edge_list[*edge_iter].start;
		}
		
		if  ( (( prevCoorX ==curStartCoorX && curStartCoorX==curEndCoorX)||
			   ( prevCoorY ==curStartCoorY && curStartCoorY==curEndCoorY)
			  )
			&&(((prevCoorX == (curStartCoorX + curEndCoorX) /2.0) && 
		     (prevCoorY == (curStartCoorY + curEndCoorY) /2.0) )
			||((curEndCoorX == (curStartCoorX + prevCoorX) /2.0) && 
		     (curEndCoorY == (curStartCoorY + prevCoorY) /2.0) )))
			{
				printf("deleting (%f,%f),(%f,%f), (%f,%f)\n", prevCoorX, prevCoorY,curStartCoorX, curStartCoorY, curEndCoorX, curEndCoorY);
				tempEdgeList.pop_front();
				tempEdgeList.pop_back();
				tempEdgeList.push_back(addEdge(prevCoorX, prevCoorY, curEndCoorX, curEndCoorY,parcel_index));
			}
	}*/
	InputEdges.clear();
	for (int i=0; i<tempEdgeList.size(); i++)
	{
		InputEdges.push_back(tempEdgeList[i]);
	}
	
	//TODO, Remove the parcel number from the edge's parcel list.
}
int cCSVFileReader::sortEdgesInParcel(vector<int> & InputEdges)
//assumption InputEdges is not empty, it contains a close loop of all the boundary edges
{
	int i,j;
	int startPt, endPt;
	vector<int> tempEdgeList;
	tempEdgeList.push_back(InputEdges[0]);
	startPt = edge_list[InputEdges[0]].start;   
	//this is the index in the vertext_list for start point of this edge
	endPt = edge_list[InputEdges[0]].end;
		
	int prevIndex = 0;
	bool FoundNextEdgeFlag;
	bool done=false;
	while((tempEdgeList.size()<InputEdges.size())&&(!done))
	{
		FoundNextEdgeFlag=false;
		for (i=0; i<InputEdges.size(); i++)
		{
			
			if (i==prevIndex)  //don't want to compare with itself
				continue;
			if (endPt==edge_list[InputEdges[i]].start)
			{
				//printf("%d(%f,%f)\n", endPt, vertex_list[endPt].xcoor,vertex_list[endPt].ycoor);
				//now we found the sucessor of the previous pt
				tempEdgeList.push_back(InputEdges[i]);
				prevIndex = i;
				endPt = edge_list[InputEdges[i]].end;
				FoundNextEdgeFlag=true;
				break;
			}
			if (endPt==edge_list[InputEdges[i]].end) 
			{
				//printf("%d(%f,%f)\n", endPt, vertex_list[endPt].xcoor,vertex_list[endPt].ycoor);
				//now we found the sucessor of the previous pt
				tempEdgeList.push_back(InputEdges[i]);
				prevIndex = i;
				endPt = edge_list[InputEdges[i]].start;
				FoundNextEdgeFlag=true;
				break;
			}
		}
		if (!FoundNextEdgeFlag)
		{
			return -1; //wrong polygon
		}
		else			
		{
			//found one, need to check if it is the start point
			if (endPt == startPt)
				done=true;
		}
	}	

	//after we rearrange the tempEdgeList, we need to assign value back to the InputEdges.
	InputEdges.clear();
	for (i=0; i<tempEdgeList.size(); i++)
	{
		InputEdges.push_back(tempEdgeList[i]);
	}
	
	//now we need to traverse it once ti see if there are any edges we need to delete.
	
	return 0;
}

double cCSVFileReader::calcNewAreaIfEdgeContracted(int parcelID, int edgeID)
{
	//calculate particularly for this parcel, the new area will going to happen
	//if we contract the edge; the two vertices will move to the midpoint of that edge
	//all other edge affected will move (not actually here, since we are only evaluating
	//the calculate the area change.
	
	/*struct S_LINE
	{
		double A;
		double B;
		double C;
	};
	
	vector<S_LINE> line_equations;
	S_LINE line_item;
	*/
	S_Parcel currentParcel = parcel_list[parcelID];
	double currentArea = currentParcel.area;
	double newArea = currentArea;
	S_Edge edgeEvaluated = edge_list[edgeID];
	S_Vertex v1 = vertex_list[edgeEvaluated.start];
	S_Vertex v2 = vertex_list[edgeEvaluated.end];
	int prevEdge_idx;
	int nextEdge_idx;
	
	double area1,area2; //the area of newly generated triangle if the edge is contracted
	
	//for the possible new position for v1, v2 after edge contraction, to simplify we choose the 
	//mid point
	
	double new_xcoor = (v1.xcoor + v2.xcoor)/2.0;
	double new_ycoor = (v1.ycoor + v2.ycoor)/2.0;
	
	//first of all for each parcel, if we contract one edge, the two vertex is moving to the midpoint
	//then 
		
	int i;
	
	//first find this edge
	for (i=0; i<currentParcel.edges.size(); i++)
	{
		if (currentParcel.edges[i]==edgeID)
			break;
	}
	
	if (i==currentParcel.edges.size())
	{
		//if we cannot find this edge in the parcel to be evaluate against
		//it means this parcel is not one of the owner of the edges, it is a dragging process.
		return calcNewAreaForDraggingVertex(parcelID, edgeEvaluated.start, edgeEvaluated.end, new_xcoor, new_ycoor);
	}
	//now i is the edge to be contracted. since in the parcel edge list are in order
	//we can find the prev edge and next edge
	if (i!=0) 
	{
		prevEdge_idx = currentParcel.edges[i-1];
	}
	else 
		prevEdge_idx = currentParcel.edges[currentParcel.edges.size()-1];
	if (i!=currentParcel.edges.size()-1)
	{
		nextEdge_idx = currentParcel.edges[i+1];
	}
	else
		nextEdge_idx =currentParcel.edges[0];
	
	//if ((parcelID==2) && (edgeID==2))
	//	printf("Prev: %d Next %d \n", prevEdge_idx,nextEdge_idx);
	
	//we know prevEdge_idx and nextEdge_idx are corresponding to v1 and v2 respectively
	//but we need to find out exactly, may need swapping 
	//after this prevEdge contain v1 and another vertice as end point
	/*if ((edgeEvaluated.start == edge_list[prevEdge_idx].end) || (edgeEvaluated.start == edge_list[prevEdge_idx].start))
	{
		//good we don't need to do any thing here
	}
	else if ((edgeEvaluated.start == edge_list[nextEdge_idx].end) || (edgeEvaluated.start == edge_list[nextEdge_idx].start))
	{
		//nextEdge_idx contains v1;
		
	}
	else
	{
		printf("sth is really wrong, parcel ID:%d, edge ID:%d\n", parcelID, edgeID);
	}
	
	if ((edgeEvaluated.end == edge_list[prevEdge_idx].end) || (edgeEvaluated.end == edge_list[prevEdge_idx].start))
	{
		//good we don't need to do any thing here
	}
	else if ((edgeEvaluated.end == edge_list[nextEdge_idx].end) || (edgeEvaluated.end == edge_list[nextEdge_idx].start))
	{
		//nextEdge_idx contains v1;
		
	}
	else
	{
		printf("sth is really wrong, parcel ID:%d, edge ID:%d\n", parcelID, edgeID);
	}
	
	*/
	
	//now we need to construct the line equation list for this parcel;
	//after this loop, we have all the line equation of all the 
	
	
	//pos p, lineSeg_p1, LingSeg_p2;
	
	/*for (i=0; i<currentParcel.edges.size(); i++)
	{
		p1.x = vertex_list[currentParcel.edges[i].start].xcoor;
		p1.y = vertex_list[currentParcel.edges[i].start].ycoor;
		
		p2.x = vertex_list[currentParcel.edges[i].end].xcoor;
		p2.y = vertex_list[currentParcel.edges[i].end].ycoor;
		two_point_line(p1, p2, &(line_item.A), &(line_item.B),&(line_item.C));
		line_equations.push_back(line_item);		
	}*/
	
	
	
	//now it is time to calculate the two new triangle areas:
	//first one: (new_xcoor,new_ycoor) -> vertex_list[edge_list[prevEdge_idx].start] ->  vertex_list[edge_list[prevEdge_idx].end] 
	area1 = calcTriangleArea(new_xcoor,new_ycoor,
							vertex_list[edge_list[prevEdge_idx].start].xcoor,
							vertex_list[edge_list[prevEdge_idx].start].ycoor,
							vertex_list[edge_list[prevEdge_idx].end].xcoor,
							vertex_list[edge_list[prevEdge_idx].end].ycoor);
	//we need to find if the triangle is inside or outside
	//inside: area is negative
	//outside: area	is positive
	//we are doing this by shooting a ray from the new midpoint to any midpoint
	//of any edge of this parcel
	
	/*pos p1,p2;
	if ((edge_list[prevEdge_idx].start == edgeEvaluated.start) || (edge_list[prevEdge_idx].start == edgeEvaluated.end))
	{
		//now the prevEdge's start point is one of our v1, v2;
		//the other end of prevEdge should used with the new coordinate to find out the middle point
		//to shoot out the ray
		p1.x = (new_xcoor + vertex_list[edge_list[prevEdge_idx].end].xcoor) /2.0;
		p1.y = (new_ycoor + vertex_list[edge_list[prevEdge_idx].end].ycoor) /2.0;
	}		
	else //otherwise it is the end point is one of the v1, v2,
	{
		p1.x = (new_xcoor + vertex_list[edge_list[prevEdge_idx].start].xcoor) /2.0;
		p1.y = (new_ycoor + vertex_list[edge_list[prevEdge_idx].start].ycoor) /2.0;
	}
	
	//once I have the midpoint(p1) we can shoot ray from it to any edge midpoint of this parcel
	//here any edge, we can just use the first one
	
	p2.x = (vertex_list[edge_list[currentParcel.edges[0]].start].xcoor +
			vertex_list[edge_list[currentParcel.edges[0]].end].xcoor) /2.0;
	p2.y = (vertex_list[edge_list[currentParcel.edges[0]].start].ycoor +
			vertex_list[edge_list[currentParcel.edges[0]].end].ycoor) / 2.0;
	
	double A,B,C; //the line equation for the ray shooting from the midpoint of the new edge of the newly generated triangle 
				  //to the midpoint of the parcel's first edge
	//now we need to calculate the A,B,C value for this ray
	two_point_line(p1,p2,&A,&B,&C);
	
	//then evaluate this line against all the edges(line segment) to find out how many intersections
	int numIntersection=0;
	pos pInter, pPrev; //the intersection point
	pPrev.x = -100.0;
	pPrev.y = -100.0;
	//since we know for sure the first intersection point is the midpoint of the first edge of this parcel
	//which will be not the 
	for (i=0; i<currentParcel.edges.size(); i++)
	{
		p1.x = vertex_list[currentParcel.edges[i].start].xcoor;
		p1.y = vertex_list[currentParcel.edges[i].start].ycoor;
		
		p2.x = vertex_list[currentParcel.edges[i].end].xcoor;
		p2.y = vertexsolve_lines_segments_list[currentParcel.edges[i].end].ycoor;
		
		if (solve_lines_segments(p1,p2,A,B,C,&pInter)) //we got an intersection
		{
			//here we need to deal with special cases
			//where the intersection point is just the connect point
			//
			numIntersection++;
		}
	}
	*/
	
	
	double px, py; //the x,y coordinate of the midpoint of newly generated edge
	if ((edge_list[prevEdge_idx].start == edgeEvaluated.start) || (edge_list[prevEdge_idx].start == edgeEvaluated.end))
	{
		
		px = (new_xcoor + vertex_list[edge_list[prevEdge_idx].end].xcoor) /2.0;
		py = (new_ycoor + vertex_list[edge_list[prevEdge_idx].end].ycoor) /2.0;
	}		
	else //otherwise it is the end point is one of the v1, v2,
	{
		px = (new_xcoor + vertex_list[edge_list[prevEdge_idx].start].xcoor) /2.0;
		py = (new_ycoor + vertex_list[edge_list[prevEdge_idx].start].ycoor) /2.0;
	}
	/*if ((parcelID==0) && (edgeID == 8))
	{
		printf("+-+-%f,%f+-+-\n", px,py);
	}*/
	if (PointInParcel(px,py,parcelID))
		area1 = - area1; //the triangle is inside
	newArea += area1;
	
	//second one: (new_xcoor,new_ycoor) -> vertex_list[edge_list[nextEdge_idx].start] ->  vertex_list[edge_list[nextEdge_idx].end] 
	area2 = calcTriangleArea(new_xcoor,new_ycoor,
							vertex_list[edge_list[nextEdge_idx].start].xcoor,
							vertex_list[edge_list[nextEdge_idx].start].ycoor,
							vertex_list[edge_list[nextEdge_idx].end].xcoor,
							vertex_list[edge_list[nextEdge_idx].end].ycoor);
	
	if ((edge_list[nextEdge_idx].start == edgeEvaluated.start) || (edge_list[nextEdge_idx].start == edgeEvaluated.end))
	{
		
		px = (new_xcoor + vertex_list[edge_list[nextEdge_idx].end].xcoor) /2.0;
		py = (new_ycoor + vertex_list[edge_list[nextEdge_idx].end].ycoor) /2.0;
	}		
	else //otherwise it is the end point is one of the v1, v2,
	{
		px = (new_xcoor + vertex_list[edge_list[nextEdge_idx].start].xcoor) /2.0;
		py = (new_ycoor + vertex_list[edge_list[nextEdge_idx].start].ycoor) /2.0;
	}
	
	if (PointInParcel(px,py,parcelID))
		area2 = - area2; //the triangle is inside
	
	
	newArea += area2;
	return newArea;
}


double cCSVFileReader::calcNewAreaForDraggingVertex(int parcelID, int start_v, int end_v, double new_x, double new_y)
{
	double old_area = parcel_list[parcelID].area;
	double old_triangleArea, new_triangleArea;
	S_Vertex p;
	int pChange=-1;
	vector<S_Vertex> twoP;
	S_Parcel currentParcel = parcel_list[parcelID];
	S_Edge currentEdge;

	int i;
	/*if (parcelID==2)
	{
		printf("%d (%f,%f), %d (%f,%f) --> (%f, %f)\n", start_v, vertex_list[start_v].xcoor,vertex_list[start_v].ycoor,
											end_v, vertex_list[end_v].xcoor,vertex_list[end_v].ycoor,new_x, new_y);
	}*/
	
	/*if (start_v==1932 && end_v==10)
	{
		printf("The problem parcel is %d\n",parcelID);
		for (i=0; i<currentParcel.edges.size(); i++)
		{	currentEdge = edge_list[currentParcel.edges[i]];
			printf("-----------%d %d--------\n", currentEdge.start, currentEdge.end);
		}
	}*/
	//first deal with start_v;
	int k;
	for (i=0; i<currentParcel.edges.size(); i++)
	{
		currentEdge = edge_list[currentParcel.edges[i]];
		
		if (currentEdge.edge_id==-1)
			continue;
		if (currentEdge.start == start_v) 
		{
			p = vertex_list[currentEdge.end];
			for (k=0; k<twoP.size(); k++)
			{
				if (twoP[k].vertex_id == p.vertex_id)
					break;	
			}
			if (k==twoP.size())
				twoP.push_back(p);
			assert(((pChange==-1) || (pChange==start_v)));		
			pChange=start_v;
			
		}
		if (currentEdge.end == start_v) 
		{
			p = vertex_list[currentEdge.start];
			for (k=0; k<twoP.size(); k++)
			{
				if (twoP[k].vertex_id == p.vertex_id)
					break;	
			}
			if (k==twoP.size())
				twoP.push_back(p);
			assert(((pChange==-1) || (pChange==start_v)));
			pChange=start_v;			
		}
		
	}
	/*if (parcelID==12)
	{
		printf("for start point %d: %d\n", start_v,twoP.size());
	}
	if (start_v==1046)
	{
		for (i=0; i<twoP.size(); i++)
			printf("(%f,%f)\t", twoP[i].xcoor, twoP[i].ycoor);
		printf("\n");
	}*/
	/*printf("Two P for start:(%f,%f)\n",vertex_list[start_v].xcoor,vertex_list[start_v].ycoor);
	for (i=0; i<twoP.size(); i++)
			printf("(%f,%f)\t", twoP[i].xcoor, twoP[i].ycoor);
	printf("\n");
	assert(twoP.size()==0 || twoP.size()==2);*/
	if (twoP.size()==2)
	{
		old_triangleArea = calcTriangleArea(twoP[0].xcoor, twoP[0].ycoor, twoP[1].xcoor, twoP[1].ycoor, vertex_list[pChange].xcoor, vertex_list[pChange].ycoor);
		new_triangleArea = calcTriangleArea(twoP[0].xcoor, twoP[0].ycoor, twoP[1].xcoor, twoP[1].ycoor, new_x,new_y);
	}
	twoP.clear();
	pChange=-1;
	
	for (i=0; i<currentParcel.edges.size(); i++)
	{
		currentEdge = edge_list[currentParcel.edges[i]];
		

		if (currentEdge.start == end_v)
		{
			p = vertex_list[currentEdge.end];
			
			for (k=0; k<twoP.size(); k++)
			{
				if (twoP[k].vertex_id == p.vertex_id)
					break;	
			}
			if (k==twoP.size())
				twoP.push_back(p);	
			assert(((pChange==-1) || (pChange==end_v)));
			pChange=end_v;		
		}
	
		if (currentEdge.end == end_v)
		{
			p = vertex_list[currentEdge.start];
			
			for (k=0; k<twoP.size(); k++)
			{
				if (twoP[k].vertex_id == p.vertex_id)
					break;	
			}
			if (k==twoP.size())
				twoP.push_back(p);
			
			assert(((pChange==-1) || (pChange==end_v)));
	
			pChange=end_v;		
		}
	}
	/*if (parcelID==12)
	{
		printf("for end point %d: %d\n",end_v, twoP.size());
	}*/
	/*if (end_v==2003)
	{
		for (i=0; i<twoP.size(); i++)
			printf("(%f,%f)\t", twoP[i].xcoor, twoP[i].ycoor);
		printf("\n");
	}*/
	/*printf("Two P for end:(%f,%f)\n",vertex_list[end_v].xcoor,vertex_list[end_v].ycoor);
	for (i=0; i<twoP.size(); i++)
			printf("(%f,%f)\t", twoP[i].xcoor, twoP[i].ycoor);
	printf("\n");
	assert(twoP.size()==0 || twoP.size()==2);*/
	if (twoP.size()==2)
	{
		old_triangleArea = calcTriangleArea(twoP[0].xcoor, twoP[0].ycoor, twoP[1].xcoor, twoP[1].ycoor, vertex_list[pChange].xcoor, vertex_list[pChange].ycoor);
		new_triangleArea = calcTriangleArea(twoP[0].xcoor, twoP[0].ycoor, twoP[1].xcoor, twoP[1].ycoor, new_x,new_y);

	}
	return (old_area-old_triangleArea + new_triangleArea);
}

inline double cCSVFileReader::calcTriangleArea(double x0, double y0, double x1, double y1, double x2, double y2)
{//this function return the triangle area, triangle is defined by the 3 points, given their
//coordinate.
//we use heron calculation: area = sqrt(s*(s-a)*(s-b)*(s-c)), a, b,c is the 3 edge's length
	//printf("(%f,%f) (%f,%f) (%f,%f) ", x0,y0,x1,y1,x2,y2);
	double area;
	double a = sqrt((x0-x1)*(x0-x1) + (y0-y1)*(y0-y1));
	double b = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
	double c = sqrt((x2-x0)*(x2-x0) + (y2-y0)*(y2-y0));
	
	double s =(a+b+c)/2.0;
	
	if(s-a<0) return 0;
	if(s-b<0) return 0;
	if(s-c<0) return 0;
	area = sqrt(s*(s-a)*(s-b)*(s-c));
	//printf("-->area:%f\n", area); 
	return area;
	
}

bool cCSVFileReader::PointInParcel(double px, double py, int parcelID)
{
	//this function will return true if the the point is inside the polygon formed by parcel
	//otherwise true for a outside point;
	S_Parcel parcel = parcel_list[parcelID];
	S_Edge edge_now;
	bool oddNodes = false;
	int i, j=0;
	int prevEndVertex=-1;
	vector<double> xCoors;
	vector<double> yCoors;
	int numEdges= parcel.edges.size();
	
	//first generate the vertex of x,y of the corner points
	for (i=0; i<numEdges; i++)
	{
		edge_now = edge_list[parcel.edges[i]];
		
		if ((i==0)||(edge_now.start == prevEndVertex))
		{
			//first one or start equal previous end vertex
			//start is what we want;
			xCoors.push_back(vertex_list[edge_now.start].xcoor);
			yCoors.push_back(vertex_list[edge_now.start].ycoor);
			prevEndVertex = edge_now.end;
		}		
		else
		{
			xCoors.push_back(vertex_list[edge_now.end].xcoor);
			yCoors.push_back(vertex_list[edge_now.end].ycoor);
			prevEndVertex = edge_now.start;
		}		
	}
	
	
	
	for (i=0; i<numEdges; i++)
	{
		j++;
		if(j==numEdges) 
			j=0;
		if ((yCoors[i]<py) && (yCoors[j]>=py) || (yCoors[j]<py) && (yCoors[i]>=py))
		{
			if (xCoors[i]+(py-yCoors[i])/(yCoors[j]-yCoors[i])*(xCoors[j]-xCoors[i])<px) 
			{
				//printf("switched\n");
				
				oddNodes =!oddNodes;
			}
		}
	}
	
	return oddNodes;
}

bool cCSVFileReader::checkVertexInEdgeList(double xcoor, double ycoor,int edgeID_TOBE_contracted, vector<int> &edges)
{
	double x0,y0,x1,y1;
	edges.clear();
	for (vector<S_Edge>::iterator iter_edge = edge_list.begin(); 
		 iter_edge!=edge_list.end(); iter_edge++)
	{
		if ((iter_edge-edge_list.begin())==edgeID_TOBE_contracted)
			continue;
		x0 = vertex_list[iter_edge->start].xcoor;
		y0 = vertex_list[iter_edge->start].ycoor;
		x1 = vertex_list[iter_edge->end].xcoor;
		y1 = vertex_list[iter_edge->end].ycoor;
		if ((calcTriangleArea(x0,y0,x1,y1,xcoor,ycoor) == 0.0) && 
		   (fabs(x0-xcoor) + fabs(x1-xcoor) == fabs(x0-x1)) &&
		   (fabs(y0-ycoor) + fabs(y1-ycoor) == fabs(y0-y1)))  
		{
			printf("================================\n");
			printf("vertex(%f,%f) on edge(%f,%f)->(%f,%f)!!\n",xcoor,ycoor,x0,y0,x1,y1);
			printf("================================\n");
			//scanf("%f",&x0);
			edges.push_back(iter_edge-edge_list.begin());
		}
	}
	if (!edges.empty())
		return false;
	for (vector<S_Vertex>::iterator iter_vertex = vertex_list.begin(); 
		 iter_vertex!=vertex_list.end(); iter_vertex++)
	{
		if ((iter_vertex->xcoor==xcoor) && ( iter_vertex->ycoor == ycoor))
			return false;
		
	}
	return true;
}
//this function is performing the edge contraction 
void cCSVFileReader::performEdgeContraction(int edgeID, vector<int> & affectedEdges)
{
	//never actually remove the edge and vertex from the global listen
	//assign the id to -1 to mark it deleted
	int i,j;
	vector<int>::iterator parcel_edge_iter;
	vector<int>::iterator parcel_id_iter;
	S_Edge edge_tobeRemoved = edge_list[edgeID];
	int start = edge_tobeRemoved.start;
	int end = edge_tobeRemoved.end;
	S_Vertex firstVertex = vertex_list[start];
	S_Vertex secondVertex = vertex_list[end];	
	S_Vertex newVertex;
	//we need to update the area of all the affected parcels
	vector<int> affectedParcelList;
	
	vector<int>::iterator affectedEdges_iter;
	affectedParcelList.clear();
	double new_area;
	//new position of v1 and v2 is the midpoint 
	newVertex.vertex_id = vertex_list.size();
	newVertex.xcoor = (
						vertex_list[edge_tobeRemoved.start].xcoor + 
					  	vertex_list[edge_tobeRemoved.end].xcoor
					   )/2.0; 
	
	newVertex.ycoor = (
						vertex_list[edge_tobeRemoved.start].ycoor + 
					  	vertex_list[edge_tobeRemoved.end].ycoor
					   )/2.0;
	bool old_vertex=false;
	vector<S_Vertex>::iterator iter_vertex;
	for (iter_vertex = vertex_list.begin();
		iter_vertex!=vertex_list.end(); iter_vertex++)
	{
		if (iter_vertex->vertex_id==-1)
			continue;
		if ((iter_vertex->xcoor == newVertex.xcoor) && (iter_vertex->ycoor == newVertex.ycoor))
		{
			//newVertex=*iter_vertex;
			old_vertex=true;
			//printf("Existing: %d\n", iter_vertex->vertex_id);
			break;
		}
	}
	//first update the new area for affected parcels;
	
	//need to find out if this vertex is already in the vertex list or it is on exist edges
	/*vector<int> edges_needto_split;
	if (checkVertexInEdgeList(newVertex.xcoor, newVertex.ycoor, edgeID,edges_needto_split)==false)
	{
		
		return; 
	}
	*/
	
	if (!old_vertex)
	{
		vertex_list.push_back(newVertex);
		iter_vertex = vertex_list.end()-1;
	}
	
	//if ((vertex_list[end].edges[i]==-1) || (vertex_list[start].edges[i]==-1))
	//	return;
	//printf("finding affected parcels\n");
	for (i=0; i<firstVertex.edges.size(); i++)
	{
		if (vertex_list[start].edges[i]!=-1)
		for (j=0; j<edge_list[firstVertex.edges[i]].owner_parcel.size(); j++)
		{
			if (edge_list[vertex_list[start].edges[i]].edge_id!=-1)
			affectedParcelList.push_back(edge_list[vertex_list[start].edges[i]].owner_parcel[j]);
		}
	}
	//for the secondVertex
	for (i=0; i<secondVertex.edges.size(); i++)
	{
		if (vertex_list[end].edges[i]!=-1)
		for (j=0; j<edge_list[secondVertex.edges[i]].owner_parcel.size(); j++)
		{			
			affectedParcelList.push_back(edge_list[vertex_list[end].edges[i]].owner_parcel[j]);
		}
	}
	//printf("before remove repeat\n");
	removeRepeat(affectedParcelList);
	
	//TODO: calculate the new area
	//in fact affectedParcelList should contain at most 4 parcels,
	//in that situation 2 of them will be the edge_TOBE_removed's owner parcel
	//another 2 will be the having a vertex of either v1 or v2 then, is dragged towards
	//the newVertex
	
	//for owner parcel, just need to call the function calcNewAreaIfEdgeContracted,
	//for vertex dragging, need to call function calcTriangleArea and add the 2 or 1 new triangle area
	//printf("How many affected Parcels; %d\n", affectedParcelList.size());
	//assert(affectedParcelList.size()<=4);
	for (i=0; i<affectedParcelList.size(); i++)
	{
		//first deal with the owner parcel
		/*parcel_id_iter = find(edge_tobeRemoved.owner_parcel.begin(),
							  edge_tobeRemoved.owner_parcel.end(),
							  affectedParcelList[i]);
		if (parcel_id_iter != edge_tobeRemoved.owner_parcel.end())
		*/
		new_area = calcNewAreaIfEdgeContracted(affectedParcelList[i],edgeID);
		//printf("parcel#%d: area %f-->%f \n",affectedParcelList[i],parcel_list[affectedParcelList[i]].area,new_area);
		parcel_list[affectedParcelList[i]].area = new_area;
	}
	
	//then remove this edge from parcel's edge list	
	
	for (i=0; i<edge_tobeRemoved.owner_parcel.size(); i++)
	{
		parcel_edge_iter = find(parcel_list[edge_tobeRemoved.owner_parcel[i]].edges.begin(),
						    parcel_list[edge_tobeRemoved.owner_parcel[i]].edges.end(),
							edgeID
							);
		if (parcel_edge_iter!=parcel_list[edge_tobeRemoved.owner_parcel[i]].edges.end())
		{
			parcel_list[edge_tobeRemoved.owner_parcel[i]].edges.erase(parcel_edge_iter);
		}
	}		
	
	
	
	//here we need to have look if there are edges which the same with the one to be removed.
	
	/*for (i=0; i<firstVertex.edges.size(); i++)
	{
		//iterate the edge list put anything
		if ((edge_list[firstVertex.edges[i]].edge_id!=-1) &&
			(edge_list[firstVertex.edges[i]].edge_id!=edge_tobeRemoved.edge_id))
		{
			if (((edge_list[firstVertex.edges[i]].end==firstVertex.vertex_id) &&
	 			(edge_list[firstVertex.edges[i]].start==secondVertex.vertex_id)) ||
               ((edge_list[firstVertex.edges[i]].start==firstVertex.vertex_id) &&
	 			(edge_list[firstVertex.edges[i]].end==secondVertex.vertex_id)))		
			{
				
				//this is a duplicate edge generated from the simplification process.
				//we need to update the parcel's edge list and 
			}
		}
	}*/
	
	//printf("affectedEdges (by FirstVertex %d): ",firstVertex.vertex_id);
	if ((firstVertex.vertex_id!=-1))//&&(firstVertex.vertex_id!=secondVertex.vertex_id))
	//update the edges which linked with first point)
	for (i=0; i<firstVertex.edges.size(); i++)
	{
		//iterate the edge list put anything
		if ((edge_list[firstVertex.edges[i]].edge_id!=-1) &&
			(edge_list[firstVertex.edges[i]].edge_id!=edge_tobeRemoved.edge_id))
		{
			
			affectedEdges.push_back(edge_list[firstVertex.edges[i]].edge_id);
			//printf("%d (%d->%d)\t",edge_list[firstVertex.edges[i]].edge_id,
			//					   edge_list[firstVertex.edges[i]].start,
			//						edge_list[firstVertex.edges[i]].end);
			//if start point==firstVertex
			//then end point should be the new Vertex, vice versa
			if (edge_list[firstVertex.edges[i]].start==firstVertex.vertex_id)
			{
				edge_list[firstVertex.edges[i]].start = iter_vertex->vertex_id;
			}
			else if (edge_list[firstVertex.edges[i]].end==firstVertex.vertex_id)
			{
				edge_list[firstVertex.edges[i]].end = iter_vertex->vertex_id;

			}
			else
			{
				//printf("Something really wrong here! edge_tobe_removed:%d vertex_id%d \n",edge_tobeRemoved.edge_id, firstVertex.vertex_id );
				//printf("%d %d, %d\n",edge_list[firstVertex.edges[i]].start,edge_list[firstVertex.edges[i]].end, firstVertex.vertex_id); 
				exit(-1);
			}
		}			
	}
		//printf("\n");
	//printf("affectedEdges (by secondVertex %d): ",secondVertex.vertex_id);
	
	//deal with duplicate edges.
	/*if (secondVertex.vertex_id!=-1)
	for (i=0; i<secondVertex.edges.size(); i++)
	{
		//iterate the edge list put anything
		if ((edge_list[secondVertex.edges[i]].edge_id!=-1) &&
			(edge_list[secondVertex.edges[i]].edge_id!=edge_tobeRemoved.edge_id))
		{
	
			for (j=0; j<affectedEdges.size(); j++)
			{
				//degraded case, triangle
				if ((edge_list[affectedEdges[j]].start == edge_list[secondVertex.edges[i]].start) || 
					(edge_list[affectedEdges[j]].end == edge_list[secondVertex.edges[i]].start) ||
					(edge_list[affectedEdges[j]].end == edge_list[secondVertex.edges[i]].end) ||
					(edge_list[affectedEdges[j]].start == edge_list[secondVertex.edges[i]].end))
				{
					//first we need to see if the two edge has one and only one common owner parcel
					//if it is that case delete it
					//assert(edge_list[affectedEdges[j]].owner_parcel.size()==1);
					//assert(edge_list[secondVertex.edges[i]].owner_parcel.size()==1);
					
					if (edge_list[affectedEdges[j]].owner_parcel[0] == edge_list[secondVertex.edges[i]].owner_parcel[0])
					{
						//remove both edge, erase the one is affectedEdges,
						edge_list[affectedEdges[j]].edge_id=-1;
						edge_list[secondVertex.edges[i]].edge_id=-1;
						affectedEdges_iter = find(affectedEdges.begin(), affectedEdges.end(),affectedEdges[j]);
						affectedEdges.erase(affectedEdges_iter);
					}						
					
					else replaceEdge(secondVertex.edges[i], affectedEdges[j]);
					break;
				}
			}
			
		}
	}
	*/
	if (secondVertex.vertex_id!=-1)
	for (i=0; i<secondVertex.edges.size(); i++)
	{
		//iterate the edge list put anything
		if ((edge_list[secondVertex.edges[i]].edge_id!=-1) &&
			(edge_list[secondVertex.edges[i]].edge_id!=edge_tobeRemoved.edge_id))
		{
			//if (edge_list[secondVertex.edges[i]].edge_id!=-1)
			affectedEdges.push_back(edge_list[secondVertex.edges[i]].edge_id);
			//printf("%d (%d->%d)\t",edge_list[secondVertex.edges[i]].edge_id,
			//					   edge_list[secondVertex.edges[i]].start,
			//						edge_list[secondVertex.edges[i]].end);

			//if start point==firstVertex
			//then end point should be the new Vertex, vice versa
			if (edge_list[secondVertex.edges[i]].start==secondVertex.vertex_id)
			{
				edge_list[secondVertex.edges[i]].start = iter_vertex->vertex_id;
			}
			else if (edge_list[secondVertex.edges[i]].end==secondVertex.vertex_id)
			{
				edge_list[secondVertex.edges[i]].end = iter_vertex->vertex_id;

			}
			else
			{
			//	printf("Something really wrong here!edge_tobe_removed:%d vertex_id%d \n",edge_tobeRemoved.edge_id, secondVertex.vertex_id );
			//	printf("%d %d, %d\n",edge_list[secondVertex.edges[i]].start,edge_list[secondVertex.edges[i]].end, firstVertex.vertex_id); 
				exit(-1);
			}
			
		}			
	}
		//printf("\n");
	//affectedEdges shouldn't have any repeat numbers, in that case it is degrading
	//assign the affectedEdges to newVertex edge list
	
	for (i=0; i<affectedEdges.size(); i++)
	{
		iter_vertex->edges.push_back(affectedEdges[i]);
	}
	
	
	//now mark the edge tobe removed deleted
	edge_list[edgeID].edge_id = -1;
 	//mark the two vertice deleted
	//printf("Removing %d(%f,%f),%d(%f,%f), Adding %d(%f,%f)\n",edge_tobeRemoved.start,vertex_list[edge_tobeRemoved.start].xcoor,vertex_list[edge_tobeRemoved.start].ycoor,
	//														  edge_tobeRemoved.end,vertex_list[edge_tobeRemoved.end].xcoor,vertex_list[edge_tobeRemoved.end].ycoor,
	//											iter_vertex->vertex_id, iter_vertex->xcoor, iter_vertex->ycoor);

	vertex_list[edge_tobeRemoved.start].vertex_id =-1;
	vertex_list[edge_tobeRemoved.end].vertex_id =-1;
	//printf("Removing %d(%f,%f),%d(%f,%f), Adding %d(%f,%f)\n",edge_tobeRemoved.start,edge_tobeRemoved.end,newVertex.vertex_id);
	
}

void cCSVFileReader::replaceEdge(int dest, int replace)
{
	//replace dest edge with src edge;
	//update owner's edge list;
	int i,j;
	for (i=0; i<edge_list[dest].owner_parcel.size(); i++)
	{
		for (j=0; j<parcel_list[edge_list[dest].owner_parcel[i]].edges.size(); j++)
		{
			if (parcel_list[edge_list[dest].owner_parcel[i]].edges[j]==dest)
				parcel_list[edge_list[dest].owner_parcel[i]].edges[j]=replace;
		}
	}
	
	edge_list[dest].edge_id=-1;
}
double cCSVFileReader::calcMaximEdgeCost(int edgeID)
{
	//this function return the maxiam area change percent if we contract the edge
	double cost=0;
	int i,j;
	S_Edge edge_item = edge_list[edgeID];
	int start  = edge_item.start;
	int end  = edge_item.end;
	int ownerID;
	double costTmp;
	double new_area;
	vector<int> affectedParcelList;
	affectedParcelList.clear();
	//if we contract the edge, the start point will be moved, we need to find out the 
	//affected parcel because of the point moving
	//the start point
	
	for (i=0; i<vertex_list[start].edges.size(); i++)
	{
		for (j=0; j<edge_list[vertex_list[start].edges[i]].owner_parcel.size(); j++)
		{
			affectedParcelList.push_back(edge_list[vertex_list[start].edges[i]].owner_parcel[j]);
		}
	}
	//for the other vertice
	for (i=0; i<vertex_list[end].edges.size(); i++)
	{
		for (j=0; j<edge_list[vertex_list[end].edges[i]].owner_parcel.size(); j++)
		{
			affectedParcelList.push_back(edge_list[vertex_list[end].edges[i]].owner_parcel[j]);
		}
	}
	//we need to clean affect the affectedParcelList a little bit;
	//to remove the repeat number;
	removeRepeat(affectedParcelList);
	
	//now for each affected Parcel we need to evaluate the cost is we remove this edge
	for(i=0; i<affectedParcelList.size(); i++)
	{
		//printf("Evaluate Edge ID %d agains Parcel %d\n", edgeID,affectedParcelList[i]);
		new_area = calcNewAreaIfEdgeContracted(affectedParcelList[i],edgeID);
		if (fabs(new_area)<0.000001)
			costTmp=1.0;
		//if (edgeID==12)
		//{
		//	printf("%d's parcel:%d %f>>%f orig area: %f\n",edgeID,affectedParcelList[i],parcel_list[affectedParcelList[i]].area,new_area , parcel_area_orig[affectedParcelList[i]]);
		//}
		//printf("area: %f->%f\n",  parcel_list[affectedParcelList[i]].area,new_area);
		
		costTmp = fabs((new_area - parcel_area_orig[affectedParcelList[i]]) / parcel_area_orig[affectedParcelList[i]]);
		if (cost<costTmp)
			cost = costTmp;
	}
	
	return cost;
}
void cCSVFileReader::removeRepeat(vector<int> & inputList)
{
	vector<int> tmpList;
	tmpList.clear();
	int data_item;
	int i,j;
	for (i=0; i<inputList.size(); i++)
	{
		data_item = inputList[i];
		for (j=0; j<tmpList.size(); j++)
		{
			if (data_item==tmpList[j])
			{
				break;
			}
		}
		if (j==tmpList.size())
		{
			//we didn't find the number, it is new
			tmpList.push_back(data_item);
		}
	}
	
	inputList.clear();
	for (i=0; i<tmpList.size(); i++)
	{
		inputList.push_back(tmpList[i]);
	}		
}

void cCSVFileReader::simplifyPolygons(double cost_threshhold, string *inputFileNamePrefix,bool BatchMode)
{
	//this is the main function of the polygon simplification
	//now we have all the data structure we have 
	//build the priority queue of edges associate with the cost of contracting the edge
	//in the order of the max error if we contract it.
	//if I have time will implement the priority queue with the capability of dynamic change  of value.
	int i;
	vector<S_PriorityEdge> edge_queue;  //unsorted queue
	vector<S_PriorityEdge>::iterator priority_edge_iter, min_edge;
	int min_id, tmp_id;
	S_PriorityEdge edge_item;
	double min_cost=-1.0;
	//const double cost_threshhold=0.005;
	bool not_fail=true;
	printf("\tGenerating Priority Queue...\n");
	vector<int> affectedEdges;
	vector<int>::iterator affectedEdges_iter;
	for (i=0; i<edge_list.size(); i++)
	{
		edge_item.edge_id = i;
		edge_item.priority = calcMaximEdgeCost(i);
		//printf("%d %f\n", edge_item.edge_id,edge_item.priority);
		edge_queue.push_back(edge_item);
	}
	
	
	//then each time find the edge with maxim priority (min cost), and perform 
	//the edge contraction
	
	min_cost=-1.0;  //we need to find the minimal cost edge to be contracted
	double output_target=0.0;
	
	/*bool needTOExtendSVGFileName=false;
	int extDotPos = inputFileNamePrefix->find_last_of("."); //find the extension file name dot
	string fileExt="";
	if (extDotPos!=inputFileNamePrefix->npos)
		fileExt=inputFileNamePrefix->substr(extDotPos);
		
	if ((fileExt!=".svg")||(fileExt==""))
	{
		needTOExtendSVGFileName=true;
	}*/
	
	if (BatchMode)
		{
			string fileName=std::string(*inputFileNamePrefix);
			fileName+="_no_simp";
			//if (needTOExtendSVGFileName)
			fileName+=".svg";
			printf("Generating SVG Image without simplification...\n");
				generateSVG(fileName);
			cout<<"SVG image "<<fileName<<" generated"<<endl;
		}
		
	while((min_cost<cost_threshhold)&&(not_fail))
	{
		min_cost=-1.0;
		affectedEdges.clear();
		for (priority_edge_iter=edge_queue.begin(); 
			 priority_edge_iter<edge_queue.end(); 
		     priority_edge_iter++)
		{
			if ((min_cost<0) || (min_cost>priority_edge_iter->priority))
			{
				//either the first one or the previous min is bigger
				min_cost = priority_edge_iter->priority;
				min_edge = priority_edge_iter;
				min_id = priority_edge_iter->edge_id;
			}
		}	
		
		if ((BatchMode)&&(min_cost>output_target))
		{
			string fileName=std::string(*inputFileNamePrefix);
			char number[5];
			sprintf(number,"%1.3f",output_target); 
			fileName+="_";
			fileName+=std::string(number);
			//if (needTOExtendSVGFileName)
			fileName+=".svg";
			printf("Generating SVG Image with thresold %f...\n", output_target);
				generateSVG(fileName);
			cout<<"SVG image "<<fileName<<" generated"<<endl;
			while (output_target<=min_cost)
			{
				output_target+=0.001;
			}
		}
		//now we have the min_cost edge;
		printf("\tperforming EdgeContraction %d (%d->%d) cost:%f\n", min_id, edge_list[min_id].start, edge_list[min_id].end,min_cost); 
		performEdgeContraction(min_id, affectedEdges);
		//
		
		//then we need to update all the data priority 
		//first erase the just contracted edge from priority queue
		edge_queue.erase(min_edge);
		if (affectedEdges.empty()) //which means we didn't actually performed the contraction.
			continue;
		//then deal with the affectedEdges
		for (priority_edge_iter=edge_queue.begin(); 
			 priority_edge_iter<edge_queue.end(); 
		     priority_edge_iter++)
		{
			tmp_id = priority_edge_iter->edge_id;
			//printf("%d ",tmp_id);
			affectedEdges_iter = find(affectedEdges.begin(), affectedEdges.end(), tmp_id);
			if (affectedEdges_iter!=affectedEdges.end())
			{
				//which means we found it
				//update the priority value
				priority_edge_iter->priority = calcMaximEdgeCost(tmp_id);
				//printf("Affected%d %f\n",*affectedEdges_iter,priority_edge_iter->priority);
				//if the just changed priority is below min_cost
				//we can update it
				if (min_cost > priority_edge_iter->priority)
				{
					min_cost = priority_edge_iter->priority;
				}
				//remove from affectedEdges list
				affectedEdges.erase(affectedEdges_iter);
			}
			else
			{
				//since priority_edge is a super set of affectedEdges, if we cannot find 
				//nothing left 
				//printf("Nothing updated\n");
				//break;
			}
		}
		
		
	}
	
	//need to massage the vertice's coordinate a bit to hide them under road patches.
	cout<<"Massaging vertices coordinates to hide under road ..."<<endl;
	massageVertexCoor();
	
	
}

void cCSVFileReader::massageVertexCoor()
{
	//to hide parcel boundary underneath road
	vector<int> candidateSet;
	vector<double> candidateDistance;
	double xcor; 
	double ycor; 
	int grid_xcor;
 	int grid_ycor;
	int patchIndex1 ;
	int patchIndex2 ;
	int patchIndex3 ;
	int patchIndex4 ;
	
	for (vector<S_Vertex>::iterator vertex_iter=vertex_list.begin();
		 vertex_iter!=vertex_list.end(); vertex_iter++)
	{
		if (vertex_iter->vertex_id==-1)
			continue;
		xcor = vertex_iter->xcoor;
		ycor = vertex_iter->ycoor;
		grid_xcor = (int)floor(xcor+0.4999);
		grid_ycor = (int)floor(ycor+0.4999);
		//cout<<"("<<xcor<<" "<<ycor<<") -- >"<<grid_xcor<<","<<grid_ycor<<endl;
		patchIndex1 = getPatchIndex(grid_xcor-1,grid_ycor-1);
		patchIndex2 = getPatchIndex(grid_xcor,grid_ycor-1);
		patchIndex3 = getPatchIndex(grid_xcor-1,grid_ycor);
		patchIndex4 = getPatchIndex(grid_xcor,grid_ycor);
		
		
		candidateSet.clear();
		candidateDistance.clear();
		if (patchIndex1!=-1) //we can probably message this vertex to grid_xcor-0.5, grid_ycor-0.5
		{
			if (data->patches[patchIndex1].isRoad)
			{
				candidateSet.push_back(1);
				candidateDistance.push_back((xcor-(grid_xcor-0.5)) * (xcor-(grid_xcor-0.5)) +
									   (ycor-(grid_ycor-0.5)) * (ycor-(grid_ycor-0.5)));
				//distance ^2
			}
		}
		
		if (patchIndex2!=-1) //we can probably message this vertex to grid_xcor+0.5, grid_ycor-0.5
		{
			if (data->patches[patchIndex2].isRoad)
			{
				candidateSet.push_back(2);
				candidateDistance.push_back((xcor-(grid_xcor+0.5)) * (xcor-(grid_xcor+0.5)) +
									   (ycor-(grid_ycor-0.5)) * (ycor-(grid_ycor-0.5)));
				//distance ^2
			}
		}
		
		if (patchIndex3!=-1) //we can probably message this vertex to grid_xcor-0.5, grid_ycor+0.5
		{
			if (data->patches[patchIndex3].isRoad)
			{
				candidateSet.push_back(3);
				candidateDistance.push_back((xcor-(grid_xcor-0.5)) * (xcor-(grid_xcor-0.5)) +
									   (ycor-(grid_ycor+0.5)) * (ycor-(grid_ycor+0.5)));
				//distance ^2
			}
		}
		if (patchIndex4!=-1) //we can probably message this vertex to grid_xcor+0.5, grid_ycor+0.5
		{
			if (data->patches[patchIndex4].isRoad)
			{
				candidateSet.push_back(4);
				candidateDistance.push_back((xcor-(grid_xcor+0.5)) * (xcor-(grid_xcor+0.5)) +
									   (ycor-(grid_ycor+0.5)) * (ycor-(grid_ycor+0.5)));
				//distance ^2
			}
		}
		
		if (candidateSet.size()==0)
			continue;
		double minDistance=10000000.0;
		int minIndex=-1;
		for (int i=0; i<candidateDistance.size(); i++)
		{
			if (minDistance>candidateDistance[i])
			{
				minDistance=candidateDistance[i];
				minIndex = candidateSet[i];
			}
		}
		
		if (minIndex>0)
		{
			switch(minIndex)
			{
				case 1:
					vertex_iter->xcoor = grid_xcor-0.5;
					vertex_iter->ycoor = grid_ycor-0.5;
					break;
				case 2:
					vertex_iter->xcoor = grid_xcor+0.5;
					vertex_iter->ycoor = grid_ycor-0.5;
					break;
				case 3:
					vertex_iter->xcoor = grid_xcor-0.5;
					vertex_iter->ycoor = grid_ycor+0.5;
					break;
				case 4:
					vertex_iter->xcoor = grid_xcor+0.5;
					vertex_iter->ycoor = grid_ycor+0.5;
					break;
				//cout<<"Massaged"<<endl;
			}
		}
	}
}
