globals [
  ticks
  mapname
]

;;;

to startup ; slow, do just once
   set mapname "";   set mapname "cruise"
   import-dat mapname
end

to setup
  clear-turtles clear-all-plots
  set ticks 0
  setup-agents
end

to import-dat [datfile]
  ifelse length datfile = 0 [
    (foreach values-from patches [self] patchcolors [set pcolor-of ?1 ?2])
  ][
    set datfile datfile + ".dat"
    file-open datfile
    ask patches [set pcolor file-read]
    file-close
  ]
  ask patches with [pcolor = 9] [set pcolor 9.9999] ; to save space, 9.9.. converted to 9 during export.
end

to-report on-road?
  report (abs (pcolor - red) < 5) or  (abs (pcolor - orange) < 5) ;or  (abs (pcolor - pink) < 5)
end
to go-home ; agent proc
  set pcolor green setup-agent ; self healing!
end

to setup-agents
  create-custom-turtles num-cars [setup-agent]
end
to setup-agent
  set size 8
  set color black
  while [not on-road?] [ ; REMIND can't start on road!  Need until
    setxy random screen-size-x random screen-size-y
  ]
end

to go
  if ticks = 0 [if (count turtles) = 0 [setup]]
  set ticks ticks + 1
  ask turtles [
    move-forward
  ]
end

to move-forward  ; turtle proc
  locals [n p l]
  set n neighbors with [on-road?]
  ifelse count n = 0 [go-home] [
    set l []
    ask patch-left-and-ahead  90 3 [if on-road? [set l lput self l]]
    ask patch-right-and-ahead 90 3 [if on-road? [set l lput self l]]
    if (length l != 0) and (0 = random 3) [ set p random-one-of l ]
    if p = 0      [ set p one-of n with [(heading-angle myself) = 0] ]
    if p = nobody [ set p random-one-of n with [(heading-angle myself) <= 45] ]
    if p = nobody [ set p random-one-of n with [(heading-angle myself) <= 90] ]
    if p = nobody [ set p min-one-of n [heading-angle myself] ]
    set heading towards p
    forward distance p
  ]
end

to-report heading-angle [t] ; patch proc
  locals [h]
  ;set h abs (heading-of t - (towards t + 180) mod 360)
  set h (towards t + 180 - heading-of t) mod 360
  if h > 180 [set h 360 - h]
  report h
end

; ---------------------------------------------------------------------------
;        Can delete; only used to build map data. Not used by model.
; ---------------------------------------------------------------------------
; These are called by hand to create the compressed (.dat) form of the map.
; To create the file, call: create-dat "mapname" (w/o .dat)
; Can remove once the .dat is created.  Here to explain the stunt.
; Note: patchcolors proc below is an array form of the .dat file.
; ---------------------------------------------------------------------------

to init-map
  create-dat mapname
end

to create-dat [mapfile]
  print "..creating patches, I'll print 'done' when completed"
  import-ppm mapfile
  export-dat mapfile
  print "..done!"
end
to import-ppm [ppmfile]
  locals [x y scale]
  set ppmfile ppmfile + ".ppm"
  file-close-all
  file-open ppmfile
  set x 1 set y file-read-line
  while [first file-read-line = "#"] [set x x + 1]
  file-close
  file-open ppmfile
  repeat x [set x file-read-line]
  set x file-read
  set y file-read
  set scale 1 + file-read
  if x != screen-size-x and y != screen-size-y [print "Oops: need to fix screen-size to match ppm file"]
  ask patches [set pcolor rgb (file-read / 256) (file-read / 256) (file-read / 256)]
  file-close
  cleanup-map
end
to cleanup-map
  ask patches with [(floor pcolor) mod 10 = 9] [set pcolor 9.9]
  ask patches with [pcolor != 9.9] [set pcolor round pcolor]
  ask patches with [pcolor > 120] [set pcolor pcolor - 110]
end
to export-dat [datfile]
  set datfile datfile + ".dat"
  file-close-all
  if file-exists? datfile [file-delete datfile]
  file-open datfile
  ask patches [file-write floor pcolor if pxcor = screen-edge-x [file-print ""]]
;  ask patches [ifelse pxcor = screen-edge-x [file-print floor pcolor][file-write floor pcolor]]
  file-close
end


; ---------------------------------------------------------------------------
;        Can be removed if data file used for map.
; ---------------------------------------------------------------------------
; Map (foo.dat) data file as string, only used if data file not used.  Avoids applet
; security exceptions in earlier systems.
; ---------------------------------------------------------------------------

to-report patchcolors
  report [
   9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 108 108 9 9 9 9 9 107 106 106 106 106 106 106 106 106 106 106 106 106 106 117 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 16 16 16 17 17 16 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 98 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 115 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 17 17 18 18 19 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 16 16 16 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 105 17 17 19 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 108 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 108 105 104 104 104 104 104 104 104 104 104 104 104 104 105 27 17 9 9 9 9 9 9 9 18 16 16 17 17 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 18 26 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 109 108 99 9 9 9 9 98 98 9 17 18 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 105 27 17 9 9 9 9 9 9 9 9 17 18 9 19 18 18 17 16 17 18 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 27 105 115 16 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 18 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 119 9 18 18 9 9 9 9 9 9 107 104 104 106 9 9 9 9 106 108 9 17 19 107 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 106 99 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 116 17 18 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 18 17 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 104 104 17 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 18 17 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 105 27 18 9 9 9 9 9 9 107 104 104 105 9 9 9 108 106 9 9 16 29 106 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 106 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 16 17 18 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 29 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 116 104 104 107 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 18 17 16 17 18 9 9 9 9 9 9 9 9 9 9 99 108 107 106 106 107 9 9 9 9 9 9 9 9 9 9 108 104 104 104 27 18 9 9 9 9 9 9 106 104 104 104 106 99 9 107 107 9 18 17 9 105 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 99 106 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 26 16 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 28 105 104 104 118 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 19 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 108 106 105 104 104 104 104 105 108 9 9 9 9 9 9 9 9 9 108 104 104 105 27 18 9 9 9 9 9 9 106 104 104 104 104 106 107 106 108 9 18 17 9 106 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 98 106 109 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 105 27 17 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 19 17 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 104 104 105 9 9 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 17 17 16 17 19 9 9 9 9 9 9 9 9 9 9 9 108 106 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 108 104 104 105 27 18 9 9 9 9 9 9 106 104 104 104 104 104 104 105 9 9 17 18 9 105 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 98 105 105 105 105 105 105 104 104 105 107 108 108 108 107 106 106 106 105 105 107 28 17 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 18 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 105 106 119 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 18 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 117 104 104 107 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 18 9 9 9 9 9 9 9 9 9 9 9 109 107 105 104 104 104 104 104 104 107 108 105 105 108 9 9 9 9 9 9 9 9 9 109 104 104 105 28 18 9 9 9 9 9 107 104 104 104 104 105 108 107 106 9 9 17 18 9 106 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 109 9 17 18 9 9 9 9 9 9 9 9 9 19 16 18 18 18 18 18 18 17 17 17 17 18 18 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 118 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 17 17 18 9 9 9 9 9 9 9 9 9 9 9 18 18 107 104 104 109 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 107 105 104 104 104 104 104 105 105 106 9 9 107 108 9 9 9 9 9 9 9 9 9 9 109 105 104 108 18 18 9 9 9 9 107 104 104 104 104 104 106 9 108 106 9 9 16 19 9 108 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 16 17 17 17 17 17 18 18 18 9 9 9 9 9 9 18 17 16 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 118 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 9 9 9 9 18 29 106 104 105 119 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 108 105 105 106 107 108 109 109 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 107 9 19 18 9 9 9 109 105 104 104 104 104 104 107 9 108 107 9 19 17 9 9 9 105 97 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 18 29 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 18 9 9 9 9 9 9 9 9 17 118 105 104 105 109 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 108 9 19 18 9 9 9 9 106 104 104 104 104 104 106 108 109 9 9 18 17 9 109 107 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 18 19 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 18 18 108 104 104 105 109 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 19 19 18 18 19 19 19 19 19 29 29 9 9 9 9 9 9 9 9 9 106 108 9 19 18 9 9 9 9 108 104 104 104 104 104 104 104 105 109 9 18 18 9 109 107 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 18 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 9 9 9 9 18 19 107 104 104 105 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 29 28 18 18 18 18 18 19 9 9 9 119 9 9 19 18 9 9 9 9 108 104 104 107 106 104 104 104 104 108 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 17 29 106 104 104 107 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 19 18 18 19 9 9 9 9 9 108 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 19 9 18 18 9 9 9 9 9 106 107 9 117 104 104 104 104 108 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 19 19 18 119 105 104 105 98 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 118 104 107 9 9 9 9 9 9 9 9 9 9 9 119 9 9 9 9 9 9 9 9 9 19 17 16 18 19 9 9 9 9 9 9 119 107 106 117 118 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 18 18 17 16 16 17 17 18 9 9 9 9 9 9 9 9 9 9 108 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 16 16 18 109 104 104 106 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 9 9 9 19 18 19 9 118 118 9 9 9 9 9 9 109 105 108 9 9 9 9 9 9 9 9 9 9 99 106 109 9 9 9 9 9 9 9 9 9 18 17 9 29 19 18 18 18 19 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 18 18 17 16 17 18 19 9 9 9 9 9 9 99 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 19 17 16 18 16 105 108 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 9 9 9 18 18 9 9 9 116 116 9 9 9 9 9 9 9 106 107 9 9 9 9 9 9 9 9 9 9 109 105 109 9 9 9 9 9 9 9 9 9 18 17 9 9 9 18 16 16 18 18 18 18 19 29 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 18 17 17 17 17 17 18 19 9 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 18 17 16 27 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 9 9 19 18 9 9 9 9 9 106 105 109 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 9 9 9 109 105 109 9 9 9 9 9 9 9 9 9 18 17 9 9 17 15 16 19 9 9 9 9 9 19 19 18 17 17 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 106 109 106 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 18 17 16 16 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 18 17 17 18 29 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 19 18 9 9 9 99 107 105 105 105 107 9 9 9 9 9 9 107 104 108 9 9 9 9 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 18 17 9 17 15 17 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 108 105 107 9 107 104 104 104 105 107 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 19 18 18 17 16 17 19 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 18 17 16 17 18 18 19 9 9 9 19 16 17 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 18 18 9 9 107 107 106 104 105 108 107 105 108 9 9 9 9 9 99 105 106 109 9 9 9 9 9 9 9 9 98 104 108 9 9 9 9 9 9 9 9 9 18 17 17 15 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 19 19 9 9 9 9 118 108 9 9 108 106 106 107 106 118 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 19 18 16 17 18 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 16 16 17 16 17 17 16 18 9 9 9 9 17 17 19 9 9 9 9 9 9 18 19 119 107 106 104 104 105 106 107 9 109 105 107 9 9 9 9 9 9 107 104 105 106 106 105 106 106 106 107 107 106 104 106 108 109 9 9 9 9 9 9 9 18 16 16 17 9 9 9 9 9 9 9 9 9 9 9 9 108 106 107 108 109 9 9 9 9 9 9 9 9 9 9 19 18 18 18 19 29 9 9 9 9 9 9 9 117 119 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 18 17 16 18 19 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 19 17 17 9 18 16 17 9 9 9 9 9 9 19 18 9 119 106 104 104 104 104 105 105 107 9 9 107 106 109 9 9 99 107 99 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 18 15 18 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 105 105 105 105 106 106 108 9 9 9 9 9 9 9 9 19 19 18 18 18 18 19 19 9 19 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 18 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 16 18 9 9 9 9 9 9 18 18 9 9 107 104 104 104 104 104 104 106 105 108 9 9 105 105 105 105 105 105 109 9 9 108 107 107 107 107 107 107 107 106 104 104 104 104 105 107 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 106 99 9 108 107 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 19 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 29 18 19 9 9 9 107 104 104 104 104 104 106 109 107 106 109 108 106 108 108 105 105 108 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 105 109 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 108 107 106 105 105 104 104 104 104 104 104 105 106 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 19 9 9 9 9 17 18 9 106 106 118 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 18 29 9 9 9 9 9 9 108 105 104 104 104 108 9 9 99 9 106 107 9 9 109 109 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 105 109 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 107 106 105 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 17 18 108 104 104 115 108 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 18 9 9 9 9 9 9 9 9 9 108 105 104 105 109 9 107 106 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 105 105 105 107 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 17 19 109 104 104 104 105 108 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 9 9 9 9 105 104 106 9 108 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 107 105 104 104 105 108 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 17 17 9 9 9 9 9 9 9 9 108 104 104 104 106 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 9 9 9 109 107 104 104 107 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 18 16 19 9 9 9 9 9 9 106 104 104 105 97 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 106 105 105 106 106 106 108 9 9 9 9 9 9 9 9 9 99 107 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 18 9 9 9 9 108 106 104 107 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 18 16 18 9 9 9 9 107 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 106 107 106 104 104 104 106 106 106 105 105 105 105 105 105 105 105 119 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 104 104 104 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 18 18 9 9 9 9 106 115 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 19 16 17 9 99 106 104 104 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 105 109 9 99 105 104 104 104 104 104 106 106 106 106 106 104 104 105 119 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 105 107 105 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 19 18 19 9 9 9 27 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 29 17 17 28 105 104 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 108 109 108 105 104 104 104 104 105 9 9 9 9 108 104 104 105 18 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 18 18 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 27 116 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 106 106 106 106 106 106 105 104 104 105 9 9 9 9 98 104 104 105 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 19 18 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 28 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 106 9 9 9 9 9 9 108 104 104 105 9 9 9 9 108 104 104 105 19 17 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 28 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 19 19 19 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 106 9 9 9 9 9 9 108 104 104 106 9 9 9 9 108 104 104 105 19 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 19 19 19 19 19 9 9 9 9 9 19 19 19 19 19 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 108 104 104 105 9 9 9 9 108 104 104 105 19 17 17 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 109 29 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 104 105 9 9 9 9 9 9 108 104 104 106 9 9 9 9 109 105 105 106 29 17 18 18 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 107 97 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 29 9 9 9 9 9 9 9 9 9 9 9 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 105 9 9 9 9 9 9 108 105 106 117 9 9 9 9 9 29 28 28 19 16 17 18 16 15 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 115 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 16 18 9 9 9 9 9 9 9 9 107 104 106 9 109 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 19 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 19 9 9 9 9 29 18 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 19 9 29 19 18 17 17 17 16 16 16 16 15 16 17 17 17 17 17 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 119 115 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 15 19 9 9 9 9 9 9 9 9 105 104 104 105 105 107 9 9 29 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 19 18 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 16 16 16 15 16 16 17 17 18 18 19 19 16 19 9 9 9 9 9 9 9 19 19 18 18 17 17 16 16 17 17 17 18 18 9 9 9 119 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 106 108 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 18 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 16 16 16 17 17 18 18 19 9 9 9 9 9 9 9 9 16 19 9 108 107 108 108 109 9 9 9 9 9 9 9 9 9 19 18 18 17 17 16 17 17 17 28 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 17 9 119 108 108 9 9 9 9 9 107 104 104 104 104 104 104 104 107 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 17 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 9 9 9 19 18 18 17 16 16 16 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 109 105 104 104 104 105 106 107 108 108 106 108 98 98 9 9 9 9 9 9 9 19 18 18 17 17 16 17 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 18 17 9 108 105 104 105 106 108 108 109 109 106 104 104 104 104 104 104 104 105 107 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 108 108 98 9 9 18 19 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 15 16 17 17 16 15 26 27 27 28 28 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 107 104 104 104 104 104 104 104 104 104 104 107 105 96 98 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 18 17 9 9 9 108 107 106 105 104 105 105 105 104 104 104 104 104 104 104 104 104 107 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 118 104 104 105 106 107 26 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 17 16 16 17 17 17 17 17 17 17 16 18 27 114 115 105 105 105 105 105 105 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 107 104 104 104 104 104 104 104 104 104 104 105 104 104 104 105 106 107 108 109 109 109 9 9 9 9 9 9 9 9 9 9 18 16 17 17 16 17 17 18 19 9 9 9 9 9 19 17 16 18 9 9 18 17 9 9 9 9 9 9 109 108 107 106 104 104 104 104 104 104 104 104 104 104 104 106 108 108 9 19 18 19 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 106 104 105 108 108 105 114 25 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 17 17 17 16 16 16 17 17 17 18 18 18 18 26 16 18 9 9 9 9 9 18 16 27 115 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 27 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 105 107 9 9 9 9 9 9 9 9 9 17 19 9 9 19 18 17 17 16 17 17 18 17 16 18 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 105 104 104 104 107 106 104 104 104 104 104 104 105 104 106 119 9 19 18 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 108 104 105 118 9 9 106 104 116 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 17 16 16 16 17 17 18 18 18 19 9 9 9 9 9 9 9 9 9 106 104 104 106 9 9 9 9 9 18 16 27 116 104 104 104 104 104 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 25 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 19 18 17 16 16 17 18 19 9 9 17 18 9 9 9 9 9 9 9 9 9 109 104 104 104 104 108 109 106 104 104 104 104 104 104 104 104 105 107 119 9 18 18 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 106 104 107 9 9 109 105 105 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 17 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 9 9 9 9 9 9 18 16 27 116 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 28 109 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 17 16 18 9 9 9 9 9 9 9 9 9 107 104 104 104 104 108 9 109 106 104 104 104 104 104 104 104 104 104 106 118 9 19 18 19 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 108 104 105 99 9 9 107 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 18 19 9 9 9 17 18 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 106 9 9 9 9 9 9 9 9 19 16 17 17 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 107 106 106 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 16 19 9 9 9 9 9 9 9 9 9 106 104 104 104 104 108 9 9 9 108 106 104 104 104 104 104 104 104 104 104 117 9 9 19 18 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 106 105 108 9 108 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 18 29 118 107 106 107 9 17 28 116 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 109 9 9 9 9 9 9 9 9 19 17 17 17 105 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 99 108 108 107 105 104 104 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 108 9 9 9 9 9 108 105 104 104 104 104 104 104 104 107 9 9 9 9 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 109 105 105 106 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 106 104 104 104 105 28 16 18 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 117 9 9 9 9 9 9 9 9 9 9 17 17 17 105 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 109 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 105 105 106 109 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 29 9 9 9 9 9 9 9 9 99 105 104 104 104 105 109 9 9 9 9 9 9 108 105 104 104 104 104 105 107 9 9 9 9 9 18 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 9 9 9 108 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 19 118 107 104 104 104 104 105 17 17 118 104 104 97 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 108 8 9 9 9 9 9 9 9 9 9 9 9 17 17 17 105 104 104 104 108 9 9 9 9 9 9 9 9 9 109 106 104 106 107 27 16 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 108 108 108 9 9 9 9 9 9 9 9 18 17 17 106 116 106 107 108 99 9 9 108 104 104 104 104 105 9 9 9 9 9 9 9 9 9 108 107 107 117 119 9 9 9 9 9 9 16 19 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 98 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 107 104 104 104 104 104 104 117 17 18 107 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 27 116 105 106 99 9 109 9 9 9 9 9 9 9 107 104 104 104 104 25 16 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 99 99 9 108 105 104 104 108 9 9 9 9 9 9 9 18 17 9 9 9 109 108 106 106 106 106 105 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 9 9 9 19 26 119 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 19 107 105 105 104 104 104 104 104 106 29 17 28 106 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 109 106 104 105 106 106 106 105 105 105 104 104 106 108 108 27 16 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 109 105 104 106 105 104 104 104 107 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 108 107 106 105 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 118 105 104 104 104 104 104 105 105 104 108 18 17 17 106 99 107 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 107 104 104 104 104 104 104 104 104 104 104 105 109 9 9 19 17 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 107 106 106 105 105 105 104 105 106 98 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 109 109 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 19 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 18 18 9 9 9 9 9 9 9 9 9 9 9 17 17 9 108 104 104 104 104 104 104 105 108 119 108 9 17 18 117 104 108 109 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 118 105 104 104 104 104 104 104 104 104 105 109 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 107 104 104 104 106 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 19 9 9 9 9 9 9 9 9 9 9 17 17 29 107 105 104 104 104 104 104 114 107 9 9 9 9 16 19 107 103 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 16 16 18 118 105 104 104 104 104 104 104 105 108 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 105 107 107 108 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 106 104 104 104 104 119 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 96 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 17 17 9 9 106 104 104 104 104 104 104 105 109 9 9 9 18 17 9 108 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 16 16 17 17 18 18 18 19 18 16 18 119 97 108 107 105 105 107 99 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 106 106 107 106 106 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 109 106 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 9 9 9 9 17 17 29 107 108 109 106 104 104 104 104 104 107 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 17 16 16 17 17 18 18 18 19 9 9 9 9 9 9 9 9 19 16 18 9 9 9 109 109 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 105 106 108 108 99 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 107 106 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 9 9 18 16 18 9 9 9 9 9 17 17 9 97 104 105 108 109 106 104 104 104 104 108 9 9 9 19 16 9 9 9 9 9 9 9 9 9 19 19 18 18 18 17 17 17 17 16 16 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 16 18 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 109 105 104 104 105 105 106 106 107 108 28 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 99 105 107 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 9 9 9 19 17 18 9 9 9 17 17 9 98 104 104 104 104 106 107 104 104 104 104 105 105 108 9 17 16 17 17 17 17 17 17 17 16 16 16 17 17 17 17 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 26 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 105 105 109 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 9 9 9 9 19 17 18 9 18 17 9 9 105 105 106 104 104 104 104 104 107 108 108 105 104 106 29 16 17 18 18 18 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 16 17 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 19 17 9 117 106 107 107 107 107 106 104 105 98 98 108 108 106 105 105 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 109 105 104 106 109 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 9 9 9 9 9 29 16 16 16 19 9 106 105 108 9 108 105 104 104 105 109 9 9 106 104 117 18 17 29 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 18 17 9 106 104 104 104 104 104 105 105 106 9 9 9 9 9 9 119 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 108 105 107 106 105 108 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 118 9 9 9 9 9 9 17 16 17 9 9 105 106 9 9 9 108 105 104 104 118 9 9 106 104 18 17 28 116 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 18 16 17 18 18 19 9 9 9 9 9 9 108 104 105 105 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 18 18 9 9 108 108 108 108 107 106 105 107 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 107 106 9 9 108 99 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 18 17 29 107 105 109 9 9 107 104 104 104 106 107 108 105 116 28 17 17 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 18 16 18 17 17 16 16 17 17 17 18 18 27 25 27 19 108 107 107 107 107 106 105 108 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 106 97 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 108 108 9 9 9 9 9 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 16 9 9 18 17 28 9 9 9 108 105 104 104 104 104 104 104 104 106 18 17 16 104 104 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 16 9 9 9 9 9 9 18 17 9 9 9 9 9 19 18 18 17 17 17 16 16 17 17 17 18 18 19 29 29 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 106 98 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 16 104 104 105 105 106 109 109 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 109 109 109 108 108 108 109 19 16 9 9 9 18 16 19 9 9 106 104 104 104 104 105 105 106 104 107 18 17 16 104 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 17 17 16 17 18 18 19 9 17 17 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 16 16 17 16 16 17 17 17 18 18 19 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 15 114 104 104 104 104 104 105 105 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 8 105 104 104 104 104 104 104 104 27 16 9 9 9 9 19 17 18 9 116 104 104 106 108 99 9 9 96 106 18 17 16 104 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 17 17 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 19 19 18 18 17 17 16 16 17 17 17 17 16 18 18 19 19 9 9 9 9 9 116 119 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 25 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 104 104 104 104 104 104 25 16 9 9 9 9 9 19 17 18 19 106 104 106 9 9 9 9 97 106 18 17 16 104 104 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 29 16 18 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 108 107 107 108 109 119 9 9 9 9 19 18 18 17 16 17 17 17 16 16 17 17 17 17 25 28 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 25 104 104 104 104 105 105 105 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 104 104 104 104 104 26 17 9 9 9 9 9 9 9 17 17 29 107 105 9 9 9 9 98 106 18 17 116 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 17 18 9 9 9 9 9 9 9 99 108 109 9 9 9 9 9 9 16 29 106 104 104 104 104 105 108 9 9 9 9 9 9 18 17 9 9 9 9 9 19 18 18 17 17 16 16 17 17 17 18 18 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 24 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 104 104 104 105 107 18 17 9 9 9 9 9 9 9 9 17 17 29 107 107 9 9 9 99 106 17 17 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 17 18 9 9 9 9 9 9 9 107 104 104 105 105 106 107 108 29 16 29 106 104 104 104 104 104 108 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 16 16 17 17 18 18 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 15 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 106 104 104 104 105 105 107 9 19 17 9 9 9 9 9 9 9 9 9 17 17 29 107 107 108 98 97 106 17 17 107 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 9 17 18 9 9 9 9 9 9 9 107 105 105 104 104 104 104 106 28 17 18 105 104 104 104 104 105 109 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 18 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 115 104 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 107 9 9 9 29 17 9 9 9 9 9 9 9 9 9 9 18 17 28 108 105 104 104 106 17 17 107 104 105 105 99 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 17 18 9 9 9 9 9 9 9 9 9 109 106 107 107 107 108 18 17 118 104 104 104 104 104 106 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 106 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 107 107 108 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 18 17 28 118 105 104 106 17 17 106 105 108 9 9 9 9 9 9 9 9 29 18 17 16 16 17 18 19 29 26 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 18 9 9 9 9 9 9 9 109 107 107 105 105 105 105 107 17 17 117 104 104 104 104 104 108 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 19 17 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 18 17 106 104 104 104 105 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 18 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 106 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 118 104 106 17 17 105 115 9 9 9 9 9 9 18 17 16 16 17 18 18 9 9 9 9 9 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 9 9 9 9 9 9 9 109 107 106 106 104 104 104 116 17 17 106 104 104 104 104 105 109 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 99 117 18 9 9 18 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 17 18 106 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 106 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 29 17 18 18 28 17 18 107 108 9 9 19 18 17 16 16 17 18 19 9 9 9 9 9 9 9 9 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 109 108 107 119 17 17 106 104 104 104 104 106 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 106 104 106 108 108 9 9 9 9 9 19 18 17 17 16 17 17 18 19 29 9 17 18 116 115 105 105 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 29 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 105 108 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 19 17 19 9 18 17 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 106 106 9 9 9 9 109 108 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 105 104 104 104 104 107 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 106 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 16 17 28 28 18 119 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 18 17 17 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 107 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 16 17 16 17 17 26 28 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 117 9 9 9 9 9 9 9 9 9 9 9 9 9 108 108 9 9 9 9 9 9 108 108 108 108 9 9 9 108 104 105 109 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 105 104 104 104 106 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 96 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 118 119 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 15 17 29 9 117 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 106 9 9 9 9 9 9 17 18 109 106 106 106 117 108 109 9 9 9 9 9 9 9 9 17 28 105 104 104 104 106 109 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 16 17 17 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 19 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 98 106 104 107 108 9 9 9 9 109 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 9 9 9 9 9 9 9 17 28 106 104 104 104 104 104 105 99 9 9 9 9 9 9 9 17 17 104 104 104 104 104 107 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 28 19 18 18 17 17 16 17 17 17 18 19 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 96 106 105 105 116 107 106 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 29 108 104 104 104 104 104 104 108 9 9 9 9 9 9 19 17 17 104 104 104 104 104 107 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 18 16 17 17 17 16 17 17 17 18 18 19 19 19 18 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 19 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 9 9 9 108 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 108 104 104 104 104 104 104 108 9 9 9 9 9 9 19 17 17 104 104 104 104 104 108 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 107 9 9 9 9 9 9 17 28 18 18 17 27 18 18 17 17 17 16 16 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 108 105 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 28 107 104 104 104 104 104 104 108 9 9 9 9 9 9 18 17 117 104 104 104 105 107 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 108 109 9 9 9 9 9 9 9 9 9 9 9 106 104 105 9 9 9 9 9 9 17 27 104 104 104 106 9 9 9 9 9 9 29 18 18 18 17 17 16 17 17 17 18 18 19 9 9 9 9 19 17 17 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 108 106 107 108 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 27 104 104 104 104 104 104 105 109 9 9 9 9 9 9 18 17 9 107 107 107 108 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 18 19 29 9 9 9 9 9 9 116 104 104 104 106 9 9 9 9 9 9 9 9 9 9 119 105 107 9 9 9 9 9 9 9 9 9 9 9 106 104 105 9 9 9 9 9 19 17 17 104 104 104 105 9 9 9 9 9 9 9 18 9 9 9 9 9 19 18 18 17 17 17 17 17 17 17 18 18 16 16 19 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 107 104 104 104 104 104 105 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 17 18 17 17 16 17 17 17 18 18 29 27 15 116 117 108 9 9 9 9 9 9 9 9 9 9 117 104 107 9 9 9 9 9 9 9 9 9 9 9 106 104 106 9 9 9 9 9 19 17 17 104 104 104 106 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 16 16 17 18 19 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 107 104 104 104 104 104 105 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 47 46 46 27 27 27 17 17 17 16 16 17 17 18 19 9 9 9 9 9 9 9 9 107 104 107 9 9 9 9 9 9 9 9 9 9 9 106 104 106 9 9 9 9 9 18 17 118 104 104 104 106 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 16 18 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 19 17 16 9 106 104 104 104 104 104 106 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 108 105 105 105 95 96 98 9 9 9 9 9 9 9 9 9 19 17 9 9 49 46 45 45 46 46 46 47 47 47 47 27 27 27 26 17 16 17 17 17 18 18 19 9 17 115 115 105 106 119 9 9 9 9 9 9 9 9 105 104 106 9 9 9 9 9 18 17 108 104 104 104 107 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 16 16 16 16 16 16 17 16 16 9 106 104 104 104 104 104 107 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 107 104 104 104 104 104 105 99 9 9 9 9 9 9 9 9 18 17 9 9 48 46 45 45 45 45 45 45 45 45 46 46 46 46 47 48 28 27 18 17 17 17 17 17 17 16 16 26 26 28 9 9 9 9 9 9 9 9 105 104 106 9 9 9 9 9 18 17 107 104 104 104 107 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 108 108 108 18 16 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 17 17 18 28 28 28 18 18 28 28 28 28 18 18 28 25 24 27 19 18 19 19 19 19 19 9 9 9 9 9 9 18 17 9 116 104 104 104 104 105 108 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 107 104 104 104 104 104 105 108 9 9 9 9 9 9 9 9 18 17 9 9 48 46 45 45 45 45 45 45 45 45 45 45 45 45 45 47 106 104 106 106 116 117 18 28 27 16 17 17 17 16 17 19 9 9 9 9 9 9 105 104 106 108 98 98 98 109 18 17 107 104 104 104 108 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 109 9 98 104 104 104 105 118 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 28 48 47 47 47 47 47 47 47 47 47 48 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 119 118 18 17 9 9 108 108 108 108 108 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 108 106 105 104 104 104 104 108 9 9 9 9 9 9 9 9 18 17 9 9 9 47 46 46 45 45 45 45 45 45 45 45 45 45 46 47 105 104 104 104 104 104 104 105 104 104 105 116 16 16 29 9 9 9 9 9 9 9 105 104 104 104 104 104 104 117 17 17 116 104 104 104 108 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 103 106 108 106 104 104 104 103 107 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 27 46 45 45 45 45 45 45 45 45 45 46 48 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 107 106 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 109 106 104 104 104 108 9 9 9 9 9 9 9 9 17 18 9 9 109 48 48 47 47 46 45 45 45 45 45 45 45 45 46 47 105 104 104 104 104 104 104 104 104 104 104 104 104 104 118 9 9 9 9 9 9 9 105 104 104 104 104 104 104 116 17 27 115 104 104 104 108 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 106 97 99 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 26 46 45 45 45 45 45 45 45 45 45 46 48 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 107 106 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 107 104 104 104 108 9 9 9 9 9 9 9 9 17 18 9 9 106 104 104 105 105 105 106 45 45 45 45 45 45 46 48 7 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 105 104 104 104 104 104 104 116 17 27 105 104 104 104 109 9 9 9 9 9 9 9 19 29 9 9 9 9 9 98 105 107 109 109 99 109 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 106 106 106 108 118 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 27 47 46 45 45 45 45 45 45 45 45 46 48 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 107 106 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 107 104 104 104 107 9 9 9 9 9 9 9 9 17 18 9 98 104 104 104 104 104 104 104 104 104 104 104 104 105 106 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 109 104 104 106 106 106 106 106 18 17 27 104 104 104 105 99 9 9 9 9 9 9 9 19 29 9 9 9 9 9 106 104 104 104 105 105 105 115 117 118 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 109 108 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 109 106 105 107 9 9 9 9 9 17 49 48 46 45 45 45 46 46 46 47 47 49 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 107 106 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 108 105 105 105 107 9 9 9 9 9 9 9 9 16 19 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 105 109 106 104 104 104 104 104 104 104 104 106 108 108 108 108 108 9 9 9 9 9 9 9 108 104 105 9 9 9 9 9 9 17 26 104 104 104 105 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 106 109 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 106 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 106 104 104 106 9 9 9 9 9 17 9 48 45 45 45 46 48 49 9 9 9 9 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 107 116 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 119 119 119 9 9 9 9 9 9 9 9 9 16 9 9 119 107 107 106 106 105 104 104 104 104 104 104 104 104 105 108 105 104 104 104 104 104 104 104 104 118 9 9 9 9 9 9 9 106 106 106 107 108 106 104 106 9 9 9 9 9 9 17 26 104 104 104 105 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 106 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 106 104 104 106 9 9 9 9 9 17 9 48 45 45 45 48 9 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 107 117 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 99 108 108 107 106 105 105 104 105 106 108 105 104 104 104 104 104 104 104 104 118 9 9 9 9 9 9 109 105 104 104 104 104 104 104 107 9 9 9 9 9 9 16 26 104 104 104 105 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 106 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 106 104 106 98 9 9 9 9 9 18 9 47 45 45 46 48 9 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 107 118 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 99 108 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 109 105 104 104 104 104 104 104 108 9 9 9 9 9 9 16 26 104 104 104 106 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 108 98 107 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 29 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 99 107 107 107 104 107 9 9 9 109 108 108 118 9 9 47 45 45 46 49 9 9 9 9 9 9 106 104 105 109 9 9 9 9 19 19 19 19 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 105 105 104 104 104 104 104 106 9 9 9 9 9 9 9 108 104 104 104 104 104 105 106 9 9 9 9 9 9 19 16 16 104 104 104 107 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 98 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 16 17 18 18 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 108 104 104 104 104 105 107 107 105 105 104 104 105 9 9 47 45 45 46 9 9 9 9 9 9 9 108 108 109 9 18 17 16 16 16 16 16 16 16 16 17 17 17 16 17 19 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 99 99 108 108 107 104 104 107 9 9 9 9 9 9 9 107 104 104 104 104 104 106 9 9 9 9 9 9 9 19 16 17 105 105 105 108 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 117 28 9 18 17 17 16 16 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 108 104 104 104 104 104 104 104 104 104 104 104 105 9 9 47 45 45 46 9 9 9 9 9 9 9 9 9 19 17 17 18 19 9 9 9 9 9 9 19 18 16 17 17 17 16 16 17 17 17 18 18 18 19 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 108 9 9 9 9 9 9 9 107 104 104 104 104 104 106 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 18 17 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 116 115 116 118 9 9 9 9 9 9 9 18 17 18 105 104 104 104 105 106 104 104 107 108 106 104 106 9 9 47 45 45 46 49 9 9 9 9 9 9 9 9 16 17 28 108 108 108 108 109 109 109 9 9 9 17 9 9 9 9 9 19 18 18 17 17 17 16 16 17 17 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 104 104 108 9 9 9 9 9 9 9 107 104 104 104 104 104 105 108 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 108 104 104 105 104 104 104 106 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 98 9 9 9 9 9 9 9 9 19 18 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 118 104 104 104 106 9 9 9 9 9 9 9 29 17 16 104 104 104 104 107 109 107 107 9 9 9 109 9 9 9 47 45 45 46 49 9 9 9 9 9 9 9 17 17 18 105 104 104 104 104 105 105 105 105 106 18 17 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 16 16 16 17 17 17 18 18 19 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 108 9 9 9 9 9 9 9 107 104 104 104 104 104 108 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 107 99 98 108 9 9 9 9 9 9 9 9 9 28 17 17 17 17 17 18 19 9 9 9 9 9 9 9 107 104 104 104 106 9 9 9 9 9 9 9 9 17 26 104 104 104 105 118 9 9 9 9 9 9 9 9 9 9 46 45 45 46 48 9 9 9 9 9 9 19 16 28 106 104 104 104 104 104 104 104 104 104 105 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 29 19 18 18 17 17 16 16 16 16 17 16 16 17 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 108 9 9 9 9 9 9 9 108 106 105 105 105 105 109 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 109 106 105 105 105 105 105 108 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 105 107 9 9 9 9 9 9 9 9 107 105 105 116 17 27 18 17 16 17 17 18 9 9 9 9 106 104 104 104 107 9 9 9 9 9 9 9 9 17 27 118 108 109 109 9 9 9 9 9 9 9 9 9 9 9 47 45 45 45 46 47 47 48 9 9 9 18 17 18 104 105 105 105 105 106 105 105 106 107 109 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 115 104 105 108 9 9 9 9 9 9 9 19 19 18 17 16 17 17 17 17 16 16 16 17 17 17 17 18 18 18 19 19 9 19 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 108 9 9 9 9 9 9 9 9 9 119 109 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 99 99 99 99 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 109 9 9 9 9 9 9 9 107 104 104 104 104 104 106 9 9 9 19 18 17 16 17 18 18 16 104 104 104 108 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 47 45 45 45 45 45 45 46 49 9 9 17 18 107 95 99 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 107 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 19 18 18 18 17 17 17 16 16 17 17 16 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 105 109 9 9 9 9 9 9 9 107 104 104 104 107 108 108 9 9 9 9 9 9 9 18 18 16 25 24 25 25 29 9 9 9 9 9 9 9 9 18 18 109 108 108 119 9 9 9 9 9 9 9 9 9 9 9 48 46 46 46 46 46 45 47 49 9 9 16 18 106 106 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 107 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 19 18 17 17 17 17 17 17 17 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 109 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 107 108 9 9 9 9 9 9 9 98 105 105 109 9 9 9 9 9 9 9 9 9 9 9 28 24 24 24 25 16 18 19 9 9 9 9 9 9 18 17 116 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 48 48 48 48 48 48 49 9 9 9 16 28 105 106 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 107 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 17 17 17 17 17 17 17 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 108 9 9 9 9 9 9 9 107 104 106 109 9 9 9 9 9 9 9 9 9 9 107 104 104 104 114 24 16 16 17 17 18 9 9 9 18 17 17 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 104 106 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 108 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 16 17 18 17 17 17 17 17 17 17 17 17 17 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 108 9 9 9 9 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 118 9 19 18 17 16 17 28 18 17 28 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 118 104 106 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 108 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 19 19 18 18 18 18 17 17 17 17 17 17 17 17 17 17 18 18 18 18 19 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 105 104 104 104 106 108 108 109 9 9 9 9 107 104 104 108 9 9 9 9 119 118 9 9 9 9 105 104 104 104 104 105 109 9 9 9 9 19 18 17 16 16 18 17 27 17 18 18 18 18 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 17 18 108 105 107 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 98 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 18 18 17 17 17 17 17 17 16 17 18 18 18 18 18 19 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 108 107 106 104 104 104 105 105 106 118 9 106 104 105 9 9 9 9 9 106 104 108 9 9 109 104 104 104 104 104 106 9 9 9 9 9 9 9 9 18 16 17 17 26 25 15 17 18 17 17 17 17 16 16 16 17 17 17 18 18 19 9 9 9 9 9 17 18 9 119 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 98 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 18 17 27 26 26 26 16 17 17 17 17 17 17 17 18 17 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 9 9 9 9 9 106 104 107 108 107 106 108 9 105 104 105 98 9 9 9 107 104 105 109 9 9 108 104 104 104 104 104 106 108 109 9 9 9 9 9 9 19 17 9 117 105 104 104 108 9 9 9 9 9 28 25 17 28 28 27 27 26 26 16 27 17 18 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 107 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 49 47 46 46 46 28 9 29 19 19 18 18 18 18 17 17 17 17 17 17 17 17 17 17 17 17 18 18 18 18 19 19 9 9 9 9 9 9
 9 9 19 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 105 106 108 9 9 107 106 9 9 9 9 9 108 104 104 104 104 105 106 106 104 105 108 9 9 9 9 106 104 104 104 105 108 105 105 106 108 9 9 9 9 18 17 9 116 104 104 104 108 9 9 9 9 9 118 104 104 105 46 46 47 47 46 46 45 25 26 16 16 16 18 18 19 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 49 46 45 45 46 48 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 19 18 18 18 18 17 17 17 17 17 16 17 17 17 18 9
 9 9 18 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 16 16 16 16 16 17 17 18 18 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 106 9 9 106 108 9 9 9 9 9 107 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 108 107 107 107 109 107 106 104 104 107 9 9 9 18 17 9 105 104 104 104 119 9 9 9 9 9 108 104 104 115 45 45 45 45 45 45 45 45 47 28 17 17 18 17 16 17 17 18 18 19 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 29 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 48 46 45 45 46 48 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 19 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 16 18 9 9 9 9 9 9 9 9 9 106 104 104 104 106 109 9 106 118 9 9 9 9 9 106 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 105 9 9 9 18 18 109 105 105 104 106 9 9 9 48 48 49 8 106 104 26 45 45 45 45 45 45 45 45 47 49 17 18 9 9 9 19 18 18 17 17 16 17 17 17 17 16 16 17 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 48 46 46 46 47 47 47 47 47 45 45 45 46 48 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 9 9 9 9 9 9 9 106 104 104 104 104 105 106 105 105 106 97 99 9 9 108 105 105 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 105 9 9 9 17 18 108 104 105 104 57 48 48 47 46 46 46 47 48 47 45 45 45 45 45 45 45 45 45 47 48 16 19 9 107 107 107 107 108 118 18 29 19 19 19 18 16 18 18 17 17 17 16 16 17 17 17 17 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 48 45 45 45 45 45 45 45 45 45 45 45 46 48 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 9 9 9 9 9 108 105 104 106 108 107 105 104 104 104 114 27 19 19 9 9 109 107 106 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 17 19 98 104 104 104 46 46 46 46 45 45 45 45 46 46 46 45 45 45 45 45 45 45 45 47 28 16 9 108 104 104 104 104 104 104 105 106 98 9 9 19 17 9 9 9 9 9 18 17 29 18 18 18 17 17 17 16 16 16 17 17 17 17 18 18 18 18 18 18 18 19 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 49 47 46 45 45 45 45 45 45 45 45 45 45 46 48 49 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 18 19 9 9 9 109 108 108 9 9 9 108 108 107 26 15 16 16 16 17 17 18 19 29 18 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 105 109 9 9 17 9 108 104 104 3 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 47 28 16 9 107 104 104 104 104 104 104 104 104 106 9 9 19 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 19 19 18 18 18 18 18 18 16 17 17 17 17 16 16 16 16 16 17 17 17 17 17 17 17 18 18 17 17 17 18 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 48 46 46 46 46 46 46 45 45 45 45 45 45 46 46 46 47 47 47 47 49 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 18 9 9 9 9 9 9 9 9 9 9 18 17 28 25 25 18 18 16 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 118 104 105 9 9 19 17 9 117 104 104 45 46 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 46 47 28 16 9 107 104 104 104 104 104 104 104 104 106 9 9 19 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 19 18 18 18 18 18 18 18 18 17 17 17 17 17 16 16 16 17 17 18 18 18 18 19 29 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 106 107 107 107 108 108 109 107 105 104 104 105 56 48 48 47 45 45 45 45 45 45 45 45 45 45 45 46 48 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 18 19 9 9 9 9 9 9 9 17 18 117 104 104 108 109 17 9 19 18 18 17 17 17 17 17 18 19 9 9 9 9 9 9 9 9 108 104 105 9 9 18 18 9 106 104 104 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 46 45 45 45 45 48 18 17 9 107 104 104 104 104 104 104 104 104 106 9 9 19 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 18 18 17 17 17 16 16 16 17 17 17 18 18 18 19 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 105 104 104 104 104 104 105 9 9 48 45 45 45 46 47 48 47 47 47 47 47 47 49 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 18 18 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 17 18 9 9 9 9 9 16 19 108 104 104 104 104 106 9 9 9 9 9 9 19 18 18 17 16 17 17 18 19 9 9 9 9 107 104 106 9 9 17 18 9 105 104 104 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 46 16 104 104 104 107 18 17 9 106 104 104 104 104 104 104 104 104 106 9 9 19 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 48 47 47 47 47 47 47 47 27 28 18 18 17 17 17 16 16 17 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 47 45 45 45 46 49 9 9 49 58 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 17 19 9 18 17 9 9 108 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 18 18 28 17 107 9 9 17 9 109 104 104 104 46 45 45 45 45 45 45 45 45 45 45 45 45 45 45 46 26 104 104 105 108 18 18 9 106 104 104 104 104 104 104 104 104 106 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 46 45 45 45 45 45 45 45 47 49 9 9 9 9 9 9 19 18 18 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 105 9 48 45 45 45 47 9 9 9 106 105 108 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 16 17 9 9 9 9 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 18 28 28 17 17 16 17 17 18 17 17 9 108 104 104 106 46 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 57 99 9 17 18 9 105 104 104 104 104 104 104 104 104 106 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 46 45 45 45 45 45 45 45 47 49 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 109 49 47 46 46 48 9 9 109 104 104 107 108 107 106 107 28 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 18 17 9 9 9 9 9 9 19 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 105 106 106 117 28 18 18 17 16 18 9 106 104 104 4 46 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 47 9 9 17 18 109 104 104 104 104 104 104 104 104 104 106 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 46 45 45 45 45 45 45 46 48 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 105 99 9 9 48 49 9 99 98 108 104 104 105 105 104 104 105 28 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 18 9 9 19 18 17 16 17 17 17 17 17 17 16 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 18 17 16 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 105 108 9 9 16 9 9 105 104 104 78 47 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 45 46 48 9 9 16 19 108 104 104 104 104 104 104 104 104 104 108 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 46 45 45 45 46 46 46 47 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 118 107 9 108 105 104 104 104 104 104 104 104 106 97 29 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 18 18 17 16 17 18 19 9 9 9 9 9 9 9 19 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 19 18 17 16 17 18 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 108 9 18 17 9 108 104 104 104 98 49 48 46 45 45 45 45 45 45 45 45 45 45 45 45 46 46 46 9 9 19 16 9 119 106 104 104 104 104 104 104 104 104 98 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 46 45 45 46 48 48 48 49 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 107 104 105 104 104 104 104 104 104 104 104 105 107 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 19 18 16 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 16 18 9 9 9 9 9 9 9 9 19 19 18 18 18 18 19 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 19 18 17 16 17 18 19 9 9 9 9 9 9 106 104 104 104 104 104 104 106 9 9 17 18 9 108 104 104 104 105 107 9 48 46 45 45 45 45 45 45 45 45 45 46 46 104 104 116 9 9 18 17 9 9 9 108 107 106 107 106 106 105 106 9 9 9 18 17 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 48 46 46 48 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 106 107 108 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 105 99 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 18 17 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 17 18 17 17 17 17 16 16 16 17 17 17 17 17 17 16 17 17 17 18 19 9 9 17 18 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 18 19 9 9 9 109 118 117 107 107 105 105 118 9 19 17 9 9 106 104 104 104 104 104 106 58 48 47 47 46 46 46 46 47 46 46 46 46 104 104 117 9 9 18 16 18 19 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 9 9 49 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 105 108 108 108 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 98 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 18 18 18 18 16 25 18 9 9 9 9 9 9 9 9 19 19 18 18 18 17 17 18 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 17 17 18 9 9 9 9 9 109 98 9 9 18 18 9 108 104 104 104 104 104 104 104 104 106 108 49 9 49 49 49 49 49 49 49 117 104 104 108 9 9 17 16 17 17 16 17 17 17 18 18 19 9 9 9 9 9 17 18 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 98 106 105 105 104 104 104 106 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 107 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 107 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 15 27 98 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 16 17 18 29 9 9 9 9 9 16 19 9 106 104 104 104 104 104 104 104 104 104 104 105 107 9 108 105 105 105 118 9 107 104 104 106 98 9 16 19 9 9 9 19 18 18 17 17 17 16 17 17 18 18 16 18 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 109 108 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 107 105 104 105 107 108 108 109 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 15 15 94 105 105 106 106 107 108 119 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 17 18 9 9 18 17 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 105 105 104 104 104 105 107 105 104 104 104 98 19 16 9 9 9 9 9 9 9 9 9 9 29 26 24 24 15 16 16 16 17 17 18 18 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 108 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 105 104 104 104 105 105 106 108 109 107 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 118 96 104 104 104 104 104 104 104 105 107 108 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 16 18 17 18 9 9 118 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 18 17 9 9 9 9 9 9 9 9 9 9 108 104 104 104 115 29 9 9 19 18 18 17 16 16 16 17 17 18 18 18 19 9 9 9 9 9 9 9 9 19 18 9 9 9 9 108 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 105 104 104 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 108 105 104 104 104 104 104 104 104 104 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 15 17 9 9 9 9 108 106 104 104 104 104 104 104 104 104 104 104 105 104 104 104 104 104 104 104 104 107 9 17 18 9 9 9 9 9 9 9 9 109 109 106 104 104 104 105 9 9 9 9 9 9 9 16 18 29 19 18 18 17 17 16 16 17 17 18 18 18 19 9 18 18 9 9 9 9 108 105 105 105 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 16 17 16 16 18 9 9 9 9 9 9 9 9 9 109 106 105 107 9 9 9 9 9 9 9 108 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 119 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 108 105 104 104 105 108 107 105 104 104 105 107 9 9 9 9 9 9 9 9 9 9 9 9 17 18 17 16 17 19 9 9 9 108 106 104 104 104 104 104 104 104 107 9 108 107 105 104 104 104 104 106 9 19 16 9 9 9 9 9 9 9 9 106 104 105 104 104 104 104 105 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 19 19 18 16 16 17 26 26 16 16 18 18 19 9 9 9 9 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 17 17 27 27 26 27 17 28 18 17 9 9 9 9 9 9 9 9 9 109 104 104 104 106 109 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 28 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 109 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 117 104 104 108 9 9 107 105 104 104 105 107 9 9 9 9 9 9 9 9 9 17 18 9 9 19 17 16 17 19 9 9 9 108 105 104 104 104 104 106 99 9 9 9 109 107 105 105 105 109 9 18 17 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 105 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 19 17 9 49 46 46 27 16 16 17 17 16 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 16 17 27 27 17 117 105 105 105 104 104 104 106 29 16 19 9 9 9 9 9 9 99 108 107 104 104 104 104 107 9 9 9 9 9 9 9 106 104 104 104 104 104 104 106 108 108 107 106 106 105 105 104 105 104 104 107 29 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 16 17 17 17 17 18 18 18 18 18 18 19 19 9 9 9 9 9 9 18 19 9 9 9 9 109 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 109 9 9 9 9 9 118 105 104 105 108 9 9 99 108 107 105 104 106 9 9 9 9 9 9 9 18 17 9 9 9 9 9 19 17 16 17 19 9 9 9 108 105 104 105 98 9 9 9 9 9 9 109 109 9 9 9 17 18 9 9 9 9 9 9 9 106 104 104 104 106 105 104 104 106 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 17 28 9 48 46 46 48 17 18 9 9 9 19 18 18 18 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 29 18 17 17 16 17 17 26 16 116 105 105 104 104 104 104 104 104 104 104 104 117 18 17 9 9 9 109 107 106 105 104 104 104 104 104 104 106 9 9 9 9 9 9 9 106 105 105 105 105 105 105 109 9 9 9 9 9 9 119 109 109 108 106 106 28 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 19 9 9 9 19 19 18 18 18 18 18 18 18 9 9 9 9 9 17 19 9 9 9 9 99 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 105 107 109 9 9 9 9 108 105 104 104 106 109 9 9 9 108 104 105 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 17 16 17 18 9 9 9 99 99 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 106 104 104 105 109 108 107 107 108 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 29 26 46 46 46 45 45 47 17 18 9 109 119 109 109 109 109 118 18 28 27 17 18 17 17 16 16 17 17 18 18 18 17 16 18 18 18 18 18 17 16 16 16 25 25 26 116 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 28 16 18 108 106 105 105 106 105 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 109 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 109 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 117 105 106 105 106 109 108 107 107 106 108 108 106 105 106 109 9 108 104 107 9 9 9 9 9 19 17 19 9 9 9 9 9 9 9 9 9 9 9 18 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 108 98 9 109 98 9 9 107 104 104 106 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 27 45 45 45 45 45 45 47 17 19 108 104 104 104 105 105 105 105 104 104 114 116 9 9 9 9 9 18 18 18 17 16 16 17 17 17 17 17 18 17 17 9 18 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 107 105 104 104 104 116 17 16 27 107 108 9 9 107 104 104 104 104 104 104 104 105 105 105 105 106 118 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 109 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 109 108 109 106 105 104 105 109 9 9 118 106 105 106 105 106 109 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 16 29 108 104 105 108 106 104 107 108 106 104 104 107 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 19 16 47 47 46 45 45 46 48 17 19 107 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 19 16 9 9 107 104 104 104 104 104 104 104 104 104 105 106 107 108 9 9 109 105 104 104 104 27 16 18 9 9 9 9 109 105 104 104 105 106 105 105 105 105 105 105 105 105 105 105 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 109 109 107 106 108 9 9 9 9 109 98 98 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 18 16 16 18 9 9 9 9 9 9 9 9 9 18 17 19 106 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 17 18 9 9 49 47 45 46 48 17 19 107 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 17 18 9 9 106 104 105 106 106 106 107 98 98 9 9 9 9 9 9 9 107 104 104 104 116 17 17 9 9 9 9 9 109 106 104 107 9 9 9 9 9 9 9 9 9 109 107 105 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 18 9 9 9 9 9 9 9 16 18 106 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 47 45 46 48 17 9 107 104 104 104 104 104 104 104 104 104 104 104 106 106 108 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 18 17 9 9 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 118 17 18 9 9 9 9 9 107 104 105 118 9 9 9 9 9 9 9 9 9 106 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 118 119 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 18 9 9 9 9 18 17 19 105 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 27 26 48 48 48 47 46 45 46 28 16 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 115 28 16 19 9 9 108 106 104 104 104 106 108 9 9 9 9 9 9 9 9 106 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 109 108 108 108 117 117 106 106 105 105 104 104 106 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 117 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 18 9 9 16 19 9 105 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 18 26 46 45 45 45 45 45 45 47 28 17 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 17 18 17 9 99 105 104 104 104 104 104 105 108 108 108 108 108 108 108 107 105 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 109 106 105 105 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 117 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 16 17 17 9 9 108 105 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 17 27 47 46 46 46 46 46 46 48 19 17 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 114 18 17 18 108 104 104 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 119 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9
 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 117 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 15 16 18 9 9 9 108 105 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 18 17 9 48 48 48 48 48 48 48 9 19 17 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 28 16 27 104 104 104 106 119 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 119 105 105 105 105 105 106 107 107 107 98 98 98 109 108 109 9 9 9
 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 117 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 16 17 19 9 9 109 107 105 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 17 18 9 106 104 105 105 105 105 105 108 18 17 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 106 28 16 27 105 106 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 8 8 49 49 48 48 49 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 118 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 19 17 16 17 9 9 9 109 106 104 104 104 106 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 18 17 9 99 105 104 104 104 104 104 104 107 18 17 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 48 47 48 49 49 49 48 48 48 49 9 9 9 9 106 104 104 104 104 117 17 17 29 9 9 9 108 105 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 48 47 47 46 46 46 46 47 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 18 16 16 18 9 9 108 104 104 104 107 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 17 19 9 109 105 104 104 104 104 104 104 107 18 17 9 9 98 108 107 107 107 105 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 47 45 46 46 46 46 46 46 46 47 48 48 9 9 108 104 104 104 104 105 18 17 17 9 9 107 104 104 104 104 104 105 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 49 46 45 45 45 45 45 45 46 48 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 19 18 17 17 16 16 16 16 17 19 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 18 16 16 18 9 107 104 104 108 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 18 17 118 107 108 109 118 108 106 104 104 104 107 18 17 119 109 109 109 108 108 108 106 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 47 45 45 45 45 45 45 45 45 45 46 46 48 9 109 105 104 104 104 104 115 28 17 18 9 106 104 104 104 104 104 108 108 108 108 108 108 107 105 104 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 29 49 46 45 45 45 45 45 45 46 48 9 9 9 9 9 9 9 9
 9 9 9 19 17 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 18 17 17 16 17 18 19 9 9 19 18 17 16 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 19 17 16 17 28 16 106 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 16 26 104 104 107 9 9 9 109 104 104 104 108 18 17 107 105 105 105 105 105 105 104 104 104 104 105 107 107 108 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 48 45 45 45 45 45 45 45 45 45 45 45 46 49 109 105 104 104 104 104 104 116 18 16 18 118 105 104 104 104 104 106 99 9 9 9 9 99 105 104 104 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 19 49 46 45 45 45 45 45 45 46 48 9 9 9 9 9 9 9 9
 9 9 18 17 18 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 18 18 17 16 17 18 18 9 9 9 9 9 9 9 9 9 18 15 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 17 17 17 18 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 19 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 17 26 115 104 104 97 9 9 9 109 105 104 104 118 18 18 9 109 109 99 99 108 105 104 104 104 104 105 106 106 106 108 9 9 9 9 9 9 9 16 28 109 118 109 118 108 108 9 9 9 9 18 17 9 9 109 118 48 45 45 45 45 45 45 45 45 45 45 45 46 49 118 104 104 104 104 104 104 104 107 18 16 28 118 105 104 104 104 104 105 106 106 107 107 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 48 48 48 47 46 46 47 48 9 9 9 9 9 9 9 9 9
 9 9 19 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 17 17 16 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 16 16 17 18 9 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 18 19 19 18 18 17 16 18 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 18 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 19 16 15 104 104 104 107 9 9 9 109 105 104 105 119 18 18 9 9 9 9 9 9 9 109 109 109 108 108 108 107 106 105 108 9 9 9 9 9 9 16 25 115 105 105 104 104 104 106 108 108 108 28 16 27 105 105 104 57 46 45 45 45 45 45 45 45 45 45 45 47 9 106 104 104 104 104 104 104 104 104 117 28 16 29 108 105 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 9
 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 18 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 19 18 17 16 17 18 18 18 18 17 16 17 17 18 18 9 9 9 9 9 9 9 9 9 9 19 16 18 9 19 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 19 9 9 9 9 9 9 18 17 9 9 9 9 9 9 17 25 105 107 105 104 108 9 9 9 108 104 104 105 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 108 9 9 9 9 9 9 17 18 9 9 107 105 105 105 104 105 105 104 17 17 25 104 104 104 104 57 47 46 45 45 45 45 45 45 45 47 49 108 104 104 104 104 104 104 104 104 104 104 117 27 16 19 108 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 18 9 9
 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 18 18 17 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 17 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 19 9 9 9 9 17 18 9 9 9 9 9 18 15 114 107 9 108 104 107 9 9 9 106 104 104 106 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 107 108 108 109 9 9 9 16 29 9 9 9 99 109 109 109 109 109 109 9 18 15 115 104 104 104 105 28 47 46 45 45 46 46 46 47 48 107 105 104 104 104 104 104 104 104 104 104 104 104 16 26 16 19 98 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 28 9 9 9 9 9 9 9 9 19 18 17 16 17 19 9 9 9 9
 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 19 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 18 9 9 16 9 9 9 9 9 19 16 25 105 9 9 9 108 108 9 9 9 106 104 104 106 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 105 105 105 106 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 16 27 105 104 104 104 105 57 47 47 48 48 47 47 47 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 114 26 16 19 108 105 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 19 9 9 9 9 19 18 17 16 17 18 9 9 9 9 9 9 9
 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 9 17 17 18 18 18 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 17 17 9 9 9 9 9 17 26 104 107 9 9 9 9 9 9 9 9 108 107 107 118 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 108 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 116 104 104 104 104 104 105 106 106 106 105 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 118 18 16 18 108 105 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 18 17 17 16 17 18 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 9 18 17 17 17 17 17 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 16 19 9 9 9 18 16 15 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 107 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 26 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 105 105 105 105 105 105 105 105 105 105 105 106 98 9 9 18 16 18 119 107 106 105 105 105 105 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 18 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 16 17 16 17 19 18 17 26 105 108 9 9 118 109 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 106 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 107 108 106 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 29 28 27 28 28 28 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 16 16 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 19 17 9 9 9 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 9 9 9 9 9 9 9 9 9 19 17 9 9 17 16 16 18 8 98 9 9 109 105 105 109 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 106 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 109 107 107 107 106 106 105 105 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 17 16 17 17 17 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 17 18 18 19 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 19 9 9 9 9 9 9 9 19 17 29 105 105 105 105 106 108 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 18 9 9 9 9 9 9 9 18 17 9 9 9 19 17 17 19 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 115 117 18 17 9 9 9 9 9 9 9 9 19 19 19 19 19 19 18 16 18 9 9 9 9 9 9 29 29 29 17 15 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 18 28 29 9 9 9 9 19 19 19 19 19 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 9 9 9 9 9 9 9 108 118 28 17 9 108 117 106 104 106 108 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 107 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 19 9 9 9 9 9 17 18 9 9 9 9 9 18 16 18 29 9 106 104 104 108 9 9 9 109 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 28 28 17 16 18 18 17 17 17 17 17 16 16 16 17 16 16 16 16 16 16 16 16 16 16 16 16 16 17 17 16 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 18 17 16 17 19 9 97 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 9 108 105 104 104 27 17 9 9 9 9 119 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 28 119 107 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 19 16 9 9 9 9 9 9 9 19 17 16 17 18 18 118 9 9 9 109 106 108 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 18 17 17 17 17 16 16 17 17 17 17 18 18 18 18 19 19 9 9 9 9 9 9 9 9 18 17 19 9 9 19 9 19 19 19 19 19 19 19 19 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 19 9 9 9 109 94 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 19 9 9 9 9 9 9 9 9 96 103 104 104 26 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 108 105 104 104 104 104 104 107 9 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 9 9 18 17 9 9 9 9 9 9 9 9 9 19 18 17 17 17 18 18 19 29 28 29 9 9 9 18 16 28 19 18 18 18 17 17 17 16 16 16 17 17 17 17 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 96 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 9 9 9 9 9 9 9 9 9 9 108 106 106 107 27 17 9 108 107 9 9 9 118 118 9 9 109 107 107 108 108 108 108 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 107 108 9 9 9 9 17 18 9 9 105 104 104 104 104 104 105 107 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 16 16 17 17 17 17 17 16 15 16 17 17 17 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 106 9 9 9 107 105 105 105 105 104 104 104 104 104 106 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 105 104 104 104 105 98 9 9 16 19 9 99 105 104 104 104 104 104 104 104 104 105 109 9 9 9 9 107 105 105 107 108 9 9 9 9 9 9 9 9 18 16 16 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 18 17 17 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 107 9 9 9 9 109 108 108 108 108 106 104 104 107 108 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 118 104 104 104 104 104 107 9 18 17 29 106 105 104 104 104 104 104 104 104 104 104 105 9 9 9 9 109 105 104 104 104 104 106 108 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 106 107 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 108 9 9 9 9 9 9 9 9 9 109 106 107 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 107 9 18 17 9 119 108 97 107 106 105 104 104 104 104 104 106 9 9 9 9 106 104 104 104 104 104 104 104 106 9 9 9 9 9 9 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 98 99 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 107 9 9 9 9 9 9 9 9 9 9 9 118 117 107 106 105 104 104 104 106 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 18 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 107 106 105 104 107 9 17 18 118 105 106 96 106 107 107 107 107 106 105 104 106 9 9 9 9 106 104 104 104 104 104 104 104 104 108 9 9 9 9 9 16 17 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 105 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 108 9 9 9 109 108 108 107 107 107 106 105 104 104 104 104 104 104 104 104 106 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 29 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 109 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 109 108 109 9 16 28 106 104 104 104 104 104 105 106 109 9 109 107 118 9 9 9 9 9 108 107 105 104 104 104 104 104 105 108 9 9 9 18 17 9 9 19 18 17 17 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 107 106 106 107 107 107 109 9 9 9 9 9 9 9 9 9 9 9 9 107 105 104 105 106 106 106 105 104 104 104 104 104 104 104 104 104 104 104 104 105 106 106 108 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 18 16 17 29 9 9 9 9 9 9 9 9 9 9 108 106 106 105 106 119 9 9 9 9 9 9 18 17 9 107 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 18 17 17 17 16 16 17 17 17 18 18 18 19 9 9 9 18 19 9 9 9 9 9 19 17 18 104 104 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 99 108 106 105 104 104 104 104 107 99 9 17 18 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 108 99 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 19 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 119 9 9 9 9 9 9 17 18 9 106 104 108 9 9 9 9 109 109 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 19 18 18 18 17 17 16 16 17 17 17 18 18 18 19 19 9 18 17 117 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 109 107 104 104 104 104 105 28 16 19 9 9 9 9 9 9 9 9 9 118 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 105 9 9 9 9 9 9 9 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 109 109 109 9 9 9 9 9 9 9 9 105 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 17 18 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 109 108 107 107 105 104 104 104 105 119 9 9 9 9 9 18 17 9 109 105 104 105 107 9 107 106 104 106 9 107 107 107 107 107 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 17 16 16 16 16 105 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 105 104 105 108 9 9 9 9 106 104 104 104 105 27 17 9 9 9 9 9 9 9 9 9 118 106 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 105 9 9 9 9 9 9 9 106 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 105 105 105 107 9 9 9 107 106 107 106 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 18 17 9 9 18 16 17 9 9 9 9 9 9 9 9 109 108 107 106 105 104 104 104 104 104 104 104 104 119 9 9 9 9 9 17 18 9 108 104 104 104 107 108 104 104 104 97 9 106 104 104 104 104 108 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 106 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 99 105 104 104 104 106 108 99 9 9 98 106 94 108 17 18 9 9 9 9 9 9 9 9 9 118 106 9 9 9 9 9 9 9 9 9 9 9 9 18 18 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 105 9 9 9 9 9 9 9 106 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 105 104 104 106 9 9 9 107 105 105 105 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 29 16 18 17 16 18 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 19 16 9 9 106 104 104 104 108 106 104 104 105 99 9 99 108 107 107 98 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 28 105 104 104 104 104 104 104 104 104 107 108 9 9 9 9 9 9 98 104 104 104 104 104 104 107 9 9 9 9 99 9 17 19 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 105 9 9 9 9 9 9 9 106 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 105 104 104 107 9 9 9 9 9 9 9 106 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 97 9 9 9 9 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 16 16 18 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 105 98 9 9 9 9 9 17 17 9 108 104 104 104 105 108 105 104 104 107 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 104 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 108 104 104 104 104 104 104 107 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 9 9 9 9 9 9 9 106 104 104 104 105 107 107 108 108 108 99 9 9 9 9 9 9 9 18 16 105 104 105 108 9 9 9 9 9 9 9 106 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 105 104 104 104 104 104 104 104 104 104 104 105 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 106 106 107 107 107 108 98 9 9 9 9 9 9 16 19 9 105 104 104 104 107 107 104 104 105 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 116 104 104 104 104 104 106 107 106 105 104 104 105 9 9 9 9 9 108 104 104 104 104 104 104 104 106 108 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 105 105 106 108 109 9 9 9 18 16 115 104 104 105 107 9 9 9 9 9 9 106 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 108 109 108 108 108 108 108 108 108 108 108 109 109 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 18 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 117 106 106 108 109 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 104 104 104 105 108 105 104 104 108 9 9 9 9 9 9 9 9 9 17 18 109 109 109 109 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 117 104 104 104 104 104 108 108 107 108 108 107 106 109 9 9 9 9 99 106 105 104 104 104 104 104 104 104 105 106 19 17 18 9 9 9 9 9 9 9 9 9 9 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 105 105 106 108 18 17 107 105 105 104 105 108 108 109 109 109 108 105 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 99 105 104 109 9 9 9 9 9 9 9 9 9 9 9 9 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 108 104 104 104 104 107 107 104 104 106 9 9 9 9 9 9 9 9 9 9 17 26 105 104 105 105 105 105 106 106 106 107 107 108 108 108 108 108 98 99 9 9 9 9 9 17 18 107 104 104 104 104 104 107 105 104 107 9 109 105 105 106 108 9 9 9 9 99 97 105 104 104 104 104 107 107 106 27 17 9 9 9 9 9 9 9 9 9 9 9 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 16 18 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 99 105 105 109 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 19 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 106 104 104 104 108 106 103 104 108 9 9 9 9 9 9 9 9 9 9 17 27 106 105 106 106 105 105 104 104 104 104 104 104 104 104 104 104 104 105 105 106 106 108 9 17 19 108 106 105 105 105 105 106 104 104 106 9 108 104 105 104 105 119 9 9 9 9 9 9 107 106 106 106 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 106 106 106 106 106 105 105 104 104 104 104 104 104 104 104 115 16 18 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 9 9 9 9 9 99 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 108 106 107 9 109 107 108 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 109 109 108 108 107 107 106 106 106 105 105 105 104 104 104 116 28 16 9 9 9 9 109 99 99 107 104 104 106 9 109 104 105 105 105 119 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 109 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 17 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 108 98 98 108 108 109 118 18 17 18 9 9 9 108 108 107 106 106 106 106 106 106 106 105 105 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 99 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 109 108 107 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 109 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 109 9 19 16 9 9 9 9 9 9 9 98 104 104 107 9 109 104 104 104 107 9 9 9 119 105 105 106 107 9 9 9 9 9 9 16 18 9 9 9 9 9 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 15 29 18 17 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 99 9 9 9 9 9 9 9 9 9 9 9 9 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 105 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 106 104 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 108 104 104 107 9 108 104 104 104 108 9 9 9 107 104 104 104 105 9 9 9 9 9 18 15 18 9 9 9 9 9 9 9 9 9 108 105 104 106 109 9 9 9 9 9 9 9 9 9 9 9 9 108 15 25 25 9 9 9 9 9 9 9 18 17 18 17 16 16 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 107 106 106 105 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 104 104 106 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 108 104 104 106 107 107 104 104 104 109 9 9 9 106 104 104 104 117 9 9 9 9 9 17 15 19 9 9 9 9 9 9 9 9 108 105 106 106 105 105 108 9 9 9 9 9 9 9 9 9 9 9 106 25 25 15 9 9 9 9 9 9 9 19 18 119 29 17 17 19 18 18 18 17 17 17 17 17 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 105 104 105 106 107 107 98 98 98 98 108 108 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 105 9 9 9 9 105 104 104 104 119 9 9 9 9 9 16 16 19 9 9 9 9 9 9 9 9 105 107 9 9 109 106 108 9 9 9 9 9 9 9 9 9 9 9 105 16 25 15 9 9 9 9 9 9 9 19 18 117 17 16 25 109 9 9 9 9 9 9 19 18 18 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 106 106 106 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 16 17 16 16 17 17 17 17 18 18 18 19 19 9 9 9 9 9 17 19 9 107 105 105 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 106 9 9 9 9 105 104 104 106 9 9 9 9 9 17 17 17 19 9 9 9 9 9 9 9 118 104 107 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 105 16 26 15 9 9 9 9 9 9 9 9 17 117 27 26 25 99 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 17 17 17 18 18 19 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 105 106 107 107 107 107 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 18 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 29 19 18 18 17 17 17 17 17 16 16 16 17 17 17 16 17 18 18 28 28 28 28 18 18 18 118 118 108 109 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 107 104 105 107 107 107 104 104 107 9 9 9 109 104 104 104 107 9 9 9 9 18 17 19 17 19 9 9 9 9 9 9 9 9 106 105 106 107 107 106 109 9 9 9 9 9 9 9 9 9 9 9 106 16 25 15 9 9 9 9 9 9 9 9 17 27 25 16 116 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 18 18 17 17 17 17 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 109 107 107 9 9 9 9 9 9 19 19 18 17 16 17 17 17 17 17 17 16 16 17 17 17 18 18 18 18 18 19 19 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 109 107 108 9 9 9 106 104 107 9 9 9 98 104 104 104 108 9 9 9 18 16 19 9 17 19 9 9 9 9 9 9 9 9 9 108 108 107 108 109 9 9 9 9 9 9 9 9 9 9 9 9 106 16 25 15 9 9 9 9 9 9 9 9 18 18 17 16 116 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 17 17 16 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 17 18 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 99 99 9 9 9 9 9 18 18 9 9 9 9 108 104 104 107 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 19 19 18 18 18 17 17 17 17 17 17 16 16 17 17 17 17 16 17 18 18 18 18 19 29 9 9 9 9 9 9 9 118 118 109 9 9 9 9 108 108 109 9 9 9 19 16 18 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 25 15 9 9 9 9 9 9 9 9 9 17 17 26 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 19 18 18 18 17 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 19 19 18 18 17 16 18 9 9 9 9 9 9 9 9 19 19 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 118 107 105 105 106 98 9 9 9 17 19 9 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 17 27 27 27 18 17 16 17 17 17 17 17 17 17 16 16 16 17 17 17 17 18 18 18 18 18 18 19 19 9 9 9 9 19 17 18 9 9 9 17 28 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 25 16 9 9 9 9 9 9 9 9 9 9 17 25 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 18 27 27 27 26 25 25 16 18 18 17 17 17 17 17 17 16 16 17 17 18 16 19 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 107 104 104 104 106 9 9 9 9 9 9 99 108 106 105 105 104 104 104 104 105 106 106 17 17 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 105 118 18 17 9 117 117 19 9 9 9 9 27 26 18 18 16 18 18 18 17 17 17 17 17 17 16 16 16 17 16 17 9 9 9 9 17 27 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 25 16 9 9 9 9 9 9 9 9 9 9 17 25 117 9 9 9 9 9 9 9 9 9 9 9 107 105 105 106 118 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 18 17 17 17 17 18 18 18 18 18 18 18 18 18 18 18 18 17 17 17 17 17 16 16 16 16 16 15 16 17 17 18 17 17 18 18 18 19 9 28 27 9 9 9 17 9 9 9 9 98 107 107 107 118 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 106 9 9 9 108 107 106 105 104 104 104 104 104 104 104 104 105 108 115 25 18 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 108 18 17 118 104 104 108 9 9 9 108 105 105 9 108 106 9 9 9 9 9 9 9 9 9 9 9 19 16 17 9 9 9 9 9 17 17 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 15 25 9 9 9 9 9 9 9 9 9 9 16 25 118 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 18 18 18 18 17 17 17 16 15 15 16 26 26 26 25 25 25 25 25 25 16 115 114 116 118 9 9 107 107 9 109 106 9 9 107 105 108 109 9 17 9 9 9 9 107 104 104 104 107 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 105 107 106 105 104 104 104 104 104 104 104 104 104 104 104 104 105 106 26 17 19 9 9 9 9 9 105 104 104 108 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 108 18 18 118 104 104 108 9 9 9 106 104 104 106 105 104 108 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 17 17 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 15 25 9 9 9 9 9 9 9 9 9 9 16 25 119 9 9 9 9 9 9 9 9 9 9 9 105 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 108 107 104 105 106 106 104 106 105 104 104 104 106 28 17 9 9 9 9 9 106 104 104 107 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 116 108 18 17 18 9 9 9 9 9 109 105 104 107 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 105 109 17 18 117 104 104 98 9 9 9 106 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 17 17 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 15 25 109 9 9 9 9 9 9 9 9 19 15 16 9 9 9 9 9 9 9 9 9 9 9 9 107 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 109 106 104 104 104 104 104 104 104 104 104 104 106 28 17 9 9 9 9 9 106 104 104 106 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 105 106 107 109 9 9 18 17 9 9 9 9 9 9 107 104 104 108 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 106 9 17 18 107 104 104 108 9 9 9 105 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 16 18 9 9 9 9 9 9 19 17 17 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 25 15 25 109 9 9 9 9 9 9 9 9 18 25 116 9 9 9 9 9 9 9 9 9 9 9 9 106 105 105 106 99 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 108 108 109 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 99 106 104 104 104 104 104 104 104 104 104 104 106 18 17 9 9 9 9 9 106 104 105 107 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 116 104 104 104 104 104 104 104 104 105 106 106 98 99 9 9 9 19 18 17 18 9 9 9 9 9 9 9 105 104 104 107 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 106 9 17 28 106 104 104 107 119 9 108 105 104 104 104 104 105 109 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 19 16 16 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 16 26 24 119 9 9 9 9 9 9 9 9 18 25 107 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 109 106 105 105 105 104 104 104 105 105 106 99 99 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 99 105 104 104 104 104 104 104 104 104 104 104 106 18 17 9 9 9 9 9 106 104 106 9 9 18 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 118 106 105 105 104 105 106 107 108 9 9 9 9 9 9 19 17 26 18 9 9 9 9 9 9 9 9 9 107 104 104 104 107 109 9 9 9 9 9 9 19 17 9 9 9 9 9 108 105 105 105 105 106 107 109 108 106 107 106 104 104 105 105 105 108 9 16 27 105 105 105 105 105 106 105 104 104 104 104 104 105 109 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 19 16 15 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 16 26 24 119 9 9 9 9 9 9 9 9 17 25 107 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 105 108 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 106 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 108 109 104 104 104 104 104 104 104 104 104 104 104 106 27 17 9 9 9 9 9 107 104 105 109 9 18 9 9 9 9 9 99 9 9 9 9 9
 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 109 109 9 9 9 9 9 9 9 9 19 17 26 14 114 118 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 105 106 108 9 9 9 18 17 9 9 9 119 107 105 104 104 104 104 104 105 107 104 104 104 104 105 108 109 109 9 9 19 16 29 9 9 9 9 9 9 109 109 108 108 108 107 108 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 19 16 14 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 16 16 24 119 9 9 9 9 9 9 9 18 16 25 107 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 109 9 108 104 104 104 104 104 104 104 104 104 104 104 106 17 17 9 9 9 9 9 107 104 104 99 9 18 9 9 9 108 105 105 97 9 9 9 9
 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 116 104 104 107 9 9 9 9 9 9 9 9 9 109 104 104 104 104 104 104 104 106 9 9 9 18 18 9 9 9 108 104 104 104 104 104 104 104 105 106 104 104 104 104 106 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 9 9 9 9 9 9 9 9 9 18 16 114 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 16 16 25 119 9 9 9 9 9 9 9 16 25 25 108 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 99 9 108 104 104 104 104 104 104 104 104 104 104 104 116 17 18 9 9 9 9 9 108 104 104 109 9 19 9 96 106 105 104 104 105 9 9 9 9
 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 107 9 9 9 17 18 9 9 9 107 104 104 104 104 104 104 104 105 106 104 104 104 104 106 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 18 26 114 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 116 16 16 28 9 9 9 9 9 9 9 109 25 26 16 108 9 9 9 9 9 9 9 9 9 9 9 109 104 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 107 9 9 108 104 104 104 104 104 104 104 104 104 104 104 117 17 18 9 9 9 9 9 108 104 104 108 9 19 109 104 104 104 104 104 104 109 9 9 9
 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 107 9 9 9 9 9 9 9 9 9 99 105 104 104 104 104 104 105 119 9 9 9 17 19 9 9 9 108 104 104 104 104 104 104 104 106 106 104 104 104 104 107 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 18 16 115 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 16 16 28 9 9 9 9 9 9 9 108 25 16 16 98 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 106 106 106 107 107 97 108 108 109 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 117 17 18 9 9 9 9 9 109 106 105 108 29 19 108 104 104 104 104 104 104 109 9 9 9
 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 106 108 109 9 9 17 9 9 9 9 119 107 106 105 105 105 104 105 108 107 104 104 104 104 108 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 18 16 116 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 16 16 25 9 9 9 9 9 9 9 108 25 16 116 99 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 17 17 19 9 9 9 9 9 9 9 109 9 29 9 108 104 104 104 104 104 104 106 108 9 9
 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 106 108 108 108 109 108 98 108 108 109 108 105 104 104 104 104 104 104 104 104 108 19 16 18 18 18 18 19 9 29 18 18 18 118 119 9 108 106 107 107 107 9 9 9 9 9 9 17 18 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 116 26 24 9 9 9 9 9 9 9 108 25 16 116 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 105 107 109 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 18 17 19 9 9 9 9 9 9 9 9 9 28 9 108 104 104 104 104 104 104 104 104 107 9
 9 9 9 17 19 9 108 105 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 18 16 17 17 17 17 16 16 16 17 17 17 17 18 18 18 18 18 29 29 9 9 9 9 9 9 17 18 106 105 105 106 106 106 107 108 108 109 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 116 26 24 109 9 9 9 9 9 9 108 25 16 16 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 115 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 28 17 9 9 9 9 9 9 9 9 9 9 18 9 108 104 104 104 104 104 104 104 104 106 9
 9 9 9 17 19 9 118 104 104 104 106 108 109 9 9 9 9 9 9 9 9 109 109 109 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 118 17 18 9 9 9 9 9 9 19 19 18 18 18 18 17 17 17 17 17 17 16 16 17 17 17 17 16 17 27 27 17 17 117 116 106 106 105 105 105 105 107 97 9 9 19 16 18 119 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 16 26 24 109 9 9 9 9 9 9 98 26 16 28 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 107 109 116 107 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 105 28 17 9 9 9 9 9 9 9 9 9 9 18 9 108 104 104 104 104 104 104 104 104 107 9
 9 9 9 17 19 9 9 118 106 104 104 104 105 106 107 108 99 99 99 108 106 105 104 105 99 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 114 18 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 16 16 16 25 26 16 17 27 27 28 28 27 17 16 115 115 98 9 17 18 19 116 116 107 99 9 9 109 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 109 108 108 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 17 27 24 109 9 9 9 9 9 9 9 28 17 18 9 9 9 9 9 9 9 9 9 9 9 29 29 18 26 26 26 27 27 19 9 28 25 28 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 28 17 9 9 9 9 9 9 9 9 9 9 18 9 108 104 104 104 104 104 104 106 108 109 9
 9 9 9 17 19 9 9 9 9 97 106 105 104 104 104 104 104 104 105 104 104 104 104 104 107 9 9 9 9 9 9 9 109 107 106 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 115 28 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 19 117 16 25 25 24 24 24 25 25 26 26 26 27 18 17 17 9 28 26 25 24 26 9 27 115 115 106 106 107 107 109 18 18 9 9 9 9 9 107 105 104 104 104 105 105 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 118 117 27 24 118 9 9 9 9 9 9 9 9 18 16 17 17 17 17 17 17 17 17 17 17 17 17 17 16 26 16 17 17 17 17 16 16 16 16 17 17 17 18 18 16 18 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 28 17 9 9 9 9 9 9 9 9 9 9 18 9 107 104 104 104 104 104 104 108 9 9 9
 9 9 9 17 19 9 9 9 9 9 9 109 108 106 104 104 104 104 104 104 104 104 104 104 105 98 9 9 9 109 108 106 105 104 104 104 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 117 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 109 107 106 105 105 106 116 116 15 24 24 17 16 16 19 18 26 24 25 25 26 27 26 26 26 26 26 26 25 27 17 18 9 9 9 99 106 104 104 104 104 104 104 104 104 104 106 106 9 9 9 106 105 105 106 107 109 109 109 107 115 26 24 118 9 9 9 9 9 9 9 9 9 29 18 18 18 18 19 19 19 29 9 9 9 9 9 17 114 116 9 9 9 28 16 29 9 19 19 18 18 18 17 16 18 9 9 9 9 9 9 109 109 109 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 18 17 9 9 9 9 9 9 9 9 9 9 18 9 107 104 104 104 104 104 104 106 9 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 106 108 107 106 105 104 104 104 104 104 104 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 106 105 118 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 98 106 105 105 106 105 104 116 29 17 18 9 27 27 104 104 104 104 115 15 26 26 26 26 26 25 26 16 16 17 18 28 27 114 104 104 104 104 104 104 104 104 104 104 106 9 9 108 104 104 104 104 104 105 105 105 105 107 17 24 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 108 9 108 104 106 109 9 9 9 9 9 9 17 18 9 9 9 9 9 106 105 106 116 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 17 17 9 9 9 9 9 9 9 9 9 9 18 9 107 104 104 104 104 104 104 105 109 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 108 104 104 104 116 108 118 17 17 9 9 18 27 15 104 104 104 104 104 104 104 104 104 104 104 116 17 17 18 17 17 16 15 24 24 24 25 25 15 114 114 104 104 106 9 9 108 104 104 104 104 104 104 105 107 9 9 17 24 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 104 107 107 104 104 105 109 9 9 9 9 9 18 18 9 9 9 9 107 104 107 9 9 106 104 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 105 17 18 9 9 9 9 9 9 9 9 9 9 18 9 107 104 104 104 104 104 104 104 108 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 109 106 104 104 107 9 18 16 29 9 9 28 17 27 26 25 15 14 114 114 104 104 104 104 104 116 17 18 9 9 28 9 19 18 18 17 17 16 16 26 25 24 25 26 9 9 9 107 105 104 104 104 104 104 108 9 9 17 24 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 106 105 104 104 107 9 9 9 9 9 18 17 9 9 9 9 106 104 109 9 9 109 105 104 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 115 17 18 9 9 9 9 9 9 9 9 9 9 18 9 106 104 104 104 104 104 104 104 106 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 17 18 9 9 9 9 109 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 118 108 9 9 16 18 108 98 98 106 116 26 27 27 27 26 26 26 26 26 25 25 25 26 16 18 9 107 106 9 9 9 9 9 9 9 19 18 18 17 17 16 16 17 17 17 17 27 26 16 16 115 118 9 9 17 24 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 106 104 104 105 108 9 9 9 9 18 17 9 108 107 106 104 105 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 115 17 18 9 9 9 9 9 9 9 9 9 9 19 9 105 104 104 104 104 104 104 104 106 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 19 16 9 9 9 9 107 105 105 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 17 17 118 104 104 105 106 105 104 115 116 116 16 17 18 28 27 28 28 19 19 17 18 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 28 26 28 18 16 17 17 17 16 16 16 17 18 19 17 24 97 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 105 106 104 104 104 108 9 9 9 9 17 29 106 104 104 104 105 99 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 104 116 17 18 9 9 9 9 9 9 9 9 9 9 19 9 105 104 104 104 104 104 104 104 106 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 99 9 9 9 18 17 9 9 9 9 106 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 99 106 105 105 107 18 17 9 9 9 9 9 9 9 9 9 9 9 18 16 9 107 104 104 104 104 104 104 106 107 104 104 104 105 106 116 116 108 9 9 17 18 106 104 104 105 9 9 9 9 9 9 9 9 9 9 9 107 105 9 108 115 119 9 9 19 19 18 18 17 17 16 16 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 108 106 104 104 106 99 9 9 9 17 18 117 104 104 104 104 107 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 116 17 19 9 9 9 9 9 9 9 9 9 9 19 9 107 104 104 104 104 104 104 104 106 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 17 19 9 9 9 9 106 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 104 107 18 17 9 9 9 9 9 9 9 9 9 9 9 16 19 9 107 104 104 104 104 104 104 105 107 104 104 104 107 9 9 9 9 9 9 17 18 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 107 105 108 105 104 109 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 107 108 104 104 104 108 9 9 9 18 17 118 106 104 104 104 105 109 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 107 105 106 106 106 106 106 106 106 106 106 106 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 109 108 107 107 107 107 107 107 108 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 18 17 9 9 9 9 9 105 104 104 104 108 9 9 9 9 9 108 106 106 106 106 106 106 105 104 104 104 107 18 17 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 108 107 105 104 104 104 105 107 104 104 104 107 9 9 9 9 9 9 17 18 106 104 104 107 9 9 9 9 9 9 9 9 9 9 98 106 104 105 104 105 108 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 99 107 104 104 108 9 9 9 19 17 9 9 107 104 104 104 107 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 17 19 9 9 9 9 109 104 104 104 104 118 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 107 18 17 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 109 108 107 107 107 108 105 104 104 107 9 9 9 9 9 9 17 18 106 104 104 108 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 107 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 106 108 106 115 107 9 9 9 9 9 17 19 9 108 104 104 104 106 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 17 17 19 9 9 9 9 19 16 115 115 106 109 9 9 9 9 9 109 105 104 104 106 106 106 105 104 104 104 107 18 17 9 9 9 9 9 9 9 9 29 16 19 9 9 9 9 9 9 9 9 9 9 9 9 108 107 107 9 9 9 9 9 9 9 17 18 106 104 104 109 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 108 9 9 9 9 9 9 9 18 15 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 107 118 119 9 9 9 9 9 9 17 18 9 9 105 104 104 105 109 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 19 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 105 104 104 104 104 104 104 104 104 104 104 104 105 9 19 16 16 16 16 16 17 17 17 17 16 17 17 18 18 18 18 19 19 28 26 16 17 29 9 9 18 116 106 106 109 18 17 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 28 105 104 105 9 9 9 9 9 9 9 9 9 108 108 105 104 104 104 104 104 105 109 9 9 9 9 9 9 9 18 24 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 9 9 9 9 9 9 9 9 9 9 18 17 9 9 106 104 104 104 108 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 109 106 104 104 104 104 104 104 104 104 104 105 118 9 17 18 9 9 9 19 19 19 18 18 18 18 17 17 17 17 17 17 17 17 16 16 16 17 17 17 17 17 17 17 18 17 16 17 17 17 18 18 19 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 119 105 105 106 107 109 9 9 9 9 9 9 17 28 105 104 106 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 18 24 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 107 104 104 104 108 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 9 9 109 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 109 9 9 108 104 104 104 104 104 104 104 104 117 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 18 17 17 17 16 17 18 18 18 18 17 17 17 16 19 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 106 9 9 9 9 9 9 17 28 105 104 106 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 18 24 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 99 105 104 104 106 9 107 104 104 104 104 104 104 104 104 104 104 105 107 108 108 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 106 104 104 104 104 104 104 116 29 17 17 17 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 29 16 18 9 9 9 9 9 9 9 107 107 9 9 9 9 109 105 104 104 104 107 9 9 9 9 9 9 17 28 105 104 107 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 109 9 9 9 9 9 9 9 18 24 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 106 104 104 105 9 107 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 9 106 104 104 104 104 104 105 18 17 17 9 97 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 108 105 106 9 9 9 9 108 106 105 104 106 108 109 9 9 9 9 17 28 105 104 107 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 18 24 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 109 104 104 104 107 106 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 108 104 104 104 104 104 104 118 17 17 9 9 106 106 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 9 108 105 109 9 9 109 106 107 107 104 104 104 105 105 105 106 28 17 28 104 104 105 106 106 107 9 9 9 9 9 98 105 105 104 104 104 105 106 117 9 9 9 9 9 9 9 9 9 18 24 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 106 106 107 108 109 9 9 9 9 9 9 9 9 17 19 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 105 104 104 104 104 104 107 18 17 9 9 107 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 16 18 9 9 9 9 9 106 105 107 107 106 106 106 9 9 9 107 104 106 9 108 106 105 104 104 104 104 27 17 28 104 105 108 98 105 104 106 109 109 9 9 98 105 106 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 18 15 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 105 105 106 99 9 9 9 9 9 9 17 18 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 108 104 104 104 104 104 104 104 104 104 115 15 15 115 114 104 104 104 104 104 104 104 104 104 104 104 97 9 9 9 9 9 107 104 104 104 104 104 106 28 17 19 9 99 105 104 106 105 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 17 17 9 9 98 108 109 108 104 104 104 104 104 104 104 108 9 9 106 104 105 99 9 9 109 108 107 107 108 18 17 28 104 104 108 9 106 104 105 106 105 105 105 105 105 106 105 104 104 106 108 108 108 108 109 109 109 109 109 109 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 105 108 105 104 104 109 9 9 9 9 9 9 18 17 9 9 9 107 105 105 105 105 105 105 105 104 104 105 105 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 9 107 104 104 104 105 115 16 25 26 26 26 16 26 25 114 104 104 104 104 104 104 104 104 104 104 104 98 9 9 9 9 108 104 104 104 104 104 105 28 17 18 9 9 109 104 107 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 18 16 9 9 9 118 106 106 105 105 108 107 105 104 104 105 108 9 9 105 104 104 105 107 107 107 107 108 108 109 9 17 28 104 104 105 106 105 104 105 105 104 104 104 104 104 105 105 106 105 104 104 104 104 104 105 105 105 105 105 105 108 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 105 104 104 105 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 109 109 109 109 109 9 108 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 96 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 18 9 29 117 15 16 26 16 17 17 16 17 17 17 27 26 15 114 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 106 104 104 104 104 104 18 17 18 9 9 9 109 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 29 16 18 9 9 9 9 9 9 109 109 9 9 105 104 104 107 9 9 9 105 104 104 104 104 104 104 104 104 104 106 29 17 18 104 104 104 104 104 104 105 105 104 104 105 105 105 104 104 106 106 104 104 104 104 104 104 104 104 104 104 104 108 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 104 104 106 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 17 17 18 16 16 16 17 17 17 18 19 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 106 108 106 106 108 9 9 9 107 104 104 104 104 104 107 18 16 19 9 9 9 109 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 17 17 9 9 9 9 9 9 9 9 9 9 9 108 106 105 108 9 9 9 106 104 104 104 104 104 104 104 104 104 107 29 17 18 105 104 104 104 104 104 105 105 104 107 109 9 9 108 106 105 105 104 104 104 104 104 104 104 104 104 104 104 108 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 106 105 104 104 107 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 28 18 18 18 18 18 17 18 18 18 18 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 18 16 17 18 18 19 9 9 9 9 9 9 9 9 9 98 104 104 104 104 104 104 104 104 104 105 108 9 9 9 9 9 9 109 105 104 104 104 104 106 28 17 17 17 19 9 9 109 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 18 18 17 17 16 16 18 15 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 107 107 107 107 107 106 105 106 9 19 17 18 105 104 104 104 104 104 106 105 104 109 9 9 9 9 107 105 105 104 104 104 104 104 104 104 104 104 104 104 108 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 106 107 104 104 105 99 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 19 19 18 18 18 18 17 17 17 17 17 17 18 18 18 18 18 17 17 17 17 17 17 17 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 107 104 104 104 104 105 18 17 18 9 18 16 18 9 119 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 17 16 16 17 17 17 17 17 17 18 18 17 16 16 18 18 17 16 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 105 104 104 104 104 104 106 106 105 109 9 9 9 9 107 105 105 104 104 104 104 104 104 104 104 104 104 104 108 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 98 107 104 104 108 9 9 9 9 9 9 9 9 9 9 17 18 18 18 17 17 17 17 17 17 17 17 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 106 104 104 104 105 118 18 17 9 9 9 18 17 17 17 16 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 17 17 16 17 17 18 9 9 9 9 9 9 9 9 9 9 18 15 18 9 9 108 16 25 15 17 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 18 105 104 104 104 104 104 106 106 106 9 9 9 9 9 107 105 105 104 104 104 104 104 104 104 104 104 104 104 98 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 104 106 108 99 9 107 106 98 9 9 9 9 9 9 19 18 18 17 17 16 16 17 17 18 28 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 107 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 106 104 104 104 108 29 17 19 9 9 9 9 9 19 18 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 119 105 104 104 104 104 118 9 19 18 17 17 16 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 18 104 104 104 104 104 104 106 106 108 9 9 9 9 9 107 105 105 104 104 104 104 104 104 104 104 104 104 104 98 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 105 119 9 9 9 9 119 9 9 19 18 18 17 17 17 17 18 18 19 18 16 29 9 9 9 119 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 105 104 104 105 108 9 9 9 9 9 9 108 104 104 104 107 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 109 108 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 106 104 104 104 104 104 117 9 9 9 9 9 19 18 17 17 16 16 17 18 18 19 9 9 9 9 9 9 9 9 9 18 17 118 104 104 104 104 104 104 106 106 107 9 9 9 9 9 107 105 106 104 106 107 107 107 107 107 107 107 107 108 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 19 18 17 17 17 17 18 18 9 9 9 9 9 9 9 17 18 9 108 106 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 105 108 108 106 105 107 9 9 9 9 9 9 9 106 104 104 105 119 18 17 9 9 9 9 9 9 9 9 9 9 9 109 107 104 104 105 106 106 106 107 107 107 107 108 98 108 98 9 9 9 9 9 9 109 107 9 9 9 9 9 17 17 9 107 104 104 104 104 104 104 117 9 9 9 9 9 9 9 9 9 9 19 18 18 16 16 16 17 17 17 18 18 19 19 29 18 17 29 117 106 105 105 104 104 107 106 105 107 107 107 107 107 105 105 98 98 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 18 17 9 107 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 108 9 17 19 9 9 9 9 9 9 9 9 9 9 106 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 108 104 107 9 9 9 18 16 9 108 105 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 118 15 16 25 27 18 17 17 17 17 16 16 16 17 17 17 17 27 27 27 29 16 13 14 15 15 116 105 104 105 9 9 9 9 9 9 9 29 18 19 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 17 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 108 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 105 109 9 9 9 9 9 9 9 9 9 9 108 104 104 104 105 9 17 17 9 9 9 9 9 9 9 9 9 9 108 104 104 105 116 108 98 105 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 99 106 104 106 9 9 19 16 18 9 106 104 104 104 104 104 104 104 107 109 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 116 9 9 9 9 9 9 18 17 18 18 17 17 17 17 17 16 16 25 25 16 16 17 28 16 106 9 9 9 9 18 17 17 17 17 17 17 17 17 18 16 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 17 17 18 19 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 104 104 107 99 9 9 9 9 9 9 9 9 9 9 119 118 9 9 9 119 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 107 9 9 9 9 108 107 119 9 9 9 9 106 104 104 104 107 19 17 9 9 9 9 9 9 9 9 9 9 9 107 104 105 48 47 47 48 58 107 107 107 107 107 107 106 107 106 104 106 98 107 108 109 106 104 104 105 109 9 16 18 9 106 104 104 104 104 104 104 104 104 104 105 106 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 106 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 29 9 9 19 19 18 17 16 16 28 9 9 18 17 17 18 9 9 9 9 9 19 18 17 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 18 19 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 17 9 98 104 105 109 9 9 9 9 9 9 9 9 9 9 9 116 104 107 9 109 105 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 105 9 9 9 9 9 106 104 106 9 9 9 9 106 104 104 106 9 17 18 9 9 9 9 9 9 9 9 9 9 9 106 104 77 46 45 45 45 47 49 9 9 9 9 49 49 9 99 105 104 104 104 104 105 104 104 104 105 19 17 17 9 98 105 105 105 105 104 104 104 104 104 104 104 105 99 9 9 9 9 9 9 9 9 9 9 107 104 104 104 106 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 18 17 17 19 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 29 19 19 19 18 18 18 18 18 18 19 19 19 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 105 104 108 9 9 9 9 9 9 9 9 9 109 106 104 104 105 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 107 9 9 9 9 9 105 104 104 106 109 9 107 104 104 105 119 18 17 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 46 45 45 45 45 47 9 9 48 47 46 47 49 109 104 104 104 104 104 104 104 104 105 118 18 16 9 9 9 109 9 9 9 108 104 104 104 104 104 104 104 106 106 106 107 107 107 9 9 9 9 108 105 104 104 104 106 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 15 17 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 106 104 107 9 9 9 9 9 9 9 9 9 105 104 104 104 105 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 105 9 9 9 9 9 108 104 104 104 104 105 106 104 104 105 108 9 17 18 9 9 9 9 9 9 9 9 9 9 99 98 106 104 104 45 45 45 45 45 46 47 47 46 45 45 45 48 108 104 106 108 108 107 106 106 107 108 19 16 19 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 118 9 9 109 104 104 104 104 104 106 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 109 105 106 9 9 9 9 9 9 9 9 99 104 104 104 104 105 9 108 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 107 9 9 9 9 9 106 104 104 104 104 104 104 104 105 98 9 18 17 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 115 45 45 45 45 45 45 45 45 45 45 46 48 116 104 108 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 105 108 108 108 9 9 9 109 104 104 104 104 104 106 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 109 104 104 104 104 104 108 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 105 109 9 9 9 9 9 105 104 104 104 104 104 105 107 108 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 114 45 46 45 45 45 45 45 45 45 46 48 29 105 104 107 108 9 9 9 9 9 18 17 9 9 108 98 108 118 108 109 109 106 104 104 104 105 106 106 106 106 106 107 9 9 9 9 9 9 9 106 108 108 108 108 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 106 105 104 104 105 9 9 9 9 9 9 9 109 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 106 9 9 9 9 9 108 104 104 104 106 105 105 108 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 104 115 46 46 45 45 45 45 45 45 46 48 7 106 104 104 104 104 105 107 9 9 19 16 19 9 108 105 105 105 105 105 105 105 104 104 104 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 108 9 9 107 107 107 107 107 108 107 105 104 105 108 109 108 108 107 107 106 105 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 107 9 9 9 9 9 106 104 104 107 9 109 99 9 9 9 29 17 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 46 46 45 45 45 45 45 45 45 48 9 107 104 104 104 104 104 104 106 9 9 17 17 9 9 9 9 9 9 9 99 97 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 108 106 104 108 9 9 9 9 9 9 9 9 108 104 104 105 105 104 104 104 104 106 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 105 99 9 9 9 9 108 104 104 106 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 105 45 45 45 45 45 45 45 45 45 47 9 98 104 104 104 104 104 104 107 9 18 16 9 9 9 9 9 9 9 9 9 98 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 99 107 105 104 108 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 107 105 104 104 105 107 9 9 9 9 9 9 9 9 108 105 108 108 108 108 108 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 99 104 104 104 104 107 9 9 9 9 9 106 104 104 108 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 45 45 45 45 45 45 45 45 45 46 9 9 105 104 104 104 104 106 9 9 16 18 9 9 9 9 9 9 9 9 9 108 104 104 104 104 106 108 107 107 107 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 98 104 105 105 107 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 109 109 9 9 9 9 9 9 9 9 9 9 9 9 18 17 29 107 105 104 104 107 98 9 9 9 9 9 9 9 9 9 9 109 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 105 9 9 9 9 9 108 104 104 106 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 104 106 46 45 45 45 45 45 45 45 45 46 48 9 106 104 104 104 104 108 9 17 17 9 9 109 99 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 108 107 106 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 107 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 17 27 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 107 9 9 9 9 9 106 104 105 108 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 106 45 46 45 45 45 45 45 45 45 45 45 47 9 105 104 104 104 105 9 19 16 19 9 107 105 105 105 105 106 107 117 108 106 105 107 107 107 107 107 107 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 106 106 106 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 17 27 115 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 119 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 9 9 9 9 9 108 104 104 106 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 45 46 45 45 45 45 45 45 45 45 45 45 46 48 105 104 104 104 108 9 17 18 9 9 9 98 107 105 104 104 106 117 117 117 109 9 9 9 9 9 9 106 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 105 105 106 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 107 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 19 17 27 106 118 108 106 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 107 9 9 9 9 9 105 104 104 107 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 45 45 45 45 46 47 46 45 45 45 45 45 45 46 3 104 104 106 9 18 16 9 9 9 9 9 9 107 104 104 108 9 9 9 9 9 9 9 9 9 9 107 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 109 108 107 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 106 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 108 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 108 105 104 104 106 9 9 9 109 9 109 104 104 104 108 9 9 9 9 9 9 17 18 9 107 106 107 109 9 9 9 9 9 9 9 9 9 9 9 9 108 104 45 46 46 47 48 9 48 46 45 45 45 45 45 45 46 104 104 107 9 17 18 9 9 9 9 9 9 107 104 104 108 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 108 108 108 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 107 104 104 104 108 9 108 108 107 107 107 107 107 107 107 106 117 27 17 9 9 9 9 9 109 107 106 107 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 106 104 104 104 108 9 9 106 105 108 9 107 106 107 9 9 9 9 9 9 19 17 9 9 105 104 104 105 107 109 9 9 9 9 9 9 9 9 9 9 98 104 105 57 56 87 107 109 9 48 46 45 45 45 45 45 45 104 104 118 18 17 9 9 9 9 9 9 99 105 104 104 108 9 9 9 9 9 9 9 9 9 9 106 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 104 106 9 9 9 9 9 9 9 9 9 17 28 107 107 107 107 107 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 19 109 108 108 107 104 104 114 115 106 106 107 107 108 108 108 108 108 106 104 104 14 15 9 9 9 9 9 9 9 109 107 106 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 118 104 104 104 105 9 9 107 104 104 98 9 9 9 9 9 9 9 9 9 9 17 18 9 9 105 104 104 104 104 105 107 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 105 108 8 47 45 45 45 45 44 116 104 107 29 16 18 9 9 9 9 9 9 98 104 104 104 108 9 9 9 9 9 108 107 107 107 107 105 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 108 104 104 107 9 9 9 9 9 9 9 9 9 17 26 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 16 106 106 107 107 107 118 118 119 109 9 9 9 9 9 9 9 9 108 104 104 114 25 18 9 9 9 9 9 9 9 9 109 107 106 106 107 107 107 107 107 107 107 8 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 106 104 104 104 107 9 107 104 104 105 99 9 9 9 9 9 9 9 9 9 19 17 9 9 9 109 108 107 107 106 104 105 109 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 105 3 45 45 45 45 104 104 106 9 18 17 9 9 9 9 9 9 9 108 104 104 104 109 9 9 9 9 9 109 108 107 107 106 106 106 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 109 105 104 98 9 9 9 9 9 9 9 9 9 17 26 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 109 107 109 9 9 9 9 9 9 9 9 9 9 17 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 26 17 9 9 9 9 9 9 9 9 9 9 9 109 108 107 107 107 107 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 108 104 104 104 105 109 99 105 104 104 108 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 106 104 108 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 108 19 16 19 9 9 9 9 9 9 9 108 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 18 16 18 9 9 9 9 9 9 9 9 9 105 104 98 9 9 9 9 9 9 9 9 9 17 27 104 104 104 106 106 106 107 9 9 9 9 9 9 9 9 9 9 9 107 104 106 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 105 104 104 106 29 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 106 104 104 104 106 9 107 104 104 106 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 99 105 104 106 9 9 9 9 9 9 9 9 9 109 107 105 105 106 106 107 109 109 108 118 107 106 106 106 108 9 17 17 9 9 9 9 9 9 9 9 99 105 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 106 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 19 17 17 19 9 9 9 9 9 9 9 9 9 9 106 104 98 9 9 9 9 9 9 9 9 9 17 28 106 104 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 106 109 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 108 104 104 104 104 108 109 105 104 104 107 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 108 107 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 98 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 18 19 19 9 9 9 9 9 16 19 9 9 19 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 105 105 99 9 9 9 9 9 9 9 9 9 17 19 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 109 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 109 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 106 104 104 104 106 9 107 104 104 104 107 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 9 9 9 19 19 19 19 29 29 9 9 9 9 19 19 17 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 18 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 17 17 17 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 105 9 9 9 9 9 9 9 9 9 9 17 19 109 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 108 104 104 104 104 107 9 98 105 104 104 108 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 16 17 17 17 17 17 17 17 18 18 18 18 18 19 19 19 19 19 19 19 19 19 18 18 18 18 17 17 17 17 17 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 19 18 18 18 16 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 104 107 9 9 9 9 9 9 9 9 9 9 17 19 9 108 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 106 104 104 104 104 107 9 9 107 104 105 109 9 9 9 9 9 9 9 9 9 17 19 9 9 9 19 18 18 17 17 17 17 17 17 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 18 9 9 9 9 19 19 18 18 18 17 17 17 17 17 16 16 17 17 17 17 17 17 17 17 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 107 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 117 118 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 107 104 104 104 104 104 104 107 108 106 104 105 9 9 9 9 9 9 9 9 9 19 17 19 19 18 17 17 17 18 18 19 9 9 9 9 9 9 9 9 9 9 9 118 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 119 119 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 19 19 18 18 18 18 18 17 17 17 17 17 17 17 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 19 17 16 17 17 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 19 19 19 19 18 18 17 17 18 18 17 17 17 17 17 17 17 17 17 17 17 17 17 18 18 18 18 19 9 9 9 9 29 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 105 109 9 9 9 9 18 18 17 16 17 16 18 9 9 9 9 9 9 9 119 109 9 108 108 9 9 9 9 9 9 9 99 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 17 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 16 18 18 18 18 18 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 108 104 104 104 104 104 106 107 97 108 118 18 29 19 18 17 17 17 18 18 9 18 17 9 9 9 9 9 109 9 109 105 105 105 107 106 9 9 9 9 9 9 9 108 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 19 18 25 28 19 19 19 19 19 19 18 18 18 18 18 18 16 18 9 9 9 9 9 19 19 19 19 19 19 19 19 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 109 105 104 105 107 108 9 9 9 19 18 17 17 17 18 18 9 9 9 9 9 18 18 9 9 9 9 106 105 107 107 104 104 104 107 106 109 9 9 9 9 9 107 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 17 17 17 17 17 17 17 17 17 17 17 17 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 119 109 9 9 19 18 17 17 17 18 18 9 9 9 9 9 9 9 9 9 17 18 9 9 9 119 104 104 106 107 104 104 104 106 106 108 9 9 9 9 9 107 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 19 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 29 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 19 18 17 17 17 17 18 19 9 9 9 9 9 9 108 107 107 108 9 9 17 19 9 9 9 109 104 104 105 106 104 104 104 105 106 107 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 18 17 17 17 18 19 9 9 9 9 109 107 108 108 106 105 104 104 104 106 9 9 17 9 9 9 9 99 104 104 104 105 104 104 104 105 107 106 9 9 9 9 9 9 105 108 9 107 106 105 107 9 9 9 9 9 9 9 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 108 108 107 104 104 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 19 19 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 106 9 19 17 9 9 9 9 9 106 104 104 104 104 104 104 104 107 106 108 108 119 9 9 9 108 109 108 104 104 104 104 106 109 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 119 106 104 104 104 104 104 104 104 106 119 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 106 9 18 17 9 9 9 9 9 119 108 107 107 108 107 104 104 108 106 104 104 106 9 9 9 9 9 108 104 104 105 106 107 106 106 109 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 116 115 104 105 104 104 104 104 104 104 105 117 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 107 9 18 17 9 9 9 9 9 9 9 9 9 9 109 105 104 106 105 104 104 105 9 9 9 9 9 108 104 104 107 9 9 99 107 106 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 116 115 108 99 106 104 104 105 105 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 107 9 18 17 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 105 9 9 9 9 9 99 105 105 9 9 9 9 9 119 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 98 108 108 107 107 107 107 107 107 107 107 107 107 107 107 107 106 106 106 106 106 106 106 117 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 109 108 108 9 9 9 17 19 9 9 9 9 9 9 9 9 9 119 9 9 9 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 107 9 18 18 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 106 9 9 9 9 9 9 108 109 9 9 9 9 9 19 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 19 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 105 107 107 107 108 108 98 108 98 98 108 108 108 108 108 108 99 99 109 109 109 109 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 108 105 104 104 104 104 104 104 104 104 107 9 18 18 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 105 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 17 17 16 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 106 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 109 105 104 104 104 104 104 104 104 104 104 106 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 109 108 99 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 28 26 16 25 26 18 19 18 18 18 18 18 18 18 18 18 18 18 28 28 28 18 18 19 19 19 19 19 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 108 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 106 9 18 17 9 9 9 9 9 9 9 109 108 107 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 17 104 104 104 105 19 9 9 9 9 9 9 9 9 9 9 9 9 119 117 119 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 108 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 105 104 104 104 104 104 104 104 104 104 104 116 9 19 17 9 9 99 108 106 106 106 106 107 106 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 9 9 9 9 9 9 9 9 9 9 18 17 18 106 105 104 104 107 109 9 9 9 9 109 109 9 9 9 9 9 9 116 118 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 106 109 108 118 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 104 106 9 19 16 9 107 105 104 106 109 99 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 17 17 18 19 9 9 9 9 9 9 18 17 9 9 99 108 106 106 106 106 106 107 107 105 104 107 109 9 9 9 9 9 9 9 9 9 9 9 9 107 106 106 108 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 106 119 19 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 105 9 18 16 29 106 104 104 106 9 9 9 9 119 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 17 18 19 9 9 18 17 9 9 9 9 9 9 9 9 109 107 105 104 104 104 105 108 9 9 9 9 9 9 9 9 9 109 107 104 104 104 108 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 104 104 104 105 106 108 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 109 108 106 106 109 9 106 104 104 104 104 104 104 104 104 104 104 104 104 104 119 18 16 18 118 104 104 104 107 9 9 9 108 104 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 17 16 17 19 9 9 9 9 9 9 9 9 9 109 106 105 106 108 9 9 9 9 9 9 9 9 9 108 104 104 106 107 108 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 107 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 104 105 106 107 108 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 107 105 104 104 104 105 106 105 104 104 104 104 104 104 104 104 104 104 104 104 104 117 18 16 18 9 107 104 104 104 106 107 106 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 17 16 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 16 17 16 17 17 18 19 9 9 9 9 9 9 109 9 9 9 9 9 9 9 9 9 9 9 109 105 105 119 9 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 107 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 99 98 108 107 106 106 106 109 9 9 9 9 9 9 9 9 9 9 9 109 105 107 99 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 107 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 19 17 17 9 108 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 16 18 9 19 17 16 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 29 29 17 18 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 105 109 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 27 106 105 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 109 106 109 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 108 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 29 17 17 18 119 105 104 104 104 104 104 104 106 106 106 9 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 18 17 17 16 16 17 17 17 18 18 18 18 18 18 18 18 17 17 17 17 16 16 17 17 16 19 9 9 9 9 9 9 9 19 18 17 17 16 17 17 18 19 9 9 9 9 9 9 9 9 9 106 105 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 108 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 26 104 104 104 104 104 104 104 104 104 108 9 109 9 9 9 9 9 9 9 9 9 109 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 28 17 18 17 28 105 104 104 105 106 108 99 9 109 106 9 9 9 9 9 9 9 9 9 9 9 17 17 9 9 9 9 9 9 9 9 9 9 19 18 18 18 17 17 17 17 17 17 17 18 18 18 18 19 19 9 9 19 16 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 17 17 17 18 19 9 9 9 9 9 9 98 99 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 16 118 108 108 108 107 107 106 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 27 105 104 104 104 104 104 104 104 104 107 107 107 9 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 118 17 18 17 17 118 105 104 107 9 9 9 9 109 105 98 9 9 9 9 9 9 9 9 9 17 17 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 18 17 17 18 9 9 9 9 9 9 19 17 17 17 17 16 17 17 17 18 18 18 18 18 18 18 18 18 19 19 19 19 19 19 19 19 19 9 9 9 9 9 9 9 9 9 9 9 9 17 15 24 24 24 24 24 23 23 24 27 18 18 18 18 18 18 18 18 18 18 17 17 17 17 17 17 17 17 17 17 17 17 17 17 16 26 105 104 104 104 104 104 104 104 104 104 104 108 9 108 105 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 109 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 106 107 106 119 18 16 18 17 19 108 105 97 9 9 9 119 117 104 108 9 9 9 9 9 9 9 19 16 17 9 99 105 108 99 99 9 109 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 107 9 9 19 17 16 17 9 9 9 9 17 17 18 18 18 16 17 18 18 17 17 17 17 17 17 17 17 17 17 17 17 17 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 16 17 17 17 17 17 17 17 17 17 17 17 17 17 17 17 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 17 16 117 106 105 105 105 105 104 104 104 105 107 108 9 106 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 106 95 109 109 108 107 107 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 104 107 9 9 9 9 19 16 17 18 17 9 107 105 108 108 106 104 104 103 106 9 9 9 9 9 9 18 16 18 9 119 105 104 105 105 105 105 105 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 118 9 9 9 9 106 105 9 9 9 9 9 17 16 18 29 18 16 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 19 19 19 19 19 19 19 19 17 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 118 108 108 108 107 108 9 18 17 9 9 99 9 9 109 106 104 104 107 9 9 9 9 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 105 104 105 105 104 104 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 109 105 104 104 104 104 104 104 104 104 104 104 104 104 104 104 105 108 9 9 9 9 9 9 17 17 18 17 18 107 104 104 104 104 104 104 106 98 9 9 9 9 9 17 16 18 9 118 115 104 104 104 105 108 108 107 104 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 106 108 108 108 107 104 105 9 9 9 9 9 9 18 17 16 17 19 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 105 105 104 104 104 104 104 105 107 27 17 9 9 9 9 9 9 109 105 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 29 99 107 105 104 104 105 106 105 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 105 104 104 104 104 104 104 104 104 104 104 104 104 104 106 109 9 9 9 9 9 9 9 18 18 17 17 16 15 104 104 104 104 104 104 107 9 9 9 9 18 16 17 19 9 9 107 108 108 97 104 105 9 9 98 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 9 9 9 9 9 9 9 9 9 9 9 9 106 106 105 105 106 105 105 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 104 104 104 104 104 104 104 104 104 25 17 9 9 9 9 9 9 9 106 104 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 116 104 104 106 108 98 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 99 105 104 104 104 104 104 104 104 104 104 104 104 105 107 9 9 9 9 9 9 9 9 9 19 17 18 17 16 17 108 106 105 105 104 104 105 99 9 19 17 16 18 9 9 9 9 9 9 9 99 105 105 9 9 108 104 105 106 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 108 108 108 108 108 118 118 109 109 118 118 108 108 108 118 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 104 104 104 104 104 104 104 105 108 28 17 9 9 9 9 9 9 9 98 107 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 28 98 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 109 105 104 104 104 104 104 104 104 104 104 105 108 9 9 9 9 9 9 9 9 9 9 9 9 17 19 19 16 16 9 9 9 9 107 106 109 9 18 16 17 9 9 9 9 9 9 9 9 9 9 105 105 108 108 107 104 106 108 108 106 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 104 106 107 106 106 106 106 106 106 106 106 106 106 106 106 17 16 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 105 105 105 105 105 115 107 108 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 105 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 18 16 18 9 9 9 9 9 18 17 17 19 9 9 9 9 9 9 9 9 9 9 9 105 105 106 106 106 105 106 106 106 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 118 108 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 106 104 104 104 104 104 104 104 104 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 18 16 9 9 9 28 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 106 104 105 106 107 107 108 108 108 9 9 9 9 9 9 9 9 108 105 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 106 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 18 18 18 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 16 9 9 107 105 105 104 104 104 104 104 106 9 9 9 9 9 9 9 108 105 99 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 106 104 104 104 104 104 104 104 104 104 105 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 18 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 109 108 107 106 105 105 117 9 9 9 9 9 9 9 108 104 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 17 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 105 104 104 104 104 104 104 104 104 104 104 108 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 18 16 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 105 104 104 104 104 104 104 104 104 104 104 106 9 9 9 9 9 9 9 9 9 9 9 9 17 17 16 17 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 106 109 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 16 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 104 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 17 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 108 107 107 107 107 107 107 106 106 106 106 107 9 9 9 9 9 9 9 9 9 9 9 9 17 17 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 107 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 108 106 108 9 9 9 9 9 9 9 9 9 9 9 9 9 9 18 19 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 19 18 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9
 9 ]
end
@#$#@#$#@
GRAPHICS-WINDOW
7
61
559
698
135
151
2.0
1
10
1
1
1

CC-WINDOW
573
10
960
212
Command Center

BUTTON
30
10
96
43
NIL
setup
NIL
1
T
OBSERVER
T

MONITOR
434
10
508
59
tick count
ticks
3
1

BUTTON
111
11
174
44
NIL
go
T
1
T
OBSERVER
T

BUTTON
183
11
246
44
step
go
NIL
1
T
OBSERVER
T

SLIDER
254
10
426
43
num-cars
num-cars
10
100
50
10
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This is a proof of concept for including GIS into NetLogo for a dynamic Car Cruising model within the Santa Fe downtown area.

A project was defined for two summer interns, Simran McKenna and Dayoung Jung by Santa Fe Economic Development Inc, under the guidance of Stephen Guerin of the RedfishGroup in collaboration with Santa Fe's Community Development Division and the city police.  I helped out evaluating NetLogo's capabilities for importing a GIS image and executing a model with sufficient performance

The background map of the model is the result of converting GIS data for downtown Santa Fe NM into an image, initally a jpeg file.  It was then converted into a text format image file: PPM (portable pix map) consisting of three RGB values for each pixel.  These were converted into NetLogo color values using the import-ppm procedure. The data was cleaned up a bit within netlogo itself with the cleanup-map procedure, then written to an external data file via the export-dat procedure.  Finally, for this version of the model, the data written out in the preceeding step was cut & paste into the model as an array of integer values.  Note that 9.9999 was converted to/from 9 as a space saving stunt for performance gains.  See http://backspaces.net/hacks/23/netlogo-cruising-model for a blog entry on the topic.

Finally, the model itself simply manages many automobiles driving on the map staying on the "streets" by using the patch colors as guide.  The cars randomly choose side streets as they see them, choosing to turn roughly 1/3 of the time in the move-forward procedure.  It chooses its next location by seeing if it should turn as just described, or by choosing a forward direction in a set of increasing angles of forward cones.  NOTE: See mail lists for newer way to do this via a reporter returning agent-sets within a cone of given degree.  It will have considerably better performance.

That said, the fact we get such reasonable performance pushing NetLogo to its limits is quite a testimonal for NetLogo!

HOW IT WORKS
------------
The agents are initialized to be on a street at a random location.  They then use a simple move-forward proceedure to choose a reasonable next location.


HOW TO USE IT
-------------
Choose a number of cars.  Then click setup then go.


THINGS TO NOTICE
----------------
That it works at all is quite surprising, at least to me!  This suggests we can start pushing NetLogo into GIS oriented models using simple patch level heuristics.


THINGS TO TRY
-------------
Not much .. its pretty limited.


EXTENDING THE MODEL
-------------------
Good homework would be to figure out how to have cars handle one way streets, crowding, street repair and so on.  Also, different speeds might be interesting.

NETLOGO FEATURES
----------------
Tiny patches => Maps.


RELATED MODELS
--------------
None that I know of .. check the mail archive though, several folks were working in the GIS area when this was discussed on the mail list.


CREDITS AND REFERENCES
----------------------
Stephen Guerin, Simran McKenna and Dayoung Jung of the intern project.
The gang at http://friam.org, especially during our GIS reviews.
Craig Fiels, Santa Fe Community Development Division
Catherine Zacher, SFEDI


@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

line
true
0
Line -7566196 true 150 0 150 301

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0.2
@#$#@#$#@
repeat 75 [ go ]
@#$#@#$#@
@#$#@#$#@
